@@ -1,7 +1,7 @@
 /*
  * charconv.c - character code conversion library
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -62,6 +62,15 @@ static struct {
     ScmInternalMutex mutex;
 } ucsconv;
 
+#define CONV_INFO(port)  ((ScmConvInfo*)(PORT_BUF(port)->data))
+
+/* external library to delegate conversion */
+static const ScmPrimitiveParameter *ext_conv = NULL;
+static ScmObj sym_iconv;  /* symbol 'iconv */
+
+#define USE_ICONV_P() \
+    (SCM_EQ(Scm_PrimitiveParameterRef(Scm_VM(), ext_conv), sym_iconv))
+
 /*------------------------------------------------------------
  * Query
  */
@@ -83,11 +92,12 @@ const char* Scm_GetCESName(ScmObj code, const char *argname)
     return c;
 }
 
-int Scm_ConversionSupportedP(const char *from, const char *to)
+int Scm_ConversionSupportedP(const char *from, const char *to,
+                             u_long flags SCM_UNUSED)
 {
-    ScmConvInfo *info = jconv_open(to, from);
-    if (info == NULL) return FALSE;
-    jconv_close(info);
+    ScmConvInfo *cinfo = jconv_open(to, from, USE_ICONV_P());
+    if (cinfo == NULL) return FALSE;
+    jconv_close(cinfo);
     return TRUE;
 }
 
@@ -118,15 +128,15 @@ static conv_guess *findGuessingProc(const char *code)
 
 static int conv_fileno(ScmPort *port)
 {
-    ScmConvInfo *info = (ScmConvInfo*)port->src.buf.data;
-    return Scm_PortFileNo(info->remote);
+    ScmConvInfo *cinfo = CONV_INFO(port);
+    return Scm_PortFileNo(cinfo->remote);
 }
 
 static int conv_ready(ScmPort *port)
 {
-    ScmConvInfo *info = (ScmConvInfo*)port->src.buf.data;
+    ScmConvInfo *cinfo = CONV_INFO(port);
     /* This isn't accurate, but for now ... */
-    return Scm_CharReady(info->remote);
+    return Scm_CharReady(cinfo->remote);
 }
 
 static ScmObj conv_name(int dir, ScmPort *remote, const char *from, const char *to)
@@ -141,27 +151,27 @@ static ScmObj conv_name(int dir, ScmPort *remote, const char *from, const char *
 /*------------------------------------------------------------
  * Input conversion
  *
- *  <-- Buffered port <--- filler <--(info->buf)--- getz(remote)
+ *  <-- Buffered port <--- filler <--(cinfo->buf)--- getz(remote)
  */
 
 static ScmSize conv_input_filler(ScmPort *port, ScmSize mincnt SCM_UNUSED)
 {
-    ScmConvInfo *info = (ScmConvInfo*)port->src.buf.data;
-    const char *inbuf = info->buf;
-    char *outbuf = port->src.buf.end;
+    ScmConvInfo *cinfo = CONV_INFO(port);
+    const char *inbuf = cinfo->buf;
+    char *outbuf = PORT_BUF(port)->end;
 
-    if (info->remoteClosed) return 0;
+    if (cinfo->remoteClosed) return 0;
 
     /* Fill the input buffer.  There may be some remaining bytes in the
        inbuf from the last conversion (insize), so we try to fill the
        rest. */
-    ScmSize insize = info->ptr - info->buf;
-    ScmSize nread = Scm_Getz(info->ptr, info->bufsiz - insize, info->remote);
+    ScmSize insize = cinfo->ptr - cinfo->buf;
+    ScmSize nread = Scm_Getz(cinfo->ptr, cinfo->bufsiz - insize, cinfo->remote);
     if (nread <= 0) {
         /* input reached EOF.  finish the output state */
         if (insize == 0) {
-            ScmSize outroom = SCM_PORT_BUFFER_ROOM(port);
-            ScmSize result = jconv_reset(info, outbuf, outroom);
+            ScmSize outroom = Scm_PortBufferRoom(port);
+            ScmSize result = jconv_reset(cinfo, outbuf, outroom);
             if (result == OUTPUT_NOT_ENOUGH) {
                 /* The port buffer doesn't have enough space to contain the
                    finishing sequence.  Its unusual, for the port buffer
@@ -169,14 +179,14 @@ static ScmSize conv_input_filler(ScmPort *port, ScmSize mincnt SCM_UNUSED)
                    sequence is usually just a few bytes.
                    We signal an error. */
                 Scm_Error("couldn't flush the ending escape sequence in the character encoding conversion port (%s -> %s).  possibly an implementation error",
-                          info->fromCode, info->toCode);
+                          cinfo->fromCode, cinfo->toCode);
                 }
-            if (info->ownerp) {
-                Scm_ClosePort(info->remote);
-                info->remoteClosed = TRUE;
+            if (cinfo->ownerp) {
+                Scm_ClosePort(cinfo->remote);
+                cinfo->remoteClosed = TRUE;
             }
 #ifdef JCONV_DEBUG
-            fprintf(stderr, "<= r=%d (reset), out(%p)%d\n",
+            fprintf(stderr, "<= r=%ld (reset), out(%p)%ld\n",
                     result, outbuf, outroom);
 #endif
             return (ScmSize)result;
@@ -187,14 +197,14 @@ static ScmSize conv_input_filler(ScmPort *port, ScmSize mincnt SCM_UNUSED)
 
     /* Conversion. */
     ScmSize inroom = insize;
-    ScmSize outroom = SCM_PORT_BUFFER_ROOM(port);
+    ScmSize outroom = Scm_PortBufferRoom(port);
 
 #ifdef JCONV_DEBUG
-    fprintf(stderr, "=> in(%p)%d out(%p)%d\n", inbuf, insize, outbuf, outroom);
+    fprintf(stderr, "=> in(%p)%ld out(%p)%ld\n", inbuf, insize, outbuf, outroom);
 #endif
-    ScmSize result = jconv(info, &inbuf, &inroom, &outbuf, &outroom);
+    ScmSize result = jconv(cinfo, &inbuf, &inroom, &outbuf, &outroom);
 #ifdef JCONV_DEBUG
-    fprintf(stderr, "<= r=%d, in(%p)%d out(%p)%d\n",
+    fprintf(stderr, "<= r=%ld, in(%p)%ld out(%p)%ld\n",
             result, inbuf, inroom, outbuf, outroom);
 #endif
     /* we've got an error. */
@@ -203,46 +213,62 @@ static ScmSize conv_input_filler(ScmPort *port, ScmSize mincnt SCM_UNUSED)
            end of the input buffer, or the output buffer is full.
            We shift the unconverted bytes to the beginning of input
            buffer. */
-        memmove(info->buf, info->buf+insize-inroom, inroom);
-        info->ptr = info->buf + inroom;
-        return info->bufsiz - (ScmSize)outroom;
-    } else if (result == ILLEGAL_SEQUENCE) {
-        /* it's likely that the input contains invalid sequence. */
-        ScmSize cnt = inroom >= 6 ? 6 : (ScmSize)inroom;
-        ScmObj s = Scm_MakeString(info->buf+insize-inroom, cnt, cnt,
-                                  SCM_STRING_COPYING|SCM_STRING_INCOMPLETE);
-        Scm_Error("invalid character sequence in the input stream: %S ...", s);
+        memmove(cinfo->buf, cinfo->buf+insize-inroom, inroom);
+        cinfo->ptr = cinfo->buf + inroom;
+        return cinfo->bufsiz - (ScmSize)outroom;
+    } else if (result == ILLEGAL_SEQUENCE || result == NO_OUTPUT_CHAR) {
+        /* input sequence can't be represented by the destination CES. */
+        if (cinfo->replacep) {
+            if (outroom < cinfo->replaceSize) {
+                /* We don't have enough room to place replace char, so
+                   we stop before the bad char and let it be handled
+                   in the next callback. */
+                memmove(cinfo->buf, cinfo->buf+insize-inroom, inroom);
+                cinfo->ptr = cinfo->buf + inroom;
+                return cinfo->bufsiz - (ScmSize)outroom;
+            } else {
+                memmove(cinfo->buf, cinfo->buf+insize-inroom+1, inroom-1);
+                cinfo->ptr = cinfo->buf + inroom - 1;
+                memcpy(outbuf, cinfo->replaceSeq, cinfo->replaceSize);
+                return cinfo->bufsiz - (ScmSize)(outroom - cinfo->replaceSize);
+            }
+        } else {
+            ScmSize cnt = inroom >= 6 ? 6 : (ScmSize)inroom;
+            ScmObj s = Scm_MakeString(cinfo->buf+insize-inroom, cnt, cnt,
+                                      SCM_STRING_COPYING|SCM_STRING_INCOMPLETE);
+            Scm_PortError(port, SCM_PORT_ERROR_DECODING,
+                          "invalid character sequence in the input stream: %S ...", s);
+        }
     }
 
     /* Conversion is done completely. */
     /* NB: There are cases that some bytes are left in the input buffer
        even iconv returns positive value.  We need to shift those bytes. */
     if (inroom > 0) {
-        memmove(info->buf, info->buf+insize-inroom, inroom);
-        info->ptr = info->buf + inroom;
-        return info->bufsiz - (ScmSize)outroom;
+        memmove(cinfo->buf, cinfo->buf+insize-inroom, inroom);
+        cinfo->ptr = cinfo->buf + inroom;
+        return cinfo->bufsiz - (ScmSize)outroom;
     } else {
-        info->ptr = info->buf;
-        return info->bufsiz - (ScmSize)outroom;
+        cinfo->ptr = cinfo->buf;
+        return cinfo->bufsiz - (ScmSize)outroom;
     }
 }
 
 static void conv_input_closer(ScmPort *p)
 {
-    ScmConvInfo *info = (ScmConvInfo*)p->src.buf.data;
-    jconv_close(info);
-    if (info->ownerp) {
-        Scm_ClosePort(info->remote);
-        info->remoteClosed = TRUE;
+    ScmConvInfo *cinfo = CONV_INFO(p);
+    jconv_close(cinfo);
+    if (cinfo->ownerp) {
+        Scm_ClosePort(cinfo->remote);
+        cinfo->remoteClosed = TRUE;
     }
 }
 
 ScmObj Scm_MakeInputConversionPort(ScmPort *fromPort,
                                    const char *fromCode,
                                    const char *toCode,
-                                   ScmObj handler SCM_UNUSED, /* for now */
                                    ScmSize bufsiz,
-                                   int ownerp)
+                                   u_long flags)
 {
     char *inbuf = NULL;
     ScmSize preread = 0;
@@ -272,13 +298,13 @@ ScmObj Scm_MakeInputConversionPort(ScmPort *fromPort,
         fromCode = guessed;
     }
 
-    ScmConvInfo *cinfo = jconv_open(toCode, fromCode);
+    ScmConvInfo *cinfo = jconv_open(toCode, fromCode, USE_ICONV_P());
     if (cinfo == NULL) {
         Scm_Error("conversion from code %s to code %s is not supported",
                   fromCode, toCode);
     }
     cinfo->remote = fromPort;
-    cinfo->ownerp = ownerp;
+    cinfo->ownerp = flags & CVPORT_OWNER;
     cinfo->bufsiz = bufsiz;
     cinfo->remoteClosed = FALSE;
     if (preread > 0) {
@@ -289,6 +315,10 @@ ScmObj Scm_MakeInputConversionPort(ScmPort *fromPort,
         cinfo->ptr = cinfo->buf;
     }
 
+    if (flags & CVPORT_REPLACE) {
+        jconv_set_replacement(cinfo);
+    }
+
     ScmPortBuffer bufrec;
     memset(&bufrec, 0, sizeof(bufrec));
     bufrec.size = cinfo->bufsiz;
@@ -314,14 +344,13 @@ static ScmPort *coding_aware_conv(ScmPort *src, const char *encoding)
     return SCM_PORT(Scm_MakeInputConversionPort(src,
                                                 encoding,
                                                 Scm_SupportedCharacterEncodings()[0],
-                                                SCM_FALSE,
-                                                0, TRUE));
+                                                0, CVPORT_OWNER));
 }
 
 /*------------------------------------------------------------
  * Output conversion
  *
- *   Buffered port ----> flusher -->(info->buf)--> putz(remote)
+ *   Buffered port ----> flusher -->(cinfo->buf)--> putz(remote)
  */
 
 /* NB: Glibc-2.1.2's iconv() has a bug in SJIS handling.  If output
@@ -341,54 +370,54 @@ static ScmPort *coding_aware_conv(ScmPort *src, const char *encoding)
 
 static void conv_output_closer(ScmPort *port)
 {
-    ScmConvInfo *info = (ScmConvInfo*)port->src.buf.data;
+    ScmConvInfo *cinfo = CONV_INFO(port);
 
     /* if there's remaining bytes in buf, send them to the remote port. */
-    if (info->ptr > info->buf) {
-        Scm_Putz(info->buf, (int)(info->ptr - info->buf), info->remote);
-        info->ptr = info->buf;
+    if (cinfo->ptr > cinfo->buf) {
+        Scm_Putz(cinfo->buf, (int)(cinfo->ptr - cinfo->buf), cinfo->remote);
+        cinfo->ptr = cinfo->buf;
     }
     /* sends out the closing sequence, if any */
-    ScmSize r = jconv_reset(info, info->buf, info->bufsiz);
+    ScmSize r = jconv_reset(cinfo, cinfo->buf, cinfo->bufsiz);
 #ifdef JCONV_DEBUG
-    fprintf(stderr, "<= r=%d(reset), buf(%p)\n",
-            r, info->buf);
+    fprintf(stderr, "<= r=%ld(reset), buf(%p)\n",
+            r, cinfo->buf);
 #endif
     if (r < 0) {
         Scm_Error("something wrong in resetting output character encoding conversion (%s -> %s).  possibly an implementation error.",
-                  info->fromCode, info->toCode);
+                  cinfo->fromCode, cinfo->toCode);
     }
     if (r > 0) {
-        Scm_Putz(info->buf, (ScmSize)r, info->remote);
+        Scm_Putz(cinfo->buf, (ScmSize)r, cinfo->remote);
     }
     /* flush remove port */
-    Scm_Flush(info->remote);
-    if (info->ownerp) {
-        Scm_ClosePort(info->remote);
-        info->remoteClosed = TRUE;
+    Scm_Flush(cinfo->remote);
+    if (cinfo->ownerp) {
+        Scm_ClosePort(cinfo->remote);
+        cinfo->remoteClosed = TRUE;
     }
-    jconv_close(info);
+    jconv_close(cinfo);
 }
 
 static ScmSize conv_output_flusher(ScmPort *port, ScmSize cnt, int forcep)
 {
-    ScmConvInfo *info = (ScmConvInfo*)port->src.buf.data;
-    ScmSize inroom = SCM_PORT_BUFFER_AVAIL(port);
+    ScmConvInfo *cinfo = CONV_INFO(port);
+    ScmSize inroom = Scm_PortBufferAvail(port);
     ScmSize len = inroom;
-    const char *inbuf = port->src.buf.buffer;
+    const char *inbuf = PORT_BUF(port)->buffer;
 
     for (;;) {
         /* Conversion. */
-        char *outbuf = info->ptr;
-        ScmSize outroom = info->bufsiz - (info->ptr - info->buf);
+        char *outbuf = cinfo->ptr;
+        ScmSize outroom = cinfo->bufsiz - (cinfo->ptr - cinfo->buf);
 #ifdef JCONV_DEBUG
-        fprintf(stderr, "=> in(%p,%p)%d out(%p,%p)%d\n",
-                inbuf, len, inroom,
-                info->buf, info->ptr, outroom);
+        fprintf(stderr, "=> in(%p,%p)%ld out(%p,%p)%ld\n",
+                inbuf, inbuf+len, inroom,
+                cinfo->buf, cinfo->ptr, outroom);
 #endif
-        ScmSize result = jconv(info, &inbuf, &inroom, &outbuf, &outroom);
+        ScmSize result = jconv(cinfo, &inbuf, &inroom, &outbuf, &outroom);
 #ifdef JCONV_DEBUG
-        fprintf(stderr, "<= r=%d, in(%p)%d out(%p)%d\n",
+        fprintf(stderr, "<= r=%ld, in(%p)%ld out(%p)%ld\n",
                 result, inbuf, inroom, outbuf, outroom);
 #endif
         if (result == INPUT_NOT_ENOUGH) {
@@ -397,34 +426,35 @@ static ScmSize conv_output_flusher(ScmPort *port, ScmSize cnt, int forcep)
                end of the input buffer.  We just return # of bytes
                flushed.  (Shifting unconverted characters is done by
                buffered port routine) */
-            info->ptr = outbuf;
+            cinfo->ptr = outbuf;
 #else
             /* See the above notes.  We always flush the output buffer
                here, so that we can avoid output buffer overrun. */
-            Scm_Putz(info->buf, (ScmSize)(outbuf - info->buf), info->remote);
-            info->ptr = info->buf;
+            Scm_Putz(cinfo->buf, (ScmSize)(outbuf - cinfo->buf), cinfo->remote);
+            cinfo->ptr = cinfo->buf;
 #endif
             return (ScmSize)(len - inroom);
         } else if (result == OUTPUT_NOT_ENOUGH) {
             /* Output buffer got full.  Flush it, and continue
                conversion. */
-            Scm_Putz(info->buf, (ScmSize)(outbuf - info->buf), info->remote);
-            info->ptr = info->buf;
+            Scm_Putz(cinfo->buf, (ScmSize)(outbuf - cinfo->buf), cinfo->remote);
+            cinfo->ptr = cinfo->buf;
             continue;
-        } else if (result == ILLEGAL_SEQUENCE) {
+        } else if (result == ILLEGAL_SEQUENCE || result == NO_OUTPUT_CHAR) {
             /* it's likely that input contains invalid sequence.
                TODO: we should handle this case gracefully. */
-            Scm_Error("invalid character sequence in the input stream");
+            Scm_PortError(port, SCM_PORT_ERROR_ENCODING,
+                          "cannot encode a character to the output stream");
             return 0;           /* dummy */
         } else {
 #ifndef GLIBC_2_1_ICONV_BUG
             /* Conversion is done completely.  Update outptr. */
-            info->ptr = outbuf;
+            cinfo->ptr = outbuf;
 #else
             /* See the above notes.  We always flush the output buffer here,
                so that we can avoid output buffer overrun. */
-            Scm_Putz(info->buf, (ScmSize)(outbuf - info->buf), info->remote);
-            info->ptr = info->buf;
+            Scm_Putz(cinfo->buf, (ScmSize)(outbuf - cinfo->buf), cinfo->remote);
+            cinfo->ptr = cinfo->buf;
 #endif
             if (forcep && len - inroom != cnt) continue;
             return len - inroom;
@@ -435,7 +465,7 @@ static ScmSize conv_output_flusher(ScmPort *port, ScmSize cnt, int forcep)
 ScmObj Scm_MakeOutputConversionPort(ScmPort *toPort,
                                     const char *toCode,
                                     const char *fromCode,
-                                    ScmSize bufsiz, int ownerp)
+                                    ScmSize bufsiz, u_long flags)
 {
     if (!SCM_OPORTP(toPort))
         Scm_Error("output port required, but got %S", toPort);
@@ -445,18 +475,22 @@ ScmObj Scm_MakeOutputConversionPort(ScmPort *toPort,
         bufsiz = MINIMUM_CONVERSION_BUFFER_SIZE;
     }
 
-    ScmConvInfo *cinfo = jconv_open(toCode, fromCode);
+    ScmConvInfo *cinfo = jconv_open(toCode, fromCode, USE_ICONV_P());
     if (cinfo == NULL) {
         Scm_Error("conversion from code %s to code %s is not supported",
                   fromCode, toCode);
     }
     cinfo->remote = toPort;
-    cinfo->ownerp = ownerp;
+    cinfo->ownerp = flags & CVPORT_OWNER;
     cinfo->bufsiz = (bufsiz > 0)? bufsiz : DEFAULT_CONVERSION_BUFFER_SIZE;
     cinfo->remoteClosed = FALSE;
     cinfo->buf = SCM_NEW_ATOMIC2(char *, cinfo->bufsiz);
     cinfo->ptr = cinfo->buf;
 
+    if (flags & CVPORT_REPLACE) {
+        jconv_set_replacement(cinfo);
+    }
+
     ScmPortBuffer bufrec;
     memset(&bufrec, 0, sizeof(bufrec));
     bufrec.size = cinfo->bufsiz;
@@ -514,7 +548,7 @@ static ScmChar ucstochar(int ucs4)
     if (r == INPUT_NOT_ENOUGH || r == OUTPUT_NOT_ENOUGH) {
         Scm_Error("can't convert UCS4 code %d to a character: implementation problem?", ucs4);
     }
-    if (r == ILLEGAL_SEQUENCE) {
+    if (r == ILLEGAL_SEQUENCE || r == NO_OUTPUT_CHAR) {
         return SCM_CHAR_INVALID;
     } else {
         ScmChar out;
@@ -545,7 +579,7 @@ static int chartoucs(ScmChar ch)
     if (r == INPUT_NOT_ENOUGH || r == OUTPUT_NOT_ENOUGH) {
         Scm_Error("can't convert character %u to UCS4 code: implementation problem?", ch);
     }
-    if (r == ILLEGAL_SEQUENCE) {
+    if (r == ILLEGAL_SEQUENCE || r == NO_OUTPUT_CHAR) {
         return -1;
     } else {
         unsigned char *ucp = (unsigned char*)outbuf;
@@ -594,21 +628,29 @@ SCM_EXTENSION_ENTRY void Scm_Init_gauche__charconv(void)
 {
     SCM_INIT_EXTENSION(gauche__charconv);
     guess.procs = NULL;
+
     (void)SCM_INTERNAL_MUTEX_INIT(guess.mutex);
 #if   defined(GAUCHE_CHAR_ENCODING_UTF_8)
     ucsconv.ucs2char = ucsconv.char2ucs = NULL;
 #elif defined(GAUCHE_CHAR_ENCODING_EUC_JP)
-    ucsconv.ucs2char = jconv_open("EUCJP", "UTF-8");
-    ucsconv.char2ucs = jconv_open("UTF-8", "EUCJP");
+    ucsconv.ucs2char = jconv_open("EUCJP", "UTF-8", TRUE);
+    ucsconv.char2ucs = jconv_open("UTF-8", "EUCJP", TRUE);
 #elif defined(GAUCHE_CHAR_ENCODING_SJIS)
-    ucsconv.ucs2char = jconv_open("SJIS", "UTF-8");
-    ucsconv.char2ucs = jconv_open("UTF-8", "SJIS");
+    ucsconv.ucs2char = jconv_open("SJIS", "UTF-8", TRUE);
+    ucsconv.char2ucs = jconv_open("UTF-8", "SJIS", TRUE);
 #else
     ucsconv.ucs2char = ucsconv.char2ucs = NULL;
 #endif
     (void)SCM_INTERNAL_MUTEX_INIT(ucsconv.mutex);
+
     Scm_Init_convguess();
     Scm_Init_convaux();
+
+    ScmModule *mod = SCM_MODULE(SCM_FIND_MODULE("gauche.charconv", 0));
+    sym_iconv = SCM_INTERN("iconv");
+    ext_conv = Scm_BindPrimitiveParameter(mod, "external-conversion-library",
+                                          sym_iconv, 0);
+
     Scm__InstallCharconvHooks(ucstochar, chartoucs);
     Scm__InstallCodingAwarePortHook(coding_aware_conv);
 }
