@@ -12,28 +12,30 @@
   (string-complete->incomplete
    (call-with-input-file file port->byte-string)))
 
-(define (file->string-conv/in file from . to)
+(define (file->string-conv/in file from . args)
   (string-complete->incomplete
    (call-with-input-file file
      (lambda (f)
-       (port->byte-string  (apply open-input-conversion-port f from to))))))
+       (port->byte-string  (apply open-input-conversion-port f from args))))))
 
-(define (file->string-conv/out file to from reader writer)
+(define (file->string-conv/out file to from reader writer . args)
   (string-complete->incomplete
    (call-with-output-string
      (lambda (out)
        (call-with-input-file file
          (lambda (in)
-           (let ((cv (open-output-conversion-port out to :from-code from)))
+           (let ((cv (apply open-output-conversion-port out to
+                            :from-code from args)))
              (let loop ((data (reader in)))
                (if (eof-object? data)
                    (close-output-port cv)
                    (begin (writer data cv) (loop (reader in))))))))))))
 
 (define (map-test tester file from-codes to-codes)
-  (for-each (lambda (from)
-              (for-each (lambda (to) (tester file from to)) to-codes))
-            from-codes))
+  (let1 srcdir (sys-dirname (current-load-path))
+    (dolist [from from-codes]
+      (dolist [to to-codes]
+        (tester #"~|srcdir|/~|file|" from to)))))
 
 ;;--------------------------------------------------------------------
 (test-section "ces-equivalent? and ces-upper-compatible?")
@@ -70,27 +72,29 @@
 ;;--------------------------------------------------------------------
 (test-section "input conversion")
 
-(define (test-input file from to . guesser)
-  (let* ((realfrom (if (null? guesser) from (car guesser)))
+(define (test-input file from to :key (guesser #f) illegal-output)
+  (let* ((realfrom (or guesser from))
          (infostr  (format #f "~a.~a (~a) => ~a" file from realfrom to))
          (fromfile (format #f "~a.~a" file from))
          (tofile   (format #f "~a.~a" file to)))
     (if (ces-conversion-supported? from to)
-        (if (supported-character-encoding? to)
-            (test infostr
-                  (file->string tofile)
-                  (lambda () (file->string-conv/in fromfile realfrom)))
-            (test infostr
-                  (file->string tofile)
-                  (lambda () (file->string-conv/in fromfile realfrom
-                                                   :to-code to))))
-        (test infostr "(not supported)"
-              (lambda () "(not supported)")))
+      (if (supported-character-encoding? to)
+        (test infostr
+              (file->string tofile)
+              (lambda () (file->string-conv/in fromfile realfrom
+                                               :illegal-output illegal-output)))
+        (test infostr
+              (file->string tofile)
+              (lambda () (file->string-conv/in fromfile realfrom
+                                               :to-code to
+                                               :illegal-output illegal-output))))
+      (test infostr "(not supported)"
+            (lambda () "(not supported)")))
     ))
 
 (map-test test-input "data/jp1"
-          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
-          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP" "UTF-16" "UTF-16BE" "UTF-16LE")
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP" "UTF-16BE" "UTF-16LE"))
 (map-test test-input "data/jp2"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
@@ -104,23 +108,63 @@
           '("EUCKR" "UTF-8" "ISO2022KR")
           '("EUCKR" "UTF-8" "ISO2022KR"))
 
+;; illegal-output replace
+(map-test (lambda (file from to)
+            (test-input file from to :illegal-output 'replace))
+          "data/jp1"
+          '("ASCII" "EUCJP" "UTF-8" "SJIS" "ISO2022JP")
+          '("ASCII"))
+(map-test (lambda (file from to)
+            (test-input file from to :illegal-output 'replace))
+          "data/jp2"
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
+          '("ASCII"))
+(map-test (lambda (file from to)
+            (test-input file from to :illegal-output 'replace))
+          "data/lat1"
+          '("ISO8859-1")
+          '("ASCII" "ISO8859-1" "EUCJP" "SJIS" "ISO2022JP" "UTF-8"
+            "UTF-16" "UTF-16BE" "UTF-16LE"
+            "UTF-32" "UTF-32BE" "UTF-32LE"))
+(map-test (lambda (file from to)
+            (test-input file from to :illegal-output 'replace))
+          "data/lat1x"
+          '("ISO8859-1" "EUCJP" "UTF-8" "SJIS"
+            "UTF-16" "UTF-16BE" "UTF-16LE"
+            "UTF-32" "UTF-32BE" "UTF-32LE")
+          '("ISO8859-1"))
+(dolist [n '(2 3 4 5 6 7 8 9 10 11 13 14 15 16)]
+  (map-test test-input #"data/lat~|n|"
+            `(,#"ISO8859-~|n|" "UTF-16BE")
+            `(,#"ISO8859-~|n|" "UTF-16BE")))
+
 ;; autodetect tester
 (map-test (lambda (file from to)
-            (test-input file from to "*JP"))
+            (test-input file from to :guesser "*JP"))
           "data/jp1"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
 (map-test (lambda (file from to)
-            (test-input file from to "*JP"))
+            (test-input file from to :guesser "*JP"))
           "data/jp2"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
 (map-test (lambda (file from to)
-            (test-input file from to "*JP"))
+            (test-input file from to :guesser "*JP"))
           "data/jp3"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
 
+;; UTF BOM illegal-output
+(let ()
+  (define (f n) #"~(sys-dirname (current-load-path))/~n")
+
+  (define (bom-test bom-src)
+    (equal? (call-with-input-file (f "data/jp1.UTF-8") port->string)
+            (call-with-input-file (f bom-src) port->string)))
+  (bom-test "data/jp1.UTF-16-BOMBE")
+  (bom-test "data/jp1.UTF-16-BOMLE"))
+
 ;;--------------------------------------------------------------------
 (test-section "output conversion")
 
@@ -139,15 +183,15 @@
 
 (define (test-output/byte file from to)
   (test-output "byte" read-byte write-byte file from to))
-        
+
 (define (test-output/char file from to)
   (test-output "char" read-char write-char file from to))
-        
+
 (define (test-output/chunk256 file from to)
-  (test-output "chunk256" (lambda (p) (read-block 256 p)) display file from to))
-        
+  (test-output "chunk256" (^p (read-block 256 p)) display file from to))
+
 (define (test-output/chunk20 file from to)
-  (test-output "chunk20" (lambda (p) (read-block 20 p)) display file from to))
+  (test-output "chunk20" (^p (read-block 20 p)) display file from to))
 
 (define internal-enc
   (case (gauche-character-encoding)
@@ -157,8 +201,8 @@
     (else '())))
 
 (map-test test-output/byte "data/jp1"
-          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
-          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP" "UTF-16" "UTF-16BE" "UTF-16LE")
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP" "UTF-16BE" "UTF-16LE"))
 (map-test test-output/chunk256 "data/jp1"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
@@ -218,15 +262,20 @@
           '("EUCKR" "UTF-8" "ISO2022KR")
           '("EUCKR" "UTF-8" "ISO2022KR"))
 
+(dolist [n '(1 2 3 4 5 6 7 8 9 10 11 13 14 15 16)]
+  (map-test test-output/chunk20 #"data/lat~|n|"
+            `(,#"ISO8859-~|n|" "UTF-16BE")
+            `(,#"ISO8859-~|n|" "UTF-16BE")))
+
 ;;--------------------------------------------------------------------
 (test-section "open -> #f")
 
 (test* "input :if-does-not-exist #f" #f
-       (with-input-from-file ":NOT;EXIST:" (lambda () (error "PANIC!"))
+       (with-input-from-file "_NOT_EXIST_" (lambda () (error "PANIC!"))
                              :if-does-not-exist #f :encoding "*JP"))
 
 (test* "output :if-does-not-exist #f" #f
-       (with-output-to-file ":NOT;EXIST:" (lambda () (error "PANIC!"))
+       (with-output-to-file "_NOT_EXIST_" (lambda () (error "PANIC!"))
                             :if-does-not-exist #f :encoding "UTF-8"))
 
 ;;--------------------------------------------------------------------
@@ -285,12 +334,35 @@
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
 
+;;-------------------------------------------------------------------
+(test-section "replacement illegal-output")
+(test* "utf-8 -> ascii noreplacement" (test-error <io-decoding-error>)
+       (ces-convert #u8(#x61 #xe3 #x81 #x82 #x62 #xe3 #x81
+                       #x84 #x63 #xe3 #x81 #x86)
+                    'utf-8 'ascii))
+(test* "utf-8 -> ascii replacement" "a?b?c?"
+       (ces-convert #u8(#x61 #xe3 #x81 #x82 #x62 #xe3 #x81
+                        #x84 #x63 #xe3 #x81 #x86)
+                    'utf-8 'ascii :illegal-output 'replace))
+
+(let ([src #u8(#x61 #xe2 #x98 #xba #x62 #xe2 #x9b #xb1 #x63)])
+  ;; src is #\a + snowman + #\b + umbrella + #\c
+  (test* "utf-8 -> eucjp noreplacement" (test-error <io-decoding-error>)
+       (ces-convert-to <u8vector> src 'utf-8 'eucjp))
+  (test* "utf-8 -> eucjp replacement" '#u8(#x61 #xa2 #xae #x62 #xa2 #xae #x63)
+       (ces-convert-to <u8vector> src 'utf-8 'eucjp :illegal-output 'replace))
+  (test* "utf-8 -> sjis noreplacement" (test-error <io-decoding-error>)
+       (ces-convert-to <u8vector> src 'utf-8 'sjis))
+  (test* "utf-8 -> sjis replacement" '#u8(#x61 #x81 #xac #x62 #x81 #xac #x63)
+       (ces-convert-to <u8vector> src 'utf-8 'sjis :illegal-output 'replace))
+  )
+
 ;;--------------------------------------------------------------------
 (test-section "wrapping conversion")
 
 (define (test-wrap/in str from to)
   (if (ces-conversion-supported? from to)
-      (test* #`"wrap-input-conversion-port (,|from|, ,|to|)"
+      (test* #"wrap-input-conversion-port (~|from|, ~|to|)"
              (string-complete->incomplete
               (port->string (open-input-conversion-port
                              (open-input-string str)
@@ -299,12 +371,12 @@
               (port->string (wrap-with-input-conversion
                              (open-input-string str)
                              from :to-code to))))
-      (test* #`"wrap-input-conversion-port (,|from|, ,|to|)"
+      (test* #"wrap-input-conversion-port (~|from|, ~|to|)"
              "(not supported)" "(not supported)")))
 
 (define (test-wrap/out str from to)
   (if (ces-conversion-supported? from to)
-      (test* #`"wrap-output-conversion-port (,|from|, ,|to|)"
+      (test* #"wrap-output-conversion-port (~|from|, ~|to|)"
              (string-complete->incomplete
               (let* ((os  (open-output-string))
                      (out (open-output-conversion-port os to :from-code from)))
@@ -317,7 +389,7 @@
                 (display str out)
                 (close-output-port out)
                 (get-output-string os))))
-      (test* #`"wrap-input-conversion-port (,|from|, ,|to|)"
+      (test* #"wrap-input-conversion-port (~|from|, ~|to|)"
              "(not supported)" "(not supported)")))
 
 (map-test test-wrap/in "foobar"
@@ -332,10 +404,10 @@
 
 (define (test-call-with file code)
   (if (ces-conversion-supported? code (gauche-character-encoding))
-    (test* #`"call-with-input-conversion (,|code| -> ,(gauche-character-encoding))"
-           (file->string-conv/in #`",|file|.,code" code)
+    (test* #"call-with-input-conversion (~code -> ~(gauche-character-encoding))"
+           (file->string-conv/in #"~|file|.~code" code)
            (string-complete->incomplete
-            (call-with-input-file #`",|file|.,code"
+            (call-with-input-file #"~|file|.~code"
               (lambda (p)
                 (call-with-input-conversion p port->string
                                             :encoding code))))))
@@ -347,7 +419,7 @@
                                              ((utf-8)  "UTF-8")
                                              ((none)   "UTF-8")))
                port->string)))
-      (test* #`"call-with-output-conversion (,(gauche-character-encoding) -> ,|code|)"
+      (test* #"call-with-output-conversion (~(gauche-character-encoding) -> ~code)"
              (string-complete->incomplete
               (ces-convert s (gauche-character-encoding) code))
              (string-complete->incomplete
@@ -360,7 +432,7 @@
 
 (dolist (file '("data/jp1" "data/jp2" "data/jp3" "data/jp4"))
   (dolist (code '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
-    (test-call-with file code)))
+    (test-call-with #"~(sys-dirname (current-load-path))/~file" code)))
 
 ;;--------------------------------------------------------------------
 (test-section "ucs <-> char")
@@ -406,4 +478,37 @@
          (lambda () (char->ucs (string-ref (read-from-string "\"\\u30421\"") 0))))
    ))
 
+;;--------------------------------------------------------------------
+(test-section "coding-aware port")
+
+(define *target-string*
+  (case (gauche-character-encoding)
+    ((euc-jp sjis utf-8)
+     (read-from-string
+      "\"\\u3053\\u3093\\u306b\\u3061\\u306f\\u3001\\u4e16\\u754c\""))
+    ((none)
+     (read-from-string
+      "\"\\xe3\\x81\\x93\\xe3\\x82\\x93\\xe3\\x81\\xab\\xe3\\x81\\xa1\\xe3\\x81\\xaf\\xe3\\x80\\x81\\xe4\\xb8\\x96\\xe7\\x95\\x8c\""))))
+
+(define (test-coding-aware-port num encoding)
+  (when (ces-conversion-supported? (gauche-character-encoding) encoding)
+    (test* (format "program source port reading from jpsrc~a.~a" num encoding)
+           *target-string*
+           (call-with-input-file (format "~a/data/jpsrc~a.~a.scm"
+                                         (sys-dirname (current-load-path))
+                                         num encoding)
+             (lambda (in)
+               (let ((src (open-coding-aware-port in)))
+                 (let loop ((x (read src)))
+                   (cond ((eof-object? x) (close-input-port src) x)
+                         ((eq? (cadr x) '*the-string*)
+                          (close-input-port src)
+                          (caddr x))
+                         (else (loop (read src)))))))))))
+
+(unless (eq? (gauche-character-encoding) 'none)
+  (dolist (num '(1 2 3))
+    (dolist (enc '("EUCJP" "SJIS" "UTF-8"))
+      (test-coding-aware-port num enc))))
+
 (test-end)
