@@ -1,7 +1,7 @@
 ;;;
 ;;; data.queue - queue (fifo) implementation
 ;;;
-;;;   Copyright (c) 2010-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2010-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -67,8 +67,6 @@
 ;;;  Data structures
 ;;;
 (inline-stub
- (declcode
-  (.include <gauche/class.h>))
 
  ;;
  ;; <queue>
@@ -98,18 +96,20 @@
      (set! (Q_LENGTH z) 0 (Q_HEAD z) SCM_NIL (Q_TAIL z) SCM_NIL)
      (return (SCM_OBJ z))))
 
- (define-type <queue> "Queue*" "queue" "QP" "Q")
  (define-cclass <queue>
    "Queue*" "QueueClass" ()
    ((length :type <uint> :c-name "len" :setter #f))
    (allocator (begin (cast void initargs) (return (makeq klass))))
-   (printer (Scm_Printf port "#<queue %d @%p>" (%qlength (Q obj)) obj)))
+   (printer (Scm_Printf port "#<queue %d @%p>" (%qlength (Q obj)) obj))
+   (c-predicate "QP")
+   (unboxer "Q"))
 
  ;;
  ;; <mtqueue>
  ;;
  (define-ctype MtQueue::(.struct
                          (q::Queue
+                          closed::_Bool       ; if TRUE, reject enqueue!
                           maxlen::ScmSmallInt ; negative if unlimited
                           mutex::ScmInternalMutex
                           locker ; thread holding the lock.  see the comment above.
@@ -123,6 +123,7 @@
 
  (.define MTQP (obj) (SCM_ISA obj (& MtQueueClass)))
  (.define MTQ (obj) (cast MtQueue* obj))
+ (.define MTQ_CLOSED (obj) (-> (MTQ obj) closed))
  (.define MTQ_MAXLEN (obj) (-> (MTQ obj) maxlen))
  (.define MTQ_MUTEX (obj) (-> (MTQ obj) mutex))
  (.define MTQ_LOCKER (obj) (-> (MTQ obj) locker))
@@ -134,6 +135,7 @@
  (define-cfn makemtq (klass::ScmClass* maxlen::int)
    (let* ([z::MtQueue* (SCM_NEW_INSTANCE MtQueue klass)])
      (set! (Q_LENGTH z) 0 (Q_HEAD z) SCM_NIL (Q_TAIL z) SCM_NIL
+           (MTQ_CLOSED z) FALSE
            (MTQ_MAXLEN z) maxlen
            (MTQ_LOCKER z) SCM_FALSE
            (MTQ_READER_SEM z) 0)
@@ -152,16 +154,22 @@
          [(SCM_FALSEP maxlen) (set! (MTQ_MAXLEN mtq) -1)]
          [else (SCM_TYPE_ERROR maxlen "non-negative fixnum or #f")]))
 
- (define-type <mtqueue> "MtQueue*" "mt-queue" "MTQP" "MTQ")
  (define-cclass <mtqueue>
    "MtQueue*" "MtQueueClass" ("QueueClass")
    ((max-length :getter "return mtq_maxlen_get(obj);"
-                :setter "mtq_maxlen_set(obj, value);"))
+                :setter "mtq_maxlen_set(obj, value);")
+    (closed     :getter "return SCM_MAKE_BOOL(MTQ_CLOSED(obj));"
+                :setter #f))
    (allocator
     (let* ([ml (Scm_GetKeyword ':max-length initargs SCM_FALSE)])
       (return (makemtq klass (?: (SCM_INTP ml) (SCM_INT_VALUE ml) -1)))))
    (printer
-    (Scm_Printf port "#<mt-queue %d @%p>" (%qlength (Q obj)) obj)))
+    (Scm_Printf port "#<mtqueue %d %s@%p>"
+                (%qlength (Q obj))
+                (?: (MTQ_CLOSED obj) "(closed)" "")
+                obj))
+   (c-predicate "MTQP")
+   (unboxer "MTQ"))
 
  ;; lock macros
  (define-cise-expr big-locked?
@@ -226,6 +234,8 @@
  ;;   Lock Q and execute INIT, and wait while WAIT-CHECK satisfies.  Once
  ;;   WAIT-CHECK returns false, execute a statement DO-OK.  CVAR is a
  ;;   condition variable (-> Q CVAR) to be waited on.
+ ;;   INIT code may set a local variable ERR.  If it is set, it gives up
+ ;;   immediately and throws an error.
  ;;   If TIMEOUT isn't NULL and waiting times out, RETVAL is set with
  ;;   TIMEOUT-VAL.  (DO-OK is supposed to set RETVAL in it).
  (define-cise-stmt do-with-timeout
@@ -235,18 +245,21 @@
       `(let* ([,ts :: (ScmTimeSpec)] [,status :: int 0]
               [,tv :: (volatile ScmObj) ,timeout-val]
               [,pts :: (ScmTimeSpec*) (Scm_GetTimeSpec ,timeout (& ,ts))]
-              [,inited :: int FALSE])
+              [,inited :: int FALSE] [err :: (const char *) NULL])
          (while TRUE
            (with-mtq-mutex-lock ,q
              (unless ,inited ,init (set! ,inited TRUE))
-             (while TRUE
-               (wait-mtq-big-lock ,q)
-               (cond [,wait-check (wait-cv ,q ,cv-slot ,pts ,status)
-                                  (when (== ,status 0) (continue))]
-                     [else ,do-ok (set! ,status 0)])
-               (set! (MTQ_LOCKER ,q) SCM_FALSE)
-               (notify-lockers ,q)
-               (break)))
+             (unless err
+               (while TRUE
+                 (wait-mtq-big-lock ,q)
+                 (cond [,wait-check (wait-cv ,q ,cv-slot ,pts ,status)
+                                    (when (== ,status 0) (continue))]
+                       [else ,do-ok (set! ,status 0)])
+                 (set! (MTQ_LOCKER ,q) SCM_FALSE)
+                 (notify-lockers ,q)
+                 (break))))
+           (when err
+             (Scm_Error "%s: %S" err ,q))
            (case ,status
              [(CW_TIMEDOUT) (set! ,retval ,tv)]
              [(CW_INTR)     (Scm_SigCheck (Scm_VM)) (continue)]) ;restart op
@@ -416,10 +429,13 @@
  (define-cise-stmt q-write-op
    [(_ op q cnt head tail)
     `(if (MTQP ,q)
-       (let* ([ovf::int FALSE])
+       (let* ([ovf::int FALSE]
+              [closed::int FALSE])
          (with-mtq-light-lock ,q
-           (cond [(mtq-overflows ,q ,cnt) (set! ovf TRUE)]
+           (cond [(MTQ_CLOSED ,q) (set! closed TRUE)]
+                 [(mtq-overflows ,q ,cnt) (set! ovf TRUE)]
                  [else (,op ,q ,cnt ,head ,tail) (notify-readers ,q)]))
+         (when closed (Scm_Error "queue is closed: %S" ,q))
          (when ovf (Scm_Error "queue is full: %S" ,q)))
        (,op ,q ,cnt ,head ,tail))])
 
@@ -433,17 +449,22 @@
      (return (SCM_OBJ q))))
 
  ;; API
- (define-cproc enqueue/wait! (q::<mtqueue> obj :optional (timeout #f)
-                                                         (timeout-val #f))
+ (define-cproc enqueue/wait! (q::<mtqueue> obj
+                                           :optional (timeout #f)
+                                                     (timeout-val #f)
+                                                     (close::<boolean> #f))
    (let* ([cell (SCM_LIST1 obj)] [retval (SCM_OBJ q)])
      (.if (defined GAUCHE_HAS_THREADS)
           (do-with-timeout q retval timeout timeout-val writerWait
-                           (begin)
+                           (when (MTQ_CLOSED q)
+                             (set! err "queue is closed"))
                            (?: (!= (MTQ_MAXLEN q) 0)
                                (mtq-overflows q 1)
                                (== (MTQ_READER_SEM q) 0))
                            (begin (enqueue_int (Q q) 1 cell cell)
                                   (set! retval '#t)
+                                  (when close
+                                    (set! (MTQ_CLOSED q) TRUE))
                                   (notify-readers (Q q))))
           (enqueue_int (Q q) 1 cell cell))
      (return retval)))
@@ -484,16 +505,21 @@
      (q-write-op queue-push-int q cnt head tail)
      (return (SCM_OBJ q))))
 
- (define-cproc queue-push/wait! (q::<mtqueue> obj :optional (timeout #f)
-                                                            (timeout-val #f))
+ (define-cproc queue-push/wait! (q::<mtqueue> obj
+                                              :optional (timeout #f)
+                                                        (timeout-val #f)
+                                                        (close::<boolean> #f))
    (let* ([cell (SCM_LIST1 obj)] [retval (SCM_OBJ q)])
      (.if (defined GAUCHE_HAS_THREADS)
           (do-with-timeout q retval timeout timeout-val writerWait
-                           (begin)
+                           (when (MTQ_CLOSED q)
+                             (set! err "queue is closed"))
                            (?: (!= (MTQ_MAXLEN q) 0)
                                (mtq-overflows q 1)
                                (== (MTQ_READER_SEM q) 0))
                            (begin (queue_push_int (Q q) 1 cell cell)
+                                  (when close
+                                    (set! (MTQ_CLOSED q) TRUE))
                                   (notify-readers (Q q))))
           (queue_push_int (Q q) 1 cell cell))
      (return retval)))
@@ -540,11 +566,13 @@
      (return r)))
 
  (define-cproc dequeue/wait! (q::<mtqueue> :optional (timeout #f)
-                                                     (timeout-val #f))
+                                                     (timeout-val #f)
+                                                     (close::<boolean> #f))
    (let* ([retval SCM_UNDEFINED])
      (.if (defined GAUCHE_HAS_THREADS)
           (do-with-timeout q retval timeout timeout-val readerWait
                            (begin (post++ (MTQ_READER_SEM q))
+                                  (when close (set! (MTQ_CLOSED q) TRUE))
                                   (notify-writers (Q q)))
                            (Q_EMPTY_P q)
                            (begin (pre-- (MTQ_READER_SEM q))
