@@ -1,7 +1,7 @@
 ;;;
 ;;; odbm - original dbm interface
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -33,7 +33,7 @@
 
 (define-module dbm.odbm
   (extend dbm)
-  (use srfi-1)
+  (use scheme.list)
   (export <odbm>
           ;; low level funcions
           odbm-init    odbm-close    odbm-closed?
@@ -230,4 +230,3 @@
      (CHECK_ODBM) (TO_DATUM dkey key) (set! dnkey (nextkey dkey))
      (FROM_DATUM SCM_RESULT dnkey)))
  )
-
