@@ -1,7 +1,7 @@
 /*
  * fcntl.h - fcntl interface
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -55,4 +55,18 @@ SCM_CLASS_DECL(Scm_SysFlockClass);
 extern ScmObj Scm_MakeSysFlock(void);
 extern ScmObj Scm_SysFcntl(ScmObj port_or_fd, int op, ScmObj arg);
 
+#ifdef HAVE_SYS_STATVFS_H
+#include <sys/statvfs.h>
+
+typedef struct ScmSysStatvfsRec {
+    SCM_HEADER;
+    struct statvfs vfs;
+} ScmSysStatvfs;
+
+SCM_CLASS_DECL(Scm_SysStatvfsClass);
+#define SCM_CLASS_SYS_STATVFS    (&Scm_SysStatvfsClass)
+#define SCM_SYS_STATVFS(obj)     ((ScmSysStatvfs*)(obj))
+#define SCM_SYS_STATVFS_P(obj)   (SCM_XTYPEP(obj, SCM_CLASS_SYS_STATVFS))
+#endif  /* HAVE_SYS_STATVFS_H */
+
 #endif /* GAUCHE_FCNTL_H */
