@@ -3,8 +3,8 @@
 (use gauche.test)
 (test-start "collections and sequences")
 
-(use util.queue)
-(use srfi-1)
+(use data.queue)
+(use scheme.list)
 (use srfi-13)
 (use gauche.collection)
 (use gauche.sequence)
@@ -20,17 +20,25 @@
   ((strings :initform '() :init-keyword :strings))
   :metaclass <string-seq-meta>)
 
-(define-method call-with-iterator ((seq <string-seq>) proc . opts)
-  (let* ((start (get-keyword :start opts #f))
-         (ss    (slot-ref seq 'strings))
-         (p     (if start (list-tail ss start) ss)))
-    (proc (lambda () (null? p))
-          (lambda () (pop! p)))))
+(define-method call-with-iterator ((seq <string-seq>) proc :key (start #f)
+                                   :allow-other-keys)
+  (let* ([ss (slot-ref seq 'strings)]
+         [p  (if start (list-tail ss start) ss)])
+    (proc (^[] (null? p))
+          (^[] (pop! p)))))
 
-(define-method call-with-builder ((seq <string-seq-meta>) proc . opts)
-  (let ((q (make-queue)))
-    (proc (lambda (item) (enqueue! q (x->string item)))
-          (lambda () (make <string-seq> :strings (dequeue-all! q))))))
+(define-method call-with-reverse-iterator ((seq <string-seq>) proc
+                                           :key (start #f) (end #f)
+                                           :allow-other-keys)
+  (let* ([ss (slot-ref seq 'strings)]
+         [p  (reverse (subseq ss (or start 0) (or end (length ss))))])
+    (proc (^[] (null? p))
+          (^[] (pop! p)))))
+
+(define-method call-with-builder ((seq <string-seq-meta>) proc . _)
+  (let1 q (make-queue)
+    (proc (^[item] (enqueue! q (x->string item)))
+          (^[] (make <string-seq> :strings (dequeue-all! q))))))
 
 (define-method referencer ((seq <string-seq>))
   (lambda (o i . args)
@@ -40,6 +48,9 @@
   (lambda (o i v)
     (set! (ref (slot-ref o 'strings) i) (x->string v))))
 
+(define-method object-equal? ((a <string-seq>) (b <string-seq>))
+  (equal? (slot-ref a 'strings) (slot-ref b 'strings)))
+
 (define (sseq . elements)
   (make <string-seq> :strings (map x->string elements)))
 
@@ -71,45 +82,41 @@
        (fold list* '() "abc" (sseq 'a 'b 'c) '(a b c)))
 
 (test* "fold2" '(21 (6 5 4 3 2 1))
-       (receive r
-           (fold2 (lambda (n s m) (values (+ n s) (cons n m)))
-                  0 '() '(1 2 3 4 5 6))
-         r))
+       (values->list
+        (fold2 (^[n s m] (values (+ n s) (cons n m)))
+               0 '() '(1 2 3 4 5 6))))
 (test* "fold2 (n-ary)" '(195 (5 15 25 4 14 24 3 13 23 2 12 22 1 11 21))
-       (receive r
-           (fold2 (lambda (n0 n1 n2 s m)
-                    (values (+ n0 n1 n2 s)
-                            (list* n0 n1 n2 m)))
-                  0 '() '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))
-         r))
+       (values->list
+        (fold2 (^[n0 n1 n2 s m]
+                 (values (+ n0 n1 n2 s)
+                         (list* n0 n1 n2 m)))
+               0 '() '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))))
 
 (test* "fold3" '(21 720 (6 5 4 3 2 1))
-       (receive r
-           (fold3 (lambda (n s m l) (values (+ n s) (* n m) (cons n l)))
-                  0 1 '() '(1 2 3 4 5 6))
-         r))
+       (values->list
+        (fold3 (^[n s m l] (values (+ n s) (* n m) (cons n l)))
+               0 1 '() '(1 2 3 4 5 6))))
 (test* "fold3 (n-ary)" '(195 275701345920000
                          (5 15 25 4 14 24 3 13 23 2 12 22 1 11 21))
-       (receive r
-           (fold3 (lambda (n0 n1 n2 s m l)
-                    (values (+ n0 n1 n2 s)
-                            (* n0 n1 n2 m)
-                            (list* n0 n1 n2 l)))
-                  0 1 '()
-                  '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))
-         r))
+       (values->list
+        (fold3 (^[n0 n1 n2 s m l]
+                 (values (+ n0 n1 n2 s)
+                         (* n0 n1 n2 m)
+                         (list* n0 n1 n2 l)))
+               0 1 '()
+               '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))))
 
 (test* "map (list)" '(2 4 6 8 10)
-       (map (lambda (x) (* x 2)) '(1 2 3 4 5)))
+       (map (^x (* x 2)) '(1 2 3 4 5)))
 (test* "map (vector)" '(2 4 6 8 10)
-       (map (lambda (x) (* x 2)) '#(1 2 3 4 5)))
+       (map (^x (* x 2)) '#(1 2 3 4 5)))
 (test* "map (string)" '(2 4 6 8 10)
-       (map (lambda (x) (* (digit->integer x) 2)) "12345"))
+       (map (^x (* (digit->integer x) 2)) "12345"))
 (test* "map (hash-table)" '((c . "c") (b . "b") (a . "a"))
        (map identity (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c")))
        (cut lset= equal? <> <>))
 (test* "map (custom)" '(2 4 6 8 10)
-       (map (lambda (x) (* (string->number x) 2))
+       (map (^x (* (string->number x) 2))
             (sseq 1 2 3 4 5)))
 
 (test* "map (n-ary)" '(3 5 7 9 11)
@@ -119,25 +126,25 @@
 (test* "map (n-ary)" '(3 5 7 9 11)
        (map + '#(1 2 3 4 5) '#(2 3 4 5 6)))
 (test* "map (n-ary)" '("123" "234" "345" "456")
-       (map (lambda arg (apply string-append (map x->string arg)))
+       (map (^ arg (apply string-append (map x->string arg)))
             "12345" (sseq 2 3 4 5) '#(3 4 5 6 7 8)))
 
 (test* "map-to (list->vector)" '#(2 4 6 8)
-       (map-to <vector> (lambda (x) (* x 2)) '(1 2 3 4)))
+       (map-to <vector> (^x (* x 2)) '(1 2 3 4)))
 (test* "map-to (list->string)" "2468"
-       (map-to <string> (lambda (x) (integer->digit (* x 2)))
+       (map-to <string> (^x (integer->digit (* x 2)))
                '(1 2 3 4)))
 (test* "map-to (list->custom)" '("2" "4" "6" "8")
        (slot-ref
-        (map-to <string-seq> (lambda (x) (* x 2)) '(1 2 3 4))
+        (map-to <string-seq> (^x (* x 2)) '(1 2 3 4))
         'strings))
 (test* "map-to (vector->list" '(2 4 6 8)
-       (map-to <list> (lambda (x) (* x 2)) '#(1 2 3 4)))
+       (map-to <list> (^x (* x 2)) '#(1 2 3 4)))
 (test* "map-to (vector->string" "1234"
        (map-to <string> integer->digit '#(1 2 3 4)))
 (test* "map-to (vector->custom)" '("2" "4" "6" "8")
        (slot-ref
-        (map-to <string-seq> (lambda (x) (* x 2)) '#(1 2 3 4))
+        (map-to <string-seq> (^x (* x 2)) '#(1 2 3 4))
         'strings))
 
 (test* "map-to (nary)" '#(3 5 7 9 11)
@@ -148,45 +155,61 @@
        (map-to <vector> + '(1 2 3 4 5) '#(2 3 4 5)))
 
 (test* "map-accum" '((45 30 15) 5)
-       (receive r 
-           (map-accum (lambda (elt seed) (values (* elt seed) seed))
-                      5 '(9 6 3))
-         r))
+       (values->list
+        (map-accum (^[elt seed] (values (* elt seed) seed))
+                   5 '(9 6 3))))
 (test* "map-accum" '((10 28 88) 19)
-       (receive r
-           (map-accum (lambda (elt seed) (values (* elt seed) (+ elt seed)))
-                      5 '(2 4 8))
-         r))
+       (values->list
+        (map-accum (^[elt seed] (values (* elt seed) (+ elt seed)))
+                   5 '(2 4 8))))
 (test* "map-accum (nary)" '((10 11 16) 15)
-       (receive r
-           (map-accum (lambda (x y seed)
-                        (values (+ x y seed) (+ x seed)))
-                      1 '(2 4 8) '(7 4 1))
-         r))
-
+       (values->list
+        (map-accum (^[x y seed] (values (+ x y seed) (+ x seed)))
+                   1 '(2 4 8) '(7 4 1))))
 
 (test* "for-each (list)" '(5 4 3 2 1)
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x)) '(1 2 3 4 5))
-         p))
+       (rlet1 p '()
+         (for-each (^x (push! p x)) '(1 2 3 4 5))))
 (test* "for-each (vector)" '(5 4 3 2 1)
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x)) '#(1 2 3 4 5))
-         p))
+       (rlet1 p '()
+         (for-each (^x (push! p x)) '#(1 2 3 4 5))))
 (test* "for-each (string)" '(#\5 #\4 #\3 #\2 #\1)
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x)) "12345")
-         p))
+       (rlet1 p '()
+         (for-each (^x (push! p x)) "12345")))
 (test* "for-each (hash-table)" '((c . "c") (b . "b") (a . "a"))
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x))
-                   (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c")))
-         p)
+       (rlet1 p '()
+         (for-each (^x (push! p x))
+                   (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c"))))
        (cut lset= equal? <> <>))
 (test* "for-each (custom)" '("5" "4" "3" "2" "1")
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x)) (sseq 1 2 3 4 5))
-         p))
+       (rlet1 p '()
+         (for-each (^x (push! p x)) (sseq 1 2 3 4 5))))
+
+(test* "map$" '(#\A #\B #\C #\D #\E)
+       ((map$ char-upcase) "abcde"))
+(test* "for-each$" '(#\A #\B #\C #\D #\E)
+       (rlet1 r '()
+         ((for-each$ (^x (push! r x))) "EDCBA")))
+(test* "fold$"  '(#\A #\B #\C #\D #\E . #t)
+       ((fold$ cons) #t "EDCBA"))
+(test* "fold$"  '(#\A #\B #\C #\D #\E . #f)
+       ((fold$ cons #f) "EDCBA"))
+(test* "fold2$"  '(195 (5 15 25 4 14 24 3 13 23 2 12 22 1 11 21))
+       (values->list
+        ((fold2$ (^[n0 n1 n2 s m]
+                   (values (+ n0 n1 n2 s)
+                           (list* n0 n1 n2 m)))
+                 0 '())
+         '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))))
+(test* "fold3$" '(195 275701345920000
+                      (5 15 25 4 14 24 3 13 23 2 12 22 1 11 21))
+       (values->list
+        ((fold3$ (^[n0 n1 n2 s m l]
+                   (values (+ n0 n1 n2 s)
+                           (* n0 n1 n2 m)
+                           (list* n0 n1 n2 l)))
+                 0 1 '())
+         '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))))
 
 (test-section "searching and selection")
 
@@ -197,17 +220,17 @@
 (test* "find (string)" #\a
        (find char-lower-case? "YAEUB4309aBrnar"))
 (test* "find (hash-table)" '(b . "b")
-       (find (lambda (x) (string=? "b" (cdr x)))
+       (find (^x (string=? "b" (cdr x)))
              (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c"))))
 (test* "find (custom)" "zoo"
-       (find (lambda (s) (= (size-of s) 3))
+       (find (^s (= (size-of s) 3))
              (sseq 'najr 'ej 'zoo 'bunr)))
 
 (define (test-find-minmax msg ex-min ex-max coll . args)
-  (test* #`"find-min (,|msg|)" ex-min (apply find-min coll args))
-  (test* #`"find-max (,|msg|)" ex-max (apply find-max coll args))
-  (test* #`"find-min&max (,|msg|)" (list ex-min ex-max)
-         (receive r (apply find-min&max coll args) r)))
+  (test* #"find-min (~msg)" ex-min (apply find-min coll args))
+  (test* #"find-max (~msg)" ex-max (apply find-max coll args))
+  (test* #"find-min&max (~msg)" (list ex-min ex-max)
+         (values->list (apply find-min&max coll args))))
 
 
 (test-find-minmax "list 0" -1 9
@@ -240,11 +263,11 @@
 (test* "filter (string)" '(#\a #\r #\b)
        (filter char-lower-case? "UBaBrGLbO"))
 (test* "filter (hash-table)" '((b . "b"))
-       (filter (lambda (x) (string=? "b" (cdr x)))
+       (filter (^x (string=? "b" (cdr x)))
                (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c")))
        (cut lset= equal? <> <>))
 (test* "filter (custom)" '("zoo" "zn")
-       (filter (lambda (s) (string-prefix? "z" s))
+       (filter (^s (string-prefix? "z" s))
                (sseq 'urnb 'zoo 'nbak 'zn 'run)))
 
 (test* "filter-to (vector)" '#(2 4 6)
@@ -256,7 +279,7 @@
 (test* "filter-to (hash-table)" '((b . "b") (z . "b"))
        (hash-table-map
         (filter-to <hash-table>
-                   (lambda (x) (string=? "b" (cdr x)))
+                   (^x (string=? "b" (cdr x)))
                    (hash-table 'eq?
                                '(a . "a") '(b . "b")
                                '(c . "c") '(z . "b")))
@@ -270,11 +293,11 @@
 (test* "remove (string)" '(#\U #\B #\B #\G #\L #\O)
        (remove char-lower-case? "UBaBrGLbO"))
 (test* "remove (hash-table)" '((a . "a") (c . "c"))
-       (remove (lambda (x) (string=? "b" (cdr x)))
+       (remove (^x (string=? "b" (cdr x)))
                (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c")))
        (cut lset= equal? <> <>))
 (test* "remove (custom)" '("urnb" "nbak" "run")
-       (remove (lambda (s) (string-prefix? "z" s))
+       (remove (^s (string-prefix? "z" s))
                (sseq 'urnb 'zoo 'nbak 'zn 'run)))
 
 (test* "remove-to (vector)" '#(1 3 5 7)
@@ -282,7 +305,7 @@
 (test* "remove-to (hash-table)" '((a . "a") (c . "c"))
        (hash-table-map
         (remove-to <hash-table>
-                   (lambda (x) (string=? "b" (cdr x)))
+                   (^x (string=? "b" (cdr x)))
                    (hash-table 'eq?
                                '(a . "a") '(b . "b")
                                '(c . "c") '(z . "b")))
@@ -294,22 +317,17 @@
         'strings))
 
 (test* "partition (list)" '((2 4 6) (1 3 5 7))
-       (receive r (partition even? '(1 2 3 4 5 6 7))
-         r))
+       (values->list (partition even? '(1 2 3 4 5 6 7))))
 (test* "partition (vector)" '((2 4 6) (1 3 5 7))
-       (receive r (partition even? '#(1 2 3 4 5 6 7))
-         r))
+       (values->list (partition even? '#(1 2 3 4 5 6 7))))
 (test* "partition (custom)" '(("2" "4" "6") ("1" "3" "5" "7"))
-       (receive r (partition (lambda (e) (even? (string->number e)))
-                             (sseq 1 2 3 4 5 6 7))
-         r))
+       (values->list (partition (^e (even? (string->number e)))
+                                (sseq 1 2 3 4 5 6 7))))
 
 (test* "partition-to (string)" '("ACE" "bdf")
-       (receive r (partition-to <string> char-upper-case? "AbCdEf")
-         r))
+       (values->list (partition-to <string> char-upper-case? "AbCdEf")))
 (test* "partition-to (vector)" '(#(2 4 6) #(1 3 5 7))
-       (receive r (partition-to <vector> even? '#(1 2 3 4 5 6 7))
-         r))
+       (values->list (partition-to <vector> even? '#(1 2 3 4 5 6 7))))
 
 (test-section "miscellaneous")
 
@@ -319,6 +337,8 @@
 (test* "size-of (custom)" 5 (size-of (sseq 1 2 3 4 5)))
 (test* "size-of (hash-table)" 4
        (size-of (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c") '(z . "b"))))
+(test* "size-of (char-set)" 0 (size-of (char-set)))
+(test* "size-of (char-set)" 5 (size-of #[abAB0]))
 
 (test* "coerce-to (list->list)" '(1 2 3)
        (coerce-to <list> '(1 2 3)))
@@ -370,7 +390,7 @@
        (group-collection '(1 2 3 2 3 1 2 1 2 3 2 3)))
 (test* "group-collection w/test" '((1 3 3 1 1 3 3) (2 2 2 2 2))
        (group-collection '(1 2 3 2 3 1 2 1 2 3 2 3)
-                         :test (lambda (x y) (= (modulo x 2) (modulo y 2)))))
+                         :test (^[x y] (= (modulo x 2) (modulo y 2)))))
 (test* "group-collection w/key" '(((1 a) (1 c)) ((2 b) (2 q)) ((3 c) (3 d)))
        (group-collection '((1 a) (2 b) (3 c) (1 c) (3 d) (2 q))
                          :key car))
@@ -388,79 +408,67 @@
 (test* "ref (custom)" "3" (ref (sseq 1 2 3 4 5) 2))
 
 (test* "(setter ref) (list)" '(1 2 a 4 5)
-       (let ((x (list 1 2 3 4 5)))
+       (let1 x (list 1 2 3 4 5)
          (set! (ref x 2) 'a) x))
 (test* "(setter ref) (vector)" '#(1 2 a 4 5)
-       (let ((x (vector 1 2 3 4 5)))
+       (let1 x (vector 1 2 3 4 5)
          (set! (ref x 2) 'a) x))
 (test* "(setter ref) (string)" "12a45"
-       (let ((x (string-copy "12345")))
+       (let1 x (string-copy "12345")
          (set! (ref x 2) #\a) x))
 (test* "(setter ref) (custom)" '("1" "2" "a" "4" "5")
-       (let ((x (sseq 1 2 3 4 5)))
+       (let1 x (sseq 1 2 3 4 5)
          (set! (ref x 2) 'a) (slot-ref x 'strings)))
 
 (test* "subseq (list)" '(3 4 5)
        (subseq '(1 2 3 4 5) 2))
 (test* "subseq (list)" '(3 4)
        (subseq '(1 2 3 4 5) 2 4))
-(test* "subseq (list)" '(1 2 3 4)
-       (subseq '(1 2 3 4 5) 0 -1))
 (test* "subseq (vector)" '#(3 4 5)
        (subseq '#(1 2 3 4 5) 2))
 (test* "subseq (vector)" '#(3 4)
        (subseq '#(1 2 3 4 5) 2 4))
-(test* "subseq (vector)" '#(1 2 3 4)
-       (subseq '#(1 2 3 4 5) 0 -1))
+(test* "subseq (bitvector)" '#*0011001
+       (subseq '#*1010011001 3))
+(test* "subseq (bitvector)" '#*0011
+       (subseq '#*1010011001 3 7))
 (test* "subseq (string)" "345"
        (subseq "12345" 2))
 (test* "subseq (string)" "34"
        (subseq "12345" 2 4))
-(test* "subseq (string)" "1234"
-       (subseq "12345" 0 -1))
 (test* "subseq (custom)" '("3" "4" "5")
        (slot-ref (subseq (sseq 1 2 3 4 5) 2) 'strings))
 (test* "subseq (custom)" '("3" "4")
        (slot-ref (subseq (sseq 1 2 3 4 5) 2 4) 'strings))
-(test* "subseq (custom)" '("1" "2" "3" "4")
-       (slot-ref (subseq (sseq 1 2 3 4 5) 0 -1) 'strings))
 
 (test* "for-each-with-index (list)" '((2 . c) (1 . b) (0 . a))
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) '(a b c))
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) '(a b c))))
 (test* "for-each-with-index (vector)" '((2 . c) (1 . b) (0 . a))
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) '#(a b c))
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) '#(a b c))))
 (test* "for-each-with-index (string)" '((2 . #\c) (1 . #\b) (0 . #\a))
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) "abc")
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) "abc")))
 (test* "for-each-with-index (custom)" '((2 . "c") (1 . "b") (0 . "a"))
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e)))
-                              (sseq 'a 'b 'c))
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e)))
+                              (sseq 'a 'b 'c))))
 (test* "for-each-with-index (custom)" '((2 "c" c c) (1 "b" b b) (0 "a" a a))
-       (let1 r '()
-         (for-each-with-index (lambda x (push! r x))
+       (rlet1 r '()
+         (for-each-with-index (^ x (push! r x))
                               (sseq 'a 'b 'c)
                               '(a b c)
-                              '#(a b c))
-         r))
+                              '#(a b c))))
 (test* "for-each-with-index (boundary)" '()
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) '())
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) '())))
 (test* "for-each-with-index (boundary)" '()
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) (sseq))
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) (sseq))))
 (test* "for-each-with-index (boundary)" '()
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) (sseq) '())
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) (sseq) '())))
 
 (test* "map-with-index (list)" '((0 . a) (1 . b) (2 . c))
        (map-with-index cons '(a b c)))
@@ -509,7 +517,7 @@
 (test* "fold-with-index (custom)" '((2 . "a") (1 . "b") (0 . "c"))
        (fold-with-index acons '() (sseq 'c 'b 'a)))
 (test* "fold-with-index (custom)" '((2 "a" a a) (1 "b" b b) (0 "c" c c))
-       (fold-with-index (lambda (i x y z r)
+       (fold-with-index (^[i x y z r]
                           (cons (list i x y z) r))
                         '()
                         (sseq 'c 'b 'a) '(c b a) '#(c b a)))
@@ -521,27 +529,27 @@
        (fold-with-index acons '() (sseq) '() '#()))
 
 (test* "find-with-index (list)" '(2 c)
-       (receive r (find-with-index (cut eq? 'c <>) '(a b c d e)) r))
+       (values->list (find-with-index (cut eq? 'c <>) '(a b c d e))))
 (test* "find-with-index (list)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'f <>) '(a b c d e)) r))
+       (values->list (find-with-index (cut eq? 'f <>) '(a b c d e))))
 (test* "find-with-index (vector)" '(2 c)
-       (receive r (find-with-index (cut eq? 'c <>) '#(a b c d e)) r))
+       (values->list (find-with-index (cut eq? 'c <>) '#(a b c d e))))
 (test* "find-with-index (vector)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'f <>) '#(a b c d e)) r))
+       (values->list (find-with-index (cut eq? 'f <>) '#(a b c d e))))
 (test* "find-with-index (string)" '(2 #\c)
-       (receive r (find-with-index (cut eqv? #\c <>) "abcde") r))
+       (values->list (find-with-index (cut eqv? #\c <>) "abcde")))
 (test* "find-with-index (string)" '(#f #f)
-       (receive r (find-with-index (cut eqv? #\f <>) "abcde") r))
+       (values->list (find-with-index (cut eqv? #\f <>) "abcde")))
 (test* "find-with-index (custom)" '(2 "c")
-       (receive r (find-with-index (cut equal? "c" <>) (sseq 'a 'b 'c 'd 'e)) r))
+       (values->list (find-with-index (cut equal? "c" <>) (sseq 'a 'b 'c 'd 'e))))
 (test* "find-with-index (custom)" '(#f #f)
-       (receive r (find-with-index (cut equal? "f" <>) (sseq 'a 'b 'c 'd 'e)) r))
+       (values->list (find-with-index (cut equal? "f" <>) (sseq 'a 'b 'c 'd 'e))))
 (test* "find-with-index (boundary)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'c <>) '()) r))
+       (values->list (find-with-index (cut eq? 'c <>) '())))
 (test* "find-with-index (boundary)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'c <>) '#()) r))
+       (values->list (find-with-index (cut eq? 'c <>) '#())))
 (test* "find-with-index (boundary)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'c <>) (sseq)) r))
+       (values->list (find-with-index (cut eq? 'c <>) (sseq))))
 
 (test* "find-index (list)" 2
        (find-index (cut eq? 'c <>) '(a b c d e)))
@@ -573,6 +581,25 @@
 (test* "fold-right (different lengths)" '(#\a 0 u #\b 1 v)
        (fold-right cons* '() "ab" '#(0 1 2 3 4) '(u v w x y z)))
 
+;; reverse iterator
+(define (generic-reverse seq start end)
+  ($ call-with-builder (class-of seq)
+     (^[add! get]
+       ($ call-with-reverse-iterator seq
+          (^[end? next]
+            (let loop ()
+              (if (end?)
+                (get)
+                (begin (add! (next)) (loop)))))
+          :start start :end end))))
+
+(test* "reverse iterator (vector)" '#(d c)
+       (generic-reverse '#(a b c d e) 2 4))
+(test* "reverse iterator (string)" "dc"
+       (generic-reverse "abcde" 2 4))
+(test* "reverse iterator (custom)" (sseq 'd 'c)
+       (generic-reverse (sseq 'a 'b 'c 'd 'e) 2 4))
+
 (test* "group-sequence" '((1 1 1) (2) (3) (4 4) (2 2) (3) (1 1) (3))
        (group-sequence '(1 1 1 2 3 4 4 2 2 3 1 1 3)))
 (test* "group-sequence w/key" '((1 1 1) (2) (3) (4 4 2 2) (3 1 1 3))
@@ -581,18 +608,161 @@
        (group-sequence '(1 1 1 2 3 4 4 2 2 3 1 1 3) :key (cut < <> 3)))
 (test* "group-sequence w/test" '((1 1 1) (2) (3) (4 4 2 2) (3 1 1 3))
        (group-sequence '(1 1 1 2 3 4 4 2 2 3 1 1 3)
-                       :test (lambda (x y)
-                               (= (modulo x 2) (modulo y 2)))))
+                       :test (^[x y] (= (modulo x 2) (modulo y 2)))))
+
+(test* "group-contiguous-sequence" '()
+       (group-contiguous-sequence '()))
+(test* "group-contiguous-sequence" '((1))
+       (group-contiguous-sequence '(1)))
+(test* "group-contiguous-sequence" '((1 2))
+       (group-contiguous-sequence '(1 2)))
+(test* "group-contiguous-sequence" '((1) (3))
+       (group-contiguous-sequence '(1 3)))
+(test* "group-contiguous-sequence" '((1 2 3))
+       (group-contiguous-sequence '(1 2 3)))
+(test* "group-contiguous-sequence" '((1) (3 4 5))
+       (group-contiguous-sequence '(1 3 4 5)))
+(test* "group-contiguous-sequence"
+       '((1 2 3 4) (7 8 9) (11) (13 14) (16))
+       (group-contiguous-sequence '(1 2 3 4 7 8 9 11 13 14 16)))
+(test* "group-contiguous-sequence" '((#\a #\b #\c) (#\e) (#\g #\h))
+       ($ group-contiguous-sequence "abcegh"
+          :next (^c (integer->char (+ 1 (char->integer c))))
+          :test char=?))
+(test* "group-contiguous-sequence squeeze" '()
+       (group-contiguous-sequence '() :squeeze #t))
+(test* "group-contiguous-sequence squeeze" '((1))
+       (group-contiguous-sequence '(1) :squeeze #t))
+(test* "group-contiguous-sequence squeeze" '((1 2))
+       (group-contiguous-sequence '(1 2) :squeeze #t))
+(test* "group-contiguous-sequence squeeze" '((1) (3))
+       (group-contiguous-sequence '(1 3) :squeeze #t))
+(test* "group-contiguous-sequence squeeze" '((1 3))
+       (group-contiguous-sequence '(1 2 3) :squeeze #t))
+(test* "group-contiguous-sequence squeeze" '((1) (3 5))
+       (group-contiguous-sequence '(1 3 4 5) :squeeze #t))
+(test* "group-contiguous-sequence squeeze" '((1 4) (7 9) (11) (13 14) (16))
+       (group-contiguous-sequence '(1 2 3 4 7 8 9 11 13 14 16) :squeeze #t))
+(test* "group-contiguous-sequence squeeze" '((#\a #\c) (#\e) (#\g #\h))
+       ($ group-contiguous-sequence "abcegh"
+          :next (^c (integer->char (+ 1 (char->integer c))))
+          :test char=?
+          :squeeze #t))
+
+(test* "delete-neighbor-dups" '(1 2 3 4 2 3 1 3)
+       (delete-neighbor-dups '(1 1 1 2 3 4 4 2 2 3 1 1 3)))
+(test* "delete-neighbor-dups" '#(1 2 3 4 2 3 1 3)
+       (delete-neighbor-dups '#(1 1 1 2 3 4 4 2 2 3 1 1 3)))
+(test* "delete-neighbor-dups" "12342313"
+       (delete-neighbor-dups "1112344223113"))
+(test* "delete-neighbor-dups" '()
+       (delete-neighbor-dups '()))
+(test* "delete-neighbor-dups" '(1)
+       (delete-neighbor-dups '(1 1 1 1 1)))
+(test* "delete-neighbor-dups w/key" '(1 2 3 4 3)
+       (delete-neighbor-dups '(1 1 1 2 3 4 4 2 2 3 1 1 3)
+                             :key (cut modulo <> 2)))
+(test* "delete-neighbor-dups w/test" '(1 2 3 4 3)
+       (delete-neighbor-dups '(1 1 1 2 3 4 4 2 2 3 1 1 3)
+                             :test (^[x y] (= (modulo x 2) (modulo y 2)))))
+(test* "delete-neighbor-dups w/range" "12342"
+       (delete-neighbor-dups "1112344223113" :start 1 :end 9))
+(test* "delete-neighbor-dups w/range" "2342"
+       (delete-neighbor-dups "1112344223113" :start 3 :end 9))
+(test* "delete-neighbor-dups w/range" ""
+       (delete-neighbor-dups "1112344223113" :start 2 :end 2))
+(test* "delete-neighbor-dups w/range" "4"
+       (delete-neighbor-dups "1112344223113" :start 5 :end 6))
+(test* "delete-neighbor-dups w/range" "4"
+       (delete-neighbor-dups "1112344223113" :start 5 :end 7))
+
+(test* "delete-neighbor-dups!" '(8 . #(1 2 3 4 2 3 1 3 2 3 1 1 3))
+       (let* ([v (vector-copy '#(1 1 1 2 3 4 4 2 2 3 1 1 3))]
+              [r (delete-neighbor-dups! v)])
+         (cons r v)))
+(test* "delete-neighbor-dups! w/range"
+       '(10 . #(1 1 1 2 3 4 2 3 1 3 1 1 3))
+       (let* ([v (vector-copy '#(1 1 1 2 3 4 4 2 2 3 1 1 3))]
+              [r (delete-neighbor-dups! v :start 2)])
+         (cons r v)))
+(test* "delete-neighbor-dups! w/range"
+       '(10 . (1 1 1 2 3 4 2 3 1 3 1 1 3))
+       (let* ([v (list-copy '(1 1 1 2 3 4 4 2 2 3 1 1 3))]
+              [r (delete-neighbor-dups! v :start 2)])
+         (cons r v)))
+
+(let ([data '(1 1 1 2 3 4 4 2 2 3 1 1 3 3)])
+  (test* "delete-neighbor-dups-squeeze!" '(#t #t)
+         (let* ([l0 (list-copy data)]
+                [l2 (delete-neighbor-dups-squeeze! l0)])
+           (list (eq? l0 l2) (equal? l2 (delete-neighbor-dups l0)))))
+  (test* "delete-neighbor-dups-squeeze! w/range"
+         '(1 2 3 4 2 3 1 3)
+         (delete-neighbor-dups-squeeze! (list-copy data) :start 1))
+  (test* "delete-neighbor-dups-squeeze! w/range"
+         '(1 2 3 4 2)
+         (delete-neighbor-dups-squeeze! (list-copy data) :end 8))
+  )
+
+(test* "sequence-contains" '(#f 0 9 #f)
+       (let1 pat "abrabrabre"
+         (list (sequence-contains "abracadabra" pat)
+               (sequence-contains "abrabrabrebrea" pat)
+               (sequence-contains "abrabrabrabrabrabrebra" pat)
+               (sequence-contains "abrabrabrabrabrabrabra" pat))))
+(test* "sequence-contains" 0
+       (sequence-contains "abracadabra" '()))
+
+(let ([data
+       ;; (input needle before after)
+       '(((1 2 3 4 5 4 5 6 7 8 9) (4 5 4 5 6) (1 2 3) (4 5 4 5 6 7 8 9))
+         ((4 5 4 5 6)             (4 5 4 5 6) ()      (4 5 4 5 6))
+         ((4 5 4 5 5 4 5 4 5 6)   (4 5 4 5 6) (4 5 4 5 5) (4 5 4 5 6))
+         (()                      (4 5 4 5 6) ()      ())
+         ((4 5 6 4 5 4 5 1)       (4 5 4 5 6) (4 5 6 4 5 4 5 1) ())
+         ((4 5 6 4 5 4 5 1)       ()          () (4 5 6 4 5 4 5 1))
+         )])
+  (define (tester !? in needle before after)
+    (test* (format "break-list-by-sequence~a ~s" (if !? '! "") in)
+           (list before after)
+           (let ([data (list-copy in)])
+             (receive (a b) ((if !?
+                               break-list-by-sequence!
+                               break-list-by-sequence)
+                             data needle)
+               (and (or (not !?)
+                        (null? a)
+                        (eq? a data))  ; check if head part is shared
+                    (list a b))))))
+
+  (dolist [datum data]
+    (apply tester #f datum)
+    (apply tester #t datum)))
+
+(test* "common-prefix" '(a b c)
+       (common-prefix-to <list> '(a b c d e) '(a b c e d)))
+(test* "common-prefix" '(a b c)
+       (common-prefix '(a b c d e) '#(a b c e d)))
+(test* "common-prefix" '#(a b c)
+       (common-prefix '#(a b c e d) '(a b c d e)))
+(test* "common-prefix" "xyz"
+       (common-prefix "xyz" "xyzw"))
+(test* "common-prefix" ""
+       (common-prefix "xyz" "abc"))
+(test* "common-prefix" '()
+       (common-prefix '(a b c) ""))
+(test* "common-prefix" "ABC"
+       (common-prefix "ABCE" "abcd" :test char-ci=?))
 
 (define (permute-tester msg expected source order . fallback)
   (define (unit type elt-coercer order-type)
-    (test* #`"permute,msg ,type by ,order-type"
+    (test* #"permute~msg ~type by ~order-type"
            (map-to type elt-coercer expected)
            (apply permute
                   (map-to type elt-coercer source)
                   (coerce-to order-type order)
                   (map elt-coercer fallback)))
-    (test* #`"permute!,msg ,type by ,order-type"
+    (test* #"permute!~msg ~type by ~order-type"
            (if (= (size-of source) (size-of expected))
              (map-to type elt-coercer expected)
              *test-error*)
@@ -605,7 +775,7 @@
   (dolist (order-type `(,<list> ,<vector>))
     (dolist (t `((,<list> ,values)
                  (,<vector> ,values)
-                 (,<string> ,(lambda (elt) (string-ref (x->string elt) 0)))))
+                 (,<string> ,(^[elt] (string-ref (x->string elt) 0)))))
       (unit (car t) (cadr t) order-type)))
   )
 
@@ -617,11 +787,11 @@
   (define (cmp a b)
     (lset= eqv? (coerce-to <list> a) (coerce-to <list> b)))
   (define (unit type)
-    (test* #`"shuffle ,type"
+    (test* #"shuffle ~type"
            (coerce-to type source)
            (shuffle (coerce-to type source))
            cmp)
-    (test* #`"shuffle! ,type"
+    (test* #"shuffle! ~type"
            (coerce-to type source)
            (let1 imp (coerce-to type source)
              (shuffle! imp)
