@@ -7,7 +7,7 @@
 
 (use gauche.test)
 (use gauche.sequence)
-(use srfi-1)
+(use scheme.list)
 (use srfi-13)
 (use srfi-60)
 (test-start "generators")
@@ -99,7 +99,7 @@
     (t r bits->generator #xcafebabe)
     (t f reverse-bits->generator #xcafebabe)
     ))
-                                 
+
 ;; file generators
 (let-syntax ((t (syntax-rules ()
                   [(t fn expect data)
@@ -155,13 +155,13 @@
                                                            '(2 . c))))
                 car))
 (test* "x->generator <tree-map>" '((0 . a) (1 . b) (2 . c))
-       (generator->list (x->generator (alist->tree-map 
+       (generator->list (x->generator (alist->tree-map
                                        '((0 . a)
                                          (1 . b)
                                          (2 . c))
                                        = <))))
 (test* "x->generator <tree-map>" '((2 . c) (1 . b) (0 . a))
-       (generator->list (x->generator (alist->tree-map 
+       (generator->list (x->generator (alist->tree-map
                                        '((0 . a)
                                          (1 . b)
                                          (2 . c))
@@ -237,6 +237,11 @@
                                  0
                                  (^s (giota 10)))))
 
+(test* "giterate" '(0 2 4 6 8 10 12 14 16 18)
+       (generator->list (giterate (pa$ + 2) 0) 10))
+(test* "giterate1" '(2 4 6 8 10 12 14 16 18 20)
+       (generator->list (giterate1 (pa$ + 2) 0) 10))
+
 (define-syntax test-list-like
   (syntax-rules ()
     [(_  gfn lfn src ...)
@@ -441,4 +446,24 @@
                                    (giota 20)
                                    :open #t :repeat #t)))
 
+;; srfi-42+generator
+;; NB: This test should be moved to ext/srfi when srfi-42 becomes precompiled.
+(use srfi-42)
+
+(test* "srfi-42 + generator" '(0 2 4 6 8 10)
+       (list-ec (:while (:generator x (giota)) (<= x 5))
+                (* x 2)))
+
+(define-class <my-coll> (<collection>)
+  ((elements :init-keyword :elements)))
+(define-method call-with-iterator ((c <my-coll>) proc)
+  (define xs (~ c'elements))
+  (proc (^[] (null? xs))
+        (^[] (pop! xs))))
+
+(test* "srfi-42 + collection" '(2 4 6 8 10)
+       (list-ec (: x (make <my-coll> :elements '(1 2 3 4 5)))
+                (+ x x)))
+
+
 (test-end)
