@@ -317,7 +317,7 @@
                      (set! (,p2-z v) 35)
                      (list (,p2-x v) (,p2-y v) (,p2-z v))))
        )))
-  
+
 (pseudo-record-test <list> list)
 (pseudo-record-test <vector> vector)
 (pseudo-record-test <u8vector> u8vector)
@@ -435,6 +435,53 @@
                (sub1-d z)
                (sub1-b z))))
 
+;; inheriting mixin class
+(define-record-type (myseq #f :mixins (<sequence>)) #t #t elements)
+(define-method referencer ((s myseq))
+  (^[o i] (ref (myseq-elements o) i)))
+(define-method modifier ((s myseq))
+  (^[o i v] (set! (ref (myseq-elements o) i) v)))
+(define-method call-with-iterator ((s myseq) proc)
+  (call-with-iterator (myseq-elements s) proc))
+
+(let1 seq (make-myseq (vector 'a 'b 'c 'd 'e))
+  (test* "inheritance (mixin) pred" #t (is-a? seq <sequence>))
+  (test* "inheritance (mixin) pred" #t (is-a? seq <record>))
+  (test* "inheritance (mixin) ref" '(a b c d e)
+         (map (cut ref seq <>) (iota 5)))
+  (test* "inheritance (mixin) iterator" '(a b c d e)
+         (coerce-to <list> seq))
+  (test* "inheritance (mixin) modifier" '(a b z d e)
+         (begin (set! (ref seq 2) 'z)
+                (coerce-to <list> seq)))
+  )
+
+(define-record-type (myseq2 myseq) #t #t)
+(let1 seq (make-myseq2 (vector 'a 'b 'c 'd 'e))
+  (test* "inheritance (mixin) pred" #t (is-a? seq <sequence>))
+  (test* "inheritance (mixin) pred" #t (is-a? seq <record>))
+  (test* "inheritance (mixin) ref" '(a b c d e)
+         (map (cut ref seq <>) (iota 5)))
+  (test* "inheritance (mixin) iterator" '(a b c d e)
+         (coerce-to <list> seq))
+  (test* "inheritance (mixin) modifier" '(a b z d e)
+         (begin (set! (ref seq 2) 'z)
+                (coerce-to <list> seq)))
+  )
+
+(define-class <myseq3-meta> (<record-meta>) ())
+(define-record-type (myseq3 #f :mixins (<sequence>) :metaclass <myseq3-meta>)
+  #t #t elements)
+(define-method call-with-iterator ((s myseq3) proc)
+  (call-with-iterator (myseq3-elements s) proc))
+(define-method call-with-builder ((s <myseq3-meta>) proc . opts)
+  (define xs '())
+  (proc (^x (push! xs x))
+        (^[] (make-myseq3 (reverse xs)))))
+
+(test* "metaclass and builder" '(a b c d e)
+       (coerce-to <list> (coerce-to myseq3 '(a b c d e))))
+
 ;;--------------------------------------------------------------------
 (test-section "describe")
 
