@@ -2,18 +2,18 @@
  * mt-random.h - implements MT19937 random number generation algorithm
  * This code is based on Makoto Matsumoto & Takuji Nishimura's mt18837ar.c
  * See mt-random.c for details.
- * $Id: mt-random.h,v 1.8 2007-09-14 11:17:40 shirok Exp $
+ *
  * The original copyright notice follows.
  */
 /*
    A C-program for MT19937, with initialization improved 2002/1/26.
    Coded by Takuji Nishimura and Makoto Matsumoto.
 
-   Before using, initialize the state by using init_genrand(seed)  
+   Before using, initialize the state by using init_genrand(seed)
    or init_by_array(init_key, key_length).
 
    Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
-   All rights reserved.                          
+   All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
@@ -26,8 +26,8 @@
         notice, this list of conditions and the following disclaimer in the
         documentation and/or other materials provided with the distribution.
 
-     3. The names of its contributors may not be used to endorse or promote 
-        products derived from this software without specific prior written 
+     3. The names of its contributors may not be used to endorse or promote
+        products derived from this software without specific prior written
         permission.
 
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -49,7 +49,6 @@
 */
 
 #include <gauche.h>
-#include <gauche/uvector.h>
 #include <gauche/extend.h>
 
 #if defined(EXTMTRANDOM_EXPORTS)
@@ -62,7 +61,12 @@
 typedef struct ScmMersenneTwisterRec {
     SCM_HEADER;
     unsigned long mt[N]; /* the array for the state vector  */
-    int mti;             /* mti==N+1 means mt[N] is not initialized */
+    int mti;             /* index into the state vector.
+                            new integer random number is computed from
+                            mt[mti], incrementing mti; once it exceeds N,
+                            the entire state vector is recomputed. */
+    ScmObj seed;         /* last seed value used to initialize this instance.
+                            useful to reproduce the same random sequence. */
 } ScmMersenneTwister;
 
 SCM_CLASS_DECL(Scm_MersenneTwisterClass);
@@ -71,7 +75,7 @@ SCM_CLASS_DECL(Scm_MersenneTwisterClass);
 
 extern void Scm_MTInitByUI(ScmMersenneTwister *mt, unsigned long s);
 extern void Scm_MTInitByArray(ScmMersenneTwister *mt,
-                              ScmInt32 init_key[],
+                              int32_t init_key[],
                               unsigned long key_length);
 extern void Scm_MTSetSeed(ScmMersenneTwister *mt, ScmObj seed);
 
@@ -79,3 +83,5 @@ extern unsigned long Scm_MTGenrandU32(ScmMersenneTwister *);
 extern float         Scm_MTGenrandF32(ScmMersenneTwister *, int);
 extern double        Scm_MTGenrandF64(ScmMersenneTwister *, int);
 extern ScmObj        Scm_MTGenrandInt(ScmMersenneTwister *mt, ScmObj n);
+
+extern void          Scm_Init_mt_random(void);
