@@ -1,7 +1,7 @@
 ;;;
 ;;; netaux.scm - network interface
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,7 +32,7 @@
 ;;;
 
 (select-module gauche.net)
-(use srfi-1)
+(use scheme.list)
 (use gauche.sequence)
 (use gauche.lazy)
 (use util.match)
@@ -230,7 +230,7 @@
   (define (make-v6socks a6s)
     (receive (socks _) (map-accum try-v6 #f a6s)
       (filter identity socks)))
-  
+
   (let* ([ss (make-sockaddrs host port)]
          [a6s (v6addrs ss)])
     ;; NB: Mingw doesn't have EADDRINUSE.  it's likely not to have ipv6 either,
