@@ -1,7 +1,7 @@
 ;;;
 ;;; netlib.stub - network interface
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,14 +36,12 @@
 (inline-stub
 
  (declcode
-  (.include "gauche-net.h")
-  (.include <gauche/class.h>)
-  (.include <gauche/bignum.h>))
+  (.include "gauche-net.h"))
 
- (define-type <socket-address> "ScmSockAddr*" "socket address"
+ (declare-stub-type <socket-address> "ScmSockAddr*" "socket address"
    "Scm_SockAddrP" "SCM_SOCKADDR")
 
- (define-type <socket> "ScmSocket*")
+ (declare-stub-type <socket> "ScmSocket*")
  )
 
 ;;----------------------------------------------------------
@@ -75,7 +73,7 @@
    (cast void addr)                     ; suppress unused var warning
    (return 'unix))
 
- (define-cmethod sockaddr-family ((addr "Scm_SockAddrInClass")) 
+ (define-cmethod sockaddr-family ((addr "Scm_SockAddrInClass"))
    (cast void addr)                     ; suppress unused var warning
    (return 'inet))
 
@@ -204,7 +202,7 @@
                                             SCM_PORT_BUFFER_LINE))])
     (return (Scm_SocketOutputPort sock bufmode))))
 
-(inline-stub 
+(inline-stub
  (.if (and (defined SHUT_RD)
            (defined SHUT_WR)
            (defined SHUT_RDWR))
@@ -415,14 +413,11 @@
 ;;----------------------------------------------------------
 ;; IPv6 routines
 
-(inline-stub 
+(inline-stub
  (declare-cfn addrinfo_allocate (klass::ScmClass* intargs))
 
  (.if (defined HAVE_IPV6)
     (begin
-      (define-type <sys-addrinfo> "ScmSysAddrinfo*" #f
-        "SCM_SYS_ADDRINFO_P" "SCM_SYS_ADDRINFO")
-
       (define-cclass <sys-addrinfo>
         "ScmSysAddrinfo*" "Scm_SysAddrinfoClass"
         ()
