@@ -32,10 +32,10 @@
 ;;;
 
 (use gauche.test)
-(use srfi-1)
+(use scheme.list)
 (use srfi-13)
 (use gauche.generator)
-(use gauche.parameter)
+(use gauche.lazy)
 
 (test-start "parser.peg")
 
@@ -115,8 +115,16 @@
 (test-fail "$char" '(0 #\a) ($char #\a) "012")
 (test-succ "$one-of" #\j ($one-of #[a-z]) "j")
 (test-fail "$one-of" '(0 #[A-Z]) ($one-of #[A-Z]) "j")
+(test-succ "$one-of" "abd" ($one-of '("abc" "abd" "acd")) "abd")
+(test-succ "$one-of" #\c ($one-of '(#\a #\b #\c)) "c")
+(test-succ "$one-of" #\c ($one-of '("ab" #\c)) "c")
+(test-fail "$one-of" '(0 (#\a #\b #\c)) ($one-of '(#\a #\b #\c)) "z")
+(test-fail "$one-of" '(0 ("abc" "abd" "acd"))
+           ($one-of '("abc" "abd" "acd")) "a")
+(let ((var #f))
+  (set! var '("a" "b" "c")) ;suppress optimization
+  (test-succ "$one-of" "b" ($one-of var) "b"))
 (test-succ "$none-of" #\j ($none-of #[A-Z]) "j")
-(test-fail "$none-of" '(0 #[^a-z]) ($none-of #[a-z]) "j")
 (test-succ "$string-ci" "aBC" ($string-ci "abc") "aBCdef")
 (test-fail "$string-ci" '(0 "abc") ($string-ci "abc") "012")
 
@@ -144,6 +152,10 @@
            ($or ($string "foo") ($string "bar"))
            "012345")
 
+(test-fail "$or" '(0 "bar")
+           ($or ($string "foo") :else ($string "bar"))
+           "012345")
+
 ;; $try
 (test-succ "$try" "foo"
            ($try ($string "foo"))
@@ -158,56 +170,77 @@
            ($try ($seq ($string "foo") ($string "bar")))
            "foobaz...")
 
-;; $do
-(test-succ "$do" "j"
-           ($do (j ($string "j"))
-                ($return j))
-           "j")
+;; $cut and $raise
+(test-fail "$cut" '(0 (error (fail-expect . "a")))
+           ($or ($cut ($. "a")) ($. "b"))
+           "b")
+(test-succ "$cut" "a"
+           ($or ($cut ($. "a")) ($. "b"))
+           "a")
 
-(test-succ "$do" "j"
-           (let1 parse ($string "j")
-             ($do (j parse) ($return j)))
-           "j")
+(test-fail "$cut" '(1 (error (fail-expect . "b")))
+           ($or ($try ($seq ($. "a") ($cut ($. "b")))) ($. "a"))
+           "ac")
+(test-succ "$cut" "b"
+           ($or ($try ($seq ($. "a") ($cut ($. "b")))) ($. "a"))
+           "ab")
 
-(test-succ "$do" '("foo" "bar")
-           ($do (foo ($string "foo"))
-                (bar ($string "bar"))
-                ($return (list foo bar)))
-           "foobar...")
-
-(test-succ "$do" '("foo" "baz")
-           ($do (foo ($string "foo"))
-                (    ($string "bar") )
-                (baz ($string "baz"))
-                ($return (list foo baz)))
-           "foobarbaz$$$")
-
-(test-succ "$do" "foo"
-           (let1 dot ($string ".")
-             ($do (foo ($string "foo"))
-                  dot
-                  ($return foo)))
-           "foo.")
-
-(test-fail "$do" '(3 "bar")
-           ($do (foo ($string "foo"))
-                (bar ($string "bar"))
-                ($return (list foo bar)))
-           "foo12")
-
-(test-succ "$do" '("AB" "cd" "EF")
-           ($do (x ($or ($string "AB") ($string "ab")))
-                (y ($or ($string "CD") ($string "cd")))
-                (z ($or ($string "EF") ($string "ef")))
-                ($return (list x y z)))
-           "ABcdEF")
-
-(test-fail "$do and $or" '(3 "foo")
-           ($do (v ($or ($string "foo") ($string "bar")))
-                ($string (rope-finalize v)))
-           "foobar")
+(test-fail "$raise" '(1 (error (fail-message . "oops")))
+           ($or ($seq ($. "a") ($raise "oops")) ($. "a"))
+           "a")
+(test-fail "$raise" '(1 (error (fail-message . "oops")))
+           ($or ($try ($seq ($. "a") ($raise "oops"))) ($. "a"))
+           "a")
 
 ;; $let, $let*
+(test-succ "$let" "j"
+           ($let ([j ($string "j")])
+             ($return j))
+           "j")
+
+(test-succ "$let" "j"
+           (let1 parse ($string "j")
+             ($let ([j parse]) ($return j)))
+           "j")
+
+(test-succ "$let" '("foo" "bar")
+           ($let ([foo ($string "foo")]
+                  [bar ($string "bar")])
+             ($return (list foo bar)))
+           "foobar...")
+
+(test-succ "$let" '("foo" "baz")
+           ($let ([foo ($string "foo")]
+                  [    ($string "bar")]
+                  [baz ($string "baz")])
+             ($return (list foo baz)))
+           "foobarbaz$$$")
+
+(test-succ "$let" "foo"
+           (let1 dot ($string ".")
+             ($let ([foo ($string "foo")]
+                    dot)
+               ($return foo)))
+           "foo.")
+
+(test-fail "$let" '(3 "bar")
+           ($let ([foo ($string "foo")]
+                  [bar ($string "bar")])
+             ($return (list foo bar)))
+           "foo12")
+
+(test-succ "$let" '("AB" "cd" "EF")
+           ($let ([x ($or ($string "AB") ($string "ab"))]
+                  [y ($or ($string "CD") ($string "cd"))]
+                  [z ($or ($string "EF") ($string "ef"))])
+             ($return (list x y z)))
+           "ABcdEF")
+
+(test-fail "$let and $or" '(3 "foo")
+           ($let ([v ($or ($string "foo") ($string "bar"))])
+             ($string (rope-finalize v)))
+           "foobar")
+
 (test-succ "$let" '("foo" "baz")
            ($let ([foo ($string "foo")]
                   [    ($string "bar")]
@@ -238,19 +271,27 @@
            "lineament")
 
 ;; $lift and $lift*
-(test-succ "$lift" '(#\a . #\b)
-           ($lift cons ($any) ($any))
-           "abc")
-(test-fail "$lift" '(1 #\z)
-           ($lift cons ($any) ($char #\z))
-           "abc")
+;; NB: They are inline expanded, so we also call via apply
+(let ()
+  (define-syntax test-lift
+    (syntax-rules ()
+      [(test-lift tester name expect (exp ...) input)
+       (begin (tester name expect (exp ...) input)
+              (tester #"~|name| via apply"
+                      expect (apply exp ... '()) input))]))
 
-(test-succ "$lift*" "abc"
-           ($lift* list->string ($any) ($any) ($any))
-           "abc")
-(test-fail "$lift" '(2 #\z)
-           ($lift* list->string ($any) ($any) ($char #\z))
-           "abc")
+  (test-lift test-succ "$lift" '(#\a . #\b)
+             ($lift cons ($any) ($any))
+             "abc")
+  (test-lift test-fail "$lift" '(1 #\z)
+             ($lift cons ($any) ($char #\z))
+             "abc")
+  (test-lift test-succ "$lift*" "abc"
+             ($lift* list->string ($any) ($any) ($any))
+             "abc")
+  (test-lift test-fail "$lift" '(2 #\z)
+             ($lift* list->string ($any) ($any) ($char #\z))
+             "abc"))
 
 ;; $fold-parsers and $fold-parsers-right
 (test-succ "$fold-parsers" '()                  ; base case
@@ -296,16 +337,16 @@
 (test-succ "$many" '("a" "a")
            ($many ($string "a") 1 2) "aaaaa")
 
-;; $skip-many
-(test-succ "$skip-many" #\a
-           ($seq ($skip-many ($string "a") 1 2) ($one-of #[a-z])) "aaaaa")
-(test-succ "$skip-many" #\b
-           ($seq ($skip-many ($string "a")) ($one-of #[a-z])) "baaaaa")
+;; $many_
+(test-succ "$many_" #\a
+           ($seq ($many_ ($string "a") 1 2) ($one-of #[a-z])) "aaaaa")
+(test-succ "$many_" #\b
+           ($seq ($many_ ($string "a")) ($one-of #[a-z])) "baaaaa")
 
 ;; $repeat
 (test-succ "$repeat" '(#\a #\b #\a)
            ($repeat ($any) 3) "abab")
-(test-fail "$repeat" '(2 "character")
+(test-fail "$repeat" '(2 "end of input")
            ($repeat ($any) 3) "ab")
 
 ;; $optional
@@ -375,13 +416,6 @@
              "aa,bb,")
   )
 
-;; $count
-(test-succ "$count" '(#\a #\b)
-           ($count ($one-of #[ab]) 2) "abab")
-
-(test-fail "$count" '(1 #[ab])
-           ($count ($one-of #[ab]) 2) "a012")
-
 ;; $between
 (test-succ "$between" "foo"
            ($between ($string "(")
@@ -408,30 +442,13 @@
            ($many-till ($one-of #[[:alnum:]]) ($one-of #[\d]))
            "ab78")
 
-;; $many-chars
-(test-succ "$many-chars" '(#\c #\b #\a)
-           ($many-chars #[a-c])
-           "cbad")
-(test-succ "$many-chars" '()
-           ($many-chars #[a-c])
-           "ABCD")
-(test-succ "$many-chars" '(#\c #\b #\a)
-           ($many-chars #[a-c] 2 5)
-           "cbad")
-(test-fail "$many-chars" '(3 #[a-c])
-           ($many-chars #[a-c] 4 5)
-           "cbad")
-(test-succ "$many-chars" '(#\c #\b)
-           ($many-chars #[a-c] 0 2)
-           "cbad")
-
 ;; $chain-left
 (let ([integer
-       ($do [v ($many ($one-of #[\d])  1)]
-            ($return (string->number (apply string v))))]
+       ($let ([v ($many ($one-of #[\d])  1)])
+         ($return (string->number (apply string v))))]
       [op
-       ($or ($do [($char #\*)] ($return *))
-            ($do [($char #\+)] ($return +)))])
+       ($or ($seq ($char #\*) ($return *))
+            ($seq ($char #\+) ($return +)))])
   (test-succ "$chain-left" 9
              ($chain-left integer op)
              "1+2*3")
@@ -444,11 +461,11 @@
 
 ;; $chain-right
 (let ([integer
-       ($do [v ($many ($one-of #[\d])  1)]
-            ($return (string->number (apply string v))))]
+       ($let ([v ($many ($one-of #[\d])  1)])
+         ($return (string->number (apply string v))))]
       [op
-       ($or ($do [($char #\*)] ($return *))
-            ($do [($char #\+)] ($return +)))])
+       ($or ($seq ($char #\*) ($return *))
+            ($seq ($char #\+) ($return +)))])
   (test-succ "$chain-right" 7
              ($chain-right integer op)
              "1+2*3")
@@ -471,26 +488,47 @@
 (test-section "backtrack control")
 
 (test-succ "$or and $try" "abc"
-           ($or ($try ($do [foo ($string "abc")]
-                           [bar ($string "foo")]
-                           ($return (list foo bar))))
-                ($do (v ($string "abc"))
-                     ($return v)))
+           ($or ($try ($list ($string "abc") ($string "foo")))
+                ($string "abc"))
            "abcdefg")
 
-(let1 parser ($or ($do [foo ($try ($do [v ($string "abc")]
-                                       [($one-of #[+-])]
-                                       ($return v)))]
-                       [bar ($string "foo")]
-                       ($return (list foo bar)))
-                  ($do (v ($string "abc"))
-                       ($return v)))
+(let1 parser ($or ($let ([foo ($try ($let ([v ($string "abc")]
+                                           [($one-of #[+-])])
+                                      ($return v)))]
+                         [bar ($string "foo")])
+                    ($return (list foo bar)))
+                  ($string "abc"))
   (test-succ "$or and $try" "abc" parser
              "abcdefg")
   (test-fail "$or and $try" '(4 "foo") parser
              "abc+efg")
   )
 
+;;;============================================================
+;;; Lookahead assertion
+;;;
+(test-section "lookahead assertion")
+
+(let ([parser ($seq0
+               ($->rope ($many
+                         ($try ($seq0 ($any)
+                                             ($assert ($. #[a-z]))))))
+               ($any))])
+  (test-succ "positive lookahead" " abc" parser
+             " abcd ")
+  )
+
+(let ([parser1 ($between ($. "/*")
+                         ($->rope ($many ($seq ($not ($. "*/"))
+                                               ($any))))
+                         ($. "*/"))])
+  (test-succ "negative lookahead 1" " abc * def " parser1
+             "/* abc * def */ def")
+  (test-fail "negative lookahead 1" '(12 "*/")  parser1
+             "/* abc * def")
+  )
+
+
 ;;;============================================================
 ;;; Error handling
 ;;;
@@ -505,35 +543,170 @@
          (test-error <parse-error> "expecting one of (#\\c #\\a #\\b) at 1")
          (peg-parse-string p2 "dd")))
 
+;; position
+(test* "error position handling"
+       (test-error <parse-error>
+                   "expecting #[\\u000aa] at \"input.txt\":2:3, but got #\\b")
+       (peg-run-parser ($many ($. #[a\n]) 10)
+                       (port->char-lseq/position
+                        (open-input-string "aaaaa\naabaa")
+                        :source-name "input.txt")))
+
+(test* "error position handling (<parse-error> slots)"
+       #t
+       (guard (e [(<parse-error> e) (sequence-position? (~ e'position))])
+         (peg-run-parser ($many ($. #[a\n]) 10)
+                         (port->char-lseq/position
+                          (open-input-string "aaaaa\naabaa")
+                          :source-name "input.txt"))))
 
 ;;;============================================================
-;;; Token Parsers
+;;; Optional arg for peg-parse-string and peg-parse-port
 ;;;
 
+(test-section "optional cont arg")
+
+(test* "peg-parse-string cont" '("aaaa" #\b #\b #\c)
+       (peg-parse-string ($->rope ($many ($. #\a))) "aaaabbc" cons))
+(test* "peg-parse-port cont" '("aaaa" #\b #\b #\c)
+       (call-with-input-string "aaaabbc"
+         (cut peg-parse-port ($->rope ($many ($. #\a))) <> cons)))
+
 ;;;============================================================
-;;; Permulate Parsers
+;;; Token parsing
 ;;;
 
+(test-section "token parsing")
+
+(let ()
+  (define %a ($many1 ($. 'a)))
+  (define %bcd ($many1 ($one-of '(b c d))))
+  (define %p ($many ($list %a %bcd)))
+
+  (test* "token parsing"
+         '(((a) (b c)) ((a) (d)) ((a a) (b)) ((a) (c d c)))
+         (peg-run-parser %p '(a b c a d a a b a c d c)))
+  )
+
+
 ;;;============================================================
 ;;; Examples
 ;;;
 (test-section "examples")
 
-;; In the manual
+;; PEG walkthrough
+(test* "walkthrough char" #\a
+       (peg-parse-string ($. #\a) "abc"))
+(test* "walkthrough error" (test-error <parse-error>
+                                       "expecting #\\a at 0, but got #\\x")
+       (peg-parse-string ($char #\a) "xyz"))
+(test* "walkthrough $many" '(#\a #\a #\a #\a #\a)
+       (peg-parse-string ($many ($. #\a)) "aaaaabc"))
+(test* "walkthrough $many" '()
+       (peg-parse-string ($many ($. #\a)) "xxxxxyz"))
 
 (let ()
-  (define integer   ($many1 ($one-of #[\d])))
-  (define ws        ($skip-many ($one-of #[\s])))
-  (define int-list0 ($seq ($char #\[ )
+  (define digits    ($many1 ($. #[\d])))
+  (define ws        ($many_ ($. #[\s])))
+  (define separator ($seq ws ($. #\,) ws))
+  (define integer
+    ($let ([ds digits])
+      ($return (x->integer (list->string ds)))))
+  (define integers1
+    ($seq integer ($many ($seq separator integer))))
+  (define integers2
+    ($let ([n  integer]
+           [ns ($many ($seq separator integer))])
+      ($return (cons n ns))))
+  (define integers3
+    ($lift cons
+           integer
+           ($many ($seq separator integer))))
+  (define integers4
+    ($or ($let ([n  integer]
+                [ns ($many ($seq separator integer))])
+           ($return (cons n ns)))
+         ($return '())))
+  (define (sep-by stuff separator)
+    ($or ($let ([n  stuff]
+                [ns ($many ($seq separator stuff))])
+           ($return (cons n ns)))
+         ($return '())))
+  (define integers5 (sep-by integer separator))
+  (define begin-list
+    ($seq0 ($. #[\(\[\{]) ws))
+  (define (end-list opener)
+    ($seq ws (case opener
+               [(#\() ($. #\))]
+               [(#\[) ($. #\])]
+               [(#\{) ($. #\})])))
+
+  (define int-list
+    ($let* ([opener begin-list]
+            [ints ($sep-by integer separator)]
+            [ (end-list opener) ])
+      ($return ints)))
+
+  (define nested-list
+    ($lazy
+     ($let* ([opener begin-list]
+             [ints ($sep-by elem separator)]
+             [ (end-list opener) ])
+       ($return ints))))
+  (define elem  ($or integer nested-list))
+
+  (test* "integers1" '(456 789)
+         (peg-parse-string integers1 "123, 456, 789"))
+  (test* "integers2" '(123 456 789)
+         (peg-parse-string integers2 "123, 456, 789"))
+  (test* "integers3" '(123 456 789)
+         (peg-parse-string integers3 "123, 456, 789"))
+  (test* "integers4" '(123 456 789)
+         (peg-parse-string integers4 "123, 456, 789"))
+  (test* "integers4" '()
+         (peg-parse-string integers4 ""))
+  (test* "integers5" '(123 456 789)
+         (peg-parse-string integers5 "123, 456, 789"))
+  (test* "integers5" '()
+         (peg-parse-string integers5 ""))
+
+  (test* "int-list" '(123 456 789)
+         (peg-parse-string int-list "[123, 456, 789]"))
+  (test* "int-list" '(123 456 789)
+         (peg-parse-string int-list "{123, 456, 789}"))
+  (test* "int-list" (test-error <parse-error>
+                                "expecting #\\) at 14, but got #\\}")
+         (peg-parse-string int-list "(123, 456, 789}"))
+
+  (test* "nested-list" '(123 (456 () 789) 987)
+         (peg-parse-string nested-list "(123, [456, {}, 789], 987)"))
+  )
+
+(let ()
+  (define paren  ($. #\())
+  (define thesis ($. #\)))
+
+  (test* "$or example"
+         (test-error <parse-error> "expecting ab at 1, but got #\\c")
+
+         (peg-parse-string ($or ($seq paren ($."ab") thesis)
+                                ($seq paren ($."cd") thesis))
+                           "(cd)"))
+  )
+
+(let ()
+  (define integer   ($many1 ($. #[\d])))
+  (define ws        ($many_ ($. #[\s])))
+  (define int-list0 ($seq ($. #\[ )
                           ($many ($seq ws integer))
                           ws
                           ($char #\] )))
   (define int-list
-    ($do [ ($char #\[ )]
-         [digs ($many ($seq ws integer))]
-         [ ws ]
-         [ ($char #\] ) ]
-         ($return (map (compose x->integer list->string) digs))))
+    ($let ([ ($char #\[ )]
+           [digs ($many ($seq ws integer))]
+           [ ws ]
+           [ ($char #\] ) ])
+      ($return (map (compose x->integer list->string) digs))))
 
   (test-succ "integer" '(#\1 #\2 #\3) integer "123")
   (test-succ "int-list0" #\] int-list0 "[123 456 789]")
@@ -542,11 +715,11 @@
 
 ;; Count the maximal nesting level
 (letrec ((nesting
-          ($lazy ($or ($do [ ($char #\() ]
-                           [n nesting]
-                           [ ($char #\)) ]
-                           [m nesting]
-                           ($return (max (+ n 1) m)))
+          ($lazy ($or ($let ([ ($char #\() ]
+                             [n nesting]
+                             [ ($char #\)) ]
+                             [m nesting])
+                        ($return (max (+ n 1) m)))
                       ($return 0)))))
   (test-succ "nesting parenthesis" 3 nesting "((()))")
   (test-succ "nesting parenthesis" 3 nesting "((()))()")
@@ -554,12 +727,12 @@
   (test-fail "nesting parenthesis" '(3 #\) ) nesting "((("))
 
 ;; number parser (1) - simple
-(let* ((sign?    ($optional ($one-of #[-+])))
-       (digits   ($seq sign? ($many-chars #[\d] 1)))
-       (point    ($seq ($char #\.) ($many ($one-of #[\d]) 1)))
-       (exponent ($seq ($one-of #[eE]) digits))
-       (number?  ($seq digits ($optional point) ($optional exponent)))
-       (p        ($do [number?] [($eos)] ($return #t))))
+(let* ([sign?    ($optional ($one-of #[-+]))]
+       [digits   ($seq sign? ($many ($. #[\d]) 1))]
+       [point    ($seq ($char #\.) ($many ($. #[\d]) 1))]
+       [exponent ($seq ($. #[eE]) digits)]
+       [number?  ($seq digits ($optional point) ($optional exponent))]
+       [p        ($seq number? ($eos) ($return #t))])
   (define (%test str . fails?)
     (if (null? fails?)
       (test-succ #"number(1) \"~str\"" #t p str)
@@ -569,7 +742,7 @@
   (%test "+27.46") (%test "0.46")
   (%test ".46" '(0 #[0-9]))
   (%test "0..3" '(2 #[0-9]))
-  (%test "0.4.2" '(3 "end of stream"))
+  (%test "0.4.2" '(3 "end of input"))
   (%test "3e5")
   (%test "3.0e5")
   (%test "3.e5" '(2 #[0-9]))
@@ -581,13 +754,13 @@
 
 ;; CSV parser
 (let ()
-  (define ws     ($many ($one-of #[ \t])))
-  (define comma  ($seq ws ($char #\,) ws))
-  (define dquote ($char #\"))
-  (define double-dquote ($do [($string "\"\"")] ($return #\")))
-  (define quoted-body ($many ($or double-dquote ($one-of #[^\"]))))
+  (define ws     ($many ($. #[ \t])))
+  (define comma  ($seq ws ($. #\,) ws))
+  (define dquote ($. #\"))
+  (define double-dquote ($seq ($. "\"\"") ($return #\")))
+  (define quoted-body ($many ($or double-dquote ($. #[^\"]))))
   (define quoted ($between dquote quoted-body dquote))
-  (define unquoted ($many-till ($any) ($or comma ($char #\newline))))
+  (define unquoted ($many-till ($any) ($or comma ($. #\newline))))
   (define field ($or quoted unquoted))
   (define record ($sep-by ($->rope field) comma))
   (test-succ "CSV" '("a" "b" "c")
@@ -599,17 +772,17 @@
 
 ;; hand-tuned version
 (let ()
-  (define ws     ($skip-many ($one-of #[ \t])))
-  (define comma  ($seq ws ($char #\,) ws))
-  (define dquote ($char #\"))
-  (define double-dquote ($do [($string "\"\"")] ($return #\")))
-  (define quoted-body ($many ($or ($one-of #[^\"]) double-dquote)))
+  (define ws     ($many_ ($. #[ \t])))
+  (define comma  ($seq ws ($. #\,) ws))
+  (define dquote ($. #\"))
+  (define double-dquote ($seq ($. "\"\"") ($return #\")))
+  (define quoted-body ($many ($or ($. #[^\"]) double-dquote)))
   (define quoted ($between dquote quoted-body dquote))
-  (define unquoted ($alternate ($many1 ($one-of #[^ \t\r\n,]))
-                               ($many  ($one-of #[ \t]))))
+  (define unquoted ($sep-end-by ($many1 ($. #[^ \t\r\n,]))
+                                ($many_ ($. #[ \t]))))
   (define field   ($or quoted unquoted))
   (define record  ($sep-by ($->rope field) comma 1))
-  (define records ($sep-end-by record ($string "\r\n")))
+  (define records ($sep-end-by record ($. "\r\n")))
   ;; NB: In CSV spec there's an ambiguity of treatment of the ending CRLF.
   ;; The CRLF after the last record can be omitted; so if the file ends with
   ;; CRLF, it may be the end of whole records, or there may be one record
@@ -625,29 +798,29 @@
              records "\"a  \"\" \n\" , b  , c"))
 
 ;; Poor-man's XML parser
-(letrec ([open-tag ($->rope ($between ($char #\<)
-                                      ($one-of #[[:alnum:]])
-                                      ($char #\>)))]
-         [close-tag (^[tagname]
-                      ($seq ($string "</") ($string tagname) ($char #\>)))]
-         [text ($->rope ($many ($none-of #[<])))]
-         [body ($do [t text]
-                    [r ($many ($do [e element]
-                                   [t text]
-                                   ($return (list e t))))]
-                    ($return `(,t ,@(apply append r))))]
-         [element ($do [tagname ($try open-tag)]
-                       [body body]
-                       [ (close-tag (rope-finalize tagname)) ]
-                       ($return (cons tagname body)))])
+(letrec* ([open-tag ($->rope ($between ($char #\<)
+                                       ($one-of #[[:alnum:]])
+                                       ($char #\>)))]
+          [close-tag (^[tagname]
+                       ($seq ($string "</") ($string tagname) ($char #\>)))]
+          [text ($->rope ($many ($none-of #[<])))]
+          [body ($let ([t text]
+                       [r ($many ($let ([e ($lazy element)]
+                                        [t text])
+                                       ($return (list e t))))])
+                      ($return `(,t ,@(apply append r))))]
+          [element ($let* ([tagname ($try open-tag)]
+                           [body body]
+                           [ (close-tag (rope-finalize tagname)) ])
+                          ($return (cons tagname body)))])
   (test-succ "tag element" '("a" "")
              element "<a></a>")
   (test-succ "tag element" '("a" "foo" ("b" "bar") "baz")
              element "<a>foo<b>bar</b>baz</a>"))
 
 ;; Calculator
-(letrec ([integer ($do (v ($many ($one-of #[\d]) 1))
-                       ($return (string->number (apply string v))))]
+(letrec ([integer ($let ([v ($many ($one-of #[\d]) 1)])
+                    ($return (string->number (apply string v))))]
          [mulop ($or ($seq ($char #\*) ($return *))
                      ($seq ($char #\/) ($return /)))]
          [addop ($or ($seq ($char #\+) ($return +))
@@ -666,7 +839,7 @@
 
 (test-section "deprecated")
 ;; kludge - we can't say (use parser.peg.deprecated) until peg/deprecated.scm
-;; is installed in place.  
+;; is installed in place.
 (or (load "peg/deprecated" :error-if-not-found #f)
     (load "parser/peg/deprecated" :error-if-not-found #f))
 (import parser.peg.deprecated)
@@ -704,25 +877,97 @@
            ($satisfy char-numeric? "numeric")
            "a")
 
-(test-succ "$match" #\a
-           ($match (#\c x #\r) x)
+(test-succ "$match1*" #\a
+           ($match1* (#\c x #\r) x)
            "car foo")
-(test-succ "$match" #\d
-           ($match (#\c x #\r) x)
+(test-succ "$match1*" #\d
+           ($match1* (#\c x #\r) x)
            "cdr foo")
-(test-succ "$match" '(#\a (#\space #\f #\o #\o))
-           ($match (#\c x #\r . t) (list x t))
+(test-succ "$match1*" '(#\a (#\space #\f #\o #\o))
+           ($match1* (#\c x #\r . t) (list x t))
            "car foo")
-(test-succ "$match (no result arg)" '(#\c #\u #\r)
-           ($match (#\c x #\r))
+(test-succ "$match1* (no result arg)" '(#\c #\u #\r)
+           ($match1* (#\c x #\r))
            "cur foo")
-(test-fail "$match" '(0 "(#\\c x #\\r)")
-           ($match (#\c x #\r) x)
+(test-fail "$match1*" '(0 "(#\\c x #\\r)")
+           ($match1* (#\c x #\r) x)
            "zxy foo")
+(let ((p ($match1* (#\c x #\r) (=> F)
+                   (if (#[ad] x) x (F "car or cdr expected")))))
+  (test-succ "$match1* fail proc" #\a p "car")
+  (test-succ "$match1* fail proc" #\d p "cdr")
+  (test-fail "$match1* fail proc" '(0 "car or cdr expected") p "cxr"))
+(let ((p ($match1* (#\c x #\r . t) (=> F)
+                   (if (#[ad] x) (list x t) (F "car or cdr expected")))))
+  (test-succ "$match1* fail proc" '(#\a (#\z)) p "carz")
+  (test-succ "$match1* fail proc" '(#\d (#\z)) p "cdrz")
+  (test-fail "$match1* fail proc" '(0 "car or cdr expected") p "cxrz"))
 
 (test-succ "$match1" #\a ($match1 #\a) "abc")
 (test-succ "$match1" #\0 ($match1 (and (? #[0-9] d)) d) "0ab")
 (test-fail "$match1" '(0 "#\\a") ($match1 #\a 'yo) "0ab")
+(let ((p ($match1 (? char-numeric? x) (=> F)
+                  (if (eqv? x #\0)
+                    'ok
+                    (F "0 expected")))))
+  (test-succ "$match1 fail proc" 'ok p "012")
+  (test-fail "$match1 fail proc" '(0 "0 expected") p "123")
+  (test-fail "$match1 not enough input" '(0 "(? char-numeric? x)") p ""))
+
+
+(let ((p ($binding ($: x ($or ($seq ($: y ($."a")) ($."b"))
+                              ($seq ($: z ($."c")) ($."d"))))
+                   (list x y z))))
+  (test-succ "$binding" `("b" "a" ,(undefined)) p "ab")
+  (test-succ "$binding" `("d" ,(undefined) "c") p "cd")
+  (test-fail "$binding" '(1 "b") p "az")
+  (test-fail "$binding" '(1 "d") p "cy")
+  (test-fail "$binding" '(0 ((fail-expect . "a") (fail-expect . "c"))) p "pq"))
+
+(let ((p ($binding ($: x ($or ($seq ($: y ($."a")) ($."b"))
+                              ($seq ($: y ($."c")) ($."d"))))
+                   (list x y))))
+  (test-succ "$binding" `("b" "a") p "ab")
+  (test-succ "$binding" `("d" "c") p "cd")
+  (test-fail "$binding" '(1 "d") p "cy"))
+
+(let ((p ($binding ($: x ($or ($seq ($: y ($."a")) ($."b"))
+                              ($seq ($: y ($."c")) ($."d"))))
+                   (=> F)
+                   (if (equal? y "c")
+                     (F "no-no")
+                     (list x y)))))
+  (test-succ "$binding fail proc" `("b" "a") p "ab")
+  (test-fail "$binding fail proc" `(2 "no-no") p "cd")
+  (test-fail "$binding fail proc" '(1 "d") p "cy"))
+
+(let ((p ($or ($binding ($: x ($. "ab"))
+                        (=> F)
+                        (F 'error "ugh"))
+              ($. "ab"))))
+  (test-fail "$binding fail/error" '(2 (error (fail-message . "ugh")))
+             p "ab"))
+
+(letrec ([p (lambda (q)
+              ($binding ($. "a")
+                        ($: y q)
+                        ($seq ($: x ($any)) ($. "."))
+                        ($. "z")
+                        (list x y)))]
+         [q ($binding ($many ($or ($: x ($. "b")) ($: y ($. "c"))))
+                      (list x y))])
+  (test-succ "$binding dynamically nested" '(#\d ("b" "c"))
+             (p q) "abcbcd.z")
+  )
+
+(test-succ "$binding -> lift optimization" '("b" "d")
+           ($binding ($. "a")
+                     ($: b ($. "b"))
+                     ($. "c")
+                     ($: d ($. "d"))
+                     ($. "e")
+                     (list b d))
+           "abcde")
 
 (test-succ "eof" (eof-object) eof "")
 (test-fail "eof" '(0 "end of input") eof "a")
@@ -899,7 +1144,7 @@
   (define (t obj)
     (test* #"writer error ~obj" (test-error <json-construct-error>)
            (construct-json-string obj)))
-  (t "a")
+  (t 'a)
   (t '#(1 2 x))
   (t '(("a" . 2) 9)))
 
@@ -913,5 +1158,21 @@
 (test* "read/write invariance"
        '#(48.529166)
        (parse-json-string (construct-json-string '#(48.529166))))
-       
+
+;; json-nesting-depth-limit
+(test* "depth-limit" '#(#(#(#(null))))
+       (parameterize ((json-nesting-depth-limit 4))
+         (parse-json-string "[[[[null]]]]")))
+(test* "depth-limit" (test-error <json-parse-error> #/nesting is too deep/)
+       (parameterize ((json-nesting-depth-limit 4))
+         (parse-json-string "[[[[[null]]]]]")))
+(test* "depth-limit" 123
+       (parameterize ((json-nesting-depth-limit 1))
+         (parse-json-string "123")))
+(test* "depth-limit" (test-error <json-parse-error> #/nesting is too deep/)
+       (parameterize ((json-nesting-depth-limit 1))
+         (parse-json-string "{\"x\":123}")))
+
+(include "test-srfi-180")
+
 (test-end)
