@@ -6,173 +6,6 @@
 
 (test-start "extension srfi modules")
 
-;;
-;; Test srfi-1
-;;
-
-(test-section "srfi-1")
-(use srfi-1)
-(test-module 'srfi-1)
-
-(define Apply apply)                    ;avoic inline expansion
-
-(test* "xcons" '(a . b) (xcons 'b 'a))
-(test* "cons*" (test-error) (Apply cons* '())) ; use apply to avoid compile error during inlining
-(test* "cons*" 'o  (cons* 'o))
-(test* "cons*" '(1 2 3 . 4) (cons* 1 2 3 4))
-(test* "make-list" 5 (length (make-list 5)))
-(test* "make-list" '() (make-list 0))
-(test* "make-list" '(m m m m m) (make-list 5 'm))
-(test* "list-tabulate" '(0 1 2 3 4) (list-tabulate 5 values))
-(test* "list-tabulate" '(#\0 #\1 #\2 #\3 #\4)
-       (list-tabulate 5 (^i (integer->char (+ i 48)))))
-(test* "list-tabulate" '(0 2 4 6) (list-tabulate 4 (^i (* i 2))))
-(test* "list-copy" '(1 2 3) (list-copy '(1 2 3)))
-(test* "list-copy" '() (list-copy '()))
-(test* "list-copy" '(1 . 2) (list-copy '(1 . 2)))
-(test* "list-copy" '(1 2 . 3) (list-copy '(1 2 . 3)))
-(test* "list-copy" '(1 2 3)
-       (let ((x (list 1 2 3)))
-         (set-car! (list-copy x) 0)
-         x))
-(test* "list-copy" '(1 2 . 3)
-       (let ((x '(1 2 . 3)))
-         (set-cdr! (cdr (list-copy x)) 0)
-         x))
-(test* "circular-list" 'a
-       (list-ref (circular-list 'a 'b) 4))
-(test* "proper-list?" #t (proper-list? '(1 2 4)))
-(test* "proper-list?" #t (proper-list? '()))
-(test* "proper-list?" #f (proper-list? 2))
-(test* "proper-list?" #f (proper-list? '(3 . 1)))
-(test* "proper-list?" #f (proper-list? (circular-list 1 2)))
-(test* "proper-list?" #f (proper-list? (cons 0 (circular-list 1 2))))
-(test* "circular-list?" #t (circular-list? (circular-list 1 2)))
-(test* "circular-list?" #f (circular-list? '()))
-(test* "circular-list?" #f (circular-list? '(1 . 2)))
-(test* "circular-list?" #t (circular-list? (cons 1 (circular-list 2 3))))
-(test* "dotted-list?" #t (dotted-list? '(1 . 2)))
-(test* "dotted-list?" #t (dotted-list? 3))
-(test* "dotted-list?" #f (dotted-list? '(1 2 3)))
-(test* "dotted-list?" #f (dotted-list? (circular-list 1 2 3)))
-(test* "dotted-list?" #f (dotted-list? (cons 0 (circular-list 1 2 3))))
-(test* "dotted-list?" #f (dotted-list? '()))
-(test* "null-list?" #t (null-list? '()))
-(test* "null-list?" #f (null-list? '(1)))
-(test* "not-pair?" #f (not-pair? '(1)))
-(test* "not-pair?" #t (not-pair? '()))
-(test* "list=" #t (list= eq?))
-(test* "list=" #t (list= eq? '(a)))
-(test* "list=" #t (list= eq? '(a) '(a)))
-(test* "list=" #f (list= eq? '(a) '(a b)))
-(test* "list=" #t (list= eq? '(a b) '(a b) '(a b)))
-(test* "list=" #f (list= eq? '(a b) '(a b) '(a b c)))
-(test* "list=" #f (list= eq? '(a b) '(a b) '(a)))
-(test* "list=" #t (list= char-ci=? '(#\a #\b #\z) '(#\A #\B #\Z)))
-(test* "list=" #f (list= char-ci=? '(#\a #\b #\z) '(#\A #\B)))
-(test* "first"  1 (first '(1 2 3 4 5 6 7 8 9 10)))
-(test* "second" 2 (second '(1 2 3 4 5 6 7 8 9 10)))
-(test* "third"  3 (third '(1 2 3 4 5 6 7 8 9 10)))
-(test* "fourth" 4 (fourth '(1 2 3 4 5 6 7 8 9 10)))
-(test* "fifth"  5 (fifth '(1 2 3 4 5 6 7 8 9 10)))
-(test* "sixth"  6 (sixth '(1 2 3 4 5 6 7 8 9 10)))
-(test* "seventh" 7 (seventh '(1 2 3 4 5 6 7 8 9 10)))
-(test* "eighth" 8 (eighth '(1 2 3 4 5 6 7 8 9 10)))
-(test* "ninth"  9 (ninth '(1 2 3 4 5 6 7 8 9 10)))
-(test* "tenth"  10 (tenth '(1 2 3 4 5 6 7 8 9 10)))
-(test* "car+cdr" '(a (b c))
-       (call-with-values (^[] (car+cdr '(a b c))) list))
-(test* "last" 'c (last '(a b c)))
-(test* "last-pair" '(c) (last-pair '(a b c)))
-(test* "length+" '(5 #f)
-       (list (length+ '(1 2 3 4 5))
-             (length+ (circular-list 1 2 3 4 5))))
-(test* "zip" '((one 1 odd) (two 2 even) (three 3 odd))
-       (zip '(one two three) '(1 2 3)
-            '(odd even odd even odd even)))
-(test* "zip" '((1) (2) (3)) (zip '(1 2 3)))
-(test* "zip" '((3 #f) (1 #t) (4 #f) (1 #t))
-       (zip '(3 1 4 1) (circular-list #f #t)))
-(test* "zip" '() (zip '(a b) '() '(c d)))
-(define unzip-data '((1 2 3 4 5 6 7 8)
-                     (a b c d e)
-                     (#\a #\b #\c #\d #\e)
-                     ("a" "b" "c" "d" "e")))
-(test* "unzip1" '(1 a #\a "a") (unzip1 unzip-data))
-(test* "unzip2" '((1 a #\a "a") (2 b #\b "b"))
-       (call-with-values (^[] (unzip2 unzip-data)) list))
-(test* "unzip3" '((1 a #\a "a") (2 b #\b "b") (3 c #\c "c"))
-       (call-with-values (^[] (unzip3 unzip-data)) list))
-(test* "unzip4" '((1 a #\a "a") (2 b #\b "b") (3 c #\c "c") (4 d #\d "d"))
-       (call-with-values (^[] (unzip4 unzip-data)) list))
-(test* "unzip5"
-       '((1 a #\a "a") (2 b #\b "b") (3 c #\c "c") (4 d #\d "d") (5 e #\e "e"))
-       (call-with-values (^[] (unzip5 unzip-data)) list))
-(test* "pair-fold" '(5 4 3 2 1)
-       (pair-fold (^[p t] (set-cdr! p t) p) '()
-                  (list 1 2 3 4 5)))
-(test* "pair-fold-right" '((a b c) (b c) (c))
-       (pair-fold-right cons '() '(a b c)))
-(test* "unfold" '(1 4 9 16 25 36 49 64 81 100)
-       (unfold (^x (> x 10))
-               (^x (* x x))
-               (^x (+ x 1))
-               1))
-(test* "unfold-right" '(1 4 9 16 25 36 49 64 81 100)
-       (unfold-right zero?
-                     (^x (* x x))
-                     (^x (- x 1))
-                     10))
-(test* "map" '(4 1 5 1)
-       (map + '(3 1 4 1) (circular-list 1 0)))
-(test* "for-each" '#(0 2 2 4 4)
-       (rlet1 v (make-vector 5)
-         (for-each (^[i n] (vector-set! v i (+ i n)))
-                   '(0 1 2 3 4)
-                   (circular-list 0 1))))
-(test* "map!" '(4 1 5 1)
-       (map! + '(3 1 4 1) (circular-list 1 0)))
-(test* "map-in-order"  '(4 1 5 1)
-       (map-in-order + '(3 1 4 1) (circular-list 1 0)))
-(test* "pair-for-each" '((c) (b c) (a b c))
-       (rlet1 r '()
-         (pair-for-each (^l (set! r (cons l r))) '(a b c))))
-(test* "partition!" '((one four five) (2 3 6))
-       (values->list (partition! symbol? (list 'one 2 3 'four 'five 6))))
-(test* "take-while" '(2 18)
-       (take-while even? '(2 18 3 10 22 9)))
-(test* "take-while!" '(2 18)
-       (take-while! even? (list 2 18 3 10 22 9)))
-(test* "drop-while" '(3 10 22 9)
-       (drop-while even? '(3 10 22 9)))
-(test* "span" '((2 18) (3 10 22 9))
-       (values->list (span even? '(2 18 3 10 22 9))))
-(test* "break" '((2 18) (3 10 22 9))
-       (values->list (break odd? '(2 18 3 10 22 9))))
-(test* "span!" '((2 18) (3 10 22 9))
-       (values->list (span! even? (list 2 18 3 10 22 9))))
-(test* "break!" '((2 18) (3 10 22 9))
-       (values->list (break! odd? (list 2 18 3 10 22 9))))
-(test* "any" #t
-       (any integer? '(a 3 b 2.8)))
-(test* "any" #f
-       (any integer? '(a 3.1 b 2.8)))
-(test* "any" #t
-       (any < '(3 1 4 1 5) '(2 7 1 8 2)))
-(test* "every" #f
-       (every integer? '(a 3 b 2.8)))
-(test* "every" #t
-       (every integer? '(2.0 3.0 8.0 -1.0)))
-(test* "every" #t
-       (every <= '(1 2 3 4 5) '(2 3 3 5 5)))
-(test* "list-index" 2
-       (list-index even? '(3 1 4 1 5 9)))
-(test* "list-index" 1
-       (list-index < '(3 1 4 1 5 9) '(2 7 1)))
-(test* "list-index" #f
-       (list-index = '(3 1 4 1 5 9) '(2 7 1)))
-;; TODO: lset stuff
-
 ;;;
 ;;; test srfi-13
 ;;;
@@ -746,7 +579,7 @@
 
 ;; NB: in Gauche, the round-trip conversion from time -> julian-day -> time
 ;; can't be guaranteed because of the limited precision of julian-day
-;; calcularion.   We round the nanosecond range.
+;; calculation.   We round the nanosecond range.
 (define (round-to-seconds time)
   (let1 n (time-nanosecond time)
     (set! (ref time 'nanosecond) 0)
@@ -805,6 +638,46 @@
          (map (cut slot-ref d <>)
               '(year month day hour minute second zone-offset))))
 
+;; ISO week number ~V should be the same as strftime %V
+;; NB: MinGW's strftime doesn't support %V yet.  Since the Scheme code is
+;; portable, we just skip the tests for now.
+;; Cf. https://sourceforge.net/p/mingw-w64/bugs/793/
+(cond-expand
+ [gauche.os.windows]
+ [else
+  (let ((dates '((2021 1 1) ; Friday, week 53
+                 (2021 1 3) ; Sunday, week 53
+                 (2021 1 4) ; Monday, week 1
+                 (2020 12 31) ; Thursday, week 53
+
+                 (2020 1 1) ; Wednesday, week 1
+                 (2019 12 31) ; Tuesday, week 1
+                 (2019 12 30) ; Monday, week 1
+                 (2019 12 29) ; Sunday, week 52
+
+                 (2017 1 1) ; Sunday, week 52
+                 (2017 1 2) ; Monday, week 1
+                 (2017 1 8) ; Sunday, week 1
+                 (2017 1 9) ; Monday, week 2
+                 (2016 12 31) ; Saturday, week 52
+
+                 (2015 1 1) ; Thursday, week 1
+                 (2015 1 2) ; Friday, week 1
+                 (2015 1 3) ; Saturday, week 1
+                 (2015 1 4) ; Sunday, week 1
+                 (2015 1 5) ; Monday, week 2
+                 (2014 12 31) ; Wednesday, week 1
+                 (2014 12 30) ; Tuesday, week 1
+                 (2014 12 29) ; Monday, week 1
+                 (2014 12 28) ; Sunday, week 52
+                 )))
+    (dolist [date dates]
+      (let1 d (make-date 0 0 0 0 (caddr date) (cadr date) (car date) 0)
+        (test* (apply format "~d/~2,'0d/~2,'0d" date)
+               (sys-strftime "%V" (sys-gmtime (date->time-utc d)))
+               (date->string d "~V")))))
+  ]) ; cond-expand
+
 ;;
 ;; testing srfi-43
 ;;
@@ -1388,72 +1261,17 @@
               (reverse-list->vector '(0 1 2 3) 2 1))
 
 ;;
-;; Test srfi-133
+;; Test srfi-178
 ;;
 
-(test-section "srfi-133")
+(define-module srfi-178-test
+  (use gauche.test)
+  (use srfi-178)
+  (test-module 'srfi-178)
+  (use srfi-78)
 
-(use compat.chibi-test)
-(use srfi-133)
-(use srfi-11)
-
-(chibi-test
- (include "vectors-test.scm"))
-
-;;
-;; Test srfi-144
-;;
-
-(test-section "srfi-144")
-(use srfi-144)
-(test-module 'srfi-144)
-
-(define-module srfi-144-test
-
-  (define-module srfi.144 (extend srfi-144))
-  (define-module tests.scheme.test
-    (use gauche.test)
-    (export test test/unspec-or-exn test/approx test/approx-1ulp)
-    (define (f=? a b)
-      (cond [(flonum? a)
-             (and (flonum? b)
-                  (cond [(nan? a) (nan? b)]
-                        [(-zero? a) (-zero? b)]
-                        [else (eqv? a b)]))]
-            [(list? a)
-             (and (list? b) (= (length a) (length b))
-                  (every f=? a b))]
-            [else (equal? a b)]))
-    (define-syntax test
-      (syntax-rules ()
-        [(_ expr expected)
-         (test* #"~'expr" expected expr f=?)]))
-    (define-syntax test/unspec-or-exn
-      (syntax-rules ()
-        [(_ expr _)
-         (test* #"~'expr (error)" (test-error) expr)]))
-    (define-syntax test/approx-1ulp
-      (syntax-rules ()
-        [(_ expr expected)
-         (test* #"~'expr (approx 1ulp)" expected expr
-                (^[x y]
-                  (if (list? x)
-                    (and (list? y) (every approx=? x y))
-                    (approx=? x y))))]))
-    ;; NB: MinGW's bessel function jn has greater errors and we need
-    ;; to relax approx check.
-    (define rel-tolerance
-      (cond-expand
-       [gauche.os.windows 5e-7]
-       [else 1e-9]))
-    (define-syntax test/approx
-      (syntax-rules ()
-        [(_ expr expected)
-         (test* #"~'expr (approx)" expected expr
-                (cut approx=? <> <> rel-tolerance 1e-15))])))
-  (use scheme.base)
-  (include "../../test/include/srfi-144-tests.scm")
-  (with-module tests.scheme.flonum
-    (run-flonum-tests)))
+  (define-syntax import (syntax-rules () [(_ _) #f]))
+  (include "../../test/include/srfi-178-tests.scm")
+  )
 
 (test-end)
