@@ -303,7 +303,7 @@
 ;;---------------------------------------------------------------------
 (test-section "port access serialization")
 
-(use srfi-1)
+(use scheme.list)
 
 (define (port-test-chunk-generator nchars c)
   (^[] (make-string nchars c)))
@@ -441,8 +441,6 @@
 ;;---------------------------------------------------------------------
 (test-section "thread-local parameters")
 
-(use gauche.parameter)
-
 (define *thr1-val* #f)
 (define *thr2-val* #f)
 
@@ -504,10 +502,152 @@
          (for-each thread-join! ts)
          (atom-ref a)))
 
+(test* "atomic-update! extra values" '(2 3 4)
+       (let ([a (atom 1 2)])
+         (values->list
+          (atomic-update! a
+                          (^[x y] (values (+ x 1) (+ y 1) (+ x y 1)))))))
+
+;;---------------------------------------------------------------------
+(test-section "semaphore")
+
+(test* "semaphore" #t (semaphore? (make-semaphore 1)))
+
+(test* "semaphore wait" '(2 3 6)
+       (let* ([d (atom 0)]
+              [s (make-semaphore 2)]
+              [ts (map (^i (make-thread (^[]
+                                          (semaphore-acquire! s)
+                                          (atomic-update! d (cut + <> 1)))))
+                       (iota 6))])
+         (for-each thread-start! ts)
+         (let loop ([d1 (atom-ref d)])
+           (cond
+            [(< d1 2) (sys-nanosleep 500) (loop (atom-ref d))]
+            [else
+             (semaphore-release! s)
+             (let loop ([d2 (atom-ref d)])
+               (cond
+                [(= d1 d2) (sys-nanosleep 500) (loop (atom-ref d))]
+                [else
+                 (semaphore-release! s 3)
+                 (for-each thread-join! ts)
+                 (list d1 d2 (atom-ref d 0))]))]))))
+
+;;---------------------------------------------------------------------
+(test-section "latch")
+
+(test* "latch" #t (latch? (make-latch 2)))
+
+(test* "latch operation" '(0 0 5 6)
+       (let* ([d (atom 0)]
+              [l (make-latch 2)]
+              [l2 (make-latch 5)]
+              [ts (map (^i (make-thread (^[]
+                                           (latch-await l)
+                                           (atomic-update! d (cut + <> 1))
+                                           (latch-dec! l2))))
+                       (iota 6))])
+         (for-each thread-start! (cdr ts))
+         (sys-nanosleep 1000)
+         (let ([d1 (atom-ref d 0)])
+           (latch-dec! l)
+           (sys-nanosleep 1000)
+           (let ([d2 (atom-ref d 0)])
+             (latch-dec! l)
+             (latch-await l2)
+             (let ([d3 (atom-ref d 0)])
+               (thread-start! (car ts))
+               (for-each thread-join! ts)
+               (list d1 d2 d3 (atom-ref d 0)))))))
+
+;;---------------------------------------------------------------------
+(test-section "barrier")
+
+(test* "barrier" #t (barrier? (make-barrier 3)))
+
+(let* ([pre (atom 0)]
+       [post (atom 0)]
+       [b (make-barrier 3)]
+       [l0 (make-latch 2)]
+       [l1 (make-latch 3)]
+       [ts (map (^i (make-thread (^[]
+                                   (atomic-update! pre (cut + <> 1))
+                                   (latch-dec! l0)
+                                   (barrier-await b)
+                                   (atomic-update! post (cut + <> 1))
+                                   (latch-dec! l1))))
+                (iota 3))])
+  (for-each thread-start! (cdr ts))
+  (latch-await l0)
+  (test* "simple barrier pre" '(2 0)
+         (list (atom-ref pre 0) (atom-ref post 0)))
+  (thread-start! (car ts))
+  (latch-await l1)
+  (test* "simple barrier post" '(3 3)
+         (list (atom-ref pre 0) (atom-ref post 0)))
+  (for-each thread-join! ts))
+
+(let* ([b (make-barrier 4)]
+       [t0 (make-thread (^[] (barrier-await b #f 'timeout0)))]
+       [t1 (make-thread (^[] (barrier-await b 1e-3 'timeout1)))]
+       [t2 (make-thread (^[] (barrier-await b #f 'timeout2)))])
+  (thread-start! t0)
+  (test* "barrier broken?" #f
+         (barrier-broken? b))
+  (test* "barrier timeout" 'timeout1
+         (thread-join! (thread-start! t1)))
+  (test* "barrier broken?" #t
+         (barrier-broken? b))
+  (test* "broken barrier pass through" 'timeout2
+         (thread-join! (thread-start! t2)))
+  (test* "barrier reset" #f
+         (begin (barrier-reset! b)
+                (barrier-broken? b)))
+  )
+
+(let* ([b (make-barrier 3)]
+       [x (atom '())]
+       [ts (map (^t (make-thread (^[]
+                                   (dotimes [i 3]
+                                     (sys-nanosleep (* t 1000.0))
+                                     (atomic-update! x (cut cons i <>))
+                                     (barrier-await b)))))
+                (iota 3))])
+  (test* "barrier cycle" '(2 2 2 1 1 1 0 0 0)
+         (begin
+           (for-each thread-start! ts)
+           (for-each thread-join! ts)
+           (atom-ref x))))
+
+(let* ([x (atom 0)]
+       [b (make-barrier 3 (^[] (atomic-update! x (cut + <> 1))))]
+       [ts (map (^t (make-thread (^[]
+                                   (sys-nanosleep (* t 100.0))
+                                   (barrier-await b))))
+                (iota 3))])
+  (test* "barrier action once" 1
+         (begin (for-each thread-start! ts)
+                (for-each thread-join! ts)
+                (atom-ref x))))
+
+(let* ([b (make-barrier 3 (^[] (raise 'oops)))]
+       [ts (map (^t (make-thread (^[]
+                                   (sys-nanosleep (* t 100.0))
+                                   (guard (e [else e])
+                                     (barrier-await b #f t)))))
+                (iota 3))])
+  (for-each thread-start! ts)
+  (test* "barrier action once" (test-one-of '(#t 0 1 oops)
+                                            '(#t 0 oops 2)
+                                            '(#t oops 1 2))
+         (let1 rs (map thread-join! ts)
+           (cons (barrier-broken? b) rs))))
+
 ;;---------------------------------------------------------------------
 (test-section "threads and promise")
 
-(use srfi-1)
+(use scheme.list)
 
 (letrec ([x 0]
          [z (delay (begin (sys-nanosleep 100000) (inc! x) 'ok))])
@@ -650,5 +790,101 @@
            (let1 r (list (dequeue/wait! qq) (dequeue/wait! qq))
              (list* r0 r1 r)))))
 
-(test-end)
+(let ()
+  (test* "mtqueue-closed?" '(#f #t)
+         (let* ([q (make-mtqueue)]
+                [a (~ q'closed)]
+                [_ (dequeue/wait! q 0 #f #t)]
+                [b (~ q'closed)])
+           (list a b)))
+  (test* "closed mtqueue rejects enqueue"
+         (test-error <error> #/queue is closed/)
+         (let1 q (make-mtqueue)
+           (dequeue/wait! q 0 #f #t)
+           (enqueue! q 'a)))
+  (test* "closed mtqueue rejects enqueue/wait"
+         (test-error <error> #/queue is closed/)
+         (let1 q (make-mtqueue)
+           (dequeue/wait! q 0 #f #t)
+           (enqueue/wait! q 'a)))
+  (test* "enqueue with closing" #t
+         (let1 q (make-mtqueue)
+           (enqueue/wait! q 'a #f #f #t)
+           (~ q'closed)))
+  )
 
+;;---------------------------------------------------------------------
+(test-section "memo tables")
+
+;; This actually tests built-in memoization table primitives, but
+;; here we stress it with MT-access.
+
+(let ([make-memo-table (with-module gauche.internal make-memo-table)]
+      [memo-table-put! (with-module gauche.internal memo-table-put!)]
+      [memo-table-get2 (with-module gauche.internal memo-table-get2)]
+      [memo-table-putv! (with-module gauche.internal memo-table-putv!)]
+      [memo-table-getv2 (with-module gauche.internal memo-table-getv2)])
+
+  (define fib
+    (let1 tab (make-memo-table 2 1)
+      (lambda (n)
+        (if (<= n 1)
+          1
+          (let1 key (vector n)
+            (receive (val exists?) (memo-table-getv2 tab key)
+              (if exists?
+                val
+                (rlet1 v (+ (fib (- n 1)) (fib (- n 2)))
+                  (memo-table-putv! tab key v)))))))))
+
+  (define string-hash-tab (make-memo-table 3 2))
+
+  (define (string-hash/memo s)
+    (let1 key (list s (box s))
+      (receive (val exists?) (memo-table-get2 string-hash-tab key)
+        (if exists?
+          val
+          (rlet1 n (string-hash s)
+            (memo-table-put! string-hash-tab key n))))))
+
+  ;; avoid using data.random, gauche.generator etc.
+  (define (random-integer n)
+    (modulo (sys-random) n)) ;; not a uniform distribution, but ok here
+
+  (define string-pool
+    (rlet1 vec (make-vector 256)
+      (dotimes [n (vector-length vec)]
+        ($ vector-set! vec n
+           $ list->string 
+           $ map (^_ (integer->char (+ (random-integer 95) 32))) (iota 5)))))
+
+  (define (string-hash-test)
+    (dotimes [n (ash (vector-length string-pool) -1)]
+      ($ string-hash/memo
+         $ vector-ref string-pool
+         $ random-integer (vector-length string-pool))))
+
+  (test* "memoizing fib" (make-list 100 20365011074)
+         (map thread-join!
+              (map (^_ (thread-start! (make-thread (^[] (fib 50)))))
+                   (iota 100))))
+
+  (sys-srandom 42)
+
+  (test* "memoizing string hash" #t
+         (let/cc return
+           ($ for-each thread-join! 
+              $ map thread-start!
+              $ map (^_ (make-thread string-hash-test)) (iota 100))
+           (positive? 
+            (count (^s (receive (val exists?)
+                           (memo-table-get2 string-hash-tab (list s (box s)))
+                         (and exists?
+                              (or (= val (string-hash s))
+                                  (return val)))))
+                   (vector->list string-pool)))))
+
+  ;;((with-module gauche.internal memo-table-dump) string-hash-tab)
+  )
+
+(test-end)
