@@ -1,7 +1,7 @@
 /*
  * thread.c - Scheme thread API
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -152,7 +152,7 @@ static struct threadRec {
    This is to prevent exited thread's system resources from being
    uncollected.
  */
-ScmObj Scm_ThreadStart(ScmVM *vm)
+ScmObj Scm_ThreadStart(ScmVM *vm, u_long flags)
 {
     int err_state = FALSE, err_create = FALSE;
 
@@ -196,7 +196,12 @@ ScmObj Scm_ThreadStart(ScmVM *vm)
 #endif
     }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(vm->vmlock);
-    if (err_state) Scm_Error("attempt to start an already-started thread: %S", vm);
+    if (err_state) {
+        if (flags & SCM_THREAD_START_TRYSTART) {
+            return SCM_FALSE;
+        }
+        Scm_Error("attempt to start an already-started thread: %S", vm);
+    }
     if (err_create) Scm_Error("couldn't start a new thread: %S", vm);
     return SCM_OBJ(vm);
 }
@@ -390,11 +395,26 @@ ScmObj Scm_ThreadSleep(ScmObj timeout)
    We try to terminate the thread gracefully as possible.
    First, we use vm->stopRequest mechanism.  If the target thread is
    in VM loop, it responds to the flag and terminates itself.
-   If that fails, then we use more aggressive means.
 
-   TODO: We should probably make it configurable whether to use the
-   forcible termination---it is too dangerous.
- */
+   If that fails, we send a signal to the thread.  If the thread is blocked
+   by a system call, this most likely unblocks the thread and termnates it.
+   (We use a dedicated signal.
+
+   There are cases when signals can fail.  On POSIX platform, if the thread
+   is waiting on pthread_cond_wait, it is not guaranteed to be interrupted
+   with a signal.  On Windows platform, system calls can't be interrupted
+   so we don't have this option.  (We may, in future, use CancelSynchronousIo
+   to interrupt at least synchronous I/O calls).
+
+   If the above steps all fails and SCM_THREAD_TERIMINATE_FORCIBLE is passed
+   in FLAGS, we resort to use pthread_cancel or TerminateThread.  This is not
+   safe---the thread may leave memory in inconsistent state.
+
+   In any way, the target thread is marked as SCM_VM_TERMINATED.  If we
+   avoid the forcible termiantion, the thread remains in the system, but
+   as soon as it unblocks and resumes execution of Gauche code, it will
+   see the terminate request and terminate itself.
+*/
 
 /* Caller must hold target->vmlock.  return TRUE if the target terminates
    gracefully. */
@@ -411,7 +431,7 @@ static int wait_for_termination(ScmVM *target)
     return (r == 0);
 }
 
-ScmObj Scm_ThreadTerminate(ScmVM *target)
+ScmObj Scm_ThreadTerminate(ScmVM *target, u_long flags)
 {
     ScmVM *vm = Scm_VM();
     if (target == vm) {
@@ -428,41 +448,61 @@ ScmObj Scm_ThreadTerminate(ScmVM *target)
     }
 
     (void)SCM_INTERNAL_MUTEX_LOCK(target->vmlock);
-    if (target->state == SCM_VM_RUNNABLE || target->state == SCM_VM_STOPPED) {
-        do {
-            /* This ensures only the first call of thread-terminate! on a
-               thread is in effect. */
-            if (target->canceller == NULL) {
-                target->canceller = vm;
+    if (target->state != SCM_VM_TERMINATED) {
+        /* This ensures only the first call of thread-terminate! on a
+           thread is in effect. */
+        if (target->canceller == NULL) {
+            target->canceller = vm;
 
-                /* First try */
-                target->stopRequest = SCM_VM_REQUEST_TERMINATE;
-                target->attentionRequest = TRUE;
-                if (wait_for_termination(target)) break;
+            if (target->state == SCM_VM_NEW) {
+                /* The thread is being killed before started.  We don't have
+                   the actual thread, so just clean up. */
+                thread_cleanup_inner(target);
+            } else {
+                do {
+                    /* First try */
+                    target->stopRequest = SCM_VM_REQUEST_TERMINATE;
+                    target->attentionRequest = TRUE;
+                    if (wait_for_termination(target)) break;
 
-                /* Second try */
-                SCM_ASSERT(target->thread);
+                    /* Second try */
+                    SCM_ASSERT(target->thread);
 #if defined(GAUCHE_USE_PTHREADS)
 # if defined(GAUCHE_PTHREAD_SIGNAL)
-                pthread_kill(target->thread, GAUCHE_PTHREAD_SIGNAL);
+                    pthread_kill(target->thread, GAUCHE_PTHREAD_SIGNAL);
 # endif /*defined(GAUCHE_PTHREAD_SIGNAL)*/
 #elif defined(GAUCHE_USE_WTHREADS)
-                /* TODO: implement signal mechanism using an event */
+                    /* TODO: implement signal mechanism using an event */
 #endif  /* defined(GAUCHE_USE_WTHREADS) */
-                if (wait_for_termination(target)) break;
+                    if (wait_for_termination(target)) break;
 
-                /* Last resort */
-                thread_cleanup_inner(target);
+                    thread_cleanup_inner(target);
+
+                    /* If forcible termination is requested, we resort to
+                       the extreme measure. */
+                    if (flags & SCM_THREAD_TERMINATE_FORCIBLE) {
 #if defined(GAUCHE_USE_PTHREADS)
-                pthread_cancel(target->thread);
+# if defined(HAVE_PTHREAD_CANCEL)
+                        /* pthread_cancel() may be redirected to
+                           GC_pthread_cancel(), but that only happens
+                           if the system has pthread_cancel anyway.
+                         */
+                        pthread_cancel(target->thread);
+# else
+                        /* The system lacks pthread_cancel(), e.g. on Android.
+                           https://github.com/shirok/Gauche/issues/790
+                           We have no choice but do nothing. */
+                        ;
+# endif
 #elif defined(GAUCHE_USE_WTHREADS)
-                TerminateThread(target->thread, 0);
+                        TerminateThread(target->thread, 0);
 #endif
+                    }
+                } while (0);
             }
-        } while (0);
+        }
     }
     /* target either is terminated or hasn't been run */
-    target->state = SCM_VM_TERMINATED;
     (void)SCM_INTERNAL_MUTEX_UNLOCK(target->vmlock);
     return SCM_UNDEFINED;
 }
