@@ -1,7 +1,7 @@
 /*
  * threads.h - Gauche threads support
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -43,13 +43,23 @@
  * Scheme-level thread API
  */
 
+/* flag for Scm_ThreadTerminate */
+enum {
+    SCM_THREAD_TERMINATE_FORCIBLE = 1
+};
+
+/* flag for Scm_ThreadStart */
+enum {
+    SCM_THREAD_START_TRYSTART = 1
+};
+
 extern ScmObj Scm_MakeThread(ScmProcedure *thunk, ScmObj name);
-extern ScmObj Scm_ThreadStart(ScmVM *vm);
+extern ScmObj Scm_ThreadStart(ScmVM *vm, u_long flags);
 extern ScmObj Scm_ThreadJoin(ScmVM *vm, ScmObj timeout, ScmObj timeoutval);
 extern ScmObj Scm_ThreadStop(ScmVM *vm, ScmObj timeout, ScmObj timeoutval);
 extern ScmObj Scm_ThreadCont(ScmVM *vm);
 extern ScmObj Scm_ThreadSleep(ScmObj timeout);
-extern ScmObj Scm_ThreadTerminate(ScmVM *vm);
+extern ScmObj Scm_ThreadTerminate(ScmVM *vm, u_long flags);
 
 /*---------------------------------------------------------
  * SYNCHRONIZATION DEVICES
