@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.uvector - uniform vectors
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,17 +44,18 @@
 (define-module gauche.uvector
   (use gauche.collection)
   (use gauche.sequence)
-  (use data.queue)
 
   (include "./exports")
   (export make-uvector
 
           open-output-uvector get-output-uvector port->uvector
-          read-block! read-uvector read-uvector! referencer
+          read-block! read-uvector read-uvector!
+          read-bytevector read-bytevector!
+          referencer
 
           string->s8vector string->s8vector!
           string->u8vector string->u8vector!
-          string->s32vector string->s32vector! 
+          string->s32vector string->s32vector!
           string->u32vector string->u32vector!
           s8vector->string u8vector->string
           s32vector->string u32vector->string
@@ -63,13 +64,15 @@
           uvector-copy uvector-copy! uvector-ref uvector-set! uvector-size
           uvector->list uvector->vector uvector-swap-bytes uvector-swap-bytes!
 
-          write-block write-uvector
+          write-block write-uvector write-bytevector
 
-          ;; R7RS compatibility
-          bytevector bytevector? make-bytevector
+          ;; R7RS compatibility (scheme base) and (scheme bytevector)
+          bytevector bytevector? make-bytevector bytevector-fill!
           bytevector-length bytevector-u8-ref bytevector-u8-set!
-          bytevector-copy bytevector-copy! bytevector-append
-          bytevector=? ; not in (scheme base), but bonus
+          bytevector-s8-ref bytevector-s8-set!
+          bytevector->u8-list u8-list->bytevector
+          bytevector-copy bytevector-copy! bytevector-copy!-r6
+          bytevector-append bytevector=?
           ))
 (select-module gauche.uvector)
 
@@ -87,6 +90,7 @@
   (.define EXTUVECTOR_EXPORTS)
   (.include "gauche/uvector.h")
   (.include "gauche/priv/vectorP.h")
+  (.include "gauche/priv/bytesP.h")
   (.include "uvectorP.h")))
 
 ;; uvlib.scm is generated by uvlib.scm.tmpl
@@ -142,7 +146,7 @@
 ;; uvector->list
 ;; uvector->vector
 (inline-stub
- (define-cproc uvector->list (v::<uvector> 
+ (define-cproc uvector->list (v::<uvector>
                               :optional (start::<fixnum> 0)
                                         (end::<fixnum> -1))
    (case (Scm_UVectorType (Scm_ClassOf (SCM_OBJ v)))
@@ -176,7 +180,7 @@
                                                        start end))]
      [else (Scm_Error "[internal] Invalid uvector type: %S" v)
            (return SCM_UNDEFINED)]))
- (define-cproc uvector->vector (v::<uvector> 
+ (define-cproc uvector->vector (v::<uvector>
                                 :optional (start::<fixnum> 0)
                                           (end::<fixnum> -1))
    (case (Scm_UVectorType (Scm_ClassOf (SCM_OBJ v)))
@@ -209,7 +213,7 @@
      [(SCM_UVECTOR_C128) (return (Scm_C128VectorToVector (SCM_C128VECTOR v)
                                                          start end))]
      [else (Scm_Error "[internal] Invalid uvector type: %S" v)
-           (return SCM_UNDEFINED)])))   
+           (return SCM_UNDEFINED)])))
 
 ;; allocation by class
 (inline-stub
@@ -219,35 +223,35 @@
    (let* ([v (Scm_MakeUVector klass size NULL)])
      (case (Scm_UVectorType klass)
        [(SCM_UVECTOR_S8)
-        (Scm_S8VectorFill (SCM_S8VECTOR v) 
+        (Scm_S8VectorFill (SCM_S8VECTOR v)
                           (Scm_GetInteger8Clamp init SCM_CLAMP_ERROR NULL)
                           0 -1)]
        [(SCM_UVECTOR_U8)
-        (Scm_U8VectorFill (SCM_U8VECTOR v) 
+        (Scm_U8VectorFill (SCM_U8VECTOR v)
                           (Scm_GetIntegerU8Clamp init SCM_CLAMP_ERROR NULL)
                           0 -1)]
        [(SCM_UVECTOR_S16)
-        (Scm_S16VectorFill (SCM_S16VECTOR v) 
+        (Scm_S16VectorFill (SCM_S16VECTOR v)
                            (Scm_GetInteger16Clamp init SCM_CLAMP_ERROR NULL)
                            0 -1)]
        [(SCM_UVECTOR_U16)
-        (Scm_U16VectorFill (SCM_U16VECTOR v) 
+        (Scm_U16VectorFill (SCM_U16VECTOR v)
                            (Scm_GetIntegerU16Clamp init SCM_CLAMP_ERROR NULL)
                            0 -1)]
        [(SCM_UVECTOR_S32)
-        (Scm_S32VectorFill (SCM_S32VECTOR v) 
+        (Scm_S32VectorFill (SCM_S32VECTOR v)
                            (Scm_GetInteger32Clamp init SCM_CLAMP_ERROR NULL)
                            0 -1)]
        [(SCM_UVECTOR_U32)
-        (Scm_U32VectorFill (SCM_U32VECTOR v) 
+        (Scm_U32VectorFill (SCM_U32VECTOR v)
                            (Scm_GetIntegerU32Clamp init SCM_CLAMP_ERROR NULL)
                            0 -1)]
        [(SCM_UVECTOR_S64)
-        (Scm_S64VectorFill (SCM_S64VECTOR v) 
+        (Scm_S64VectorFill (SCM_S64VECTOR v)
                            (Scm_GetInteger64Clamp init SCM_CLAMP_ERROR NULL)
                            0 -1)]
        [(SCM_UVECTOR_U64)
-        (Scm_U64VectorFill (SCM_U64VECTOR v) 
+        (Scm_U64VectorFill (SCM_U64VECTOR v)
                            (Scm_GetIntegerU64Clamp init SCM_CLAMP_ERROR NULL)
                            0 -1)]
        [(SCM_UVECTOR_F16)
@@ -267,7 +271,7 @@
        [(SCM_UVECTOR_C64)
         (Scm_C64VectorFill (SCM_C64VECTOR v) (Scm_GetFloatComplex init) 0 -1)]
        [(SCM_UVECTOR_C128)
-        (Scm_C128VectorFill (SCM_C128VECTOR v) 
+        (Scm_C128VectorFill (SCM_C128VECTOR v)
                             (Scm_GetDoubleComplex init) 0 -1)]
        [else (Scm_Error "[internal] Invalid uvector class: %S" klass)])
      (return v))))
@@ -293,7 +297,7 @@
          [(SCM_FALSEP arg) (return fallback)]
          [else (Scm_Error "%s expects fixnum or #f, but got: %S" name arg)
                (return 0)])) ; dummy
- 
+
  (define-cproc uvector-binary-search (v::<uvector> key::<number>
                                                    :optional
                                                    (start #f)
@@ -355,11 +359,11 @@
          [(SCM_UVECTOR_F16)
           (let* ([k::ScmHalfFloat (Scm_DoubleToHalf (Scm_GetDouble key))])
             (set! r (Scm_BinarySearchF16 (+ (SCM_F16VECTOR_ELEMENTS v) s)
-                                         (- e s) k p (& lb) (& ub))))]         
+                                         (- e s) k p (& lb) (& ub))))]
          [(SCM_UVECTOR_F32)
           (let* ([k::float (Scm_GetDouble key)])
             (set! r (Scm_BinarySearchF32 (+ (SCM_F32VECTOR_ELEMENTS v) s)
-                                         (- e s) k p (& lb) (& ub))))]         
+                                         (- e s) k p (& lb) (& ub))))]
          [(SCM_UVECTOR_F64)
           (let* ([k::double (Scm_GetDouble key)])
             (set! r (Scm_BinarySearchF64 (+ (SCM_F64VECTOR_ELEMENTS v) s)
@@ -367,15 +371,15 @@
          [(SCM_UVECTOR_C32)
           (let* ([k::ScmHalfComplex (Scm_GetHalfComplex key)])
             (set! r (Scm_BinarySearchC32 (+ (SCM_C32VECTOR_ELEMENTS v) s)
-                                         (- e s) k p (& lb) (& ub))))]         
+                                         (- e s) k p (& lb) (& ub))))]
          [(SCM_UVECTOR_C64)
-          (let* ([k::(complex float) (Scm_GetFloatComplex key)])
+          (let* ([k::(ScmFloatComplex) (Scm_GetFloatComplex key)])
             (set! r (Scm_BinarySearchC64 (+ (SCM_C64VECTOR_ELEMENTS v) s)
-                                         (- e s) k p (& lb) (& ub))))]         
+                                         (- e s) k p (& lb) (& ub))))]
          [(SCM_UVECTOR_C128)
-          (let* ([k::(complex double) (Scm_GetDoubleComplex key)])
+          (let* ([k::(ScmDoubleComplex) (Scm_GetDoubleComplex key)])
             (set! r (Scm_BinarySearchC128 (+ (SCM_C128VECTOR_ELEMENTS v) s)
-                                          (- e s) k p (& lb) (& ub))))] 
+                                          (- e s) k p (& lb) (& ub))))]
          [else (Scm_Error "[internal] Invalid uvector type: %S" v)]
          )
        (when (== r (cast (size_t) -1))
@@ -399,7 +403,7 @@
                                         (endian::<symbol>? #f))
    Scm_ReadBlockX)
 
- (define-cproc read-uvector (klass::<class> size::<fixnum> 
+ (define-cproc read-uvector (klass::<class> size::<fixnum>
                              :optional (port::<input-port> (current-input-port))
                                        (endian::<symbol>? #f))
    (unless (Scm_SubtypeP klass SCM_CLASS_UVECTOR)
@@ -427,6 +431,20 @@
    Scm_WriteBlock)
  )
 
+;; R7RS bytevectors
+(define (read-bytevector k :optional (port (current-input-port)))
+  (read-uvector <u8vector> k port))
+(define (read-bytevector! bv :optional (port (current-input-port))
+                          (start 0)
+                          (end (uvector-length bv)))
+  (assume-type bv <u8vector>)
+  (read-uvector! bv port start end))
+(define (write-bytevector bv :optional (port (current-output-port))
+                                       (start 0)
+                                       (end (uvector-length bv)))
+  (assume-type bv <u8vector>)
+  (write-uvector bv port start end))
+
 ;; copy
 (inline-stub
  (define-cproc uvector-copy! (dest::<uvector> dstart::<int> src::<uvector>
@@ -467,9 +485,9 @@
                          (+ ,ss ,size)
                          (Scm_StringBodyPosition ,sb ,end))])
            ,@body)))])
- 
- (define-cfn string->bytevector (klass::ScmClass* 
-                                 s::ScmString* 
+
+ (define-cfn string->bytevector (klass::ScmClass*
+                                 s::ScmString*
                                  start::ScmSmallInt
                                  end::ScmSmallInt
                                  immutable::int)
@@ -494,10 +512,10 @@
     :optional (start::<fixnum> 0) (end::<fixnum> -1) (immutable?::<boolean> #f))
    (return (string->bytevector SCM_CLASS_U8VECTOR s start end immutable?)))
 
- (define-cfn string->bytevector! (v::ScmUVector* 
+ (define-cfn string->bytevector! (v::ScmUVector*
                                   tstart::ScmSmallInt
                                   s::ScmString*
-                                  start::ScmSmallInt 
+                                  start::ScmSmallInt
                                   end::ScmSmallInt)
    :static
    (let* ([tlen::ScmSmallInt (SCM_UVECTOR_SIZE v)])
@@ -524,8 +542,8 @@
                                   (end::<fixnum> -1))
    (return (string->bytevector! (SCM_UVECTOR v) tstart s start end)))
 
- (define-cfn bytevector->string (v::ScmUVector* 
-                                 start::ScmSmallInt 
+ (define-cfn bytevector->string (v::ScmUVector*
+                                 start::ScmSmallInt
                                  end::ScmSmallInt
                                  term)
    :static
@@ -575,45 +593,65 @@
    (return (bytevector->string (SCM_UVECTOR v) start end terminator)))
 
  (define-cfn string->wordvector (klass::ScmClass* s::ScmString*
-                                 start::ScmSmallInt end::ScmSmallInt)
+                                 start::ScmSmallInt end::ScmSmallInt
+                                 endian::ScmObj)
    :static
+   (unless (SCM_SYMBOLP endian)
+     (set! endian (Scm_DefaultEndian)))
    (with-input-string-pointers (s start end sp ep)
      (let* ([v (Scm_MakeUVector klass (- end start) NULL)]
-            [eltp::int32_t* (cast int32_t* (SCM_UVECTOR_ELEMENTS v))]
-            [i::ScmSmallInt 0])
+            [eltp::uint32_t* (cast uint32_t* (SCM_UVECTOR_ELEMENTS v))]
+            [i::ScmSmallInt 0]
+            [do_swap::int (SWAP_REQUIRED endian)])
        (for [() (< sp ep) (post++ i)]
          (let* ([ch::ScmChar])
            (SCM_CHAR_GET sp ch)
-           (set! (aref eltp i) ch)
+           (if do_swap
+             (let* ([v::swap_u32_t])
+               (set! (ref v val) (cast uint32_t ch))
+               (SWAP_4 v)
+               (set! (aref eltp i) (ref v val)))
+             (set! (aref eltp i) (cast uint32_t ch)))
            (+= sp (SCM_CHAR_NBYTES ch))))
        (return v))))
 
  (define-cproc string->s32vector (s::<string>
                                   :optional (start::<fixnum> 0)
-                                            (end::<fixnum> -1))
-   (return (string->wordvector SCM_CLASS_S32VECTOR s start end)))
+                                            (end::<fixnum> -1)
+                                            endian)
+   (return (string->wordvector SCM_CLASS_S32VECTOR s start end endian)))
 
  (define-cproc string->u32vector (s::<string>
                                   :optional (start::<fixnum> 0)
-                                            (end::<fixnum> -1))
-   (return (string->wordvector SCM_CLASS_U32VECTOR s start end)))
+                                            (end::<fixnum> -1)
+                                            endian)
+   (return (string->wordvector SCM_CLASS_U32VECTOR s start end endian)))
 
  (define-cfn string->wordvector! (v::ScmUVector*
                                   tstart::ScmSmallInt
                                   s::ScmString*
                                   start::ScmSmallInt
-                                  end::ScmSmallInt)
+                                  end::ScmSmallInt
+                                  endian::ScmObj)
    :static
+   (unless (SCM_SYMBOLP endian)
+     (set! endian (Scm_DefaultEndian)))
    (let* ([tlen::ScmSmallInt (SCM_UVECTOR_SIZE v)])
      (when (and (>= tstart 0) (< tstart tlen))
        (SCM_UVECTOR_CHECK_MUTABLE v)
        (with-input-string-pointers (s start end sp ep)
-         (let* ([buf::int32_t* (cast int32_t* (SCM_UVECTOR_ELEMENTS v))]
-                [i::ScmSmallInt tstart])
+         (let* ([buf::uint32_t* (cast uint32_t* (SCM_UVECTOR_ELEMENTS v))]
+                [i::ScmSmallInt tstart]
+                [do_swap::int (SWAP_REQUIRED endian)])
            (for [() (and (< sp ep) (< i tlen)) (post++ i)]
              (let* ([ch::ScmChar])
                (SCM_CHAR_GET sp ch)
-               (set! (aref buf i) ch)
+               (if do_swap
+                 (let* ([v::swap_u32_t])
+                   (set! (ref v val) (cast uint32_t ch))
+                   (SWAP_4 v)
+                   (set! (aref buf i) (ref v val)))
+                 (set! (aref buf i) (cast uint32_t ch)))
                (+= sp (SCM_CHAR_NBYTES ch)))))))
      (return (SCM_OBJ v))))
 
@@ -621,27 +659,40 @@
                                    tstart::<fixnum>
                                    s::<string>
                                    :optional (start::<fixnum> 0)
-                                             (end::<fixnum> -1))
-   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end)))
+                                             (end::<fixnum> -1)
+                                             endian)
+   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end endian)))
 
  (define-cproc string->u32vector! (v::<u32vector>
                                    tstart::<fixnum>
                                    s::<string>
                                    :optional (start::<fixnum> 0)
-                                             (end::<fixnum> -1))
-   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end)))
+                                             (end::<fixnum> -1)
+                                             endian)
+   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end endian)))
 
- (define-cfn wordvector->string (v::ScmUVector* 
+ (define-cfn wordvector->string (v::ScmUVector*
                                  start::ScmSmallInt
                                  end::ScmSmallInt
-                                 term)
+                                 term
+                                 endian)
    :static
+   (unless (SCM_SYMBOLP endian)
+     (set! endian (Scm_DefaultEndian)))
    (let* ([len::ScmSmallInt (SCM_UVECTOR_SIZE v)]
-          [s (Scm_MakeOutputStringPort FALSE)])
+          [s (Scm_MakeOutputStringPort FALSE)]
+          [do_swap::int (SWAP_REQUIRED endian)])
      (SCM_CHECK_START_END start end len)
      (let* ([eltp::int32_t* (cast int32_t* (SCM_UVECTOR_ELEMENTS v))])
        (while (< start end)
-         (let* ([ch::ScmChar (cast ScmChar (aref eltp (post++ start)))])
+         (let* ([ch::ScmChar]
+                [val::uint32_t (aref eltp (post++ start))])
+           (if do_swap
+             (let* ([v::swap_u32_t])
+               (set! (ref v val) val)
+               (SWAP_4 v)
+               (set! ch (cast ScmChar (ref v val))))
+             (set! ch (cast ScmChar val)))
            (when (and (SCM_INTP term)
                       (== (SCM_INT_VALUE term) ch))
              (break))
@@ -651,14 +702,16 @@
  (define-cproc s32vector->string (v::<s32vector>
                                   :optional (start::<fixnum> 0)
                                             (end::<fixnum> -1)
-                                            (terminator #f))
-   (return (wordvector->string (SCM_UVECTOR v) start end terminator)))
+                                            (terminator #f)
+                                            endian)
+   (return (wordvector->string (SCM_UVECTOR v) start end terminator endian)))
 
  (define-cproc u32vector->string (v::<u32vector>
                                   :optional (start::<fixnum> 0)
                                             (end::<fixnum> -1)
-                                            (terminator #f))
-   (return (wordvector->string (SCM_UVECTOR v) start end terminator)))
+                                            (terminator #f)
+                                            endian)
+   (return (wordvector->string (SCM_UVECTOR v) start end terminator endian)))
  )
 
 ;; for the bakcward compatibility
@@ -751,9 +804,18 @@
 ;; Bytevector aliases (R7RS compatibility)
 ;;
 
+(define (%adjust-fill-arg fill)
+  (cond [(<= 0 fill 255) fill]
+        [(<= -128 fill -1) (logand fill #xff)]
+        [else (error "fill argument out of range" fill)]))
+
+(define (make-bytevector len :optional (fill 0))
+  (make-u8vector len (%adjust-fill-arg fill)))
+(define (bytevector-fill! v fill)       ; scheme.bytevector
+  (u8vector-fill! v (%adjust-fill-arg fill)))
+
 (define-inline bytevector         u8vector)
 (define-inline bytevector?        u8vector?)
-(define-inline make-bytevector    make-u8vector)
 (define-inline bytevector-length  u8vector-length)
 (define-inline bytevector-u8-ref  u8vector-ref)
 (define-inline bytevector-u8-set! u8vector-set!)
@@ -761,3 +823,18 @@
 (define-inline bytevector-copy!   u8vector-copy!)
 (define-inline bytevector-append  u8vector-append)
 (define-inline bytevector=?       u8vector=?)
+
+(define (bytevector-s8-set! v k b)      ; scheme.bytevector
+  (bytevector-u8-set! v k (logand b #xff)))
+(define bytevector-s8-ref         ; scheme.bytevector
+  (letrec ([bytevector-s8-ref
+            (^[v k]
+              (let1 b (bytevector-u8-ref v k)
+                (if (>= b 128) (- b 256) b)))])
+    (getter-with-setter bytevector-s8-ref bytevector-s8-set!)))
+
+(define (bytevector-copy!-r6 src sstart target tstart len) ; scheme.bytevector
+  (u8vector-copy! target tstart src sstart (+ sstart len)))
+
+(define (bytevector->u8-list v) (u8vector->list v))     ; scheme.bytevector
+(define (u8-list->bytevector lis) (list->u8vector lis)) ; scheme.bytevector
