@@ -1,7 +1,7 @@
 /*
  * vport.c - 'virtual port'
  *
- *   Copyright (c) 2004-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -32,8 +32,8 @@
  */
 
 #include <gauche.h>
-#include <gauche/class.h>
 #include <gauche/extend.h>
+#include <gauche/priv/portP.h>
 
 #include "vport.h"
 
@@ -63,6 +63,10 @@ SCM_DEFINE_BASE_CLASS(Scm_VirtualOutputPortClass, ScmPort,
                       vport_print, NULL, NULL,
                       vport_allocate, vport_cpa);
 
+SCM_DEFINE_BASE_CLASS(Scm_VirtualIOPortClass, ScmPort,
+                      vport_print, NULL, NULL,
+                      vport_allocate, vport_cpa);
+
 /*
  * Scheme handlers.  They are visible from Scheme as instance slots.
  * Any of these slots can be #f - if possible, the vport tries to fulfill
@@ -80,14 +84,18 @@ typedef struct vport_rec {
     ScmObj flush_proc;          /* () -> () */
     ScmObj close_proc;          /* () -> () */
     ScmObj seek_proc;           /* (Offset, Whence) -> Offset */
+    ScmObj getpos_proc;         /* () -> Pos */
+    ScmObj setpos_proc;         /* (Pos) -> () */
 } vport;
 
+#define VPORT(port)   ((vport*)PORT_VT(port)->data)
+
 /*------------------------------------------------------------
  * Vport Getb
  */
 static int vport_getb(ScmPort *p)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
 
     if (SCM_FALSEP(data->getb_proc)) {
@@ -122,7 +130,7 @@ static int vport_getb(ScmPort *p)
  */
 static int vport_getc(ScmPort *p)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
 
     if (SCM_FALSEP(data->getc_proc)) {
@@ -133,7 +141,7 @@ static int vport_getc(ScmPort *p)
         ScmObj b = Scm_ApplyRec(data->getb_proc, SCM_NIL);
         if (!SCM_INTP(b)) return EOF;
         buf[0] = (char)SCM_INT_VALUE(b);
-        int n = SCM_CHAR_NFOLLOWS(p->scratch[0]);
+        int n = SCM_CHAR_NFOLLOWS(P_(p)->scratch[0]);
         for (int i=0; i<n; i++) {
             b = Scm_ApplyRec(data->getb_proc, SCM_NIL);
             if (!SCM_INTP(b)) {
@@ -157,7 +165,7 @@ static int vport_getc(ScmPort *p)
  */
 static ScmSize vport_getz(char *buf, ScmSize buflen, ScmPort *p)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
 
     if (!SCM_FALSEP(data->gets_proc)) {
@@ -192,7 +200,7 @@ static ScmSize vport_getz(char *buf, ScmSize buflen, ScmPort *p)
  */
 static int vport_ready(ScmPort *p, int charp)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
 
     if (!SCM_FALSEP(data->ready_proc)) {
@@ -210,7 +218,7 @@ static int vport_ready(ScmPort *p, int charp)
  */
 static void vport_putb(ScmByte b, ScmPort *p)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
 
     if (SCM_FALSEP(data->putb_proc)) {
@@ -234,7 +242,7 @@ static void vport_putb(ScmByte b, ScmPort *p)
  */
 static void vport_putc(ScmChar c, ScmPort *p)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
 
     if (SCM_FALSEP(data->putc_proc)) {
@@ -259,7 +267,7 @@ static void vport_putc(ScmChar c, ScmPort *p)
  */
 static void vport_putz(const char *buf, ScmSize size, ScmPort *p)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
 
     if (!SCM_FALSEP(data->puts_proc)) {
@@ -282,7 +290,7 @@ static void vport_putz(const char *buf, ScmSize size, ScmPort *p)
  */
 static void vport_puts(ScmString *s, ScmPort *p)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     const ScmStringBody *b = SCM_STRING_BODY(s);
     SCM_ASSERT(data != NULL);
 
@@ -312,7 +320,7 @@ static void vport_puts(ScmString *s, ScmPort *p)
  */
 static void vport_flush(ScmPort *p)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
     if (!SCM_FALSEP(data->flush_proc)) {
         Scm_ApplyRec(data->flush_proc, SCM_NIL);
@@ -324,7 +332,7 @@ static void vport_flush(ScmPort *p)
  */
 static void vport_close(ScmPort *p)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
     if (!SCM_FALSEP(data->close_proc)) {
         Scm_ApplyRec(data->close_proc, SCM_NIL);
@@ -336,7 +344,7 @@ static void vport_close(ScmPort *p)
  */
 static off_t vport_seek(ScmPort *p, off_t off, int whence)
 {
-    vport *data = (vport*)p->src.vt.data;
+    vport *data = VPORT(p);
     SCM_ASSERT(data != NULL);
     if (!SCM_FALSEP(data->seek_proc)) {
         ScmObj r = Scm_ApplyRec(data->seek_proc,
@@ -349,6 +357,33 @@ static off_t vport_seek(ScmPort *p, off_t off, int whence)
     return (off_t)-1;
 }
 
+static ScmObj vport_getpos(ScmPort *p)
+{
+    vport *data = VPORT(p);
+    SCM_ASSERT(data != NULL);
+    if (!SCM_FALSEP(data->getpos_proc)) {
+        return Scm_ApplyRec(data->getpos_proc, SCM_NIL);
+    } else if (!SCM_FALSEP(data->seek_proc)) {
+        return Scm_ApplyRec(data->seek_proc,
+                            SCM_LIST2(SCM_MAKE_INT(0),
+                                      SCM_MAKE_INT(SEEK_CUR)));
+    }
+    return SCM_UNDEFINED;
+}
+
+static void vport_setpos(ScmPort *p, ScmObj pos)
+{
+    vport *data = VPORT(p);
+    SCM_ASSERT(data != NULL);
+    if (!SCM_FALSEP(data->setpos_proc)) {
+        Scm_ApplyRec(data->setpos_proc, SCM_LIST1(pos));
+    } else if (!SCM_FALSEP(data->seek_proc)) {
+        Scm_ApplyRec(data->seek_proc,
+                     SCM_LIST2(pos, SCM_MAKE_INT(SEEK_SET)));
+    }
+}
+
+
 /*------------------------------------------------------------
  * Allocation & wiring
  */
@@ -367,6 +402,8 @@ static ScmObj vport_allocate(ScmClass *klass, ScmObj initargs)
     data->flush_proc = SCM_FALSE;
     data->close_proc = SCM_FALSE;
     data->seek_proc = SCM_FALSE;
+    data->getpos_proc = SCM_FALSE;
+    data->setpos_proc = SCM_FALSE;
 
     ScmPortVTable vtab;
     vtab.Getb = vport_getb;
@@ -379,23 +416,30 @@ static ScmObj vport_allocate(ScmClass *klass, ScmObj initargs)
     vtab.Puts = vport_puts;
     vtab.Flush = vport_flush;
     vtab.Close = vport_close;
-    vtab.Seek  = vport_seek;
+    /* these procedures will be set according to the slot values */
+    vtab.Seek  = NULL;
+    vtab.GetPos = NULL;
+    vtab.SetPos = NULL;
+    vtab.flags = 0;
 
     int dir = 0;
     if (Scm_SubtypeP(klass, SCM_CLASS_VIRTUAL_INPUT_PORT)) {
         dir = SCM_PORT_INPUT;
     } else if (Scm_SubtypeP(klass, SCM_CLASS_VIRTUAL_OUTPUT_PORT)) {
         dir = SCM_PORT_OUTPUT;
+    } else if (Scm_SubtypeP(klass, SCM_CLASS_VIRTUAL_IO_PORT)) {
+        dir = SCM_PORT_INPUT | SCM_PORT_OUTPUT;
     } else {
         Scm_Panic("vport_allocate: implementation error (class wiring screwed?)");
     }
-    ScmObj port = Scm_MakeVirtualPort(klass, dir, &vtab);
-    SCM_PORT(port)->src.vt.data = data;
-    SCM_PORT(port)->name = Scm_GetKeyword(key_name, initargs, SCM_FALSE);
+    ScmObj name = Scm_GetKeyword(key_name, initargs, SCM_FALSE);
+    ScmObj port = Scm_MakeVirtualPortFull(klass, name, dir, &vtab,
+                                          SCM_PORT_WITH_POSITION);
+    PORT_VT(port)->data = data;
     return port;
 }
 
-static void vport_print(ScmObj obj, ScmPort *port, 
+static void vport_print(ScmObj obj, ScmPort *port,
                         ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(port, "#<%A%s %A %p>",
@@ -406,20 +450,24 @@ static void vport_print(ScmObj obj, ScmPort *port,
 }
 
 /* Accessors */
-#define VPORT_ACC(name)                                                 \
-    static ScmObj SCM_CPP_CAT3(vport_,name,_get) (ScmObj p)             \
-    {                                                                   \
-        vport *data = (vport*)SCM_PORT(p)->src.vt.data;                 \
-        SCM_ASSERT(data != NULL);                                       \
-        return data->SCM_CPP_CAT(name,_proc);                           \
-    }                                                                   \
+#define VPORT_GET(name)                                         \
+    static ScmObj SCM_CPP_CAT3(vport_,name,_get) (ScmObj p)     \
+    {                                                           \
+        vport *data = VPORT(p);                                 \
+        SCM_ASSERT(data != NULL);                               \
+        return data->SCM_CPP_CAT(name,_proc);                   \
+    }
+
+#define VPORT_SET(name)                                                 \
     static void SCM_CPP_CAT3(vport_,name,_set) (ScmObj p, ScmObj v)     \
     {                                                                   \
-        vport *data = (vport*)SCM_PORT(p)->src.vt.data;                 \
+        vport *data = VPORT(p);                                         \
         SCM_ASSERT(data != NULL);                                       \
         data->SCM_CPP_CAT(name,_proc) = v;                              \
     }
 
+#define VPORT_ACC(name)   VPORT_GET(name) VPORT_SET(name)
+
 VPORT_ACC(getb)
 VPORT_ACC(getc)
 VPORT_ACC(gets)
@@ -429,7 +477,49 @@ VPORT_ACC(putc)
 VPORT_ACC(puts)
 VPORT_ACC(flush)
 VPORT_ACC(close)
-VPORT_ACC(seek)
+
+VPORT_GET(seek)
+VPORT_GET(getpos)
+VPORT_GET(setpos)
+
+static void vport_seek_set(ScmObj p, ScmObj v)
+{
+    vport *data = VPORT(p);
+    SCM_ASSERT(data != NULL);
+    data->seek_proc = v;
+    ScmPortVTable *vt = Scm_PortVTableStruct(SCM_PORT(p));
+    if (SCM_FALSEP(v)) {
+        vt->Seek = NULL;
+    } else {
+        vt->Seek = vport_seek;
+    }
+}
+
+static void vport_getpos_set(ScmObj p, ScmObj v)
+{
+    vport *data = VPORT(p);
+    SCM_ASSERT(data != NULL);
+    data->getpos_proc = v;
+    ScmPortVTable *vt = Scm_PortVTableStruct(SCM_PORT(p));
+    if (SCM_FALSEP(v)) {
+        vt->GetPos = NULL;
+    } else {
+        vt->GetPos = vport_getpos;
+    }
+}
+
+static void vport_setpos_set(ScmObj p, ScmObj v)
+{
+    vport *data = VPORT(p);
+    SCM_ASSERT(data != NULL);
+    data->setpos_proc = v;
+    ScmPortVTable *vt = Scm_PortVTableStruct(SCM_PORT(p));
+    if (SCM_FALSEP(v)) {
+        vt->SetPos = NULL;
+    } else {
+        vt->SetPos = vport_setpos;
+    }
+}
 
 #define VPORT_SLOT(name)                                \
     SCM_CLASS_SLOT_SPEC(#name,                          \
@@ -443,6 +533,8 @@ static ScmClassStaticSlotSpec viport_slots[] = {
     VPORT_SLOT(ready),
     VPORT_SLOT(close),
     VPORT_SLOT(seek),
+    VPORT_SLOT(getpos),
+    VPORT_SLOT(setpos),
     SCM_CLASS_SLOT_SPEC_END()
 };
 
@@ -453,10 +545,11 @@ static ScmClassStaticSlotSpec voport_slots[] = {
     VPORT_SLOT(flush),
     VPORT_SLOT(close),
     VPORT_SLOT(seek),
+    VPORT_SLOT(getpos),
+    VPORT_SLOT(setpos),
     SCM_CLASS_SLOT_SPEC_END()
 };
 
-#if 0
 static ScmClassStaticSlotSpec vioport_slots[] = {
     VPORT_SLOT(getb),
     VPORT_SLOT(getc),
@@ -468,9 +561,10 @@ static ScmClassStaticSlotSpec vioport_slots[] = {
     VPORT_SLOT(flush),
     VPORT_SLOT(close),
     VPORT_SLOT(seek),
+    VPORT_SLOT(getpos),
+    VPORT_SLOT(setpos),
     SCM_CLASS_SLOT_SPEC_END()
 };
-#endif
 
 /*================================================================
  * <buffered-port>
@@ -497,21 +591,25 @@ typedef struct bport_rec {
     ScmObj close_proc;          /* () -> () */
     ScmObj ready_proc;          /* () -> Bool */
     ScmObj filenum_proc;        /* () -> Maybe Int */
+    ScmObj getpos_proc;         /* () -> Maybe Int */
+    ScmObj setpos_proc;         /* (Int) ->  () */
     ScmObj seek_proc;           /* (Offset, Whence) -> Offset */
 } bport;
 
+#define BPORT(port)   ((bport*)PORT_BUF(port)->data)
+
 /*------------------------------------------------------------
  * Bport fill
  */
 static ScmSize bport_fill(ScmPort *p, ScmSize cnt)
 {
-    bport *data = (bport*)p->src.buf.data;
+    bport *data = BPORT(p);
     SCM_ASSERT(data != NULL);
     if (SCM_FALSEP(data->fill_proc)) {
         return 0;               /* indicates EOF */
     }
     ScmObj vec =
-        Scm_MakeU8VectorFromArrayShared(cnt, (unsigned char*)p->src.buf.buffer);
+        Scm_MakeU8VectorFromArrayShared(cnt, (u_char*)PORT_BUF(p)->buffer);
     ScmObj r = Scm_ApplyRec(data->fill_proc, SCM_LIST1(vec));
     if (SCM_INTP(r)) return SCM_INT_VALUE(r);
     else if (SCM_EOFP(r)) return 0;
@@ -523,13 +621,13 @@ static ScmSize bport_fill(ScmPort *p, ScmSize cnt)
  */
 static ScmSize bport_flush(ScmPort *p, ScmSize cnt, int forcep)
 {
-    bport *data = (bport*)p->src.buf.data;
+    bport *data = BPORT(p);
     SCM_ASSERT(data != NULL);
     if (SCM_FALSEP(data->flush_proc)) {
         return cnt;             /* blackhole */
     }
-    ScmObj vec = 
-        Scm_MakeU8VectorFromArrayShared(cnt, (unsigned char*)p->src.buf.buffer);
+    ScmObj vec =
+        Scm_MakeU8VectorFromArrayShared(cnt, (u_char*)PORT_BUF(p)->buffer);
     ScmObj r = Scm_ApplyRec(data->flush_proc,
                             SCM_LIST2(vec, SCM_MAKE_BOOL(forcep)));
     if (SCM_INTP(r)) return SCM_INT_VALUE(r);
@@ -542,7 +640,7 @@ static ScmSize bport_flush(ScmPort *p, ScmSize cnt, int forcep)
  */
 static void bport_close(ScmPort *p)
 {
-    bport *data = (bport*)p->src.buf.data;
+    bport *data = BPORT(p);
     SCM_ASSERT(data != NULL);
     if (!SCM_FALSEP(data->close_proc)) {
         Scm_ApplyRec(data->close_proc, SCM_NIL);
@@ -554,7 +652,7 @@ static void bport_close(ScmPort *p)
  */
 static int bport_ready(ScmPort *p)
 {
-    bport *data = (bport*)p->src.buf.data;
+    bport *data = BPORT(p);
     SCM_ASSERT(data != NULL);
 
     if (!SCM_FALSEP(data->ready_proc)) {
@@ -571,7 +669,7 @@ static int bport_ready(ScmPort *p)
  */
 static int bport_filenum(ScmPort *p)
 {
-    bport *data = (bport*)p->src.buf.data;
+    bport *data = BPORT(p);
     SCM_ASSERT(data != NULL);
 
     if (SCM_FALSEP(data->filenum_proc)) {
@@ -584,11 +682,31 @@ static int bport_filenum(ScmPort *p)
 }
 
 /*------------------------------------------------------------
- * Bport seek
+ * Bport positioning
  */
+static ScmObj bport_getpos(ScmPort *p)
+{
+    bport *data = BPORT(p);
+    SCM_ASSERT(data != NULL);
+    if (!SCM_FALSEP(data->getpos_proc)) {
+        return Scm_ApplyRec(data->getpos_proc, SCM_NIL);
+    } else {
+        return SCM_FALSE;
+    }
+}
+
+static void bport_setpos(ScmPort *p, ScmObj pos)
+{
+    bport *data = BPORT(p);
+    SCM_ASSERT(data != NULL);
+    if (!SCM_FALSEP(data->setpos_proc)) {
+        Scm_ApplyRec(data->setpos_proc, SCM_LIST1(pos));
+    }
+}
+
 static off_t bport_seek(ScmPort *p, off_t off, int whence)
 {
-    bport *data = (bport*)p->src.buf.data;
+    bport *data = BPORT(p);
     SCM_ASSERT(data != NULL);
     if (!SCM_FALSEP(data->seek_proc)) {
         ScmObj r = Scm_ApplyRec(data->seek_proc,
@@ -616,6 +734,8 @@ static ScmObj bport_allocate(ScmClass *klass, ScmObj initargs)
     data->close_proc = SCM_FALSE;
     data->ready_proc = SCM_FALSE;
     data->filenum_proc = SCM_FALSE;
+    data->getpos_proc = SCM_FALSE;
+    data->setpos_proc = SCM_FALSE;
     data->seek_proc  = SCM_FALSE;
 
     ScmPortBuffer buf;
@@ -635,8 +755,12 @@ static ScmObj bport_allocate(ScmClass *klass, ScmObj initargs)
     buf.closer  = bport_close;
     buf.ready   = bport_ready;
     buf.filenum = bport_filenum;
-    buf.seeker  = bport_seek;
     buf.data    = data;
+    /* these procedures will be set according to the slot values */
+    buf.seeker  = NULL;
+    buf.getpos  = NULL;
+    buf.setpos  = NULL;
+    buf.flags = 0;
 
     int dir = 0;
     if (Scm_SubtypeP(klass, SCM_CLASS_BUFFERED_INPUT_PORT)) {
@@ -646,32 +770,79 @@ static ScmObj bport_allocate(ScmClass *klass, ScmObj initargs)
     } else {
         Scm_Panic("bport_allocate: implementation error (class wiring screwed?)");
     }
-    ScmObj port = Scm_MakeBufferedPort(klass, SCM_FALSE, dir, TRUE, &buf);
-    SCM_PORT(port)->name = Scm_GetKeyword(key_name, initargs, SCM_FALSE);
+    ScmObj name = Scm_GetKeyword(key_name, initargs, SCM_FALSE);
+    ScmObj port = Scm_MakeBufferedPortFull(klass, name, dir, &buf,
+                                           (SCM_PORT_OWNER|SCM_PORT_WITH_POSITION));
     return port;
 }
 
 /* Accessors */
-#define BPORT_ACC(name)                                                 \
+#define BPORT_GET(name)                                                 \
     static ScmObj SCM_CPP_CAT3(bport_,name,_get) (ScmObj p)             \
     {                                                                   \
-        bport *data = (bport*)SCM_PORT(p)->src.buf.data;                \
+        bport *data = BPORT(p);                                         \
         SCM_ASSERT(data != NULL);                                       \
         return data->SCM_CPP_CAT(name,_proc);                           \
-    }                                                                   \
+    }
+
+#define BPORT_SET(name)                                                 \
     static void SCM_CPP_CAT3(bport_,name,_set) (ScmObj p, ScmObj v)     \
     {                                                                   \
-        bport *data = (bport*)SCM_PORT(p)->src.buf.data;                \
+        bport *data = BPORT(p);                                         \
         SCM_ASSERT(data != NULL);                                       \
         data->SCM_CPP_CAT(name,_proc) = v;                              \
     }
 
+#define BPORT_ACC(name)   BPORT_GET(name) BPORT_SET(name)
+
 BPORT_ACC(fill)
 BPORT_ACC(ready)
 BPORT_ACC(flush)
 BPORT_ACC(close)
 BPORT_ACC(filenum)
-BPORT_ACC(seek)
+
+BPORT_GET(seek)
+BPORT_GET(getpos)
+BPORT_GET(setpos)
+
+static void bport_seek_set(ScmObj p, ScmObj v)
+{
+    bport *data = BPORT(p);
+    SCM_ASSERT(data != NULL);
+    data->seek_proc = v;
+    ScmPortBuffer *b = Scm_PortBufferStruct(SCM_PORT(p));
+    if (SCM_FALSEP(v)) {
+        b->seeker = NULL;
+    } else {
+        b->seeker = bport_seek;
+    }
+}
+
+static void bport_getpos_set(ScmObj p, ScmObj v)
+{
+    bport *data = BPORT(p);
+    SCM_ASSERT(data != NULL);
+    data->getpos_proc = v;
+    ScmPortBuffer *b = Scm_PortBufferStruct(SCM_PORT(p));
+    if (SCM_FALSEP(v)) {
+        b->getpos = NULL;
+    } else {
+        b->getpos = bport_getpos;
+    }
+}
+
+static void bport_setpos_set(ScmObj p, ScmObj v)
+{
+    bport *data = BPORT(p);
+    SCM_ASSERT(data != NULL);
+    data->setpos_proc = v;
+    ScmPortBuffer *b = Scm_PortBufferStruct(SCM_PORT(p));
+    if (SCM_FALSEP(v)) {
+        b->setpos = NULL;
+    } else {
+        b->setpos = bport_setpos;
+    }
+}
 
 #define BPORT_SLOT(name)                                \
     SCM_CLASS_SLOT_SPEC(#name,                          \
@@ -684,6 +855,8 @@ static ScmClassStaticSlotSpec biport_slots[] = {
     BPORT_SLOT(close),
     BPORT_SLOT(filenum),
     BPORT_SLOT(seek),
+    BPORT_SLOT(getpos),
+    BPORT_SLOT(setpos),
     SCM_CLASS_SLOT_SPEC_END()
 };
 
@@ -692,6 +865,8 @@ static ScmClassStaticSlotSpec boport_slots[] = {
     BPORT_SLOT(close),
     BPORT_SLOT(filenum),
     BPORT_SLOT(seek),
+    BPORT_SLOT(getpos),
+    BPORT_SLOT(setpos),
     SCM_CLASS_SLOT_SPEC_END()
 };
 
@@ -708,6 +883,8 @@ SCM_EXTENSION_ENTRY void Scm_Init_gauche__vport(void)
                         "<virtual-input-port>", mod, viport_slots, 0);
     Scm_InitStaticClass(&Scm_VirtualOutputPortClass,
                         "<virtual-output-port>", mod, voport_slots, 0);
+    Scm_InitStaticClass(&Scm_VirtualIOPortClass,
+                        "<virtual-io-port>", mod, vioport_slots, 0);
     Scm_InitStaticClass(&Scm_BufferedInputPortClass,
                         "<buffered-input-port>", mod, biport_slots, 0);
     Scm_InitStaticClass(&Scm_BufferedOutputPortClass,
