@@ -1,23 +1,23 @@
 ;;;
 ;;; port.stk - stk compatibility interface
-;;;  
-;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -34,10 +34,9 @@
 #!no-fold-case
 
 (define-module compat.stk
-  (use srfi-1)
+  (use scheme.list)
+  (use scheme.charset)
   (use srfi-13)
-  (use srfi-14)
-  (use gauche.auxsys)
   (use gauche.sequence)
   (export *argc*
           copy-tree remq remv remove string->uninterned-symbol bignum?
@@ -72,7 +71,7 @@
           posix-rmdir posix-time posix-ctime posix-localtime
           posix-gmtime posix-mktime posix-tm->vector vector->posix-tm
           posix-strftime posix-fork posix-wait posix-uname
-          posix-host-name posix-domain-name 
+          posix-host-name posix-domain-name
           )
   )
 (select-module compat.stk)
@@ -85,7 +84,7 @@
 (define (remv item list)   (delete item list eqv?))
 (define (remove item list) (delete item list equal?))
 
-; string->uninterned-symbol 
+; string->uninterned-symbol
 ; bignum?
 
 (define (string-find sub str)
@@ -203,7 +202,7 @@
 (define posix-stat sys-stat)
 (define (posix-stat->vector stat)
   (apply vector
-         (map (lambda (p) (p stat))
+         (map (^p (p stat))
               (list sys-stat->dev
                     sys-stat->ino
                     sys-stat->mode
@@ -240,5 +239,3 @@
   (list->vector (sys-uname)))
 (define posix-host-name sys-gethostname)
 (define posix-domain-name sys-getdomainname)
-
-
