@@ -1,7 +1,7 @@
 ;;;
 ;;; collection.scm - collection generics
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,7 +35,6 @@
 ;; a set of objects, possibly containing infinite objects.
 
 (define-module gauche.collection
-  (use srfi-1)
   (export call-with-iterator with-iterator call-with-iterators
           call-with-builder  with-builder
           fold fold2 fold3 map map-to map-accum for-each
@@ -47,7 +46,23 @@
   )
 (select-module gauche.collection)
 
-;; utility - we can't depend on data.queue, so this is a simple
+;; Avoid hairy dependency issues
+(autoload gauche.uvector
+          u8vector-length
+          s8vector-length
+          u16vector-length
+          s16vector-length
+          u32vector-length
+          s32vector-length
+          u64vector-length
+          s64vector-length
+          f16vector-length
+          f32vector-length
+          f64vector-length
+          c32vector-length
+          c64vector-length
+          c128vector-length)
+
 ;; alternative.
 (define (make-queue)   (let1 anchor (list #f) (cons anchor anchor)))
 (define (enqueue! q x) (set! (cddr q) (list x)) (set! (cdr q) (cddr q)))
@@ -70,17 +85,45 @@
   (let1 p (if start (list-tail coll start) coll)
     (proc (cut null? p) (^[] (pop! p)))))
 
-(define-method call-with-iterator ((coll <vector>) proc
-                                   :key (start 0) :allow-other-keys)
-  (let1 len (vector-length coll)
-    (proc (cut >= start len)
-          (^[] (begin0 (vector-ref coll start) (inc! start))))))
+(define-syntax *vector-iter
+  (syntax-rules ()
+    [(_ %length %ref coll proc start)
+     (let1 len (%length coll)
+       (proc (cut >= start len)
+             (^[] (rlet1 v (%ref coll start) (inc! start)))))]))
 
-(define-method call-with-iterator ((coll <weak-vector>) proc
-                                   :key (start 0) :allow-other-keys)
-  (let1 len (weak-vector-length coll)
-    (proc (cut >= start len)
-          (^[] (begin0 (weak-vector-ref coll start) (inc! start))))))
+(define-syntax define-vector-iterator
+  (er-macro-transformer
+   (^[f r c]
+     (let* ([type (cadr f)]
+            [%class (r (symbol-append '< type '>))]
+            [%length (r (symbol-append type '-length))]
+            [%ref    (r (if (c (r type) (r'bitvector))
+                          'bitvector-ref/int
+                          (symbol-append type '-ref)))])
+       (quasirename r
+         `(define-method call-with-iterator ((coll ,%class) proc
+                                             ,':key (start 0)
+                                             ,':allow-other-keys)
+            (*vector-iter ,%length ,%ref coll proc start)))))))
+
+(define-vector-iterator vector)
+(define-vector-iterator u8vector)
+(define-vector-iterator s8vector)
+(define-vector-iterator u16vector)
+(define-vector-iterator s16vector)
+(define-vector-iterator u32vector)
+(define-vector-iterator s32vector)
+(define-vector-iterator u64vector)
+(define-vector-iterator s64vector)
+(define-vector-iterator f16vector)
+(define-vector-iterator f32vector)
+(define-vector-iterator f64vector)
+(define-vector-iterator c32vector)
+(define-vector-iterator c64vector)
+(define-vector-iterator c128vector)
+(define-vector-iterator bitvector)
+(define-vector-iterator weak-vector)
 
 (define-method call-with-iterator ((coll <string>) proc
                                    :key (start #f) :allow-other-keys)
@@ -141,33 +184,57 @@
 
 (define-method call-with-builder ((class <vector-meta>) proc
                                   :key (size #f) :allow-other-keys)
-  (if size
-    (let ([v (make-vector size)]
-          [i 0])
-      (proc (^[item] (when (< i size)
-                       (vector-set! v i item)
-                       (inc! i)))
-            (^[] v)))
-    (let1 q (make-queue)
-      (proc (cut enqueue! q <>)
-            (cut list->vector (queue->list q))))))
+  (let1 size (force size)
+    (if size
+      (let ([v (make-vector size)]
+            [i 0])
+        (proc (^[item] (when (< i size)
+                         (vector-set! v i item)
+                         (inc! i)))
+              (^[] v)))
+      (let1 q (make-queue)
+        (proc (cut enqueue! q <>)
+              (cut list->vector (queue->list q)))))))
+
+(define-syntax *vector-builder
+  (syntax-rules ()
+    [(_ %make %set! %list-> class proc size)
+     (let1 size (force size)
+       (if size
+         (let ([v (%make size)]
+               [i 0])
+           (proc (^[item] (when (< i size)
+                            (%set! v i item)
+                            (inc! i)))
+                 (^[] v)))
+         (let1 q (make-queue)
+           (proc (cut enqueue! q <>)
+                 (cut %list-> (queue->list q))))))]))
+
+;; NB: We don't have list->uvector in core yet, so builder definition is
+;; in gauche.uvector.  It should be here, though.
+
+(define-method call-with-builder ((class <vector-meta>) proc
+                                  :key (size #f) :allow-other-keys)
+  (*vector-builder make-vector vector-set! list->vector class proc size))
+
+(define-method call-with-builder ((class <bitvector-meta>) proc
+                                  :key (size #f) :allow-other-keys)
+  (*vector-builder make-bitvector bitvector-set! list->bitvector
+                   class proc size))
 
 (define-method call-with-builder ((class <weak-vector-meta>) proc
                                   :key (size #f) :allow-other-keys)
-  (if size
-    (let ([v (make-weak-vector size)]
-          [i 0])
-      (proc (^[item] (when (< i size)
-                       (weak-vector-set! v i item)
-                       (inc! i)))
-            (^[] v)))
-    (let ([q (make-queue)]
-          [cnt 0])
-      (proc (^[item] (enqueue! q item) (inc! cnt))
-            (^[] (let1 v (make-weak-vector cnt)
-                   (do ([i 0 (+ i 1)])
-                       [(= i cnt) v]
-                     (weak-vector-set! v i (dequeue! q)))))))))
+  (*vector-builder make-weak-vector weak-vector-set! %list->weak-vector
+                   class proc size))
+
+(define (%list->weak-vector lis)
+  (rlet1 v (make-weak-vector (length lis))
+    ;; for-each-with-index is in gauche.sequence, so we roll on our own
+    (do ([lis lis (cdr lis)]
+         [i   0   (+ i 1)])
+        [(null? lis)]
+      (weak-vector-set! v i (car lis)))))
 
 (define-method call-with-builder ((class <string-meta>) proc :allow-other-keys)
   (let1 s (open-output-string)
@@ -209,6 +276,8 @@
               ((with-module gauche.internal %char-set-add-chars!) cs (list c)))
           (^[] cs))))
 
+
+
 ;; utility.  return minimum size of collections if it's easily known, or #f.
 (define (maybe-minimum-size col more)
   (let1 size (and-let* ([siz (lazy-size-of col)]
@@ -269,15 +338,13 @@
 (define-fold-k fold3 (knil1 knil2 knil3))
 
 ;; partial applied version
-(define-method fold$ (proc)
-  (^[knil . lists] (apply fold proc knil lists)))
-(define-method fold$ (proc knil)
-  (^ lists (apply fold proc knil lists)))
-
-(define-method fold2$ (proc knil1 knil2)
+(define fold$
+  (case-lambda
+    ([proc] (^[knil . lists] (apply fold proc knil lists)))
+    ([proc knil] (^ lists (apply fold proc knil lists)))))
+(define (fold2$ proc knil1 knil2)
   (^ lists (apply fold2 proc knil1 knil2 lists)))
-
-(define-method fold3$ (proc knil1 knil2 knil3)
+(define (fold3$ proc knil1 knil2 knil3)
   (^ lists (apply fold3 proc knil1 knil2 knil3 lists)))
 
 ;; map --------------------------------------------------
@@ -309,7 +376,7 @@
         (next-method)))))
 
 ;; redefine map$ to use generic version of map
-(define (map$ proc) (pa$ map proc))
+(define ((map$ proc) . args) (apply map proc args))
 
 ;; map-to -----------------------------------------------
 
@@ -383,7 +450,7 @@
             (next-method)))))
 
 ;; redefine for-each$ to use generic version of for-each
-(define (for-each$ proc) (pa$ for-each proc))
+(define ((for-each$ proc) . args) (apply for-each proc args))
 
 ;; size-of ----------------------------------------------
 
@@ -395,16 +462,24 @@
   (delay (size-of coll)))
 
 ;; shortcut
+;; NB: For uvectors, gauche.uvector defines more specific method for each
+;; concrete uvector types.  This is a fallback when the user only
+;; loads gauche.collection.
 (define-method size-of ((coll <list>))        (length coll))
 (define-method size-of ((coll <vector>))      (vector-length coll))
 (define-method size-of ((coll <weak-vector>)) (weak-vector-length coll))
 (define-method size-of ((coll <string>))      (string-length coll))
 (define-method size-of ((coll <char-set>))    (char-set-size coll))
+(define-method size-of ((coll <uvector>))     (uvector-length coll))
+(define-method size-of ((coll <bitvector>))   (bitvector-length coll))
 
-(define-method lazy-size-of ((coll <list>))        (length coll))
+(define-method lazy-size-of ((coll <list>))        (delay (length coll)))
 (define-method lazy-size-of ((coll <vector>))      (vector-length coll))
 (define-method lazy-size-of ((coll <weak-vector>)) (weak-vector-length coll))
 (define-method lazy-size-of ((coll <string>))      (string-length coll))
+(define-method lazy-size-of ((coll <char-set>))    (char-set-size coll))
+(define-method lazy-size-of ((coll <uvector>))     (uvector-length coll))
+(define-method lazy-size-of ((coll <bitvector>))   (bitvector-length coll))
 
 ;; find -------------------------------------------------
 
@@ -419,7 +494,7 @@
 
 ;; shortcut
 (define-method find (pred (coll <list>))
-  ((with-module srfi-1 find) pred coll))
+  ((with-module gauche find) pred coll))
 
 ;; find-min, find-max, find-min&max ---------------------
 
@@ -490,7 +565,7 @@
 
 ;; shortcut
 (define-method filter (pred (coll <list>))
-  ((with-module srfi-1 filter) pred coll))
+  ((with-module gauche filter) pred coll))
 
 (define-method filter-to ((class <list-meta>) pred coll)
   (filter pred coll))
@@ -513,7 +588,7 @@
 
 ;; shortcut
 (define-method remove (pred (coll <list>))
-  ((with-module srfi-1 remove) pred coll))
+  ((with-module gauche remove) pred coll))
 
 (define-method remove-to ((class <list-meta>) pred coll)
   (remove pred coll))
@@ -542,7 +617,7 @@
 
 ;; shortcut
 (define-method partition (pred (coll <list>))
-  ((with-module srfi-1 partition) pred coll))
+  ((with-module gauche partition) pred coll))
 
 (define-method partition-to ((class <list-meta>) pred coll)
   (partition pred coll))
@@ -584,4 +659,3 @@
                         [else (cons (list key elt) buckets)])))
               '()
               col)))
-
