@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.generator - generator framework
 ;;;
-;;;   Copyright (c) 2011-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2011-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,7 +32,6 @@
 ;;;
 
 (define-module gauche.generator
-  (use srfi-1)
   (use gauche.sequence)
   (use gauche.partcont)
   (export list->generator vector->generator reverse-vector->generator
@@ -54,7 +53,8 @@
 
           null-generator gcons* gappend gflatten
           gconcatenate gmerge ggroup
-          circular-generator gunfold giota grange gindex gselect ginterval
+          circular-generator gunfold giterate giterate1
+          giota grange gindex gselect ginterval
           gmap gmap-accum gfilter gremove gdelete gdelete-neighbor-dups
           gfilter-map gstate-filter gbuffer-filter
           gtake gtake* gdrop gtake-while gdrop-while grxmatch gslices
@@ -231,6 +231,22 @@
                 (tail)]
                [else (%begin0 (f seed) (set! seed (g seed)))]))))
 
+;; special case of gunfold, for speed and convenience.
+;;  giterate f seed == gunfold (^_ #f) identity f seed
+;; basically, it generates [seed, (f seed), (f (f seed)), ...]
+;; cf. stream-iterate (scheme.stream)
+(define (giterate f seed)
+  (let ([seed seed]
+        [first? #t])
+    (^[] (if first?
+           (begin (set! first? #f) seed)
+           (begin (set! seed (f seed)) seed)))))
+
+;; variation of giterate, where seed itself is not included.
+;; this is ~10% faster than giterate, mainly because this one is so simple.
+(define (giterate1 f seed)
+  (^[] (set! seed (f seed)) seed))
+
 (define (giota :optional (count +inf.0) (start 0) (step 1))
   (if (and (exact? start) (exact? step))
     (let1 val start
@@ -270,7 +286,7 @@
                (if (<= v end)
                  (eof-object)
                  (begin (inc! k) v))))))))
-        
+
 
 ;;;
 ;;; Generator operations
@@ -654,10 +670,9 @@
 ;;; Consumers
 ;;;
 
-;; NB: The following consumers are in autoloaded lib/gauche/procedure.scm
-;; for historical reasons:
+;; NB: The following consumers are built-in (libproc.scm)
 ;; generator-fold generator-fold-right
-;; genergenerator-for-each generator-map generator-find 
+;; genergenerator-for-each generator-map generator-find
 
 (define (%generator->list gen reverse? n)
   (if (integer? n)
@@ -719,7 +734,7 @@
 (define (generator->bytevector! vec at gen)
   (check-arg u8vector? vec)
   (generator->uvector! vec at gen))
-  
+
 (define (generator->string gen :optional (n #f))
   (with-output-to-string
     (^[]
@@ -753,7 +768,7 @@
 (define (generator-count pred gen)
   (rlet1 n 0
     (do-generator [v gen] (when (pred v) (inc! n)))))
-  
+
 (define (generator-unfold gen unfold . args)
   (apply unfold eof-object? identity (^_ (gen)) (gen) args))
 
