@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.record - record implementation
 ;;;
-;;;   Copyright (c) 2010-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2010-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -72,7 +72,6 @@
 ;;     the caller is a foreign entity.
 
 ;; some convenience utility
-
 (define-macro (sym+ . args)
   `((with-module gauche.internal identifier-append) ,@args))
 
@@ -90,6 +89,8 @@
 ;;; Infrastructure
 ;;;
 
+;; All record class must be an instance of <record-meta>.
+
 (define-class <record-meta> (<class>)
   ((field-specs :init-keyword :field-specs :init-value '#())
    ;; - A vector of field specs, as specified in the fieldspecs argument
@@ -108,7 +109,8 @@
 
 (for-each-subst
  (list vector u8vector s8vector u16vector s16vector u32vector s32vector
-       u64vector s64vector f16vector f32vector f64vector)
+       u64vector s64vector f16vector f32vector f64vector
+       c32vector c64vector c128vector)
  (define-class <??-pseudo-record-meta> (<pseudo-record-meta>) ())
  (define-class <??-pseudo-record> () ()
    :metaclass <??-pseudo-record-meta>)
@@ -129,7 +131,8 @@
       (error "pseudo-rtd requires a class object <vector>, <list>, \
               <u8vector>, <s8vector>, <u16vector>, <s16vector>, \
               <u32vector>, <s32vector>, <u64vector>, <s64vector>, \
-              <f16vector>, <f32vector>, <f64vector>, \
+              <f16vector>, <f32vector>, <f64vector>, <c32vector>, \
+              <c64vector>, <c128vector>, \
               or other pseudo-rtd, but got" class)))
 
 (define-method initialize ((class <record-meta>) initargs)
@@ -171,15 +174,14 @@
       s)))
 
 (define (%valid-fieldspecs? specs signal-error?)
-  (define (id? x) (or (symbol? x) (identifier? x)))
   (if (not (vector? specs))
     (if signal-error?
       (error "make-rtd: fieldspecs must be a vector, but got" specs)
       #f)
     (every (^[spec]
              (match spec
-               [((or 'mutable 'immutable) (? id? name)) #t]
-               [(? id? name) #t]
+               [((or 'mutable 'immutable) (? identifier? name)) #t]
+               [(? identifier? name) #t]
                [other (if signal-error?
                         (error "make-rtd: invalid field spec:" other)
                         #f)]))
@@ -259,15 +261,39 @@
 ;;; Procedural layer
 ;;;
 
-(define (make-rtd name fieldspecs :optional (parent #f) :rest opts)
-  (make (if parent (class-of parent) <record-meta>)
-    :name name :field-specs fieldspecs :metaclass <record-meta>
-    :supers (list (or parent <record>))
+;; NB: If record is created procedurally, we don't know what to set
+;; in :defined-module.
+(define (make-rtd name fieldspecs :optional (parent #f)
+                  :key (mixins '()) (metaclass #f) (module #f))
+  ;; We only allow to inherit either other record class, or classes that
+  ;; don't add slots.  The metaclass be alwayas <record-meta>
+  (when parent
+    (unless (is-a? parent <record-meta>)
+      (error "Inheriting from non-record class is prohibited:" parent
+             (class-of parent) (is-a? parent <record-meta>))))
+  (when (list? mixins)
+    (dolist [m mixins]
+      (unless (null? (class-slots m))
+        (error "Cannot use a class with slots as a record mixin:" m))))
+  (when metaclass
+    (unless (subtype? metaclass <record-meta>)
+      (error "Cannot use a metaclass that isn't a subtype of <record-meta>:"
+             metaclass)))
+  (make (or metaclass
+            (and parent (class-of parent))
+            <record-meta>)
+    :name name :field-specs fieldspecs
+    :supers (append mixins
+                    (cond [(is-a? parent <record-meta>) (list parent)]
+                          [else (list <record>)]))
     :slots ($ fieldspecs->slotspecs fieldspecs
-              $ if parent (length (class-slots parent)) 0)))
+              $ if parent (length (class-slots parent)) 0)
+    :defined-modules (if module (list module) '())))
 
 (define (rtd? obj) (is-a? obj <record-meta>))
 
+;; The following several macros are only used locally, so legacy macros are ok.
+
 ;; For conciseness.  We need to use macros (for now) to ensure Gauche compiler
 ;; optimize the gref away---a kludge not recommended in general.
 (define-macro (%make)  '(with-module gauche.object %make-record))
@@ -287,7 +313,7 @@
         [else (lambda (,@(cdr tmps) . ,(car tmps))
                 ,,rest-maker)])))
 
- (define-macro (define-ctor-generators ctor-name-base make1 make* makev)
+(define-macro (define-ctor-generators ctor-name-base make1 make* makev)
   (define default-name (sym+ ctor-name-base '-default))
   (define custom-name  (sym+ ctor-name-base '-custom))
   `(begin
@@ -319,7 +345,8 @@
 
 (for-each-subst
  (list vector u8vector s8vector u16vector s16vector u32vector s32vector
-       u64vector s64vector f16vector f32vector f64vector)
+       u64vector s64vector f16vector f32vector f64vector
+       c32vector c64vector c128vector)
  (define-ctor-generators %??-ctor
    `(?? ,@vars)
    `(apply ?? ,@(cdr tmps) ,(car tmps))
@@ -376,7 +403,7 @@
            (errorf "slot ~a of record ~s is immutable" field rtd))
          (^(o v) (,@mutator* o k v))))
      ))
-  
+
 (define-rtd-methods <record-meta>
   %record-ctor
   ((with-module gauche.object %record-ref) rtd)
@@ -384,7 +411,8 @@
 
 (for-each-subst
  (list vector u8vector s8vector u16vector s16vector u32vector s32vector
-       u64vector s64vector f16vector f32vector f64vector)
+       u64vector s64vector f16vector f32vector f64vector
+       c32vector c64vector c128vector)
  (define-rtd-methods <??-pseudo-record-meta>
    %??-ctor
    (??-ref)
@@ -412,73 +440,102 @@
                    (find (^s (= num (get-keyword :index (cdr s))))
                          (class-slots class)))))))
 
+;; Manually seal infrastructure
+;; (Hopefully this will help inlining constructors and accessors in future;
+;;  for now, this has no effect.)
+(for-each (with-module gauche.object generic-seal!)
+          (list %rtd-predicate %rtd-constructor %rtd-accessor %rtd-mutator))
+
+
 ;;;
 ;;; Syntactic layer
 ;;;
 
-;; TODO: Rewriter this w/ er-macro after 0.9.6 release
-(define-macro (define-record-type type-spec ctor-spec pred-spec . field-specs)
-  (define (->id x) ((with-module gauche.internal make-identifier) x
-                    (find-module 'gauche.record) '()))
-  (define (id? x)  (or (symbol? x) (identifier? x)))
-  (define %make (->id 'make-rtd))
-  (define %ctor (->id 'rtd-constructor))
-  (define %pred (->id 'rtd-predicate))
-  (define %asor (->id 'rtd-accessor))
-  (define %mtor (->id 'rtd-mutator))
-  (define %define-inline (->id 'define-inline))
-  (define tmp   (gensym))
-  (define (build-field-spec)
-    (map-to <vector> (match-lambda
-                       [((? id? f) a s) f]
-                       [((? id? f) a) `(immutable ,f)]
-                       [((? id? f)) f]
-                       [(? id? f) `(immutable ,f)]
-                       [x (error "invalid field spec:" x)])
+(define-syntax define-record-type
+  (er-macro-transformer
+   (^[f r c]
+     (define (parse-type-spec type-spec)
+       (match type-spec
+         [(? identifier? name) (values name #f '())]
+         [((? identifier? name) parent) (values name parent '())]
+         [((? identifier? name) parent . opts)
+          (let-keywords opts ([mixins #f]
+                              [metaclass #f])
+            (values name parent
+                    (cond-list [mixins @ (quasirename r
+                                           `(:mixins (list ,@mixins)))]
+                               [metaclass @ (quasirename r
+                                              `(:metaclass ,metaclass))])))]
+         [_ (error "invalid type-spec" type-spec)]))
+     (define (build-field-spec field-specs)
+       (map-to <vector> (match-lambda
+                          [((? identifier? f) a s) f]
+                          [((? identifier? f) a) `(immutable ,f)]
+                          [((? identifier? f)) f]
+                          [(? identifier? f) `(immutable ,f)]
+                          [x (error "invalid field spec:" x)])
+               field-specs))
+     (define (build-def typename parent opts field-specs)
+       (quasirename r
+         `(define-inline ,typename
+            (make-rtd ',typename ',(build-field-spec field-specs) ,parent
+                      ':module (current-module)
+                      ,@opts))))
+     (define (build-ctor typename ctor-spec)
+       (match ctor-spec
+         [#f '()]
+         [#t (quasirename r
+               `((define-inline ,(sym+ 'make- typename)
+                   (rtd-constructor ,typename))))]
+         [((? identifier? ctor-name) field ...)
+          (quasirename r
+            `((define-inline ,ctor-name
+                (rtd-constructor ,typename ',(list->vector field)))))]
+         [(? identifier? ctor-name)
+          (quasirename r
+            `((define-inline ,ctor-name (rtd-constructor ,typename))))]
+         [x (error "invalid constructor spec" ctor-spec)]))
+     (define (build-pred typename pred-spec)
+       (match pred-spec
+         [#f '()]
+         [#t (quasirename r
+               `((define-inline ,(sym+ typename '?)
+                   (rtd-predicate ,typename))))]
+         [(? identifier? pred-name)
+          (quasirename r
+            `((define-inline ,pred-name (rtd-predicate ,typename))))]
+         [_ (error "invalid predicate spec" pred-spec)]))
+     (define (build-accessors typename field-specs)
+       (map (match-lambda
+              [(f a . _) (quasirename r
+                           `(define-inline ,a (rtd-accessor ,typename ',f)))]
+              [(f)       (quasirename r
+                           `(define-inline ,(sym+ typename '- f)
+                              (rtd-accessor ,typename ',f)))]
+              [f         (quasirename r
+                           `(define-inline ,(sym+ typename '- f)
+                              (rtd-accessor ,typename ',f)))])
             field-specs))
-  (define (build-def typename parent)
-    `(,%define-inline ,typename
-       (,%make ',typename ',(build-field-spec) ,@(if parent `(,parent) '()))))
-  (define (build-ctor typename)
-    (match ctor-spec
-      [#f '()]
-      [#t `((,%define-inline ,(sym+ 'make- typename) (,%ctor ,typename)))]
-      [((? id? ctor-name) field ...)
-       `((,%define-inline ,ctor-name (,%ctor ,typename ',(list->vector field))))]
-      [(? id? ctor-name)
-       `((,%define-inline ,ctor-name (,%ctor ,typename)))]
-      [x (error "invalid constructor spec" ctor-spec)]))
-  (define (build-pred typename)
-    (match pred-spec
-      [#f '()]
-      [#t `((,%define-inline (,(sym+ typename '?) ,tmp)
-              ((,%pred ,typename) ,tmp)))]
-      [(? id? pred-name)
-       `((,%define-inline (,pred-name ,tmp) ((,%pred ,typename) ,tmp)))]
-      [x (error "invalid predicate spec" pred-spec)]))
-  (define (build-accessors typename)
-    (map (match-lambda
-           [(f a . _) `(,%define-inline ,a (,%asor ,typename ',f))]
-           [(f)       `(,%define-inline ,(sym+ typename '- f)
-                         (,%asor ,typename ',f))]
-           [f         `(,%define-inline ,(sym+ typename '- f)
-                         (,%asor ,typename ',f))])
-         field-specs))
-  (define (build-mutators typename)
-    (append-map (match-lambda
-                  [(f a m) `((,%define-inline ,m (,%mtor ,typename ',f)))]
-                  [(f)     `((,%define-inline ,(sym+ typename '- f '-set!)
-                               (,%mtor ,typename ',f)))]
-                  [_ '()])
-                field-specs))
+     (define (build-mutators typename field-specs)
+       (append-map
+        (match-lambda
+          [(f a m) (quasirename r
+                     `((define-inline ,m
+                         (rtd-mutator ,typename ',f))))]
+          [(f)     (quasirename r
+                     `((define-inline ,(sym+ typename '- f '-set!)
+                         (rtd-mutator ,typename ',f))))]
+          [_ '()])
+        field-specs))
 
-  (receive (typename parent) (match type-spec
-                               [((? id? name) parent) (values name parent)]
-                               [(? id? name) (values name #f)]
-                               [x (error "invalid type-spec" type-spec)])
-    `(begin ,(build-def typename parent)
-            ,@(build-ctor typename)
-            ,@(build-pred typename)
-            ,@(build-accessors typename)
-            ,@(build-mutators typename)))
-    )
+     (match f
+       [(_ type-spec ctor-spec pred-spec . field-specs)
+        (receive (typename parent opts)
+            (parse-type-spec type-spec)
+          (quasirename r
+            `(begin ,(build-def typename parent opts field-specs)
+                    ,@(build-ctor typename ctor-spec)
+                    ,@(build-pred typename pred-spec)
+                    ,@(build-accessors typename field-specs)
+                    ,@(build-mutators typename field-specs))))]
+       [_ (error "malformed define-record-type" f)]))))
