@@ -1,7 +1,7 @@
 ;;;
 ;;; sequence.scm - sequence operations
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,9 +36,9 @@
 ;; See also gauche.collection, that defines various mapping functions.
 
 (define-module gauche.sequence
-  (use srfi-1)
   (extend gauche.collection)
   (export referencer modifier subseq
+          call-with-reverse-iterator call-with-reverse-iterators
           fold-right
           fold-with-index map-with-index map-to-with-index for-each-with-index
           find-index find-with-index group-sequence group-contiguous-sequence
@@ -55,8 +55,12 @@
 ;; used by shuffle
 (autoload srfi-27 default-random-source random-source-make-integers)
 
+(autoload srfi-152 string-fold-right)
+(autoload scheme.vector vector-fold-right)
+
 (define-method referencer ((obj <list>))   list-ref)
 (define-method referencer ((obj <vector>)) vector-ref)
+(define-method referencer ((obj <bitvector>)) bitvector-ref/int)
 (define-method referencer ((obj <weak-vector>)) weak-vector-ref)
 (define-method referencer ((obj <string>)) string-ref)
 
@@ -76,6 +80,7 @@
 
 (define-method modifier ((obj <list>)) list-set!)
 (define-method modifier ((obj <vector>)) vector-set!)
+(define-method modifier ((obj <bitvector>)) bitvector-set!)
 (define-method modifier ((obj <weak-vector>)) weak-vector-set!)
 (define-method modifier ((obj <string>)) string-set!)
 
@@ -102,7 +107,6 @@
   (subseq seq start (size-of seq)))
 
 (define-method subseq ((seq <sequence>) start end)
-  (when (< end 0) (set! end (modulo end (size-of seq))))
   (when (> start end)
     (errorf "start ~a must be smaller than or equal to end ~a" start end))
   (let1 size (- end start)
@@ -110,6 +114,11 @@
       (with-iterator (seq end? next :start start)
         (dotimes [i size (get)] (add! (next)))))))
 
+(define-method subseq ((seq <vector>) . args)
+  (apply vector-copy seq args))
+(define-method subseq ((seq <bitvector>) . args)
+  (apply bitvector-copy seq args))
+
 (define-method (setter subseq) ((seq <sequence>) start vals)
   (with-iterator (vals end? next)
     (do ([index start (+ index 1)])
@@ -123,26 +132,124 @@
       (when (end?) (error "not enough values for (setter subseq)" vals))
       (set! (ref seq index) (next)))))
 
+(define-method (setter subseq) ((seq <vector>) start end (vals <vector>))
+  (vector-copy! seq start vals 0 (- end start)))
+(define-method (setter subseq) ((seq <bitvector>) start end (vals <bitvector>))
+  (bitvector-copy! seq start vals 0 (- end start)))
+
+;; reverse iterator ------------------------------------
+
+(define-method call-with-reverse-iterator ((str <string>) proc
+                                           :key
+                                           (start (string-cursor-start str))
+                                           (end (string-cursor-end str))
+                                           :allow-other-keys)
+  (let ((start (string-index->cursor str start))
+        (cur (string-index->cursor str end)))
+    (proc (cut string-cursor<=? cur start)
+          (^[]
+            (set! cur (string-cursor-prev str cur))
+            (string-ref str cur)))))
+
+(define-syntax *vector-reverse-iter
+  (syntax-rules ()
+    [(_ %ref coll proc start end)
+     (proc (cut <= end start)
+           (^[] (%ref coll (dec! end))))]))
+
+(define-syntax define-vector-reverse-iterator
+  (er-macro-transformer
+   (^[f r c]
+     (let* ([type (cadr f)]
+            [%class (r (symbol-append '< type '>))]
+            [%length (r (symbol-append type '-length))]
+            [%ref    (r (if (c (r type) (r'bitvector))
+                          'bitvector-ref/int
+                          (symbol-append type '-ref)))])
+       (quasirename r
+         `(define-method call-with-reverse-iterator ((coll ,%class) proc
+                                                     ,':key
+                                                     (start 0)
+                                                     (end (,%length coll))
+                                                     ,':allow-other-keys)
+            (*vector-reverse-iter ,%ref coll proc start end)))))))
+
+(define-vector-reverse-iterator vector)
+(define-vector-reverse-iterator u8vector)
+(define-vector-reverse-iterator s8vector)
+(define-vector-reverse-iterator u16vector)
+(define-vector-reverse-iterator s16vector)
+(define-vector-reverse-iterator u32vector)
+(define-vector-reverse-iterator s32vector)
+(define-vector-reverse-iterator u64vector)
+(define-vector-reverse-iterator s64vector)
+(define-vector-reverse-iterator f16vector)
+(define-vector-reverse-iterator f32vector)
+(define-vector-reverse-iterator f64vector)
+(define-vector-reverse-iterator c32vector)
+(define-vector-reverse-iterator c64vector)
+(define-vector-reverse-iterator c128vector)
+(define-vector-reverse-iterator bitvector)
+(define-vector-reverse-iterator weak-vector)
+
+(define-method call-with-reverse-iterator ((coll <tree-map>) proc :allow-other-keys)
+  (let ([eof-marker (cons #f #f)]
+        [iter ((with-module gauche.internal %tree-map-iter) coll)])
+    (receive (k v) (iter eof-marker #t)
+      (proc (cut eq? k eof-marker)
+            (^[] (begin0 (cons k v)
+                   (set!-values (k v) (iter eof-marker #t))))))))
+
+(define (call-with-reverse-iterators colls proc)
+  (let loop ([colls colls]
+             [eprocs '()]
+             [nprocs '()])
+    (if (null? colls)
+      (proc (reverse! eprocs) (reverse! nprocs))
+      (call-with-reverse-iterator
+       (car colls)
+       (^[end? next]
+         (loop (cdr colls) (cons end? eprocs) (cons next nprocs)))))))
+
 ;; fold-right --------------------------------------------
 
 ;;  (proc e1 (proc e2 ... (proc eN seed)))
 
 (define-method fold-right (proc seed (seq <sequence>) . more)
   (if (null? more)
-    (with-iterator (seq end? next)
-      (let rec ()
-        (if (end?)
-          seed
-          (let1 elt (next)
-            (proc elt (rec))))))
-    (call-with-iterators
-     (cons seq more)
-     (^[ends? nexts]
-       (let rec ()
-         (if (any (cut <>) ends?)
-           seed
-           (let1 elts (map (cut <>) nexts)
-             (apply proc (append! elts (list (rec)))))))))))
+    (if (applicable? call-with-reverse-iterator (class-of seq) <bottom>)
+      (call-with-reverse-iterator
+       seq
+       (^[end? next]
+         (let loop ([acc seed])
+           (if (end?)
+             acc
+             (loop (proc (next) acc))))))
+      (with-iterator (seq end? next)
+        (let rec ()
+          (if (end?)
+            seed
+            (let1 elt (next)
+              (proc elt (rec)))))))
+    (let1 seqs (cons seq more)
+      (if (every (^ (seq)
+                   (applicable? call-with-reverse-iterator (class-of seq) <bottom>))
+                 seqs)
+        (call-with-reverse-iterators
+         seqs
+         (^[ends? nexts]
+           (let loop ([acc seed])
+             (if (any (cut <>) ends?)
+               acc
+               (loop (apply proc (append! (map (cut <>) nexts) (list acc))))))))
+        (call-with-iterators
+            seqs
+          (^[ends? nexts]
+            (let rec ()
+              (if (any (cut <>) ends?)
+                seed
+                (let1 elts (map (cut <>) nexts)
+                  (apply proc (append! elts (list (rec)))))))))))))
 
 ;; for list arguments, built-in fold-right is faster.
 (define-method fold-right (proc seed (seq <list>))
@@ -151,6 +258,16 @@
 (define-method fold-right (proc seed (seq1 <list>) (seq2 <list>))
   ((with-module gauche fold-right) proc seed seq1 seq2))
 
+;; shortcut
+(define-method fold-right (proc seed (seq <string>))
+  (string-fold-right proc seed seq))
+
+(define-method fold-right (proc seed (seq <vector>))
+  (vector-fold-right (^ (r e) (proc e r)) seed seq))
+
+(define-method fold-right (proc seed (seq1 <vector>) (seq2 <vector>))
+  (vector-fold-right (^ (r e1 e2) (proc e1 e2 r)) seed seq1 seq2))
+
 ;; mapping with index ------------------------------------
 
 (define-method fold-with-index (proc knil (seq <sequence>) . more)
@@ -335,7 +452,7 @@
             (reverse! results)
             (reverse! `((,start ,last) ,@(cdr results)))))
         (reverse! (map reverse! results))))))
-                                          
+
 (define-method delete-neighbor-dups ((seq <sequence>)
                                      :key ((:key key-proc) identity)
                                           ((:test test-proc) eqv?)
@@ -385,7 +502,7 @@
 ;; NB: We can't define generic version, since there's no generic way
 ;; for length-changing mutation.  Each capable sequence should implement
 ;; the method.  Here we only provide for <list>.
-(define-method delete-neighbor-dups-squeeze! ((seq <list>)            
+(define-method delete-neighbor-dups-squeeze! ((seq <list>)
                                               :key ((:key key-proc) identity)
                                                    ((:test test-proc) eqv?)
                                                    (start 0)
@@ -586,4 +703,3 @@
   ;; string-set! is super-slow, so we provide the alternative.
   ((with-module gauche.internal %string-replace-body!)
    seq (apply shuffle seq args)))
-
