@@ -1,7 +1,7 @@
 ;;;
 ;;; mime.scm - parsing MIME (rfc2045) message
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,7 +44,7 @@
 
 (define-module rfc.mime
   (use srfi-13)
-  (use srfi-14)
+  (use scheme.charset)
   (use rfc.822)
   (use util.match)
   (export mime-parse-version
@@ -570,4 +570,3 @@
    [(string-ci=? transfer-enc "quoted-printable") (quoted-printable-encode)]
    [else (error "Unsupported transfer encoding encountered while composing \
                  mime message: " transfer-enc)]))
-
