@@ -1,7 +1,7 @@
 /*
  * bignum.c - multiple precision exact integer arithmetic
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -83,7 +83,7 @@ char *alloca ();
 #include "gauche/priv/arith.h"
 #include "gauche/bits.h"
 #include "gauche/bits_inline.h"
-#include "gauche/bignum.h"
+#include "gauche/priv/bignumP.h"
 
 #undef min
 #define min(x, y)   (((x) < (y))? (x) : (y))
@@ -309,7 +309,7 @@ int64_t Scm_BignumToSI64(const ScmBignum *b, int clamp, int *oor)
             r = b->values[0];
         } else if (b->size > 2 || b->values[1] > LONG_MAX) {
             if (!(clamp & SCM_CLAMP_HI)) goto err;
-            SCM_SET_INT64_MAX(r);
+            r = INT64_MAX;
         } else {
             r = ((int64_t)b->values[1] << 32) + (uint64_t)b->values[0];
         }
@@ -318,7 +318,7 @@ int64_t Scm_BignumToSI64(const ScmBignum *b, int clamp, int *oor)
             r = -(int64_t)b->values[0];
         } else if (b->size > 2 || (b->values[1] > LONG_MAX && b->values[0] > 0)) {
             if (!(clamp&SCM_CLAMP_LO)) goto err;
-            SCM_SET_INT64_MIN(r);
+            r = INT64_MIN;
         } else {
             r = -(int64_t)(((int64_t)b->values[1] << 32) + (uint64_t)b->values[0]);
         }
@@ -340,7 +340,7 @@ uint64_t Scm_BignumToUI64(const ScmBignum *b, int clamp, int *oor)
     if (b->sign > 0) {
         if (b->size > 2) {
             if (!(clamp&SCM_CLAMP_HI)) goto err;
-            SCM_SET_UINT64_MAX(r);
+            r = UINT64_MAX;
         } else if (b->size == 2) {
             r = (((uint64_t)b->values[1]) << 32) + (uint64_t)b->values[0];
         } else {
@@ -899,8 +899,8 @@ static ScmBignum *bignum_gdiv(const ScmBignum *dividend,
 #undef DIV_DEBUG
 #ifdef DIV_DEBUG
     Scm_Printf(SCM_CUROUT, "shift=%d, n=%d, m=%d\n", d, n, m);
-    Scm_Printf(SCM_CUROUT, "u="); Scm_DumpBignum(u, SCM_CUROUT);
-    Scm_Printf(SCM_CUROUT, "\nv="); Scm_DumpBignum(v, SCM_CUROUT);
+    Scm_Printf(SCM_CUROUT, "u="); Scm_BignumDump(u, SCM_CUROUT);
+    Scm_Printf(SCM_CUROUT, "\nv="); Scm_BignumDump(v, SCM_CUROUT);
     Scm_Printf(SCM_CUROUT, "\nvn_1=%08lx, vn_2=%08lx\n", vn_1, vn_2);
 #endif
 
@@ -930,7 +930,7 @@ static ScmBignum *bignum_gdiv(const ScmBignum *dividend,
         }
 #ifdef DIV_DEBUG
         Scm_Printf(SCM_CUROUT, "subtract cy = %d, ", cy);
-        Scm_Printf(SCM_CUROUT, "u="); Scm_DumpBignum(u, SCM_CUROUT);
+        Scm_Printf(SCM_CUROUT, "u="); Scm_BignumDump(u, SCM_CUROUT);
         Scm_Printf(SCM_CUROUT, "\n");
 #endif
         if (cy) {
@@ -949,8 +949,8 @@ static ScmBignum *bignum_gdiv(const ScmBignum *dividend,
     }
     bignum_rshift(u, u, d);
 #ifdef DIV_DEBUG
-    Scm_Printf(SCM_CUROUT, "quot q="); Scm_DumpBignum(quotient, SCM_CUROUT);
-    Scm_Printf(SCM_CUROUT, "\nrem  u="); Scm_DumpBignum(u, SCM_CUROUT);
+    Scm_Printf(SCM_CUROUT, "quot q="); Scm_BignumDump(quotient, SCM_CUROUT);
+    Scm_Printf(SCM_CUROUT, "\nrem  u="); Scm_BignumDump(u, SCM_CUROUT);
     Scm_Printf(SCM_CUROUT, "\n");
 #endif
     return u;
@@ -1254,7 +1254,7 @@ ScmObj Scm_BignumToString(const ScmBignum *b, int radix, int use_upper)
     return Scm_ListToString(Scm_ReverseX(h));
 }
 
-int Scm_DumpBignum(const ScmBignum *b, ScmPort *out)
+void Scm_BignumDump(const ScmBignum *b, ScmPort *out)
 {
     Scm_Printf(out, "#<bignum ");
     if (b->sign < 0) SCM_PUTC('-', out);
@@ -1262,7 +1262,6 @@ int Scm_DumpBignum(const ScmBignum *b, ScmPort *out)
         Scm_Printf(out, "%08lx ", b->values[i]);
     }
     SCM_PUTC('>', out);
-    return 0;
 }
 
 /*-----------------------------------------------------------------------
