@@ -1,7 +1,7 @@
 /*
  * code.c - compiled code builder/handler
  *
- *   Copyright (c) 2005-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2005-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,7 +33,6 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/class.h"
 #include "gauche/code.h"
 #include "gauche/vminsn.h"
 #include "gauche/priv/codeP.h"
@@ -222,7 +221,7 @@ void Scm_CompiledCodeDump(ScmCompiledCode *cc)
                                SCM_CDR(bindinfo));
                 } else {
                     Scm_Printf(SCM_CUROUT, "; %#40.1S\n",
-                               Scm_UnwrapSyntax(SCM_CDR(srcinfo)));
+                               Scm_UnwrapSyntax2(SCM_CDR(srcinfo), FALSE));
                 }
             }
         }
@@ -719,7 +718,7 @@ static inline void fill_current_insn(cc_builder *b, int code)
         }
     }
 #undef SET_LREF_ARGS
-    
+
     /* Compose insn word */
     switch (Scm_VMInsnNumParams(code)) {
     case 0: b->currentInsn = SCM_VM_INSN(code); break;
@@ -986,4 +985,3 @@ ScmWord Scm_VMInsnBuild(ScmObj obj)
     Scm_Error("Bad VM insn spec: %S", obj);
     return 0;       /* dummy */
 }
-
