@@ -1,7 +1,7 @@
 /*
  * compaux.c - C API bridge for the compiler
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -39,10 +39,11 @@
 #include "gauche.h"
 #include "gauche/vm.h"
 #include "gauche/vminsn.h"
-#include "gauche/class.h"
 #include "gauche/code.h"
+#include "gauche/priv/glocP.h"
 #include "gauche/priv/identifierP.h"
 #include "gauche/priv/builtin-syms.h"
+#include "gauche/priv/readerP.h" /* for Scm_MakeReadReference */
 
 /*
  * Syntax
@@ -137,21 +138,21 @@ static ScmClassStaticSlotSpec synclo_slots[] = {
  * Identifier object
  */
 
-/* 
+/*
  * About identifier's ENV slot.
- * We close identifier's binding environment (list of frames), so that later 
+ * We close identifier's binding environment (list of frames), so that later
  * we can look up its bindings hygienically.  We truncate the frames up to
  * where the binding occur, for the efficient lookup and comparison.
- * Notably, the identifiers that are unbound or refer to toplevel variable 
+ * Notably, the identifiers that are unbound or refer to toplevel variable
  * has () in its env.
  * A caveat--we can't truncate frames at the time of construction, since
  * the entire frame structure may not be fixed while we're processing internal
- * defines.  The identifier may refer to another identifier that will be 
+ * defines.  The identifier may refer to another identifier that will be
  * inserted later.  Thus, we delay the truncation operation until it is
  * needed.
  * The ENV slot itself now contains (<flag> . <frames>), where <flag> is #f
  * if truncation hasn't be done, #t otherwise.  ENV should be treated as
- * an opaque data for the others; you should always get it with 
+ * an opaque data for the others; you should always get it with
  * Scm_IdentifierEnv(), and never directly access ENV slot itself.
  */
 
@@ -205,8 +206,8 @@ ScmObj Scm_IdentifierEnv(ScmIdentifier *id)
         /* MT safety: This operation is idempotent, so it's ok if more than
            one thread execute here. */
         ScmObj f = get_binding_frame(id->name, SCM_CDR(id->frames));
-        SCM_SET_CDR(id->frames, f);
-        SCM_SET_CAR(id->frames, SCM_TRUE);
+        SCM_SET_CDR_UNCHECKED(id->frames, f);
+        SCM_SET_CAR_UNCHECKED(id->frames, SCM_TRUE);
     }
     return SCM_CDR(id->frames);
 }
@@ -286,30 +287,107 @@ static ScmClassStaticSlotSpec identifier_slots[] = {
 };
 
 /*------------------------------------------------------------------
- * Utility functions
+ * Unwrapping syntax
+ *
+ * Traverses a form and replaces identifiers for bare symbols.
+ * It's complicated because the form may have cycles.
+ * We use Scm_ReadReference as the temporary placeholder to handle
+ * circular structure.  All the read references are replaced before
+ * Scm_UnwrapSyntax returns.
  */
 
-/* Convert all identifiers in form into a symbol.
-   This keeps linear history to avoid entering infinite loop if
-   the given form is circular; but it doens't recover the shared
-   substructure. */
-static ScmObj unwrap_rec(ScmObj form, ScmObj history)
+typedef struct unwrap_ctx_rec {
+    ScmHashCore history;        /* object -> #f | R() | R(v)
+                                   where
+                                     SCM_UNBOUND - the object has been visited
+                                         only once
+                                     value - the object is realized
+                                     R() - empty ReadReference.  The object
+                                         has been visited more than once,
+                                         but its final replacement hasn't
+                                         been materialized yet.
+                                     R(v) - ReadReference with value.
+                                         The object should correspond to
+                                         the value.
+                                 */
+    ScmHashCore refs;           /* location -> read reference*/
+    int immutable;              /* flag */
+} unwrap_ctx;
+
+static void register_location(unwrap_ctx *ctx,
+                              ScmObj *loc,
+                              ScmObj ref)
+{
+    if (!SCM_READ_REFERENCE_P(ref)) return;
+    if (SCM_READ_REFERENCE_REALIZED(ref)) {
+        *loc = SCM_READ_REFERENCE(ref)->value;
+    } else {
+        ScmDictEntry *e =
+            Scm_HashCoreSearch(&ctx->refs, (intptr_t)loc, SCM_DICT_CREATE);
+        e->value = (intptr_t)ref;
+    }
+}
+
+static void fill_history(ScmDictEntry *e, ScmObj value)
+{
+    if (e->value) {
+        if (SCM_READ_REFERENCE_P(e->value)) {
+            SCM_READ_REFERENCE(e->value)->value = value;
+        }
+    } else {
+        e->value = (intptr_t)value;
+    }
+}
+
+static void patch_locations(unwrap_ctx *ctx)
+{
+    ScmHashIter iter;
+    Scm_HashIterInit(&iter, &ctx->refs);
+    for (;;) {
+        ScmDictEntry *e = Scm_HashIterNext(&iter);
+        if (e == NULL) break;
+        ScmObj *loc = (ScmObj*)SCM_DICT_KEY(e);
+        ScmReadReference *ref = SCM_READ_REFERENCE(SCM_DICT_VALUE(e));
+        if (SCM_READ_REFERENCE_P(ref)) {
+            SCM_ASSERT(SCM_READ_REFERENCE_REALIZED(ref));
+            *loc = SCM_READ_REFERENCE(ref)->value;
+        }
+    }
+}
+
+/* Returns either original form, or converted form, or a read reference. */
+static ScmObj unwrap_rec(ScmObj form, unwrap_ctx *ctx)
 {
     if (!SCM_PTRP(form)) return form;
-    if (!SCM_FALSEP(Scm_Memq(form, history))) return form;
-
-    ScmObj newh;
+    ScmDictEntry *e = Scm_HashCoreSearch(&ctx->history,
+                                         (intptr_t)form,
+                                         SCM_DICT_GET);
+    if (e) {
+        /* We've visited FORM before.  If this is the second time,
+           we allocate ReadReference to hold the value.  It will be
+           filled later by the caller. */
+        if (!e->value) {
+            e->value = (intptr_t)Scm_MakeReadReference();
+        }
+        return SCM_DICT_VALUE(e);
+    }
 
     if (SCM_PAIRP(form)) {
-        ScmObj ca, cd;
-        newh = Scm_Cons(form, history);
-        ca = unwrap_rec(SCM_CAR(form), newh);
-        cd = unwrap_rec(SCM_CDR(form), newh);
-        if (ca == SCM_CAR(form) && cd == SCM_CDR(form)) {
+        e = Scm_HashCoreSearch(&ctx->history, (intptr_t)form, SCM_DICT_CREATE);
+        ScmObj ca = unwrap_rec(SCM_CAR(form), ctx);
+        ScmObj cd = unwrap_rec(SCM_CDR(form), ctx);
+        if (ca == SCM_CAR(form) && cd == SCM_CDR(form)
+            && (!ctx->immutable || Scm_ImmutablePairP(form))) {
+            fill_history(e, form);
             return form;
-        } else {
-            return Scm_Cons(ca, cd);
         }
+        ScmObj p = (ctx->immutable
+                    ? Scm_MakeImmutablePair(ca, cd)
+                    : Scm_Cons(ca, cd));
+        fill_history(e, p);
+        register_location(ctx, &SCM_PAIR(p)->car, ca);
+        register_location(ctx, &SCM_PAIR(p)->cdr, cd);
+        return p;
     }
     if (SCM_IDENTIFIERP(form)) {
         return SCM_OBJ(Scm_UnwrapIdentifier(SCM_IDENTIFIER(form)));
@@ -317,31 +395,54 @@ static ScmObj unwrap_rec(ScmObj form, ScmObj history)
     if (SCM_VECTORP(form)) {
         int len = SCM_VECTOR_SIZE(form);
         ScmObj *pelt = SCM_VECTOR_ELEMENTS(form);
-        newh = Scm_Cons(form, history);
+        e = Scm_HashCoreSearch(&ctx->history, (intptr_t)form, SCM_DICT_CREATE);
         for (int i=0; i<len; i++, pelt++) {
-            ScmObj elt = unwrap_rec(*pelt, newh);
-            if (elt != *pelt) {
+            ScmObj elt = unwrap_rec(*pelt, ctx);
+            if (elt != *pelt
+                || (ctx->immutable && !SCM_VECTOR_IMMUTABLE_P(form))) {
                 ScmObj newvec = Scm_MakeVector(len, SCM_FALSE);
                 pelt = SCM_VECTOR_ELEMENTS(form);
                 int j;
                 for (j=0; j<i; j++, pelt++) {
                     SCM_VECTOR_ELEMENT(newvec, j) = *pelt;
                 }
+                register_location(ctx, &SCM_VECTOR_ELEMENT(newvec, i), elt);
                 SCM_VECTOR_ELEMENT(newvec, i) = elt;
-                for (; j<len; j++, pelt++) {
-                    SCM_VECTOR_ELEMENT(newvec, j) = unwrap_rec(*pelt, newh);
+                for (j=i+1, pelt++; j<len; j++, pelt++) {
+                    elt = unwrap_rec(*pelt, ctx);
+                    register_location(ctx, &SCM_VECTOR_ELEMENT(newvec, i), elt);
+                    SCM_VECTOR_ELEMENT(newvec, j) = elt;
                 }
+                if (ctx->immutable) {
+                    SCM_VECTOR_IMMUTABLE_SET(newvec, TRUE);
+                }
+                fill_history(e, newvec);
                 return newvec;
             }
         }
+        fill_history(e, form);
         return form;
     }
     return form;
 }
 
+#if GAUCHE_API_VERSION < 98
 ScmObj Scm_UnwrapSyntax(ScmObj form)
 {
-    return unwrap_rec(form, SCM_NIL);
+    return Scm_UnwrapSyntax2(form, FALSE);
+}
+ScmObj Scm_UnwrapSyntax2(ScmObj form, int immutablep)
+#else  /* GAUCHE_API_VERSION >= 98 */
+ScmObj Scm_UnwrapSyntax(ScmObj form, int immutablep)
+#endif /* GAUCHE_API_VERSION >= 98 */
+{
+    unwrap_ctx ctx;
+    Scm_HashCoreInitSimple(&ctx.history, SCM_HASH_EQ, 0, NULL);
+    Scm_HashCoreInitSimple(&ctx.refs, SCM_HASH_EQ, 0, NULL);
+    ctx.immutable = immutablep;
+    ScmObj r = unwrap_rec(form, &ctx);
+    patch_locations(&ctx);
+    return r;
 }
 
 /*===================================================================
