@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-1.scm - The compiler: Pass 1
 ;;;
-;;;   Copyright (c) 2004-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -59,12 +59,12 @@
 ;; we need to treat PROGRAM as a special form or an ordinary procedure.
 ;; It would be a large change, so this is a compromise...
 (define-inline (pass1/lookup-head head cenv)
-  (or (and (variable? head)
-           (cenv-lookup-syntax cenv head))
+  (or (and (identifier? head)
+           (cenv-lookup cenv head))
       (and (pair? head)
            (module-qualified-variable? head cenv)
            (let1 mod (ensure-module (cadr head) 'with-module #f)
-             (cenv-lookup-syntax (cenv-swap-module cenv mod) (caddr head))))))
+             (cenv-lookup (cenv-swap-module cenv mod) (caddr head))))))
 
 ;;--------------------------------------------------------------
 ;; pass1 :: Sexpr, Cenv -> IForm
@@ -84,7 +84,13 @@
           [(macro)  (pass1 (call-macro-expander gval program cenv) cenv)]
           [(syntax) (call-syntax-handler gval program cenv)]
           [(inline) (or (pass1/expand-inliner program id gval cenv)
-                        (pass1/call program ($gref id) (cdr program) cenv))])
+                        (pass1/call program ($gref id) (cdr program) cenv))]
+          [(type-ctor)
+           ;; Type constructor have to be called during compilation,
+           ;; and the resulting node becomes ($const <type-instance>).
+           (type/construct gval
+                           (pass1/call program ($gref id) (cdr program) cenv)
+                           cenv)])
         (pass1/call program ($gref id) (cdr program) cenv))))
 
   ;; main body of pass1
@@ -95,7 +101,7 @@
     (cond
      [(pass1/lookup-head (car program) cenv)
       => (^h (cond
-              [(identifier? h) (pass1/global-call h)]
+              [(wrapped-identifier? h) (pass1/global-call h)]
               [(lvar? h) (pass1/call program ($lref h) (cdr program) cenv)]
               [(macro? h) ;; local macro
                (pass1 (call-macro-expander h program cenv) cenv)]
@@ -117,26 +123,28 @@
                            (cdr program) cenv)))]
      [else (pass1/call program (pass1 (car program) (cenv-sans-name cenv))
                        (cdr program) cenv)])]
-   [(variable? program)                 ; variable reference
-    (let1 r (cenv-lookup-variable cenv program)
+   [(identifier? program)               ; variable reference
+    (let1 r (cenv-lookup cenv program)
       (cond [(lvar? r) ($lref r)]
-            [(identifier? r)
+            [(wrapped-identifier? r)
              (or (and-let* ([const (find-const-binding r)]) ($const const))
                  ($gref r))]
+            [(macro? r) ;; local macro appearing in non-head pos
+             (pass1 (call-id-macro-expander r cenv) cenv)]
             [else (error "[internal] cenv-lookup returned weird obj:" r)]))]
    [else ($const program)]))
 
 ;; If op is (setter <var>), check if <var> has inlinable binding and
-;; its setter is locked; if so, returns the setter.  
+;; its setter is locked; if so, returns the setter.
 ;; There are a bunch of hoops to go through to satisfy the condition.
 (define (pass1/detect-constant-setter-call op cenv)
   (and (pair? op) (pair? (cdr op)) (null? (cddr op))
        (not (vm-compiler-flag-is-set? SCM_COMPILE_NOINLINE_SETTERS))
        (global-identifier=? (car op) setter.)
        (and-let* ([var (cadr op)]
-                  [ (variable? var) ]   ;ok, <var> is variable
+                  [ (identifier? var) ] ;ok, <var> is variable
                   [hd (pass1/lookup-head var cenv)]
-                  [ (identifier? hd) ]
+                  [ (wrapped-identifier? hd) ]
                   [gloc (id->bound-gloc hd)] ; <var> has inlinable binding
                   [val (gloc-ref gloc)]
                   [ (procedure? val) ])
@@ -157,7 +165,7 @@
              [opt?  (slot-ref proc 'optional)])
          (unless (argcount-ok? args (slot-ref proc 'required) opt?)
            (errorf "wrong number of arguments: ~a requires ~a, but got ~a"
-                   (if (variable? name) (variable-name name) name)
+                   (if (identifier? name) (identifier->symbol name) name)
                    (slot-ref proc 'required) nargs))
          ;; We might get away with this limit by transforming inline calls
          ;; to apply or something.  Maybe in future.
@@ -191,12 +199,20 @@
 ;; thus we can afford the time.
 (define (module-qualified-variable? expr cenv)
   (match expr
-    [((? variable? wm) mod (? variable? v))
-     (and-let* ([var (cenv-lookup-syntax cenv wm)]
+    [((? identifier? wm) mod (? identifier? v))
+     (and-let* ([var (cenv-lookup cenv wm)]
                 [ (identifier? var) ])
        (global-identifier=? var with-module.))]
     [_ #f]))
 
+;;--------------------------------------------------------------
+;; pass1/type-expression - Compile input as a type expression
+;;
+
+(define (pass1/type-expression expr cenv)
+  (or (type/ensure (pass1 expr cenv) cenv)
+      (error "Invalid type expression:" expr)))
+
 ;;--------------------------------------------------------------
 ;; pass1/body - Compiling body with internal definitions.
 ;;
@@ -249,7 +265,8 @@
              (pass1/body-finish exprs mframe vframe cenv)]
             [(and (pair? head) (eq? (car head) :rec))
              (pass1/body-finish exprs mframe vframe cenv)]
-            [(not (identifier? head)) (error "[internal] pass1/body" head)]
+            [(not (wrapped-identifier? head))
+             (error "[internal] pass1/body" head)]
             [(or (global-identifier=? head define.)
                  (global-identifier=? head define-inline.)
                  (global-identifier=? head r5rs-define.))
@@ -302,7 +319,7 @@
             [(global-identifier=? head include-ci.)
              (let1 sexpr&srcs (pass1/expand-include args cenv #t)
                (pass1/body-rec (append sexpr&srcs rest) mframe vframe cenv))]
-            [(identifier? head)
+            [(wrapped-identifier? head)
              (or (and-let* ([gloc (id->bound-gloc head)]
                             [gval (gloc-ref gloc)]
                             [ (macro? gval) ])
@@ -363,9 +380,9 @@
 
 ;; get symbol or id, and returns identiier.
 (define (ensure-identifier sym-or-id cenv)
-  (if (identifier? sym-or-id)
-    sym-or-id
-    (make-identifier sym-or-id (cenv-module cenv) (cenv-frames cenv))))
+  (if (symbol? sym-or-id)
+    (make-identifier sym-or-id (cenv-module cenv) (cenv-frames cenv))
+    sym-or-id))
 
 ;; Does the given argument list satisfy procedure's reqargs/optarg?
 (define (argcount-ok? args reqargs optarg?)
@@ -380,15 +397,14 @@
 
 ;; returns a module specified by THING.
 (define (ensure-module thing name create?)
-  (let1 mod (cond [(symbol? thing) (find-module thing)]
-                  [(identifier? thing) (find-module (unwrap-syntax thing))]
+  (let1 mod (cond [(identifier? thing) (find-module (identifier->symbol thing))]
                   [(module? thing) thing]
                   [else
                    (errorf "~a requires a module name or a module, but got: ~s"
                            name thing)])
     (or mod
         (if create?
-          (make-module (if (identifier? thing) (unwrap-syntax thing) thing))
+          (make-module (identifier->symbol thing))
           (errorf "~a: no such module: ~s" name thing)))))
 
 ;; IFORM must be a $LAMBDA node.  This expands the application of IFORM
@@ -519,8 +535,9 @@
        ;; R7RS define doesn't allow it.
        (error "define without an expression is not allowed in R7RS (it is in R6RS):" oform))]
     [(_ name expr)
-     (unless (variable? name) (error "syntax-error:" oform))
+     (unless (identifier? name) (error "syntax-error:" oform))
      (let1 cenv (cenv-add-name cenv (variable-name name))
+       ;; Hygiene alert
        ;; If NAME is an identifier, it is inserted by macro expander; we
        ;; can't simply place it in $define, since it would insert a toplevel
        ;; definition into the toplevel of macro-definition environment---
@@ -530,14 +547,14 @@
        ;; other code except the code generated in the same macro expansion.
        ;; A trick - we directly modify the identifier, so that other forms
        ;; referring to the same (eq?) identifier can keep referring it.
-       (let1 id (if (identifier? name)
+       (let1 id (if (wrapped-identifier? name)
                   (%rename-toplevel-identifier! name)
                   (make-identifier name module '()))
          ;; Insert dummy binding at compile time, if we don't have one yet.
          ;; This matters when we compile multiple modules at once, and
          ;; one need to import from another with qualifiers.
          (unless (vm-compiler-flag-is-set? SCM_COMPILE_LEGACY_DEFINE)
-           (%insert-binding module (unwrap-syntax name) 
+           (%insert-binding module (unwrap-syntax name)
                             (%uninitialized) '(fresh)))
          ($define oform flags id (pass1 expr cenv))))]
     [_ (error "syntax-error:" oform)]))
@@ -584,7 +601,7 @@
     [(_ (name . args) . body)
      (pass1/define-inline form name `(,lambda. ,args ,@body) cenv)]
     [(_ name expr)
-     (unless (variable? name) (error "syntax-error:" form))
+     (unless (identifier? name) (error "syntax-error:" form))
      (pass1/define-inline form name expr cenv)]
     [_ (error "syntax-error: malformed define-inline:" form)]))
 
@@ -670,43 +687,52 @@
     (when name
       ;; record inliner function for compiler.  this is used only when
       ;; the procedure needs to be inlined in the same compiler unit.
-      (%insert-binding module (unwrap-syntax name) dummy-proc)
-      (set! (%procedure-inliner dummy-proc) (pass1/inliner-procedure packed)))))
+      (%insert-binding module (unwrap-syntax name) dummy-proc
+                       '(inlinable dummy))
+      (set! (%procedure-inliner dummy-proc) packed))))
 
 (define (pass1/make-inlinable-binding form name iform cenv)
   ;; See the comment in pass1/define about renaming the toplevel identifier.
-  (let1 id (if (identifier? name)
+  (let1 id (if (wrapped-identifier? name)
              (%rename-toplevel-identifier! name)
              (make-identifier name (cenv-module cenv) '()))
     ($define form '(inlinable) id iform)))
 
-(define (pass1/inliner-procedure inline-info)
-  (unless (vector? inline-info)
-    (error "[internal] pass1/inliner-procedure got invalid info" inline-info))
-  (^[form cenv]
-    (expand-inlined-procedure form (unpack-iform inline-info)
-                              (imap (cut pass1 <> cenv) (cdr form)))))
-
 ;; Toplevel macro definitions
 
 (define-pass1-syntax (define-macro form cenv) :gauche
   (check-toplevel form cenv)
   (match form
     [(_ (name . formals) body ...)
-     (pass1/define-macro form name `(,lambda. ,formals ,@body) cenv)]
+     ($ pass1/define-macro form name
+        (with-source-info `(,lambda. ,formals ,@body) form)
+        cenv)]
     [(_ name expr)
      (pass1/define-macro form name expr cenv)]
     [_ (error "syntax-error:" form)]))
 
+(define (with-source-info obj src)
+  (or (and-let* ([ (pair? obj) ]
+                 [si (debug-source-info src)])
+        (extended-cons (car obj) (cdr obj) `((source-info . ,si))))
+      obj))
+
 (define (pass1/define-macro src name expr cenv)
-  (unless (variable? name) (error "syntax-error:" src))
+  (unless (identifier? name) (error "syntax-error:" src))
   ;; TODO: macro autoload
   (let* ([proc (eval expr (cenv-module cenv))]
-         [trans (%make-macro-transformer name
-                                         (^[form env] (apply proc (cdr form)))
-                                         expr #f)])
-    ;; See the "Hygiene alert" in pass1/define.
-    (%insert-syntax-binding (cenv-module cenv) (unwrap-syntax name) trans)
+         [source-info (debug-source-info expr)]
+         [trans ($ %make-macro-transformer name
+                   (^[form env] (apply proc (cdr form)))
+                   `((source . ,expr)
+                     ,@(cond-list [source-info `(source-info . ,source-info)])))]
+         ;; See the "Hygiene alert" in pass1/define.
+         [id (if (wrapped-identifier? name)
+               (%rename-toplevel-identifier! name)
+               (make-identifier name (cenv-module cenv) '()))])
+    (%insert-syntax-binding (identifier-module id)
+                            (unwrap-syntax name)
+                            trans)
     ($const-undef)))
 
 (define-pass1-syntax (define-syntax form cenv) :null
@@ -716,26 +742,35 @@
   (match form
     [(_ name expr)
      (let* ([cenv (cenv-add-name cenv (variable-name name))]
-            [transformer (pass1/eval-macro-rhs 'define-syntax expr cenv)])
-       ;; See the "Hygiene alert" in pass1/define.
-       (%insert-syntax-binding (cenv-module cenv) (unwrap-syntax name)
-                               transformer)
+            [trans (pass1/eval-macro-rhs 'define-syntax expr cenv)]
+            ;; See the "Hygiene alert" in pass1/define.
+            [id (if (wrapped-identifier? name)
+                  (%rename-toplevel-identifier! name)
+                  (make-identifier name (cenv-module cenv) '()))])
+       (%insert-syntax-binding (identifier-module id)
+                               (unwrap-syntax name)
+                               trans)
        ($const-undef))]
     [_ (error "syntax-error: malformed define-syntax:" form)]))
 
 ;; Experimental
-(define-pass1-syntax (define-inline/syntax form cenv) :gauche
+(define-pass1-syntax (define-hybrid-syntax form cenv) :gauche
+  (pass1-define-hybrid-syntax form cenv))
+(define-pass1-syntax (define-inline/syntax form cenv) :gauche ;deprecated
+  (pass1-define-hybrid-syntax form cenv))
+
+(define (pass1-define-hybrid-syntax form cenv)
   (check-toplevel form cenv)
   (match form
     [(_ name expr macro-expr)
      (let* ([cenv (cenv-add-name cenv (variable-name name))]
-            [xformer (pass1/eval-macro-rhs 'define-inline/syntax
+            [xformer (pass1/eval-macro-rhs 'define-hybrid-syntax
                                            macro-expr cenv)]
             [body (pass1/call expr ($gref %with-inline-transformer.)
                               (list expr xformer) cenv)])
        (pass1/make-inlinable-binding form name body cenv))]
-    [_ (error "syntax-error: define-inline/syntax")]))
-            
+    [_ (error "syntax-error: define-hybrid-syntax")]))
+
 ;; Returns either <syntax> or <macro>
 (define (pass1/eval-macro-rhs who expr cenv)
   (rlet1 transformer ((make-toplevel-closure (compile expr cenv)))
@@ -754,29 +789,62 @@
 
 (define-pass1-syntax (er-macro-transformer form cenv) :gauche
   (match form
-    [(_ xformer)
-     ;; We need to capture the current CENV as the macro definition
-     ;; environment.  There's a catch, though---if we're AOT compiling
-     ;; a macro, the captured CENV must be serializable to a file,
-     ;; which isn't generally the case.
-     ;; So, if we're compiling toplevel, we call a special API that takes
-     ;; the current module and cenv-exp-name, and reconstruct the cenv
-     ;; at runtime.
-     ;; If cenv has local environment, we don't bother that, for the macro
-     ;; will be fully expanded during AOT compilation.  HOWEVER - we can't
-     ;; embed cenv as a vector literal (e.g. `',cenv) since quoting will
-     ;; strip all identifier information in cenv.  The right thing would be
-     ;; to make cenv as a record.  For now, we take advantage that unquoted
-     ;; vector evaluates to itself, and insert cenv without quoting.  This
-     ;; has to change if we prohibit unquoted vector literals.
-     (if (cenv-toplevel? cenv)
-       (pass1 `(,%make-er-transformer/toplevel. ,xformer
-                                                ,(cenv-module cenv)
-                                                ',(cenv-exp-name cenv))
-              cenv)
-       (pass1 `(,%make-er-transformer. ,xformer ,cenv) cenv))]
+    [(_ xformer) (%pass1/er-macro-maker form xformer cenv #f #f)]
+    [(_ xformer ':identifier-macro? v)
+     (assume (boolean? v))
+     (%pass1/er-macro-maker form xformer cenv #f v)]
     [_ (error "syntax-error: malformed er-macro-transformer:" form)]))
 
+(define-pass1-syntax (eri-macro-transformer form cenv) :gauche
+  (match form
+    [(_ xformer) (%pass1/er-macro-maker form xformer cenv #t #f)]
+    [(_ xformer ':identifier-macro? v)
+     (assume (boolean? v))
+     (%pass1/er-macro-maker form xformer cenv #t v)]
+    [_ (error "syntax-error: malformed eri-macro-transformer:" form)]))
+
+;; Build an expression to construct er macro at runtime, and run pass 1
+;; on it.
+;;
+;; We need to capture the current CENV as the macro definition
+;; environment.  There's a catch, though---if we're AOT compiling
+;; a macro, the captured CENV must be serializable to a file,
+;; which isn't generally the case.
+;; So, if we're compiling toplevel, we call a special API that takes
+;; the current module and cenv-exp-name, and reconstruct the cenv
+;; at runtime.
+;; If cenv has local environment, we don't bother that, for the macro
+;; will be fully expanded during AOT compilation.  HOWEVER - we can't
+;; embed cenv as a vector literal (e.g. `',cenv) since quoting will
+;; strip all identifier information in cenv.  The right thing would be
+;; to make cenv as a record.  For now, we take advantage that unquoted
+;; vector evaluates to itself, and insert cenv without quoting.  This
+;; has to change if we prohibit unquoted vector literals.
+(define (%pass1/er-macro-maker form xformer cenv has-inject? id-macro?)
+  ;; We don't save source---it's not so much useful, and we don't want it to
+  ;; take up space in precompiled code.
+  (define info
+    (or (and-let* ([ (pair? form) ]
+                   [si (pair-attribute-get form 'source-info #f)])
+          `((source-info . ,si)))
+        '()))
+
+  (if (cenv-toplevel? cenv)
+    (pass1 `(,%make-er-transformer/toplevel.
+             ,xformer ,(cenv-module cenv) ',(cenv-exp-name cenv)
+             ,@(cond-list
+                [has-inject? @ `(:has-inject? ',has-inject?)]
+                [(pair? info) @ `(:info-alist ',info)]
+                [id-macro? @ `(:identifier-macro? #t)]))
+           cenv)
+    (pass1 `(,%make-er-transformer.
+             ,xformer ,cenv
+             ,@(cond-list
+                [has-inject? @ `(:has-inject? ',has-inject?)]
+                [(pair? info) @ `(:info-alist ',info)]
+                [id-macro? @ `(:identifier-macro? #t)]))
+           cenv)))
+
 (define-pass1-syntax (%macroexpand form cenv) :gauche
   (match form
     [(_ expr) ($const (%internal-macro-expand expr cenv #f))]
@@ -790,10 +858,10 @@
 (define (%internal-macro-expand expr cenv once?)
   (define (xpand expr)
     (match expr
-      [((? variable? op) . args)
-       (let1 var (cenv-lookup-syntax cenv op)
+      [((? identifier? op) . args)
+       (let1 var (cenv-lookup cenv op)
          (cond [(macro? var) (call-macro-expander var expr cenv)]
-               [(identifier? var)
+               [(wrapped-identifier? var)
                 (if-let1 gval (and-let* ([gloc (id->bound-gloc var)]
                                          [gval (gloc-ref gloc)]
                                          [ (macro? gval) ])
@@ -1013,34 +1081,42 @@
     (match binds
       [() (pass1/body body cenv)]
       [((exp) . more)
-       ($if form (pass1 exp (cenv-sans-name cenv))
-            (process-binds more body cenv)
-            ($it))]
-      [([? variable? var] . more)
-       ($if form (pass1 var (cenv-sans-name cenv))
-            (process-binds more body cenv)
-            ($it))]
-      [(([? variable? var] init) . more)
-       (let* ([lvar (make-lvar var)]
-              [newenv (cenv-extend cenv `((,var . ,lvar)) LEXICAL)]
-              [itree (pass1 init (cenv-add-name cenv var))])
-         (lvar-initval-set! lvar itree)
-         ($let form 'let
-               (list lvar)
-               (list itree)
-               ($if form ($lref lvar)
-                    (process-binds more body newenv)
-                    ($it))))]
+       (if (and (null? more) (null? body))
+         (pass1 exp (cenv-sans-name cenv))
+         ($if form (pass1 exp (cenv-sans-name cenv))
+              (process-binds more body cenv)
+              ($it)))]
+      [([? identifier? var] . more)
+       (if (and (null? more) (null? body))
+         (pass1 var (cenv-sans-name cenv))
+         ($if form (pass1 var (cenv-sans-name cenv))
+              (process-binds more body cenv)
+              ($it)))]
+      [(([? identifier? var] init) . more)
+       (if (and (null? more) (null? body))
+         (pass1 init (cenv-add-name cenv var))
+         (let* ([lvar (make-lvar var)]
+                [newenv (cenv-extend cenv `((,var . ,lvar)) LEXICAL)]
+                [itree (pass1 init (cenv-add-name cenv var))])
+           (lvar-initval-set! lvar itree)
+           ($let form 'let
+                 (list lvar)
+                 (list itree)
+                 ($if form ($lref lvar)
+                      (process-binds more body newenv)
+                      ($it)))))]
       [_ (error "syntax-error: malformed and-let*:" form)]))
 
   (match form
+    [(_ ()) ($const #t)]                ;special base case
     [(_ binds . body) (process-binds binds body cenv)]
     [_ (error "syntax-error: malformed and-let*:" form)]))
 
 ;; Quote and quasiquote ................................
 
 (define (pass1/quote obj)
-  ($const (unwrap-syntax obj)))
+  ($const ($ unwrap-syntax obj
+             $ not $ vm-compiler-flag-is-set? SCM_COMPILE_MUTABLE_LITERALS)))
 
 (define-pass1-syntax (quote form cenv) :null
   (match form
@@ -1105,7 +1181,7 @@
              ($cons obj ($const op) xx))))]
       [(? pair?)       (quasi* obj level)]
       [(? vector?)     (quasi-vector obj level)]
-      [(? identifier?) ($const (unwrap-syntax obj))]
+      [(? wrapped-identifier?) ($const (unwrap-syntax obj))]
       [() ($const-nil)]
       [_  ($const obj)]))
 
@@ -1174,7 +1250,7 @@
             [(and (pair? (vector-ref obj i))
                   (unquote-splicing? (car (vector-ref obj i))))]
             [else (loop (+ i 1))])))
-  
+
   (quasi obj 0))
 
 
@@ -1183,16 +1259,16 @@
 (define-pass1-syntax (lambda form cenv) :null ;RnRS lambda
   (match form
     [(_ formals . body)
-     (receive (reqs rest) 
+     (receive (reqs rest)
          (let loop ((xs formals) (ys '()))
            (cond [(null? xs) (values (reverse ys) #f)]
-                 [(variable? xs) (values (reverse ys) xs)]
-                 [(pair? xs) 
-                  (unless (variable? (car xs))
+                 [(identifier? xs) (values (reverse ys) xs)]
+                 [(pair? xs)
+                  (unless (identifier? (car xs))
                     (error "Invalid formal parameter:" (car xs)))
                   (loop (cdr xs) (cons (car xs) ys))]
                  [else (error "Invalid formal parameter:" formals)]))
-       (pass1/vanilla-lambda (add-arg-info form formals) 
+       (pass1/vanilla-lambda (add-arg-info form formals #f)
                              (if rest (append reqs (list rest)) reqs)
                              (length reqs)
                              (if rest 1 0)
@@ -1202,26 +1278,33 @@
 (define-pass1-syntax (lambda form cenv) :gauche ;Extended lambda
   (match form
     [(_ formals . body)
-     (receive (args nreqs nopts kargs) (parse-extended-lambda-args formals)
-       (if (null? kargs)
-         (pass1/vanilla-lambda (add-arg-info form formals)
-                               args nreqs nopts body cenv)
-         ;; Convert extended lambda into vanilla lambda
-         (let1 restarg (gensym "rest")
-           (pass1/vanilla-lambda (add-arg-info form formals)
-                                 (append args (list restarg))
-                                 nreqs 1
-                                 (pass1/extended-lambda-body form cenv restarg
-                                                             kargs body)
-                                 cenv))))]
+     (receive (args formals types nreqs nopts kargs)
+         (parse-extended-lambda-args formals)
+       (let1 typeinfo (and types
+                           (pass1-for-type-annotations types args cenv))
+         (if (null? kargs)
+           (pass1/vanilla-lambda (add-arg-info form formals typeinfo)
+                                 args nreqs nopts body cenv)
+           ;; Convert extended lambda into vanilla lambda
+           (let1 restarg (gensym "rest")
+             (pass1/vanilla-lambda (add-arg-info form formals typeinfo)
+                                   (append args (list restarg))
+                                   nreqs 1
+                                   (pass1/extended-lambda-body form cenv restarg
+                                                               kargs body)
+                                   cenv)))))]
     [_ (error "syntax-error: malformed lambda:" form)]))
 
-;; Add formals list as 'arg-info attributes of the source form
-(define (add-arg-info form formals)
+;; Add formals list as 'arg-info, and argument type list to 'arg-types,
+;; in the source form.  They're retrieved by compiled-code-attach-source-info.
+;; NB: For arg-types, We need to retrieve <type> from names.  To be done.
+(define (add-arg-info form formals arg-types)
   (rlet1 xform (if (extended-pair? form)
                  form
                  (extended-cons (car form) (cdr form)))
-    (pair-attribute-set! xform 'arg-info formals)))
+    (pair-attribute-set! xform 'arg-info formals)
+    (when arg-types
+      (pair-attribute-set! xform 'arg-types arg-types))))
 
 (define (pass1/vanilla-lambda form formals nreqs nopts body cenv) ; R7RS lambda
   (let* ([lvars (imap make-lvar+ formals)]
@@ -1234,7 +1317,8 @@
 (define-pass1-syntax (receive form cenv) :gauche
   (match form
     [(_ formals expr body ...)
-     (receive (args nreqs nopts kargs) (parse-extended-lambda-args formals)
+     (receive (args formals types nreqs nopts kargs)
+         (parse-extended-lambda-args formals)
        (unless (null? kargs)
          (error "syntax-error: extended lambda list isn't allowed in receive:"
                 form))
@@ -1244,15 +1328,48 @@
                    (pass1/body body newenv))))]
     [_ (error "syntax-error: malformed receive:" form)]))
 
-;; Returns <list of args>, <# of reqargs>, <has optarg?>, <kargs>
-;; <kargs> is like (:optional (x #f) (y #f) :rest k) etc.
+;; Returns <args>, <formals>, <types>, <nreqs>, <nopts>, <kargs>
+;;   <args> is the list of requried args, plus a restarg if any.
+;;   <formals> is a copy of input sans type annotation.
+;;   <types> is a list of types for required args.  If there's a restarg,
+;;      the lenght of <types> is one shorter than the length of <args>.
+;;      Can be #f if there's no type annotation.
+;;   <nreqs> is the number of required arguments.
+;;   <nopts> is 0 if no optional arg, 1 if not.
+;;   <kargs> is like (:optional (x #f) (y #f) :rest k) etc.  '() if restarg.
 (define (parse-extended-lambda-args formals)
-  (let loop ([formals formals] [args '()] [n 0])
-    (match formals
-      [()      (values (reverse args) n 0 '())]
-      [((? keyword-like?) . _) (values (reverse args) n 1 formals)]
-      [(x . y) (loop (cdr formals) (cons (car formals) args) (+ n 1))]
-      [x       (values (reverse (cons x args)) n 1 '())])))
+  ;; If formals don't have type annotations at all, we skip building
+  ;; copy of formals and type lists.
+  (if (and (pair? formals) (member ':: formals))
+    (let loop ([rest formals] [as '()] [fs '()] [ts '()] [n 0])
+      (match rest
+        [()      (values (reverse as) (reverse fs) (reverse ts) n 0 '())]
+        [((? keyword-like?) . _)
+         (values (reverse as) (reverse fs formals) (reverse ts) n 0 rest)]
+        [(x ':: t . y)
+         (loop y (cons x as) (cons x fs) (cons t ts) (+ n 1))]
+        [(x . y)
+         (loop y (cons x as) (cons x fs) (cons '<top> ts) (+ n 1))]
+        [x (values (reverse (cons x as)) (reverse fs x) (reverse ts) n 1 '())]))
+    (let loop ([rest formals] [as '()] [n 0])
+      (match rest
+        [()      (values (reverse as) formals #f n 0 '())]
+        [((? keyword-like?) . _) (values (reverse as) formals #f n 0 rest)]
+        [(x . y) (loop y (cons x as) (+ n 1))]
+        [x (values (reverse (cons x as)) formals #f n 1 '())]))))
+
+;; Run pass1 on each type expression in the argument type annotaions.
+;; NB: Written in a way to avoid closure allocation.
+(define (pass1-for-type-annotations types args cenv)
+  (define (type-1 type arg cenv)
+    (or (type/ensure (pass1 type cenv) cenv)
+        (errorf "Invalid type expression ~s for argument ~s" type arg)))
+  (define (type-n types args cenv)
+    (if (null? types)
+      '()
+      (cons (type-1 (car types) (car args) cenv)
+            (type-n (cdr types) (cdr args) cenv))))
+  (type-n types args cenv))
 
 ;; Handles extended lambda list.  garg is a gensymed var that receives
 ;; restarg.
@@ -1297,14 +1414,13 @@
         `((,let. ((,r ,garg)) ,@(expand-key ks garg a)))
         (expand-key ks garg a))
       (let ([binds (map (match-lambda
-                          [[? symbol? o] o]
                           [[? identifier? o] o]
                           [(o init) `(,o ,init)]
                           [_ (error "illegal optional argument spec in " kargs)])
                         os)]
             [rest (or r (gensym))])
         `((,let-optionals*. ,garg ,(append binds rest)
-           ,@(if (and (not r) (null? ks))
+           ,@(if (and (not r) (null? ks) (not a))
                ;; TODO: better error message!
                `((unless (null? ,rest)
                    (error "too many arguments for" ',form))
@@ -1312,14 +1428,17 @@
                (expand-key ks rest a)))))))
   (define (expand-key ks garg a)
     (if (null? ks)
-      body
+      (if a
+        ;; The case when we have :allow-other-keys without :key.
+        ;; We don't deal with specific keyword arguments, but expecting
+        ;; the user provides some.  Using let-keywords* checks the
+        ;; argument list is even.
+        `((,let-keywords*. ,garg ,(if (boolean? a) (gensym) a) ,@body))
+        body)
       (let1 args (map (match-lambda
-                        [[? symbol? o] o]
                         [[? identifier? o] o]
                         [(([? keyword-like? key] o) init)
-                         (let1 k (if (identifier? key)
-                                   (identifier->symbol key)
-                                   key)
+                         (let1 k (unwrap-syntax-1 key)
                            `(,o ,k ,init))]
                         [(o init) `(,o ,init)]
                         [_ (error "illegal keyword argument spec in " kargs)])
@@ -1370,7 +1489,7 @@
                   expr lvars)
              (pass1/body body newenv)))]
     [(_ name ((var expr) ...) body ...)
-     (unless (variable? name) (error "bad name for named let:" name))
+     (unless (identifier? name) (error "bad name for named let:" name))
      ;; Named let.  (let name ((var exp) ...) body ...)
      ;;
      ;;  We don't use the textbook expansion here
@@ -1478,9 +1597,9 @@
      ;; have a chance of optimization.
      (pass1 (with-original-source `((,setter. ,op) ,@args ,expr) form) cenv)]
     [(_ name expr)
-     (unless (variable? name)
+     (unless (identifier? name)
        (error "syntax-error: malformed set!:" form))
-     (let ([var (cenv-lookup-variable cenv name)]
+     (let ([var (cenv-lookup cenv name)]
            [val (pass1 expr cenv)])
        (if (lvar? var)
          ($lset var val)
@@ -1672,7 +1791,7 @@
         (pass1/report-include iport #f)
         (close-input-port iport))))
   (map do-include args))
-  
+
 ;; If filename is relative, we try to resolve it with the source file.
 ;; whenever possible.
 (define (pass1/open-include-file path includer-path)
@@ -1779,4 +1898,3 @@
        exp)]
     [_ (error "syntax-error: malformed with-meta:" form)]))
 |#
-
