@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-2.scm - The compiler: Pass 2
 ;;;
-;;;   Copyright (c) 2004-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -164,7 +164,7 @@
 ;;
 ;;     NB: In the following pattern, we can't do the conversion unless p is
 ;;     constant or unmodified $LREF, for (g b) might modify p.
-;;     
+;;
 ;;       (let ([q (g b)]) (h p q) (foo))
 ;;
 ;;     NB: We don't need to consider the possibility that (g b) modifies h,
@@ -493,6 +493,8 @@
                                       (cons iform penv) #t))
   iform)
 
+(define (pass2/$CLAMBDA iform penv tail?) iform)
+
 (define (pass2/$LABEL iform penv tail?)
   ;; $LABEL's body should already be processed by pass2, so we don't need
   ;; to do it again.
@@ -627,8 +629,13 @@
 ;;
 ;; We only do this transformation if every x y z ... is either $lref
 ;; or $const.  If there's more complex expression, we can't change the order
-;; of evaluation, so we'd have to modify the outer $let form---which isn't 
-;; trivial.
+;; of evaluation, so we'd have to modify the outer $let form---which isn't
+;; trivial, so we just give up such a case.
+;;
+;; If the argument list is constructed (as opposed to be a constant),
+;; we also need to consider the possibility that the list can be mutated
+;; before passed to apply.  We check if the intermediate variables are
+;; used elsewhere, and give up if they are passed to other procedures.
 ;;
 ;; IForm must be an $ASM node.
 ;; Returns (potentially updated) iform.
@@ -658,10 +665,10 @@
                       [y (check-safe (drop-right ($list*-args rarg) 1))])
              (append y z))]
           [(has-tag? rarg $ASM)
-           (case/unquote 
+           (case/unquote
             (car ($asm-insn rarg))
             [(LIST) (check-safe ($asm-args rarg))]
-            [(LIST-STAR) 
+            [(LIST-STAR)
              (and-let* ([z (expand-restarg (last ($asm-args rarg)))]
                         [y (check-safe (drop-right ($asm-args rarg) 1))])
                (append y z))]
@@ -678,7 +685,8 @@
   (assume (length>=? args 2))
   (if (vm-compiler-flag-is-set? SCM_COMPILE_NODISSOLVE_APPLY)
     iform
-    (or (and-let1 iargs (expand-restarg (last args))
+    (or (and-let* ([iargs (expand-restarg (last args))]
+                   [ (every (^v (= (lvar-ref-count v) 1)) freed-lvars) ])
           (ifor-each lvar-ref--! freed-lvars)
           (ifor-each (^x (when ($lref? x) (lvar-ref++! ($lref-lvar x)))) iargs)
           (pass2/rec ($call ($*-src iform) (car args)
@@ -723,6 +731,7 @@
             [(VEC-LEN) (pass2/const-veclen args)]
             [(EQ)      (pass2/const-op2 eq? args)]
             [(EQV)     (pass2/const-op2 eqv? args)]
+            [(NUMEQ2)  (pass2/const-numop2 = args)]
             [(NUMADD2) (pass2/const-numop2 + args)]
             [(NUMSUB2) (pass2/const-numop2 - args)]
             [(NUMMUL2) (pass2/const-numop2 * args)]
@@ -840,4 +849,3 @@
 
 ;; Dispatch table.
 (define *pass2-dispatch-table* (generate-dispatch-table pass2))
-
