@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-5.scm - The compiler: Pass 5
 ;;;
-;;;   Copyright (c) 2004-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -544,7 +544,7 @@
       (unless (lvar-immutable? (car lvars))
         (compiled-code-emit1! ccb BOX cnt))
       (loop (cdr lvars) (- cnt 1)))))
-       
+
 ;; emit LSET or ENV-SET insn to initialize lvars that aren't closures or
 ;; constants.  init-alist is the second value partition-letrec-inits
 ;; returned, which is a list of (<lvar-count> <init-iform>).
@@ -593,6 +593,9 @@
     (compiled-code-emit0oi! ccb CLOSURE code info))
   0)
 
+(define (pass5/$CLAMBDA ifrom ccb renv ctx)
+  0)
+
 (define (pass5/lambda iform ccb renv)
   (let* ([inliner (let1 v ($lambda-flag iform)
                    (and (vector? v) v))]
@@ -602,7 +605,7 @@
                                           ccb  ; parent
                                           inliner)])
     (compiled-code-attach-source-info! ccb ($lambda-src iform))
-    
+
     ;; If any of procedure parameters are set!, we should box it
     ;; upon entering the procedure.
     (let loop ([lvs ($lambda-lvars iform)]
@@ -612,11 +615,12 @@
           (compiled-code-emit1i! ccb BOX k (lvar-name (car lvs))))
         (loop (cdr lvs) (- k 1))))
     ;; Save list of unused arguments in the attributes of (car signature-info).
-    (pair-attribute-set! (car (slot-ref ccb 'signature-info))
-                         'unused-args
-                         (filter-map (^[lv] (and (zero? (lvar-ref-count lv))
-                                                 (lvar-name lv)))
-                                     ($lambda-lvars iform)))
+    (let1 uargs (filter-map (^[lv] (and (zero? (lvar-ref-count lv))
+                                        (lvar-name lv)))
+                            ($lambda-lvars iform))
+      (unless (null? uargs)
+        (pair-attribute-set! (car (slot-ref ccb 'signature-info))
+                             'unused-args uargs)))
     ;; Run pass5 on body
     (pass5 ($lambda-body iform)
            ccb
@@ -1168,4 +1172,3 @@
         (unless (lvar-immutable? (car lvars))
           (compiled-code-emit1i! ccb BOX (- envsize k) (lvar-name (car lvars))))
         (loop (cdr lvars) (+ k 1))))))
-
