@@ -1,7 +1,7 @@
 ;;;
 ;;; compile.scm - The compiler
 ;;;
-;;;   Copyright (c) 2004-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -40,13 +40,13 @@
 
 (inline-stub
  (declcode
-  (.include <gauche/class.h>
-            <gauche/code.h>
+  (.include <gauche/code.h>
             <gauche/vminsn.h>
-            <gauche/priv/macroP.h>
-            <gauche/priv/moduleP.h>
+            <gauche/priv/builtin-syms.h>
+            <gauche/priv/glocP.h>
             <gauche/priv/identifierP.h>
-            <gauche/priv/builtin-syms.h>)))
+            <gauche/priv/macroP.h>
+            <gauche/priv/moduleP.h>)))
 
 ;;; THE COMPILER
 ;;;
@@ -72,7 +72,7 @@
 ;;;     - Global constant variables are substituted to its value.
 ;;;     - Variable bindings are resolved.  Local variables are marked
 ;;;       according to its usage (# of reference count and set count).
-;;;     - Constant expressons are folded.
+;;;     - Constant expressions are folded.
 ;;;
 ;;;   Pass 2 (Variable and closure optimization):
 ;;;     - Traverses IForm and modify the tree to optimize it.
@@ -115,7 +115,7 @@
 ;; Compile-time constants
 ;;
 
-;; Used by env-lookup-int.  the frame is marked as either LEXICAL or SYNTAX.
+;; A local frame is marked as either LEXICAL or SYNTAX in its car.
 (eval-when (:compile-toplevel)
   (define-constant LEXICAL 0)
   (define-constant SYNTAX  1))
@@ -152,6 +152,7 @@
   $LET
   $RECEIVE
   $LAMBDA
+  $CLAMBDA
   $LABEL
   $SEQ
   $CALL
@@ -174,13 +175,18 @@
 (eval-when (:compile-toplevel)
   (use gauche.vm.insn)
   (define-macro (define-insn-constants)
-    (let1 name&codes
-        (map (^[insn] (cons (car insn) (ref (cdr insn)'code)))
-             (class-slot-ref <vm-insn-info> 'all-insns))
+    (let ([name&codes
+           (map (^[insn] (cons (car insn) (ref (cdr insn)'code)))
+                (class-slot-ref <vm-insn-info> 'all-insns))]
+          [mv-insn-codes
+           (filter-map (^[insn] (and (~ (cdr insn)'multi-value)
+                                     (~ (cdr insn)'code)))
+                       (class-slot-ref <vm-insn-info> 'all-insns))])
       `(begin
          ,@(map (^[n&c] `(define-constant ,(car n&c) ,(cdr n&c)))
                 name&codes)
          (define-constant .insn-alist. ',name&codes)
+         (define-constant .insn-multi-value-codes. ',mv-insn-codes)
          )))
   (define-insn-constants)
   )
@@ -188,9 +194,9 @@
 ;; Maximum size of $LAMBDA node we allow to duplicate and inline.
 (define-constant SMALL_LAMBDA_SIZE 12)
 
-(define-inline (variable? arg) (or (symbol? arg) (identifier? arg)))
+;; In case keyword is disjoint from symbols
 (define-inline (variable-or-keyword? arg)
-  (or (symbol? arg) (keyword? arg) (identifier? arg)))
+  (or (identifier? arg) (keyword? arg)))
 
 ;;============================================================
 ;; Data structures
@@ -201,7 +207,7 @@
 ;;   Slots:
 ;;     name  - name of the variable (symbol)
 ;;     initval   - initialized value (Maybe IForm)
-;;     ref-count - in how many places this variable is referefnced?
+;;     ref-count - in how many places this variable is referenced?
 ;;     set-count - in how many places this variable is set!
 ;;
 
@@ -280,7 +286,7 @@
 ;;                local binding).   This slot may be #f.
 ;;
 ;;     current-proc - Holds the information of the current
-;;                compilig procedure.  It accumulates information needed
+;;                compiling procedure.  It accumulates information needed
 ;;                in later stages for the optimization.  This slot may
 ;;                be #f.
 ;;
@@ -292,22 +298,18 @@
 (define (make-cenv module :optional (frames '()) (exp-name #f))
   (%make-cenv module frames exp-name))
 
-;; Some cenv-related proceduers are in C for better performance.
+;; Some cenv-related procedures are in C for better performance.
 (inline-stub
- ;; env-lookup-int :: Name, LookupAs, Module, [Frame] -> Var
+ ;; env-lookup-int :: Name, Module, [Frame] -> Var
  ;;         where Var = Lvar | Identifier | Macro
  ;;
  ;;  PERFORMANCE KLUDGE:
  ;;     - We assume the frame structure is well-formed, so skip some tests.
- ;;     - We assume 'lookupAs' and the car of each frame are small non-negative
- ;;       integers, so we directly compare them without unboxing them.
  ;;
- (define-cfn env-lookup-int (name lookup-as module::ScmModule* frames) :static
+ (define-cfn env-lookup-int (name module::ScmModule* frames) :static
    (let* ([y name])
      (while 1
        (dopairs [fp1 frames]
-         (when (> (SCM_CAAR fp1) lookup-as) ; see PERFORMANCE KLUDGE above
-           (continue))
          ;; inline assq here to squeeze performance.
          (dolist [vp (SCM_CDAR fp1)]
            (when (SCM_EQ y (SCM_CAR vp)) (return (SCM_CDR vp)))))
@@ -327,18 +329,11 @@
 
  ;; Internal API - used while macro expansion
  (define-cproc env-lookup (name module frames)
-   (return (env-lookup-int name (SCM_MAKE_INT 1) ;; SYNTAX
-                           (SCM_MODULE module) frames)))
+   (return (env-lookup-int name (SCM_MODULE module) frames)))
  ;; Internal API - for faster CENV lookup
- (define-cproc cenv-lookup-syntax (cenv name)
+ (define-cproc cenv-lookup (cenv name)
    (return
-    (env-lookup-int name (SCM_MAKE_INT 1)                      ; SYNTAX
-                    (SCM_MODULE (SCM_VECTOR_ELEMENT cenv 0))   ; module
-                    (SCM_VECTOR_ELEMENT cenv 1))))             ; frames
- ;; Internal API - for faster CENV lookup
- (define-cproc cenv-lookup-variable (cenv name)
-   (return
-    (env-lookup-int name (SCM_MAKE_INT 0)                      ; LEXICAL
+    (env-lookup-int name
                     (SCM_MODULE (SCM_VECTOR_ELEMENT cenv 0))   ; module
                     (SCM_VECTOR_ELEMENT cenv 1))))             ; frames
 
@@ -362,10 +357,10 @@
                ,(get-keyword :current-proc kvs `(cenv-current-proc ,cenv))
                ,(get-keyword :source-path kvs `(cenv-source-path ,cenv))))
 
-(define-macro (make-bottom-cenv . maybe-module)
+(define-inline (make-bottom-cenv . maybe-module)
   (if (null? maybe-module)
-    `(%make-cenv (vm-current-module) '())
-    `(%make-cenv ,(car maybe-module) '())))
+    (%make-cenv (vm-current-module) '())
+    (%make-cenv (car maybe-module) '())))
 
 (define-inline (cenv-swap-module cenv mod)
   (cenv-copy-except cenv :module mod))
@@ -421,6 +416,7 @@
 ;;                           ;; local binding (mv)
 ;;    #($lambda <src> <name> <reqarg> <optarg> (<lvar> ...) <iform> <flag>)
 ;;                           ;; closure
+;;    #($clambda <src> <name> <lambda-node> ...)
 ;;    #($label <src> <label> <iform>) ;; merge point of local call.  see below.
 ;;    #($seq (<iform> ...))   ;; sequencing
 ;;    #($call <src> <proc-expr> (<arg-expr> ...) <flag>) ;; procedure call
@@ -452,12 +448,10 @@
 ;;      side-effects may vary depends on the path to the node.  The only
 ;;      exception is $label node.
 
-(define-macro (iform-tag iform)
-  `(vector-ref ,iform 0))
+(define-inline (iform-tag iform) (vector-ref iform 0))
 
 ;; check intermediate tag
-(define-macro (has-tag? iform tag)
-  `(eqv? (vector-ref ,iform 0) ,tag))
+(define-inline (has-tag? iform tag) (eqv? (vector-ref iform 0) tag))
 
 ;; [IForm Definitions]
 
@@ -532,7 +526,7 @@
 ;;   by this node (let* is expanded to the nested $let in pass 1).
 (define-simple-struct $let $LET $let
   (src       ; original source for debugging
-   type      ; indicates scope: 'let for normal let, 'rec[*] for letrec[*], 
+   type      ; indicates scope: 'let for normal let, 'rec[*] for letrec[*],
    lvars     ; list of lvars
    inits     ; list of IForms to initialize lvars
    body      ; IForm for the body
@@ -549,7 +543,7 @@
    body      ; IForm for the body
    ))
 
-;; $lambda <src> <reqargs> <optarg> <lvars> <body> [<flag>]
+;; $lambda <src> <name> <reqargs> <optarg> <lvars> <body> [<flag>]
 ;;   Closure.
 ;;   $lambda has a couple of transient slots, which are used only
 ;;   during the optimization paths and not be saved by pack-iform.
@@ -577,6 +571,13 @@
                     ; is to be bound.  See pass 4.
    ))
 
+;; $clambda <src> <name> <lambda-node> ...
+;;   Case-lambda.
+(define-simple-struct $clambda $CLAMBDA $clambda
+  (src
+   name
+   closures))
+
 ;; $label <src> <label> <body>
 ;;    This kind of IForm node is introduced in Pass2 to record a shared
 ;;    node.  It marks the destination of LOCAL-ENV-JUMP, and also is
@@ -588,7 +589,7 @@
    ))
 
 ;; $seq <body>
-;;    Sequensing.  <body> is a list of IForms.
+;;    Sequencing.  <body> is a list of IForms.
 ;;    The compile tries to avoid creating $seq node if <body> has only
 ;;    one expression, but optimization paths may introduce such a $seq node.
 ;;    There can also be an empty $seq node, ($seq '()).
@@ -660,13 +661,13 @@
 (define-simple-struct $list->vector $LIST->VECTOR $list->vector (src arg0))
 
 ;; common accessors
-(define-macro ($*-src  iform)  `(vector-ref ,iform 1))
-(define-macro ($*-args iform)  `(vector-ref ,iform 2))
-(define-macro ($*-arg0 iform)  `(vector-ref ,iform 2))
-(define-macro ($*-arg1 iform)  `(vector-ref ,iform 3))
-(define-macro ($*-args-set! iform val)  `(vector-set! ,iform 2 ,val))
-(define-macro ($*-arg0-set! iform val)  `(vector-set! ,iform 2 ,val))
-(define-macro ($*-arg1-set! iform val)  `(vector-set! ,iform 3 ,val))
+(define-inline ($*-src  iform)  (vector-ref iform 1))
+(define-inline ($*-args iform)  (vector-ref iform 2))
+(define-inline ($*-arg0 iform)  (vector-ref iform 2))
+(define-inline ($*-arg1 iform)  (vector-ref iform 3))
+(define-inline ($*-args-set! iform val)  (vector-set! iform 2 val))
+(define-inline ($*-arg0-set! iform val)  (vector-set! iform 2 val))
+(define-inline ($*-arg1-set! iform val)  (vector-set! iform 3 val))
 
 ;; look up symbolic name of iform tag (for debugging)
 (define (iform-tag-name tag)
@@ -684,6 +685,9 @@
           [(eqv? (cdar p) code) (caar p)]
           [else (loop (cdr p))])))
 
+(define (insn-multi-value? code)
+  (memv code .insn-multi-value-codes.))
+
 ;; prettyprinter of intermediate form
 (define (pp-iform iform :optional (lines +inf.0))
   (let/cc return
@@ -739,8 +743,9 @@
         (nl (+ ind 4))
         (rec (+ ind 4) ($receive-expr iform)) (nl (+ ind 2))
         (rec (+ ind 2) ($receive-body iform)) (display ")")]
-       [($LAMBDA) (format #t "($lambda[~a.~a~a] ~a" ($lambda-name iform)
+       [($LAMBDA) (format #t "($lambda[~a.~a~a~a] ~a" ($lambda-name iform)
                           (length ($lambda-calls iform))
+                          (if (null? ($lambda-free-lvars iform)) "" "c")
                           (if (vector? ($lambda-flag iform)) " inlinable" "")
                           (map lvar->string ($lambda-lvars iform)))
         (nl (+ ind 2))
@@ -950,7 +955,7 @@
        [($LIST*)  ($list* (V i 1) (map unpack-rec (V i 2)))]
        [($LIST->VECTOR) ($list->vector (V i 1) (unpack-rec (V i 2)))]
        [(lvar)    (make-lvar (V i 1))]
-       [else 
+       [else
         (errorf "[internal error] unpack-iform: ivec broken at ~a (tag=~s): ~S"
                 i (V i) ivec)]))
 
@@ -1186,7 +1191,7 @@
 ;; won't change even if we move iform to a different place in the subtree.
 ;; NB: This may be called after pass2, so $LABEL node may have circular
 ;; reference.  We have to be careful not to diverge.
-(define (transparent? iform) 
+(define (transparent? iform)
   (define (side-effect-free-proc? iform) #f) ;for now
 
   ;; This is used when eliminating dead $ASM node.  We don't need to consider
@@ -1373,7 +1378,6 @@
             (imap (cut subst <> mapping dict) ($*-args iform))))
   (subst iform mapping (make-hash-table 'eq?)))
 
-;; EXPERIMENTAL
 ;; Convert IForm back to S-expr.
 ;; This is used by macroexpand-all.  Note that pass2 can make IForm DG, and
 ;; it's not generally possible to convert it back to S-expr.  You can only
@@ -1383,7 +1387,7 @@
   (define (get-lvar lvar)
     (or (hash-table-get lvar-dict lvar #f)
         (let* ([name (lvar-name lvar)]
-               [name (if (identifier? name) (identifier->symbol name) name)])
+               [name (identifier->symbol name)])
           (rlet1 s ($ string->symbol $ format "~a.~d" name
                       $ hash-table-num-entries lvar-dict)
             (hash-table-put! lvar-dict lvar s)))))
@@ -1396,7 +1400,12 @@
                        ,(rec ($lset-expr iform)))]
      [($GREF)   ($gref-id iform)]
      [($GSET)   `(set! ,($gset-id iform) ,(rec ($gset-expr iform)))]
-     [($CONST)  `',($const-value iform)]
+     [($CONST)  (let1 v ($const-value iform)
+                  ;; This is just for better readability
+                  (if (or (number? v) (string? v) (char? v) (boolean? v)
+                          (undefined? v) (port? v) (procedure? v))
+                    v
+                    `',v))]
      [($IF)     (cond [(has-tag? ($if-then iform) $IT)
                        `(or ,(rec ($if-test iform))
                             ,(rec ($if-else iform)))]
@@ -1495,6 +1504,7 @@
 (include "compile-4")
 (include "compile-5")
 (include "compile-i")
+(include "compile-t")
 
 ;;============================================================
 ;; Macro support basis
@@ -1511,10 +1521,10 @@
    (unless (and (SCM_IDENTIFIERP id1) (SCM_IDENTIFIERP id2)) (return FALSE))
    (when (SCM_EQ id1 id2) (return TRUE))
    ;; Look at their binidngs
-   (let* ([b1 (env-lookup-int id1 (SCM_MAKE_INT 1) 
+   (let* ([b1 (env-lookup-int id1
                               (-> (SCM_IDENTIFIER id1) module)
                               (Scm_IdentifierEnv (SCM_IDENTIFIER id1)))]
-          [b2 (env-lookup-int id2 (SCM_MAKE_INT 1) 
+          [b2 (env-lookup-int id2
                               (-> (SCM_IDENTIFIER id2) module)
                               (Scm_IdentifierEnv (SCM_IDENTIFIER id2)))])
      (if (and (SCM_IDENTIFIERP b1) (SCM_IDENTIFIERP b2))
@@ -1526,11 +1536,12 @@
          ;; If at least either one is unbound, we just compare their names.
          (if (and g1 g2)
            (return (SCM_EQ g1 g2))
-           (return (SCM_EQ (Scm_UnwrapSyntax id1) (Scm_UnwrapSyntax id2)))))
+           (return (SCM_EQ (Scm_UnwrapSyntax2 id1 FALSE)
+                           (Scm_UnwrapSyntax2 id2 FALSE)))))
        ;; At least one of id1 and id2 is lvar or macro.
        (return (SCM_EQ b1 b2)))))
 
- (define-cproc free-identifier=? (id1 id2) ::<boolean> 
+ (define-cproc free-identifier=? (id1 id2) ::<boolean>
    %free-identifier=?)
  )
 
@@ -1543,7 +1554,7 @@
 ;; identifier for those occurrences.
 (define (er-rename form dict module env)
   (cond
-   [(variable? form)
+   [(identifier? form)
     (if-let1 id (assq-ref dict form)
       (values id dict)
       (let1 id (make-identifier form module env)
@@ -1575,8 +1586,8 @@
  (define-cfn Scm__ERCompare (a b module::ScmModule* frames) ::int
    (cond [(and (or (SCM_SYMBOLP a) (SCM_IDENTIFIERP a))
                (or (SCM_SYMBOLP b) (SCM_IDENTIFIERP b)))
-          (let* [(a1 (env-lookup-int a (SCM_MAKE_INT 1) module frames))
-                 (b1 (env-lookup-int b (SCM_MAKE_INT 1) module frames))]
+          (let* [(a1 (env-lookup-int a module frames))
+                 (b1 (env-lookup-int b module frames))]
             (when (SCM_EQ a1 b1) (return TRUE))
             (unless (and (SCM_IDENTIFIERP a1) (SCM_IDENTIFIERP b1)) (return FALSE))
             (return (%free-identifier=? a1 b1)))]
@@ -1588,27 +1599,47 @@
    Scm__ERCompare)
  )
 
-;; xformer :: (Sexpr, (Sym -> Sym), (Sym, Sym -> Bool)) -> Sexpr
-(define (%make-er-transformer xformer def-env)
+;; xformer  :: (Sexpr, (Sym -> Sym), (Sym, Sym -> Bool)) -> Sexpr
+;; xformer+ :: (Sexpr, (Sym -> Sym), (Sym, Sym -> Bool), (Sym -> Sym)) -> Sexpr
+(define (%make-er-transformer xformer def-env
+                              :key (has-inject? #f)
+                                   (info-alist '())
+                                   (identifier-macro? #f))
   (define def-module (cenv-module def-env))
   (define def-frames (cenv-frames def-env))
   (define (expand form use-env)
     (define use-module (cenv-module use-env))
     (define use-frames (cenv-frames use-env))
     (let1 dict '()
-      (xformer form
-               (^[sym]
-                 (receive [id dict_] (er-rename sym dict def-module def-frames)
-                   (set! dict dict_)
-                   id))
-               (^[a b] (er-compare a b use-module use-frames)))))
-  (%make-macro-transformer (cenv-exp-name def-env) expand))
+      (define (%rename sym)
+        (receive [id dict_] (er-rename sym dict def-module def-frames)
+          (set! dict dict_)
+          id))
+      (define (%compare a b) (er-compare a b use-module use-frames))
+      (define (%inject sym)
+        (receive [id dict_] (er-rename sym dict use-module use-frames)
+          (set! dict dict_)
+          id))
+      (if has-inject?
+        (xformer form %rename %compare %inject)
+        (xformer form %rename %compare))))
+  (%make-macro-transformer (cenv-exp-name def-env) expand info-alist
+                           (if identifier-macro?
+                             '(identifier-macro)
+                             '())))
 
 ;; Call to this procedure can be inserted in the output of er-macro expander
 ;; and emitted in the precompiled file, so do not change this API unless
 ;; you're sure no existing precompiled code refers to this.
-(define (%make-er-transformer/toplevel xformer def-module def-name)
-  (%make-er-transformer xformer (%make-cenv def-module '() def-name)))
+(define (%make-er-transformer/toplevel xformer def-module def-name
+                                       :key (has-inject? #f)
+                                            (info-alist '())
+                                            (identifier-macro? #f))
+  (%make-er-transformer xformer
+                        (%make-cenv def-module '() def-name)
+                        :has-inject? has-inject?
+                        :info-alist info-alist
+                        :identifier-macro? identifier-macro?))
 
 ;; Returns an S-expr all macros in which are expanded.
 ;; The resulting form may not be equivalent to the input form, though,
@@ -1623,25 +1654,23 @@
               [else (error "a boolean or a module required but got:" env)])])
     (unwind-protect
         (begin
-          ;; TODO: We suppress global inline expansion, otherwise we'll see
-          ;; $ASM nodes.  NB: This also suppress expansion of define-inline'd
-          ;; procedures and compiler macros, which may not be desirable.
-          ;; We'll think of it later.
-          (vm-compiler-flag-set! SCM_COMPILE_NOINLINE_GLOBALS)
+          ;; Suppress inline expansion to ASM node, for it is useless as
+          ;; the macro output.
+          (vm-compiler-flag-set! SCM_COMPILE_NOINLINE_INLINER)
           ($ unwrap-syntax $ iform->sexpr
              $ pass1 form (make-bottom-cenv env)))
       (begin
-        (vm-compiler-flag-clear! SCM_COMPILE_NOINLINE_GLOBALS)
+        (vm-compiler-flag-clear! SCM_COMPILE_NOINLINE_INLINER)
         (vm-compiler-flag-set! flags-save)))))
 
 ;;--------------------------------------------------------
 ;; Customizable inliner interface
 ;;
 
-;; This is invoked from the expansion of define-inline/syntax.
+;; This is invoked from the expansion of define-hybrid-syntax.
 (define (%with-inline-transformer proc xformer)
   (unless (procedure? proc)
-    (error "define-inline/syntax: expression yields non-procedure value" proc))
+    (error "define-hybrid-syntax: expression yields non-procedure value" proc))
   (unless (macro? xformer)
     (errorf "macro required, but got" xformer))
   ;; Copy PROC, in case the given proc refers to already defined global
@@ -1677,8 +1706,7 @@
        (<= 0 obj #x7ffff)))
 
 (define (variable-name arg)
-  (cond [(symbol? arg) arg]
-        [(identifier? arg) (unwrap-syntax arg)]
+  (cond [(identifier? arg) (identifier->symbol arg)]
         [(lvar? arg) (lvar-name arg)]
         [else (error "variable required, but got:" arg)]))
 
@@ -1701,7 +1729,8 @@
 ;;
 ;;   The first value is #f if this global call would be a simple call.
 ;;   Otherwise, the first value is a bound value of the global variable,
-;;   and the second value is one of the symbols 'macro, 'syntax or 'inline.
+;;   and the second value is one of the symbols 'macro, 'syntax, 'inline,
+;;   or 'type-ctor.
 ;;
 ;;   We write this in C because it is in critical path of the compiler.
 ;;
@@ -1739,13 +1768,27 @@
                 (set! SCM_RESULT0 gval SCM_RESULT1 'macro)]
                [(SCM_SYNTAXP gval)
                 (set! SCM_RESULT0 gval SCM_RESULT1 'syntax)]
-               [(and (SCM_PROCEDUREP gval)
-                     (SCM_PROCEDURE_INLINER gval) ; inliner may be NULL
-                     (not (SCM_FALSEP (SCM_PROCEDURE_INLINER gval)))
-                     (Scm_GlocInlinableP gloc)
-                     (not (SCM_VM_COMPILER_FLAG_IS_SET
-                           (Scm_VM) SCM_COMPILE_NOINLINE_GLOBALS)))
-                (set! SCM_RESULT0 gval SCM_RESULT1 'inline)]
+               [(SCM_PROCEDUREP gval)
+                (let* ([inl (SCM_PROCEDURE_INLINER gval)])
+                  (if (and inl          ; inliner may be NULL
+                           (not (SCM_FALSEP inl))
+                           (Scm_GlocInlinableP gloc)
+                           (not (SCM_VM_COMPILER_FLAG_IS_SET
+                                 (Scm_VM) SCM_COMPILE_NOINLINE_GLOBALS))
+                           ;; When SCM_COMPILE_NOINLINE_INLINER is set, we
+                           ;; suppress inliner expansion except inl is a macro
+                           ;; xformer (define-hybrid-syntax creates it).
+                           ;; It is only for macroexpand-all; other inliners
+                           ;; may expand into $asm node, which isn't useful
+                           ;; as the output of macro expansion.
+                           (not (and (SCM_VM_COMPILER_FLAG_IS_SET
+                                      (Scm_VM) SCM_COMPILE_NOINLINE_INLINER)
+                                     (not (SCM_MACROP inl)))))
+                    (set! SCM_RESULT0 gval SCM_RESULT1 'inline)
+                    (goto normal)))]
+               [(and (Scm_TypeConstructorP gval)
+                     (Scm_GlocInlinableP gloc))
+                (set! SCM_RESULT0 gval SCM_RESULT1 'type-ctor)]
                [else (goto normal)])
          (.if "defined(RECORD_DEPENDED_MODULES)"
               (begin
@@ -1768,12 +1811,12 @@
 ;; TODO: We should also check iff k isn't bound to something else!
 (define (keyword-like? k)
   (or (keyword? k)
-      (and (identifier? k) (keyword? (identifier->symbol k)))))
+      (and (wrapped-identifier? k) (keyword? (identifier->symbol k)))))
 
 (define (global-eq? var id cenv)  ; like free-identifier=?, used in pass1.
-  (and (variable? var)
-       (let1 v (cenv-lookup-variable cenv var)
-         (and (identifier? v)
+  (and (identifier? var)
+       (let1 v (cenv-lookup cenv var)
+         (and (wrapped-identifier? v)
               (global-identifier=? v id)))))
 
 ;; TRANSIENT: To compare keyword hygienically.  We need to treat keywords
@@ -1781,7 +1824,7 @@
 (define (global-keyword-eq? var key cenv)
   (or (eq? var key)
       (and (symbol? key)
-           (identifier? var)
+           (wrapped-identifier? var)
            (global-eq? var (global-id key) cenv))))
 
 (define (everyc proc lis c)             ;avoid closure allocation
@@ -1806,7 +1849,7 @@
 (define (global-eq?? sym srcmod modgen)
   (let1 id-gloc (find-binding (find-module srcmod) sym #f)
     (^[var] (eq? id-gloc
-                 (if (identifier? var)
+                 (if (wrapped-identifier? var)
                    (find-binding (slot-ref var'module) (slot-ref var'name) #f)
                    (find-binding (modgen) var #f))))))
 
@@ -1908,7 +1951,7 @@
  (define-cproc vm-compiler-flag-clear! (flag::<uint>) ::<void>
    (SCM_VM_COMPILER_FLAG_CLEAR (Scm_VM) flag))
  (define-cproc vm-compiler-flag () ::<uint>
-   (result (-> (Scm_VM) compilerFlags)))
+   (return (-> (Scm_VM) compilerFlags)))
 
  (define-cproc vm-compiler-flag-noinline-locals? () ::<boolean>
    (return (SCM_VM_COMPILER_FLAG_IS_SET (Scm_VM) SCM_COMPILE_NOINLINE_LOCALS)))
@@ -1921,16 +1964,18 @@
  (define-enum SCM_COMPILE_NOINLINE_LOCALS)
  (define-enum SCM_COMPILE_NOINLINE_CONSTS)
  (define-enum SCM_COMPILE_NOINLINE_SETTERS)
+ (define-enum SCM_COMPILE_NOINLINE_INLINER)
  (define-enum SCM_COMPILE_NOSOURCE)
  (define-enum SCM_COMPILE_SHOWRESULT)
  (define-enum SCM_COMPILE_NOCOMBINE)
  (define-enum SCM_COMPILE_NO_POST_INLINE_OPT)
  (define-enum SCM_COMPILE_NO_LIFTING)
  (define-enum SCM_COMPILE_INCLUDE_VERBOSE)
- (define-enum SCM_COMPILE_ENABLE_CEXPR)
  (define-enum SCM_COMPILE_NOINLINE_SETTERS)
  (define-enum SCM_COMPILE_NODISSOLVE_APPLY)
  (define-enum SCM_COMPILE_LEGACY_DEFINE)
+ (define-enum SCM_COMPILE_MUTABLE_LITERALS)
+ (define-enum SCM_COMPILE_SRFI_FEATURE_ID)
 
  ;; Set/get VM's current module info. (temporary)
  (define-cproc vm-current-module () (return (SCM_OBJ (-> (Scm_VM) module))))
@@ -1950,33 +1995,31 @@
 ;; Dummy macros
 ;;
 
-(define (%precomp-only name)
-  (warn "The ~a form can only be used for Scheme sources \
-         to be pre-compiled.  Since you're loading the file without \
-         pre-compilation, this form is ignored (Current module=~s)"
-        name (current-module))
-  (undefined))
-
 (select-module gauche)
 
-(declare (keep-private-macro inline-stub define-cproc declare))
+(define-macro (%precomp-only . names)
+  `(begin
+     (declare (keep-private-macro ,@names))
+     ,@(map (^[name]
+              `(define-macro (,name . _)
+                 (warn "The ~a form can only be used for Scheme sources \
+                        to be pre-compiled.  Since you're loading the file \
+                        ~s without pre-compilation, this form is ignored.\n"
+                       ',name (current-load-path))
+                 (undefined)))
+            names)))
 
-;; Those forms are recognized by the AOT compiler (precomp), and
+;; Some forms are recognized by the AOT compiler (precomp), and
 ;; translated to C.  They can't be evaluated at runtime, so
 ;; we have dummy macros to warn.
-(define-macro (inline-stub . _)
-  ((with-module gauche.internal %precomp-only) 'inline-stub))
-(define-macro (define-cproc . _)
-  ((with-module gauche.internal %precomp-only) 'define-cproc))
-(define-macro (define-cfn . _)
-  ((with-module gauche.internal %precomp-only) 'define-cfn))
-(define-macro (define-enum . _)
-  ((with-module gauche.internal %precomp-only) 'define-enum))
-(define-macro (define-enum-conditionally . _)
-  ((with-module gauche.internal %precomp-only) 'define-enum-conditionally))
-
+(%precomp-only inline-stub
+               define-cproc
+               define-cfn
+               define-enum
+               define-enum-conditionally)
 
 ;; The form (declare ...) may be used in wider purpose.  For the time
 ;; being we use it in limited purposes for compilers.  In interpreter
 ;; we just ignore it.
+(declare (keep-private-macro declare))
 (define-macro (declare . _) #f)
