@@ -1,7 +1,7 @@
 /*
  * core.c - core kernel interface
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -69,6 +69,8 @@ extern void Scm__InitSymbol(void);
 extern void Scm__InitNumber(void);
 extern void Scm__InitChar(void);
 extern void Scm__InitClass(void);
+extern void Scm__InitMemoTable(void);
+extern void Scm__InitList(void);
 extern void Scm__InitExceptions(void);
 extern void Scm__InitPort(void);
 extern void Scm__InitWrite(void);
@@ -85,9 +87,12 @@ extern void Scm__InitVM(void);
 extern void Scm__InitAutoloads(void);
 extern void Scm__InitCollection(void);
 extern void Scm__InitComparator(void);
+extern void Scm__InitExecenv(void);
+extern void Scm__InitNative(void);
 
 extern void Scm_Init_libalpha(void);
 extern void Scm_Init_libbool(void);
+extern void Scm_Init_libbox(void);
 extern void Scm_Init_libchar(void);
 extern void Scm_Init_libcode(void);
 extern void Scm_Init_libcmp(void);
@@ -98,8 +103,10 @@ extern void Scm_Init_libfmt(void);
 extern void Scm_Init_libio(void);
 extern void Scm_Init_liblazy(void);
 extern void Scm_Init_liblist(void);
+extern void Scm_Init_libmemo(void);
 extern void Scm_Init_libmisc(void);
 extern void Scm_Init_libmod(void);
+extern void Scm_Init_libnative(void);
 extern void Scm_Init_libnum(void);
 extern void Scm_Init_libobj(void);
 extern void Scm_Init_libproc(void);
@@ -108,18 +115,24 @@ extern void Scm_Init_libsrfis(void);
 extern void Scm_Init_libstr(void);
 extern void Scm_Init_libsym(void);
 extern void Scm_Init_libsys(void);
+extern void Scm_Init_libtype(void);
 extern void Scm_Init_libvec(void);
 extern void Scm_Init_libmacbase(void);
 extern void Scm_Init_compile(void);
 extern void Scm_Init_libmacro(void);
+extern void Scm_Init_libparam(void);
 extern void Scm_Init_libomega(void);
 
+extern void Scm__FinishModuleInitialization(void);
+
 static void finalizable(void);
 static void init_cond_features(void);
 
 #ifdef GAUCHE_USE_PTHREADS
 /* a trick to make sure the gc thread object is linked */
-static int (*ptr_pthread_create)(void) = NULL;
+static int (*ptr_pthread_create)(pthread_t *,
+                                 GC_PTHREAD_CREATE_CONST pthread_attr_t *,
+                                 void *(*)(void *), void * /* arg */) = NULL;
 #endif
 
 /* flag to see if Scheme infrastructure is fully initialized or not */
@@ -131,7 +144,17 @@ static int scheme_initialized = FALSE;
 void Scm_Init(const char *signature)
 {
     if (scheme_initialized) return;
-    
+
+    int debug_init = FALSE;
+    if (Scm_GetEnv("GAUCHE_DEBUG_INITIALIZATION") != NULL) {
+        debug_init = TRUE;
+    }
+#define CALL_INIT(f)                                            \
+    do {                                                        \
+        if (debug_init) fprintf(stderr, "Calling %s...\n", #f); \
+        f();                                                    \
+    } while (0)
+
     /* make sure the main program links the same version of libgauche */
     if (strcmp(signature, GAUCHE_SIGNATURE) != 0) {
         Scm_Panic("libgauche ABI version mismatch: libgauche %s, expected %s",
@@ -153,70 +176,82 @@ void Scm_Init(const char *signature)
 
     /* Initialize components.  The order is important, for some components
        rely on the other components to be initialized. */
-    Scm__InitParameter();
-    Scm__InitVM();
-    Scm__InitHash();
-    Scm__InitSymbol();
-    Scm__InitModule();
-    Scm__InitNumber();
-    Scm__InitChar();
-    Scm__InitClass();
-    Scm__InitCollection();
-    Scm__InitExceptions();
-    Scm__InitProc();
-    Scm__InitPort();
-    Scm__InitWrite();
-    Scm__InitMacro();
-    Scm__InitLoad();
-    Scm__InitRegexp();
-    Scm__InitRead();
-    Scm__InitSignal();
-    Scm__InitSystem();
-    Scm__InitComparator();
+    CALL_INIT(Scm__InitParameter);
+    CALL_INIT(Scm__InitVM);
+    CALL_INIT(Scm__InitHash);
+    CALL_INIT(Scm__InitSymbol);
+    CALL_INIT(Scm__InitModule);
+    CALL_INIT(Scm__InitNumber);
+    CALL_INIT(Scm__InitChar);
+    CALL_INIT(Scm__InitClass);
+    CALL_INIT(Scm__InitMemoTable);
+    CALL_INIT(Scm__InitList);
+    CALL_INIT(Scm__InitCollection);
+    CALL_INIT(Scm__InitExceptions);
+    CALL_INIT(Scm__InitProc);
+    CALL_INIT(Scm__InitPort);
+    CALL_INIT(Scm__InitWrite);
+    CALL_INIT(Scm__InitMacro);
+    CALL_INIT(Scm__InitLoad);
+    CALL_INIT(Scm__InitRegexp);
+    CALL_INIT(Scm__InitRead);
+    CALL_INIT(Scm__InitSignal);
+    CALL_INIT(Scm__InitSystem);
+    CALL_INIT(Scm__InitComparator);
+    CALL_INIT(Scm__InitExecenv);
+    CALL_INIT(Scm__InitNative);
 
-    Scm_Init_libalpha();
-    Scm_Init_libbool();
-    Scm_Init_libchar();
-    Scm_Init_libcode();
-    Scm_Init_libcmp();
-    Scm_Init_libdict();
-    Scm_Init_libeval();
-    Scm_Init_libexc();
-    Scm_Init_libfmt();
-    Scm_Init_libio();
-    Scm_Init_liblazy();
-    Scm_Init_liblist();
-    Scm_Init_libmisc();
-    Scm_Init_libmod();
-    Scm_Init_libnum();
-    Scm_Init_libobj();
-    Scm_Init_libproc();
-    Scm_Init_librx();
-    Scm_Init_libsrfis();
-    Scm_Init_libstr();
-    Scm_Init_libsym();
-    Scm_Init_libsys();
-    Scm_Init_libvec();
-    Scm_Init_libmacbase();
-    Scm_Init_compile();
-    Scm_Init_libmacro();
-    Scm_Init_libomega();
+    CALL_INIT(Scm_Init_libalpha);
+    CALL_INIT(Scm_Init_libbool);
+    CALL_INIT(Scm_Init_libbox);
+    CALL_INIT(Scm_Init_libchar);
+    CALL_INIT(Scm_Init_libcode);
+    CALL_INIT(Scm_Init_libcmp);
+    CALL_INIT(Scm_Init_libdict);
+    CALL_INIT(Scm_Init_libeval);
+    CALL_INIT(Scm_Init_libexc);
+    CALL_INIT(Scm_Init_libfmt);
+    CALL_INIT(Scm_Init_libio);
+    CALL_INIT(Scm_Init_liblazy);
+    CALL_INIT(Scm_Init_liblist);
+    CALL_INIT(Scm_Init_libmemo);
+    CALL_INIT(Scm_Init_libmisc);
+    CALL_INIT(Scm_Init_libmod);
+    CALL_INIT(Scm_Init_libnative);
+    CALL_INIT(Scm_Init_libnum);
+    CALL_INIT(Scm_Init_libobj);
+    CALL_INIT(Scm_Init_libproc);
+    CALL_INIT(Scm_Init_librx);
+    CALL_INIT(Scm_Init_libsrfis);
+    CALL_INIT(Scm_Init_libstr);
+    CALL_INIT(Scm_Init_libsym);
+    CALL_INIT(Scm_Init_libsys);
+    CALL_INIT(Scm_Init_libvec);
+    CALL_INIT(Scm_Init_libmacbase);
+    CALL_INIT(Scm_Init_compile);
+    /* lib* that uses er-macro-transformer must be initialized
+       after libmacro */
+    CALL_INIT(Scm_Init_libmacro);
+    CALL_INIT(Scm_Init_libparam);
+    CALL_INIT(Scm_Init_libtype);
+    CALL_INIT(Scm_Init_libomega);
 
-    Scm__InitCompaux();
+    CALL_INIT(Scm__InitCompaux);
 
     Scm_SelectModule(Scm_GaucheModule());
-    Scm__InitAutoloads();
+    CALL_INIT(Scm__InitAutoloads);
 
     Scm_SelectModule(Scm_UserModule());
 
     /* Final setup of cond-features alist. */
-    init_cond_features();
+    CALL_INIT(init_cond_features);
 
 #ifdef GAUCHE_USE_PTHREADS
     /* a trick to make sure the gc thread object is linked */
-    ptr_pthread_create = (int (*)(void))GC_pthread_create;
+    ptr_pthread_create = GC_pthread_create;
 #endif
 
+    Scm__FinishModuleInitialization();
     scheme_initialized = TRUE;
 }
 
@@ -502,7 +537,7 @@ init_cond_features()
         /* NB: We should probably make checking 'r7rs' trigger loading
            r7rs module and setting up the environment. */
         { "r7rs", NULL },
-            
+
         /* R7RS Appendix B */
         { "exact-closed", NULL },
         // { "exact-complex", NULL }, /* not yet */
@@ -521,6 +556,9 @@ init_cond_features()
         { "little-endian", NULL }, /* NB: r7rs say nothing on mixed endian */
 #endif
 
+        /* SRFIs that are not libraries */
+        { "srfi-22", NULL },    /* Scheme scripts */
+
         /* Threads */
 #if   defined(GAUCHE_USE_PTHREADS)
         { "gauche.sys.threads", "gauche.threads" },
@@ -560,7 +598,7 @@ init_cond_features()
         { "regexp-unicode", "srfi.115" },
 
 #include "features.c"
-        
+
         { NULL, NULL }
     };
 
@@ -573,6 +611,7 @@ init_cond_features()
    the entries just for ABI compatibility.  These shouldn't be called.
    Remove on 1.0 release.
 */
+#if GAUCHE_API_VERSION < 98
 const char *Scm_GetLibraryDirectory(void (*errfn)(const char *, ...) SCM_UNUSED)
 {
     Scm_Abort("Scm_GetLibraryDirectory is called (it shouldn't).");
@@ -597,21 +636,7 @@ const char *Scm_GetRuntimeDirectory(void (*errfn)(const char *, ...) SCM_UNUSED)
 {
     Scm_Abort("Scm_GetRuntimeDirectory is called (it shouldn't).");
 }
-
-/*=============================================================
- * Command line arguments
- */
-
-ScmObj Scm_InitCommandLine(int argc, const char *argv[])
-{
-    static ScmObj command_line_proc = SCM_UNDEFINED;
-    ScmObj args = Scm_CStringArrayToList(argv, argc, SCM_STRING_IMMUTABLE);
-    SCM_BIND_PROC(command_line_proc, "command-line", Scm_GaucheModule());
-    Scm_ApplyRec1(command_line_proc, args);
-    SCM_DEFINE(Scm_UserModule(), "*program-name*", SCM_CAR(args));
-    SCM_DEFINE(Scm_UserModule(), "*argv*", SCM_CDR(args));
-    return args;
-}
+#endif /*GAUCHE_API_VERSION < 98*/
 
 /*=============================================================
  * 'Main'
@@ -637,7 +662,7 @@ void Scm_SimpleMain(int argc, const char *argv[],
                     const char *script, u_long flags SCM_UNUSED)
 {
     SCM_ASSERT(argc > 0);
-    ScmObj args = Scm_InitCommandLine(argc, argv);
+    ScmObj args = Scm_InitCommandLine2(argc, argv, SCM_COMMAND_LINE_BOTH);
 
     if (script) {
         ScmObj s = SCM_MAKE_STR(script);
