@@ -1,3 +1,4 @@
+;; -*- coding:utf-8 -*-
 ;;;
 ;;;  gen-unicode.scm - generate unicode-handling tables
 ;;;
@@ -14,7 +15,7 @@
 ;;   and the resulting unicode-data.scm is checked in to the source
 ;;   tree so that other developers don't need UCD.
 ;;
-;;    gosh ./gen-unicode.scm --import <unicode-database-directory> unicode-data.scm
+;;    gosh ./gen-unicode.scm --import <unicode-database-directory> unicode-data.scm [unicode-test.scm]
 ;;
 ;; (2) Generate source files from unicode-data.scm
 ;;   To reduce runtime overhead, unicode properties are saved in binary
@@ -22,6 +23,7 @@
 ;;
 ;;   char_attr.c                - General category and case mappings.
 ;;   gauche/priv/unicode_attr.h - Grapheme break, word break, East-asian width
+;;   ext/gauche/unicode-attr.scm - Break property symbols
 ;;
 ;;   This is done when you build from git source tree.
 ;;
@@ -36,13 +38,14 @@
 ;;   You might need to adjust code for newer version of Unicode.
 ;;   See doc/HOWTO-unicode.txt for the details.
 
-(use srfi-1)
+(use scheme.list)
 (use srfi-13)
 (use srfi-42)
 (use text.csv)
 (use util.match)
 (use file.util)
 (use gauche.dictionary)
+(use gauche.generator)
 (use gauche.sequence)
 (use gauche.record)
 (use gauche.charconv)
@@ -153,61 +156,30 @@
 ;;    http://www.unicode.org/Public/UNIDATA/auxiliary/WordBreakProperty.txt
 ;;    http://www.unicode.org/Public/UNIDATA/auxiliary/SentenceBreakProperty.txt
 ;;
-;;    An entry is 8-bit, indicating the character's Word_Break and
-;;    Grapheme_Break properties.  In unicode_attr.h, they are
-;;    prefixed with WB_ and GB_, respectively.
-;;    Note that WB_CR, WB_LF, WB_Single_Quote, WB_Double_Quote,
-;;    GB_CR and GB_LF values are *not* stored in the table.  Each
-;;    property value is assigned to single character and we check
-;;    them separately.
+;;    NB: As of Unicode 13, we also need Extended_Poctographic (ExtPict)
+;;    Emoji property to do clustering properly.  For grapheme break
+;;    properties, all ExtPict codepoints are GB_Other, so we simply name
+;;    them GB_ExtPict.  For word break properties, ExtPict falls on WB_Other
+;;    and WB_ALetter.  We Name the former WB_ExtPict, and the latter
+;;    WB_ExtPict_ALetter.
 ;;
-;;    bit7-4:  Word_Break property
-;;             10    CR     (not stored in the table)
-;;             11    LF     (not stored in the table)
-;;              0    Newline
-;;              1    Extend
-;;              2    Regional_Indicator
-;;              3    Format
-;;              4    Katakana
-;;              5    Hebrew_Letter
-;;              6    ALetter
-;;             12    Single_Quote (not stored in the table)
-;;             13    Double_Quote (not stored in the table)
-;;              7    MidLetter
-;;              8    MidNum
-;;              9    MidNumLet
-;;              a    Numeric
-;;              b    ExtendNumLet
-;;              c    WSegSpace
-;;              d    ZWJ
-;;              e    Other
-;;    bit3-0:  Graphene_Break property
-;;             10    CR (not stored in the table)
-;;             11    LF (not stored in the table)
-;;              0    Control
-;;              1    Extend
-;;              2    Regional_Indicator
-;;              3    Prepend
-;;              4    SpacingMark
-;;              5    L
-;;              6    V
-;;              7    T
-;;              8    LV
-;;              9    LVT
-;;              a    ZWJ
-;;              b    Other
+;;    The definitive list of properties are in text.unicode.ucd,
+;;    ucd-grapheme-break-properties and ucd-word-break-properties
 ;;
 ;;   Codepoints below U+20000 are looked up by two-staged tables.
 ;;   First, look up this table with (codepoint >> 8).
 ;;
-;;     static unsigned char break_table[0x200]
+;;     static unsigned char {GB|WB}_break_table[0x200]
 ;;
-;;   If the value is 255, both properties are 'Other'.
-;;   Otherwise, the value is an index to the secondary table.
+;;   If the value is up to single-property-offset (224), the value is
+;;   the index to the secondary table.
 ;;
-;;     static unsigned char break_subtable[index][256]
+;;     static unsigned char {GB|WB}_break_subtable[index][256]
 ;;
-;;   The value of this table encodes WB and GB properties.
+;;   The value of this table encodes GB|WB properties.
+;;   If th evalue is equal to or more than single-property-offset,
+;;   it indicates the entire block of 256 codepoints shared the same
+;;   property, (- value single-property-offset).
 ;;
 ;;   Codepoints on or above U+20000 are all 'Other', except the following
 ;;   ranges.  They are handled specially in the lookup procedure.
@@ -216,6 +188,12 @@
 ;;    E0020..E007F   GB_Control, WB_Format
 ;;    E0100..E01EF   GB_Extend, WB_Extend
 ;;
+;;
+;; * Emoji Properties
+;;
+;;    Also http://unicode.org/reports/tr51
+;;    https://www.unicode.org/Public/13.0.0/ucd/emoji/emoji-data.txt
+;;
 
 ;;;
 ;;;  Main entry
@@ -224,11 +202,11 @@
 (define (main args)
   (match (cdr args)
     [("--fetch" dir . maybe-version) (apply fetch-ucd dir maybe-version)]
-    [("--import" dir ucdfile)
-     (unless (file-is-directory? dir)
-       (exit 1 "Directory required, but got: ~a" dir))
-     (with-output-to-file ucdfile
-       (cut ucd-save-db (ucd-parse-files dir)))]
+    [("--import" dir ucdfile) (import-data dir ucdfile #f)]
+    [("--import" dir ucdfile testfile) (import-data dir ucdfile testfile)]
+    [("--analyze" ucdfile lobits)
+     (analyze-break-tables (call-with-input-file ucdfile ucd-load-db)
+                           (string->number lobits))]
     [("--compile" ucdfile)
      (unless (file-exists? ucdfile)
        (exit 1 "Couldn't open unicode data file: ~a" ucdfile))
@@ -241,7 +219,7 @@
   0)
 
 ;;;
-;;; Fetching 
+;;; Fetching
 ;;;
 (define (fetch-ucd dir :optional (version #f))
   (let ([path (if version
@@ -253,8 +231,13 @@
                        "EastAsianWidth.txt"
                        "auxiliary/GraphemeBreakProperty.txt"
                        "auxiliary/SentenceBreakProperty.txt"
-                       "auxiliary/WordBreakProperty.txt")])
+                       "auxiliary/WordBreakProperty.txt"
+                       "auxiliary/GraphemeBreakTest.txt"
+                       "auxiliary/SentenceBreakTest.txt"
+                       "auxiliary/WordBreakTest.txt"
+                       "emoji/emoji-data.txt")])
     (make-directory* (build-path dir "auxiliary"))
+    (make-directory* (build-path dir "emoji"))
     (dolist [f datafiles]
       (let1 p (build-path dir f)
         (display #"Getting ~|f|... ") (flush)
@@ -263,31 +246,89 @@
                         :sink o :flusher (^ _ #t))))
         (display "done\n")))))
 
+;;;
+;;;  Importing
+;;;
+
+;; Dump unicode character data in ucdfile.
+(define (import-data dir ucdfile testfile)
+  (unless (file-is-directory? dir)
+    (exit 1 "Directory required, but got: ~a" dir))
+  (let1 ucd-db (ucd-parse-files dir)
+    (with-output-to-file ucdfile
+      (cut ucd-save-db ucd-db))
+    (when testfile
+      (with-output-to-file testfile
+        (^[] (import-break-test ucd-db dir))))))
+
+(define (import-break-test ucd-db dir)
+  ;; break test file format:
+  ;;  <line>: <sep> (<code> <sep>)+ '#' <comment>
+  ;;  <sep> : '÷' | '×'
+  ;;  <code> : #[[:xdigit:]]+
+  ;;
+  ;; returns (<count> <seq> <comment>)
+  ;;   where
+  ;;     <count> : integer, to make it easy to find tests
+  ;;     <seq> : <bool> (<integer> <bool>)+
+  ;;     <bool> : #t as breakable (÷), #f as unbreakable (×)
+  (define (parse-1 line)
+    (and-let1 m (#/^([÷×])\s+/ line)
+      (let loop ([line (m'after)] [r (list (break-flag (m 1)))])
+        (rxmatch-case line
+          [#/^([[:xdigit:]]+)\s+([÷×])\s+(.*)/ (_ cc ff rr)
+              (loop rr (list* (break-flag ff) (string->number cc 16) r))]
+          [#/^#\s+(.*)/ (_ comment) (list (reverse r) comment)]
+          [else (list (reverse r) "")]))))
+  (define (break-flag s) (equal? s "÷"))
+  (define (parse name file)
+    (let1 path (build-path dir file)
+      (print)
+      (print ";; " file)
+      (print #"(define ~name '(")
+      (generator-for-each (^[r k] (pprint (cons k r)))
+                          (gfilter-map parse-1 (file->line-generator path))
+                          (giota))
+      (print "))")))
+  (print ";; -*- coding:utf-8 -*-")
+  (print ";; Generated from Unicode " (ucd-version ucd-db) " test data files")
+  (parse '*grapheme-break-tests* "auxiliary/GraphemeBreakTest.txt")
+  (parse '*word-break-tests*     "auxiliary/WordBreakTest.txt")
+  (parse '*sentence-break-tests* "auxiliary/SentenceBreakTest.txt")
+  )
+
 ;;;
 ;;;  Generate source file
 ;;;
 
 (define (generate-tables db)
-  (receive (char_attr.p char_attr.c) (sys-mkstemp "char_attr.c.")
-    (receive (unicode_attr.p unicode_attr.h) (sys-mkstemp "unicode_attr.h.")
-      (with-output-to-port char_attr.p
-        (^() (preamble db)
-          (generate-category-tables db)
-          (generate-case-tables db)
-          (generate-digit-value-tables db)))
-      (with-output-to-port unicode_attr.p
-        (^() (preamble db)
-          (generate-break-tables db)
-          (generate-width-tables db)))
-      (close-port char_attr.p)
-      (close-port unicode_attr.p)
-      (sys-rename char_attr.c "char_attr.c")
-      (sys-rename unicode_attr.h "gauche/priv/unicode_attr.h"))))
+  (let-values ([(c.p char_attr.c) (sys-mkstemp "char_attr.c.")]
+               [(h.p unicode_attr.h) (sys-mkstemp "unicode_attr.h.")]
+               [(scm.p unicode-attr.scm) (sys-mkstemp "unicode-attr.scm.")])
+    (with-output-to-port c.p
+      (^() (preamble db "/*" "*/")
+        (generate-category-tables db)
+        (generate-case-tables db)
+        (generate-digit-value-tables db)))
+    (with-output-to-port h.p
+      (^() (preamble db "/*" "*/")
+        (generate-break-tables db)
+        (generate-width-tables db)))
+    (with-output-to-port scm.p
+      (^() (preamble db ";;" "")
+        (generate-break-symbols db)))
+    (close-port c.p)
+    (close-port h.p)
+    (close-port scm.p)
+    (make-directory* "gauche/priv") ; in case we're building out-of-tree
+    (sys-rename char_attr.c "char_attr.c")
+    (sys-rename unicode_attr.h "gauche/priv/unicode_attr.h")
+    (sys-rename unicode-attr.scm "../ext/gauche/unicode-attr.scm")))
 
-(define (preamble db)
-  (print "/* Generated automatically from Unicode character database */")
-  (print "/* See src/gen-unicode.scm for the description of data structures. */")
-  (print #"/* Unicode version ~(ucd-version db).  Do not edit. */")
+(define (preamble db op cl)
+  (print #"~|op| Generated automatically from Unicode character database ~|cl|")
+  (print #"~|op| See src/gen-unicode.scm for the description of data structures. ~|cl|")
+  (print #"~|op| Unicode version ~(ucd-version db).  Do not edit. ~|cl|")
   )
 
 ;; NB: The caracter category tables are generated for each supported
@@ -408,22 +449,22 @@
   ;; The CAT entry can be just a symbol or (alias symbol).
   (print "static void init_predefined_charsets() {")
   (dolist [cat (append (ucd-general-categories)
-                       '(L M N P S Z C
+                       '(L LC M N P S Z C
                          (LETTER L) ASCII_LETTER
                          (DIGIT Nd) ASCII_DIGIT
-                         LETTER_DIGIT ASCII_LETTER_DIGIT 
-                         (LOWER Ll) ASCII_LOWER 
+                         LETTER_DIGIT ASCII_LETTER_DIGIT
+                         (LOWER Ll) ASCII_LOWER
                          (UPPER Lu) ASCII_UPPER
                          (TITLE Lt)
-                         GRAPHIC ASCII_GRAPHIC 
+                         GRAPHIC ASCII_GRAPHIC
                          PRINTING ASCII_PRINTING
-                         (PUNCTUATION P) ASCII_PUNCTUATION 
+                         (PUNCTUATION P) ASCII_PUNCTUATION
                          (SYMBOL S) ASCII_SYMBOL
                          (ISO_CONTROL Cc) ASCII_CONTROL
                          HEX_DIGIT
-                         WHITESPACE ASCII_WHITESPACE 
+                         WHITESPACE ASCII_WHITESPACE
                          BLANK ASCII_BLANK
-                         ASCII EMPTY 
+                         ASCII EMPTY
                          ASCII_WORD (WORD ASCII_WORD)))]
     (let1 setname (if (pair? cat) (car cat) cat)
       (print #"  predef_sets[SCM_CHAR_SET_~|setname|] =")
@@ -554,7 +595,7 @@
   (let1 ranges ($ reverse
                   $ fold (^[p s] (match s
                                    [() `(,p (0 . #f))]
-                                   [((_ . last) . rest)  
+                                   [((_ . last) . rest)
                                     (if (= (car p) last)
                                       `(,p ,@s)
                                       `(,p (,last . #f) ,@s))]))
@@ -640,7 +681,7 @@
   ;; general category class charsets (char-set:L etc.)
   (dolist [gcats (group-collection categories
                                   :key (^c (string-ref (symbol->string c) 0)))]
-    (let1 cs 
+    (let1 cs
         (make <char-code-set>
           :name (string->symbol (substring (symbol->string (car gcats)) 0 1)))
       (for-each (^c (walker (cut register cs c <> <>))) gcats)
@@ -649,12 +690,12 @@
   ;; is handled implicitly.
   (hash-table-put! sets 'ASCII_UPPER
                    (rlet1 cs (make <char-code-set> :name 'ASCII_UPPER)
-                     (add-code-range! cs 
+                     (add-code-range! cs
                                       (char->integer #\A)
                                       (char->integer #\Z))))
   (hash-table-put! sets 'ASCII_LOWER
                    (rlet1 cs (make <char-code-set> :name 'ASCII_LOWER)
-                     (add-code-range! cs 
+                     (add-code-range! cs
                                       (char->integer #\a)
                                       (char->integer #\z))))
   (hash-table-put! sets 'ASCII_LETTER
@@ -663,7 +704,7 @@
                                    (hash-table-ref sets 'ASCII_LOWER)))
   (hash-table-put! sets 'ASCII_DIGIT
                    (rlet1 cs (make <char-code-set> :name 'ASCII_DIGIT)
-                     (add-code-range! cs 
+                     (add-code-range! cs
                                       (char->integer #\0)
                                       (char->integer #\9))))
   (hash-table-put! sets 'LETTER_DIGIT
@@ -754,13 +795,13 @@
                      (add-code! cs #x7f)))
   (hash-table-put! sets 'HEX_DIGIT
                    (rlet1 cs (make <char-code-set> :name 'HEX_DIGIT)
-                     (add-code-range! cs 
+                     (add-code-range! cs
                                       (char->integer #\0)
                                       (char->integer #\9))
-                     (add-code-range! cs 
+                     (add-code-range! cs
                                       (char->integer #\A)
                                       (char->integer #\F))
-                     (add-code-range! cs 
+                     (add-code-range! cs
                                       (char->integer #\a)
                                       (char->integer #\f))))
   (hash-table-put! sets 'ASCII
@@ -768,95 +809,165 @@
                      (add-code-range! cs 0 127)))
   (hash-table-put! sets 'ASCII_WORD
                    (rlet1 cs (make <char-code-set> :name 'ASCII_WORD)
-                     (add-code-range! cs 
+                     (add-code-range! cs
                                       (char->integer #\0)
                                       (char->integer #\9))
-                     (add-code-range! cs 
+                     (add-code-range! cs
                                       (char->integer #\A)
                                       (char->integer #\Z))
-                     (add-code-range! cs 
+                     (add-code-range! cs
                                       (char->integer #\a)
                                       (char->integer #\z))
                      (add-code! cs (char->integer #\_))))
+  (hash-table-put! sets 'LC
+                   (code-set-union 'LC
+                                   (hash-table-ref sets 'Lt)
+                                   (hash-table-ref sets 'Ll)
+                                   (hash-table-ref sets 'Lu)))
   (hash-table-put! sets 'EMPTY (make <char-code-set> :name 'EMPTY))
   sets)
 
 ;; Break property values
-;; This goes to src/gauche/priv/unicode_attr.h
+;; This goes to src/gauche/priv/unicode_attr.h and ext/gauche/unicode-attr.scm
 ;;
 ;; The data structure is two-level tables that maps codepoint 0-1FFFFF to
 ;; an octet.
-;; 
-;; The first level, break_table, is 512-length byte vector.
-;; Upper 9bit of codepoint is used to index this table.  If the entry
-;; is 255, the break property of that codepoint takes the default value.
 ;;
-;; Otherwise, the first table entry is the subtable, break_subtable[N][],
-;; and the lower 8bit of codepoint is used to index the subtable.
-;; it returns an octet, in which upper 4 bit is for grapheme break property
-;; and lower 4 bit is for word break property.
+;; The first level, {GB|WB}_break_table, is 512-length byte vector.
+;; Upper 9bit of codepoint is used to index this table.
+;; If the entry is [0..224], it's the index to the {GB|WB}_break_subtable[N][],
+;; which is indexed with the lower 8 bit of the codepoint.
+;; If the entry is above that, it indicates the entire block shares the
+;; same property.
 
 (define (generate-break-tables db)
+  (define single-property-offset 224)
   (define subtable-count 0)
 
-  (define (gen-entry code)
-    (let1 e (ucd-get-break-property db code)
-      (if (not e)
-        (format #t "    BREAK_ENTRY(GB_Other, WB_Other),\n")
-        (format #t "    BREAK_ENTRY(GB_~a, WB_~a),\n"
-                (let1 gp (ucd-break-property-grapheme e)
-                  (if (memq gp '(CR LF))
-                    'Other
-                    gp))
-                (let1 wp (ucd-break-property-word e)
-                  (if (memq wp '(CR LF Single_Quote Double_Quote))
-                    'Other
-                    wp))))))
+  (define (get-property entry type); type : GB | WB
+    (if entry
+      (case type
+        [(GB) (ucd-break-property-grapheme entry)]
+        [(WB) (ucd-break-property-word entry)])
+      'Other))
+
+  (define (gen-entry code type) ; type : GB | WB
+    (format #t "    ~a_~a,\n" type
+            (get-property (ucd-get-break-property db code) type)))
 
   ;; returns subtable-number
-  (define (gen-subtable start-code)
-    (if (any?-ec (: lb 256) (ucd-get-break-property db (+ start-code lb)))
+  (define (gen-subtable start-code type)
+    (define kind #f)
+    (do-ec (: lb 256)
+           (let* ([e (ucd-get-break-property db (+ start-code lb))]
+                  [p (get-property e type)])
+             (cond [(not kind) (set! kind p)]
+                   [(eqv? kind p)]
+                   [else (set! kind #t)])))
+    (if (symbol? kind)
+      ;; same property in the entire block
+      (format "(BOFF+~a_~a)" type kind)
       (rlet1 subtable-num subtable-count
         (format #t "  {\n")
-        (do-ec (: lb 256) (gen-entry (+ start-code lb)))
+        (do-ec (: lb 256) (gen-entry (+ start-code lb) type))
         (format #t "  },\n")
-        (inc! subtable-count))
-      255))
+        (inc! subtable-count))))
+
+  (define (gen-main-table type)
+    (set! subtable-count 0)
+    (print)
+    (print #"static unsigned char ~|type|_break_subtable[][256] = {")
+    (let1 nlist (list-ec (: n 512) (gen-subtable (* n 256) type))
+      (print "};")
+      (when (>= subtable-count single-property-offset)
+        (error #"~|type| subtable index overflow (~|subtable-count|)"))
+      (print)
+      (format #t "static unsigned char ~a_break_table[] = {" type)
+      (do-ec (:parallel (: n nlist) (:integers i))
+             (begin (when (zero? (mod i 4)) (format #t "\n   "))
+                    (format #t " ~3d," n)))
+      (format #t "\n};\n")))
 
   ;; Generate table of symbols
-  (define (gen-symbol-table constants prefix)
-    (receive (normals specials) (break not constants)
-      (for-each-with-index (^(i c) (format #t "#define ~a_~a ~a\n" prefix c i))
-                           normals)
-      (for-each-with-index (^(i c) (format #t "#define ~a_~a ~a\n" prefix c
-                                           (+ 16 i)))
-                           (cdr specials)))
-    (format #t "static void init_~a_symbols(ScmModule *mod) {\n" prefix)
-    (for-each (^c
-               (and c
-                    (format #t
-                            "Scm_DefineConst(mod, \
-                          SCM_SYMBOL(SCM_INTERN(\"~a_~a\")),\
-                          SCM_MAKE_INT(~a_~a));\n"
-                            prefix c prefix c)))
-              constants)
-    (print "}"))
+  (define (gen-symbol-table constants type)
+    (for-each-with-index (^[i c] (format #t "#define ~a_~a ~a\n" type c i))
+                         constants))
 
   (print)
-  (gen-symbol-table (ucd-grapheme-break-properties) "GB")
-  (gen-symbol-table (ucd-word-break-properties) "WB")
+  (gen-symbol-table (ucd-grapheme-break-properties) 'GB)
+  (gen-symbol-table (ucd-word-break-properties) 'WB)
   (print)
-  (print "#define BREAK_ENTRY(g, w)  (((g)<<4)|(w))")
-  (print)
-  (print "static unsigned char break_subtable[][256] = {")
-  (let1 nlist (list-ec (: n 512) (gen-subtable (* n 256)))
-    (print "};")
-    (print)
-    (format #t "static unsigned char break_table[] = {")
-    (do-ec (:parallel (: n nlist) (:integers i))
-           (begin (when (zero? (mod i 8)) (format #t "\n   "))
-                  (format #t " ~3d," n)))
-    (format #t "\n};\n"))
+  (print #"#define BOFF ~|single-property-offset|")
+  (gen-main-table 'GB)
+  (gen-main-table 'WB)
+  )
+
+;; Auxiliary stuff to find out optimal table configuration
+;; NB: It looks like 3-level table doesn't save at all.
+(define (analyze-break-tables db :optional (lobits 4))
+  (define num-leaf-tables 0)
+  (define num-intermediate-tables 0)
+
+  ;; Check break properties of 2^lobits consecutive codepoints.
+  ;; Returns (<codepoint-string> . <property-or-#t>).  If properties of
+  ;; all codeponds are the same, cdr is the property; otherwise it's #t.
+  (define (probe-lo start accessor)
+    (define kind #f)
+    (define (get-1 i)
+      (if-let1 e (ucd-get-break-property db (+ start i))
+        (accessor e)
+        'Other))
+    (do-ec (: i (expt 2 lobits))
+           (let1 kk (get-1 i)
+             (cond [(not kind) (set! kind kk)]
+                   [(eqv? kind #t)]
+                   [(eqv? kind kk)]
+                   [else (set! kind #t)])))
+    (when (eqv? kind #t) (inc! num-leaf-tables))
+    (cons (format "~5,'0x" start) kind))
+  ;; Check break properties of 16 consecutive 16 codepoint blocks.
+  (define (probe-hi start accessor)
+    (define kind #f)
+    (define ps
+      (list-ec (: ii (expt 2 (- 8 lobits)))
+               (rlet1 p (probe-lo (+ start (ash ii lobits)) accessor)
+                 (cond [(not kind) (set! kind (cdr p))]
+                       [(eqv? kind (cdr p))]
+                       [else (set! kind #t)]))))
+    (case kind
+      [(#t)
+       (format #t "~3,'0xXX -------------\n" (ash start -8))
+       (for-each (^p (format #t "  ~a ~a\n" (car p) (cdr p)))
+                 (remove (^p (eq? (cdr p) 'Other)) ps))
+       (inc! num-intermediate-tables)]
+      [(Other)]
+      [else
+       (format #t "~3,'0xXX  ~a\n" (ash start -8) kind)]))
+
+  (define (probe-all name accessor)
+    (format #t "~a distribution\n" name)
+    (set! num-leaf-tables 0)
+    (set! num-intermediate-tables 0)
+    (do-ec (: u #x200)
+           (probe-hi (ash u 8) accessor))
+    (list name num-leaf-tables num-intermediate-tables))
+
+  (dolist [rr (list (probe-all "GB" ucd-break-property-grapheme)
+                    (probe-all "WB" ucd-break-property-word))]
+    (format #t "~a # int. tables = ~d, # leaf tables = ~d\n"
+            (car rr) (cadr rr) (caddr rr)))
+  )
+
+(define (generate-break-symbols db)
+  (define (gen-symbols constants type)
+    (for-each-with-index
+     (^[i c] (pprint `(define-constant ,(symbol-append type '_ c) ,i)))
+     constants)
+    (pprint `(define-constant ,(symbol-append '* type '-properties*)
+               ',(map-with-index (^[i c] (cons c i)) constants))))
+
+  (gen-symbols (ucd-grapheme-break-properties) 'GB)
+  (gen-symbols (ucd-word-break-properties) 'WB)
   )
 
 ;; EastAsianWidth property values
@@ -864,7 +975,7 @@
 ;;
 ;; The data structure is two-level tables that maps codepoint 0-1FFFFF to
 ;; an octet.
-;; 
+;;
 ;; The first level, width_table, is 512-length byte vector.
 ;; Upper 9bit of codepoint is used to index this table.  If the entry
 ;; is one of WIDTH_x values, all codepoints of that range has that property
@@ -879,7 +990,7 @@
   (define prop-count (length (ucd-east-asian-widths)))
   (define subtable-count prop-count)
   (define width-table (~ db'width-table))
-  
+
   ;; returns subtable-number
   (define (gen-subtable start-code)
     (let1 e (dict-get width-table start-code 'N)
@@ -894,7 +1005,7 @@
           (print "  },")
           (inc! subtable-count))
         ;; all codepoints of this range shares the value.
-        (find-index (cut eqv? e <>) (ucd-east-asian-widths))))) 
+        (find-index (cut eqv? e <>) (ucd-east-asian-widths)))))
 
   ;; Generate table of symbols
   (define (gen-symbol-table)
