@@ -1,7 +1,7 @@
 /*
  * hash.c - hash table implementation
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -34,7 +34,6 @@
 #define LIBGAUCHE_BODY
 #include <math.h>
 #include "gauche.h"
-#include "gauche/class.h"
 #include "gauche/priv/atomicP.h"
 
 /*============================================================
@@ -238,7 +237,7 @@ static u_long internal_uvector_hash(ScmUVector *u, u_long salt, int portable)
         size_t s = SCM_UVECTOR_SIZE(u);
 
         switch (uvtype) {
-            /* We can use siphash if u is u8vector or s8vector. */ 
+            /* We can use siphash if u is u8vector or s8vector. */
         case SCM_UVECTOR_S8:
         case SCM_UVECTOR_U8:
             return Scm__DwSipPortableHash((uint8_t*)SCM_UVECTOR_ELEMENTS(u),
@@ -249,13 +248,13 @@ static u_long internal_uvector_hash(ScmUVector *u, u_long salt, int portable)
                for we need to allocate a buffer.
                (Unless we directly access siphash setup/round function
                and feed one word at a time--which is tedious.)
-               The current code may not be ideal, but just as good as our 
+               The current code may not be ideal, but just as good as our
                other primitive portable hash functions. */
 
             /* Initial hash value.  The seed value for each uvector type
                is just a random value I generated. */
 #define INIT_R(r, seed) SMALL_INT_HASH(r, seed^salt)
-                                                         
+
         case SCM_UVECTOR_S16:
             seed = 3499211612ul;
             goto do16;
@@ -297,7 +296,7 @@ static u_long internal_uvector_hash(ScmUVector *u, u_long salt, int portable)
             goto do64;
         case SCM_UVECTOR_U64:
             seed = 1323567403ul;
-            do64: 
+            do64:
             {
                 INIT_R(r, seed);
                 for (size_t i=0; i<s; i++) {
@@ -319,7 +318,7 @@ static u_long internal_uvector_hash(ScmUVector *u, u_long salt, int portable)
             }
             return r;
         }
-#undef INIT_R            
+#undef INIT_R
         case SCM_UVECTOR_RESERVED1:
         case SCM_UVECTOR_C32:
         case SCM_UVECTOR_C64:
@@ -338,7 +337,7 @@ static u_long internal_uvector_hash(ScmUVector *u, u_long salt, int portable)
 
 /* equal-hash, which satisfies
      forall x, y: equal(x,y) => hash(x) = hash(y)
-  
+
    Both default-hash and portable-hash have this property but their
    requirements are slightly different, so here's the common part.
 */
@@ -406,34 +405,40 @@ static u_long equal_hash_common(ScmObj obj, u_long salt, int portable)
             ADDRESS_HASH(hashval, obj);
             return hashval;
         }
-    } else {
-        /* Call specialized object-hash method
-           We need some trick; See libomega.scm for the details. */
-        static ScmObj call_object_hash_proc = SCM_UNDEFINED;
-        static ScmObj portable_hash_proc = SCM_UNDEFINED;
-        static ScmObj default_hash_proc = SCM_UNDEFINED;
-        SCM_BIND_PROC(call_object_hash_proc, "%call-object-hash",
-                      Scm_GaucheInternalModule());
-        SCM_BIND_PROC(portable_hash_proc, "portable-hash", Scm_GaucheModule());
-        SCM_BIND_PROC(default_hash_proc, "default-hash", Scm_GaucheModule());
-        ScmObj r = Scm_ApplyRec3(call_object_hash_proc, obj,
-                                 (portable
-                                  ? portable_hash_proc
-                                  : default_hash_proc),
-                                 (portable
-                                  ? Scm_MakeIntegerU(salt)
-                                  : SCM_FALSE));
-        if (SCM_INTP(r)) {
-            return (u_long)SCM_INT_VALUE(r);
-        }
-        if (SCM_BIGNUMP(r)) {
-            /* NB: Scm_GetUInteger clamps the result to [0, ULONG_MAX],
-               so taking the LSW would give better distribution. */
-            return SCM_BIGNUM(r)->values[0];
-        }
-        Scm_Error("object-hash returned non-integer: %S", r);
-        return 0;               /* dummy */
     }
+
+    ScmClass *k = SCM_CLASS_OF(obj);
+    if (k->hash) {
+        return (u_long)k->hash(obj, salt,
+                               (portable? SCM_HASH_PORTABLE:0));
+    }
+
+    /* Call specialized object-hash method
+       We need some trick; See libomega.scm for the details. */
+    static ScmObj call_object_hash_proc = SCM_UNDEFINED;
+    static ScmObj portable_hash_proc = SCM_UNDEFINED;
+    static ScmObj default_hash_proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(call_object_hash_proc, "%call-object-hash",
+                  Scm_GaucheInternalModule());
+    SCM_BIND_PROC(portable_hash_proc, "portable-hash", Scm_GaucheModule());
+    SCM_BIND_PROC(default_hash_proc, "default-hash", Scm_GaucheModule());
+    ScmObj r = Scm_ApplyRec3(call_object_hash_proc, obj,
+                             (portable
+                              ? portable_hash_proc
+                              : default_hash_proc),
+                             (portable
+                              ? Scm_MakeIntegerU(salt)
+                              : SCM_FALSE));
+    if (SCM_INTP(r)) {
+        return (u_long)SCM_INT_VALUE(r);
+    }
+    if (SCM_BIGNUMP(r)) {
+        /* NB: Scm_GetUInteger clamps the result to [0, ULONG_MAX],
+           so taking the LSW would give better distribution. */
+        return SCM_BIGNUM(r)->values[0];
+    }
+    Scm_Error("object-hash returned non-integer: %S", r);
+    return 0;               /* dummy */
 }
 
 /* For recursive call to the current hash function - see call-object-hash
@@ -450,7 +455,7 @@ ScmObj Scm_CurrentRecursiveHash(ScmObj newval)
 }
 
 /* 'Portable' general hash function.
-   
+
    It is guaranteed that the hash value won't change for the same objects
    (roughly, indistinguishable in their external representation)
    across the runs of the program, and among different platforms.
@@ -467,6 +472,36 @@ ScmSmallInt Scm_DefaultHash(ScmObj obj)
     return equal_hash_common(obj, Scm_HashSaltRef(), FALSE) & HASHMASK;
 }
 
+/* This is to be called from ScmClass->hash if it needs to compute
+   hash value of field values. */
+ScmSmallInt Scm_RecursiveHash(ScmObj obj, ScmSmallInt salt, u_long flags)
+{
+    if (flags & SCM_HASH_PORTABLE) {
+        return equal_hash_common(obj, salt, TRUE) & PORTABLE_HASHMASK;
+    } else {
+        return equal_hash_common(obj, salt, FALSE) & HASHMASK;
+    }
+}
+
+ScmSmallInt Scm_SmallIntHash(ScmSmallInt val,
+                             ScmSmallInt salt SCM_UNUSED,
+                             u_long flags SCM_UNUSED)
+{
+    ScmSmallInt v;
+    SMALL_INT_HASH(v, val);
+    return v;
+}
+
+ScmSmallInt Scm_Int64Hash(int64_t val,
+                          ScmSmallInt salt SCM_UNUSED,
+                          u_long flags SCM_UNUSED)
+{
+    ScmSmallInt v;
+    SMALL_INT_HASH(v, val);
+    return v;
+}
+
+
 /* This is to expose string hash function.  Modulo is for the compatibility
    of srfi-13; just give 0 as modulo if you don't need it.  */
 u_long Scm_HashString(ScmString *str, u_long modulo)
@@ -622,7 +657,7 @@ static u_long address_hash(const ScmHashCore *ht SCM_UNUSED, intptr_t obj)
     return hashval;
 }
 
-static int address_cmp(const ScmHashCore *ht SCM_UNUSED, 
+static int address_cmp(const ScmHashCore *ht SCM_UNUSED,
                        intptr_t key, intptr_t k2)
 {
     return (key == k2);
@@ -1078,7 +1113,7 @@ ScmObj Scm_HashTableStat(ScmHashTable *table)
  * Printer
  */
 
-static void hash_print(ScmObj obj, ScmPort *port, 
+static void hash_print(ScmObj obj, ScmPort *port,
                        ScmWriteContext *ctx SCM_UNUSED)
 {
     ScmHashTable *ht = (ScmHashTable*)obj;
@@ -1132,15 +1167,15 @@ void Scm__InitHash()
     u_long salt = ((u_long)getpid() * ((u_long)t.tv_sec^(u_long)t.tv_usec));
     ADDRESS_HASH(salt, salt);
     salt &= SCM_SMALL_INT_MAX;
-    /* 
+    /*
      * We can't use Scm_BindPrimitiveParameter here, since symbol table
      * is not initialized yet (symbol table uses hashtable!)
      */
-    hash_salt = 
-        Scm_MakePrimitiveParameter(SCM_CLASS_PRIMITIVE_PARAMETER, SCM_FALSE, 
+    hash_salt =
+        Scm_MakePrimitiveParameter(SCM_CLASS_PRIMITIVE_PARAMETER, SCM_FALSE,
                                    Scm_MakeIntegerU(salt), 0);
-    current_recursive_hash = 
-        Scm_MakePrimitiveParameter(SCM_CLASS_PRIMITIVE_PARAMETER, SCM_FALSE, 
+    current_recursive_hash =
+        Scm_MakePrimitiveParameter(SCM_CLASS_PRIMITIVE_PARAMETER, SCM_FALSE,
                                    SCM_FALSE, 0);
 }
 
@@ -1148,6 +1183,7 @@ void Scm__InitHash()
  * For backward compatibility
  */
 
+#if GAUCHE_API_VERSION < 1000
 /* Backward compatibility.
    NB: Casting ScmDictEntry* to ScmHashEntry* would be invalid if
    sizeof(intptr_t) and sizeof(void*) differ.  I know only one
@@ -1232,6 +1268,7 @@ ScmObj Scm_MakeHashTable(ScmHashProc *hashfn,
     return SCM_UNDEFINED;
 #endif
 }
+#endif /*GAUCHE_API_VERSION < 1000*/
 
 /* Legacy hash function.
  *
@@ -1360,7 +1397,7 @@ static u_long legacy_flonum_hash(double f)
     /* This condition eliminates NaN as well. */
     if (!(minus_two_pow_63 < d && d < two_pow_63)) return 0;
     if (-0.5 < d && d < 0.5) return 0;
-                                 
+
     double dm = trunc(fmod(d, two_pow_32));
     if (dm < 0) dm += two_pow_32;
     return (u_long)trunc(dm);
