@@ -1,7 +1,7 @@
 /*
  * lazy.c - lazy evaluation constructs
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -34,6 +34,7 @@
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/priv/atomicP.h"
+#include "gauche/priv/pairP.h"
 
 /*==================================================================
  * Promise
@@ -169,10 +170,10 @@ static ScmObj force_cc(ScmObj result, void **data)
         SCM_INTERNAL_MUTEX_UNLOCK(p->content->mutex);
     }
     Scm_VM()->handlers = handlers;
-    SCM_RETURN(Scm_Force(SCM_OBJ(p)));
+    SCM_RETURN(Scm_VMForce(SCM_OBJ(p)));
 }
 
-ScmObj Scm_Force(ScmObj obj)
+ScmObj Scm_VMForce(ScmObj obj)
 {
     if (!SCM_PROMISEP(obj)) {
         SCM_RETURN(obj);
@@ -211,7 +212,20 @@ ScmObj Scm_Force(ScmObj obj)
     }
 }
 
-#if GAUCHE_LAZY_PAIR
+ScmObj Scm_Force(ScmObj obj)
+{
+    if (!SCM_PROMISEP(obj)) {
+        SCM_RETURN(obj);
+    } else {
+        ScmPromiseContent *c = SCM_PROMISE(obj)->content;
+        if (c->forced) SCM_RETURN(c->code);
+
+        static ScmObj force = SCM_UNDEFINED;
+        SCM_BIND_PROC(force, "force", Scm_SchemeModule());
+        return Scm_ApplyRec1(force, obj);
+    }
+}
+
 /*=================================================================
  * Lazy pairs
  *
@@ -219,9 +233,9 @@ ScmObj Scm_Force(ScmObj obj)
  *  If you check whether the object is pair or not by SCM_PAIRP,
  *  it is 'forced' to become a pair.  The forcing is
  *  identity-preserving; that is, once a lazy pair is forced, the pointer
- *  now becomes a pair.  It is a critical attribute to make the
- *  forcing implicit---we can't do it for general values.
- *  Since the forcing is implicit, majority of the code won't see
+ *  now becomes a pair (actually, an extended pair).  It is a critical
+ *  attribute to make the forcing implicit---we can't do it for general
+ *  values.   Since the forcing is implicit, majority of the code won't see
  *  ScmLazyPair.
  *
  *  The identity-preserving property requires us to generate
@@ -232,45 +246,55 @@ ScmObj Scm_Force(ScmObj obj)
 /*
 
   (0) Initial state
+    A lazypair is allocated on 4-word boundary, but we return a pointer to
+    the second word.  That makes the object look like odd-word boundary
+    aligned, a marker of Extended Pair (see priv/pairP.h).   The 'hidden'
+    first word will be used for synchronization.
+    NB: An extended pair must have preceding hidden word with 111 in the lower
+    bits for extra check (when !SCM_PAIR_ALWAYS_ALIGNED_EVEN_WORDS).  That
+    check is done after SCM_PAIRP, however, so it won't see (AO_t)0.
 
-  +---------------+
-  |  LazyPair tag |
-  +---------------+
-  |  ScmObj item  |
-  +---------------+
-  |    ScmObj     | ----> generator
-  +---------------+
-  |   (AO_t)0     |
-  +---------------+
+            +---------------+
+            |   (AO_t)0     |
+   ScmObj > +---------------+
+            |  LazyPair tag |
+            +---------------+
+            |  ScmObj item  |
+            +---------------+
+            |    ScmObj     | ----> generator
+            +---------------+
 
-
-  (1) The first one (owner) grabs the packet, then evaluates the generator.
+  (1) The first thread (owner) grabs the packet, then evaluates the generator.
   The grabbing is done by CAS to make it atomic.
 
-  +---------------+
-  |  LazyPair tag |
-  +---------------+
-  |  ScmObj item  |
-  +---------------+
-  |    ScmObj     | ----> generator
-  +---------------+
-  |  (AO_t)owner  |
-  +---------------+
+            +---------------+
+            |  (AO_t)owner  |
+   ScmObj > +---------------+
+            |  LazyPair tag |
+            +---------------+
+            |  ScmObj item  |
+            +---------------+
+            |    ScmObj     | ----> generator
+            +---------------+
 
 
-  (2) If generator yields a non-nil value, owner first creates a
+  (2) If generator yields a non-EOF value, owner first creates a
       new LazyPair...
 
-  +---------------+
-  |  LazyPair tag |
-  +---------------+
-  |  ScmObj item  |
-  +---------------+
-  |    ScmObj     | ------------------------+-> generator
-  +---------------+                         |
-  |  (AO_t)owner  |                         |
-  +---------------+                         |
+            +---------------+
+            |  (AO_t)owner  |
+   ScmObj > +---------------+
+            |  LazyPair tag |
+            +---------------+
+            |  ScmObj item  |
+            +---------------+
+            |    ScmObj     | --------------+-> generator
+            +---------------+               |
                                             |
+                                            |
+                                            |
+                         +---------------+  |
+                         |   (AO_t)0     |  |
                          +---------------+  |
                          |  LazyPair tag |  |
                          +---------------+  |
@@ -278,146 +302,265 @@ ScmObj Scm_Force(ScmObj obj)
                          +---------------+  |
                          |    ScmObj     | -/
                          +---------------+
-                         |    (AO_t)0    |
-                         +---------------+
 
   (3) then it replaces the cdr pointer with the new LazyPair, and clear
-      the third slot.
+      the last slot.
 
-  +---------------+
-  |  LazyPair tag |
-  +---------------+
-  |     ScmObj    | -\
-  +---------------+  |
-  |      NIL      |  |
-  +---------------+  |
-  |  (AO_t)owner  |  |
-  +---------------+  |
-                     |
-                     |   +---------------+
-                     \-> |  LazyPair tag |
-                         +---------------+
-                         | ScmObj newitem|
-                         +---------------+
-                         |    ScmObj     | ---> generator
-                         +---------------+
-                         |    (AO_t)0    |
-                         +---------------+
+            +---------------+
+            |  (AO_t)owner  |
+   ScmObj > +---------------+
+            |  LazyPair tag |
+            +---------------+
+            |     ScmObj    | -\
+            +---------------+  |
+            |      NIL      |  |
+            +---------------+  |
+                               |
+                               |   +---------------+
+                               |   |    (AO_t)0    |
+                               |   +---------------+
+                               \-> |  LazyPair tag |
+                                   +---------------+
+                                   | ScmObj newitem|
+                                   +---------------+
+                                   |    ScmObj     | ---> generator
+                                   +---------------+
 
+  (4) replaces the hidden word with the extended pair descriptor.
+      At this moment, it still look like a lazy pair and someone working on
+      it, from the thread other than the owner.
 
-  (4) and replaces the car pointer with the lookahead value, which makes
-      the original object an (extended) pair,
+            +---------------+
+            |    desc + 7   | ---> ScmExtendedPairDescriptor
+   ScmObj > +---------------+
+            |  LazyPair tag |
+            +---------------+
+            |     ScmObj    | -\
+            +---------------+  |
+            |      NIL      |  |
+            +---------------+  |
+                               |
+                               |   +---------------+
+                               |   |    (AO_t)0    |
+                               |   +---------------+
+                               \-> |  LazyPair tag |
+                                   +---------------+
+                                   | ScmObj newitem|
+                                   +---------------+
+                                   |    ScmObj     | ---> generator
+                                   +---------------+
 
-  +---------------+
-  |  ScmObj item  |
-  +---------------+
-  |     ScmObj    | -\
-  +---------------+  |
-  |      NIL      |  |
-  +---------------+  |
-  |  (AO_t)owner  |  |
-  +---------------+  |
-                     |
-                     |   +---------------+
-                     \-> |  LazyPair tag |
-                         +---------------+
-                         | ScmObj newitem|
-                         +---------------+
-                         |    ScmObj     | ---> generator
-                         +---------------+
-                         |    (AO_t)0    |
-                         +---------------+
+  (5) finally we set the car field with the precomputed item value.
+      Now it looks like just an extended pair.
 
-  (5) finally set the fourth slot with 1, just not to grab the pointer
-      to the owner thread so that the owner thread won't be retained
-      unnecessarily.
-
-  +---------------+
-  |  ScmObj item  |
-  +---------------+
-  |     ScmObj    | -\
-  +---------------+  |
-  |      NIL      |  |
-  +---------------+  |
-  |    (AO_t)1    |  |
-  +---------------+  |
-                     |
-                     |   +---------------+
-                     \-> |  LazyPair tag |
-                         +---------------+
-                         | ScmObj newitem|
-                         +---------------+
-                         |    ScmObj     | ---> generator
-                         +---------------+
-                         |    (AO_t)0    |
-                         +---------------+
+            +---------------+
+            |    desc + 7   | ---> ScmExtendedPairDescriptor
+   ScmObj > +---------------+
+            |  ScmObj item  |
+            +---------------+
+            |     ScmObj    | -\
+            +---------------+  |
+            |      NIL      |  |
+            +---------------+  |
+                               |
+                               |   +---------------+
+                               |   |    (AO_t)0    |
+                               |   +---------------+
+                               \-> |  LazyPair tag |
+                                   +---------------+
+                                   | ScmObj newitem|
+                                   +---------------+
+                                   |    ScmObj     | ---> generator
+                                   +---------------+
 
 
   (2') If generator yields EOF, we don't create a new lazy pair.
   We first replace the second and third slot by NIL,
 
-  +---------------+
-  | LazyPair tag  |
-  +---------------+
-  |      NIL      |
-  +---------------+
-  |      NIL      |
-  +---------------+
-  |  (AO_t)owner  |
-  +---------------+
+            +---------------+
+            |  (AO_t)owner  |
+   ScmObj > +---------------+
+            | LazyPair tag  |
+            +---------------+
+            |      NIL      |
+            +---------------+
+            |      NIL      |
+            +---------------+
 
   (3') then replace the car part by the cached value, which turns
   the object to an (extended) pair,
 
-  +---------------+
-  |  ScmObj item  |
-  +---------------+
-  |      NIL      |
-  +---------------+
-  |      NIL      |
-  +---------------+
-  |  (AO_t)owner  |
-  +---------------+
+            +---------------+
+            |   desc + 7    | ---> ScmExtendedPairDescriptor
+   ScmObj > +---------------+
+            | LazyPair tag  |
+            +---------------+
+            |      NIL      |
+            +---------------+
+            |      NIL      |
+            +---------------+
 
 
-  (4') then clear the fourth slot to be GC-friendly.
-
-  +---------------+
-  |  ScmObj item  |
-  +---------------+
-  |      NIL      |
-  +---------------+
-  |      NIL      |
-  +---------------+
-  |    (AO_t)1    |
-  +---------------+
+  (4') then set the car field with the cached value, which turns
+  the object to an (extended) pair, with its cdr being NIL.
 
+            +---------------+
+            |   desc + 7    | ---> ScmExtendedPairDescriptor
+   ScmObj > +---------------+
+            |     item      |
+            +---------------+
+            |      NIL      |
+            +---------------+
+            |      NIL      |
+            +---------------+
 
   Each step of the state transitions (0)->(1)->(2)->(3)->(4)->(5) and
   (0)->(1)->(2')->(3')->(4') are atomic, so the observer see either
   one of those states.
+
+  [Extended protocol]
+
+  Experimentally, the generator can return more than just the next item.
+
+  - It can return the second value as pair attribute alist, to be attached
+    to the pair containing the generated item.
+
+  - It can return a (lazy) pair or (), instead of just an item.  This is
+    useful to implement lcons.  We need a special marker to distinguish
+    such generator from the ordinary one that returns a pair as an element.
+    So such lazy pair should be constructed with Scm_LazyCons, instead of
+    Scm_MakeLazyPair.
  */
 
+
 SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_LazyPairClass, NULL);
 
-/* The order is important - must correspond to ScmExtendedPair. */
+/* This is what everyone see */
 struct ScmLazyPairRec {
     SCM_HEADER;
     ScmObj item;
     ScmObj generator;
-    ScmAtomicVar owner;
 };
 
-ScmObj Scm_MakeLazyPair(ScmObj item, ScmObj generator)
+/* And this is the real thing */
+typedef struct ScmRealLazyPairRec {
+    ScmAtomicVar owner;
+    ScmLazyPair data;
+} ScmRealLazyPair;
+
+/* Clean up pair attribute list given from outside.
+   (1) We need to copy it so that future mutation won't interfere
+   (2) If the given obj contains bad items, we simply ignore it,
+       for raising error lazily isn't useful.
+*/
+static ScmObj copy_attrs(ScmObj attrs)
 {
-    ScmLazyPair *z = SCM_NEW(ScmLazyPair);
-    z->owner = (ScmAtomicWord)0;
-    SCM_SET_CLASS(z, SCM_CLASS_LAZY_PAIR);
-    z->generator = generator;
-    z->item = item;
-    return SCM_OBJ(z);
+    ScmObj h = SCM_NIL, t = SCM_NIL, cp;
+    SCM_FOR_EACH(cp, attrs) {
+        if (SCM_PAIRP(SCM_CAR(cp))) {
+            SCM_APPEND1(h, t, Scm_Cons(SCM_CAAR(cp), SCM_CDAR(cp)));
+        }
+    }
+    return h;
 }
 
+static ScmObj make_lazy_pair(ScmObj item, ScmObj generator, ScmObj attrs)
+{
+    ScmRealLazyPair *z = SCM_NEW(ScmRealLazyPair);
+    z->owner = (ScmAtomicWord)0;
+    ScmLazyPair *r = &z->data;
+    SCM_SET_CLASS(r, SCM_CLASS_LAZY_PAIR);
+    r->generator = generator;
+    if (!SCM_NULLP(attrs) || SCM_MVBOXP(item)) {
+        ScmMVBox *b = Scm_MakeMVBox(2, SCM_UNDEFINED);
+        SCM_MVBOX_SET(b, 0, item);
+        SCM_MVBOX_SET(b, 1, copy_attrs(attrs));
+        r->item = SCM_OBJ(b);
+    } else {
+        r->item = item;
+    }
+    return SCM_OBJ(r);
+}
+
+ScmObj Scm_MakeLazyPair(ScmObj item, ScmObj generator, ScmObj attrs)
+{
+    /* A check to eliminate the caller accidentally create a 'marked' lazy
+       pair.  Using cons as a generator to mark it is just a provisional
+       solution, so do not count on it. */
+    if (SCM_PAIRP(generator)) {
+        Scm_Error("generator must be a procedure, but got: %S", generator);
+    }
+    return make_lazy_pair(item, generator, attrs);
+}
+
+ScmObj Scm_LazyCons(ScmObj item, ScmObj thunk, ScmObj attrs)
+{
+    /* A 'marked' lazy pair, where thunk returns the next pair instead of
+       next item.   Provisionally, we insert extra indirection in generator
+       slot to mark it.  It may change in future, so external code shouldn't
+       count on it. */
+    return make_lazy_pair(item, SCM_LIST1(thunk), attrs);
+}
+
+/* This can return () */
+static ScmObj generator_to_lazy_pair(ScmObj generator)
+{
+    ScmVM *vm = Scm_VM();
+    int lazy_cons = FALSE;
+
+    if (SCM_PAIRP(generator)) {
+        /* 'marked' */
+        generator = SCM_CAR(generator);
+        lazy_cons = TRUE;
+    }
+
+    /* This may be called with incomplete VM stack, so we protect it. */
+    int extra_frame_pushed = Scm__VMProtectStack(vm);
+
+    ScmObj val = Scm_ApplyRec0(generator);
+    ScmObj r = SCM_NIL;
+
+    if (!lazy_cons) {
+        /* standard lazy pair.  val is the next item. */
+        if (SCM_EOFP(val)) {
+            r = SCM_NIL;
+        } else {
+            ScmObj attrs = ((vm->numVals > 1)
+                            ? vm->vals[0] /* second value */
+                            : SCM_NIL);
+            r = Scm_MakeLazyPair(val, generator, attrs);
+        }
+    } else {
+        /* Generator returns lazy pair diretly.
+           Check LAZY_PAIR_P first to avoid forcing it */
+        if (SCM_LAZY_PAIR_P(val) || SCM_LISTP(val)) {
+            r = val;
+        }
+        /* If lcons's second argument doesn't yield a () or a pair,
+           we just treat it as if () is returned.  Since it is delayed
+           evaluation, raising error won't help. */
+    }
+    vm->numVals = 1; /* make sure the extra vals won't leak out */
+    if (extra_frame_pushed) Scm__VMUnprotectStack(vm);
+    return r;
+}
+
+ScmObj Scm_GeneratorToLazyPair(ScmObj generator)
+{
+    /* A check to eliminate the caller accidentally create a 'marked' lazy
+       pair.  Using cons as a generator to mark it is just a provisional
+       solution, so do not count on it. */
+    if (SCM_PAIRP(generator)) {
+        Scm_Error("generator must be a procedure, but got: %S", generator);
+    }
+    return generator_to_lazy_pair(generator);
+}
+
+#define REAL_LAZY_PAIR(obj) \
+    (ScmRealLazyPair*)((char*)(obj) - offsetof(ScmRealLazyPair, data))
+#define XPAIR_DESC() \
+    (ScmAtomicWord)SCM_CLASS2TAG(Scm__GetDefaultExtendedPairDesctiptor())
+
 /* Force a lazy pair.
    NB: When an error occurs during forcing, we release the lock of the
    pair, so that the pair can be forced again.  However, the generator
@@ -426,8 +569,9 @@ ScmObj Scm_MakeLazyPair(ScmObj item, ScmObj generator)
    is to mark the pair 'unforcible' permanently, by lp->owner == (AO_t)2,
    and let subsequent attempt of forcing the pair fail.
  */
-ScmObj Scm_ForceLazyPair(volatile ScmLazyPair *lp)
+ScmObj Scm_ForceLazyPair(volatile ScmLazyPair *obj)
 {
+    volatile ScmRealLazyPair *lp = REAL_LAZY_PAIR(obj);
     static const ScmTimeSpec req = {0, 1000000};
     ScmTimeSpec rem;
     ScmVM *vm = Scm_VM();
@@ -436,35 +580,24 @@ ScmObj Scm_ForceLazyPair(volatile ScmLazyPair *lp)
     do {
         if (AO_compare_and_swap_full(&lp->owner, zero, SCM_WORD(vm))) {
             /* Here we own the lazy pair. */
-            ScmObj item = lp->item;
-            /* Calling generator might change VM state, so we protect
-               incomplete stack frame if there's any. */
-            int extra_frame_pushed = Scm__VMProtectStack(vm);
-            SCM_UNWIND_PROTECT {
-                ScmObj val = Scm_ApplyRec0(lp->generator);
-                ScmObj newgen = (vm->numVals == 1)? lp->generator : vm->vals[0];
-                vm->numVals = 1; /* make sure the extra val won't leak out */
+            volatile ScmObj item = lp->data.item;
+            volatile ScmObj attrs = SCM_NIL;
+            if (SCM_MVBOXP(item)) {
+                attrs = SCM_MVBOX_VALUES(item)[1];
+                item = SCM_MVBOX_VALUES(item)[0];
+            }
 
-                if (SCM_EOFP(val)) {
-                    lp->item = SCM_NIL;
-                    lp->generator = SCM_NIL;
-                } else {
-                    ScmObj newlp = Scm_MakeLazyPair(val, newgen);
-                    lp->item = newlp;
-                    lp->generator = SCM_NIL;
-                }
+            SCM_UNWIND_PROTECT {
+                lp->data.item = generator_to_lazy_pair(lp->data.generator);
+                lp->data.generator = attrs;
+                lp->owner = XPAIR_DESC();
                 AO_nop_full();
-                SCM_SET_CAR(lp, item);
-                /* We don't need barrier here. */
-                lp->owner = (ScmAtomicWord)1;
+                SCM_SET_CAR_UNCHECKED(obj, item); /* Overwrite LazyPair tag */
             } SCM_WHEN_ERROR {
                 lp->owner = (ScmAtomicWord)0; /*NB: See above about error handling*/
                 SCM_NEXT_HANDLER;
             } SCM_END_PROTECT;
-            if (extra_frame_pushed) {
-                Scm__VMUnprotectStack(vm);
-            }
-            return SCM_OBJ(lp); /* lp is now an (extended) pair */
+            return SCM_OBJ(&lp->data); /* lp is now an (extended) pair */
         }
         /* Check if we're already working on forcing this pair.  Unlike
            force/delay, We don't allow recursive forcing of lazy pair.
@@ -478,73 +611,13 @@ ScmObj Scm_ForceLazyPair(volatile ScmLazyPair *lp)
         }
         /* Somebody's already working on forcing.  Let's wait for it
            to finish, or to abort. */
-        while (SCM_HTAG(lp) == 7 && lp->owner != 0) {
+        while (SCM_HTAG(obj) == 7 && lp->owner != 0) {
             Scm_NanoSleep(&req, &rem);
         }
     } while (lp->owner == 0); /* we retry if the previous owner abandoned. */
     return SCM_OBJ(lp);
 }
 
-/* Extract item and generator from lazy pair OBJ, without forcing it.
-   If OBJ is a lazy pair, item and generator is filled and TRUE is returned.
-   If OBJ is an ordinary pair (including the case that it was a lazy pair
-   but forced during execution of Scm_DecomposeLazyPair), returns its CAR
-   and a generator that returns its CDR.
-   Otherwise, returns FALSE.  */
-static ScmObj dummy_gen(ScmObj *args SCM_UNUSED,
-                        int nargs SCM_UNUSED,
-                        void *data)
-{
-    ScmObj item;
-    ScmObj generator;
-    if (Scm_DecomposeLazyPair(SCM_OBJ(data), &item, &generator)) {
-        return Scm_Values2(item, generator);
-    } else {
-        return Scm_Values2(SCM_EOF, SCM_FALSE);
-    }
-}
-
-int Scm_DecomposeLazyPair(ScmObj obj, ScmObj *item, ScmObj *generator)
-{
-    if (SCM_LAZY_PAIR_P(obj)) {
-        volatile ScmLazyPair *lp = SCM_LAZY_PAIR(obj);
-        static const ScmTimeSpec req = {0, 1000000};
-        ScmTimeSpec rem;
-        ScmVM *vm = Scm_VM();
-        ScmAtomicWord zero = 0;	/* Need to use C11 intrinsic */
-
-        for (;;) {
-            if (AO_compare_and_swap_full(&lp->owner, zero, SCM_WORD(vm))) {
-                *item = lp->item;
-                *generator = lp->generator;
-                AO_nop_full();
-                lp->owner = 0;
-                return TRUE;
-            }
-            if (lp->owner == (ScmAtomicWord)1) {
-                /* Somebody else has forced OBJ.  In the typical cases
-                   where we call this function for co-recursive lazy
-                   algorithms, this situation rarely happens.   We fallthrough
-                   to the SCM_PAIRP check below to return appropriate
-                   values. */
-                SCM_ASSERT(SCM_HTAG(lp) != 7);
-                break;
-            }
-            Scm_NanoSleep(&req, &rem);
-        }
-        /*FALLTHROUGH*/
-    }
-    if (SCM_PAIRP(obj)) {
-        ScmObj next;
-        *item = SCM_CAR(obj);
-        next = SCM_NULLP(SCM_CDR(obj)) ? SCM_EOF : SCM_CDR(obj);
-        *generator = Scm_MakeSubr(dummy_gen, (void*)next, 0, 0, SCM_FALSE);
-        return TRUE;
-    } else {
-        return FALSE;
-    }
-}
-
 int Scm_PairP(ScmObj x)
 {
     if (SCM_LAZY_PAIR_P(x)) {
@@ -554,5 +627,3 @@ int Scm_PairP(ScmObj x)
         return FALSE;
     }
 }
-
-#endif /* GAUCHE_LAZY_PAIR */
