@@ -1,7 +1,7 @@
 ;;;
 ;;; libdict.scm - dictionary-like data structures (hashtables etc.)
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -130,7 +130,7 @@
 
 (select-module gauche.internal)
 (define-cproc %current-recursive-hash (:optional obj) Scm_CurrentRecursiveHash)
-  
+
 (select-module gauche)
 (define-cproc hash-salt () ::<fixnum> Scm_HashSaltRef)
 
@@ -221,10 +221,18 @@
        (error "make-hash-table requires a comparator or \
                one of the symbols in eq?, eqv?, equal? or string=?, but got:"
               comparator))
+     ;; If comparator's equality predicate is eq?/eqv?, we ignore
+     ;; the its hash function and force using eq-/eqv-hash,
+     ;; as permitted in srfi-125.  It allows object that doesn't have
+     ;; custom hash method can still be used with hashtables based on
+     ;; (make-eq[v]-comparator).
+     ;; https://github.com/shirok/Gauche/issues/708
      (cond
-      [(eq? comparator eq-comparator)
+      [(or (eq? comparator eq-comparator)
+           (eq? (comparator-equality-predicate comparator) eq?))
        (make-hash-table 'eq? init-size)]
-      [(eq? comparator eqv-comparator)
+      [(or (eq? comparator eqv-comparator)
+           (eq? (comparator-equality-predicate comparator) eqv?))
        (make-hash-table 'eqv? init-size)]
       [(eq? comparator equal-comparator)
        (make-hash-table 'equal? init-size)]
@@ -307,8 +315,7 @@
   (dict-pop! hash Scm_HashCoreSearch SCM_HASH_TABLE_CORE))
 
 (inline-stub
- (define-cfn hash-table-iter (args::ScmObj* nargs::int data::void*) :static
-   (cast void nargs) ; suppress unused var warning
+ (define-cfn hash-table-iter (args::ScmObj* _::int data::void*) :static
    (let* ([iter::ScmHashIter* (cast ScmHashIter* data)]
           [e::ScmDictEntry* (Scm_HashIterNext iter)]
           [eofval (aref args 0)])
@@ -324,9 +331,9 @@
     (return (Scm_MakeSubr hash_table_iter iter 1 0 '"hash-table-iterator"))))
 
 (select-module gauche)
-(define-cproc hash-table-copy (ht::<hash-table> :optional mutable?)
-  ;; we don't have immutable hash table, and ignore mutable? argument (srfi-125)
-  (cast void mutable?) ; suppress unused var warning
+(define-cproc hash-table-copy (ht::<hash-table> :optional _)
+  ;; we don't have immutable hash table, and ignore the optional 
+  ;; mutable? argument (srfi-125)
   (return (Scm_HashTableCopy ht)))
 (define-cproc hash-table-keys (ht::<hash-table>)   Scm_HashTableKeys)
 (define-cproc hash-table-values (ht::<hash-table>) Scm_HashTableValues)
@@ -421,7 +428,7 @@
 (define (hash-table-delete!-r7 ht . keys) ; r7rs
   (fold (^[key count] (if (hash-table-delete! ht key) (+ count 1) count))
         0 keys))
-        
+
 (define (hash-table-update!/default ht key updater default) ;r7rs
   (hash-table-update! ht key updater default))
 
@@ -467,7 +474,7 @@
 
 (define (hash-table-map!-r7 proc ht) ; r7rs
   (assume-type ht <hash-table>)
-  (hash-table-for-each ht (^[k v] (hash-table-put! ht k (proc v)))))
+  (hash-table-for-each ht (^[k v] (hash-table-put! ht k (proc k v)))))
 
 (define (hash-table-map->list-r7 proc ht) ; r7rs
   (hash-table-map ht proc))
@@ -484,12 +491,12 @@
   (hash-table-for-each ht proc))
 
 (define (hash-table-fold-r7 kons knil ht) ; r7rs
-  (hash-table-fold ht cons knil))
+  (hash-table-fold ht kons knil))
 
 (define (hash-table-prune!-r7 proc ht) ; r7rs
   (hash-table-for-each ht (^[k v] (when (proc k v) (hash-table-delete! ht k)))))
 
-(define (hash-table-seek ht pred succ fail)
+(define (hash-table-seek ht pred fail succ)
   (assume-type ht <hash-table>)
   (let1 i ((with-module gauche.internal %hash-table-iter) ht)
     (let loop ()
@@ -501,10 +508,10 @@
 ;; hash-table-find.  This doesn't align with other '*-find' API in a way that
 ;; it returns the result of PRED.
 (define (hash-table-find ht pred :optional (failure (^[] #f)))
-  (hash-table-seek ht pred (^[r k v] r) failure))
-  
+  (hash-table-seek ht pred failure (^[r k v] r)))
+
 (define (hash-table-find-r7 pred ht failure) ; r7rs
-  (hash-table-seek ht pred (^[r k v] r) failure))
+  (hash-table-seek ht pred failure (^[r k v] r)))
 
 (define (hash-table-count-r7 pred ht)   ; r7rs
   ($ hash-table-fold ht
@@ -591,8 +598,8 @@
                      (^[k v] (let1 w (hash-table-get larger k unique)
                                (or (eq? unique w)
                                    (not (value=? v w)))))
-                     (^[r k v] #f)
-                     (^[] #t)))
+                     (^[] #t)
+                     (^[r k v] #f)))
 
   ;; Check comparator compatibility
   (let ([c1 (hash-table-comparator h1)]
@@ -601,7 +608,7 @@
           [(or c1 c2) (error "hash-tables with different comparators can't be \
                               compared:" h1 h2)]
           [else (error "hash-tables don't have comparators and can't be \
-                        compared:" h1 h2)]))  
+                        compared:" h1 h2)]))
   ;; Let's start
   (if (eq? h1 h2)
     0                 ;fast path
@@ -615,6 +622,7 @@
 (define (hash-table=? value-cmpr h1 h2)
   (or (eq? h1 h2)
       (and (= (hash-table-num-entries h1) (hash-table-num-entries h2))
+           (equal? (hash-table-comparator h1) (hash-table-comparator h2))
            (zero? (hash-table-compare-as-sets h1 h2
                                               (cut =? value-cmpr <> <>)
                                               100))))) ;; any non-zero number
@@ -708,8 +716,7 @@
       (return '#f))))
 
 (inline-stub
- (define-cfn tree-map-iter (args::ScmObj* nargs::int data::void*) :static
-   (cast void nargs) ; suppress unused var warning
+ (define-cfn tree-map-iter (args::ScmObj* _::int data::void*) :static
    (let* ([iter::ScmTreeIter* (cast ScmTreeIter* data)]
           [e::ScmDictEntry*   (?: (SCM_FALSEP (aref args 1))
                                   (Scm_TreeIterNext iter)
