@@ -1,7 +1,7 @@
 ;;;
 ;;; libeval.scm - eval, load and related stuff
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,7 +35,6 @@
 
 (inline-stub
  (declcode (.include <gauche/vminsn.h>
-                     <gauche/class.h>
                      <gauche/priv/readerP.h>)))
 
 (declare (keep-private-macro autoload add-load-path))
@@ -70,7 +69,8 @@
                                          (suffixes *load-suffixes*)
                                          (error-if-not-found #t)
                                          (environment #f)
-                                         (ignore-coding #f))
+                                         (ignore-coding #f)
+                                         (main-script #f))
   ;; NB: 'File not found' error is handled in find-load-file.
   (and-let* ([r (find-load-file file paths suffixes
                                 :error-if-not-found error-if-not-found
@@ -80,6 +80,10 @@
            [hooked? (pair? (cddr r))]
            [opener (if hooked? (caddr r) open-input-file)]
            [port (guard (e [else e]) (opener path))])
+      (when main-script
+        ;; record full path of the script
+        (script-file (sys-normalize-pathname path
+                                             :absolute #t :canonicalize #t)))
       (when (%load-verbose?)
         (format (current-error-port) ";;~aLoading ~a~a...\n"
                 (make-string (* (length (current-load-history)) 2) #\space)
@@ -110,7 +114,8 @@
         [prev-next    (current-load-next)]
         [prev-reader-lexical-mode (reader-lexical-mode)]
         [prev-eval-situation (vm-eval-situation)]
-        [prev-read-context (current-read-context)])
+        [prev-read-context (current-read-context)]
+        [load-read-context (%new-read-context-for-load)])
 
     (define (setup-load-context)
       (when (port-closed? port) (error "port already closed:" port))
@@ -124,7 +129,6 @@
                (list #f))
              prev-history))
       (vm-eval-situation SCM_VM_LOADING)
-      (current-read-context (%new-read-context-for-load))
       (%record-load-stat (or (current-load-path) "(unnamed source)")))
 
     (define (restore-load-context)
@@ -139,6 +143,14 @@
       (%record-load-stat #f)
       (%port-unlock! port))
 
+    ;; Read a source form with load-read-context.  We don't use dynamic-wind,
+    ;; for the error from read will be captured by the guard and the context
+    ;; will be restored anyway.
+    (define (read+ port)
+      (current-read-context load-read-context)
+      (begin0 (read port)
+        (current-read-context prev-read-context)))
+
     (guard (e [else (let1 e2 (if (condition? e)
                                ($ make-compound-condition e
                                   $ make <load-condition-mixin>
@@ -153,7 +165,7 @@
       (when (eq? (gauche-character-encoding) 'utf-8)
         (when (eqv? (peek-char port) #\ufeff)
           (read-char port)))
-      (do ([s (read port) (read port)])
+      (do ([s (read+ port) (read+ port)])
           [(eof-object? s)]
         (eval s #f)))
     (restore-load-context)
@@ -206,12 +218,18 @@
 (select-module gauche)
 
 ;; API
+;; returns #<dlobj>
 (define-cproc dynamic-load (file::<string>
-                            :key (init-function #f)
+                            :key (init-function #t)
                             (export-symbols #f)); for backward compatibility
   (cast void export-symbols) ; suppress unused var warning
   (return (Scm_DynLoad file init_function 0)))
 
+;; API (experimental)
+;; returns #<dlptr>
+(define-cproc dlobj-get-entry-address (dlo::<dlobj> name::<string>)
+  Scm_DLOGetEntryAddress)
+
 ;; API
 (define-cproc provide (feature)   Scm_Provide)
 ;; API
@@ -479,20 +497,6 @@
             version prefix))
     (sys-exec (car goshes) (cons (car goshes) args))))
 
-;;;
-;;; Parameter access
-;;;
-
-;; Parameter internal API
-;; These will be called by the public API in gauche.parameter.
-(select-module gauche.internal)
-
-(define-cproc %primitive-parameter-ref (p::<primitive-parameter>)
-  (return (Scm_PrimitiveParameterRef (Scm_VM) p)))
-(define-cproc %primitive-parameter-set! (p::<primitive-parameter> val)
-  (return (Scm_PrimitiveParameterSet (Scm_VM) p val)))
-
-
 ;;;
 ;;; System termination
 ;;;
@@ -513,11 +517,10 @@
  (initcode
   (Scm_BindPrimitiveParameter (Scm_GaucheModule) "exit-handler"
                               SCM_FALSE 0)))
-(%primitive-parameter-set! exit-handler 
-                           (^[code fmt args]
-                             (when fmt
-                               (apply format (standard-error-port) fmt args)
-                               (newline (standard-error-port)))))
+(exit-handler (^[code fmt args]
+                (when fmt
+                  (apply format (standard-error-port) fmt args)
+                  (newline (standard-error-port)))))
 
 ;; API
 (define-in-module gauche (exit :optional (code 0) (fmt #f) :rest args)
@@ -554,61 +557,10 @@
 ;;; Some system introspection
 ;;;
 
-(select-module gauche)
 ;; API
 ;; Obtain info about gauche itself
 
-(inline-stub
- (declcode "#include \"paths.c\"")
- (declcode "#include \"paths_arch.c\"")  ; generated by genconfig
- (define-cfn Scm_HostArchitecture () :: (const char*)
-   (return gauche_arch))
- (define-cfn Scm_LibraryDirectory ()
-   (C: "static ScmObj dir = SCM_FALSE;")
-   (when (SCM_FALSEP dir)
-     (set! dir (Scm_MakeString (replace_install_dir gauche_lib_dir
-                                                    Scm_Error)
-                               -1 -1 
-                               (logior SCM_STRING_COPYING
-                                       SCM_STRING_IMMUTABLE))))
-   (return dir))
- (define-cfn Scm_ArchitectureDirectory ()
-   (C: "static ScmObj dir = SCM_FALSE;")
-   (when (SCM_FALSEP dir)
-     (set! dir (Scm_MakeString (replace_install_dir gauche_arch_dir
-                                                    Scm_Error)
-                               -1 -1 
-                               (logior SCM_STRING_COPYING
-                                       SCM_STRING_IMMUTABLE))))
-   (return dir))
- (define-cfn Scm_SiteLibraryDirectory ()
-   (C: "static ScmObj dir = SCM_FALSE;")
-   (when (SCM_FALSEP dir)
-     (set! dir (Scm_MakeString (replace_install_dir gauche_site_lib_dir
-                                                    Scm_Error)
-                               -1 -1 
-                               (logior SCM_STRING_COPYING
-                                       SCM_STRING_IMMUTABLE))))
-   (return dir))
- (define-cfn Scm_SiteArchitectureDirectory ()
-   (C: "static ScmObj dir = SCM_FALSE;")
-   (when (SCM_FALSEP dir)
-     (set! dir (Scm_MakeString (replace_install_dir gauche_site_arch_dir
-                                                    Scm_Error)
-                               -1 -1 
-                               (logior SCM_STRING_COPYING
-                                       SCM_STRING_IMMUTABLE))))
-   (return dir))
- ;; TRANSIENT: Make it static in 1.0; we keep it public for ABI compatibility.
- (define-cfn Scm__RuntimeDirectory ()
-   (C: "static ScmObj dir = SCM_FALSE;")
-   (when (SCM_FALSEP dir)
-     (set! dir (Scm_MakeString (get_install_dir Scm_Error)
-                               -1 -1 
-                               (logior SCM_STRING_COPYING
-                                       SCM_STRING_IMMUTABLE))))
-   (return dir))
- )
+(select-module gauche)
 
 (define-cproc gauche-version () ::<const-cstring> (return GAUCHE_VERSION))
 (define-cproc gauche-architecture () ::<const-cstring> Scm_HostArchitecture)
@@ -619,18 +571,9 @@
 (define-cproc gauche-dso-suffix () ::<const-cstring> (return SHLIB_SO_SUFFIX))
 
 (select-module gauche.internal)
-(define-cproc %gauche-runtime-directory () Scm__RuntimeDirectory)
-(define-cproc %gauche-replace-runtime-directory (str::<const-cstring>)
-  ::<const-cstring>
-  (return (replace_install_dir str Scm_Error)))
-
-;; API
-;; Command line - R7RS adds 'command-line' procedure.  We provide it as
-;; a predefined parameter.
-(select-module gauche.internal)
-(inline-stub
- (initcode
-  (Scm_BindPrimitiveParameter (Scm_GaucheModule) "command-line" SCM_NIL 0)))
+(define-cproc %gauche-runtime-directory () Scm_RuntimeDirectory)
+(define-cproc %gauche-libgauche-path () Scm_LibgauchePath)
+(define-cproc %gauche-executable-path () Scm_ExecutablePath)
 
 ;;
 ;; External view of VM.
@@ -723,7 +666,7 @@
 ;; Returns ((<file1> <line1> <form1>) (<file2> <line2> <form2>) ...)
 ;; Where <form1> is a result of macro expansion of <form2> etc.
 ;; <fileN> and <lineN> can be #f if unknown.
-;; For public use, gauche.vm.debugger#debug-source-info is provided.
+;; For public use, debug-source-info below should be provided.
 (select-module gauche.internal)
 (define (%source-info obj)
   (let loop ([obj obj] [r '()])
@@ -734,3 +677,17 @@
           (reverse r `((,(car si) ,(cadr si) ,obj)))))
       '())))
 
+;; Returns source file info attached to OBJ.  If there's no info
+;; attached, returns #f.
+;; The return value can be either (<filename> <line-no>) or
+;; (<filename> <line-no> <original-form>).  The latter happens when
+;; OBJ is a result of macro expansion, and <orginal-form> being
+;; the original source form.
+(define-in-module gauche (debug-source-info obj)
+  (and-let1 sis ((with-module gauche.internal %source-info) obj)
+    (any (^[si] ;; si :: (<file> <line> <form>)
+           (and (car si) (cadr si)
+                (if (eq? (caddr si) obj)
+                  `(,(car si) ,(cadr si))
+                  si)))
+         (reverse sis))))
