@@ -1,7 +1,7 @@
 ;;;
 ;;; libfmt.scm - format
 ;;;
-;;;   Copyright (c) 2013-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -55,7 +55,10 @@
 ;;         | (R flags mincol padchar commachar interval)
 ;;         | (r flags mincol padchar commachar interval)
 ;;         | (F flags width digits scale ovfchar padchar)
+;;         | ($ flags digits pre-digits width padchar)
 ;;         | (* flags count)
+;;         | (? flags)
+;;         | (char flags <char> count) ; single-character insertion
 ;; flags : '() | (Cons #\@ flags) | (Cons #\: flags)
 
 ;; NB: This procedure is written in a way that all closures can be
@@ -64,10 +67,15 @@
 (define formatter-lex
   (let ()
     (define (fmtstr p) (port-attribute-ref p 'format-string))
-    (define (directive? c) (string-scan "sSaAcCwWdDbBoOxXrR*fF" c))
-    (define directive-param-spec ; (type max-#-of-params)
+    (define (directive? c) (string-scan "sSaAcCwWdDbBoOxXrR*?fF$~%tT|(){}[];" c))
+    (define directive-param-spec ; (type max-#-of-params [token-id])
       '((S 5) (A 5) (W 0) (C 0)
-        (D 4) (B 4) (O 4) (X 4) (x 4) (* 1) (R 5) (r 5) (F 5)))
+        (D 4) (B 4) (O 4) (X 4) (x 4) (* 1) (R 5) (r 5) (F 5) ($ 4) (? 0)
+        ;; single-character instertion
+        (~ 1 #\~) (% 1 #\newline) (T 1 #\tab) (|\|| 1 #\page)
+        ;; tokens for structures
+        (|(| 0 paren) (|)| 0 thesis) (|[| 1 bra) (|]| 0 cket)
+        (|{| 0 curly) (|}| 0 brace) (|\;| 0 sep)))
     (define (flag? c) (memv c '(#\@ #\:)))
     (define (next p)
       (rlet1 c (read-char p)
@@ -82,18 +90,16 @@
                        (directive p (next p)))]
                 [else (loop (cons c cs))]))))
     ;; NB: this keeps params and flags in reverse order.
-    ;; check-param 
+    ;; check-param reverses them.
     (define (directive p c)
       (let loop ([params '()] [c c])
         (receive (param c) (fc-param p c params)
           (cond [param (loop (cons param params) c)]
                 [(directive? c)
-                 => (^_  (let1 directive
-                             ($ string->symbol $ string
-                                $ if (memv c '(#\x #\r)) c (char-upcase c))
-                           (acons directive params (init p))))]
-                [(eqv? c #\~) (cons (string c) (init p))]
-                [(eqv? c #\%) (cons "\n" (init p))] ;TODO: handle param
+                 (let1 directive
+                     ($ string->symbol $ string
+                        $ if (memv c '(#\x #\r)) c (char-upcase c))
+                   (acons directive params (init p)))]
                 [else (errorf "Invalid format directive character `~a' in ~s"
                               c (fmtstr p))]))))
     ;; initial state
@@ -151,12 +157,22 @@
                     [(flag? (car ps)) (outer (cdr ps) (cons (car ps) fs))]
                     [(eq? (car ps) 'empty) (outer (cdr ps) fs)]
                     [else (values fs (reverse! ps))]))
-          (let1 spec (assq (car directive) directive-param-spec)
-            (unless (length<=? params (cadr spec))
+          (receive (nparams type)
+              (match (assq (car directive) directive-param-spec)
+                [(_ nparams) (values nparams (car directive))]
+                [(_ nparams dir) (values nparams dir)])
+            (unless (length<=? params nparams)
               (errorf "Too many parameters for directive `~a' in ~s"
-                      (car directive) (fmtstr p))))
-          (list* (car directive) flags params))
+                      (car directive) (fmtstr p)))
+            (if (char? type)
+              (char-node flags params type)
+              (list* type flags params))))
         directive))
+    ;; single character node optimization
+    (define (char-node flags params char)
+      (cond [(null? params) (string char)]
+            [(integer? (car params)) (make-string (car params) char)]
+            [else (list* 'char flags char params)]))
 
     ;; Body of formatter-lex
     (^[format-str]
@@ -184,7 +200,10 @@
 ;;      | (X flags mincol padchar commachar interval)
 ;;      | (x flags mincol padchar commachar interval)
 ;;      | (F flags width digits scale ovfchar padchar)
+;;      | ($ flags digits pre-digits width padchar)
 ;;      | (* flags count)
+;;      | (? flags)
+;;      | (char flags <char> count)
 ;;
 ;; argcnt : An integer if the formatter takes fixed number of arguments,
 ;;          #f otherwise.
@@ -228,7 +247,7 @@
             [(string? (car ds)) (string-node (car ds) (cdr ds) r cnt)]
             [(eq? (caar ds) '*) (jump-node (car ds) (cdr ds) r cnt)]
             [else (simple-node (car ds) (cdr ds) r cnt)]))
-    
+
     (^[directives]
       (let1 branches (parse directives '() 0)
         (if (and (pair? branches) (null? (cdr branches)))
@@ -266,6 +285,12 @@
     (rlet1 arg (cadr argptr)
       (set! (cdr argptr) (cddr argptr)))))
 
+;; returns the current arg, without changing pointer
+(define (fr-peek-arg fmtstr argptr)
+  (if (null? (cdr argptr))
+    (errorf "Running out arguments for format string ~s" fmtstr)
+    (cadr argptr)))
+
 ;; check if we used all args.  if we've reordered args, this always succeeds.
 (define (fr-args-used? argptr)
   (or (caar argptr)
@@ -312,7 +337,7 @@
             (error "Bad type of argument for the format parameter ~a: ~s"
                    ',var ,var))))
   (define (gen-vbind var default :optional (check #f))
-    `(,var 
+    `(,var
       (if (eq? ,var 'variable)
         ,(if check
            `(rlet1 ,var (fr-next-arg! fmtstr argptr)
@@ -333,7 +358,7 @@
              (let* ,vbinds
                ,@body))
            (^[argptr port ctrl] ,@body))))))
-           
+
 ;; ~S, ~A, ~W
 ;; NB: ~W doesn't take params, so justifying part won't be called for ~W.
 (define (make-format-expr fmtstr params flags printer)
@@ -368,7 +393,7 @@
                (display s port)
                (unless (has-@? flags)
                  (dotimes [_ npad] (write-char padchar port))))))))))
-        
+
 (define (chop-and-out str limit flags port)
   (if (has-:? flags)
     (begin (display (substring str 0 (- limit 4)) port)
@@ -381,7 +406,7 @@
   (define (char-formatter writer)
     (^[argptr port ctrl]
       (let1 c (fr-next-arg! fmtstr argptr)
-        (unless (char? c) 
+        (unless (char? c)
           (error "Character required for ~c format directive, but got:" c))
         (writer c port))))
   (char-formatter (if (has-@? flags) write display)))
@@ -462,35 +487,100 @@
 ;; @ flag is used to force plus sign (CL)
 ;; : flag is used for notational rounding (Gauche only)
 (define (make-format-flo fmtstr params flags kind)
-  (define (do-fmt n w d ovchar padchar flags port)
-    (let* ([s (number->string n 10
-                              (cond-list
-                               [(has-@? flags) 'plus]
-                               [(has-:? flags) 'notational])
-                              d)]
-           [l (string-length s)])
-      (if (< w l)
-        (if ovchar
-          (dotimes [w] (write-char ovchar port))
-          (display s port))
-        (begin
-          (dotimes [(- w l)] (write-char padchar port))
-          (display s port)))))
   ($ with-format-params ([width 0]
                          [digits -1]
                          [scale 0]
                          [ovchar #f]
                          [padchar #\space])
      (let1 arg (fr-next-arg! fmtstr argptr)
+       (cond [(real? arg)
+              (flo-fmt (inexact (* arg (expt 10 scale)))
+                       width digits 0 ovchar padchar
+                       (has-@? flags) (has-:? flags) #f port)]
+             [(complex? arg)
+              (let1 s (call-with-output-string
+                        (^p
+                         (flo-fmt (* (real-part arg) (expt 10 scale))
+                                  0 digits 0 ovchar padchar
+                                  (has-@? flags) (has-:? flags) #f p)
+                         (flo-fmt (* (imag-part arg) (expt 10 scale))
+                                  0 digits 0 ovchar padchar
+                                  #t (has-:? flags) #f p)
+                         (display "i" p)))
+                (let1 len (string-length s)
+                  (when (< len width)
+                    (dotimes [(- width len)]  (write-char padchar port)))
+                  (display s port)))]
+             [else
+              (let1 sarg (write-to-string arg display)
+                (let1 len (string-length sarg)
+                  (when (< len width)
+                    (dotimes [(- width len)] (write-char padchar port)))
+                  (display sarg port)))]))))
+
+;; ~$ (monetary floating point)
+;; The order of parameters and their defaults differ from ~F
+;; We use notatational rounding (:-flag is used for sign-front.
+(define (make-format-currency fmtstr params flags)
+  ($ with-format-params ([digits 2]
+                         [pre-digits 1]
+                         [width 0]
+                         [padchar #\space])
+     (let1 arg (fr-peek-arg fmtstr argptr)
        (if (real? arg)
-         (do-fmt (inexact (* arg (expt 10 scale)))
-                 width digits ovchar padchar flags port)
-         (let1 sarg (write-to-string arg display)
-           (let1 len (string-length sarg)
-             (when (< len width)
-               (dotimes [(- width len)] (write-char padchar port)))
-             (display sarg port)))))))
-           
+         (flo-fmt (inexact (fr-next-arg! fmtstr argptr))
+                  width digits pre-digits
+                  #f padchar (has-@? flags) #t (has-:? flags) port)
+         ;; if arg isn't real, use ~wD.
+         (format-num-body fmtstr argptr port ctrl 10 #f
+                          '() width #\space #\, 3 #\.)))))
+
+(define (flo-fmt val width digits pre-digits
+                 ovchar padchar plus? notational? sign-front? port)
+  (let* ([s0 (number->string val 10
+                             (cond-list
+                              [plus? 'plus]
+                              [notational? 'notational])
+                             digits)]
+         [s (if (> pre-digits 0)        ;special for ~$
+              (let* ([m (#/^([+-]?)(\d+)/ s0)]
+                     [pre (m 2)])
+                (if (> pre-digits (string-length pre))
+                  (string-append
+                   (m 1)
+                   (make-string (- pre-digits (string-length pre)) #\0)
+                   pre (m 'after))
+                  s0))
+              s0)]
+         [l (string-length s)])
+    (if (< width l)
+      (if ovchar
+        (dotimes [width] (write-char ovchar port))
+        (display s port))
+      (receive (pre-sign body)
+          (if (and sign-front? (memv (string-ref s 0) '(#\+ #\-)))
+            (values (string-copy s 0 1) (string-copy s 1))
+            (values "" s))
+        (display pre-sign port)
+        (dotimes [(- width l)] (write-char padchar port))
+        (display body port)))))
+
+;; ~?
+(define (make-format-recur fmtstr flags)
+  (define (fcompile str)
+    (unless (string? str)
+      (error "Argument for ~? must be a string, but got" str))
+    ($ formatter-compile-rec str $ formatter-parse $ formatter-lex str))
+  (if (has-@? flags)
+    ;; take
+    (^[argptr port ctrl]
+      (let1 formatter (fcompile (fr-next-arg! fmtstr argptr))
+        (formatter argptr port ctrl)))
+    (^[argptr port ctrl]
+      (let* ([formatter (fcompile (fr-next-arg! fmtstr argptr))]
+             [xargptr (fr-make-argptr (fr-next-arg! fmtstr argptr))])
+        (formatter xargptr port ctrl)))))
+
 ;; ~*
 (define (make-format-jump fmtstr params flags)
   ($ with-format-params ([count 1])
@@ -499,6 +589,11 @@
          (fr-jump-arg! argptr count)
          (fr-jump-arg-relative! argptr count)))))
 
+;; ~t, ~%, ~~
+(define (make-format-single fmtstr ch params flags)
+  ($ with-format-params ([count 1])
+     (display (make-string count ch) port)))
+
 ;; Tree -> Formatter
 ;; src : source format string for error message
 (define (formatter-compile-rec src tree)
@@ -517,9 +612,12 @@
     [('x fs . ps) (make-format-num src ps fs 16 #f)]
     [('X fs . ps) (make-format-num src ps fs 16 #t)]
     [('F fs . ps) (make-format-flo src ps fs 'F)]
+    [('$ fs . ps) (make-format-currency src ps fs)]
     [((or 'R 'r) fs . ps) (make-format-r src ps fs (eq? (car tree) 'R))]
+    [('? fs)      (make-format-recur src fs)]
     [('* fs . ps) (make-format-jump src ps fs)]
-    [_ (error "boo!")]))
+    [('char fs c . ps) (make-format-single src c ps fs)]
+    [_ (error "Unsupported formatter directive:" tree)]))
 
 ;; Toplevel compiler
 ;; Returns formatter procedure
