@@ -1,7 +1,7 @@
 ;;;
 ;;; libio.scm - builtin port and I/O procedures
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,7 +35,7 @@
 
 (inline-stub
  (declcode (.include <gauche/vminsn.h>
-                     <gauche/class.h>
+                     <gauche/exception.h>
                      <gauche/priv/portP.h>
                      <gauche/priv/writerP.h>
                      <stdlib.h>
@@ -96,6 +96,11 @@
 (define-cproc standard-error-port (:optional (p::<output-port>? #f))
   (return (?: p (Scm_SetStderr p) (Scm_Stderr))))
 
+(inline-stub
+ (initcode
+  (Scm_BindPrimitiveParameter (Scm_GaucheModule)
+                              "current-trace-port"
+                              (Scm_Stderr) 0)))
 
 ;;
 ;; Query and low-level properties
@@ -106,9 +111,15 @@
 (define-cproc port-name (port::<port>) Scm_PortName)
 (define-cproc port-current-line (port::<port>) ::<fixnum> Scm_PortLine)
 
-(define-cproc port-file-number (port::<port>)
+(define-cproc port-file-number (port::<port> :optional (dup?::<boolean> #f))
   (let* ([i::int (Scm_PortFileNo port)])
-    (return (?: (< i 0) SCM_FALSE (Scm_MakeInteger i)))))
+    (when (< i 0) (return SCM_FALSE))
+    (when dup?
+      (let* ([r::int 0])
+        (SCM_SYSCALL r (dup i))
+        (when (< r 0) (Scm_SysError "dup(2) failed"))
+        (set! i r)))
+    (return (Scm_MakeInteger i))))
 (define-cproc port-fd-dup! (dst::<port> src::<port>) ::<void> Scm_PortFdDup)
 
 (define-cproc port-attribute-set! (port::<port> key val)
@@ -116,9 +127,6 @@
 (define-cproc port-attribute-ref (port::<port> key :optional fallback)
   (setter port-attribute-set!)
   Scm_PortAttrGet)
-(define-cproc port-attribute-create! (port::<port> key
-                                      :optional (get #f) (set #f))
-  Scm_PortAttrCreate)
 (define-cproc port-attribute-delete! (port::<port> key)
   Scm_PortAttrDelete)
 (define-cproc port-attributes (port::<port>)
@@ -260,21 +268,41 @@
 
 (define-cproc open-input-fd-port (fd::<fixnum>
                                   :key (buffering #f)
-                                  (owner?::<boolean> #f)
+                                  (owner? #f)
                                   (name #f))
   (let* ([bufmode::int (Scm_BufferingMode buffering SCM_PORT_INPUT
                                           SCM_PORT_BUFFER_FULL)])
     (when (< fd 0) (Scm_Error "bad file descriptor: %ld" fd))
-    (return (Scm_MakePortWithFd name SCM_PORT_INPUT fd bufmode ownerP))))
+    (cond
+     [(SCM_EQ owner? 'dup)
+      (let* ([r::int 0])
+        (SCM_SYSCALL r (dup fd))
+        (when (< r 0) (Scm_SysError "dup(2) failed"))
+        (set! fd r))]
+     [(not (SCM_BOOLP owner?))
+      (Scm_Error "owner? argument must be either #f, #t or a symbol dup, \n\
+                  but go t%S" owner?)])
+    (return (Scm_MakePortWithFd name SCM_PORT_INPUT fd bufmode
+                                (not (SCM_FALSEP owner?))))))
 
 (define-cproc open-output-fd-port (fd::<fixnum>
                                    :key (buffering #f)
-                                   (owner?::<boolean> #f)
+                                   (owner? #f)
                                    (name #f))
   (let* ([bufmode::int (Scm_BufferingMode buffering SCM_PORT_OUTPUT
                                           SCM_PORT_BUFFER_FULL)])
     (when (< fd 0) (Scm_Error "bad file descriptor: %d" fd))
-    (return (Scm_MakePortWithFd name SCM_PORT_OUTPUT fd bufmode owner?))))
+    (cond
+     [(SCM_EQ owner? 'dup)
+      (let* ([r::int 0])
+        (SCM_SYSCALL r (dup fd))
+        (when (< r 0) (Scm_SysError "dup(2) failed"))
+        (set! fd r))]
+     [(not (SCM_BOOLP owner?))
+      (Scm_Error "owner? argument must be either #f, #t or a symbol dup, \n\
+                  but go t%S" owner?)])
+    (return (Scm_MakePortWithFd name SCM_PORT_OUTPUT fd bufmode
+                                (not (SCM_FALSEP owner?))))))
 
 ;; Buffered port
 (select-module gauche)
@@ -283,18 +311,18 @@
  ;; NB: the interface may be changed soon!!
  (define-cfn bufport-closer (p::ScmPort*) ::void :static
    (when (== (SCM_PORT_DIR p) SCM_PORT_OUTPUT)
-     (let* ((scmflusher (SCM_OBJ (ref (-> p src) buf data)))
-            (siz::int (cast int (- (ref (-> p src) buf current)
-                                   (ref (-> p src) buf buffer)))))
+     (let* ((scmflusher (SCM_OBJ (-> (PORT_BUF p) data)))
+            (siz::int (cast int (- (-> (PORT_BUF p) current)
+                                   (-> (PORT_BUF p) buffer)))))
        (when (> siz 0)
          (Scm_ApplyRec1 scmflusher
-                        (Scm_MakeString (ref (-> p src) buf buffer) siz siz
+                        (Scm_MakeString (-> (PORT_BUF p) buffer) siz siz
                                         (logior SCM_STRING_INCOMPLETE
                                                 SCM_STRING_COPYING))))
        (Scm_ApplyRec1 scmflusher SCM_FALSE))))
 
  (define-cfn bufport-filler (p::ScmPort* cnt::ScmSize) ::ScmSize :static
-   (let* ([scmfiller (SCM_OBJ (ref (-> p src) buf data))]
+   (let* ([scmfiller (SCM_OBJ (-> (PORT_BUF p) data))]
           [r (Scm_ApplyRec1 scmfiller (Scm_MakeInteger cnt))])
      (cond [(or (SCM_EOFP r) (SCM_FALSEP r)) (return 0)]
            [(not (SCM_STRINGP r))
@@ -302,7 +330,7 @@
      (let* ([b::(const ScmStringBody*) (SCM_STRING_BODY r)]
             [siz::ScmSize (SCM_STRING_BODY_SIZE b)])
        (when (> siz cnt) (set! siz cnt)) ; for safety
-       (memcpy (ref (-> p src) buf end) (SCM_STRING_BODY_START b) siz)
+       (memcpy (-> (PORT_BUF p) end) (SCM_STRING_BODY_START b) siz)
        (return (SCM_STRING_BODY_SIZE b)))))
  )
 
@@ -321,11 +349,10 @@
     (return (Scm_MakeBufferedPort SCM_CLASS_PORT SCM_FALSE SCM_PORT_INPUT TRUE (& bufrec)))))
 
 (inline-stub
- (define-cfn bufport-flusher (p::ScmPort* cnt::ScmSize forcep::int)
+ (define-cfn bufport-flusher (p::ScmPort* cnt::ScmSize _::int)
    ::ScmSize :static
-   (cast void forcep) ; suppress unused var warning
-   (let* ([scmflusher (SCM_OBJ (ref (-> p src) buf data))]
-          [s (Scm_MakeString (ref (-> p src) buf buffer) cnt cnt
+   (let* ([scmflusher (SCM_OBJ (-> (PORT_BUF p) data))]
+          [s (Scm_MakeString (-> (PORT_BUF p) buffer) cnt cnt
                              (logior SCM_STRING_INCOMPLETE SCM_STRING_COPYING))])
      (Scm_ApplyRec1 scmflusher s)
      (return cnt)))
@@ -355,22 +382,16 @@
 ;;   port source path.  Until then, be careful to name the ports.
 (select-module gauche)
 
-(define-cproc open-input-string (string::<string> :key (private?::<boolean> #f)
-                                                       name)
-  (let* ([p (Scm_MakeInputStringPort string private?)])
-    ;; NB: We don't have an API to set port name in the constructor; but
-    ;; this is a bad manner.  Don't duplicate this.
-    (unless (SCM_UNBOUNDP name)
-      (set! (-> (SCM_PORT p) name) name))
-    (return p)))
+(define-cproc open-input-string (string::<string>
+                                 :key (private?::<boolean> #f)
+                                      (name "(input string port)"))
+  (let* ([flags::u_long (?: private? SCM_PORT_STRING_PRIVATE 0)])
+    (return (Scm_MakeInputStringPortFull string name flags))))
 
-(define-cproc open-output-string (:key (private?::<boolean> #f) name)
-  (let* ([p (Scm_MakeOutputStringPort private?)])
-    ;; NB: We don't have an API to set port name in the constructor; but
-    ;; this is a bad manner.  Don't duplicate this.
-    (unless (SCM_UNBOUNDP name)
-      (set! (-> (SCM_PORT p) name) name))
-    (return p)))
+(define-cproc open-output-string (:key (private?::<boolean> #f)
+                                       (name "(output string port)"))
+  (let* ([flags::u_long (?: private? SCM_PORT_STRING_PRIVATE 0)])
+    (return (Scm_MakeOutputStringPortFull name flags))))
 
 (define-cproc get-output-string (oport::<output-port>) ;SRFI-6
   (return (Scm_GetOutputString oport 0)))
@@ -391,6 +412,17 @@
 ;; Miscellaneous
 ;;
 
+;; srfi-191
+(define-cproc port-has-port-position? (port::<port>) ::<boolean>
+  (return (Scm_PortPositionable port FALSE)))
+(define-cproc port-has-set-port-position!? (port::<port>) ::<boolean>
+  (return (Scm_PortPositionable port TRUE)))
+
+(define-cproc port-position (port::<port>)
+  (return (Scm_GetPortPosition port)))
+(define-cproc set-port-position! (port::<port> pos)
+  (return (Scm_SetPortPosition port pos)))
+
 (select-module gauche)
 (inline-stub
  (define-enum SEEK_SET)
@@ -408,12 +440,14 @@
 
 ;; useful for error messages
 (define (port-position-prefix port)
-  (if-let1 n (port-name port)
-    (let1 l (port-current-line port)
-      (if (positive? l)
-        (format #f "~s:line ~a: " n l)
-        (format #f "~s: " n))
-      "")))
+  (if (port? port)
+    (if-let1 n (port-name port)
+      (let1 l (port-current-line port)
+        (if (positive? l)
+          (format #f "~s:line ~a: " n l)
+          (format #f "~s: " n))
+        ""))
+    "???"))
 
 (select-module gauche.internal)
 
@@ -432,14 +466,11 @@
   PORT_WRITESS_P)
 
 (inline-stub
- (define-cfn write_state_allocate (klass::ScmClass* initargs) :static
-   (cast void klass)                    ; suppress unused var warning
-   (cast void initargs)                 ; suppress unused var warning
+ (define-cfn write_state_allocate (_::ScmClass* _) :static
    (return (SCM_OBJ (Scm_MakeWriteState NULL))))
 
- (define-cfn write_state_print (obj port::ScmPort* ctx::ScmWriteContext*)
+ (define-cfn write_state_print (obj port::ScmPort* _::ScmWriteContext*)
    ::void :static
-   (cast void ctx)                      ; suppress unused var warning
    (Scm_Printf port "#<write-state %p>" obj))
 
  (define-cclass <write-state>
@@ -454,11 +485,10 @@
 (define-cproc %port-write-state (port::<port>)
   (setter (port::<port> obj) ::<void>
           (if (SCM_WRITE_STATE_P obj)
-            (set! (-> port writeState) (SCM_WRITE_STATE obj))
-            (set! (-> port writeState) NULL)))
-  (return (?: (-> port writeState)
-              (SCM_OBJ (-> port writeState))
-              SCM_FALSE)))
+            (Scm_PortWriteStateSet port (SCM_WRITE_STATE obj))
+            (Scm_PortWriteStateSet port NULL)))
+  (let* ([r::ScmWriteState* (Scm_PortWriteState port)])
+    (return (?: r (SCM_OBJ r) SCM_FALSE))))
 
 (define-cproc %port-lock! (port::<port>) ::<void>
   (let* ([vm::ScmVM* (Scm_VM)])
@@ -530,15 +560,21 @@
 (define-cproc byte-ready? (:optional (port::<input-port> (current-input-port)))
   ::<boolean> Scm_ByteReady)
 
+(define u8-ready? byte-ready?)          ;R7RS
+
 (define-cproc read-byte (:optional (port::<input-port> (current-input-port)))
   (let* ([b::int])
     (SCM_GETB b port)
     (return (?: (< b 0) SCM_EOF (SCM_MAKE_INT b)))))
 
+(define read-u8 read-byte)              ;R7RS
+
 (define-cproc peek-byte (:optional (port::<input-port> (current-input-port)))
   (let* ([b::int (Scm_Peekb port)])
     (return (?: (< b 0) SCM_EOF (SCM_MAKE_INT b)))))
 
+(define peek-u8 peek-byte)              ;R7RS
+
 (define-cproc read-line (:optional (port::<input-port> (current-input-port))
                                    (allowbytestr #f))
   (let* ([r (Scm_ReadLine port)])
@@ -617,6 +653,23 @@
             (Scm_DStringPutz (& ds) buf nbytes)))
     (return (Scm_DStringGet (& ds) SCM_STRING_INCOMPLETE))))
 
+(define (port->string port)
+  (let1 out (open-output-string :private? #t)
+    (copy-port port out :unit 'byte)
+    (get-output-string out)))
+
+(define (port->list reader port)
+  (with-port-locking port
+    (^[]
+      (let loop ([obj (reader port)]
+                 [result '()])
+        (if (eof-object? obj)
+          (reverse! result)
+          (loop (reader port) (cons obj result)))))))
+
+(define (port->string-list port) (port->list (cut read-line <> #t) port))
+(define (port->sexp-list port)   (port->list read port))
+
 ;; Reader parameters
 (define-cproc reader-lexical-mode (:optional k)
   (if (SCM_UNBOUNDP k)
@@ -642,10 +695,10 @@
   Scm_DefineReaderDirective)
 
 (inline-stub
- (define-type <read-context> "ScmReadContext*" "read context"
+ (declare-stub-type <read-context> "ScmReadContext*" "read context"
    "SCM_READ_CONTEXT_P" "SCM_READ_CONTEXT" "")
 
- (define-type <read-reference> "ScmReadReference*" "read reference"
+ (declare-stub-type <read-reference> "ScmReadReference*" "read reference"
    "SCM_READ_REFERENCE_P" "SCM_READ_REFERENCE" "")
  )
 
@@ -671,33 +724,12 @@
 (define-in-module gauche read-with-shared-structure read)
 (define-in-module gauche read/ss read)
 
-;; srfi-105
-(select-module gauche.internal)
-(define (%xform-cexpr cex)
-  (define (simple-cexpr? op tail)
-    (cond [(not (pair? tail)) #f]
-          [(null? (cdr tail)) #t]
-          [(and (pair? (cdr tail)) (eq? op (cadr tail)))
-           (simple-cexpr? op (cddr tail))]
-          [else #f]))
-  (define (gather-args args) ; args is a list with even elements
-    (if (null? (cdr args))
-      args
-      (cons (car args) (gather-args (cddr args)))))
-  (cond [(not (pair? cex)) cex]         ;includes {} -> ()
-        [(null? (cdr cex)) (car cex)]   ; {e} -> e
-        [(and (pair? (cdr cex)) (null? (cddr cex))) cex] ; {x y} -> (x y)
-        [(and (pair? (cdr cex)) (pair? (cddr cex)) (symbol? (cadr cex))
-              (simple-cexpr? (cadr cex) (cddr cex)))
-         (cons* (cadr cex) (car cex) (gather-args (cddr cex)))]
-        [else (cons 'nfx cex)]))
-
 ;;;
 ;;; Output
 ;;;
 
 (inline-stub
- (define-type <write-controls> "ScmWriteControls*" "write controls"
+ (declare-stub-type <write-controls> "ScmWriteControls*" "write controls"
    "SCM_WRITE_CONTROLS_P" "SCM_WRITE_CONTROLS" "")
 
  (define-cfn parse-write-optionals (opt1 opt2
@@ -767,6 +799,8 @@
   (SCM_PUTB byte port)
   (return 1))
 
+(define write-u8 write-byte)            ;R7RS
+
 (define-cproc write-limited (obj limit::<fixnum>
                                  :optional (port (current-output-port)))
   ::<int> (return (Scm_WriteLimited obj port SCM_WRITE_WRITE limit)))
@@ -787,7 +821,7 @@
   (return (not (or (not (SCM_PTRP obj))
                    (SCM_NUMBERP obj)
                    (SCM_KEYWORDP obj)
-                   (and (SCM_SYMBOLP obj) (SCM_SYMBOL_INTERNED obj)) 
+                   (and (SCM_SYMBOLP obj) (SCM_SYMBOL_INTERNED obj))
                    (and (SCM_STRINGP obj) (== (SCM_STRING_SIZE obj) 0))
                    (and (SCM_VECTORP obj) (== (SCM_VECTOR_SIZE obj) 0))))))
 
@@ -810,6 +844,9 @@
          [(vector? obj)
           (dotimes [i (vector-length obj)]
             (%write-walk-rec (vector-ref obj i) port tab))]
+         [(box? obj)
+          (dotimes [i (box-arity obj)]
+            (%write-walk-rec (unbox-value obj i) port tab))]
          [else ; generic objects.  we go walk pass via write-object
           (write-object obj port)])
         ;; If circular-only, we don't count non-circular objects.
@@ -834,9 +871,7 @@
 
 (select-module gauche)
 (inline-stub
- (define-cfn write_controls_allocate (klass::ScmClass* initargs) :static
-   (cast void klass)                    ; suppress unused var warning
-   (cast void initargs)                 ; suppress unused var warning
+ (define-cfn write_controls_allocate (_::ScmClass* _) :static
    (return (SCM_OBJ (Scm_MakeWriteControls NULL))))
 
  ;; TODO: We want to treat <write-controls> as immutable structure, but
@@ -1077,10 +1112,3 @@
   (^[sym port ctx]
     (port-attribute-set! port 'reader-lexical-mode 'strict-r7)
     (values)))
-
-;; HIGHLY EXPERIMENTAL
-(define-reader-directive 'c-expr
-  (^[sym port ctx]
-    ((with-module gauche.internal vm-compiler-flag-set!)
-     SCM_COMPILE_ENABLE_CEXPR)
-    (values)))
