@@ -1,7 +1,7 @@
 ;;;
 ;;; liblist.scm - builtin list procedures
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -43,13 +43,16 @@
 (select-module scheme)
 (define-cproc pair? (obj) ::<boolean> :fast-flonum :constant
   (inliner PAIRP) SCM_PAIRP)
-(define-cproc cons (obj1 obj2) (inliner CONS) Scm_Cons)
+(define-cproc cons (obj1 obj2) ::<pair> (inliner CONS)
+  (return (SCM_PAIR (Scm_Cons obj1 obj2))))
 (define-cproc car (obj::<pair>) :constant
   (inliner CAR) (setter set-car!) SCM_CAR)
 (define-cproc cdr (obj::<pair>) :constant
   (inliner CDR) (setter set-cdr!) SCM_CDR)
-(define-cproc set-car! (obj::<pair> value) ::<void> SCM_SET_CAR)
-(define-cproc set-cdr! (obj::<pair> value) ::<void> SCM_SET_CDR)
+(define-cproc set-car! (obj::<pair> value) ::<void>
+  (Scm_SetCar (SCM_OBJ obj) value))
+(define-cproc set-cdr! (obj::<pair> value) ::<void>
+  (Scm_SetCdr (SCM_OBJ obj) value))
 
 (inline-stub
  "#define CXR_SETTER(PRE, pre, tail) \
@@ -59,14 +62,18 @@
   SCM_SET_C##PRE##R(cell, value);
 "
  )
-(define-cproc caar (obj) :fast-flonum :constant
-  (inliner CAAR) (setter (obj value) ::<void> (CXR_SETTER A a a)) Scm_Caar)
-(define-cproc cadr (obj) :fast-flonum :constant
-  (inliner CADR) (setter (obj value) ::<void> (CXR_SETTER A a d)) Scm_Cadr)
-(define-cproc cdar (obj) :fast-flonum :constant
-  (inliner CDAR) (setter (obj value) ::<void> (CXR_SETTER D d a)) Scm_Cdar)
-(define-cproc cddr (obj) :fast-flonum :constant
-  (inliner CDDR) (setter (obj value) ::<void> (CXR_SETTER D d d)) Scm_Cddr)
+(define-cproc caar (obj::<pair>) :fast-flonum :constant
+  (inliner CAAR) (setter (obj value) ::<void> (CXR_SETTER A a a))
+  (return (Scm_Caar (SCM_OBJ obj))))
+(define-cproc cadr (obj::<pair>) :fast-flonum :constant
+  (inliner CADR) (setter (obj value) ::<void> (CXR_SETTER A a d))
+  (return (Scm_Cadr (SCM_OBJ obj))))
+(define-cproc cdar (obj::<pair>) :fast-flonum :constant
+  (inliner CDAR) (setter (obj value) ::<void> (CXR_SETTER D d a))
+  (return (Scm_Cdar (SCM_OBJ obj))))
+(define-cproc cddr (obj::<pair>) :fast-flonum :constant
+  (inliner CDDR) (setter (obj value) ::<void> (CXR_SETTER D d d))
+  (return (Scm_Cddr (SCM_OBJ obj))))
 
 ;; NB: we avoid using getter-with-setter here, since
 ;;   - The current compiler doesn't take advantage of locked setters
@@ -107,12 +114,29 @@
 (%define-cxr cdddar cddr cdar)
 (%define-cxr cddddr cddr cddr)
 
+;; primitives for immutable pars
+(define-cproc ipair? (obj) ::<boolean> Scm_ImmutablePairP)
+(define-cproc ipair (car cdr) ::<pair>
+  (return (SCM_PAIR (Scm_MakeImmutablePair car cdr))))
+(define-cproc ilist (:rest args) ::<list>
+  (if (SCM_NULLP args)
+    (return SCM_NIL)
+    (let* ([h SCM_NIL] [t SCM_NIL])
+      (dopairs (cp args)
+        (if (SCM_NULLP t)
+          (set! h (Scm_MakeImmutablePair (SCM_CAR cp) SCM_NIL)
+                t h)
+          (let* ([p (Scm_MakeImmutablePair (SCM_CAR cp) SCM_NIL)])
+            (SCM_SET_CDR_UNCHECKED t p)
+            (set! t p))))
+      (return h))))
+
 (select-module scheme)
 (define-cproc null? (obj) ::<boolean> :fast-flonum :constant
   (inliner NULLP) SCM_NULLP)
 (define-cproc list? (obj) ::<boolean> :fast-flonum :constant
   SCM_PROPER_LIST_P)
-(define-cproc list (:rest args) (inliner LIST) (return args))
+(define-cproc list (:rest args) ::<list> (inliner LIST) (return args))
 
 (define-cproc length (list) ::<long> :constant (inliner LENGTH)
   (let* ([len::long (Scm_Length list)])
@@ -122,20 +146,25 @@
 (select-module gauche)
 (define-cproc length<=? (list k::<integer>) ::<boolean> :constant
   (if (SCM_INTP k)
-    (let* ([n::int (SCM_INT_VALUE k)])
+    (let* ([n::ScmSmallInt (SCM_INT_VALUE k)])
       (dolist [_ list] (when (<= (post-- n) 0) (return FALSE)))
       (return (<= 0 n)))
-    (return TRUE)))  ; k is bignum. it is impossible to have that long list.
+    ;; k is bignum. it is impossible to have that long list, but list
+    ;; can be circular, so we need to scan list entirely anyway.
+    (if (< (Scm_Sign k) 0)
+      (return FALSE)
+      (let* ([ln::ScmSmallInt (Scm_Length list)])
+        (return (>= ln 0))))))
 (define-cproc length=? (list k::<integer>) ::<boolean> :constant
   (if (SCM_INTP k)
-    (let* ([n::int (SCM_INT_VALUE k)])
+    (let* ([n::ScmSmallInt (SCM_INT_VALUE k)])
       (dolist [_ list] (when (<= (post-- n) 0) (return FALSE)))
       (return (== 0 n)))
     (return FALSE)))
 (define (length<? list k) (length<=? list (- k 1)))
 (define (length>? list k) (not (length<=? list k)))
 (define (length>=? list k) (not (length<? list k)))
-                  
+
 (select-module scheme)
 (define-cproc append (:rest lists) (inliner APPEND) Scm_Append)
 (define-cproc reverse (list::<list> :optional (tail ())) Scm_Reverse2)
@@ -148,7 +177,7 @@
 (define-cproc list-set! (lis k::<fixnum> v) ::<void>
   (let* ([p (Scm_ListTail lis k SCM_FALSE)])
     (if (SCM_PAIRP p)
-      (SCM_SET_CAR p v)
+      (Scm_SetCar p v)
       (Scm_Error "list-set!: index out of bound: %d" k))))
 (define-cproc list-ref (list k::<fixnum> :optional fallback) :constant
   (setter list-set!)
@@ -172,7 +201,7 @@
 ;;
 
 (select-module gauche)
-(define-cproc length+ (list) :constant ;; srfi-1
+(define-cproc length+ (list) ::<integer>? :constant ;; srfi-1
   (let* ([i::int (Scm_Length list)])
     (if (< i 0) (return SCM_FALSE) (return (Scm_MakeInteger i)))))
 
@@ -180,7 +209,8 @@
 (define-cproc dotted-list? (obj)   ::<boolean> :constant SCM_DOTTED_LIST_P)
 (define-cproc circular-list? (obj) ::<boolean> :constant SCM_CIRCULAR_LIST_P)
 (define-cproc make-list (len::<fixnum> :optional (fill #f)) Scm_MakeList)
-(define-cproc acons (caa cda cd) Scm_Acons)
+(define-cproc acons (caa cda cd) ::<pair>
+  (return (SCM_PAIR (Scm_Acons caa cda cd))))
 (define-cproc last-pair (list) :constant Scm_LastPair)
 (define-cproc list-copy (list) Scm_CopyList)
 
@@ -191,7 +221,7 @@
     (let* ([head (SCM_LIST1 arg)] [tail head])
       (dopairs [cp args]
         (unless (SCM_PAIRP (SCM_CDR cp))
-          (SCM_SET_CDR tail (SCM_CAR cp))
+          (SCM_SET_CDR_UNCHECKED tail (SCM_CAR cp))
           (break))
         (SCM_APPEND1 head tail (SCM_CAR cp)))
       (return head))))
@@ -202,7 +232,7 @@
       (when (SCM_NULLP (SCM_CDR cp))
         (if (SCM_NULLP h)
           (set! h (SCM_CAR cp))
-          (SCM_SET_CDR t (SCM_CAR cp)))
+          (Scm_SetCdr t (SCM_CAR cp)))
         (break))
       (SCM_APPEND h t (SCM_CAR cp))
       (unless (or (SCM_NULLP t) (SCM_NULLP (SCM_CDR t)))
@@ -211,7 +241,7 @@
 
 (define-cproc reverse! (list :optional (tail ())) Scm_Reverse2X)
 
-(define-cproc monotonic-merge (sequences::<list>) Scm_MonotonicMerge1)
+(define-cproc monotonic-merge (sequences::<list>) ::<list> Scm_MonotonicMerge1)
 
 (select-module gauche.internal)
 (define-in-module scheme (map proc lis . more)
@@ -753,7 +783,7 @@
       (let1 val (assoc-ref alist key '() eq)
         (assoc-adjoin alist key (rec val (car keys) (cdr keys)) eq))))
   (if (null? keys)
-    (error "nees at least one key in assoc-update")
+    (error "needs at least one key in assoc-update")
     (rec alist (car keys) (cdr keys))))
 
 
@@ -761,11 +791,19 @@
 ;;; Extended pairs
 ;;;
 
-(select-module gauche.internal)
-;; Pair attributes
-;;
-;;  Pair attributes are almost exclusively used to attach source-code
-;;  information to s-exprs.
+(select-module gauche)
+
+(define-cproc extended-pair? (obj) ::<boolean> SCM_EXTENDED_PAIR_P)
+(define-cproc extended-cons (car cdr :optional (attrs ()))
+  (let* ([z (Scm_ExtendedCons car cdr)])
+    (unless (SCM_NULLP attrs)
+      (for-each (lambda (p)
+                  (unless (SCM_PAIRP p)
+                    (Scm_Error "Assoc list required, but got: %S" attrs))
+                  (Scm_PairAttrSet (SCM_PAIR z) (SCM_CAR p) (SCM_CDR p)))
+                (Scm_Reverse attrs)))
+    (return z)))
+(define-cproc extended-list (elt :rest more) ::<list> Scm_ExtendedCons)
 
 (define-cproc pair-attributes (pair::<pair>) Scm_PairAttr)
 
@@ -774,7 +812,3 @@
 
 (define-cproc pair-attribute-set! (pair::<pair> key value)
   (return (Scm_PairAttrSet (SCM_PAIR pair) key value)))
-
-(define-cproc extended-pair? (obj) ::<boolean> SCM_EXTENDED_PAIR_P)
-(define-cproc extended-cons (car cdr) Scm_ExtendedCons)
-(define-cproc extended-list (elt :rest more) Scm_ExtendedCons)
