@@ -1,7 +1,7 @@
 ;;;
 ;;; libmacro.scm - built-in macros
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -33,7 +33,7 @@
 
 (select-module gauche)
 (use util.match)
-(declare (keep-private-macro cond-expand quasirename 
+(declare (keep-private-macro cond-expand quasirename
                              syntax-error syntax-errorf
                              ^ ^_ ^a ^b ^c ^d ^e ^f ^g ^h ^i ^j ^k ^l ^m ^n
                              ^o ^p ^q ^r ^s ^t ^u ^v ^w ^x ^y ^z $ cut cute rec
@@ -42,10 +42,13 @@
                              let1 if-let1 and-let1 let/cc begin0 rlet1
                              let-values let*-values define-values set!-values
                              values-ref values->list
-                             assume assume-type 
+                             assume assume-type
                              dotimes dolist do-plist doplist
-                             ecase cond-list unwind-protect
+                             ecase cond-list define-condition-type condition
+                             unwind-protect
                              let-keywords let-keywords* let-optionals*
+                             lcons lcons* llist*
+                             rxmatch-let rxmatch-if rxmatch-cond rxmatch-case
                              define-compiler-macro))
 
 ;; This file defines built-in macros.
@@ -53,6 +56,9 @@
 
 ;;; cond-expand
 
+(autoload gauche.version valid-version-spec? version-satisfy?)
+(autoload gauche.package find-gauche-package-description)
+
 (define-syntax cond-expand
   (er-macro-transformer
    (^[f r c]
@@ -67,9 +73,9 @@
      ;; by Gauche built-in features).
      (define (fulfill? req seed)
        (cond
-        [(identifier? req) (fulfill? (identifier->symbol req) seed)]
-        [(symbol? req)
-         (and-let1 p (assq req features)
+        [(identifier? req)
+         (and-let1 p (assq (identifier->symbol req) features)
+           ((with-module gauche.internal %check-feature-id) (car p))
            (if (null? (cdr p)) seed (cons (cadr p) seed)))]
         [(not (pair? req)) (error "Invalid cond-expand feature-id:" req)]
         [else
@@ -78,6 +84,7 @@
            [(or)  (fulfill-or  (cdr req) seed)]
            [(not) (fulfill-not (cadr req) seed)]
            [(library) (fulfill-library (cdr req) seed)]
+           [(package) (fulfill-package (cdr req) seed)]
            [else (error "Invalid cond-expand feature expression:" req)])]))
 
      (define (fulfill-and reqs seed)
@@ -101,7 +108,6 @@
        (let* ([libname (car rest)]
               [modname (cond
                         [(identifier? libname) (identifier->symbol libname)]
-                        [(symbol? libname) libname]
                         [(list? libname) (library-name->module-name libname)]
                         [else (error "Invalid library name in 'library' clause \
                                       of cond-expand:" libname)])])
@@ -109,9 +115,23 @@
          (and (library-exists? modname)
               seed)))
 
+     (define (fulfill-package rest seed)
+       (match rest
+         [(package) 
+          (or (equal? package 'gauche)
+              (find-gauche-package-description package))]
+         [(package version-spec)
+          (unless (valid-version-spec? version-spec)
+            (error "Invalid version spec in package clause:" `(package ,@rest)))
+          (if (equal? package 'gauche)
+            (and (version-satisfy? version-spec (gauche-version)) seed)
+            (and-let1 gpd (find-gauche-package-description package)
+              (and (version-satisfy? version-spec (~ gpd'version)) seed)))]
+         [_ (error "Malformed package clause:" `(package ,@rest))]))
+
      (define (rec cls)
        (cond
-        [(null? cls) (error "Unfulfilled cond-expand:" cls)]
+        [(null? cls) (error "Unfulfilled cond-expand (likely missing feature to run this code)")]
         [(not (pair? (car cls)))
          (error "Bad clause in cond-expand:" (car cls))]
         [(c (r (caar cls)) else.)
@@ -126,6 +146,14 @@
 
      (rec (cdr f)))))
 
+;; transitional
+(with-module gauche.internal
+  (define (%check-feature-id sym)
+    (when (and (#/^srfi-\d+$/ (symbol->string sym))
+               (not (vm-compiler-flag-is-set? SCM_COMPILE_SRFI_FEATURE_ID)))
+      (warn "Feature identifier ~a is deprecated.  Use (library ~a)\n"
+            sym sym))))
+
 ;;; quasirename
 
 ;; NB: The walk code has the same structure as quasiquote expander; the
@@ -138,19 +166,19 @@
    (^[f r c]
      (define unquote. (r'unquote))
      (define (unquote? x)
-       (and (or (symbol? x) (identifier? x))
+       (and (identifier? x)
             (c (r x) unquote.)))
      (define unquote-splicing. (r'unquote-splicing))
      (define (unquote-splicing? x)
-       (and (or (symbol? x) (identifier? x))
+       (and (identifier? x)
             (c (r x) unquote-splicing.)))
      (define (unquote*? x)
-       (and (or (symbol? x) (identifier? x))
+       (and (identifier? x)
             (or (c (r x) unquote.)
                 (c (r x) unquote-splicing.))))
      (define quasiquote. (r'quasiquote))
      (define (quasiquote? x)
-       (and (or (symbol? x) (identifier? x))
+       (and (identifier? x)
             (c (r x) quasiquote.)))
      (define cons. (r'cons))
      (define list. (r'list))
@@ -184,7 +212,7 @@
                 `(,cons. ',op ,xxs))))]
          [(? pair?)       (quasi* obj level)]
          [(? vector?)     (quasi-vector obj level)]
-         [(or (? symbol?) (? identifier?)) `(,rename. ',obj)]
+         [(? identifier?) `(,rename. ',obj)]
          [_  `',obj]))
 
      ;; In the spliceable context.  objs is always a list.
@@ -249,10 +277,10 @@
        (global-variable-ref 'gauche.internal '*quasirename-mode* #f))
      (define (legacy-message f)
        (if-let1 srcinfo (debug-source-info f)
-         (format "Legacy quasirename form (~a:~a): ~s" 
+         (format "Legacy quasirename form (~a:~a): ~s"
                  (car srcinfo) (cadr srcinfo) f)
          (format "Legacy quasirename form: ~s" f)))
-  
+
      (match f
        [(_ rr ((? quasiquote? qq) ff))
         (if (eq? qmode 'legacy)
@@ -296,8 +324,7 @@
   (er-macro-transformer
    (^[f r c]
      (let ([args (map unwrap-syntax (cdr f))]
-           [original ((with-module gauche.internal pair-attribute-get)
-                      f 'original #f)])
+           [original (pair-attribute-get f 'original #f)])
        (if original
          (raise (make-compound-condition
                  (apply make-error (car args) (cdr args))
@@ -308,8 +335,7 @@
   (er-macro-transformer
    (^[f r c]
      (let ([args (map unwrap-syntax (cdr f))]
-           [original ((with-module gauche.internal pair-attribute-get)
-                      f 'original #f)])
+           [original (pair-attribute-get f 'original #f)])
        (if original
          (raise (make-compound-condition
                  (make-error (apply format/ss (car args) (cdr args)))
@@ -380,7 +406,7 @@
      (if (null? (cdr f))
        (error "malformed $ form:" f)
        (let1 x (last f)
-         (cond [(c x $.) 
+         (cond [(c x $.)
                 (let1 arg (gensym)
                   `(,(r'lambda) (,arg) ,($-fold (cdr f) '() arg)))]
                [(c x $*.)
@@ -403,7 +429,7 @@
              [(c (car args) <...>.)
               (if (null? (cdr args))
                 (let1 restarg (gensym)
-                  (values (reverse tmps restarg) 
+                  (values (reverse tmps restarg)
                           (cons (r'apply) (reverse elts (list restarg)))))
                 (error "Malformed cut:" f))]
              [(c (car args) <>.)
@@ -522,7 +548,7 @@
    (^[f r c]
      (match f
        [(_ var test exp . more) (quasirename r
-                                  `(let ((,var ,test)) 
+                                  `(let ((,var ,test))
                                      (and ,var (begin ,exp ,@more))))]
        [_ (error "malformed and-let1:" f)]))))
 
@@ -595,7 +621,7 @@
        [(_ formals expr)
         (match formals
           [()  ; allowed in r7rs
-           (quasirename r 
+           (quasirename r
              `(define ,(gensym) (receive ,(gensym) ,expr #f)))]
           [(v) ; trivial case
            (quasirename r
@@ -669,7 +695,7 @@
                 `(receive (,@vars . _) ,mv-expr (values ,@rvars))))
             (quasirename r
               `(receive vals ,mv-expr
-                 (apply values (map (^[i] (list-ref vals i)) 
+                 (apply values (map (^[i] (list-ref vals i))
                                     (list ,@(cons n ns))))))))]))))
 
 (define-syntax values->list
@@ -678,7 +704,7 @@
      (match f
        [(_ mv-expr) (quasirename r
                       `(receive x ,mv-expr x))]))))
-        
+
 ;;; generalized set! family
 
 (define-syntax push!
@@ -770,9 +796,11 @@
      (match f
        [(_ expr . objs)
         (quasirename r
-          `(unless ,expr
-             (error (format "Invalid assumption: ~s" ',expr) ,@objs)))]))))
+          `(or ,expr
+               (error (format "Invalid assumption: ~s" ',expr) ,@objs)))]))))
 
+;; This will eventually folded into the compiler.  The argumet must be
+;; a literal <type>.
 (define-syntax assume-type
   (er-macro-transformer
    (^[f r c]
@@ -780,8 +808,9 @@
        [(_ expr type)
         (quasirename r
           `(let1 v ,expr
-             (unless (is-a? v ,type)
-               (type-error 'expr ,type v))))]))))
+             (if (of-type? v ,type)
+               v
+               (type-error ',expr ,type v))))]))))
 
 ;;; repeat construct
 
@@ -823,7 +852,7 @@
        [(_ (var lis res) . body) (expand var lis res body)]
        [(_ (var lis) . body)     (expand var lis (undefined) body)]
        [(_ (lis) . body)         (expand (gensym) lis (undefined) body)]
-       [_ (error "Malformed dolist:" f)]))))   
+       [_ (error "Malformed dolist:" f)]))))
 
 (define-syntax do-plist
   (er-macro-transformer
@@ -840,7 +869,7 @@
        [(_ ((k v) plis default) . body)
         (expand k v plis body default)]
        [(_ ((k v) plis) . body)
-        (expand k v plis body 
+        (expand k v plis body
                 (quasirename r `(error "plist is not even:" ,plis)))]
        [_ (error "Malformed do-plist:" f)]))))
 
@@ -863,7 +892,7 @@
             (let1 choices (append-map car clause)
               (quasirename r
                 `(let ([v ,expr])
-                   (case v 
+                   (case v
                      ,@clause
                      (else (errorf "ecase test fell through: got ~s, \
                                     expecting one of ~s" v ',choices)))))))]
@@ -913,6 +942,67 @@
                (if tmp (cons (begin ,@expr) r) r)))]
          )))))
 
+;;; srfi-35 condition macros
+
+;; we extend srfi-35 to allow #f as predicate and accessors, as well as
+;; omitting accessors.
+
+(define-syntax define-condition-type
+  (er-macro-transformer
+   (^[f r c]
+     (match (cdr f)
+       [(name super pred . field-specs)
+        (define (badfield-error field)
+          (error "bad field spec for define-condition-type:" field))
+        (define (scan-specs specs slots readers)
+          (match specs
+            [() (emit-defs slots readers)]
+            [((field #f) . rest)
+             (scan-specs rest (cons field slots) readers)]
+            [((field) . rest)
+             (scan-specs rest (cons field slots) readers)]
+            [((field reader) . rest)
+             (scan-specs rest (cons field slots)
+                         (cons (quasirename r
+                                 `(define (,reader obj)
+                                    (condition-ref obj ',field)))
+                               readers))]
+            [_ (badfield-error (car specs))]))
+        (define (emit-defs slots readers)
+          (quasirename r
+            `(begin
+               (define-class ,name (,super)
+                 ,(map (^s (quasirename r
+                             `(,s :init-keyword ',(make-keyword s))))
+                       slots)
+                 :metaclass <condition-meta>)
+               ,@readers
+               ,@(if pred
+                   (quasirename r
+                     `((define (,pred obj)
+                        (condition-has-type? obj ,name))))
+                   '()))))
+        (scan-specs field-specs '() '())]
+       ))))
+
+(define-syntax condition
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ type-field-binding ...)
+        (quasirename r
+          `(make-compound-condition
+            ,@(map (match-lambda
+                     [(type (field expr) ...)
+                      (quasirename r
+                        `(make-condition
+                          ,type
+                          ,@(append-map (^[f e] `(',f ,e)) field expr)))]
+                     [_ (error "malformed condition:" f)])
+                   type-field-binding)))]
+       [_ (error "malformed condition:" f)]))))
+
+
 ;;; unwind-protect
 
 ;; HANDLERS are called when BODY exits, either normally or abnormally.  This
@@ -931,10 +1021,10 @@
 ;;
 ;; We also tweak exit-handler parameter inside the dyanmic scope of BODY
 ;; to ensure HANDLERS are executed even if BODY calls exit.
-;; (An alternative idea is to treat exit as if it's another kind of a 
+;; (An alternative idea is to treat exit as if it's another kind of a
 ;; condition, so that guard clauses are invoked.  We tried it, but the
-;; problem is how to deal with "ignore-errors" idiom, 
-;; e.g. (guard (e [else #f]) body).  The exit condition shouldn't be 
+;; problem is how to deal with "ignore-errors" idiom,
+;; e.g. (guard (e [else #f]) body).  The exit condition shouldn't be
 ;; stopped in such a way.)
 ;;
 ;; TODO: Current definition doesn't work when unwind-protect is used
@@ -955,7 +1045,7 @@
         (let* ([sinfo (debug-source-info f)]
                [lambda. (r 'lambda)]
                [%unwind-protect. ((with-module gauche.internal make-identifier)
-                                  '%unwind-protect 
+                                  '%unwind-protect
                                   (find-module 'gauche.internal)
                                   '())])
           (if sinfo
@@ -1039,7 +1129,7 @@
                                (cond [(and (pair? s) (pair? (cdr s)) (null? (cddr s)))
                                       (cons (car s) (cadr s))]
                                      [(or (symbol? s) (identifier? s))
-                                      (cons s '(undefined))]
+                                      (cons s `(,(r'undefined)))]
                                      [else (error "malformed let-optionals* bindings:"
                                                   specs)]))
                               (^_ '()) ; ignore last cdr of dotted list
@@ -1059,6 +1149,11 @@
 (select-module gauche.internal)
 (use util.match)
 (define (%let-keywords-rec form rename %let)
+  ;; arg is either:
+  ;;   identidfir
+  ;;   (identifier default-expr)
+  ;;   (identifier keyword default-expr)
+  ;; returns (values identifier keyword default-expr)
   (define (triplet var&default)
     (or (and-let* ([ (list? var&default) ]
                    [var (unwrap-syntax (car var&default))]
@@ -1075,6 +1170,12 @@
                    [ (symbol? var) ])
           (values var (make-keyword var) (undefined)))
         (error "bad binding form in let-keywords" var&default)))
+  ;; Loop over var-specs, returns
+  ;;  (values vars        ; list of local variables to bound
+  ;;          keys        ; list of keywords to detect
+  ;;          defaults    ; list of expressions for the default
+  ;;          tmps        ; list of temporary variables
+  ;;          restvar)    ; #f, #t or identifier
   (define (process-specs specs)
     (let loop ((specs specs)
                (vars '()) (keys '()) (defaults '()) (tmps '()))
@@ -1114,7 +1215,7 @@
               [(null? (cdr ,argvar))
                (error "keyword list not even" ,argvar)]
               [else
-               (case (car ,argvar)
+               (case (unwrap-syntax-1 (car ,argvar))
                  ,@(map (^[key]
                           (quasirename rename
                             `((,key)
@@ -1147,11 +1248,151 @@
                (if (eq? %let 'let) 'let-keywords 'let-keywords*)
                form)]))
 
+;;; lseq
+(select-module gauche)
+(define-syntax lcons
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ x y)
+        (quasirename r
+          `(,(with-module gauche.internal %lcons) ,x (lambda () ,y)))]
+       [(_ x y attrs)
+        (quasirename r
+          `(,(with-module gauche.internal %lcons) ,x (lambda () ,y) ,attrs))]))))
+
+(define-syntax lcons*
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ x) x]
+       [(_ x y) (quasirename r `(lcons ,x ,y))]
+       [(_ x y z ...) (quasirename r
+                        `(cons ,x (lcons* ,y ,@z)))]))))
+
+(define-syntax llist* lcons*)
+
+;;; rxmatch-* macros
+
+;; rxmatch-let expr (var ...) form ...
+(define-syntax rxmatch-let
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ expr (var ...) form ...)
+        (quasirename r
+          `(if-let1 m ,expr
+             (let ,(map (^[v i] (quasirename r
+                                  `(,v (rxmatch-substring m ,i))))
+                        var (iota (length var)))
+               ,@form)
+             (error "rxmatch-let: match failed:" ',expr)))]
+       [_ (error "malformed rxmatch-let:" f)]))))
+
+;; rxmatch-if expr (var ...) then else
+(define-syntax rxmatch-if
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ expr (var ...) then else)
+        (quasirename r
+          `(if-let1 m ,expr
+             (let ,(append-map (^[v i]
+                                 (if v
+                                   (quasirename r
+                                     `((,v (rxmatch-substring m ,i))))
+                                   '()))
+                               var (iota (length var)))
+               ,then)
+             ,else))]
+       [_ (error "malformed rxmatch-if:" f)]))))
+
+;; rxmatch-cond (expr bind form ...) ... [(else form ...)]
+(define-syntax rxmatch-cond
+  (er-macro-transformer
+   (^[f r c]
+     (define (test.? x) (c (r'test) x))
+     (define (else.? x) (c (r'else) x))
+     (define (=>.? x)   (c (r'=>) x))
+     (let loop ([clauses (cdr f)])
+       (match clauses
+         [() `(,(r'undefined))]
+         [(((? else.?) form ...) . rest)
+          (if (null? rest)
+            (quasirename r `(begin ,@form))
+            (error "extra stuff after else clause in rxmatch-cond:" f))]
+         [(((? test.?) expr (? =>.?) proc) . rest)
+          (quasirename r
+            `(if-let1 x ,expr
+               (,proc x)
+               ,(loop rest)))]
+         [(((? test.?) expr form ...) . rest)
+          (quasirename r
+            `(if ,expr
+               (begin ,@form)
+               ,(loop rest)))]
+         [((matchexp (var ...) form ...) . rest)
+          (quasirename r
+            `(rxmatch-if ,matchexp ,var
+               (begin ,@form)
+               ,(loop rest)))]
+         [_ (error "bad clause in rxmatch-cond:" (car clause))])))))
+
+;; rxmatch-case exp clause ...
+;;   clause: (re bind form ...)
+;;           (test pred form ...)
+;;           (test pred => proc)
+;;           (else form ...)
+;;           (else => proc)
+(define-syntax rxmatch-case
+  (er-macro-transformer
+   (^[f r c]
+     (define (test.? x) (c (r'test) x))
+     (define (else.? x) (c (r'else) x))
+     (define (=>.? x)   (c (r'=>) x))
+     (define tmp. (r'tmp))
+     (define strp. (r'strp))
+     (define (loop clauses)
+       (match clauses
+         [() `(,(r'undefined))]
+         [(((? else.?) (? =>.?) proc) . rest)
+          (if (null? rest)
+            `(,proc ,tmp.)
+            (error "extra stuff after else clause in rxmatch-case:" f))]
+         [(((? else.?) form ...) . rest)
+          (if (null? rest)
+            (quasirename r `(begin ,@form))
+            (error "extra stuff after else clause in rxmatch-case:" f))]
+         [(((? test.?) pred (? =>.?) proc) . rest)
+          (quasirename r
+            `(if-let1 x (,pred ,tmp.)
+               (,proc x)
+               ,(loop rest)))]
+         [(((? test.?) pred form ...) . rest)
+          (quasirename r
+            `(if (,pred ,tmp.)
+               (begin ,@form)
+               ,(loop rest)))]
+         [((re (var ...) form ...) . rest)
+          (quasirename r
+            `(rxmatch-if (and ,strp. (rxmatch ,re ,tmp.)) ,var
+               (begin ,@form)
+               ,(loop rest)))]
+         [_ (error "bad clause in rxmatch-case:" (car clause))]))
+     (match f
+       [(_ exp . clauses)
+        (quasirename r
+          `(let* ([,tmp. ,exp]
+                  [,strp. (string? ,tmp.)])
+             ,(loop clauses)))]
+       [_ (error "malformed rxmatch-case:" f)]))))
+
+
 ;;;
-;;; OBSOLETED - Tentative compiler macro 
+;;; OBSOLETED - Tentative compiler macro
 ;;;
 
 (select-module gauche)
 ;; TRANSIENT: Remove by 1.0
 (define-macro (define-compiler-macro name xformer-spec)
-  (error "define-compiler-macro is obsoleted.  Use define-inline/syntax."))
+  (error "define-compiler-macro is obsoleted.  Use define-hybrid-syntax."))
