@@ -1,7 +1,7 @@
 ;;;
 ;;; libmisc.scm - miscellaneous built-in procedures
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -58,7 +58,7 @@
 
 ;; Debug label
 (select-module gauche)
-(define-cproc debug-label (obj) (result (Scm_Sprintf "@%lx" obj)))
+(define-cproc debug-label (obj) (return (Scm_Sprintf "@%lx" obj)))
 
 ;; Foreign pointer (may be in libsys.scm?)
 
@@ -92,4 +92,3 @@
 
 (inline-stub
  (define-constant SLIB_DIR (c (SCM_MAKE_STR_IMMUTABLE SLIB_DIR))))
-
