@@ -1,7 +1,7 @@
 ;;;
 ;;; libnum.stub - builtin number libraries
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,7 +35,7 @@
 
 (inline-stub
  (declcode (.include <gauche/vminsn.h>
-                     <gauche/bignum.h>
+                     <gauche/priv/bignumP.h>
                      <stdlib.h>
                      <float.h>
                      <math.h>))
@@ -77,6 +77,7 @@
 (define-cproc fixnum? (x) ::<boolean> :fast-flonum SCM_INTP)
 (define-cproc bignum? (x) ::<boolean> :fast-flonum SCM_BIGNUMP)
 (define-cproc flonum? (x) ::<boolean> :fast-flonum :constant SCM_FLONUMP)
+(define-cproc ratnum? (x) ::<boolean> :fast-flonum :constant SCM_RATNUMP)
 
 (define-cproc finite?   (x::<number>) ::<boolean> :fast-flonum Scm_FiniteP)
 (define-cproc infinite? (x::<number>) ::<boolean> :fast-flonum Scm_InfiniteP)
@@ -99,16 +100,21 @@
 (define-cproc least-fixnum ()    ::<long> (return SCM_SMALL_INT_MIN))
 (define-cproc greatest-fixnum () ::<long> (return SCM_SMALL_INT_MAX))
 
-;; default-endian is defined in Scm__InitNumber().
+(inline-stub
+ (initcode
+  ;; default-endian is defined in number.c.  This call ensures it is
+  ;; initialized.
+  (Scm_DefaultEndian)))
+
 (define-cproc native-endian () Scm_NativeEndian)
 
 ;; DBL_EPSILON, etc.
-(define-cproc flonum-epsilon () 
+(define-cproc flonum-epsilon ()
   (let* ([x::(static ScmObj) SCM_UNBOUND])
     (when (== x SCM_UNBOUND)
       (set! x (Scm_MakeFlonum DBL_EPSILON)))
     (return x)))
-(define-cproc flonum-min-normalized () 
+(define-cproc flonum-min-normalized ()
   (let* ([x::(static ScmObj) SCM_UNBOUND])
     (when (== x SCM_UNBOUND)
       (set! x (Scm_MakeFlonum DBL_MIN)))
@@ -125,7 +131,7 @@
 (select-module gauche.internal)
 (define-cproc %bignum-dump (obj) ::<void>
   (when (SCM_BIGNUMP obj)
-    (Scm_DumpBignum (SCM_BIGNUM obj) SCM_CUROUT)))
+    (Scm_BignumDump (SCM_BIGNUM obj) SCM_CUROUT)))
 
 ;;
 ;; Comparison
@@ -184,6 +190,10 @@
   (Scm_MinMax arg0 args (& SCM_RESULT) NULL))
 
 (select-module gauche)
+(define-cproc <=:< (x y z) ::<boolean> :fast-flonum :constant
+  (return (and (Scm_NumLE x y) (Scm_NumLT y z))))
+(define-cproc <:<= (x y z) ::<boolean> :fast-flonum :constant
+  (return (and (Scm_NumLT x y) (Scm_NumLE y z))))
 (define-cproc min&max (arg0 :rest args) ::(<top> <top>)
   (Scm_MinMax arg0 args (& SCM_RESULT0) (& SCM_RESULT1)))
 
@@ -209,7 +219,7 @@
          [o::ScmPort* (SCM_PORT (Scm_MakeOutputStringPort TRUE))])
     (when (or (< radix SCM_RADIX_MIN) (> radix SCM_RADIX_MAX))
       (Scm_Error "radix must be an integer between %d and %d, but got %d"
-                 SCM_RADIX_MIN SCM_RADIX_MAX radix))  
+                 SCM_RADIX_MIN SCM_RADIX_MAX radix))
     (cond [(or (SCM_FALSEP flags) (SCM_NULLP flags)) (set! f 0)]
           [(SCM_TRUEP flags) (set! f SCM_NUMBER_FORMAT_USE_UPPER)];compatibility
           [(SCM_PAIRP flags)
@@ -232,8 +242,18 @@
     (Scm_PrintNumber o obj (& fmt))
     (return (Scm_GetOutputString o 0))))
 
-(define-cproc string->number (obj::<string> :optional (radix::<fixnum> 10))
-  (return (Scm_StringToNumber obj radix 0)))
+(define-cproc string->number (obj::<string>
+                              :optional (radix::<fixnum> 10)
+                                        (default-exactness #f))
+  (let* ([flags::u_long 0])
+    (cond
+     [(SCM_EQ default-exactness 'exact)
+      (set! flags SCM_NUMBER_FORMAT_EXACT)]
+     [(SCM_EQ default-exactness 'inexact)
+      (set! flags SCM_NUMBER_FORMAT_INEXACT)]
+     [(SCM_FALSEP default-exactness)]
+     [else (Scm_Error "default-exactness must be either #f, exact or inexact, but got: %S" default-exactness)])
+    (return (Scm_StringToNumber obj radix flags))))
 
 (select-module gauche)
 (define-cproc floor->exact (num) :fast-flonum :constant
@@ -262,31 +282,31 @@
            (return v))]
         [else (SCM_TYPE_ERROR num "real number") (return SCM_UNDEFINED)]))
 
-(define-cproc fmod (x::<real> y::<real>) ::<double> fmod)
+(define-cproc fmod (x::<double> y::<double>) ::<double> fmod)
 
-(define-cproc frexp (d::<real>) ::(<double> <int>)
+(define-cproc frexp (d::<double>) ::(<double> <int>)
   (set! SCM_RESULT0 (frexp d (& SCM_RESULT1))))
 
-(define-cproc modf (x::<real>) ::(<double> <double>)
+(define-cproc modf (x::<double>) ::(<double> <double>)
   (set! SCM_RESULT0 (modf x (& SCM_RESULT1))))
 
-(define-cproc ldexp (x::<real> exp::<int>) ::<real> ldexp)
+(define-cproc ldexp (x::<double> exp::<int>) ::<double> ldexp)
 
-(define-cproc log10 (x::<real>) ::<real> log10)
+(define-cproc log10 (x::<double>) ::<double> log10)
 
 ;; NB: Alternative implemenation of gamma and log-abs-gamma functions are
 ;; provided in Scheme (lib/gauche/numerical.scm).
 (select-module gauche.internal)
 (inline-stub
  (if "defined(HAVE_TGAMMA)"
-   (define-cproc %gamma (x::<real>) ::<real> :fast-flonum :constant
+   (define-cproc %gamma (x::<double>) ::<double> :fast-flonum :constant
      tgamma))
  (if "defined(HAVE_LGAMMA)"
-   (define-cproc %lgamma (x::<real>) ::<real> :fast-flonum :constant
+   (define-cproc %lgamma (x::<double>) ::<double> :fast-flonum :constant
      lgamma)))
 ;; Returns the HalfFloat representation as integer.  For now,
 ;; we keep it in gauche.internal.
-(define-cproc flonum->f16bits (x::<real>) ::<int> Scm_DoubleToHalf)
+(define-cproc flonum->f16bits (x::<double>) ::<int> Scm_DoubleToHalf)
 
 ;;
 ;; Arithmetics
@@ -389,7 +409,7 @@
 ;; Transcedental functions.   First, real-only versions.
 
 (select-module gauche)
-(define-cproc %exp (x::<real>) ::<real> :fast-flonum :constant exp)
+(define-cproc %exp (x::<double>) ::<double> :fast-flonum :constant exp)
 
 (define-cproc %log (x) ::<number> :fast-flonum :constant
   (unless (SCM_REALP x) (SCM_TYPE_ERROR x "real number"))
@@ -411,15 +431,15 @@
       (return (Scm_MakeComplex (+ (log (- d)) shift) M_PI))
       (return (Scm_VMReturnFlonum (+ (log d) shift))))))
 
-(define-cproc %sin (x::<real>) ::<real> :fast-flonum :constant sin)
-(define-cproc %cos (x::<real>) ::<real> :fast-flonum :constant cos)
-(define-cproc %tan (x::<real>) ::<real> :fast-flonum :constant tan)
+(define-cproc %sin (x::<double>) ::<double> :fast-flonum :constant sin)
+(define-cproc %cos (x::<double>) ::<double> :fast-flonum :constant cos)
+(define-cproc %tan (x::<double>) ::<double> :fast-flonum :constant tan)
 
-(define-cproc %sinpi (x::<real>) ::<real> :fast-flonum :constant Scm_SinPi)
-(define-cproc %cospi (x::<real>) ::<real> :fast-flonum :constant Scm_CosPi)
-(define-cproc %tanpi (x::<real>) ::<real> :fast-flonum :constant Scm_TanPi)
+(define-cproc %sinpi (x::<double>) ::<double> :fast-flonum :constant Scm_SinPi)
+(define-cproc %cospi (x::<double>) ::<double> :fast-flonum :constant Scm_CosPi)
+(define-cproc %tanpi (x::<double>) ::<double> :fast-flonum :constant Scm_TanPi)
 
-(define-cproc %asin (x::<real>) ::<number> :fast-flonum :constant
+(define-cproc %asin (x::<double>) ::<number> :fast-flonum :constant
   (cond [(> x 1.0)
          (return (Scm_MakeComplex (/ M_PI 2.0)
                                   (- (log (+ x (sqrt (- (* x x) 1.0)))))))]
@@ -428,24 +448,24 @@
                                   (- (log (- (- x) (sqrt (- (* x x) 1.0)))))))]
         [else (return (Scm_VMReturnFlonum (asin x)))]))
 
-(define-cproc %acos (x::<real>) ::<number> :fast-flonum :constant
+(define-cproc %acos (x::<double>) ::<number> :fast-flonum :constant
   (cond [(> x 1.0)
          (return (Scm_MakeComplex 0 (log (+ x (sqrt (- (* x x) 1.0))))))]
         [(< x -1.0)
          (return (Scm_MakeComplex 0 (log (+ x (sqrt (- (* x x) 1.0))))))]
         [else (return (Scm_VMReturnFlonum (acos x)))]))
 
-(define-cproc %atan (z::<real> :optional x) ::<double> :fast-flonum :constant
+(define-cproc %atan (z::<double> :optional x) ::<double> :fast-flonum :constant
   (cond [(SCM_UNBOUNDP x) (return (atan z))]
         [else (unless (SCM_REALP x) (SCM_TYPE_ERROR x "real number"))
               (return (atan2 z (Scm_GetDouble x)))]))
 
-(define-cproc %sinh (x::<real>) ::<real> :fast-flonum :constant sinh)
-(define-cproc %cosh (x::<real>) ::<real> :fast-flonum :constant cosh)
-(define-cproc %tanh (x::<real>) ::<real> :fast-flonum :constant tanh)
+(define-cproc %sinh (x::<double>) ::<double> :fast-flonum :constant sinh)
+(define-cproc %cosh (x::<double>) ::<double> :fast-flonum :constant cosh)
+(define-cproc %tanh (x::<double>) ::<double> :fast-flonum :constant tanh)
 ;; NB: asinh and acosh are not in POSIX.
 
-(define-cproc %sqrt (x::<real>) :fast-flonum :constant
+(define-cproc %sqrt (x::<double>) :fast-flonum :constant
   (if (< x 0)
     (return (Scm_MakeComplex 0.0 (sqrt (- x))))
     (return (Scm_VMReturnFlonum (sqrt x)))))
@@ -557,7 +577,7 @@
 (define-cproc %exact-integer-expt (x y) :constant Scm_ExactIntegerExpt)
 (define (%exact-expt x y) ;; x, y :: exact
   (cond [(integer? y) (%exact-integer-expt x y)]
-        [(< x 0) (%expt x y)] ; we don't have exact compnum  
+        [(< x 0) (%expt x y)] ; we don't have exact compnum
         [(< y 0) (/ (%exact-expt x (- y)))]
         [(integer? x)
          (let ([a (numerator y)]
@@ -720,6 +740,11 @@
     0
     (/ (- (log (+ 1 z)) (log (- 1 z))) 2)))
 
+(select-module gauche)
+(define-cproc radians->degrees (r::<double>) ::<double> :constant :fast-flonum
+  (return (* r (/ 180 M_PI))))
+(define-cproc degrees->radians (d::<double>) ::<double> :constant :fast-flonum
+  (return (* d (/ M_PI 180))))
 
 (select-module gauche)
 
@@ -858,8 +883,10 @@
 ;;
 
 (select-module scheme)
-(define-cproc make-rectangular (a::<real> b::<real>) :constant Scm_MakeComplex)
-(define-cproc make-polar (r::<real> t::<real>) :constant Scm_MakeComplexPolar)
+(define-cproc make-rectangular (a::<double> b::<double>) :constant
+  Scm_MakeComplex)
+(define-cproc make-polar (r::<double> t::<double>) :constant
+  Scm_MakeComplexPolar)
 
 ;; we don't use Scm_RealPart and Scm_ImagPart, for preserving exactness
 ;; and avoiding extra allocation.
@@ -889,6 +916,3 @@
           (Scm_Error "clamp argument must be either 'both, 'high, 'low or #f, \
                     but got %S" clamp)])
    (return SCM_CLAMP_ERROR)))
-
-
-
