@@ -1,7 +1,7 @@
 ;;;
 ;;; object.scm - object system
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,12 +35,11 @@
 
 ;; preparing inline stub code
 (inline-stub
- (declcode (.include <gauche/class.h>
-                     <gauche/priv/classP.h>
+ (declcode (.include <gauche/priv/classP.h>
                      <gauche/vminsn.h>))
- (define-type <slot-accessor> "ScmSlotAccessor*")
- (define-type <generic> "ScmGeneric*")
- (define-type <method> "ScmMethod*")
+ (declare-stub-type <slot-accessor> "ScmSlotAccessor*")
+ (declare-stub-type <generic> "ScmGeneric*")
+ (declare-stub-type <method> "ScmMethod*")
  )
 
 ;; This module is not meant to be `use'd.   It is just to hide
@@ -103,11 +102,9 @@
 
 ;; allow (setter name) type declaration
 (define (%check-setter-name name)
-  (cond [(symbol? name) (values name #f)]
-        [(identifier? name) (values name #f)]
+  (cond [(identifier? name) (values name #f)]
         [(and (pair? name) (eq? (car name) 'setter)
-              (pair? (cdr name)) (or (symbol? (cadr name))
-                                     (identifier? (cadr name)))
+              (pair? (cdr name)) (identifier? (cadr name))
               (null? (cddr name)))
          (values (%make-setter-name (cadr name)) (cadr name))]
         [else (error "Bad name for generic function or method" name)]))
@@ -115,6 +112,12 @@
 (define (%make-setter-name name)
   (string->symbol (format #f "setter of ~a" name)))
 
+;; These are hidden procedure in gauche.object, for now.
+(define-cproc generic-seal! (gf::<generic>) ::<void>
+  (set! (SCM_PROCEDURE_CONSTANT gf) TRUE))
+(define-cproc generic-unseal! (gf::<generic>) ::<void>
+  (set! (SCM_PROCEDURE_CONSTANT gf) FALSE))
+
 ;;----------------------------------------------------------------
 ;; Method
 ;;
@@ -152,7 +155,7 @@
                         (quasirename %id
                           `(lambda ,real-args ,@body)))])
       (receive (true-name getter-name) (%check-setter-name name)
-        (let1 gf (gensym)
+        (let1 gf (gensym "gf")
           (quasirename %id
             `(rlet1 ,gf (%ensure-generic-function ',true-name (current-module))
                (add-method! ,gf (make <method>
@@ -186,7 +189,7 @@
    (let* ([val (Scm_GlobalVariableRef module name 0)])
      (cond
       [(Scm_TypeP val SCM_CLASS_GENERIC)
-       (when (SCM_UNBOUNDP (Scm_GlobalVariableRef module name 
+       (when (SCM_UNBOUNDP (Scm_GlobalVariableRef module name
                                                   SCM_BINDING_STAY_IN_MODULE))
          (Scm_Define module name val))]
       [else
@@ -212,6 +215,20 @@
 ;; Class
 ;;
 
+;; TEMPORARY: We need to detect keywords such as :metaclass, :slots
+;; or :supers at macro-expansion time.  If the define-class form
+;; is macro-generated, however, these keywords may be renamed, so
+;; we can't simply use get-keyword.  This will eventually addressed
+;; when we rewrote define-class expander as macro.  Meanwhile,
+;; this ad-hoc solution would work.
+
+(define (%get-keyword key args default)
+  (let loop ((as args))
+    (cond [(null? as) default]
+          [(null? (cdr as)) (error "Key-value list is not even:" args)]
+          [(eqv? key (unwrap-syntax-1 (car as))) (cadr as)]
+          [else (loop (cddr as))])))
+
 ;; TRANSIENT: We should employ er-macro-transformer to expand
 ;; define-class etc., but this file need to be compiled by 0.9.5
 ;; and we've improved the transformer since then.  So we postponed
@@ -222,14 +239,19 @@
 ;  (%expand-define-class name supers slots options))
 
 (define (%expand-define-class name supers slots options)
-  (let* ([metaclass (or (get-keyword :metaclass options #f)
+  (let* ([metaclass (or (%get-keyword :metaclass options #f)
                         (quasirename %id
                           `(%get-default-metaclass (list ,@supers))))]
          [slot-defs (map %process-slot-definition slots)]
          [class     (gensym)]
          [slot      (gensym)])
     (quasirename %id
-      `(define ,name
+      ;; We expand this into define-inline, to make the binding inlinable.
+      ;; It allows the compiler to use the information of the class
+      ;; reference.  Note that class redefinition does override the
+      ;; inlinable binding (which should be rare, though).  We handle
+      ;; redefinition of class specially in gloc.c.
+      `(define-inline ,name
          (rlet1 ,class (make ,metaclass
                          ':name ',name ':supers (list ,@supers)
                          ':slots (list ,@slot-defs)
@@ -300,8 +322,8 @@
 
 (define-method initialize :locked ((class <class>) initargs)
   (next-method)
-  (let* ([slots  (get-keyword :slots  initargs '())]
-         [sup    (get-keyword :supers initargs '())]
+  (let* ([slots  (%get-keyword :slots  initargs '())]
+         [sup    (%get-keyword :supers initargs '())]
          ;;  NB: we always add <object> to the direct supers, for C defined
          ;;  base classes may not be inheriting from it.
          [supers (append sup (list <object>))])
@@ -517,8 +539,11 @@
 ;; which has been redefined.  Should only be used in class redefinition
 ;; routines.
 (define-cproc current-class-of (obj) (return (SCM_OBJ (Scm_ClassOf obj))))
+
 (define-cproc is-a? (obj klass::<class>) (inliner IS-A) Scm_VMIsA)
-(define-cproc subtype? (c1::<class> c2::<class>) ::<boolean> Scm_SubtypeP)
+(define-cproc subclass? (c1::<class> c2::<class>) ::<boolean> Scm_SubclassP)
+
+;; NB: subtype? and of-type? is in libtype.scm.
 
 (define-cproc slot-ref (obj slot)
   (inliner SLOT-REF) (setter slot-set!)
@@ -543,7 +568,9 @@
 
 ;; Internal API - undocumented
 (define-cproc instance-slot-ref (obj num::<fixnum> :optional fallback)
-  (return (Scm_InstanceSlotRef3 obj num fallback)))
+  (.if "GAUCHE_API_VERSION < 98"
+       (return (Scm_InstanceSlotRef3 obj num fallback))
+       (return (Scm_InstanceSlotRef obj num fallback))))
 ;; Internal API - undocumented
 (define-cproc instance-slot-set! (obj num::<fixnum> value) ::<void>
   Scm_InstanceSlotSet)
@@ -571,7 +598,9 @@
 (define-cproc %record-ref (klass::<class> obj k::<fixnum>)
   (unless (SCM_ISA obj klass)
     (Scm_Error "record-ref: instance of %S expected, got %S" klass obj))
-  (return (Scm_InstanceSlotRef3 obj k SCM_UNBOUND)))
+  (.if "GAUCHE_API_VERSION < 98"
+       (return (Scm_InstanceSlotRef3 obj k SCM_UNBOUND))
+       (return (Scm_InstanceSlotRef obj k SCM_UNBOUND))))
 
 (define-cproc %record-set! (klass::<class> obj k::<fixnum> val) ::<void>
   (unless (SCM_ISA obj klass)
@@ -752,18 +781,40 @@
 (define-method (setter ref) :locked ((obj <tree-map>) key value)
   (tree-map-put! obj key value))
 
+(define-method ref  ((obj <box>) field) ; srfi-123
+  (unless (eq? field '*)
+    (error "Box content can be accessed with a field name '*, but got:"
+           field))
+  (unbox obj))
+(define-method (setter ref) :locked ((obj <box>) field value)
+  (unless (eq? field '*)
+    (error "Box content can be accessed with a field name '*, but got:"
+           field))
+  (set-box! obj value))
+
 ;; gauche.sequence has the generic version for <sequence>, but these
 ;; shortcuts would be faster.
+;; NB: (ref <bitvector> k) has a choice of using /int or /bool.  We chose
+;; int, somewhat arbitrary but more intuitive with correspondence of the
+;; literal bitvector notation.
 (define-method ref :locked ((obj <list>) (index <integer>))
   (list-ref obj index))
 (define-method ref :locked ((obj <vector>) (index <integer>))
   (vector-ref obj index))
+(define-method ref :locked ((obj <uvector>) (index <integer>))
+  (uvector-ref obj index))
+(define-method ref :locked ((obj <bitvector>) (index <integer>))
+  (bitvector-ref/int obj index))
 (define-method ref :locked ((obj <string>) (index <integer>))
   (string-ref obj index))
 (define-method (setter ref) :locked ((obj <list>) (index <integer>) val)
   (set-car! (list-tail obj index) val))
 (define-method (setter ref) :locked ((obj <vector>) (index <integer>) val)
   (vector-set! obj index val))
+(define-method (setter ref) :locked ((obj <uvector>) (index <integer>) val)
+  (uvector-set! obj index val))
+(define-method (setter ref) :locked ((obj <bitvector>) (index <integer>) val)
+  (bitvector-set! obj index val))
 (define-method (setter ref) :locked ((obj <string>) (index <integer>) val)
   (string-set! obj index val))
 
@@ -828,7 +879,7 @@
           (cond [(char=? #\/ c) (display #\\ out) (display c out)]
                 [(memq (char-general-category c) '(Mn Mc Me Cc Cf Cs Co Cn))
                  (let1 code (char->ucs c)
-                   (if (< code #x10000)  
+                   (if (< code #x10000)
                      (format out "\\u~4,'0x" code)
                      (format out "\\U~8,'0x" code)))]
                 [else (display c out)])
@@ -878,40 +929,41 @@
 ;;; Make exported symbol visible from outside
 ;;;
 
-(define-macro (insert-symbols . syms)
-  `(begin ,@(map (^[s] `(define-in-module gauche ,s ,s)) syms)))
-
-(insert-symbols ;define-generic define-method define-class
-                compute-slots compute-get-n-set compute-slot-accessor
-                class-slot-ref class-slot-set! class-slot-bound?
-                slot-push! slot-pop! slot-unbound slot-missing
-                slot-exists? slot-exists-using-class?
-                change-class
-                apply-generic sort-applicable-methods
-                apply-methods apply-method
-                class-of current-class-of is-a? subtype? slot-ref slot-set!
-                slot-bound? slot-ref-using-accessor slot-bound-using-accessor?
-                slot-set-using-accessor! slot-initialize-using-accessor!
-                instance-slot-ref instance-slot-set! touch-instance!
-                class-name class-precedence-list class-direct-supers
-                class-direct-methods class-direct-subclasses
-                class-direct-slots class-slots
-                class-post-initialize
-                slot-definition-name slot-definition-options
-                slot-definition-option
-                slot-definition-allocation slot-definition-getter
-                slot-definition-setter slot-definition-accessor
-                class-slot-definition class-slot-accessor
-                x->string x->integer x->number ref |setter of ref|
-                ~ ref*
-                describe describe-common describe-slots
-
-                ;; These shouldn't be necessary to be injected into gauche
-                ;; module; unfortunately, the current define-method and
-                ;; define-class are unhygienic, and we need them visible
-                ;; from the expanded code.  Should be removed once we rewrite
-                ;; related macros hygienic.
-                %ensure-generic-function
-                %check-class-binding
-                )
+(let ((xfer (with-module gauche.internal %transfer-bindings)))
+  (xfer (current-module)
+        (find-module 'gauche)
+        '(;define-generic define-method define-class
+          compute-slots
+          compute-get-n-set compute-slot-accessor
+          class-slot-ref class-slot-set! class-slot-bound?
+          slot-push! slot-pop! slot-unbound slot-missing
+          slot-exists? slot-exists-using-class?
+          change-class
+          apply-generic sort-applicable-methods
+          apply-methods apply-method
+          class-of current-class-of is-a? subclass?
+          slot-ref slot-set!
+          slot-bound? slot-ref-using-accessor slot-bound-using-accessor?
+          slot-set-using-accessor! slot-initialize-using-accessor!
+          instance-slot-ref instance-slot-set! touch-instance!
+          class-name class-precedence-list class-direct-supers
+          class-direct-methods class-direct-subclasses
+          class-direct-slots class-slots
+          class-post-initialize
+          slot-definition-name slot-definition-options
+          slot-definition-option
+          slot-definition-allocation slot-definition-getter
+          slot-definition-setter slot-definition-accessor
+          class-slot-definition class-slot-accessor
+          x->string x->integer x->number ref |setter of ref|
+          ~ ref*
+          describe describe-common describe-slots
 
+          ;; These shouldn't be necessary to be injected into gauche
+          ;; module; unfortunately, the current define-method and
+          ;; define-class are unhygienic, and we need them visible
+          ;; from the expanded code.  Should be removed once we rewrite
+          ;; related macros hygienic.
+          %ensure-generic-function
+          %check-class-binding
+          )))
