@@ -1,7 +1,7 @@
 ;;;
 ;;; libsys.scm - builtin system inteface
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -37,7 +37,8 @@
 (select-module gauche)
 (inline-stub
  (declcode
-  (.include <stdlib.h> <locale.h> <math.h> <sys/types.h> <sys/stat.h>)
+  (.include <gauche/priv/mmapP.h>)
+  (.include <stdlib.h> <locale.h> <math.h> <sys/types.h> <sys/stat.h> <fcntl.h>)
   (.cond ["TIME_WITH_SYS_TIME" (.include <sys/time.h> <time.h>)]
          ["HAVE_SYS_TIME_H"    (.include <sys/time.h>)]
          [else                 (.include <time.h>)])
@@ -48,6 +49,7 @@
   (.if "HAVE_SYS_RESOURCE_H" (.include <sys/resource.h>))
   (.if "HAVE_SYS_LOADAVG_H"  (.include <sys/loadavg.h>))
   (.if "HAVE_UNISTD_H"       (.include <unistd.h>))
+  (.if "HAVE_SYS_MMAN_H"     (.include <sys/mman.h>))
 
   (.if "defined(GAUCHE_WINDOWS)"
        (.undef _SC_CLK_TCK)) ;; avoid undefined reference to sysconf
@@ -59,36 +61,88 @@
 
 (select-module gauche)
 (define-cproc sys-readdir (pathname::<string>) Scm_ReadDirectory)
-
-;; Bonus
-
-(define-cproc sys-normalize-pathname (pathname::<string>
-                                      :key (absolute #f)
-                                      (expand #f)
-                                      (canonicalize #f))
-  (let* ([flags::int 0])
-    (unless (SCM_FALSEP absolute) (logior= flags SCM_PATH_ABSOLUTE))
-    (unless (SCM_FALSEP expand)   (logior= flags SCM_PATH_EXPAND))
-    (unless (SCM_FALSEP canonicalize) (logior= flags SCM_PATH_CANONICALIZE))
-    (return (Scm_NormalizePathname pathname flags))))
-
 (define-cproc sys-tmpdir () Scm_TmpDir)
 (define-cproc sys-basename (pathname::<string>) Scm_BaseName)
 (define-cproc sys-dirname (pathname::<string>) Scm_DirName)
 
+(select-module gauche.internal)
+
+;; This isn't POSIX, but we need it in bootstrap so we have it here.
+(define-in-module gauche (sys-normalize-pathname pathname
+                                                 :key
+                                                 (absolute #f)
+                                                 (expand #f)
+                                                 (canonicalize #f))
+  ;; NB: We can't use cond-expand, for this file is cross-precompiled.
+  (define windows? (assq 'gauche.os.windows (cond-features)))
+  (define separator (if windows? "\\" "/"))
+  (define (expand-tilde path)
+    (if-let1 m (and expand (rxmatch #/^~([^\\\/]*)/ path))
+      (let* ([user (m 1)]
+             [home (if (equal? user "")
+                     (if windows?
+                       (get-windows-home)
+                       (get-unix-home (sys-getpwuid (sys-getuid))))
+                     (if windows?
+                       (error "'~user' expansil isn't supported on Windows:"
+                              path)
+                       (get-unix-home (sys-getpwnam (m 1)))))])
+        (string-append home (m 'after)))
+      path))
+  (define (get-windows-home)
+    (or (sys-getenv "HOME") ; MSYS
+        (sys-getenv "USERPROFILE") ; cmd.exe
+        ""))
+  (define (get-unix-home pw)
+    (if pw (~ pw'dir) (error "Couldn't obtain username for :" pathname)))
+  (define (absolute? path)
+    (if windows?
+      (rxmatch #/^([A-Za-z]:)?[\\\/]/ path)
+      (rxmatch #/^\// path)))
+  (define (abs-path path)
+    (if (and absolute (not (absolute? path)))
+      (string-append (sys-getcwd) separator path)
+      path))
+  (define (root? comps)
+    (or (equal? comps '(""))
+        (and windows?
+             (length=? comps 1)
+             (rxmatch #/^[A-Za-z]:$/ (car comps)))))
+  (define (canon-path path)
+    (if canonicalize
+      (let loop ([comps (string-split path #[\\/])]
+                 [r '()]
+                 [dir? #f])             ;whether we add '/' at end
+        (cond [(null? comps)
+               (let1 r (if dir? (cons "" r) r)
+                 (string-join (reverse r) separator))]
+              [(equal? (car comps) ".") (loop (cdr comps) r #t)]
+              [(equal? (car comps) "..")
+               ;; If we've reached to the root dir, we go like "/../"
+               (if (or (null? r) (equal? (car r) "..") (root? r))
+                 (loop (cdr comps) (cons (car comps) r) #f)
+                 (loop (cdr comps) (cdr r) #t))]
+              [else (loop (cdr comps) (cons (car comps) r) #f)]))
+      (if windows?
+        (string-join (string-split path #\/) "\\")
+        path)))
+  ($ canon-path $ abs-path $ expand-tilde pathname))
+
 ;;---------------------------------------------------------------------
 ;; errno.h - error numbers
 
 ;; We won't (and can't) cover every possible errnos, including system
 ;; specific ones.  The following list is taken from Linux asm/errno.h.
 
+(select-module gauche)
+
 (inline-stub
  "ScmHashTable *errno_n2y;"             ;integer -> symbol
  "ScmHashTable *errno_y2n;"             ;symbol -> integer
  (initcode
   (set! errno_n2y (SCM_HASH_TABLE (Scm_MakeHashTableSimple SCM_HASH_EQV 0)))
   (set! errno_y2n (SCM_HASH_TABLE (Scm_MakeHashTableSimple SCM_HASH_EQ 0)))))
-  
+
 
 (define-macro (define-errno symbol)
   `(inline-stub
@@ -238,10 +292,6 @@
 ;;---------------------------------------------------------------------
 ;; grp.h - groups
 
-(inline-stub
- (define-type <sys-group> "ScmSysGroup*")
- )
-
 (define-cproc sys-getgrgid (gid::<int>) Scm_GetGroupById)
 (define-cproc sys-getgrnam (name::<string>) Scm_GetGroupByName)
 
@@ -302,10 +352,6 @@
 ;;---------------------------------------------------------------------
 ;; pwd.h - passwords
 
-(inline-stub
- (define-type <sys-passwd> "ScmSysPasswd*")
- )
-
 (define-cproc sys-getpwuid (uid::<int>) Scm_GetPasswdById)
 (define-cproc sys-getpwnam (name::<string>) Scm_GetPasswdByName)
 
@@ -323,8 +369,6 @@
 ;; signal.h
 
 (inline-stub
- (define-type <sys-sigset> "ScmSysSigset*")
-
  (define-enum SIG_SETMASK)
  (define-enum SIG_BLOCK)
  (define-enum SIG_UNBLOCK)
@@ -406,11 +450,11 @@
 (define-cproc sys-mkdtemp (template::<string>) Scm_SysMkdtemp)
 
 ;; ctermid
-(define-cproc sys-ctermid () ::<const-cstring>
+(define-cproc sys-ctermid ()
   (.if "defined(GAUCHE_WINDOWS)"
-       (return "CON")
+       (return '"CON")
        (let* ([buf::(.array char [(+ L_ctermid 1)])])
-         (return (ctermid buf)))))
+         (return (SCM_MAKE_STR_COPYING (ctermid buf))))))
 
 ;;---------------------------------------------------------------------
 ;; stdlib.h
@@ -499,23 +543,71 @@
 ;;---------------------------------------------------------------------
 ;; sys/loadavg.h
 
-(define-cproc sys-getloadavg (:optional (nsamples::<int> 3))
-  (cast void nsamples) ; suppress unused var warning
-  (.if "defined HAVE_GETLOADAVG"
-       (let* ([samples::(.array double [3])])
-         (when (or (<= nsamples 0) (> nsamples 3))
-           (Scm_Error "sys-getloadavg: argument out of range: %d" nsamples))
-         (let* ([count::int (getloadavg samples nsamples)])
-           (if (< count 0)
-             (return '#f)
-             (let* ([h '()] [t '()])
-               (dotimes [i count]
-                 (let* ([n (Scm_MakeFlonum (aref samples i))])
-                   (SCM_APPEND1 h t n)))
-               (return h)))))
-       (begin
-         (Scm_Error "sys-getloadavg isn't supported on this platform")
-         (return SCM_UNDEFINED))))      ;dummy
+(inline-stub
+(.if (defined HAVE_GETLOADAVG)
+  (define-cproc sys-getloadavg (:optional (nsamples::<int> 3))
+    (let* ([samples::(.array double [3])])
+      (when (or (<= nsamples 0) (> nsamples 3))
+        (Scm_Error "sys-getloadavg: argument out of range: %d" nsamples))
+      (let* ([count::int (getloadavg samples nsamples)])
+        (if (< count 0)
+          (return '#f)
+          (let* ([h '()] [t '()])
+            (dotimes [i count]
+              (let* ([n (Scm_MakeFlonum (aref samples i))])
+                (SCM_APPEND1 h t n)))
+            (return h))))))
+  (define-cproc sys-getloadavg (:optional _::<int>)
+    (begin
+      (Scm_Error "sys-getloadavg isn't supported on this platform")
+      (return SCM_UNDEFINED))))
+)
+;;---------------------------------------------------------------------
+;; sys/mman.h
+
+(inline-stub
+ (define-cclass <memory-region>
+   "ScmMemoryRegion*" "Scm_MemoryRegionClass"
+   ()
+   ((address :c-name "ptr"
+             :c-spec "Scm_MakeIntegerU((uintptr_t)obj->ptr)" :setter #f)
+    ;; TRANSIENT: After 0.9.11, we can say :c-type <size_t>.
+    (size :c-spec "Scm_MakeIntegerU(obj->size)" :setter #f)
+    (protection :c-name "prot" :type <int> :setter #f)
+    (flags :type <int> :setter #f))
+   (printer (let* ((m::ScmMemoryRegion* (SCM_MEMORY_REGION obj)))
+              (Scm_Printf port "#<memory-region %p[%lx] (%s%s%s)>"
+                          (-> m ptr) (-> m size)
+                          (?: (logand (-> m prot) PROT_READ) "r" "")
+                          (?: (logand (-> m prot) PROT_WRITE) "w" "")
+                          (?: (logand (-> m prot) PROT_EXEC) "x" ""))))))
+
+(define-cproc sys-mmap (maybe-port prot::<int> flags::<int> size::<integer>
+                                   :optional (off::<integer> 0))
+  (let* ([fd::int -1] [isize::size_t 0] [ioff::off_t 0])
+    (cond [(SCM_PORTP maybe-port)
+           (set! fd (Scm_PortFileNo (SCM_PORT maybe-port)))
+           (when (< fd 0)
+             (Scm_Error "non-file-backed port can't be used to mmap: %S"
+                        maybe-port))]
+          [(SCM_FALSEP maybe-port)]
+          [else (SCM_TYPE_ERROR maybe-port "port or #f")])
+    (cond [(SCM_UNBOUNDP size)]
+          [(SCM_INTEGERP size) (set! isize (Scm_IntegerToSize size))]
+          [else (SCM_TYPE_ERROR size "integer suitable for size_t")])
+    (cond [(SCM_UNBOUNDP off)]
+          [(SCM_INTEGERP off) (set! ioff (Scm_IntegerToOffset off))]
+          [else (SCM_TYPE_ERROR off "integer suitable for off_t")])
+    (return (Scm_SysMmap NULL fd isize ioff prot flags))))
+
+(inline-stub
+ (define-enum PROT_EXEC)
+ (define-enum PROT_READ)
+ (define-enum PROT_WRITE)
+ (define-enum PROT_NONE)
+ (define-enum MAP_SHARED)
+ (define-enum MAP_PRIVATE)
+ (define-enum MAP_ANONYMOUS))
 
 ;;---------------------------------------------------------------------
 ;; sys/resource.h
@@ -579,8 +671,6 @@
 ;; sys/stat.h
 
 (inline-stub
- (define-type <sys-stat> "ScmSysStat*")
-
  ;; Commn code for stat and lstat.
  (define-cise-stmt stat-common
    [(_ statfn)
@@ -673,22 +763,28 @@
  )
 
 ;; utime.h
+(define-cfn utime-ts (ts::ScmTimeSpec* arg) ::void :static
+  (cond [(SCM_FALSEP arg) (set! (-> ts tv_nsec) UTIME_NOW)]
+        [(SCM_TRUEP arg)  (set! (-> ts tv_nsec) UTIME_OMIT)]
+        [(SCM_REALP arg)
+         (let* ([s::double])
+           (set! (-> ts tv_nsec)
+                 (cast u_long (* (modf (Scm_GetDouble arg) (& s))
+                                 1.0e9)))
+           (set! (-> ts tv_sec) (cast u_long s))
+           (while (>= (-> ts tv_nsec) #e1e9)
+             (set! (-> ts tv_nsec) (- (-> ts tv_nsec) #e1e9))
+             (pre++ (-> ts tv_sec))))]
+        [(SCM_TIMEP arg) (Scm_GetTimeSpec arg ts)]
+        [else (Scm_Error "<time> object, real number or boolean required, but got: %S" arg)]))
+
 (define-cproc sys-utime
   (path::<const-cstring> :optional (atime #f) (mtime #f)) ::<void>
-  (let* ([tim::(struct utimbuf)] [r::int])
-    (cond [(and (SCM_FALSEP atime) (SCM_FALSEP mtime))
-           (SCM_SYSCALL r (utime path NULL))]
-          [else
-           (set! (ref tim actime)
-                 (?: (SCM_FALSEP atime) 
-                     (time NULL)
-                     (cast time_t (Scm_GetUInteger atime))))
-           (set! (ref tim modtime)
-                 (?: (SCM_FALSEP mtime) 
-                     (time NULL)
-                     (cast time_t (Scm_GetUInteger mtime))))
-           (SCM_SYSCALL r (utime path (& tim)))])
-    (when (< r 0) (Scm_SysError "utime failed on %s" path))))
+  (let* ([tss::(.array ScmTimeSpec [2])] [r::int])
+    (utime-ts (& (aref tss 0)) atime)
+    (utime-ts (& (aref tss 1)) mtime)
+    (SCM_SYSCALL r (utimensat AT_FDCWD path tss 0))
+    (when (< r 0) (Scm_SysError "utimensat failed on %s" path))))
 
 ;;---------------------------------------------------------------------
 ;; sys/times.h
@@ -749,11 +845,6 @@
 ;;---------------------------------------------------------------------
 ;; time.h
 
-(inline-stub
- (define-type <time> "ScmTime*")
- (define-type <sys-tm> "ScmSysTm*")
- )
-
 (define-cproc sys-time () (return (Scm_MakeSysTime (time NULL))))
 
 (define-cproc sys-gettimeofday () ::(<ulong> <ulong>)
@@ -791,34 +882,75 @@
 (define-cproc time->seconds (t::<time>) ;SRFI-18
   Scm_TimeToSeconds)
 
-(define-cproc seconds->time (t::<real>) ;SRFI-18
+(define-cproc seconds->time (t::<double>) ;SRFI-18
   Scm_RealSecondsToTime)
 
-(define-cproc sys-asctime (tm::<sys-tm>)
-  (return (SCM_MAKE_STR_COPYING (asctime (& (SCM_SYS_TM_TM tm))))))
+(inline-stub
+ (declare-cfn tm_print (obj port::ScmPort* ctx::ScmWriteContext*)
+              ::void :static)
 
-(define-cproc sys-ctime (time) ::<const-cstring>
-  (let* ([tim::time_t (Scm_GetSysTime time)]) (return (ctime (& tim)))))
+ (define-cstruct <sys-tm> "struct tm"
+   (sec::<int> "tm_sec"
+    min::<int> "tm_min"
+    hour::<int> "tm_hour"
+    mday::<int> "tm_mday"
+    mon::<int>  "tm_mon"
+    year::<int> "tm_year"
+    wday::<int> "tm_wday"
+    yday::<int> "tm_yday"
+    isdst::<int> "tm_isdst")
+   (printer (c "tm_print")))
+
+ (define-cfn tm_print (obj port::ScmPort* _::ScmWriteContext*) ::void
+   (let* ([buf::(.array char (30))]
+          [st::(struct tm*) (SCM_SYS_TM obj)])
+     (.if (not (defined "GAUCHE_WINDOWS"))
+          (strftime buf 30 "%a %b %e %T %Y" st)
+          (strftime buf 30 "%a %b %d %H:%M:%S %Y" st))
+     (Scm_Printf port "#<sys-tm \"%s\">" buf)))
+ )
+
+(define-cproc sys-asctime (tm::<sys-tm>)
+  (return (SCM_MAKE_STR_COPYING (asctime tm))))
+
+;; NB: For sys-ctime and sys-strftime, we don't use <const-cstring> return
+;; type to use autoboxing.
+;; See https://github.com/shirok/Gauche/issues/638#issuecomment-601777334
+(define-cproc sys-ctime (time)
+  (let* ([tim::time_t (Scm_GetSysTime time)])
+    (return (SCM_MAKE_STR_COPYING (ctime (& tim))))))
 
 (define-cproc sys-difftime (time1 time0) ::<double>
   (return (difftime (Scm_GetSysTime time1) (Scm_GetSysTime time0))))
 
 (define-cproc sys-strftime (format::<const-cstring> tm::<sys-tm>)
-  ::<const-cstring>
   (let* ([tmpbuf::(.array char [256])])
-    (strftime tmpbuf (sizeof tmpbuf) format (& (SCM_SYS_TM_TM tm)))
-    (return tmpbuf)))
+    (strftime tmpbuf (sizeof tmpbuf) format tm)
+    (return (SCM_MAKE_STR_COPYING tmpbuf))))
 
+;; NB: Windows doesn't have gmtime_r/localtime_r, but its gmtime/localtime
+;; is thread-safe.
+;; On Unix, we pass struct tm buf to gmtime_r/localtime_r, and we need
+;; to copy it before the buf goes out of scope.  So we don't use
+;; auto-boxing feature but manually call Scm_Make_sys_tm.
 (define-cproc sys-gmtime (time)
-  (let* ([tim::time_t (Scm_GetSysTime time)])
-    (return (Scm_MakeSysTm (gmtime (& tim))))))
+  (.if (defined "GAUCHE_WINDOWS")
+       (let* ([tim::time_t (Scm_GetSysTime time)])
+         (return (Scm_Make_sys_tm (gmtime (& tim)))))
+       (let* ([tim::time_t (Scm_GetSysTime time)]
+              [buf::(struct tm)])
+         (return (Scm_Make_sys_tm (gmtime_r (& tim) (& buf)))))))
 
 (define-cproc sys-localtime (time)
-  (let* ([tim::time_t (Scm_GetSysTime time)])
-    (return (Scm_MakeSysTm (localtime (& tim))))))
+  (.if (defined "GAUCHE_WINDOWS")
+       (let* ([tim::time_t (Scm_GetSysTime time)])
+         (return (Scm_Make_sys_tm (localtime (& tim)))))
+       (let* ([tim::time_t (Scm_GetSysTime time)]
+              [buf::(struct tm)])
+         (return (Scm_Make_sys_tm (localtime_r (& tim) (& buf)))))))
 
 (define-cproc sys-mktime (tm::<sys-tm>)
-  (return (Scm_MakeSysTime (mktime (& (SCM_SYS_TM_TM tm))))))
+  (return (Scm_MakeSysTime (mktime tm))))
 
 ;;---------------------------------------------------------------------
 ;; unistd.h - miscellaneous functions
@@ -932,6 +1064,8 @@
 (define-cproc sys-geteuid () ::<int> geteuid)
 (define-cproc sys-getuid ()  ::<int> getuid)
 
+(define-cproc sys-setugid? () ::<boolean> Scm_IsSugid) ; xBSD's issetugid()
+
 (define-cproc sys-getpid ()  ::<int> getpid)
 (define-cproc sys-getppid () ::<int> getppid)
 
@@ -972,6 +1106,18 @@
      (SCM_SYSCALL SCM_RESULT (setuid uid))
      (when (< SCM_RESULT 0) (Scm_SysError "setuid failed")))
 
+   (define-cfn call-nice (inc::int errno_save::int*) ::int
+     (set! errno 0)
+     (let* ([r::int (nice inc)])
+       (set! (* errno_save) errno)
+       (return r)))
+
+   (define-cproc sys-nice (inc::<int>) ::<int>
+     (let* ([errno_save::int 0])
+       (SCM_SYSCALL SCM_RESULT (call-nice inc (& errno_save)))
+       (when (and (< SCM_RESULT 0) (!= errno 0))
+         (Scm_SysError "nice failed"))))
+
    ;; some less-frequently used get-*
 
    (define-cproc sys-getgroups ()
@@ -1174,23 +1320,23 @@
 #define HOSTNAMELEN 1024
 #endif")
 
-(define-cproc sys-gethostname () ::<const-cstring>
+(define-cproc sys-gethostname ()
   (.if "defined HAVE_GETHOSTNAME"
        (let* ([buf::(.array char [HOSTNAMELEN])] [r::int])
          (SCM_SYSCALL r (gethostname buf HOSTNAMELEN))
          (when (< r 0) (Scm_SysError "gethostname failed"))
-         (return buf))
+         (return (SCM_MAKE_STR_COPYING buf)))
        ;; TODO: find better alternative
-       (return "localhost")))
+       (return (SCM_MAKE_STR_IMMUTABLE "localhost"))))
 
-(define-cproc sys-getdomainname () ::<const-cstring>
+(define-cproc sys-getdomainname ()
   (.if "defined HAVE_GETDOMAINNAME"
        (let* ([buf::(.array char [HOSTNAMELEN])] [r::int])
          (SCM_SYSCALL r (getdomainname buf HOSTNAMELEN))
          (when (< r 0) (Scm_SysError "getdomainame failed"))
-         (return buf))
+         (return (SCM_MAKE_STR_COPYING buf)))
        ;; TODO: find better alternative
-       (return "local")))
+       (return (SCM_MAKE_STR_IMMUTABLE "local"))))
 
 ;; not supported yet:
 ;;  fpathconf lseek pathconf read sysconf write
@@ -1224,8 +1370,6 @@
 ;; select
 
 (inline-stub
- (define-type <sys-fdset> "ScmSysFdset*")
-
  (when "defined(HAVE_SELECT)"
    ;; NB: On Windows, FD_SETSIZE merely indicates the maximum # of socket
    ;; descriptors fd_set can contain, and unrelated to the actual value
@@ -1304,10 +1448,9 @@
    (define-cfn handle-cleanup (h) ::void :static
      (CloseHandle (SCM_FOREIGN_POINTER_REF HANDLE h)))
 
-   (define-cfn handle-print (h p::ScmPort* c::ScmWriteContext*) ::void :static
+   (define-cfn handle-print (h p::ScmPort* _::ScmWriteContext*) ::void :static
      (let* ([type (Scm_ForeignPointerAttrGet (SCM_FOREIGN_POINTER h)
                                              'handle-type '#f)])
-       (cast void c) ; suppress unused var warning
        (cond
         [(SCM_EQ type 'process)
          (Scm_Printf p "#<win:handle process %d @%p>" (Scm_WinProcessPID h) h)]
@@ -1379,7 +1522,7 @@
     (and-let* ([ (global-variable-bound? (find-module 'gauche)
                                          'sys-win-pipe-name) ]
                [n (sys-win-pipe-name port-or-fd)])
-      (boolean (#/^\\msys-[\da-f]+-pty\d+-(to|from)-master$/ n))))
+      (boolean (#/^\\msys-[\da-f]+-pty\d+-(to|from)-master.*$/ n))))
   )
 
 ;; This is originally a part of shell-escape-string in gauche.process,
