@@ -1,7 +1,7 @@
 ;;;
 ;;; libvec.scm - builtin vector procedures
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,7 +35,8 @@
 
 (inline-stub
  (declcode (.include <gauche/vminsn.h>)
-           (.include <gauche/priv/vectorP.h>)))
+           (.include <gauche/priv/vectorP.h>)
+           (.include <gauche/priv/mmapP.h>)))
 
 ;;;
 ;;; Standard Vector
@@ -66,11 +67,13 @@
         [else (return (SCM_VECTOR_ELEMENT vec (SCM_INT_VALUE k)))]))
 
 (define-cproc vector-set! (vec::<vector> k::<integer> obj) ::<void>
-  (if (or (SCM_BIGNUMP k)
-          (< (SCM_INT_VALUE k) 0)
-          (>= (SCM_INT_VALUE k) (SCM_VECTOR_SIZE vec)))
-    (Scm_Error "vector-set! index out of range: %S" k)
-    (set! (SCM_VECTOR_ELEMENT vec (SCM_INT_VALUE k)) obj)))
+  (begin
+    (SCM_VECTOR_CHECK_MUTABLE vec)
+    (if (or (SCM_BIGNUMP k)
+            (< (SCM_INT_VALUE k) 0)
+            (>= (SCM_INT_VALUE k) (SCM_VECTOR_SIZE vec)))
+      (Scm_Error "vector-set! index out of range: %S" k)
+      (set! (SCM_VECTOR_ELEMENT vec (SCM_INT_VALUE k)) obj))))
 
 (define-cproc vector->list
   (vec::<vector> :optional (start::<fixnum> 0) (end::<fixnum> -1))
@@ -92,6 +95,7 @@
   (t::<vector> tstart::<fixnum> s::<vector>
                :optional (sstart::<fixnum> 0) (send::<fixnum> -1)) ::<void>
   (let* ([tsize::long (SCM_VECTOR_SIZE t)])
+    (SCM_VECTOR_CHECK_MUTABLE t)
     (when (< send 0) (set! send (SCM_VECTOR_SIZE s)))
     (unless (and (<= 0 tstart) (<= tstart tsize))
       (Scm_Error "tstart out of range: %ld" tstart))
@@ -134,6 +138,10 @@
           (set! j (+ j k))))
       (return dst))))
 
+(select-module gauche)
+(define-cproc vector-immutable? (v::<vector>) ::<boolean>
+  SCM_VECTOR_IMMUTABLE_P)
+
 ;; TRANSIENT: :optional thing will be expanded unhygienically, inserting
 ;; reference to 'error'.  If we define these within #<module scheme> it
 ;; would cause 'error' to be unbound.  Better fix would be to make
@@ -284,7 +292,7 @@
   (return (Scm_VMUVectorRef v SCM_UVECTOR_C128 i fallback)))
 
 (define-cproc uvector-set! (v::<uvector> i::<fixnum> val :optional clamp)
-  ::<void> :fast-flonum 
+  ::<void> :fast-flonum
   (Scm_UVectorSet v (Scm_UVectorType (Scm_ClassOf (SCM_OBJ v))) i val
                   (Scm_ClampMode clamp)))
 (define-cproc uvector-ref (v::<uvector> i::<fixnum> :optional fallback)
@@ -299,10 +307,100 @@
 (define-cproc uvector? (obj) ::<boolean> :constant
   SCM_UVECTORP)
 
+(define-cproc u8vector? (x) ::<boolean> SCM_U8VECTORP)
+(define-cproc s8vector? (x) ::<boolean> SCM_S8VECTORP)
+(define-cproc u16vector? (x) ::<boolean> SCM_U16VECTORP)
+(define-cproc s16vector? (x) ::<boolean> SCM_S16VECTORP)
+(define-cproc u32vector? (x) ::<boolean> SCM_U32VECTORP)
+(define-cproc s32vector? (x) ::<boolean> SCM_S32VECTORP)
+(define-cproc u64vector? (x) ::<boolean> SCM_U64VECTORP)
+(define-cproc s64vector? (x) ::<boolean> SCM_S64VECTORP)
+(define-cproc f16vector? (x) ::<boolean> SCM_F16VECTORP)
+(define-cproc f32vector? (x) ::<boolean> SCM_F32VECTORP)
+(define-cproc f64vector? (x) ::<boolean> SCM_F64VECTORP)
+(define-cproc c32vector? (x) ::<boolean> SCM_C32VECTORP)
+(define-cproc c64vector? (x) ::<boolean> SCM_C64VECTORP)
+(define-cproc c128vector? (x) ::<boolean> SCM_C128VECTORP)
+
+(define-cproc make-view-uvector (mem klass::<class> length::<integer>?
+                                     :optional (offset::<fixnum> 0)
+                                               (immutable?::<boolean> #f))
+  (unless (SCM_MEMORY_REGION_P mem)
+    (SCM_TYPE_ERROR mem "<memory-region>"))
+  (let* ([len::ScmSmallInt 0])
+    (cond [(SCM_FALSEP length) (set! len -1)]
+          [(SCM_INTP length) (set! len (SCM_INT_VALUE length))]
+          [else (Scm_Error "length is out of range: %S" length)])
+    (return (Scm_MakeViewUVector (SCM_MEMORY_REGION mem)
+                                 klass len offset immutable?))))
+
+;;;
+;;; Bitvectors
+;;;
+
+(define-cproc bitvector? (obj) ::<boolean> SCM_BITVECTORP) ;srfi-178
+(define-cproc bitvector-length (v::<bitvector>) ::<int>    ;srfi-178
+  SCM_BITVECTOR_SIZE)
+
+(define-cproc make-bitvector (len::<fixnum> :optional (init #f)) ;srfi-178
+  Scm_MakeBitvector)
+(define-cproc bitvector (:rest bits) Scm_ListToBitvector) ;srfi-178
+(define-cproc list->bitvector (bits) Scm_ListToBitvector) ;srfi-178
+
+(define-cproc string->bitvector (s::<string>) ;srfi-178
+  (return (Scm_StringToBitvector s TRUE)))
+(define-cproc bitvector->string (v::<bitvector>) ;srfi-178
+  (return (Scm_BitvectorToString v TRUE)))
+
+(define-cproc bit->integer (bit) ::<int> Scm_Bit2Int) ;srfi-178
+(define-cproc bit->boolean (bit) ::<boolean> Scm_Bit2Int) ;srfi-178
+
+(define-cproc bitvector-set! (v::<bitvector> i::<fixnum> b) ::<void> ;srfi-178
+  (when (or (< i 0) (>= i (SCM_BITVECTOR_SIZE v)))
+    (Scm_Error "bitvector index out of range: %ld" i))
+  (SCM_BITVECTOR_CHECK_MUTABLE v)
+  (if (Scm_Bit2Int b)
+    (SCM_BITS_SET (SCM_BITVECTOR_BITS v) i)
+    (SCM_BITS_RESET (SCM_BITVECTOR_BITS v) i)))
+
+(define-cproc bitvector-ref/int (v::<bitvector> i::<fixnum> ;srfi-178
+                                                :optional fallback)
+  (setter bitvector-set!)
+  (when (or (< i 0) (>= i (SCM_BITVECTOR_SIZE v)))
+    (if (SCM_UNBOUNDP fallback)
+      (Scm_Error "bitvector index out of range: %ld" i)
+      (return fallback)))
+  (return (?: (SCM_BITS_TEST (SCM_BITVECTOR_BITS v) i)
+              (SCM_MAKE_INT 1)
+              (SCM_MAKE_INT 0))))
+
+(define-cproc bitvector-ref/bool (v::<bitvector> i::<fixnum> ;srfi-178
+                                                 :optional fallback)
+  (setter bitvector-set!)
+  (when (or (< i 0) (>= i (SCM_BITVECTOR_SIZE v)))
+    (if (SCM_UNBOUNDP fallback)
+      (Scm_Error "bitvector index out of range: %ld" i)
+      (return fallback)))
+  (return (?: (SCM_BITS_TEST (SCM_BITVECTOR_BITS v) i) SCM_TRUE SCM_FALSE)))
+
+(define-cproc bitvector-copy (v::<bitvector>
+                              :optional (start::<fixnum> 0)
+                                        (end::<fixnum> -1)) ;srfi-178
+  Scm_BitvectorCopy)
+
+(define-cproc bitvector-copy! (dest::<bitvector>
+                               dstart::<fixnum>
+                               src::<bitvector>
+                               :optional (start::<fixnum> 0)
+                                         (end::<fixnum> -1)) ;srfi-178
+  Scm_BitvectorCopyX)
+
 ;;;
 ;;; Flat vector API (interact with underlying C array)
 ;;;
 
+(select-module gauche)
+
 (inline-stub
  (.define common_eqv (a b) (== a b))
  (.define common_lt  (a b) (< a b))
@@ -319,7 +417,7 @@
  (.define dc_lt (a b) (or (< (creal a) (creal b))
                           (and (== (creal a) (creal b))
                                (< (cimag a) (cimag b)))))
- 
+
  (define-cise-stmt %binary-search
    [(_ elttype eq lt)
     `(let* ([esize::u_int  (+ skip 1)]
@@ -344,7 +442,7 @@
            (set! (* ceil) (cast (size_t) -1))
            (set! (* ceil) (* hi esize))))
        (return (cast (size_t) -1)))])
- 
+
  (define-cfn Scm_BinarySearchS8 (vec::(const int8_t*)
                                  len::size_t
                                  key::int8_t
@@ -408,7 +506,7 @@
                                   floor::size_t*
                                   ceil::size_t*)
    ::size_t (%binary-search uint64_t common-eqv common-lt))
- 
+
  (define-cfn Scm_BinarySearchF16 (vec::(const ScmHalfFloat*)
                                   len::size_t
                                   key::ScmHalfFloat
@@ -440,20 +538,20 @@
                                   floor::size_t*
                                   ceil::size_t*)
    ::size_t (%binary-search ScmHalfComplex hc-eqv hc-lt))
- 
- (define-cfn Scm_BinarySearchC64 (vec::(const complex float*)
+
+ (define-cfn Scm_BinarySearchC64 (vec::(const ScmFloatComplex*)
                                   len::size_t
-                                  key::(complex float)
+                                  key::(ScmFloatComplex)
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search (complex float) common-eqv fc-lt))
+   ::size_t (%binary-search (ScmFloatComplex) common-eqv fc-lt))
 
- (define-cfn Scm_BinarySearchC128 (vec::(const complex double*)
+ (define-cfn Scm_BinarySearchC128 (vec::(const ScmDoubleComplex*)
                                    len::size_t
-                                   key::(complex double)
+                                   key::(ScmDoubleComplex)
                                    skip::u_int
                                    floor::size_t*
                                    ceil::size_t*)
-   ::size_t (%binary-search (complex double) common-eqv dc-lt))
+   ::size_t (%binary-search (ScmDoubleComplex) common-eqv dc-lt))
  )
