@@ -1,7 +1,7 @@
 /*
  * list.c - List related functions
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,6 +33,7 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+#include "gauche/priv/pairP.h"
 
 /*
  * Classes
@@ -63,8 +64,8 @@ ScmObj Scm_Cons(ScmObj car, ScmObj cdr)
        the register. */
     SCM_FLONUM_ENSURE_MEM(car);
     SCM_FLONUM_ENSURE_MEM(cdr);
-    SCM_SET_CAR(z, car);
-    SCM_SET_CDR(z, cdr);
+    SCM_SET_CAR_UNCHECKED(z, car);
+    SCM_SET_CDR_UNCHECKED(z, cdr);
     return SCM_OBJ(z);
 }
 
@@ -72,10 +73,10 @@ ScmObj Scm_Acons(ScmObj caar, ScmObj cdar, ScmObj cdr)
 {
     ScmPair *y = SCM_NEW(ScmPair);
     ScmPair *z = SCM_NEW(ScmPair);
-    SCM_SET_CAR(y, caar);
-    SCM_SET_CDR(y, cdar);
-    SCM_SET_CAR(z, SCM_OBJ(y));
-    SCM_SET_CDR(z, cdr);
+    SCM_SET_CAR_UNCHECKED(y, caar);
+    SCM_SET_CDR_UNCHECKED(y, cdar);
+    SCM_SET_CAR_UNCHECKED(z, SCM_OBJ(y));
+    SCM_SET_CDR_UNCHECKED(z, cdr);
     return SCM_OBJ(z);
 }
 
@@ -114,15 +115,15 @@ ScmObj Scm_VaList(va_list pvar)
     {
         if (SCM_NULLP(start)) {
             start = SCM_OBJ(SCM_NEW(ScmPair));
-            SCM_SET_CAR(start, obj);
-            SCM_SET_CDR(start, SCM_NIL);
+            SCM_SET_CAR_UNCHECKED(start, obj);
+            SCM_SET_CDR_UNCHECKED(start, SCM_NIL);
             cp = start;
         } else {
             ScmObj item;
             item = SCM_OBJ(SCM_NEW(ScmPair));
-            SCM_SET_CDR(cp, item);
-            SCM_SET_CAR(item, obj);
-            SCM_SET_CDR(item, SCM_NIL);
+            SCM_SET_CDR_UNCHECKED(cp, item);
+            SCM_SET_CAR_UNCHECKED(item, obj);
+            SCM_SET_CDR_UNCHECKED(item, SCM_NIL);
             cp = item;
         }
     }
@@ -201,6 +202,44 @@ CXR(Scm_Cadr, "cadr", D A)
 CXR(Scm_Cdar, "cdar", A D)
 CXR(Scm_Cddr, "cddr", D D)
 
+void Scm_SetCar(ScmObj pair, ScmObj obj)
+{
+    if (!SCM_PAIRP(pair)) {
+        Scm_Error("set-car!: Pair required, but got: %S", pair);
+    }
+    ScmExtendedPairDescriptor *d = Scm__GetExtendedPairDescriptor(pair);
+    if (d) {
+        if (d->flags & SCM_PAIR_IMMUTABLE) {
+            Scm_Error("attempt to mutate car of an immutable pair %S with %S",
+                      pair, obj);
+        }
+        if (d->setCar) {
+            d->setCar(pair, obj);
+            return;
+        }
+    }
+    SCM_CAR(pair) = obj;
+}
+
+void Scm_SetCdr(ScmObj pair, ScmObj obj)
+{
+    if (!SCM_PAIRP(pair)) {
+        Scm_Error("set-cdr!: Pair required, but got: %S", pair);
+    }
+    ScmExtendedPairDescriptor *d = Scm__GetExtendedPairDescriptor(pair);
+    if (d) {
+        if (d->flags & SCM_PAIR_IMMUTABLE) {
+            Scm_Error("attempt to mutate cdr of an immutable pair %S with %S",
+                      pair, obj);
+        }
+        if (d->setCdr) {
+            d->setCdr(pair, obj);
+            return;
+        }
+    }
+    SCM_CDR(pair) = obj;
+}
+
 /*
  * List manipulate routines:
  */
@@ -240,12 +279,23 @@ ScmSize Scm_Length(ScmObj obj)
 ScmObj Scm_CopyList(ScmObj list)
 {
     if (!SCM_PAIRP(list)) return list;
+    ScmObj tortoise = list, hare = list;
 
     ScmObj start = SCM_NIL, last = SCM_NIL;
-    SCM_FOR_EACH(list, list) {
-        SCM_APPEND1(start, last, SCM_CAR(list));
+    for (;;) {
+        if (!SCM_PAIRP(hare)) break;
+        SCM_APPEND1(start, last, SCM_CAR(hare));
+        hare = SCM_CDR(hare);
+
+        if (!SCM_PAIRP(hare)) break;
+        SCM_APPEND1(start, last, SCM_CAR(hare));
+        hare = SCM_CDR(hare);
+        tortoise = SCM_CDR(tortoise);
+        if (hare == tortoise) {
+            Scm_Error("Attempt to copy a circular list: %S", list);
+        }
     }
-    if (!SCM_NULLP(list)) SCM_SET_CDR(last, list);
+    if (!SCM_NULLP(hare)) SCM_SET_CDR_UNCHECKED(last, hare);
     return start;
 }
 
@@ -277,7 +327,7 @@ ScmObj Scm_Append2X(ScmObj list, ScmObj obj)
     ScmObj cp;
     SCM_FOR_EACH(cp, list) {
         if (SCM_NULLP(SCM_CDR(cp))) {
-            SCM_SET_CDR(cp, obj);
+            Scm_SetCdr(cp, obj);
             return list;
         }
     }
@@ -300,7 +350,7 @@ ScmObj Scm_Append2(ScmObj list, ScmObj obj)
     if (!SCM_NULLP(cp)) {
         Scm_Error("proper list required, but got %S", list);
     }
-    SCM_SET_CDR(last, obj);
+    SCM_SET_CDR_UNCHECKED(last, obj);
 
     return start;
 }
@@ -311,7 +361,7 @@ ScmObj Scm_Append(ScmObj args)
     SCM_FOR_EACH(cp, args) {
         if (!SCM_PAIRP(SCM_CDR(cp))) {
             if (SCM_NULLP(start)) return SCM_CAR(cp);
-            SCM_SET_CDR(last, SCM_CAR(cp));
+            SCM_SET_CDR_UNCHECKED(last, SCM_CAR(cp));
             break;
         } else if (SCM_NULLP(SCM_CAR(cp))) {
             continue;
@@ -340,15 +390,15 @@ ScmObj Scm_Reverse2(ScmObj list, ScmObj tail)
     if (!SCM_PAIRP(list)) return tail;
 
     ScmPair *p = SCM_NEW(ScmPair);
-    SCM_SET_CAR(p, SCM_NIL);
-    SCM_SET_CDR(p, tail);
+    SCM_SET_CAR_UNCHECKED(p, SCM_NIL);
+    SCM_SET_CDR_UNCHECKED(p, tail);
     ScmObj result = SCM_OBJ(p);
     ScmObj cp;
     SCM_FOR_EACH(cp, list) {
-        SCM_SET_CAR(result, SCM_CAR(cp));
+        SCM_SET_CAR_UNCHECKED(result, SCM_CAR(cp));
         p = SCM_NEW(ScmPair);
-        SCM_SET_CAR(p, SCM_NIL);
-        SCM_SET_CDR(p, result);
+        SCM_SET_CAR_UNCHECKED(p, SCM_NIL);
+        SCM_SET_CDR_UNCHECKED(p, result);
         result = SCM_OBJ(p);
     }
     return SCM_CDR(result);
@@ -373,7 +423,7 @@ ScmObj Scm_Reverse2X(ScmObj list, ScmObj tail)
     ScmObj first, next, result = tail;
     for (first = list; SCM_PAIRP(first); first = next) {
         next = SCM_CDR(first);
-        SCM_SET_CDR(first, result);
+        Scm_SetCdr(first, result);
         result = first;
     }
     return result;
@@ -492,7 +542,7 @@ ScmObj Scm_DeleteX(ScmObj obj, ScmObj list, int cmpmode)
             if (SCM_NULLP(prev)) {
                 list = SCM_CDR(cp);
             } else {
-                SCM_SET_CDR(prev, SCM_CDR(cp));
+                Scm_SetCdr(prev, SCM_CDR(cp));
             }
         } else {
             prev = cp;
@@ -583,7 +633,7 @@ ScmObj Scm_AssocDeleteX(ScmObj elt, ScmObj alist, int cmpmode)
                     alist = SCM_CDR(cp);
                     continue;
                 } else {
-                    SCM_SET_CDR(prev, SCM_CDR(cp));
+                    Scm_SetCdr(prev, SCM_CDR(cp));
                 }
             }
         }
@@ -613,7 +663,7 @@ ScmObj Scm_DeleteDuplicatesX(ScmObj list, int cmpmode)
     SCM_FOR_EACH(lp, list) {
         ScmObj obj = SCM_CAR(lp);
         ScmObj tail = Scm_DeleteX(obj, SCM_CDR(lp), cmpmode);
-        if (SCM_CDR(lp) != tail) SCM_SET_CDR(lp, tail);
+        if (SCM_CDR(lp) != tail) Scm_SetCdr(lp, tail);
     }
     return list;
 }
@@ -636,23 +686,24 @@ ScmObj Scm_DeleteDuplicatesX(ScmObj list, int cmpmode)
  *  Since the algorithm is generally useful, I implement the core routine
  *  of the algorithm here.
  *
- *  TODO at 1.0: I noticed START argument actually isn't used in the
+ *  TRANSIENT: I noticed START argument actually isn't used in the
  *  algorithm at all.  We can drop it and the caller can just say
  *  Scm_Cons(start, Scm_MonotonicMerge(sequences)).   We can't change it
  *  now because of ABI compatibility, but it will be nice to do so when
  *  releasing 1.0.
  */
 
-/* DEPRECATED 1.0 */
+#if GAUCHE_API_VERSION < 98
 ScmObj Scm_MonotonicMerge(ScmObj start, ScmObj sequences)
 {
     ScmObj r = Scm_MonotonicMerge1(sequences);
     if (!SCM_FALSEP(r)) r = Scm_Cons(start, r);
     return r;
 }
-
-/* WILL RENAME IN 1.0 */
 ScmObj Scm_MonotonicMerge1(ScmObj sequences)
+#else  /*GAUCHE_API_VERSION >= 98 */
+ScmObj Scm_MonotonicMerge(ScmObj sequences)
+#endif /*GAUCHE_API_VERSION >= 98 */
 {
     ScmObj result = SCM_NIL;
     int nseqs = Scm_Length(sequences);
@@ -703,9 +754,57 @@ ScmObj Scm_MonotonicMerge1(ScmObj sequences)
 }
 
 /*
- * Pair attributes
+ * Extended pairs
  */
 
+static ScmObj make_extended_pair(ScmExtendedPairDescriptor *desc,
+                                 ScmObj car, ScmObj cdr, ScmObj attrs)
+{
+    ScmRealExtendedPair *xp = SCM_NEW(ScmRealExtendedPair);
+    /* ScmRealExtendedPair is not an ScmObj, and
+       ScmExtendedPairDescriptor is not an ScmClass.   To avoid confusion,
+       we manually tweak tag bits.
+    */
+    xp->hiddenTag = SCM_WORD((ScmByte*)desc + 7);
+    xp->data.car = car;
+    xp->data.cdr = cdr;
+    xp->data.attributes = attrs;
+    return SCM_OBJ(&xp->data);  /* hide the first word  */
+}
+
+/* "vanilla" extended pair.  used mainly to hold extra attributes, but
+   otherwise behaves like normal pairs.
+   NB: Static initialization with ScmClass* requires extra care on
+   Windows.  To avoid complication, we initialize the klass field in
+   _Init() routine.
+*/
+static ScmExtendedPairDescriptor mpair_desc = {
+    NULL,                       /* will be SCM_CLASS_PAIR */
+    0,
+    NULL,
+    NULL
+};
+
+ScmObj Scm_MakeExtendedPair(ScmObj car, ScmObj cdr, ScmObj attrs)
+{
+    return make_extended_pair(&mpair_desc, car, cdr, attrs);
+}
+
+/* Returns NULL if p isn't an extended pair. */
+ScmExtendedPairDescriptor *Scm__GetExtendedPairDescriptor(ScmObj p)
+{
+    if (!SCM_EXTENDED_PAIR_P(p)) return NULL;
+    ScmRealExtendedPair *z = (ScmRealExtendedPair*)(((ScmObj*)p) - 1);
+    SCM_ASSERT((z->hiddenTag&0x7) == 0x7);
+    return (ScmExtendedPairDescriptor *)(z->hiddenTag-7);
+}
+
+/* Returns vanilla extended pair descriptor */
+ScmExtendedPairDescriptor *Scm__GetDefaultExtendedPairDesctiptor()
+{
+    return &mpair_desc;
+}
+
 ScmObj Scm_PairAttr(ScmPair *pair)
 {
     if (SCM_EXTENDED_PAIR_P(pair)) {
@@ -715,15 +814,6 @@ ScmObj Scm_PairAttr(ScmPair *pair)
     }
 }
 
-ScmObj Scm_MakeExtendedPair(ScmObj car, ScmObj cdr, ScmObj attrs)
-{
-    ScmExtendedPair *xp = SCM_NEW(ScmExtendedPair);
-    xp->car = car;
-    xp->cdr = cdr;
-    xp->attributes = attrs;
-    return SCM_OBJ(xp);
-}
-
 /* The common scenario is to use ExtendedCons in place of Cons,
    then add attributes later.  So we provide this.  */
 ScmObj Scm_ExtendedCons(ScmObj car, ScmObj cdr)
@@ -753,11 +843,77 @@ ScmObj Scm_PairAttrSet(ScmPair *pair, ScmObj key, ScmObj value)
                   key, SCM_OBJ(pair));
     }
 
-    ScmObj p = Scm_Assq(key, SCM_EXTENDED_PAIR(pair)->attributes);
-    if (SCM_PAIRP(p)) SCM_SET_CDR(p, value);
-    else SCM_EXTENDED_PAIR(pair)->attributes
-        = Scm_Acons(key, value, SCM_EXTENDED_PAIR(pair)->attributes);
+    /* We update the attribute list non-destructively.
+       In case if two threads update the list simultaneously, one change
+       can be lost, but no invalid access can occur.  Guaranteeing mutex
+       is the caller's responsibility, though.
+    */
+    ScmObj xs, attrs = SCM_EXTENDED_PAIR(pair)->attributes;
+    SCM_FOR_EACH(xs, attrs) {
+        SCM_ASSERT(SCM_PAIRP(SCM_CAR(xs)));
+        if (SCM_EQ(SCM_CAAR(xs), key)) {
+            /* need to copy up to here */
+            ScmObj h = SCM_NIL, t = SCM_NIL, ys;
+            SCM_FOR_EACH(ys, attrs) {
+                if (SCM_EQ(ys, xs)) {
+                    SCM_APPEND1(h, t, Scm_Cons(key, value));
+                    SCM_APPEND(h, t, SCM_CDR(xs));
+                    SCM_EXTENDED_PAIR(pair)->attributes = h;
+                    return SCM_UNDEFINED;
+                }
+                SCM_APPEND1(h, t, SCM_CAR(ys));
+            }
+            SCM_ASSERT(FALSE);  /* Shouldn't be here */
+        }
+    }
+    SCM_EXTENDED_PAIR(pair)->attributes = Scm_Acons(key, value, attrs);
     return SCM_UNDEFINED;
 }
 
+/*
+ * Immutable pairs
+ */
 
+/* not cheap.  usually you don't want to check this and let it catched
+   in set-car!/set-cdr!. */
+int Scm_ImmutablePairP(ScmObj obj)
+{
+    if (!SCM_EXTENDED_PAIR_P(obj)) return FALSE;
+    ScmExtendedPairDescriptor *d = Scm__GetExtendedPairDescriptor(obj);
+    return d->flags & SCM_PAIR_IMMUTABLE;
+}
+
+static ScmExtendedPairDescriptor ipair_desc = {
+    NULL,                       /* will be SCM_CLASS_PAIR.  see above. */
+    SCM_PAIR_IMMUTABLE,
+    NULL,
+    NULL
+};
+
+ScmObj Scm_MakeImmutablePair(ScmObj car, ScmObj cdr)
+{
+    return make_extended_pair(&ipair_desc, car, cdr, SCM_NIL);
+}
+
+
+void Scm__InitList()
+{
+    mpair_desc.klass = SCM_CLASS_PAIR;
+    ipair_desc.klass = SCM_CLASS_PAIR;
+}
+
+/* Temporary - Check if normal pairs are all aligned with 2-word boundary */
+#if GAUCHE_CHECK_PAIR_ALIGNMENT
+int Scm_CheckingPairP(ScmObj obj)
+{
+    if (SCM_HPTRP(obj)&&(SCM_HTAG(obj)!=7||Scm_PairP(SCM_OBJ(obj)))) {
+        if (SCM_WORD(obj) & SIZEOF_LONG) {
+            if ((SCM_WORD((ScmObj*)obj - 1) & 7) != 7) {
+                fprintf(stderr, "Unaligned pair %p\n", obj);
+            }
+        }
+        return TRUE;
+    }
+    return FALSE;
+}
+#endif /*GAUCHE_CHECK_PAIR_ALIGNMENT*/
