@@ -1,7 +1,7 @@
 /*
  * load.c - load a program
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,7 +33,6 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/class.h"
 #include "gauche/port.h"
 #include "gauche/priv/builtin-syms.h"
 #include "gauche/priv/readerP.h"
@@ -73,7 +72,7 @@ static struct {
                                             searched. */
     ScmPrimitiveParameter *load_port;    /* current port from which we are
                                             loading */
-    
+
     /* Dynamic linking */
     ScmObj dso_suffixes;
     ScmHashTable *dso_table;      /* Hashtable path -> <dlobj> */
@@ -84,6 +83,8 @@ static struct {
                                      are already called by the application,
                                      but we may change this design in future.
                                   */
+    ScmClass *dlptr_class;        /* Foreign pointer class for the address
+                                     retrieved from dso. */
     ScmInternalMutex dso_mutex;
 } ldinfo;
 
@@ -93,6 +94,7 @@ static ScmObj key_macro              = SCM_UNBOUND;
 static ScmObj key_ignore_coding      = SCM_UNBOUND;
 static ScmObj key_paths              = SCM_UNBOUND;
 static ScmObj key_environment        = SCM_UNBOUND;
+static ScmObj key_main_script        = SCM_UNBOUND;
 
 #define PARAM_REF(vm, loc)      Scm_PrimitiveParameterRef(vm, ldinfo.loc)
 
@@ -195,6 +197,9 @@ ScmObj Scm_VMLoad(ScmString *filename, ScmObj paths, ScmObj env, int flags)
     if (flags&SCM_LOAD_IGNORE_CODING) {
         opts = Scm_Cons(key_ignore_coding, Scm_Cons(SCM_TRUE, opts));
     }
+    if (flags&SCM_LOAD_MAIN_SCRIPT) {
+        opts = Scm_Cons(key_main_script, Scm_Cons(SCM_TRUE, opts));
+    }
     if (SCM_NULLP(paths) || SCM_PAIRP(paths)) {
         opts = Scm_Cons(key_paths, Scm_Cons(paths, opts));
     }
@@ -217,6 +222,9 @@ int Scm_Load(const char *cpath, u_long flags, ScmLoadPacket *packet)
     if (flags&SCM_LOAD_IGNORE_CODING) {
         opts = Scm_Cons(key_ignore_coding, Scm_Cons(SCM_TRUE, opts));
     }
+    if (flags&SCM_LOAD_MAIN_SCRIPT) {
+        opts = Scm_Cons(key_main_script, Scm_Cons(SCM_TRUE, opts));
+    }
 
     load_packet_prepare(packet);
     if (flags&SCM_LOAD_PROPAGATE_ERROR) {
@@ -251,7 +259,7 @@ int Scm_LoadFromCString(const char *program, u_long flags, ScmLoadPacket *p)
 ScmObj Scm_GetLoadPath(void)
 {
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
-    ScmObj paths = Scm_CopyList(ldinfo.load_path_rec->value);
+    ScmObj paths = Scm_CopyList(Scm_GlocGetValue(ldinfo.load_path_rec));
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
     return paths;
 }
@@ -259,7 +267,7 @@ ScmObj Scm_GetLoadPath(void)
 ScmObj Scm_GetDynLoadPath(void)
 {
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
-    ScmObj paths = Scm_CopyList(ldinfo.dynload_path_rec->value);
+    ScmObj paths = Scm_CopyList(Scm_GlocGetValue(ldinfo.dynload_path_rec));
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
     return paths;
 }
@@ -284,16 +292,12 @@ static ScmObj break_env_paths(const char *envname)
     }
 }
 
-static ScmObj add_list_item(ScmObj orig, ScmObj item, int afterp)
+static void add_gloc_list_item(ScmGloc *gloc, ScmObj item, int afterp)
 {
-    if (afterp) {
-        return Scm_Append2(orig, SCM_LIST1(item));
-    } else {
-        return Scm_Cons(item, orig);
-    }
+    ScmObj vs = Scm_GlocGetValue(gloc);
+    ScmObj r = afterp? Scm_Append2(vs, SCM_LIST1(item)) : Scm_Cons(item, vs);
+    Scm_GlocSetValue(gloc, r);
 }
-#define ADD_LIST_ITEM(list, item, afterp) \
-    list = add_list_item(list, item, afterp)
 
 /* Add CPATH to the current list of load path.  The path is
  * added before the current list, unless AFTERP is true.
@@ -323,9 +327,9 @@ ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
     }
 
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
-    ADD_LIST_ITEM(ldinfo.load_path_rec->value, spath, afterp);
-    ADD_LIST_ITEM(ldinfo.dynload_path_rec->value, dpath, afterp);
-    ScmObj r = ldinfo.load_path_rec->value;
+    add_gloc_list_item(ldinfo.load_path_rec, spath, afterp);
+    add_gloc_list_item(ldinfo.dynload_path_rec, dpath, afterp);
+    ScmObj r = Scm_GlocGetValue(ldinfo.load_path_rec);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
 
     return r;
@@ -334,7 +338,7 @@ ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
 void Scm_AddLoadPathHook(ScmObj proc, int afterp)
 {
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
-    ADD_LIST_ITEM(ldinfo.load_path_hooks_rec->value, proc, afterp);
+    add_gloc_list_item(ldinfo.load_path_hooks_rec, proc, afterp);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
 }
 
@@ -343,8 +347,9 @@ void Scm_DeleteLoadPathHook(ScmObj proc)
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
     /* we should use Scm_Delete, instead of Scm_DeleteX,
        to avoid race with reader of the list */
-    ldinfo.load_path_hooks_rec->value
-        = Scm_Delete(proc, ldinfo.load_path_hooks_rec->value, SCM_CMP_EQ);
+    Scm_GlocSetValue(ldinfo.load_path_hooks_rec,
+                     Scm_Delete(proc, Scm_GlocGetValue(ldinfo.load_path_hooks_rec),
+                                SCM_CMP_EQ));
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
 }
 
@@ -371,7 +376,7 @@ void Scm_DeleteLoadPathHook(ScmObj proc)
  *     of the calling program itself in such a case, but we never need that
  *     behavior.
  *
- *   ScmDynloadInitFn dl_sym(void *handle, const char *symbol)
+ *   ScmDynLoadEntry dl_sym(void *handle, const char *symbol)
  *     Finds the address of SYMBOL in the dl_openModule()-ed module
  *     HANDLE.
  *
@@ -410,14 +415,7 @@ void Scm_DeleteLoadPathHook(ScmObj proc)
    keyword arguments.
  */
 
-typedef void (*ScmDynLoadInitFn)(void);
-
-typedef struct dlobj_initfn_rec {
-    struct dlobj_initfn_rec *next; /* chain */
-    const char *name;           /* name of initfn (always w/ leading '_') */
-    ScmDynLoadInitFn fn;        /* function ptr */
-    int initialized;            /* TRUE once fn returns */
-} dlobj_initfn;
+typedef void (*ScmDynLoadEntry)(void); /* Dynamically loaded function pointer */
 
 struct ScmDLObjRec {
     SCM_HEADER;
@@ -428,15 +426,15 @@ struct ScmDLObjRec {
     void *handle;               /* whatever dl_open returned */
     ScmVM *loader;              /* The VM that's holding the lock to operate
                                    on this DLO. */
-    dlobj_initfn *initfns;      /* list of initializer functions */
+    ScmHashCore entries;        /* name -> <foreign-pointer> */
     ScmInternalMutex mutex;
     ScmInternalCond  cv;
 };
 
-static void dlobj_print(ScmObj obj, ScmPort *sink, 
+static void dlobj_print(ScmObj obj, ScmPort *sink,
                         ScmWriteContext *mode SCM_UNUSED)
 {
-    Scm_Printf(sink, "#<dlobj \"%s\">", SCM_DLOBJ(obj)->path);
+    Scm_Printf(sink, "#<dlobj %S>", SCM_DLOBJ(obj)->path);
 }
 
 SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_DLObjClass, dlobj_print);
@@ -448,7 +446,8 @@ static ScmDLObj *make_dlobj(ScmString *path)
     z->path = path;
     z->loader = NULL;
     z->loaded = FALSE;
-    z->initfns = NULL;
+    z->handle = NULL;
+    Scm_HashCoreInitSimple(&z->entries, SCM_HASH_STRING, 0, NULL);
     (void)SCM_INTERNAL_MUTEX_INIT(z->mutex);
     (void)SCM_INTERNAL_COND_INIT(z->cv);
     return z;
@@ -502,21 +501,43 @@ static void unlock_dlobj(ScmDLObj *dlo)
     (void)SCM_INTERNAL_MUTEX_UNLOCK(dlo->mutex);
 }
 
-/* find dlobj_initfn from the given dlobj with name.
+/* Find NAME in the looked-up entries.
+   NAME must begin with '_'.
    Assuming the caller holding the lock of OBJ. */
-static dlobj_initfn *find_initfn(ScmDLObj *dlo, const char *name)
+static ScmObj find_entry(ScmDLObj *dlo, ScmString *name)
 {
-    dlobj_initfn *fns = dlo->initfns;
-    for (; fns != NULL; fns = fns->next) {
-        if (strcmp(name, fns->name) == 0) return fns;
+    ScmDictEntry *e = Scm_HashCoreSearch(&dlo->entries, (intptr_t)name,
+                                         SCM_DICT_GET);
+    if (e) return SCM_DICT_VALUE(e);
+    else   return SCM_FALSE;
+}
+
+/* lookup the symbol within DLO.
+   NAME must begin with '_'.   We look up both with and without '_'.
+   Assuming the caller holding the lock of OBJ. */
+static ScmObj lookup_entry(ScmDLObj *dlo, ScmString *name)
+{
+    ScmObj fptr = find_entry(dlo, name);
+    if (SCM_FALSEP(fptr)) {
+        /* locate the entry.  Name always has '_'.  Whether the actual
+           symbol dl_sym returns has '_' or not depends on the platform,
+           so we first try without '_', then '_'. */
+        const char *cname = Scm_GetStringConst(name);
+        void *ptr = dl_sym(dlo->handle, cname+1);
+        if (ptr == NULL) {
+            ptr = dl_sym(dlo->handle, cname);
+            if (ptr == NULL) {
+                return SCM_FALSE; /* not found */
+            }
+        }
+        fptr = Scm_MakeForeignPointer(ldinfo.dlptr_class, ptr);
+        Scm_ForeignPointerAttrSet(SCM_FOREIGN_POINTER(fptr),
+                                  SCM_SYM_NAME, SCM_OBJ(name));
+        ScmDictEntry *e = Scm_HashCoreSearch(&dlo->entries, (intptr_t)name,
+                                             SCM_DICT_CREATE);
+        (void)SCM_DICT_SET_VALUE(e, fptr);
     }
-    fns = SCM_NEW(dlobj_initfn);
-    fns->name = name;
-    fns->fn = NULL;
-    fns->initialized = FALSE;
-    fns->next = dlo->initfns;
-    dlo->initfns = fns;
-    return fns;
+    return fptr;
 }
 
 /* Load the DSO.  The caller holds the lock of dlobj.  May throw an error;
@@ -545,28 +566,22 @@ static void load_dlo(ScmDLObj *dlo)
 
 /* Call the DSO's initfn.  The caller holds the lock of dlobj, and responsible
    to release the lock even when this fn throws an error. */
-static void call_initfn(ScmDLObj *dlo, const char *name)
+static void call_initfn(ScmDLObj *dlo, ScmString *name)
 {
-    dlobj_initfn *ifn = find_initfn(dlo, name);
+    ScmObj fptr = lookup_entry(dlo, name);
 
-    if (ifn->initialized) return;
+    if (!SCM_FOREIGN_POINTER_P(fptr)) {
+        dl_close(dlo->handle);
+        dlo->handle = NULL;
+        dlo->loaded = FALSE;
+        Scm_Error("dynamic linking of %A failed: "
+                  "couldn't find initialization function %S",
+                  dlo->path, name);
+    }
 
-    if (!ifn->fn) {
-        /* locate initfn.  Name always has '_'.  Whether the actual
-           symbol dl_sym returns has '_' or not depends on the platform,
-           so we first try without '_', then '_'. */
-        ifn->fn = dl_sym(dlo->handle, name+1);
-        if (ifn->fn == NULL) {
-            ifn->fn = (void(*)(void))dl_sym(dlo->handle, name);
-            if (ifn->fn == NULL) {
-                dl_close(dlo->handle);
-                dlo->handle = NULL;
-                Scm_Error("dynamic linking of %A failed: "
-                          "couldn't find initialization function %s",
-                          dlo->path, name);
-                /*NOTREACHED*/
-            }
-        }
+    if (!SCM_FALSEP(Scm_ForeignPointerAttrGet(SCM_FOREIGN_POINTER(fptr),
+                                              SCM_SYM_CALLED, SCM_FALSE))) {
+        return;
     }
 
     /* Call initialization function.  note that there can be arbitrary
@@ -577,8 +592,10 @@ static void call_initfn(ScmDLObj *dlo, const char *name)
        loading right now.  However, if the code follows the Gauche's
        standard module structure, such circular dependency is detected
        by Scm_Load, so we don't worry about it here. */
-    ifn->fn();
-    ifn->initialized = TRUE;
+    ScmDynLoadEntry fn = SCM_FOREIGN_POINTER_REF(ScmDynLoadEntry, fptr);
+    fn();
+    Scm_ForeignPointerAttrSet(SCM_FOREIGN_POINTER(fptr),
+                              SCM_SYM_NAME, SCM_TRUE);
 }
 
 /* Experimental: Prelink feature---we allow the extension module to be
@@ -594,7 +611,7 @@ static void call_initfn(ScmDLObj *dlo, const char *name)
    names with '_' first. */
 void Scm_RegisterPrelinked(ScmString *dsoname,
                            const char *initfn_names[],
-                           ScmDynLoadInitFn initfns[])
+                           ScmDynLoadEntry initfns[])
 {
     ScmObj path = Scm_StringAppend2(SCM_STRING(SCM_MAKE_STR_IMMUTABLE("@/")),
                                     dsoname);
@@ -603,9 +620,7 @@ void Scm_RegisterPrelinked(ScmString *dsoname,
 
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.dso_mutex);
     for (int i=0; initfns[i] && initfn_names[i]; i++) {
-        dlobj_initfn *ifn = find_initfn(dlo, initfn_names[i]);
-        SCM_ASSERT(ifn->fn == NULL);
-        ifn->fn = initfns[i];
+        lookup_entry(dlo, SCM_STRING(SCM_MAKE_STR_IMMUTABLE(initfn_names[i])));
     }
     ldinfo.dso_prelinked = Scm_Cons(SCM_OBJ(dsoname), ldinfo.dso_prelinked);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
@@ -630,8 +645,16 @@ static ScmObj find_prelinked(ScmString *dsoname)
 /* Dynamically load the specified object by DSONAME.
    DSONAME must not contain the system's suffix (.so, for example).
    The same name of DSO can be only loaded once.
+
    A DSO may contain multiple initialization functions (initfns), in
    which case each initfn is called at most once.
+
+   If INITFN is SCM_TRUE, the name of initialization function is derived
+   from the DSO name (see %get-initfn-name in libeval.scm).  This is
+   the default value of 'dynamic-load'.
+
+   If INITFN is SCM_FALSE, the initialization function won't be called.
+   It is to load DSO for FFI.
 */
 ScmObj Scm_DynLoad(ScmString *dsoname, ScmObj initfn,
                    u_long flags SCM_UNUSED /*reserved*/)
@@ -651,11 +674,18 @@ ScmObj Scm_DynLoad(ScmString *dsoname, ScmObj initfn,
         dsopath = SCM_CAR(spath);
         SCM_ASSERT(SCM_STRINGP(dsopath));
     }
-    static ScmObj get_initfn_name_proc = SCM_UNDEFINED;
-    SCM_BIND_PROC(get_initfn_name_proc, "%get-initfn-name",
-                  Scm_GaucheInternalModule());
-    ScmObj s_initname = Scm_ApplyRec2(get_initfn_name_proc, initfn, dsopath);
-    const char *initname = Scm_GetStringConst(SCM_STRING(s_initname));
+
+    ScmObj initname = SCM_FALSE;
+
+    if (SCM_EQ(initfn, SCM_TRUE) || SCM_STRINGP(initfn)) {
+        static ScmObj get_initfn_name_proc = SCM_UNDEFINED;
+        SCM_BIND_PROC(get_initfn_name_proc, "%get-initfn-name",
+                      Scm_GaucheInternalModule());
+        initname = Scm_ApplyRec2(get_initfn_name_proc, initfn, dsopath);
+    } else if (!SCM_FALSEP(initfn)) {
+        SCM_TYPE_ERROR(initfn, "a string or a boolean");
+    }
+
     ScmDLObj *dlo = find_dlobj(dsopath);
 
     /* Load the dlobj if necessary. */
@@ -669,12 +699,14 @@ ScmObj Scm_DynLoad(ScmString *dsoname, ScmObj initfn,
     /* Now the dlo is loaded.  We need to call initializer. */
     SCM_ASSERT(dlo->loaded);
 
-    SCM_UNWIND_PROTECT { call_initfn(dlo, initname); }
-    SCM_WHEN_ERROR { unlock_dlobj(dlo);  SCM_NEXT_HANDLER; }
-    SCM_END_PROTECT;
+    if (SCM_STRINGP(initname)) {
+        SCM_UNWIND_PROTECT { call_initfn(dlo, SCM_STRING(initname)); }
+        SCM_WHEN_ERROR { unlock_dlobj(dlo);  SCM_NEXT_HANDLER; }
+        SCM_END_PROTECT;
+    }
 
     unlock_dlobj(dlo);
-    return SCM_TRUE;
+    return SCM_OBJ(dlo);
 }
 
 /* Expose dlobj to Scheme world */
@@ -689,16 +721,19 @@ static ScmObj dlobj_loaded_get(ScmObj obj)
     return SCM_MAKE_BOOL(SCM_DLOBJ(obj)->loaded);
 }
 
-static ScmObj dlobj_initfns_get(ScmObj obj)
+static ScmObj dlobj_entries_get(ScmObj obj)
 {
     ScmObj h = SCM_NIL;
     ScmObj t = SCM_NIL;
+    ScmDLObj *dlo = SCM_DLOBJ(obj);
+    ScmHashIter iter;
+
     lock_dlobj(SCM_DLOBJ(obj));
-    dlobj_initfn *ifn = SCM_DLOBJ(obj)->initfns;
-    for (;ifn != NULL; ifn = ifn->next) {
-        ScmObj p = Scm_Cons(SCM_MAKE_STR_IMMUTABLE(ifn->name),
-                            SCM_MAKE_BOOL(ifn->initialized));
-        SCM_APPEND1(h, t, p);
+    Scm_HashIterInit(&iter, &dlo->entries);
+    for (;;) {
+        ScmDictEntry *e = Scm_HashIterNext(&iter);
+        if (e == NULL) break;
+        SCM_APPEND1(h, t, SCM_DICT_VALUE(e));
     }
     unlock_dlobj(SCM_DLOBJ(obj));
     return h;
@@ -707,7 +742,7 @@ static ScmObj dlobj_initfns_get(ScmObj obj)
 static ScmClassStaticSlotSpec dlobj_slots[] = {
     SCM_CLASS_SLOT_SPEC("path", dlobj_path_get, NULL),
     SCM_CLASS_SLOT_SPEC("loaded?", dlobj_loaded_get, NULL),
-    SCM_CLASS_SLOT_SPEC("init-functions", dlobj_initfns_get, NULL),
+    SCM_CLASS_SLOT_SPEC("entries", dlobj_entries_get, NULL),
     SCM_CLASS_SLOT_SPEC_END()
 };
 
@@ -725,6 +760,31 @@ ScmObj Scm_DLObjs()
     return z;
 }
 
+/* name should have '_' prefix.  We look for a symbol with and without it.
+   Returns a foreign pointer or #f. */
+ScmObj Scm_DLOGetEntryAddress(ScmDLObj *dlo, ScmString *name)
+{
+    lock_dlobj(dlo);
+    ScmObj fptr = lookup_entry(dlo, name);
+    unlock_dlobj(dlo);
+    return fptr;
+}
+
+/* dlptr interface (we don't expose <dlptr> class pointer */
+int Scm_DLPtrP(ScmObj obj)
+{
+    return SCM_XTYPEP(obj, ldinfo.dlptr_class);
+}
+
+ScmObj Scm_DLPtrValue(ScmObj obj)
+{
+    if (!Scm_DLPtrP(obj)) {
+        SCM_TYPE_ERROR(obj, "dlptr");
+    }
+    intptr_t val = SCM_FOREIGN_POINTER_REF(intptr_t, obj);
+    return Scm_IntptrToInteger(val);
+}
+
 /*------------------------------------------------------------------
  * Require and provide
  */
@@ -917,7 +977,7 @@ ScmObj Scm_Provide(ScmObj feature)
     ScmObj cp;
     SCM_FOR_EACH(cp, ldinfo.providing) {
         if (SCM_CADR(SCM_CAR(cp)) == SCM_OBJ(self)) {
-            SCM_SET_CDR(SCM_CDR(SCM_CAR(cp)), SCM_LIST1(feature));
+            SCM_SET_CDR_UNCHECKED(SCM_CDR(SCM_CAR(cp)), SCM_LIST1(feature));
             break;
         }
     }
@@ -938,7 +998,7 @@ int Scm_ProvidedP(ScmObj feature)
  * Autoload
  */
 
-static void autoload_print(ScmObj obj, ScmPort *out, 
+static void autoload_print(ScmObj obj, ScmPort *out,
                            ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(out, "#<autoload %A::%A (%A)>",
@@ -1082,18 +1142,18 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags SCM_UNUSED)
             ScmGloc *g = Scm_FindBinding(adata->module, adata->name, 0);
             SCM_ASSERT(f != NULL);
             SCM_ASSERT(g != NULL);
-            adata->value = SCM_GLOC_GET(f);
+            adata->value = Scm_GlocGetValue(f);
             if (SCM_UNBOUNDP(adata->value) || SCM_AUTOLOADP(adata->value)) {
                 Scm_Error("Autoloaded symbol %S is not defined in the module %S",
                           adata->name, adata->import_from);
             }
-            SCM_GLOC_SET(g, adata->value);
+            Scm_GlocSetValue(g, adata->value);
         } else {
             /* Normal import.  The binding must have been inserted to
                adata->module */
             ScmGloc *g = Scm_FindBinding(adata->module, adata->name, 0);
             SCM_ASSERT(g != NULL);
-            adata->value = SCM_GLOC_GET(g);
+            adata->value = Scm_GlocGetValue(g);
             if (SCM_UNBOUNDP(adata->value) || SCM_AUTOLOADP(adata->value)) {
                 Scm_Error("Autoloaded symbol %S is not defined in the file %S",
                           adata->name, adata->path);
@@ -1122,6 +1182,7 @@ ScmObj Scm_CurrentLoadPort()    { return PARAM_REF(Scm_VM(), load_port); }
  * Compatibility stuff
  */
 
+#if GAUCHE_API_VERSION < 98
 /* TRANSIENT: Pre-0.9 Compatibility routine.  Kept for the binary compatibility.
    Will be removed on 1.0 */
 void Scm__LoadFromPortCompat(ScmPort *port, int flags)
@@ -1166,6 +1227,8 @@ ScmObj Scm_LoadMainScript()
 {
     return SCM_UNDEFINED;
 }
+#endif /*GAUCHE_API_VERSION < 98*/
+
 
 /*------------------------------------------------------------------
  * Initialization
@@ -1201,6 +1264,7 @@ void Scm__InitLoad(void)
     key_ignore_coding = SCM_MAKE_KEYWORD("ignore-coding");
     key_paths = SCM_MAKE_KEYWORD("paths");
     key_environment = SCM_MAKE_KEYWORD("environment");
+    key_main_script = SCM_MAKE_KEYWORD("main-script");
 
     Scm_InitStaticClass(SCM_CLASS_DLOBJ, "<dlobj>",
                         m, dlobj_slots, 0);
@@ -1223,6 +1287,9 @@ void Scm__InitLoad(void)
     ldinfo.dso_table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_STRING,0));
     ldinfo.dso_prelinked = SCM_NIL;
 
+    ldinfo.dlptr_class = Scm_MakeForeignPointerClass(m, "<dlptr>",
+                                                     NULL, NULL, 0);
+
 #define PARAM_INIT(var, name, val) ldinfo.var = Scm_BindPrimitiveParameter(m, name, val, 0)
     PARAM_INIT(load_history, "current-load-history", SCM_NIL);
     PARAM_INIT(load_next, "current-load-next", SCM_NIL);
