@@ -1,7 +1,7 @@
 /*
  * macro.c - macro implementation
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -50,7 +50,7 @@
  * Syntax object
  */
 
-static void syntax_print(ScmObj obj, ScmPort *port, 
+static void syntax_print(ScmObj obj, ScmPort *port,
                          ScmWriteContext *mode SCM_UNUSED)
 {
     ScmSymbol *name = SCM_SYNTAX(obj)->name;
@@ -58,7 +58,7 @@ static void syntax_print(ScmObj obj, ScmPort *port,
     if (mod == NULL) {
         Scm_Printf(port, "#<syntax %A>", (name ? SCM_OBJ(name) : SCM_FALSE));
     } else {
-        Scm_Printf(port, "#<syntax %A#%A>", 
+        Scm_Printf(port, "#<syntax %A#%A>",
                    mod->name,
                    (name ? SCM_OBJ(name) : SCM_FALSE));
     }
@@ -81,14 +81,14 @@ ScmObj Scm_MakeSyntax(ScmSymbol *name, ScmModule *mod, ScmObj handler)
  */
 
 ScmObj Scm_MakeMacro(ScmObj name, ScmObj transformer,
-                     ScmObj src, ScmObj describer)
+                     ScmObj info, u_long flags)
 {
     ScmMacro *s = SCM_NEW(ScmMacro);
     SCM_SET_CLASS(s, SCM_CLASS_MACRO);
     s->name = name;
     s->transformer = transformer;
-    s->src = src;
-    s->describer = describer;
+    s->info_alist = info;
+    s->flags = flags;
     return SCM_OBJ(s);
 }
 
@@ -130,7 +130,7 @@ SCM_CLASS_DECL(Scm_SyntaxPatternClass);
 #define SCM_SYNTAX_PATTERN(obj)   ((ScmSyntaxPattern*)(obj))
 #define SCM_SYNTAX_PATTERN_P(obj) SCM_XTYPEP(obj, SCM_CLASS_SYNTAX_PATTERN)
 
-static void pattern_print(ScmObj obj, ScmPort *port, 
+static void pattern_print(ScmObj obj, ScmPort *port,
                           ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(port, "#<pattern:%d%S %S%s>",
@@ -158,7 +158,7 @@ ScmSyntaxPattern *make_syntax_pattern(int level, int numFollowing)
  *   Internal object to construct pattern matcher
  */
 
-static void synrule_print(ScmObj obj, ScmPort *port, 
+static void synrule_print(ScmObj obj, ScmPort *port,
                           ScmWriteContext *mode SCM_UNUSED)
 {
     ScmSyntaxRules *r = SCM_SYNTAX_RULES(obj);
@@ -191,11 +191,13 @@ ScmSyntaxRules *make_syntax_rules(int nr)
 /* TRANSIENT
    This used to be called via make-macro-transformer, but no longer.
    We leave stub here for ABI compatibility. */
+#if GAUCHE_API_VERSION < 98
 ScmObj Scm_MakeMacroTransformerOld(ScmSymbol *name SCM_UNUSED,
                                    ScmProcedure *proc SCM_UNUSED)
 {
     Scm_Panic("Obsoleted Scm_MakeMacroTransformerOld called!  Something is wrong!");
 }
+#endif /*GAUCHE_API_VERSION < 98*/
 
 static ScmMacro *resolve_macro_autoload(ScmAutoload *adata)
 {
@@ -232,7 +234,7 @@ ScmObj Scm_MakeMacroAutoload(ScmSymbol *name, ScmAutoload *adata)
                                      sizeof(AUTOLOAD_MACRO_SUFFIX)-1,
                                      sizeof(AUTOLOAD_MACRO_SUFFIX)-1);
     return Scm_MakeMacro(Scm_MakeSymbol(SCM_STRING(name1), FALSE),
-                         transformer, SCM_FALSE, SCM_FALSE);
+                         transformer, SCM_NIL, 0);
 }
 
 /*===================================================================
@@ -271,7 +273,7 @@ typedef struct {
     ScmObj literals;            /* list of literals */
     ScmObj pvars;               /* list of (pvar . pvref) */
     ScmObj renames;             /* list of (var . identifier)  Keep mapping
-                                   of input symbol/identifier to fresh 
+                                   of input symbol/identifier to fresh
                                    identifier */
     ScmObj ellipsis;            /* symbol/identifier/keyword for ellipsis
                                    SCM_TRUE means default (...)
@@ -388,7 +390,7 @@ static ScmObj rename_variable(ScmObj var,
    In the pattern, we replace variables to identifiers.
    We also recognize pattern variables, and replace them for PVARs in
    the pattern, and for PVREFs in the template.
-   When encounters a repeatable subpattern, replace it with 
+   When encounters a repeatable subpattern, replace it with
    SyntaxPattern node.
 */
 static ScmObj compile_rule1(ScmObj form,
@@ -502,11 +504,11 @@ static ScmObj compile_rule1(ScmObj form,
         if (!SCM_FALSEP(Scm_Memq(form, ctx->literals))) {
             if (patternp)
                 return rename_variable(form, &ctx->renames, ctx->mod, ctx->env);
-            else        
+            else
                 return form;  /* template renaming is done in expansion time */
         }
         if (patternp && Scm__ERCompare(form, SCM_SYM_UNDERBAR,
-                                       ctx->mod, ctx->env)) { 
+                                       ctx->mod, ctx->env)) {
             return SCM_SYM_UNDERBAR;
         }
         if (patternp) {
@@ -697,17 +699,17 @@ static void grow_branch(MatchVar *rec, int level)
         }
         if (SCM_NULLP(SCM_CAR(trunc))) {
             for (i++; i<level-1; i++, trunc = SCM_CAR(trunc)) {
-                SCM_SET_CAR(trunc, SPROUT);
+                SCM_SET_CAR_UNCHECKED(trunc, SPROUT);
             }
             rec->sprout = SPROUT;
-            SCM_SET_CAR(trunc, rec->sprout);
+            SCM_SET_CAR_UNCHECKED(trunc, rec->sprout);
             return;
         }
     }
     SCM_FOR_EACH(trunc, trunc) {
         if (SCM_NULLP(SCM_CDR(trunc))) {
             rec->sprout = SPROUT;
-            SCM_SET_CDR(trunc, rec->sprout);
+            SCM_SET_CDR_UNCHECKED(trunc, rec->sprout);
             break;
         }
     }
@@ -733,8 +735,8 @@ static void exit_subpattern(ScmSyntaxPattern *subpat, MatchVar *mvec)
             if (subpat->level == 1) {
                 mvec[count].root = Scm_ReverseX(mvec[count].branch);
             } else {
-                SCM_SET_CAR(mvec[count].sprout,
-                            Scm_ReverseX(mvec[count].branch));
+                SCM_SET_CAR_UNCHECKED(mvec[count].sprout,
+                                      Scm_ReverseX(mvec[count].branch));
                 mvec[count].branch = SCM_NIL;
             }
         }
@@ -999,7 +1001,20 @@ ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj src, ScmObj ellipsis,
                                        SCM_MODULE(mod), env);
     ScmObj sr_xform = Scm_MakeSubr(synrule_transform, sr,
                                    2, 0, SCM_FALSE);
-    return Scm_MakeMacro(name, sr_xform, src, SCM_FALSE);
+
+    ScmObj info = SCM_NIL, info_tail = SCM_NIL;
+    if (SCM_PAIRP(src)) {
+        SCM_APPEND1(info, info_tail, Scm_Cons(SCM_SYM_SOURCE, src));
+        ScmObj source_info = Scm_PairAttrGet(SCM_PAIR(src),
+                                             SCM_SYM_SOURCE_INFO,
+                                             SCM_FALSE);
+        if (SCM_PAIRP(source_info)) {
+            SCM_APPEND1(info, info_tail,
+                        Scm_Cons(SCM_SYM_SOURCE_INFO, source_info));
+        }
+    }
+
+    return Scm_MakeMacro(name, sr_xform, info, 0);
 }
 
 /*===================================================================
@@ -1010,6 +1025,7 @@ ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj src, ScmObj ellipsis,
    Now it's in compile.scm (%internal-macro-expand).  This is kept
    for ABI compatibility, but nobody is supposed to call this.
  */
+#if GAUCHE_API_VERSION < 98
 ScmObj Scm_VMMacroExpand(ScmObj expr SCM_UNUSED,
                          ScmObj env SCM_UNUSED,
                          int oncep SCM_UNUSED)
@@ -1017,6 +1033,7 @@ ScmObj Scm_VMMacroExpand(ScmObj expr SCM_UNUSED,
     Scm_Error("Scm_VMMacroExpand is obsoleted.");
     return SCM_UNDEFINED;
 }
+#endif /*GAUCHE_API_VERSION < 98*/
 
 ScmObj Scm_CallMacroExpander(ScmMacro *mac, ScmObj expr, ScmObj cenv)
 {
