@@ -1,7 +1,7 @@
 /*
  * main.c - interpreter main program
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -62,6 +62,7 @@ int interactive_mode = FALSE;   /* force interactive mode */
 int test_mode = FALSE;          /* add . and ../lib implicitly  */
 int profiling_mode = FALSE;     /* profile the script? */
 int stats_mode = FALSE;         /* collect stats (EXPERIMENTAL) */
+int version_mode = FALSE;       /* show version and exit (-V option) */
 
 ScmObj pre_cmds = SCM_NIL;      /* assoc list of commands that needs to be
                                    processed before entering repl.
@@ -120,7 +121,7 @@ void usage(int errorp)
             "  -v<version> If <version> is not the running Gauche's version, but\n"
             "           the specified version is installed in the system, execute\n"
             "           that version's gosh instead.  This is useful when you want\n"
-            "           to invoke specific version of Gauche.  <Version> must be\n"
+            "           to invoke a specific version of Gauche.  <Version> must be\n"
             "           0.9.6 or later.\n"
             "  -r<standard>  Starts gosh with the default environment defined\n"
             "           in RnRS, where n is determined by <standard>.  The following\n"
@@ -128,25 +129,34 @@ void usage(int errorp)
             "      7               R7RS (R7RS-small)\n"
             "  -f<flag> Sets various flags\n"
             "      case-fold       uses case-insensitive reader (as in R5RS)\n"
-            "      load-verbose    report while loading files\n"
             "      include-verbose report while including files\n"
-            "      warn-legacy-syntax\n"
-            "                      print warning when legacy Gauche syntax is encountered\n"
+            "      load-verbose    report while loading files\n"
             "      no-inline       don't inline procedures & constants (combined\n"
-            "                      no-inline-globals, no-inline-locals, and\n"
-            "                      no-inline-constants.)\n"
+            "                      no-inline-globals, no-inline-locals,\n"
+            "                      no-inline-constants and no-inline-setters.)\n"
+            "      no-inline-constants\n"
+            "                      don't inline constants.\n"
             "      no-inline-globals\n"
             "                      don't inline global procedures.\n"
             "      no-inline-locals\n"
             "                      don't inline local procedures.\n"
-            "      no-inline-constants\n"
-            "                      don't inline constants.\n"
+            "      no-inline-setters\n"
+            "                      don't inline setter procedures (srfi-17).\n"
             "      no-lambda-lifting-pass\n"
             "                      don't run lambda lifting pass.\n"
             "      no-post-inline-pass\n"
             "                      don't run post-inline optimization pass.\n"
             "      no-source-info  don't preserve source information for debugging\n"
+            "      read-edit\n"
+            "                      enable input-editing mode, if terminal supports it.\n"
+            "      no-read-edit\n"
+            "                      disable input-editing mode.\n"
+            "      safe-string-cursors\n"
+            "                      performs extra validation for use of string cursors\n"
             "      test            test mode, to run gosh inside the build tree\n"
+            "                      without installing.\n"
+            "      warn-legacy-syntax\n"
+            "                      print warning when legacy Gauche syntax is encountered\n"
             "Environment variables:\n"
             "  GAUCHE_AVAILABLE_PROCESSORS\n"
             "      Value must be an integer.  If set, it overrides the number of\n"
@@ -159,6 +169,10 @@ void usage(int errorp)
             "      to search dynamically loadable files.\n"
             "  GAUCHE_EDITOR\n"
             "      Path of the editor invoked with `ed'.\n"
+            "  GAUCHE_HISTORY_FILE\n"
+            "      Specify alternative file to save history of what you typed in REPL\n"
+            "      while you're using input editing.  By default, it is saved in\n"
+            "      `~/.gosh_history'.\n"
             "  GAUCHE_KEYWORD_IS_SYMBOL\n"
             "      If set, keywords become subclass of symbols, to be fully R7RS\n"
             "      conformant.  This is the current default behavior.  Legacy code may\n"
@@ -175,14 +189,19 @@ void usage(int errorp)
             "  GAUCHE_LOAD_PATH\n"
             "      Directories separated by colon (on Unix) or semilcolon (on Windows)\n"
             "      to search scheme files to load.\n"
+            "  GAUCHE_MUTABLE_LITERALS\n"
+            "      Make quoted pairs and lists mutable.  Only to be used to run legacy\n"
+            "      code that accidentally mutates literal pairs (you shouldn't do it)\n"
+            "  GAUCHE_NO_READ_EDIT\n"
+            "      If set, disable input editing feature.  See also -fno-read-edit\n"
+            "      option above.\n"
             "  GAUCHE_QUASIRENAME_MODE\n"
             "      Customize quasirename compatibility mode.  See 'Explicit-renaming\n"
             "      macro transformer' section of the manual for the details.\n"
-            "  GAUCHE_READ_EDIT\n"
-            "      If set, and the terminal is capable, turn on the experimental\n"
-            "      line-editing feature in REPL.\n"
             "  GAUCHE_REPL_NO_PPRINT\n"
             "      If set, suppress using pretty printer in REPL.\n"
+            "  GAUCHE_WARN_SRFI_FEATURE_ID\n"
+            "      If set, warn if srfi-N is used as a feature id in cond-expand.\n"
             "  GAUCHE_SUPPRESS_WARNING\n"
             "      Suppress warnings displayed by ``warn''.  This should be a quick-fix\n"
             "      when you need to get rid of warnings but do not have time to fix the\n"
@@ -213,16 +232,17 @@ void usage(int errorp)
 
 void version(void)
 {
-    printf("Gauche scheme shell, version %s [%s%s], %s\n",
-           GAUCHE_VERSION, SCM_CHAR_ENCODING_NAME, THREAD_OPT,
-           Scm_HostArchitecture());
-    printf("(version \"%s\")\n", GAUCHE_VERSION);
-    printf("(command \"gosh\")\n");
-    printf("(scheme-id gauche)\n");
-    printf("(language scheme r5rs r7rs)\n");
-    printf("(website \"https://practical-scheme.net/gauche\")\n");
-    printf("(platform \"%s\")\n", Scm_HostArchitecture());
-    exit(0);
+    Scm_Printf(SCM_CUROUT, "Gauche scheme shell, version %s [%s%s], %s\n",
+               GAUCHE_VERSION, SCM_CHAR_ENCODING_NAME, THREAD_OPT,
+               Scm_HostArchitecture());
+    static ScmObj version_alist_proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(version_alist_proc, "version-alist", Scm_GaucheModule());
+    ScmObj alist = Scm_ApplyRec0(version_alist_proc), cp;
+
+    SCM_FOR_EACH(cp, alist) {
+        Scm_Printf(SCM_CUROUT, "%S\n", SCM_CAR(cp));
+    }
+    Scm_Exit(0);
 }
 
 void invoke_other_version(const char *version, int argc, char **argv)
@@ -284,10 +304,25 @@ void further_options(const char *optarg)
     }
     else if (strcmp(optarg, "warn-legacy-syntax") == 0) {
         Scm_SetReaderLexicalMode(SCM_INTERN("warn-legacy"));
+        /* standard input port is already initialized, so we should switch it */
+        Scm_SetPortReaderLexicalMode(SCM_CURIN, SCM_INTERN("warn-legacy"));
     }
     else if (strcmp(optarg, "test") == 0) {
         test_mode = TRUE;
     }
+    else if (strcmp(optarg, "safe-string-cursors") == 0) {
+        SCM_VM_RUNTIME_FLAG_SET(vm, SCM_SAFE_STRING_CURSORS);
+    }
+    else if (strcmp(optarg, "read-edit") == 0) {
+        Scm_Define(Scm_GaucheInternalModule(),
+                   SCM_SYMBOL(SCM_INTERN("*read-edit*")),
+                   SCM_TRUE);
+    }
+    else if (strcmp(optarg, "no-read-edit") == 0) {
+        Scm_Define(Scm_GaucheInternalModule(),
+                   SCM_SYMBOL(SCM_INTERN("*read-edit*")),
+                   SCM_FALSE);
+    }
     /* For development; not for public use */
     else if (strcmp(optarg, "collect-stats") == 0) {
         stats_mode = TRUE;
@@ -305,13 +340,15 @@ void further_options(const char *optarg)
     else if (strcmp(optarg, "limit-module-mutation") == 0) {
         SCM_VM_RUNTIME_FLAG_SET(vm, SCM_LIMIT_MODULE_MUTATION);
     }
-    /* Experimental */
-    else if (strcmp(optarg, "c-expr") == 0) {
-        SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_ENABLE_CEXPR);
-    }
     else {
         fprintf(stderr, "unknown -f option: %s\n", optarg);
-        fprintf(stderr, "supported options are: -fcase-fold, -fload-verbose, -finclude-verbose, -fno-dissolve-apply -fno-inline, -fno-inline-globals, -fno-inline-locals, -fno-inline-constants, -fno-inline-setters, -fno-source-info, -fno-post-inline-pass, -fno-lambda-lifting-pass, -fwarn-legacy-syntax, or -ftest\n");
+        fprintf(stderr, "supported options are: -fcase-fold, -fload-verbose, "
+                "-finclude-verbose, -fno-dissolve-apply -fno-inline, "
+                "-fno-inline-globals, -fno-inline-locals, "
+                "-fno-inline-constants, -fno-inline-setters, -fno-source-info, "
+                "-fno-post-inline-pass, -fno-lambda-lifting-pass, "
+                "-fread-edit, -fno-read-edit, "
+                "-fsafe-string-cursors, -fwarn-legacy-syntax, or -ftest\n");
         exit(1);
     }
 }
@@ -349,7 +386,7 @@ int parse_options(int argc, char *argv[])
         case 'b': batch_mode = TRUE; break;
         case 'i': interactive_mode = TRUE; break;
         case 'q': load_initfile = FALSE; break;
-        case 'V': version(); break;
+        case 'V': version_mode = TRUE; break;
         case 'f': further_options(optarg); break;
         case 'p': profiler_options(optarg); break;
         case 'F': feature_options(optarg); break;
@@ -389,27 +426,78 @@ static void sig_setup(void)
     Scm_SetMasterSigmask(&set);
 }
 
+/*
+ * On Linux, LD_LIBRARY_PATH takes precedence over -rpath (which is
+ * already set to $(top_srcdir)/src) and could make GNU ld.so load the
+ * wrong libgauche.so.
+ *
+ * To avoid this, if we detect $LD_LIBRARY_PATH, adjust it and exec()
+ * again.
+ *
+ * This is only called when -ftest option is given.
+ */
+static void test_ld_path_setup(char **av, const char *src_path)
+{
+#if defined(__GNUC__) && defined(__linux__)
+    char *ld_library_path = getenv("LD_LIBRARY_PATH");
+    if (!ld_library_path) return;
+
+    if (getenv("GAUCHE_CHANGED_LD_PATH")) return; /* avoid recursive addition */
+    setenv("GAUCHE_CHANGED_LD_PATH", "1", 1);
+
+    ScmObj new_path = Scm_NormalizePathname(SCM_STRING(SCM_MAKE_STR(src_path)),
+                                            SCM_PATH_ABSOLUTE);
+    new_path = Scm_StringAppendC(SCM_STRING(new_path), ":", 1, 1);
+    new_path = Scm_StringAppendC(SCM_STRING(new_path), ld_library_path, -1, -1);
+    setenv("LD_LIBRARY_PATH", strdup(Scm_GetString(SCM_STRING(new_path))), 1);
+
+    execv("/proc/self/exe", av);
+#else
+    (void)av;
+    (void)src_path;
+#endif
+}
+
 /* Path setup.  When we're ran with -ftest flag, add paths to refer
    to the files in the source tree.  This is called after option processing
    and before loading init file. */
-void test_paths_setup(void)
+static void test_paths_setup(char **av)
 {
-    /* The order of directories is important.  'lib' should
+    /* We first try from the location of executable; if it's not
+       available, we assume we're in the source tree, either in
+       src/ or ext/$EXT/.
+
+       The order of directories is important.  'lib' should
        be searched first (hence it should come latter), since some
        extension modules are built from the file in src then linked
        from lib, and we want to test the one in lib. */
-    if (access("../src/gauche/config.h", R_OK) == 0
-        && access("../libsrc/srfi-1.scm", R_OK) == 0
+    ScmObj self = Scm_ExecutablePath();
+    if (SCM_STRINGP(self)) {
+        ScmString *base = SCM_STRING(Scm_DirName(SCM_STRING(self)));
+        ScmObj srcdir = Scm_StringAppendC(base, "/../src", -1, -1);
+        ScmObj libsrcdir = Scm_StringAppendC(base, "/../libsrc", -1, -1);
+        ScmObj libdir = Scm_StringAppendC(base, "/../lib", -1, -1);
+        test_ld_path_setup(av, Scm_GetStringConst(SCM_STRING(srcdir)));
+        Scm_AddLoadPath(Scm_GetStringConst(SCM_STRING(srcdir)), FALSE);
+        Scm_AddLoadPath(Scm_GetStringConst(SCM_STRING(libsrcdir)), FALSE);
+        Scm_AddLoadPath(Scm_GetStringConst(SCM_STRING(libdir)), FALSE);
+    } else if (access("../src/gauche/config.h", R_OK) == 0
+        && access("../libsrc/scheme/list.scm", R_OK) == 0
         && access("../lib/r7rs-setup.scm", R_OK) == 0) {
+        test_ld_path_setup(av, "../src");
         Scm_AddLoadPath("../src", FALSE);
         Scm_AddLoadPath("../libsrc", FALSE);
         Scm_AddLoadPath("../lib", FALSE);
     } else if (access("../../src/gauche/config.h", R_OK) == 0
-               && access("../../libsrc/srfi-1.scm", R_OK) == 0
+               && access("../../libsrc/scheme/list.scm", R_OK) == 0
                && access("../../lib/r7rs-setup.scm", R_OK) == 0) {
+        test_ld_path_setup(av, "../../src");
         Scm_AddLoadPath("../../src", FALSE);
         Scm_AddLoadPath("../../libsrc", FALSE);
         Scm_AddLoadPath("../../lib", FALSE);
+    } else {
+        Scm_Warn("Unable to find source tree.  The -ftest option is ignored.");
+        return;
     }
     /* Also set a feature identifier gauche.in-place, so that other modules
        may initialize differently if needed. */
@@ -542,10 +630,16 @@ static void process_command_args(ScmObj cmd_args)
                              SCM_FALSE, 0);
             break;
         case 'e':
-            if (Scm_EvalCString(Scm_GetStringConst(SCM_STRING(v)),
-                                SCM_OBJ(Scm_UserModule()),
-                                &epak) < 0) {
-                error_exit(epak.exception);
+            /* -e option may have multiple sexprs */
+            {
+                ScmObj p = Scm_MakeInputStringPort(SCM_STRING(v), TRUE);
+                epak.module = Scm_UserModule();
+                ScmObj form;
+                while ((form = Scm_Read(p)) != SCM_EOF) {
+                    if (Scm_Eval(form, SCM_OBJ(epak.module), &epak) < 0) {
+                        error_exit(epak.exception);
+                    }
+                }
             }
             break;
         case 'E':
@@ -586,10 +680,8 @@ static void process_command_args(ScmObj cmd_args)
 int execute_script(const char *scriptfile, ScmObj args)
 {
     /* If script file is specified, load it. */
-    ScmObj mainproc = SCM_FALSE;
     ScmLoadPacket lpak;
-
-    Scm_Load(scriptfile, SCM_LOAD_PROPAGATE_ERROR, &lpak);
+    Scm_Load(scriptfile, SCM_LOAD_PROPAGATE_ERROR|SCM_LOAD_MAIN_SCRIPT, &lpak);
     if (!lpak.loaded) return 1;
 
     /* If symbol 'main is bound, call it (SRFI-22).   */
@@ -597,19 +689,19 @@ int execute_script(const char *scriptfile, ScmObj args)
                           ? Scm_FindModule(SCM_SYMBOL(main_module), 0)
                           : Scm_UserModule());
     if (mainmod) {
-        mainproc = Scm_GlobalVariableRef(mainmod,
-                                         SCM_SYMBOL(SCM_INTERN("main")),
-                                         SCM_BINDING_STAY_IN_MODULE);
-    }
-    if (SCM_PROCEDUREP(mainproc)) {
-        static ScmObj run_main_proc = SCM_UNDEFINED;
-        SCM_BIND_PROC(run_main_proc, "run-main", Scm_GaucheInternalModule());
-        SCM_ASSERT(SCM_PROCEDUREP(run_main_proc));
-        
-        ScmEvalPacket epak;
-        int r = Scm_Apply(run_main_proc, SCM_LIST2(mainproc, args), &epak);
-        SCM_ASSERT(r == 1 && SCM_INTP(epak.results[0]));
-        return SCM_INT_VALUE(epak.results[0]);
+        ScmObj mainproc = Scm_GlobalVariableRef(mainmod,
+                                                SCM_SYMBOL(SCM_INTERN("main")),
+                                                SCM_BINDING_STAY_IN_MODULE);
+        if (SCM_PROCEDUREP(mainproc)) {
+            static ScmObj run_main_proc = SCM_UNDEFINED;
+            SCM_BIND_PROC(run_main_proc, "run-main", Scm_GaucheInternalModule());
+            SCM_ASSERT(SCM_PROCEDUREP(run_main_proc));
+
+            ScmEvalPacket epak;
+            int r = Scm_Apply(run_main_proc, SCM_LIST2(mainproc, args), &epak);
+            SCM_ASSERT(r == 1 && SCM_INTP(epak.results[0]));
+            return SCM_INT_VALUE(epak.results[0]);
+        }
     }
     return 0;
 }
@@ -624,8 +716,8 @@ static int has_terminal()
        %sys-mintty? is defined in libsys.scm. */
     ScmEvalPacket pac;
     int r = Scm_EvalCString("(%sys-mintty? 0)",
-			    SCM_OBJ(Scm_GaucheInternalModule()),
-			    &pac);
+                            SCM_OBJ(Scm_GaucheInternalModule()),
+                            &pac);
     if (r == 1 && !SCM_FALSEP(pac.results[0])) return TRUE;
 #endif /*GAUCHE_WINDOWS*/
     return FALSE;
@@ -651,7 +743,7 @@ void enter_repl()
     if (SCM_VM_RUNTIME_FLAG_IS_SET(Scm_VM(), SCM_CASE_FOLD)) {
         Scm_SetPortCaseFolding(SCM_PORT(Scm_Stdin()), TRUE);
     }
-    
+
     if (batch_mode || (!has_terminal() && !interactive_mode)) {
         Scm_LoadFromPort(SCM_PORT(Scm_Stdin()), SCM_LOAD_PROPAGATE_ERROR, NULL);
     } else {
@@ -695,7 +787,7 @@ int main(int ac, char **av)
             Scm_SetCallTraceSize(strtoul(call_trace, NULL, 10));
         }
     }
-    
+
     GC_INIT();
     Scm_Init(GAUCHE_SIGNATURE);
     sig_setup();
@@ -719,7 +811,7 @@ int main(int ac, char **av)
     if (strcmp(safe_basename(argv[0]), "scheme-r7rs") == 0) {
         main_module = SCM_INTERN("r7rs.user");
     }
-    
+
     /* Check command-line options */
     int argind = parse_options(argc, argv);
 
@@ -727,7 +819,11 @@ int main(int ac, char **av)
        tree, adds build directories to the library path _before_
        loading init file.   This is to help development of Gauche
        itself; normal user should never need this. */
-    if (test_mode) test_paths_setup();
+    if (test_mode) test_paths_setup(av);
+
+    /* If -V is given, show version info and exit
+       (must come after test_mode setup)  */
+    if (version_mode) version();
 
     /* prepare *program-name* and *argv* */
     if (argind < argc) {
@@ -758,9 +854,12 @@ int main(int ac, char **av)
         }
 
         /* sets up arguments. */
-        args = Scm_InitCommandLine(argc - argind, (const char**)argv + argind);
+        Scm_InitCommandLine2(argc, (const char**)argv, SCM_COMMAND_LINE_OS);
+        args = Scm_InitCommandLine2(argc - argind, (const char**)argv + argind,
+                                    SCM_COMMAND_LINE_SCRIPT);
     } else {
-        args = Scm_InitCommandLine(1, (const char**)argv);
+        /* For REPL, we leave command-line to the default '(""). */
+        Scm_InitCommandLine2(argc, (const char**)argv, SCM_COMMAND_LINE_OS);
     }
 
     process_command_args(Scm_Reverse(pre_cmds));
