@@ -1,7 +1,7 @@
 /*
  * module.c - module implementation
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,8 +33,8 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/class.h"
 #include "gauche/priv/builtin-syms.h"
+#include "gauche/priv/glocP.h"
 #include "gauche/priv/moduleP.h"
 
 /*
@@ -91,7 +91,7 @@
  *   each bound to itself, *and* exports all of such bindings.
  *
  * It turned out we can't use one 'keyword' module for both purpose; if
- * we have a keyword module that exports all bindings, they are automatically   
+ * we have a keyword module that exports all bindings, they are automatically
  * exported from modules that inherits them.  This means if a R7RS program
  * imports any of Gauche modules, it carries all of keyword bindings.  It's
  * not only nasty, but also dangerous for it can shadow bindings to the
@@ -119,6 +119,13 @@
    a module is simply a bad idea and shouldn't be allowed.
  */
 
+/* Note on gauche.bootstrap module
+   This module is only exists during boot process.  Once booting is
+   complete, it is destroyed.  We use the module to define some
+   under-the-hood procedures that we don't want ordinary programs
+   to call directly.
+ */
+
 /* Global module table */
 static struct {
     ScmHashTable *table;    /* Maps name -> module. */
@@ -142,6 +149,7 @@ DEFINE_STATIC_MODULE(reqbaseModule);  /* #<module gauche.require-base> */
 
 static ScmObj defaultParents = SCM_NIL; /* will be initialized */
 static ScmObj defaultMpl =     SCM_NIL; /* will be initialized */
+static ScmObj bootstrapModule = SCM_FALSE; /* will be initialized */
 
 /*----------------------------------------------------------------------
  * Constructor
@@ -162,12 +170,7 @@ static void init_module(ScmModule *m, ScmObj name, ScmHashTable *internal)
     m->external = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
     m->origin = m->prefix = SCM_FALSE;
     m->sealed = FALSE;
-
-    if (name == SCM_INTERN("A") || name == SCM_INTERN("B")
-        || name == SCM_INTERN("C")) {
-        Scm_Printf(SCM_CURERR, "making %S\n", name);
-    }
-    
+    m->placeholding = FALSE;
 }
 
 /* Internal */
@@ -247,7 +250,7 @@ static void err_sealed(ScmObj source, ScmModule *target)
     const char *what = "";
     if (SCM_MODULEP(source)) what = "import a module";
     else                     what = "create a binding";
-    
+
     if (target == Scm__RequireBaseModule()) {
         Scm_Error("Attempted to %s (%S) into gauche.require-base. "
                   "This may be caused by trying to 'use' or 'require' a file"
@@ -435,24 +438,16 @@ ScmGloc *Scm_MakeBinding(ScmModule *module, ScmSymbol *symbol,
                          ScmObj value, int flags)
 {
     if (module->sealed) err_sealed(SCM_OBJ(symbol), module);
-    
+
     ScmGloc *g;
-    ScmObj oldval = SCM_UNDEFINED;
-    int prev_kind = 0;
-    int kind = ((flags&SCM_BINDING_CONST)
-                ? SCM_BINDING_CONST
-                : ((flags&SCM_BINDING_INLINABLE)
-                   ? SCM_BINDING_INLINABLE
-                   : 0));
+    int existing = FALSE;
 
     SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(modules.mutex);
     ScmObj v = Scm_HashTableRef(module->internal, SCM_OBJ(symbol), SCM_FALSE);
     /* NB: this function bypasses check of gloc setter */
     if (SCM_GLOCP(v)) {
         g = SCM_GLOC(v);
-        if (Scm_GlocConstP(g))          prev_kind = SCM_BINDING_CONST;
-        else if (Scm_GlocInlinableP(g)) prev_kind = SCM_BINDING_INLINABLE;
-        oldval = g->value;
+        existing = TRUE;
     } else {
         g = SCM_GLOC(Scm_MakeGloc(symbol, module));
         Scm_HashTableSet(module->internal, SCM_OBJ(symbol), SCM_OBJ(g), 0);
@@ -463,18 +458,14 @@ ScmGloc *Scm_MakeBinding(ScmModule *module, ScmSymbol *symbol,
     }
     SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
 
-    g->value = value;
-    Scm_GlocMark(g, kind);
-
-    if (prev_kind != 0) {
-        /* NB: Scm_EqualP may throw an error.  It won't leave the state
-           inconsistent, but be aware. */
-        if (prev_kind != kind || !Scm_EqualP(value, oldval)) {
-            Scm_Warn("redefining %s %S::%S",
-                     (prev_kind == SCM_BINDING_CONST)? "constant" : "inlinable",
-                     g->module->name, g->name);
-        }
+    if (existing && !Scm_GlocSupersedableP(g, flags, value)) {
+        Scm_Warn("redefining %s %S#%S",
+                 Scm_GlocConstP(g)? "constant" : "inlinable",
+                 g->module->name, g->name);
     }
+
+    g->value = value;
+    Scm_GlocMark(g, flags);
     return g;
 }
 
@@ -510,7 +501,7 @@ void Scm_HideBinding(ScmModule *module, ScmSymbol *symbol)
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     ScmObj v = Scm_HashTableRef(module->external, SCM_OBJ(symbol), SCM_FALSE);
     if (!SCM_FALSEP(v)) {
-        err_exists = TRUE;
+        if (!SCM_GLOC(v)->hidden) err_exists = TRUE;
     } else {
         ScmGloc *g = SCM_GLOC(Scm_MakeGloc(symbol, module));
         g->hidden = TRUE;
@@ -594,7 +585,7 @@ ScmObj Scm_ImportModule(ScmModule *module,
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     {
         ScmObj ms, prev = p;
-        SCM_SET_CDR(p, module->imported);
+        SCM_SET_CDR_UNCHECKED(p, module->imported);
         /* Remove duplicate module, if any.
            NB: We allow to import the same module multiple times if they are
            qualified by :only, :prefix, etc.  Theoretically we should check
@@ -607,7 +598,7 @@ ScmObj Scm_ImportModule(ScmModule *module,
                 prev = ms;
                 continue;
             }
-            SCM_SET_CDR(prev, SCM_CDR(ms));
+            SCM_SET_CDR_UNCHECKED(prev, SCM_CDR(ms));
             break;
         }
         module->imported = p;
@@ -739,21 +730,54 @@ ScmObj Scm_ExportAll(ScmModule *module)
     return SCM_OBJ(module);
 }
 
+/* internal routine form Scm_ModuleExports */
+static void add_parent_module_exports(ScmHashCore *syms,
+                                      ScmModule *module,
+                                      module_cache *visited)
+{
+    ScmObj mp;
+    SCM_FOR_EACH(mp, module->mpl) {
+        ScmModule *m = SCM_MODULE(SCM_CAR(mp));
+        if (module_visited_p(visited, m)) continue;
+        module_add_visited(visited, m);
+
+        ScmHashIter iter;
+        Scm_HashIterInit(&iter, SCM_HASH_TABLE_CORE(m->external));
+        ScmDictEntry *e;
+        while ((e = Scm_HashIterNext(&iter)) != NULL) {
+            if (!(SCM_GLOC(SCM_DICT_VALUE(e)))->hidden) {
+                (void)Scm_HashCoreSearch(syms, e->key, SCM_DICT_CREATE);
+            }
+        }
+        add_parent_module_exports(syms, m, visited);
+    }
+}
+
 /* Returns list of exported symbols.   We assume this is infrequent
    operation, so we build the list every call.  If it becomes a problem,
    we can cache the result. */
 ScmObj Scm_ModuleExports(ScmModule *module)
 {
-    ScmObj h = SCM_NIL, t = SCM_NIL;
+    ScmHashCore syms;
+    Scm_HashCoreInitSimple(&syms, SCM_HASH_EQ, 0, NULL);
+    module_cache visited;
+    init_module_cache(&visited);
 
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
+    Scm_HashCoreCopy(&syms, SCM_HASH_TABLE_CORE(module->external));
+    add_parent_module_exports(&syms, module, &visited);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
+
+    ScmObj h = SCM_NIL, t = SCM_NIL;
     ScmHashIter iter;
-    Scm_HashIterInit(&iter, SCM_HASH_TABLE_CORE(module->external));
+    Scm_HashIterInit(&iter, &syms);
     ScmDictEntry *e;
     while ((e = Scm_HashIterNext(&iter)) != NULL) {
-        SCM_APPEND1(h, t, SCM_DICT_KEY(e));
+        intptr_t v = e->value;
+        if (!v || (SCM_GLOCP(v) && !(SCM_GLOC(v)->hidden))) {
+            SCM_APPEND1(h, t, SCM_OBJ(e->key));
+        }
     }
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     return h;
 }
 
@@ -771,7 +795,7 @@ ScmObj Scm_ExtendModule(ScmModule *module, ScmObj supers)
     if (module->sealed) {
         Scm_Error("Attempt to extend a sealed module: %S", SCM_OBJ(module));
     }
-    
+
     ScmObj seqh = SCM_NIL, seqt = SCM_NIL;
     ScmObj sp;
     SCM_FOR_EACH(sp, supers) {
@@ -803,6 +827,30 @@ void Scm_ModuleSeal(ScmModule *module)
     module->sealed = TRUE;
 }
 
+/*----------------------------------------------------------------------
+ * Module destroying
+ */
+
+/* Destroying modules isn't a usual operation; it can break existing
+   code if used carelessly.  We treat it as an internal API.
+*/
+void Scm__DestroyModule(ScmModule *m)
+{
+    if (!SCM_FALSEP(m->name)) {
+        (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
+        Scm_HashTableDelete(modules.table, m->name);
+        (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
+    }
+    m->imported = SCM_NIL;
+    m->parents = SCM_NIL;
+    m->mpl = SCM_NIL;
+    m->depended = SCM_NIL;
+    Scm_HashCoreClear(&m->internal->core);
+    Scm_HashCoreClear(&m->external->core);
+    m->origin = SCM_FALSE;
+    m->prefix = SCM_FALSE;
+}
+
 /*----------------------------------------------------------------------
  * Finding modules
  */
@@ -813,6 +861,15 @@ ScmModule *Scm_FindModule(ScmSymbol *name, int flags)
         int created;
         ScmModule *m = lookup_module_create(name, &created);
         SCM_ASSERT(m != NULL);
+        /* If the module is ever called with CREATE and PLACEHOLDING flag,
+           turn placeholding flag on.  The flag is cleared if the module
+           is ever called with CREATE but without PLACEHOLDING. */
+        if (created && (flags & SCM_FIND_MODULE_PLACEHOLDING)) {
+            m->placeholding = TRUE;
+        }
+        if (!(flags & SCM_FIND_MODULE_PLACEHOLDING)) {
+            m->placeholding = FALSE;
+        }
         return m;
     } else {
         ScmModule *m = lookup_module(name);
@@ -836,7 +893,10 @@ ScmObj Scm_AllModules(void)
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     Scm_HashIterInit(&iter, SCM_HASH_TABLE_CORE(modules.table));
     while ((e = Scm_HashIterNext(&iter)) != NULL) {
-        SCM_APPEND1(h, t, SCM_DICT_VALUE(e));
+        ScmModule *m = SCM_MODULE(SCM_DICT_VALUE(e));
+        if (!m->placeholding) {
+            SCM_APPEND1(h, t, SCM_DICT_VALUE(e));
+        }
     }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     return h;
@@ -970,8 +1030,19 @@ void Scm__InitModule(void)
              keywordModule.internal);
     gkeywordModule.exportAll = TRUE;
 
+    bootstrapModule = Scm_MakeModule(SCM_SYMBOL(SCM_INTERN("gauche.bootstrap")),
+                                     FALSE);
+
     /* create predefined moudles */
     for (modname = builtin_modules; *modname; modname++) {
         (void)SCM_FIND_MODULE(*modname, SCM_FIND_MODULE_CREATE);
     }
 }
+
+void Scm__FinishModuleInitialization(void)
+{
+#if !GAUCHE_DEBUG_BOOTSTRAP
+    Scm__DestroyModule(SCM_MODULE(bootstrapModule));
+    bootstrapModule = SCM_FALSE;
+#endif
+}
