@@ -13,22 +13,22 @@
 (use gauche.sequence)
 (use gauche.dictionary)
 (use gauche.hook)
-(use gauche.parameter)
 (use gauche.uvector)
 (use gauche.regexp)
 (use gauche.record)
 (use gauche.generator)
 (use gauche.interpolate)
+(use gauche.process)
+(cond-expand
+ [gauche-0.9.9 (use gauche.stringutil) (use gauche.portutil)]
+ [else])
 (use gauche.threads)
-(use gauche.hashutil) ; TRANSIENT: only needed to compile 0.9.9 with 0.9.8
-(use srfi-1)
+(cond-expand
+ [gauche-0.9.10 (use srfi-1)]
+ [else (use scheme.list)])
 (use srfi-13)
 (use file.util)
 (use text.tr)
 (use util.match)
 (use rfc.822)
 (use rfc.mime)
-
-
-
-
