@@ -1,7 +1,7 @@
 /*
  * read.c - reader
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -69,6 +69,7 @@ static ScmObj read_keyword(ScmPort *port, ScmReadContext *ctx);
 static ScmObj read_regexp(ScmPort *port);
 static ScmObj read_charset(ScmPort *port);
 static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx);
+static ScmObj read_sharp_asterisk(ScmPort *port, ScmReadContext *ctx);
 static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
                                   ScmReadContext *ctx, int has_ref);
 static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx);
@@ -235,7 +236,7 @@ void Scm_ReadError(ScmPort *port, const char *msg, ...)
 {
     ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
     ScmObj name = Scm_PortName(port);
-    int line = Scm_PortLine(port);
+    ScmSize line = Scm_PortLine(port);
 
     Scm_Printf(SCM_PORT(ostr), "Read error at %S:",
                SCM_STRINGP(name)? name : SCM_OBJ(SCM_MAKE_STR("??")));
@@ -318,11 +319,11 @@ static void read_context_flush(ScmReadContext *ctx)
         } else if (SCM_PAIRP(obj)) {
             SCM_FOR_EACH(ep, obj) {
                 if (SCM_READ_REFERENCE_P(SCM_CAR(ep))) {
-                    SCM_SET_CAR(ep, ref_val(SCM_CAR(ep)));
+                    SCM_SET_CAR_UNCHECKED(ep, ref_val(SCM_CAR(ep)));
                 }
                 if (SCM_READ_REFERENCE_P(SCM_CDR(ep))) {
                     /* in case we have (... . #N#) */
-                    SCM_SET_CDR(ep, ref_val(SCM_CDR(ep)));
+                    SCM_SET_CDR_UNCHECKED(ep, ref_val(SCM_CDR(ep)));
                     break;
                 }
             }
@@ -411,11 +412,11 @@ static int char_is_delimiter(ScmChar ch)
             || SCM_CHAR_EXTRA_WHITESPACE(ch));
 }
 
-static void read_nested_comment(ScmPort *port, 
+static void read_nested_comment(ScmPort *port,
                                 ScmReadContext *ctx SCM_UNUSED)
 {
     int nesting = 0;
-    int line = Scm_PortLine(port);
+    ScmSize line = Scm_PortLine(port);
 
     for (ScmChar c = Scm_GetcUnsafe(port);;) {
         switch (c) {
@@ -454,12 +455,7 @@ static void read_comment(ScmPort *port) /* leading semicolon is already read */
         /* NB: comment may contain unexpected character code.
            for the safety, we read bytes here. */
         int c = Scm_GetbUnsafe(port);
-        if (c == '\n') {
-            /* oops.  ugly. */
-            port->line++;
-            break;
-        }
-        if (c == EOF) break;
+        if (c == '\n' || c == EOF) break;
     }
 }
 
@@ -573,6 +569,19 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                     ScmObj form = read_item(port, ctx);
                     return SCM_LIST2(SCM_SYM_DEBUG_FUNCALL, form);
                 }
+                case '@': {
+                    int c3 = Scm_GetcUnsafe(port);
+                    switch (c3) {
+                    case '=': {
+                        /* #?@=form - debug thread log */
+                        ScmObj form = read_item(port, ctx);
+                        return SCM_LIST2(SCM_SYM_DEBUG_THREAD_LOG, form);
+                    }
+                    default:
+                        Scm_ReadError(port, "unsupported #?-syntax: #?@%C", c3);
+                        return SCM_UNDEFINED; /* dummy */
+                    }
+                }
                 case EOF:
                     return SCM_EOF;
                 default:
@@ -586,12 +595,9 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                 return read_num_prefixed(port, c1, ctx);
             case '*': {
                 reject_in_r7(port, ctx, "#*");
-                /* #*"...." byte string
-                   #*01001001 for bit vector, maybe in future. */
-                int c2 = Scm_GetcUnsafe(port);
-                if (c2 == '"') return read_string(port, TRUE, ctx);
-                Scm_ReadError(port, "unsupported #*-syntax: #*%C", c2);
-                return SCM_UNDEFINED; /* dummy */
+                /* #**"...." byte string
+                   #*01001001 for bit vector. */
+                return read_sharp_asterisk(port, ctx);
             }
             case ':': {
                 reject_in_r7(port, ctx, "#:");
@@ -642,18 +648,7 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
         return read_list(port, ']', ctx);
     case '{':
         reject_in_r7(port, ctx, "{}");
-        /* srfi-105 experimental support */
-        {
-            ScmObj r = read_list(port, '}', ctx);
-            if (SCM_VM_COMPILER_FLAG_IS_SET(Scm_VM(),SCM_COMPILE_ENABLE_CEXPR)){
-                static ScmObj xform_cexpr = SCM_UNDEFINED;
-                SCM_BIND_PROC(xform_cexpr, "%xform-cexpr",
-                              Scm_GaucheInternalModule());
-                return Scm_ApplyRec1(xform_cexpr, r);
-            } else {
-                return r;
-            }
-        }
+        return read_list(port, '}', ctx);
     case '+':; case '-':
         /* Note: R5RS doesn't permit identifiers beginning with '+' or '-',
            but some Scheme programs use such identifiers. */
@@ -934,7 +929,7 @@ static ScmObj read_list_int(ScmPort *port, ScmChar closer,
                 Scm_UngetcUnsafe(c2, port);
                 item = read_item(port, ctx);
                 if (SCM_READ_REFERENCE_P(item)) ref_seen = TRUE;
-                SCM_SET_CDR(last, item);
+                SCM_SET_CDR_UNCHECKED(last, item);
                 dot_seen = TRUE;
                 continue;
             }
@@ -964,7 +959,7 @@ static ScmObj read_list_int(ScmPort *port, ScmChar closer,
 static ScmObj read_list(ScmPort *port, ScmChar closer, ScmReadContext *ctx)
 {
     int has_ref;
-    int line = -1;
+    ScmSize line = -1;
 
     if (ctx->flags & RCTX_SOURCE_INFO) line = Scm_PortLine(port);
 
@@ -985,7 +980,7 @@ static ScmObj read_list(ScmPort *port, ScmChar closer, ScmReadContext *ctx)
 static ScmObj read_vector(ScmPort *port, ScmChar closer, ScmReadContext *ctx)
 {
     int has_ref;
-    int line = -1;
+    ScmSize line = -1;
     ScmObj r;
 
     if (ctx->flags & RCTX_SOURCE_INFO) line = Scm_PortLine(port);
@@ -997,7 +992,7 @@ static ScmObj read_vector(ScmPort *port, ScmChar closer, ScmReadContext *ctx)
 
 static ScmObj read_quoted(ScmPort *port, ScmObj quoter, ScmReadContext *ctx)
 {
-    int line = -1;
+    ScmSize line = -1;
     ScmObj r;
 
     if (ctx->flags & RCTX_SOURCE_INFO) line = Scm_PortLine(port);
@@ -1274,7 +1269,7 @@ static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
    the read context setting.  INCLUDE_HASH_SIGN allows '#' to appear in
    the word.
 */
-static ScmObj read_word(ScmPort *port, ScmChar initial, 
+static ScmObj read_word(ScmPort *port, ScmChar initial,
                         ScmReadContext *ctx SCM_UNUSED,
                         int temp_case_fold, int include_hash_sign)
 {
@@ -1605,7 +1600,8 @@ static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx)
                       next);
     }
 
-    int has_ref, line = -1;
+    int has_ref;
+    ScmSize line = -1;
     if (ctx->flags & RCTX_SOURCE_INFO) line = Scm_PortLine(port);
 
     ScmObj form = read_list_int(port, ')', ctx, &has_ref, line);
@@ -1653,11 +1649,11 @@ ScmObj Scm_DefineReaderDirective(ScmObj symbol, ScmObj proc)
 
 static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx)
 {
-    /* If '#!' appears in the beginning of the input port, and it is
-       followed by '/' or a space, then we consider it as the
-       beginning of shebang and discards entire line.  Otherwise, we
-       take this as #!<identifier> directive as specified in R6RS, and
-       calls appropriate handler.
+    /* If '#!' appears at the beginning of the input port, and it is
+       followed by '/' or a space or a newline, then we consider it as
+       the beginning of shebang and discard the entire line.
+       Otherwise, we take this as #!<identifier> directive as
+       specified in R6RS, and calls appropriate handler.
 
        R6RS is actually not very clean at this corner.  It requires
        distinct modes to parse a script (which always begins with
@@ -1666,12 +1662,12 @@ static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx)
        parser that strictly covers both situation.
     */
     int c2 = Scm_GetcUnsafe(port);
-    if (port->bytes == 3 && (c2 == '/' || c2 == ' ')) {
+    if (Scm_PortBytes(port) == 3 && (c2 == '/' || c2 == ' ' || c2 == '\n')) {
         /* shebang */
         for (;;) {
-            c2 = Scm_GetcUnsafe(port);
             if (c2 == '\n') return SCM_UNDEFINED;
             if (c2 == EOF) return SCM_EOF;
+            c2 = Scm_GetcUnsafe(port);
         }
         /*NOTREACHED*/
     } else {
@@ -1760,7 +1756,7 @@ static ScmObj read_sharp_word_1(ScmPort *port, char ch, ScmReadContext *ctx)
             || strcmp(w, "c64") == 0
             || strcmp(w, "c128") == 0) {
             tag = w;
-        } 
+        }
         break;
     case 'f':
         if (strcmp(w, "f16") == 0
@@ -1807,6 +1803,83 @@ static ScmObj read_sharp_word(ScmPort *port, char ch, ScmReadContext *ctx)
     }
 }
 
+/*----------------------------------------------------------------
+ * #**"....", #*1010010...
+ */
+static ScmObj read_sharp_asterisk(ScmPort *port, ScmReadContext *ctx)
+{
+    /* We used to use #*"..." for incomplete string literals, but it turned
+       out it can be read as an zero-length bitvector #* followed by an
+       ordinary string (since a double quote is a <delimiter> in R7RS).
+
+       So we changed the literal to #**"..." (yeah, ugly, but incomplete
+       strings are anomalies and shouldn't be used often).
+
+       The reader behavior can be customized by reader-lexical-mode:
+
+         permissive (default) - For the next release, #*"..." is read as
+             an incomplete string, without warning.  In future, it warns.
+         legacy - #*"..." is read as an incomplete string, without warning.
+         warn-legacy - #*"..." is read as an incomplete string, but warns.
+         strict-r7 - #*"..." is read as a zero-length bitvector followed by
+                     a string.  #**"..." is rejected.
+    */
+
+    int c = Scm_GetcUnsafe(port);
+    if (c == '*') {
+        c = Scm_GetcUnsafe(port);
+        if (c == '"') return read_string(port, TRUE, ctx);
+        Scm_ReadError(port, "Invalid #* syntax: #**%C", c);
+    }
+    if (c == '"') {
+        ScmObj m = Scm_GetPortReaderLexicalMode(port);
+        if (SCM_EQ(m, SCM_SYM_PERMISSIVE)) {
+            /* TRANSIENT: We switch this default behavior to warn in future */
+            return read_string(port, TRUE, ctx);
+        }
+        if (SCM_EQ(m, SCM_SYM_LEGACY)) {
+            return read_string(port, TRUE, ctx);
+        }
+        if (SCM_EQ(m, SCM_SYM_WARN_LEGACY)) {
+            Scm_Warn("Deprecated incomplete string syntax #*\"...\" at %A:%ld",
+                     Scm_PortName(port), Scm_PortLine(port));
+            return read_string(port, TRUE, ctx);
+        }
+        if (SCM_EQ(m, SCM_SYM_STRICT_R7)) {
+            Scm_UngetcUnsafe(c, port);
+            return Scm_MakeBitvector(0, SCM_FALSE);
+        }
+        Scm_Panic("invalid reader-lexical-mode");
+    }
+
+    ScmDString ds;
+    Scm_DStringInit(&ds);
+    for (;;) {
+        switch (c) {
+        case EOF:
+            goto finish;
+        case '(': case ')': case '"': case '|': case ';':
+            Scm_UngetcUnsafe(c, port);
+            goto finish;
+        case '0': case '1':
+            Scm_DStringPutc(&ds, c);
+            break;
+        default:
+            if ((SCM_CHAR_ASCII_P(c) && isspace(c))
+                || (!SCM_CHAR_ASCII_P(c) && SCM_CHAR_EXTRA_WHITESPACE(c))) {
+                Scm_UngetcUnsafe(c, port);
+                goto finish;
+            }
+            Scm_ReadError(port, "Invalid char in bitvector literal: %C", c);
+        }
+        c = Scm_GetcUnsafe(port);
+    }
+
+ finish:;
+    ScmObj s = Scm_DStringGet(&ds, 0);
+    return Scm_StringToBitvector(SCM_STRING(s), FALSE);
+}
+
 /* OBSOLETED: gauche.uvector used to call this to set up reader pointer.
    Now it is read in src/vector.c.   We keep this entry for ABI compatibility.
    Remove on 1.0 release. */
@@ -1833,7 +1906,7 @@ void Scm__InitRead(void)
         SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
     (void)SCM_INTERNAL_MUTEX_INIT(hashBangData.mutex);
 
-    defaultReadContext = 
+    defaultReadContext =
         Scm_MakePrimitiveParameter(SCM_CLASS_PRIMITIVE_PARAMETER, SCM_FALSE,
                                    SCM_OBJ(make_read_context(NULL)),
                                    0);
