@@ -1,7 +1,7 @@
 /*
  * signal.c - signal handling
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -34,7 +34,6 @@
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/vm.h"
-#include "gauche/class.h"
 
 /* Signals
  *
@@ -307,7 +306,7 @@ static ScmObj sigset_allocate(ScmClass *klass, ScmObj initargs);
 SCM_DEFINE_BUILTIN_CLASS(Scm_SysSigsetClass, sigset_print,
                          NULL, NULL, sigset_allocate, SCM_CLASS_DEFAULT_CPL);
 
-void sigset_print(ScmObj obj, ScmPort *out, 
+void sigset_print(ScmObj obj, ScmPort *out,
                   ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(out, "#<sys-sigset [");
@@ -376,9 +375,6 @@ void Scm_SigFillSetMostly(sigset_t *set) /* out */
 #ifdef SIGKILL
     sigdelset(set, SIGKILL);
 #endif
-#ifdef SIGCONT
-    sigdelset(set, SIGCONT);
-#endif
 #ifdef SIGSTOP
     sigdelset(set, SIGSTOP);
 #endif
@@ -509,7 +505,7 @@ void Scm_SigCheck(ScmVM *vm)
             if (SCM_NULLP(tail)) {
                 q->pending = tail = cell;
             } else {
-                SCM_SET_CDR(tail, cell);
+                SCM_SET_CDR_UNCHECKED(tail, cell);
                 tail = SCM_CDR(tail);
             }
         }
@@ -852,6 +848,7 @@ void Scm_SetMasterSigmask(sigset_t *set)
  * aren't used anywhere.  Scm_SysSigmask covers those functionalities,
  * so we'll drop them by 1.0.
  */
+#if GAUCHE_API_VERSION < 98
 void Scm_GetSigmask(sigset_t *mask)
 {
     if (SIGPROCMASK(SIG_SETMASK, NULL, mask) != 0) {
@@ -865,6 +862,7 @@ void Scm_SetSigmask(sigset_t *mask)
         Scm_SysError("sigprocmask failed");
     }
 }
+#endif /*GAUCHE_API_VERSION < 98*/
 
 /*
  * set signal mask
