@@ -15,14 +15,16 @@
 (use util.match)
 (use file.util)
 (use gauche.cgen.literal)
-(use srfi-1)
+(use scheme.list)
 (use gauche.generator)
 
 ;;; Parser
 
 ;; returns ((num module exports doc-en doc-ja) ...)
 (define (parse self)
-  (let1 gen ($ record-generator $ file->string-list self :encoding 'utf-8)
+  (let1 gen ($ record-generator
+               $ remove #/^\;/
+               $ file->string-list self :encoding 'utf-8)
     ($ generator->list
        $ gmap ($ paragraphs->final-record $ split-by-empty-line $) gen)))
 
@@ -278,14 +280,14 @@ srfi-14, srfi-14
 Character-set library
 This has become a part of R7RS large.
 Character-set object and a few procedures are built-in.
-@xref{Character set}.
+@xref{Character sets}.
 Complete set of SRFI-14 is supported by the module @code{scheme.charset}.
 @xref{R7RS character sets}.
 
 文字集合のライブラリ
 これはR7RS largeの一部になりました。
 文字集合と基本的な手続きは組み込みになっています。
-@ref{Character set}参照。
+@ref{Character sets}参照。
 SRFI-14の完全なサポートはモジュール@code{scheme.charset}
 で提供されています。@ref{R7RS character sets}参照。
 
@@ -444,14 +446,10 @@ srfi-34
 Exception Handling for Programs
 This has become a part of R7RS small.
 Built-in.  @xref{Exceptions}.
-(However, Gauche implements srfi-18's semantics of @code{raise} literally,
-which differs slightly from srfi-34's.  This may be changed in future.)
 
 プログラムの例外処理
 これはR7RS smallの一部になりました。
 組み込みです。@ref{Exceptions}参照。
-(但し、Gaucheは@code{raise}に関してはsrfi-18のセマンティクスを文字通り
-実装していて、それはsrfi-34と若干異なります。将来はsrfi-34に合わせるかもしれません。)
 
 
 srfi-35
@@ -496,18 +494,16 @@ Built-in.  See @ref{Reading data} and @ref{Output}.
 組み込みです。@ref{Reading data}と@ref{Output}参照。
 
 
-srfi-39, gauche.parameter
-()
+srfi-39
+(make-parameter parameterize)
 
 Parameter objects
 This has become a part of R7RS small.
-Supported by the module @code{gauche.parameter}.
-@xref{Parameters}.
+Built-in (@pxref{Parameters}).
 
 パラメータオブジェクト
 これはR7RS smallの一部になりました。
-モジュール@code{gauche.parameter}でサポートされます。
-@ref{Parameters}参照。
+組み込みです。@ref{Parameters}参照。
 
 
 srfi-40, util.stream
@@ -607,11 +603,11 @@ srfi-60, srfi-60
 ()
 
 Integers as bits
-Most procedures are built-in: @xref{Bitwise operations}.
+Most procedures are built-in: @xref{R7RS bitwise operations}.
 The complete support is in @code{srfi-60} module: @xref{Integers as bits}.
 
 整数に対するビット操作
-ほとんどの手続きは組み込みになっています。@ref{Bitwise operations}参照。
+ほとんどの手続きは組み込みになっています。@ref{R7RS bitwise operations}参照。
 完全なサポートは@code{srfi-60}モジュールで提供されます。
 @ref{Integers as bits}参照。
 
@@ -642,10 +638,10 @@ srfi-64, srfi-64
 ()
 
 A Scheme API for test suites
-Supported by the module @code{srfi-64}.
+Supported by the module @code{srfi-64}. @xref{A Scheme API for test suites}.
 
 テストスイートのためのScheme API
-モジュール@code{srfi-64}でサポートされます。
+モジュール@code{srfi-64}でサポートされます。@ref{A Scheme API for test suites}参照。
 
 
 srfi-66, srfi-66
@@ -694,14 +690,11 @@ srfi-78, srfi-78
 
 Lightweight testing
 Supported by the module @code{srfi-78}.
-Since Gauche already has its own test framework (@pxref{Unit testing}),
-this is mainly for third-party modules that adopt srfi-78 for testing.
+It can work with @code{gauche.test}.  @xref{Lightweight testing}.
 
 軽いテストフレームワーク
 モジュール@code{srfi-78}でサポートされます。
-Gaucheは既に独自のテストフレームワークを持っているので(@ref{Unit testing}参照)、
-このモジュールは主としてサードパーティ製のモジュールがsrfi-78をテストに
-使っている場合をサポートするためのものです。
+@code{gauche.test}と一緒に使うこともできます。@ref{Lightweight testing}参照。
 
 
 srfi-87
@@ -733,7 +726,7 @@ SLIB Prerequisites
 This srfi is not exactly a library, but
 rather a description of features the platform should provide
 to support SLIB.  In order to load this module, SLIB must be
-already installed.  @xref{SLIB}, for the details.  
+already installed.  @xref{SLIB}, for the details.
 
 SLIB必要事項
 このsrfiはライブラリというよりは、SLIBをサポートするためにプラットフォームが
@@ -760,6 +753,23 @@ Supported by the module @code{gauche.record}.  @xref{Record types}.
 ERR5RS レコード
 モジュール@code{gauche.record}でサポートされます。@ref{Record types}参照。
 
+
+;; We don't load srfi-101 on (cond-expand ((srfi-101) ...)) for now, since
+;; it would shadow many list primitives and may create wreck havoc.  Especialy
+;; it would make SLIB unusable, for it uses cond-expand to check library
+;; availability.
+srfi-101
+()
+
+Purely functional random-access pairs and lists
+This has become a part of R7RS large.
+Supported by the module @code{scheme.rlist}.
+
+純粋に関数的でランダムアクセス可能なペアとリスト
+これはR7RS largeの一部となりました。
+モジュール@code{scheme.rlist}でサポートされます。
+
+
 srfi-106, srfi-106
 ()
 
@@ -770,16 +780,16 @@ Supported by the module @code{srfi-106}.  @xref{Basic socket interface}.
 モジュール@code{srfi-106}でサポートされます。@ref{Basic socket interface}参照。
 
 
-srfi-111, srfi-111
+srfi-111
 (box box? unbox set-box!)
 
 Boxes
-This has become a part of R7RS large.
-Supported by the module @code{scheme.box}.  @xref{R7RS boxes}.
+This has become a part of R7RS large as @code{scheme.box}.
+Gauche has it as built-in.  @xref{Boxes}.
 
 ボックス
-これはR7RS largeの一部となりました。
-モジュール@code{scheme.box}でサポートされます。@ref{R7RS boxes}参照。
+これは@code{scheme.box}としてR7RS largeの一部となりました。
+Gaucheは組み込みでサポートしています。@ref{Boxes}参照。
 
 
 srfi-112, srfi-112
@@ -829,6 +839,21 @@ Scheme正規表現
 モジュール@code{scheme.regex}でサポートされます。@ref{R7RS regular expressions}参照。
 
 
+srfi-116, srfi-116
+()
+
+Immutable List Library
+This has become a part of R7RS large.
+Immutable pairs are supported natively (@pxref{Mutable and immutable pairs}).
+Full set of APIs are available
+in the module @code{scheme.ilist} (@pxref{R7RS immutable lists}).
+
+変更不可リストのライブラリ
+これはR7RS largeの一部となりました。
+変更不可なペアはGauche本体で直接サポートされます(@ref{Mutable and immutable pairs}参照)。
+フルセットのAPIは モジュール@code{scheme.ilist}でサポートされます(@ref{R7RS immutable lists}参照)。
+
+
 srfi-117, srfi-117
 ()
 
@@ -855,6 +880,18 @@ Supported by the module @code{srfi-118}.
 (@ref{Simple adjustable-size strings}参照)
 
 
+srfi-120, srfi-120
+()
+
+Timer APIs
+Supported by the module @code{srfi-120} (@pxref{Timer APIs}).
+It is a wrapper of @code{control.scheduler} (@pxref{Scheduler}).
+
+タイマーAPI
+モジュール@code{srfi-120}でサポートされます (@ref{Timer APIs}参照)。
+これは@code{control.scheduler}のラッパーです (@ref{Scheduler}参照)。
+
+
 srfi-121, gauche.generator
 ()
 
@@ -945,6 +982,24 @@ Titlecase手続き
 この3つの手続きがインポートされます。
 
 
+srfi-130, srfi-130
+()
+
+Cursor-based string library
+String cursors are supported natively (@pxref{String cursors}).
+Most of built-in and srfi-13 string procedures accept cursors in addition
+to indexes.  The module srfi-130 provides several procedures that
+has the same name as srfi-13 but returns a string cursor instead
+of an index (@pxref{Cursor-based string library}).
+
+カーソルベースの文字列ライブラリ
+文字列カーソルは組み込みでサポートされています (@ref{String cursors}参照)。
+また、多くの組み込みおよびsrfi-13手続きはインデックスに加えてカーソルも
+取れるようになっています。モジュールsrfi-130は、srfi-13と同名だけれど
+インデックスの代わりにカーソルを返す手続きを提供します
+(@ref{Cursor-based string library}参照)。
+
+
 srfi-131, gauche.record
 ()
 
@@ -997,6 +1052,24 @@ The module @code{data.ideque} is compatible to srfi-134.
 @ref{変更不可な両端キュー}参照。
 
 
+srfi-135, srfi-135
+()
+
+Immutable Texts
+This has become a part of R7RS large.
+In Gauche, the text type is not disjoint from the string type.  Instead,
+a text is simply an immutable and indexed string.
+@xref{String indexing}, for the detail of indexed string.
+The API is described in @ref{R7RS immutable texts}.
+
+変更不可なテキスト
+これはR7RS largeの一部となりました。
+Gaucheではtext型はstring型と独立してはおらず、
+textは単に「変更不可でインデクスされた文字列」にすぎません。
+文字列のインデクシングについては@ref{String indexing}を参照してください。
+APIは@ref{R7RS immutable texts}で説明しています。
+
+
 srfi-141, srfi-141
 ()
 
@@ -1072,12 +1145,12 @@ srfi-151, srfi-151
 ()
 
 Bitwise operations
-Supported by the module @code{srfi-151} (@pxref{Bitwise operations}).
+Supported by the module @code{srfi-151} (@pxref{R7RS bitwise operations}).
 Note that many equivalent procedures are provided built-in
 (@pxref{Basic bitwise operations}).
 
 ビット演算
-モジュール@code{srfi-151}でサポートされます (@ref{Bitwise operations}参照)。
+モジュール@code{srfi-151}でサポートされます (@ref{R7RS bitwise operations}参照)。
 なお、組み込みでも多くの手続きについては同等の手続きが提供されています
 (@ref{Basic bitwise operations}参照).
 
@@ -1120,6 +1193,15 @@ Note that most of generator procedures are supported by @code{gauche.generator}
 (@ref{Generators}参照)。
 
 
+srfi-159, srfi-159
+()
+
+Combinator Formatting
+This has become a part of R7RS large.  @xref{R7RS combinator formatting}.
+
+コンビネータによるフォーマッタ
+これはR7RS largeの一部になりました。@ref{R7RS combinator formatting}参照。
+
 srfi-160, srfi-160
 ()
 
@@ -1151,6 +1233,16 @@ Supported by the module @code{srfi-162}.  @xref{Comparator sublibrary}.
 モジュール@code{srfi-162}でサポートされます。@ref{Comparator sublibrary}参照。
 
 
+srfi-170, srfi-170
+()
+
+POSIX API
+Supported by the module @code{srfi-170}.  @xref{POSIX API}.
+
+POSIX API
+モジュール@code{srfi-170}でサポートされます。@ref{POSIX API}参照。
+
+
 srfi-173, srfi-173
 ()
 
@@ -1162,3 +1254,212 @@ which is a thin layer on top of @code{gauche.hook} (@pxref{Hooks})
 モジュール@code{srfi-173}でサポートされます (@ref{Hooks (srfi)}参照) 。
 それ自体は@code{gauche.hook}の上にかぶせた薄いレイヤです (@ref{Hooks}参照)。
 
+
+srfi-174, srfi-174
+()
+
+POSIX Timespecs
+Supported by the module @code{srfi-174} (@pxref{POSIX timespecs}).
+In Gauche, the timespec type is the same as built-in @code{<time>}
+object, which is also the same as srfi-19 time.
+
+POSIX Timespec
+モジュール@code{srfi-174}でサポートされます (@ref{POSIX timespecs}参照)。
+Gaucheでは、timespecオブジェクトは組み込みの@code{<time>}オブジェクトであり、
+またsrfi-19のtimeオブジェクトとも共通です。
+
+
+srfi-175, srfi-175
+()
+
+ASCII character library
+Supported by the module @code{srfi-175} (@pxref{ASCII character library}).
+
+ASCII文字ライブラリ
+モジュール@code{srfi-175}でサポートされます(@ref{ASCII character library}参照)。
+
+
+srfi-176, srfi-176
+()
+
+Version flag
+Supported as a command-line flag of @code{gosh}.
+The @code{version-alist} procedure is built-in.
+
+バージョンフラグ
+@code{gosh}のコマンドライン引数としてサポートされます。
+@code{version-alist}手続きは組み込みです。
+
+
+srfi-178, srfi-178
+()
+
+Bitvector library
+The basic support is built-in (@pxref{Bitvectors}).
+Complete support is in the module @code{srfi-178} (@pxref{Bitvector library}.
+
+ビットベクタライブラリ
+基本的なサポートは組み込みです (@ref{Bitvectors}参照)。
+完全なサポートはモジュール@code{srfi-178}にあります(@ref{Bitvector library}参照)。
+
+
+srfi-180, srfi-180
+()
+
+JSON
+Supported by the module @code{srfi-180} (@pxref{JSON}).
+Note that Gauche also has @code{rfc.json},
+and @code{srfi-180} is implemented on top of it.
+
+JSON
+モジュール@code{srfi-180}でサポートされます (@ref{JSON}参照)。
+Gaucheには@code{rfc.json}もあって、@code{srfi-180}は
+@code{rfc.json}の上に実装されています。
+
+
+srfi-181, srfi-181
+()
+
+Custom ports
+Supported by the module @code{srfi-181} (@pxref{Custom ports}).
+Gauche has an original custom port mechanism (@pxref{Virtual ports}), and
+This srfi is built in top of it.
+
+カスタムポート
+モジュール@code{srfi-181}でサポートされます (@ref{Custom ports}参照)。
+Gaucheは独自のカスタムポートメカニズムを持っており(@ref{Virtual ports}参照)、
+このsrfiはその上に作られています。
+
+
+srfi-185, srfi-185
+()
+
+Linear adjustable-length strings
+Supported by the module @code{srfi-185}.
+@xref{Linear adjustable-length strings}.
+
+線形更新な可変長文字列
+モジュール@code{srfi-185}でサポートされます。
+@ref{Linear adjustable-length strings}参照。
+
+
+srfi-189, srfi-189
+()
+
+Maybe and Either: optional container types
+Supported by the module @code{srfi-189}.
+
+MaybeとEither: オプショナルなコンテナ型
+モジュール@code{srfi-189}でサポートされます。
+
+
+srfi-192, srfi-192
+()
+
+Port positioning
+Gauche's port already has positining mechanism, so main procedures
+are built-in (@pxref{Common port operations}).
+A few additional procedures are provided by the module @code{srfi-192}
+(@pxref{Port positioning}).
+
+ポート位置
+Gaucheのポートは既に位置の問い合わせと設定機能を持っており、
+主要な手続きは組み込みになっています(@ref{Common port operations}参照)。
+組み込みでない追加手続きはモジュール@code{srfi-192}にあります
+(@ref{Port positioning}参照)。
+
+
+srfi-193, srfi-193
+()
+
+Command line
+Two procedures, @code{command-line} and @code{script-file}, are
+built-in.  Other APIs are provided by the module @code{srfi-193}
+(@pxref{Command line}).
+
+コマンドライン
+@code{command-line}と@code{script-file}の二つの手続きは組み込みです。
+他のAPIはモジュール@code{srfi-193}で提供されます (@ref{Command line}参照)。
+
+
+srfi-195
+(box box? unbox set-box! box-arity unbox-value set-box-value!)
+
+Multiple-value boxes
+Built-in.  @xref{Boxes}.
+
+多値ボックス
+組み込みです。@ref{Boxes}参照。
+
+
+srfi-196, srfi-196
+()
+
+Range objects
+Suppored by the module @code{srfi-196}.  Also the @code{data.range}
+module is the superset of this srfi (@pxref{Range}).
+
+モジュール@code{srfi-196}でサポートされます。また、@code{data.range}モジュールは
+このsrfiの上位互換になっています (@ref{Range}参照)。
+
+
+srfi-197, srfi-197
+()
+
+Pipeline operators
+Supported by the module @code{srfi-197} (@pxref{Pipeline operators}).
+
+パイプラインオペレータ
+モジュール@code{srfi-197}でサポートされます (@ref{Pipeline operators}参照)。
+
+
+srfi-217, srfi-217
+()
+
+Integer sets
+Supported by the module @code{srfi-217}
+(@pxref{Integer sets}).
+
+整数集合
+@code{srfi-217}でサポートされます
+(@ref{Integer sets}参照)。
+
+
+srfi-219
+(define)
+
+Define higher-order lambda
+Gauche's built-in @code{define} (both R7RS-compatible one and
+extended one) supports this feature.
+If you import srfi-219 explicitly, it imports @code{null#define}.
+@xref{Define higher-order lambda}, for the details.
+
+高階lambdaの定義
+Gauche組み込みの@code{define}は (R7RS互換のものも拡張のものも)
+この機能をサポートしています。
+陽にsrfi-219をインポートした場合は、@code{null#define}がインポートされます。
+詳しくは@ref{Define higher-order lambda}参照。
+
+
+srfi-221, srfi-221
+()
+
+Generator/accumulator sub-library
+Supported by the module @code{srfi-221}
+(@pxref{Generator/accumulator sub-library}).
+
+ジェネレータ/アキュムレータ サブライブラリ
+@code{srfi-221}でサポートされます
+(@ref{Generator/accumulator sub-library}参照)。
+
+
+srfi-229, srfi-229
+()
+
+Tagged procedures
+Supported by the module @code{srfi-229}
+(@pxref{Tagged procedures}).
+
+手続きタグ
+@code{srfi-229}でサポートされます。
+(@ref{Tagged procedures}参照)。
