@@ -1,7 +1,7 @@
 /*
  * symbol.c - symbol implementation
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -194,7 +194,7 @@ static int escape_required_p(const ScmStringBody *b, u_int flags, int casefold)
 {
     int siz = SCM_STRING_BODY_SIZE(b);
     if (siz == 0) return !(flags & SCM_SYMBOL_WRITER_NOESCAPE_EMPTY);
-    
+
     const char *p = SCM_STRING_BODY_START(b);
     const char *e = p + siz;
     ScmChar ch;
@@ -410,7 +410,7 @@ ScmObj Scm_DeleteKeywordX(ScmObj key, ScmObj list)
                 return Scm_DeleteKeywordX(key, SCM_CDR(SCM_CDR(cp)));
             } else {
                 ScmObj tail = Scm_DeleteKeywordX(key, SCM_CDR(SCM_CDR(cp)));
-                SCM_SET_CDR(prev, tail);
+                Scm_SetCdr(prev, tail);
                 return list;
             }
         }
