@@ -1,10 +1,9 @@
 /*
  * Test the lowest-level numeric routines.
- * $Id: test-arith.c,v 1.9 2007-08-24 23:55:44 shirok Exp $
  */
 
 #include "gauche.h"
-#include "gauche/arith.h"
+#include "gauche/priv/arith.h"
 #include "gauche/scmconst.h"
 
 #define UMAX SCM_ULONG_MAX
@@ -26,7 +25,7 @@ void message(FILE *out, const char *m, int filler)
 }
 
 /*=============================================================
- * Testing macros in gauche/arith.h
+ * Testing macros in gauche/priv/arith.h
  */
 
 #define TEST_SECTION(name) message(stdout, "<" name ">", '-')
@@ -80,7 +79,7 @@ void message(FILE *out, const char *m, int filler)
  * UADD
  */
 #define TEST_UADD(x_, y_, c_, rexp, cexp)               \
-  TEST5(x_, y_, c_, rexp, cexp, "%u", "+", UADD)
+  TEST5(x_, y_, c_, rexp, cexp, "%lu", "+", UADD)
 
 void test_uadd(void)
 {
@@ -88,36 +87,36 @@ void test_uadd(void)
     TEST_SECTION("UADD");
 
     /* MAX + 0 + 0 => [MAX, 0] */
-    TEST_UADD(UMAX, 0, 0, UMAX, 0);
+    TEST_UADD(UMAX, 0UL, 0UL, UMAX, 0UL);
     /* MAX + 1 + 0 => [0, 1]   */
-    TEST_UADD(UMAX, 1, 0, 0, 1);
+    TEST_UADD(UMAX, 1UL, 0UL, 0UL, 1UL);
     /* MAX + 2 + 0 => [1, 1]   */
-    TEST_UADD(UMAX, 2, 0, 1, 1);
+    TEST_UADD(UMAX, 2UL, 0UL, 1UL, 1UL);
     /* MAX + MAX + 0 => [MAX-1, 1] */
-    TEST_UADD(UMAX, UMAX, 0, UMAX-1, 1);
+    TEST_UADD(UMAX, UMAX, 0UL, UMAX-1, 1UL);
     /* MAX/2 + MAX/2 + 0 => [MAX-1, 0] */
-    TEST_UADD(UMAX/2, UMAX/2, 0, UMAX-1, 0);
+    TEST_UADD(UMAX/2, UMAX/2, 0UL, UMAX-1, 0UL);
     /* MAX/2 + MAX + 0 => [MAX/2-1, 1] */
-    TEST_UADD(UMAX/2, UMAX, 0, UMAX/2-1, 1);
+    TEST_UADD(UMAX/2, UMAX, 0UL, UMAX/2-1, 1UL);
     /* MAX-1 + 0 + 1 => [MAX, 0] */
-    TEST_UADD(UMAX-1, 0, 1, UMAX, 0);
+    TEST_UADD(UMAX-1, 0UL, 1UL, UMAX, 0UL);
     /* MAX + 0 + 1 => [0, 1] */
-    TEST_UADD(UMAX, 0, 1, 0, 1);
+    TEST_UADD(UMAX, 0UL, 1UL, 0UL, 1UL);
     /* MAX + 1 + 1 => [1, 1] */
-    TEST_UADD(UMAX, 1, 1, 1, 1);
+    TEST_UADD(UMAX, 1UL, 1UL, 1UL, 1UL);
     /* MAX + MAX + 1 => [MAX, 1] */
-    TEST_UADD(UMAX, UMAX, 1, UMAX, 1);
+    TEST_UADD(UMAX, UMAX, 1UL, UMAX, 1UL);
     /* MAX/2 + MAX/2 + 1 => [MAX, 0] */
-    TEST_UADD(UMAX/2, UMAX/2, 1, UMAX, 0);
+    TEST_UADD(UMAX/2, UMAX/2, 1UL, UMAX, 0UL);
     /* MAX/2 + MAX + 1 => [MAX/2, 1] */
-    TEST_UADD(UMAX/2, UMAX, 1, UMAX/2, 1);
+    TEST_UADD(UMAX/2, UMAX, 1UL, UMAX/2, 1UL);
 }
 
 /*
  * UADDOV
  */
 #define TEST_UADDOV(x_, y_, rexp, cexp)         \
-    TESTOV(x_, y_, rexp, cexp, "%u", "+", UADDOV)
+    TESTOV(x_, y_, rexp, cexp, "%lu", "+", UADDOV)
 
 void test_uaddov(void)
 {
@@ -125,21 +124,21 @@ void test_uaddov(void)
     TEST_SECTION("UADDOV");
 
     /* MAX + 0 => MAX */
-    TEST_UADDOV(UMAX, 0, UMAX, 0);
+    TEST_UADDOV(UMAX, 0UL, UMAX, 0UL);
     /* MAX + 1 => overflow */
-    TEST_UADDOV(UMAX, 1, 0, 1);
+    TEST_UADDOV(UMAX, 1UL, 0UL, 1UL);
     /* MAX + 2 => overflow */
-    TEST_UADDOV(UMAX, 2, 0, 1);
+    TEST_UADDOV(UMAX, 2UL, 0UL, 1UL);
     /* MAX + MAX => overflow */
-    TEST_UADDOV(UMAX, UMAX, 0, 1);
+    TEST_UADDOV(UMAX, UMAX, 0UL, 1UL);
     /* MAX/2 + MAX/2 => MAX-1, 0 */
-    TEST_UADDOV(UMAX/2, UMAX/2, UMAX-1, 0);
+    TEST_UADDOV(UMAX/2, UMAX/2, UMAX-1, 0UL);
     /* MAX/2 + MAX => overflow */
-    TEST_UADDOV(UMAX/2, UMAX, 0, 1);
+    TEST_UADDOV(UMAX/2, UMAX, 0UL, 1UL);
     /* MAX-1 + 1 => MAX */
-    TEST_UADDOV(UMAX-1, 1, UMAX, 0);
+    TEST_UADDOV(UMAX-1, 1UL, UMAX, 0UL);
     /* MAX-1 + 2 => overflow */
-    TEST_UADDOV(UMAX-1, 2, 0, 1);
+    TEST_UADDOV(UMAX-1, 2UL, 0UL, 1UL);
 }
 
 /*
@@ -154,46 +153,46 @@ void test_saddov(void)
     TEST_SECTION("SADDOV");
 
     /* 1 + 1 => 2 */
-    TEST_SADDOV(1, 1, 2, 0);
+    TEST_SADDOV(1L, 1L, 2L, 0L);
     /* -1 + -1 => -2 */
-    TEST_SADDOV(-1, -1, -2, 0);
+    TEST_SADDOV(-1L, -1L, -2L, 0L);
     /* SMAX + 0 => SMAX */
-    TEST_SADDOV(SMAX, 0, SMAX, 0);
+    TEST_SADDOV(SMAX, 0L, SMAX, 0L);
     /* SMAX + 1 => overflow */
-    TEST_SADDOV(SMAX, 1, 0, 1);
+    TEST_SADDOV(SMAX, 1L, 0L, 1L);
     /* SMAX + 2 => overflow */
-    TEST_SADDOV(SMAX, 2, 0, 1);
+    TEST_SADDOV(SMAX, 2L, 0L, 1L);
     /* SMAX + -1 => SMAX-1 */
-    TEST_SADDOV(SMAX, -1, SMAX-1, 0);
+    TEST_SADDOV(SMAX, -1L, SMAX-1, 0L);
     /* SMAX + -SMAX => 0 */
-    TEST_SADDOV(SMAX, -SMAX, 0, 0);
+    TEST_SADDOV(SMAX, -SMAX, 0L, 0L);
     /* SMAX + -SMAX-1 => -1 */
-    TEST_SADDOV(SMAX, -SMAX-1, -1, 0);
+    TEST_SADDOV(SMAX, -SMAX-1, -1L, 0L);
     /* SMAX/2 + SMAX/2 => SMAX-1 */
-    TEST_SADDOV(SMAX/2, SMAX/2, SMAX-1, 0);
+    TEST_SADDOV(SMAX/2, SMAX/2, SMAX-1, 0L);
     /* SMAX/2 + -SMAX/2 => 0 */
-    TEST_SADDOV(SMAX/2, -(SMAX/2), 0, 0);
+    TEST_SADDOV(SMAX/2, -(SMAX/2), 0L, 0L);
     /* -SMAX/2 + -SMAX/2 => -(SMAX-1) */
-    TEST_SADDOV(-(SMAX/2), -(SMAX/2), -(SMAX-1), 0);
+    TEST_SADDOV(-(SMAX/2), -(SMAX/2), -(SMAX-1), 0L);
     /* 0 + -SMAX => -SMAX */
-    TEST_SADDOV(0, -SMAX, -SMAX, 0);
+    TEST_SADDOV(0L, -SMAX, -SMAX, 0L);
     /* -1 + -SMAX => SMIN */
-    TEST_SADDOV(-1, -SMAX, SMIN, 0);
+    TEST_SADDOV(-1L, -SMAX, SMIN, 0L);
     /* -2 + -SMAX => -overflow */
-    TEST_SADDOV(-2, -SMAX, 0, -1);
+    TEST_SADDOV(-2L, -SMAX, 0L, -1L);
     /* -SMAX + -2 => -overflow */
-    TEST_SADDOV(-SMAX, -2, 0, -1);
+    TEST_SADDOV(-SMAX, -2L, 0L, -1L);
     /* -SMAX + -SMAX => -overflow */
-    TEST_SADDOV(-SMAX, -SMAX, 0, -1);
+    TEST_SADDOV(-SMAX, -SMAX, 0L, -1L);
     /* -SMAX + SMIN => -overflow */
-    TEST_SADDOV(-SMAX, SMIN, 0, -1);
+    TEST_SADDOV(-SMAX, SMIN, 0L, -1L);
 }
 
 /*
  * USUB
  */
 #define TEST_USUB(x_, y_, c_, rexp, cexp)               \
-    TEST5(x_, y_, c_, rexp, cexp, "%u", "-", USUB)
+    TEST5(x_, y_, c_, rexp, cexp, "%lu", "-", USUB)
 
 void test_usub(void)
 {
@@ -201,36 +200,36 @@ void test_usub(void)
     TEST_SECTION("USUB");
 
     /* MAX - 0 - 0 => [MAX, 0] */
-    TEST_USUB(UMAX, 0, 0, UMAX, 0);
+    TEST_USUB(UMAX, 0UL, 0UL, UMAX, 0UL);
     /* MAX - 1 - 0 => [MAX-1, 0] */
-    TEST_USUB(UMAX, 1, 0, UMAX-1, 0);
+    TEST_USUB(UMAX, 1UL, 0UL, UMAX-1, 0UL);
     /* 0 - MAX - 0 => [1, 1] */
-    TEST_USUB(0, UMAX, 0, 1, 1);
+    TEST_USUB(0UL, UMAX, 0UL, 1UL, 1UL);
     /* 1 - MAX - 0 => [2, 1] */
-    TEST_USUB(1, UMAX, 0, 2, 1);
+    TEST_USUB(1UL, UMAX, 0UL, 2UL, 1UL);
     /* MAX - MAX/2 - 0 => [MAX/2+1, 0] */
-    TEST_USUB(UMAX, UMAX/2, 0, UMAX/2+1, 0);
+    TEST_USUB(UMAX, UMAX/2, 0UL, UMAX/2+1, 0UL);
     /* MAX/2 - MAX - 0 => [MAX/2+1, 1] */
-    TEST_USUB(UMAX/2, UMAX, 0, UMAX/2+1, 1);
+    TEST_USUB(UMAX/2, UMAX, 0UL, UMAX/2+1, 1UL);
     /* MAX - 0 - 1 => [MAX-1, 0] */
-    TEST_USUB(UMAX, 0, 1, UMAX-1, 0);
+    TEST_USUB(UMAX, 0UL, 1UL, UMAX-1, 0UL);
     /* MAX - 1 - 1 => [MAX-2, 0] */
-    TEST_USUB(UMAX, 1, 1, UMAX-2, 0);
+    TEST_USUB(UMAX, 1UL, 1UL, UMAX-2, 0UL);
     /* 0 - MAX - 1 => [0, 1] */
-    TEST_USUB(0, UMAX, 1, 0, 1);
+    TEST_USUB(0UL, UMAX, 1UL, 0UL, 1UL);
     /* 1 - MAX - 1 => [1, 1] */
-    TEST_USUB(1, UMAX, 1, 1, 1);
+    TEST_USUB(1UL, UMAX, 1UL, 1UL, 1UL);
     /* MAX - MAX/2 - 1 => [MAX/2, 0] */
-    TEST_USUB(UMAX, UMAX/2, 1, UMAX/2, 0);
+    TEST_USUB(UMAX, UMAX/2, 1UL, UMAX/2, 0UL);
     /* MAX/2 - MAX - 1 => [MAX/2, 1] */
-    TEST_USUB(UMAX/2, UMAX, 1, UMAX/2, 1);
+    TEST_USUB(UMAX/2, UMAX, 1UL, UMAX/2, 1UL);
 }
 
 /*
  * USUBOV
  */
 #define TEST_USUBOV(x_, y_, rexp, cexp)         \
-    TESTOV(x_, y_, rexp, cexp, "%u", "-", USUBOV)
+    TESTOV(x_, y_, rexp, cexp, "%lu", "-", USUBOV)
 
 void test_usubov(void)
 {
@@ -238,19 +237,19 @@ void test_usubov(void)
     TEST_SECTION("USUBOV");
 
     /* MAX - 0  => MAX */
-    TEST_USUBOV(UMAX, 0, UMAX, 0);
+    TEST_USUBOV(UMAX, 0UL, UMAX, 0UL);
     /* MAX - 1  => MAX-1 */
-    TEST_USUBOV(UMAX, 1, UMAX-1, 0);
+    TEST_USUBOV(UMAX, 1UL, UMAX-1, 0UL);
     /* MAX - MAX => 0 */
-    TEST_USUBOV(UMAX, UMAX, 0, 0);
+    TEST_USUBOV(UMAX, UMAX, 0UL, 0UL);
     /* MAX-1 - MAX => overflow */
-    TEST_USUBOV(UMAX-1, UMAX, 0, 1);
+    TEST_USUBOV(UMAX-1, UMAX, 0UL, 1UL);
     /* MAX - MAX-1 => 1 */
-    TEST_USUBOV(UMAX, UMAX-1, 1, 0);
+    TEST_USUBOV(UMAX, UMAX-1, 1UL, 0UL);
     /* 0 - 1 => overflow */
-    TEST_USUBOV(0, 1, 0, 1);
+    TEST_USUBOV(0UL, 1UL, 0UL, 1UL);
     /* 0 - MAX => overflow */
-    TEST_USUBOV(0, UMAX, 0, 1);
+    TEST_USUBOV(0UL, UMAX, 0UL, 1UL);
 }
 
 /*
@@ -265,41 +264,41 @@ void test_ssubov(void)
     TEST_SECTION("SSUBOV");
 
     /* 1 - 1 => 0 */
-    TEST_SSUBOV(1, 1, 0, 0);
+    TEST_SSUBOV(1L, 1L, 0L, 0L);
     /* -1 - -1 => 0 */
-    TEST_SSUBOV(-1, -1, 0, 0);
+    TEST_SSUBOV(-1L, -1L, 0L, 0L);
     /* SMAX - 0 => SMAX */
-    TEST_SSUBOV(SMAX, 0, SMAX, 0);
+    TEST_SSUBOV(SMAX, 0L, SMAX, 0L);
     /* SMAX - 1 => SMAX-1 */
-    TEST_SSUBOV(SMAX, 1, SMAX-1, 0);
+    TEST_SSUBOV(SMAX, 1L, SMAX-1, 0L);
     /* SMAX - -1 => overflow */
-    TEST_SSUBOV(SMAX, -1, 0, 1);
+    TEST_SSUBOV(SMAX, -1L, 0L, 1L);
     /* SMAX - -2 => overflow */
-    TEST_SSUBOV(SMAX, -2, 0, 1);
+    TEST_SSUBOV(SMAX, -2L, 0L, 1L);
     /* SMAX - -SMAX => overflow */
-    TEST_SSUBOV(SMAX, -SMAX, 0, 1);
+    TEST_SSUBOV(SMAX, -SMAX, 0L, 1L);
     /* SMAX - SMAX => 0 */
-    TEST_SSUBOV(SMAX, SMAX, 0, 0);
+    TEST_SSUBOV(SMAX, SMAX, 0L, 0L);
     /* SMAX/2 - -SMAX/2 => SMAX-1 */
-    TEST_SSUBOV(SMAX/2, -(SMAX/2), SMAX-1, 0);
+    TEST_SSUBOV(SMAX/2, -(SMAX/2), SMAX-1, 0L);
     /* SMAX/2 - SMAX/2 => 0 */
-    TEST_SSUBOV(SMAX/2, SMAX/2, 0, 0);
+    TEST_SSUBOV(SMAX/2, SMAX/2, 0L, 0L);
     /* -SMAX/2 - SMAX/2 => 0 */
-    TEST_SSUBOV(-(SMAX/2), SMAX/2, -(SMAX-1), 0);
+    TEST_SSUBOV(-(SMAX/2), SMAX/2, -(SMAX-1), 0L);
     /* 0 - -SMAX => SMAX */
-    TEST_SSUBOV(0, -SMAX, SMAX, 0);
+    TEST_SSUBOV(0L, -SMAX, SMAX, 0L);
     /* 1 - -SMAX => overflow */
-    TEST_SSUBOV(1, -SMAX, 0, 1);
+    TEST_SSUBOV(1L, -SMAX, 0L, 1L);
     /* 0 - -SMAX => SMAX */
-    TEST_SSUBOV(0, -SMAX, SMAX, 0);
+    TEST_SSUBOV(0L, -SMAX, SMAX, 0L);
     /* SMIN - 1 => -overflow */
-    TEST_SSUBOV(SMIN, 1, 0, -1);
+    TEST_SSUBOV(SMIN, 1L, 0L, -1L);
     /* SMIN - -1 => SMIN+1 */
-    TEST_SSUBOV(SMIN, -1, SMIN+1, 0);
+    TEST_SSUBOV(SMIN, -1L, SMIN+1, 0L);
     /* -SMAX - -SMAX => 0 */
-    TEST_SSUBOV(-SMAX, -SMAX, 0, 0);
+    TEST_SSUBOV(-SMAX, -SMAX, 0L, 0L);
     /* -SMAX - SMIN => 1 */
-    TEST_SSUBOV(-SMAX, SMIN, 1, 0);
+    TEST_SSUBOV(-SMAX, SMIN, 1L, 0L);
 }
 
 /*
@@ -307,7 +306,7 @@ void test_ssubov(void)
  */
 #define TEST_UMUL(x_, y_, hiexp, loexp)                                 \
     do {                                                                \
-        printf("testing %u*%u expects hi=%u, lo=%u =>", x_, y_,        \
+        printf("testing %lu*%lu expects hi=%lu, lo=%lu =>", x_, y_,     \
                hiexp, loexp);                                           \
         x = x_;                                                         \
         y = y_;                                                         \
@@ -316,7 +315,7 @@ void test_ssubov(void)
             printf("ok\n");                                             \
         } else {                                                        \
             errcount++;                                                 \
-            printf("ERROR: got hi=%u, lo=%u\n", hi, lo);                \
+            printf("ERROR: got hi=%lu, lo=%lu\n", hi, lo);              \
         }                                                               \
     } while (0)
 
@@ -326,13 +325,13 @@ void test_umul(void)
     TEST_SECTION("UMUL");
 
     /* MAX * MAX => [MAX-1, 1] */
-    TEST_UMUL(UMAX, UMAX, UMAX-1, 1);
+    TEST_UMUL(UMAX, UMAX, UMAX-1, 1UL);
     /* MAX-1 * MAX-1 => [MAX-3, 4] */
-    TEST_UMUL(UMAX-1, UMAX-1, UMAX-3, 4);
+    TEST_UMUL(UMAX-1, UMAX-1, UMAX-3, 4UL);
     /* MAX/2 * 2 => [0, MAX-1] */
-    TEST_UMUL(UMAX/2, 2, 0, UMAX-1);
+    TEST_UMUL(UMAX/2, 2UL, 0UL, UMAX-1);
     /* MAX/2+1 * 2 => [1, 0] */
-    TEST_UMUL(UMAX/2+1, 2, 1, 0);
+    TEST_UMUL(UMAX/2+1, 2UL, 1UL, 0UL);
 }
 
 /*
@@ -347,27 +346,27 @@ void test_umulov(void)
     TEST_SECTION("UMULOV");
 
     /* 0 * 1 => 0 */
-    TEST_UMULOV(0, 1, 0, 0);
+    TEST_UMULOV(0UL, 1UL, 0UL, 0UL);
     /* 1 * 0 => 0 */
-    TEST_UMULOV(1, 0, 0, 0);
+    TEST_UMULOV(1UL, 0UL, 0UL, 0UL);
     /* 1 * 1 => 1 */
-    TEST_UMULOV(1, 1, 1, 0);
+    TEST_UMULOV(1UL, 1UL, 1UL, 0UL);
     /* 1 * UMAX => UMAX */
-    TEST_UMULOV(1, UMAX, UMAX, 0);
+    TEST_UMULOV(1UL, UMAX, UMAX, 0UL);
     /* 2 * UMAX => overflow */
-    TEST_UMULOV(2, UMAX, 0, 1);
+    TEST_UMULOV(2UL, UMAX, 0UL, 1UL);
     /* UMAX/2 * 2 => UMAX-1 */
-    TEST_UMULOV(UMAX/2, 2, UMAX-1, 0);
+    TEST_UMULOV(UMAX/2, 2UL, UMAX-1, 0UL);
     /* UMAX/2 * 3 => overflow */
-    TEST_UMULOV(UMAX/2, 3, 0, 1);
+    TEST_UMULOV(UMAX/2, 3UL, 0UL, 1UL);
     /* UMAX>>4 * UMAX>>4 => overflow */
-    TEST_UMULOV((UMAX>>4), (UMAX>>4), 0, 1);
+    TEST_UMULOV((UMAX>>4), (UMAX>>4), 0UL, 1UL);
     /* UMAX>>8 * UMAX>>8 => overflow */
-    TEST_UMULOV((UMAX>>8), (UMAX>>8), 0, 1);
+    TEST_UMULOV((UMAX>>8), (UMAX>>8), 0UL, 1UL);
     /* UMAX * UMAX => overflow */
-    TEST_UMULOV(UMAX, UMAX, 0, 1);
+    TEST_UMULOV(UMAX, UMAX, 0UL, 1UL);
     /* UMAX-1 * UMAX-1 => overflow */
-    TEST_UMULOV(UMAX-1, UMAX-1, 0, 1);
+    TEST_UMULOV(UMAX-1, UMAX-1, 0UL, 1UL);
 }
 
 /*
@@ -378,81 +377,81 @@ void test_umulov(void)
 
 void test_smulov(void)
 {
-    long r, c, x, y;
+    long r = 0, c, x, y;
     TEST_SECTION("SMULOV");
 
     /* 0 * 1 => 0 */
-    TEST_SMULOV(0, 1, 0, 0);
+    TEST_SMULOV(0L, 1L, 0L, 0L);
     /* 1 * 0 => 0 */
-    TEST_SMULOV(1, 0, 0, 0);
+    TEST_SMULOV(1L, 0L, 0L, 0L);
     /* 1 * 1 => 1 */
-    TEST_SMULOV(1, 1, 1, 0);
+    TEST_SMULOV(1L, 1L, 1L, 0L);
     /* 1 * -1 => -1 */
-    TEST_SMULOV(1, -1, -1, 0);
+    TEST_SMULOV(1L, -1L, -1L, 0L);
     /* -1 * 1 => -1 */
-    TEST_SMULOV(-1, 1, -1, 0);
+    TEST_SMULOV(-1L, 1L, -1L, 0L);
     /* -1 * -1 => 1 */
-    TEST_SMULOV(-1, -1, 1, 0);
+    TEST_SMULOV(-1L, -1L, 1L, 0L);
     /* 1 * SMAX => SMAX */
-    TEST_SMULOV(1, SMAX, SMAX, 0);
+    TEST_SMULOV(1L, SMAX, SMAX, 0L);
     /* 1 * -SMAX => -SMAX */
-    TEST_SMULOV(1, -SMAX, -SMAX, 0);
+    TEST_SMULOV(1L, -SMAX, -SMAX, 0L);
     /* -1 * SMAX => -SMAX */
-    TEST_SMULOV(-1, SMAX, -SMAX, 0);
+    TEST_SMULOV(-1L, SMAX, -SMAX, 0L);
     /* -1 * -SMAX => SMAX */
-    TEST_SMULOV(-1, -SMAX, SMAX, 0);
+    TEST_SMULOV(-1L, -SMAX, SMAX, 0L);
     /* 1 * SMIN => SMIN */
-    TEST_SMULOV(1, SMIN, SMIN, 0);
+    TEST_SMULOV(1L, SMIN, SMIN, 0L);
     /* -1 * SMIN => overflow */
-    TEST_SMULOV(-1, SMIN, 0, 1);
+    TEST_SMULOV(-1L, SMIN, 0L, 1L);
     /* 2 * SMAX => overflow */
-    TEST_SMULOV(2, SMAX, 0, 1);
+    TEST_SMULOV(2L, SMAX, 0L, 1L);
     /* 2 * -SMAX => -overflow */
-    TEST_SMULOV(2, -SMAX, 0, -1);
+    TEST_SMULOV(2L, -SMAX, 0L, -1L);
     /* -2 * SMAX => -overflow */
-    TEST_SMULOV(-2, SMAX, 0, -1);
+    TEST_SMULOV(-2L, SMAX, 0L, -1L);
     /* -2 * -SMAX => overflow */
-    TEST_SMULOV(-2, -SMAX, 0, 1);
+    TEST_SMULOV(-2L, -SMAX, 0L, 1L);
     /* SMAX/2 * 2 => SMAX-1 */
-    TEST_SMULOV(SMAX/2, 2, SMAX-1, 0);
+    TEST_SMULOV(SMAX/2, 2L, SMAX-1, 0L);
     /* SMAX/2 * -2 => -(SMAX-1) */
-    TEST_SMULOV(SMAX/2, -2, -(SMAX-1), 0);
+    TEST_SMULOV(SMAX/2, -2L, -(SMAX-1), 0L);
     /* -SMAX/2 * 2 => -(SMAX-1) */
-    TEST_SMULOV(-(SMAX/2), 2, -(SMAX-1), 0);
+    TEST_SMULOV(-(SMAX/2), 2L, -(SMAX-1), 0L);
     /* -SMAX/2 * -2 => SMAX-1 */
-    TEST_SMULOV(-(SMAX/2), -2, SMAX-1, 0);
+    TEST_SMULOV(-(SMAX/2), -2L, SMAX-1, 0L);
     /* SMAX/2+1 * 2 => overflow */
-    TEST_SMULOV(SMAX/2+1, 2, 0, 1);
+    TEST_SMULOV(SMAX/2+1, 2L, 0L, 1L);
     /* SMAX/2+1 * -2 => SMIN */
-    TEST_SMULOV(SMAX/2+1, -2, SMIN, 0);
+    TEST_SMULOV(SMAX/2+1, -2L, SMIN, 0L);
     /* -(SMAX/2+1) * 2 => SMIN */
-    TEST_SMULOV(-(SMAX/2+1), 2, SMIN, 0);
+    TEST_SMULOV(-(SMAX/2+1), 2L, SMIN, 0L);
     /* -(SMAX/2+1) * -2 => overflow */
-    TEST_SMULOV(-(SMAX/2+1), -2, 0, 1);
+    TEST_SMULOV(-(SMAX/2+1), -2L, 0L, 1L);
     /* SMAX>>4 * SMAX>>4 => overflow */
-    TEST_SMULOV((SMAX>>4), (SMAX>>4), 0, 1);
+    TEST_SMULOV((SMAX>>4), (SMAX>>4), 0L, 1L);
     /* SMAX>>4 * -SMAX>>4 => -overflow */
-    TEST_SMULOV((SMAX>>4), -(SMAX>>4), 0, -1);
+    TEST_SMULOV((SMAX>>4), -(SMAX>>4), 0L, -1L);
     /* -SMAX>>4 * SMAX>>4 => -overflow */
-    TEST_SMULOV(-(SMAX>>4), (SMAX>>4), 0, -1);
+    TEST_SMULOV(-(SMAX>>4), (SMAX>>4), 0L, -1L);
     /* -SMAX>>4 * -SMAX>>4 => overflow */
-    TEST_SMULOV(-(SMAX>>4), -(SMAX>>4), 0, 1);
+    TEST_SMULOV(-(SMAX>>4), -(SMAX>>4), 0L, 1L);
     /* SMAX>>8 * SMAX>>8 => overflow */
-    TEST_SMULOV((SMAX>>8), (SMAX>>8), 0, 1);
+    TEST_SMULOV((SMAX>>8), (SMAX>>8), 0L, 1L);
     /* SMAX>>8 * -SMAX>>8 => -overflow */
-    TEST_SMULOV((SMAX>>8), -(SMAX>>8), 0, -1);
+    TEST_SMULOV((SMAX>>8), -(SMAX>>8), 0L, -1L);
     /* -SMAX>>8 * SMAX>>8 => -overflow */
-    TEST_SMULOV(-(SMAX>>8), (SMAX>>8), 0, -1);
+    TEST_SMULOV(-(SMAX>>8), (SMAX>>8), 0L, -1L);
     /* -SMAX>>8 * -SMAX>>8 => overflow */
-    TEST_SMULOV(-(SMAX>>8), -(SMAX>>8), 0, 1);
+    TEST_SMULOV(-(SMAX>>8), -(SMAX>>8), 0L, 1L);
     /* SMAX * SMAX => overflow */
-    TEST_SMULOV(SMAX, SMAX, 0, 1);
+    TEST_SMULOV(SMAX, SMAX, 0L, 1L);
     /* SMAX * -SMAX => -overflow */
-    TEST_SMULOV(SMAX, -SMAX, 0, -1);
+    TEST_SMULOV(SMAX, -SMAX, 0L, -1L);
     /* -SMAX * SMAX => -overflow */
-    TEST_SMULOV(-SMAX, SMAX, 0, -1);
+    TEST_SMULOV(-SMAX, SMAX, 0L, -1L);
     /* -SMAX * -SMAX => overflow */
-    TEST_SMULOV(-SMAX, -SMAX, 0, 1);
+    TEST_SMULOV(-SMAX, -SMAX, 0L, 1L);
 }
 
 /*=============================================================
@@ -532,104 +531,104 @@ void test_32_64(void)
 #endif
 
     vv = Scm_Add(SCM_2_31, SCM_MAKE_INT(-1));
-    test_scm_c_scm("ScmInt32 roundtrip 2^31-1", vv,
+    test_scm_c_scm("int32_t roundtrip 2^31-1", vv,
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, 0, NULL)));
     vv = Scm_Add(SCM_2_31, SCM_MAKE_INT(-3));
-    test_scm_c_scm("ScmInt32 roundtrip 2^31-3", vv,
+    test_scm_c_scm("int32_t roundtrip 2^31-3", vv,
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, 0, NULL)));
     vv = Scm_Negate(SCM_2_31);
-    test_scm_c_scm("ScmInt32 roundtrip -2^31", vv,
+    test_scm_c_scm("int32_t roundtrip -2^31", vv,
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, 0, NULL)));
     vv = Scm_Add(Scm_Negate(SCM_2_31), SCM_MAKE_INT(2));
-    test_scm_c_scm("ScmInt32 roundtrip -2^31+2", vv,
+    test_scm_c_scm("int32_t roundtrip -2^31+2", vv,
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, 0, NULL)));
     vv = Scm_Add(SCM_2_32, SCM_MAKE_INT(-1));
-    test_scm_c_scm("ScmUInt32 roundtrip 2^32-1", vv,
+    test_scm_c_scm("uint32_t roundtrip 2^32-1", vv,
                    Scm_MakeIntegerU(Scm_GetIntegerU32Clamp(vv, 0, NULL)));
     vv = Scm_Add(SCM_2_32, SCM_MAKE_INT(-3));
-    test_scm_c_scm("ScmUInt32 roundtrip 2^32-3", vv,
+    test_scm_c_scm("uint32_t roundtrip 2^32-3", vv,
                    Scm_MakeIntegerU(Scm_GetIntegerU32Clamp(vv, 0, NULL)));
 
     vv = Scm_Add(SCM_2_31, SCM_MAKE_INT(-1));
-    test_scm_c_scm("ScmInt64 roundtrip 2^31-1", vv,
+    test_scm_c_scm("int64_t roundtrip 2^31-1", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
     vv = SCM_2_31;
-    test_scm_c_scm("ScmInt64 roundtrip 2^31", vv,
+    test_scm_c_scm("int64_t roundtrip 2^31", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
     vv = Scm_Add(SCM_2_32, SCM_MAKE_INT(-1));
-    test_scm_c_scm("ScmInt64 roundtrip 2^32-1", vv,
+    test_scm_c_scm("int64_t roundtrip 2^32-1", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
     vv = SCM_2_32;
-    test_scm_c_scm("ScmInt64 roundtrip 2^32", vv,
+    test_scm_c_scm("int64_t roundtrip 2^32", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
 
     vv = Scm_Add(Scm_Negate(SCM_2_31), SCM_MAKE_INT(1));
-    test_scm_c_scm("ScmInt64 roundtrip -2^31+1", vv,
+    test_scm_c_scm("int64_t roundtrip -2^31+1", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
     vv = Scm_Negate(SCM_2_31);
-    test_scm_c_scm("ScmInt64 roundtrip -2^31", vv,
+    test_scm_c_scm("int64_t roundtrip -2^31", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
     vv = Scm_Add(Scm_Negate(SCM_2_32), SCM_MAKE_INT(1));
-    test_scm_c_scm("ScmInt64 roundtrip -2^32+1", vv,
+    test_scm_c_scm("int64_t roundtrip -2^32+1", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
     vv = Scm_Negate(SCM_2_32);
-    test_scm_c_scm("ScmInt64 roundtrip -2^32", vv,
+    test_scm_c_scm("int64_t roundtrip -2^32", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
 
 
     vv = Scm_Add(SCM_2_63, SCM_MAKE_INT(-1));
-    test_scm_c_scm("ScmInt64 roundtrip 2^63-1", vv,
+    test_scm_c_scm("int64_t roundtrip 2^63-1", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
     vv = Scm_Add(SCM_2_63, SCM_MAKE_INT(-3));
-    test_scm_c_scm("ScmInt64 roundtrip 2^63-3", vv,
+    test_scm_c_scm("int64_t roundtrip 2^63-3", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
     vv = Scm_Negate(SCM_2_63);
-    test_scm_c_scm("ScmInt64 roundtrip -2^63", vv,
+    test_scm_c_scm("int64_t roundtrip -2^63", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
     vv = Scm_Add(Scm_Negate(SCM_2_63), SCM_MAKE_INT(2));
-    test_scm_c_scm("ScmInt64 roundtrip -2^63+2", vv,
+    test_scm_c_scm("int64_t roundtrip -2^63+2", vv,
                    Scm_MakeInteger64(Scm_GetInteger64(vv)));
 
     vv = Scm_Add(SCM_2_31, SCM_MAKE_INT(-1));
-    test_scm_c_scm("ScmUInt64 roundtrip 2^31", vv,
+    test_scm_c_scm("uint64_t roundtrip 2^31", vv,
                    Scm_MakeIntegerU64(Scm_GetIntegerU64(vv)));
     vv = SCM_2_31;
-    test_scm_c_scm("ScmUInt64 roundtrip 2^31", vv,
+    test_scm_c_scm("uint64_t roundtrip 2^31", vv,
                    Scm_MakeIntegerU64(Scm_GetIntegerU64(vv)));
     vv = Scm_Add(SCM_2_32, SCM_MAKE_INT(-1));
-    test_scm_c_scm("ScmUInt64 roundtrip 2^32-1", vv,
+    test_scm_c_scm("uint64_t roundtrip 2^32-1", vv,
                    Scm_MakeIntegerU64(Scm_GetIntegerU64(vv)));
     vv = SCM_2_32;
-    test_scm_c_scm("ScmUInt64 roundtrip 2^32", vv,
+    test_scm_c_scm("uint64_t roundtrip 2^32", vv,
                    Scm_MakeIntegerU64(Scm_GetIntegerU64(vv)));
 
     vv = Scm_Add(SCM_2_64, SCM_MAKE_INT(-1));
-    test_scm_c_scm("ScmUInt64 roundtrip 2^64-1", vv,
+    test_scm_c_scm("uint64_t roundtrip 2^64-1", vv,
                    Scm_MakeIntegerU64(Scm_GetIntegerU64(vv)));
 
     TEST_SECTION("integer conversions, clamping");
     vv = SCM_2_32;
-    test_scm_c_scm("ScmInt32 clamp 2^32",
+    test_scm_c_scm("int32_t clamp 2^32",
                    Scm_Add(SCM_2_31, SCM_MAKE_INT(-1)),
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, SCM_CLAMP_BOTH, NULL)));
     vv = SCM_2_63;
-    test_scm_c_scm("ScmInt32 clamp 2^63",
+    test_scm_c_scm("int32_t clamp 2^63",
                    Scm_Add(SCM_2_31, SCM_MAKE_INT(-1)),
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, SCM_CLAMP_BOTH, NULL)));
     vv = SCM_2_64;
-    test_scm_c_scm("ScmInt32 clamp 2^64",
+    test_scm_c_scm("int32_t clamp 2^64",
                    Scm_Add(SCM_2_31, SCM_MAKE_INT(-1)),
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, SCM_CLAMP_BOTH, NULL)));
     vv = Scm_Negate(SCM_2_32);
-    test_scm_c_scm("ScmInt32 clamp -2^32",
+    test_scm_c_scm("int32_t clamp -2^32",
                    Scm_Negate(SCM_2_31),
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, SCM_CLAMP_BOTH, NULL)));
     vv = Scm_Negate(SCM_2_63);
-    test_scm_c_scm("ScmInt32 clamp -2^63",
+    test_scm_c_scm("int32_t clamp -2^63",
                    Scm_Negate(SCM_2_31),
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, SCM_CLAMP_BOTH, NULL)));
     vv = Scm_Negate(SCM_2_64);
-    test_scm_c_scm("ScmInt32 clamp -2^64",
+    test_scm_c_scm("int32_t clamp -2^64",
                    Scm_Negate(SCM_2_31),
                    Scm_MakeInteger(Scm_GetInteger32Clamp(vv, SCM_CLAMP_BOTH, NULL)));
 
@@ -638,15 +637,47 @@ void test_32_64(void)
 
     vv = SCM_MAKE_INT(-1);
     Scm_GetIntegerU64Clamp(vv, SCM_CLAMP_NONE, &oor);
-    test_true("ScmUInt64 oor -1", oor);
+    test_true("uint64_t oor -1", oor);
 
     vv = Scm_Add(Scm_Negate(SCM_2_31), SCM_MAKE_INT(1));
     Scm_GetIntegerU64Clamp(vv, SCM_CLAMP_NONE, &oor);
-    test_true("ScmUInt64 oor -2^31+1", oor);
+    test_true("uint64_t oor -2^31+1", oor);
 
     vv = Scm_Add(Scm_Negate(SCM_2_32), SCM_MAKE_INT(1));
     Scm_GetIntegerU64Clamp(vv, SCM_CLAMP_NONE, &oor);
-    test_true("ScmUInt64 oor -2^32+1", oor);
+    test_true("uint64_t oor -2^32+1", oor);
+}
+
+/*=============================================================
+ * Testing rounding to machine-word
+ */
+
+void test_round_ulong_1(u_long expect, ScmObj obj)
+{
+    Scm_Printf(SCM_CUROUT, "testing Scm_GetIntegerUMod(%S), expects %lu =>",
+               obj, expect);
+    u_long r = Scm_GetIntegerUMod(obj);
+    if (expect == r) {
+        Scm_Printf(SCM_CUROUT, "ok\n");
+    } else {
+        Scm_Printf(SCM_CUROUT, "ERROR: got %lu\n", r);
+        errcount++;
+    }
+}
+
+void test_round_ulong()
+{
+    TEST_SECTION("integers modulo ulong range");
+    test_round_ulong_1(142857, SCM_MAKE_INT(142857));
+    test_round_ulong_1(285714, Scm_Add(Scm_Expt(SCM_MAKE_INT(2),
+                                                SCM_MAKE_INT(100)),
+                                       SCM_MAKE_INT(285714)));
+    test_round_ulong_1(UMAX, SCM_MAKE_INT(-1));
+#if SIZEOF_LONG == 4
+    test_round_ulong_1(571428, Scm_Sub(SCM_MAKE_INT(571428), SCM_2_32));
+#else  /*SIZEOF_LONG > 4 */
+    test_round_ulong_1(571428, Scm_Sub(SCM_MAKE_INT(571428), SCM_2_64));
+#endif /*SIZEOF_LONG > 4 */
 }
 
 /*=============================================================
@@ -705,7 +736,7 @@ void test_f16(void)
                 Scm_HalfToDouble(0xfc00));
     z = Scm_HalfToDouble(0xffff);
     test_true("half->double nan", !(z==z));
-    
+
 
     test_half("double->half 0",    0,      Scm_DoubleToHalf(0.0));
     test_half("double->half 1",    0x3c00, Scm_DoubleToHalf(1.0));
@@ -714,7 +745,7 @@ void test_f16(void)
     test_half("double->half -1.5", 0xbe00, Scm_DoubleToHalf(-1.5));
     test_half("double->half 0.75", 0x3a00, Scm_DoubleToHalf(0.75));
     test_half("double->half 0.875",0x3b00, Scm_DoubleToHalf(0.875));
-    
+
     test_half("double->half all 1", 0x3fff, Scm_DoubleToHalf(1.9990234375));
     test_half("double->half max", 0x7bff, Scm_DoubleToHalf(65504.0));
     test_half("double->half normalized min", 0x0400,
@@ -761,12 +792,12 @@ void test_f16(void)
 /*=============================================================
  * main
  */
-int main(int argc, char **argv)
+int main(int argc SCM_UNUSED, char **argv SCM_UNUSED)
 {
     const char *testmsg = "Testing integer arithmetic macros ... ";
 
     Scm_Init(GAUCHE_SIGNATURE);
-    
+
     fprintf(stderr, "%-65s", testmsg);
     message(stdout, testmsg, '=');
 
@@ -782,6 +813,8 @@ int main(int argc, char **argv)
 
     test_32_64();
 
+    test_round_ulong();
+
     test_f16();
 
     if (errcount) {
@@ -793,4 +826,3 @@ int main(int argc, char **argv)
     }
     return 0;
 }
-
