@@ -1,10 +1,8 @@
-/* 
+/*
  * Test VM stack sanity
- * $Id: test-vmstack.c,v 1.8 2008-06-02 01:13:13 shirok Exp $
  */
 
 #include <stdio.h>
-#define GAUCHE_API_0_8_8
 #include "gauche.h"
 #include "gauche/vm.h"
 
@@ -28,7 +26,7 @@ void test_eval(const char *msg, const char *sexp)
     ScmObj x = Scm_ReadFromCString(sexp);
     printf("%s ... ", msg);
     Scm_Eval(x, SCM_UNBOUND, NULL); /* ignore errors */
-        
+
     post_stack = Scm_VM()->sp;
     if (pre_stack != post_stack) {
         printf("ERROR.\n");
@@ -38,12 +36,14 @@ void test_eval(const char *msg, const char *sexp)
     }
 }
 
-ScmObj dummy_eproc(ScmObj *args, int nargs, void *data)
+ScmObj dummy_eproc(ScmObj *args SCM_UNUSED,
+                   int nargs SCM_UNUSED,
+                   void *data SCM_UNUSED)
 {
     return SCM_UNDEFINED;
 }
 
-int main(int argc, char **argv)
+int main(int argc SCM_UNUSED, char **argv SCM_UNUSED)
 {
     ScmObj eproc;
     const char *testmsg = "Testing VM stack sanity... ";
@@ -51,10 +51,10 @@ int main(int argc, char **argv)
     fprintf(stderr, "%-65s", testmsg);
     message(stdout, testmsg, '=');
     Scm_Init(GAUCHE_SIGNATURE);
-    
+
     eproc = Scm_MakeSubr(dummy_eproc, NULL, 0, 1, SCM_FALSE);
-    Scm_VM()->defaultEscapeHandler = eproc;
-    
+    Scm_VM()->customErrorReporter = eproc;
+
     test_eval("simple expression", "(+ 1 2 3)");
     test_eval("with-error-handler (1)",
               "(with-error-handler (lambda (e) #f) (lambda () 1)))");
