@@ -1,7 +1,7 @@
 /*
  * vector.c - vector implementation
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -34,6 +34,11 @@
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/priv/writerP.h"
+#include "gauche/priv/mmapP.h"
+
+#if defined(HAVE_SYS_MMAN_H)
+#include <sys/mman.h>
+#endif
 
 /* Catch integer overflow.
    NB: If total size is too big, GC_malloc aborts.  But we have to prevent
@@ -44,12 +49,15 @@
    (8 or 16 bytes).  If there's not enough heap, then it tries to expand
    the heap by the size rounded up to the pagesize.  We don't want the final
    value overflows signed long.
-   (In reality, expanding heap with close to LONG_MAX surely fails, so it  
+   (In reality, expanding heap with close to LONG_MAX surely fails, so it
    should suffice to avoid overflow before calling GC_MALLOC. But it won't
    harm to have a bit of margin here...)
  */
 static void check_size(ScmSmallInt size, int eltsize)
 {
+    if (size < 0) {
+        Scm_Error("Invalid size: %ld", size);
+    }
     if (size >= (ScmSmallInt)((LONG_MAX - 0x400000)/eltsize)) {
         Scm_Error("Size too big: %ld", size);
     }
@@ -102,10 +110,21 @@ SCM_DEFINE_BUILTIN_CLASS_FLAGS(Scm_VectorClass, vector_print, vector_compare,
 static ScmVector *make_vector(ScmSmallInt size)
 {
     check_size(size, sizeof(ScmObj));
+
+    /* We allocate at least one element for zero-length vector, since
+       ScmVector is declared to have at least one element.
+       Alternatively, we can have a singleton instance for #().
+     */
+    ScmSmallInt alloc_size = (size == 0)? 0 : (size-1);
+
     ScmVector *v = SCM_NEW2(ScmVector *,
-                            sizeof(ScmVector) + sizeof(ScmObj)*(size-1));
+                            sizeof(ScmVector) + sizeof(ScmObj)*alloc_size);
     SCM_SET_CLASS(v, SCM_CLASS_VECTOR);
+#if GAUCHE_API_VERSION >= 98
+    v->size_flags = (size << 1);
+#else
     v->size = size;
+#endif
     return v;
 }
 
@@ -161,19 +180,21 @@ ScmObj Scm_VectorToList(ScmVector *v, ScmSmallInt start, ScmSmallInt end)
 
 ScmObj Scm_VectorRef(ScmVector *vec, ScmSmallInt i, ScmObj fallback)
 {
-    if (i < 0 || i >= vec->size) return fallback;
+    if (i < 0 || i >= SCM_VECTOR_SIZE(vec)) return fallback;
     return vec->elements[i];
 }
 
 ScmObj Scm_VectorSet(ScmVector *vec, ScmSmallInt i, ScmObj obj)
 {
-    if (i >= 0 && i < vec->size) vec->elements[i] = obj;
+    SCM_VECTOR_CHECK_MUTABLE(vec);
+    if (i >= 0 && i < SCM_VECTOR_SIZE(vec)) vec->elements[i] = obj;
     return obj;
 }
 
 ScmObj Scm_VectorFill(ScmVector *vec, ScmObj fill,
                       ScmSmallInt start, ScmSmallInt end)
 {
+    SCM_VECTOR_CHECK_MUTABLE(vec);
     ScmSmallInt len = SCM_VECTOR_SIZE(vec);
     SCM_CHECK_START_END(start, end, len);
     for (ScmSmallInt i=start; i < end; i++) {
@@ -299,8 +320,8 @@ int Scm_UVectorElementSize(ScmClass *klass)
     static const int sizes[] = { 1, 1, 2, 2, 4, 4, 8, 8,
                                  2, sizeof(float), sizeof(double), -1,
                                  sizeof(ScmHalfComplex),
-                                 sizeof(complex float),
-                                 sizeof(complex double),
+                                 sizeof(ScmFloatComplex),
+                                 sizeof(ScmDoubleComplex),
                                  -1 };
     int ind = (int)Scm_UVectorType(klass);
     if (ind >= 0) return sizes[ind];
@@ -567,8 +588,8 @@ DEF_UVCTOR_FILL(F16, ScmHalfFloat)
 DEF_UVCTOR_FILL(F32, float)
 DEF_UVCTOR_FILL(F64, double)
 DEF_UVCTOR_FILL(C32, ScmHalfComplex)
-DEF_UVCTOR_FILL(C64, complex float)
-DEF_UVCTOR_FILL(C128,complex double)
+DEF_UVCTOR_FILL(C64, ScmFloatComplex)
+DEF_UVCTOR_FILL(C128,ScmDoubleComplex)
 
 DEF_UVCTOR_ARRAY(S8,  int8_t)
 DEF_UVCTOR_ARRAY(U8,  uint8_t)
@@ -582,8 +603,8 @@ DEF_UVCTOR_ARRAY(F16, ScmHalfFloat)
 DEF_UVCTOR_ARRAY(F32, float)
 DEF_UVCTOR_ARRAY(F64, double)
 DEF_UVCTOR_ARRAY(C32, ScmHalfComplex)
-DEF_UVCTOR_ARRAY(C64, complex float)
-DEF_UVCTOR_ARRAY(C128,complex double)
+DEF_UVCTOR_ARRAY(C64, ScmFloatComplex)
+DEF_UVCTOR_ARRAY(C128,ScmDoubleComplex)
 
 /*
  * Reader
@@ -612,7 +633,7 @@ ScmObj Scm_ReadUVector(ScmPort *port, const char *tag, ScmReadContext *ctx)
     else Scm_Error("invalid unform vector tag: %s", tag);
 
     ScmObj uv = Scm_ListToUVector(klass, list, 0);
-    
+
     /* If we are reading source file, let literal uvectors be immutable. */
     if (Scm_ReadContextLiteralImmutable(ctx)) {
         SCM_UVECTOR_IMMUTABLE_SET(uv, TRUE);
@@ -632,7 +653,7 @@ static void SCM_CPP_CAT3(print_,tag,vector)(ScmObj obj,                 \
                                             ScmWriteContext *ctx)       \
 {                                                                       \
     const ScmWriteControls *wp =                                        \
-        Scm_GetWriteControls(ctx, out->writeState);                     \
+        Scm_GetWriteControls(ctx, Scm_PortWriteState(out));             \
     Scm_Printf(out, "#"#tag"(");                                        \
     for (int i=0; i<SCM_CPP_CAT3(SCM_,TAG,VECTOR_SIZE)(obj); i++) {     \
         T elt = SCM_CPP_CAT3(SCM_,TAG,VECTOR_ELEMENTS)(obj)[i];         \
@@ -670,7 +691,7 @@ static void SCM_CPP_CAT3(print_,tag,vector)(ScmObj obj,                 \
         Scm_PrintDouble(out, cimag(elt), 0);    \
         Scm_Putz("i", 1, out);                  \
     } while (0)
-    
+
 
 static inline void s64pr(ScmPort *out, int64_t elt)
 {
@@ -711,8 +732,8 @@ DEF_PRINT(F16, f16, ScmHalfFloat, f16pr)
 DEF_PRINT(F32, f32, float, fpr)
 DEF_PRINT(F64, f64, double, fpr)
 DEF_PRINT(C32, c32, ScmHalfComplex, c32pr)
-DEF_PRINT(C64, c64, complex float, c64pr)
-DEF_PRINT(C128, c128, complex double, c128pr)
+DEF_PRINT(C64, c64, ScmFloatComplex, c64pr)
+DEF_PRINT(C128, c128, ScmDoubleComplex, c128pr)
 
 
 /* comparer */
@@ -761,14 +782,14 @@ static inline int c32lt(ScmHalfComplex x, ScmHalfComplex y)
                 && SCM_HALF_FLOAT_CMP(<, x.i, y.i)));
 }
 
-static inline int c64lt(complex float x, complex float y)
+static inline int c64lt(ScmFloatComplex x, ScmFloatComplex y)
 {
     return (crealf(x) < crealf(y)
             || (crealf(x) == crealf(y)
                 && cimagf(x) < cimagf(y)));
 }
 
-static inline int c128lt(complex double x, complex double y)
+static inline int c128lt(ScmDoubleComplex x, ScmDoubleComplex y)
 {
     return (creal(x) < creal(y)
             || (creal(x) == creal(y)
@@ -788,8 +809,196 @@ DEF_CMP(F16, f16, ScmHalfFloat, f16eqv, f16lt)
 DEF_CMP(F32, f32, float, common_eqv, common_lt)
 DEF_CMP(F64, f64, double, common_eqv, common_lt)
 DEF_CMP(C32, c32, ScmHalfComplex, c32eqv, c32lt)
-DEF_CMP(C64, c64, complex float, common_eqv, c64lt)
-DEF_CMP(C128, c128, complex double, common_eqv, c128lt)
+DEF_CMP(C64, c64, ScmFloatComplex, common_eqv, c64lt)
+DEF_CMP(C128, c128, ScmDoubleComplex, common_eqv, c128lt)
+
+/*
+ * Extract mmapped region
+ */
+
+/* Returns a uvector of KLASS with length LEN, that provides a view
+   to the memory region MEM starting from OFFSET.
+   LEN can be negative, in that case byte OFFSET to the end of the memory region
+   is viewed.  If MEM is mapped read-only, the resulting uvector becomes
+   immutable.  Otherwise, the resulting vector's immutablity is
+   defined by IMMUTABLE flag.
+ */
+ScmObj Scm_MakeViewUVector(ScmMemoryRegion *mem, ScmClass *klass,
+                           ScmSmallInt len, ScmSmallInt offset,
+                           int immutable)
+{
+    if (offset < 0) Scm_Error("offset must not be negative: %ld", offset);
+    int esize = Scm_UVectorElementSize(klass);
+    if (esize < 0) Scm_Error("uvector class required, but got: %S", klass);
+    if ((offset % esize) != 0) {
+        Scm_Error("offset %ld is not properly aligned for %S", offset, klass);
+    }
+    if (len < 0) len = (mem->size - offset)/esize;
+
+    if (!(mem->prot & PROT_WRITE)) immutable = TRUE;
+    return Scm_MakeUVectorFull(klass, len, mem->ptr + offset, immutable,
+                               (void*)mem);
+}
+
+/*=====================================================================
+ * Bitvectors
+ */
+
+static void bitvector_write_int(ScmBitvector *v, int prefix, ScmPort *port)
+{
+    if (prefix) Scm_Putz("#*", -1, port);
+    for (int i=0; i<SCM_BITVECTOR_SIZE(v); i++) {
+        if (SCM_BITS_TEST(v->bits, i)) SCM_PUTC('1', port);
+        else SCM_PUTC('0', port);
+    }
+}
+
+static void bitvector_print(ScmObj obj,
+                            ScmPort *port,
+                            ScmWriteContext *ctx SCM_UNUSED)
+{
+    bitvector_write_int(SCM_BITVECTOR(obj), TRUE, port);
+}
+
+static int bitvector_compare(ScmObj x, ScmObj y, int equalp SCM_UNUSED)
+{
+    SCM_ASSERT(SCM_BITVECTORP(x)&&SCM_BITVECTORP(y));
+    ScmBits *bx = SCM_BITVECTOR_BITS(x);
+    ScmBits *by = SCM_BITVECTOR_BITS(y);
+    ScmWord xlen = SCM_BITVECTOR_SIZE(x);
+    ScmWord ylen = SCM_BITVECTOR_SIZE(y);
+
+    /* NB: The ordering is somewhat counterintuitive, for the bits are
+       stored in little-endian.  That is, #*0100 comes after #*1000.
+       This is a lot faster.  Srfi-178 doesn't define the ordering. */
+    if (xlen < ylen) return -1;
+    if (xlen > ylen) return 1;
+    size_t nw = SCM_BITS_NUM_WORDS(xlen);
+    for (size_t i = 0; i < nw; i++, bx++, by++) {
+        if (*bx < *by) return -1;
+        if (*bx > *by) return 1;
+    }
+    return 0;
+}
+
+SCM_DEFINE_BUILTIN_CLASS_FLAGS(Scm_BitvectorClass,
+                               bitvector_print, bitvector_compare,
+                               NULL, NULL, SCM_CLASS_SEQUENCE_CPL,
+                               SCM_CLASS_AGGREGATE);
+
+
+int Scm_Bit2Int(ScmObj bit)
+{
+    if (SCM_EQ(bit, SCM_TRUE) || SCM_EQ(bit, SCM_MAKE_INT(1))) return 1;
+    if (SCM_FALSEP(bit) || SCM_EQ(bit, SCM_MAKE_INT(0))) return 0;
+    Scm_Error("bit value must be 0, 1, #f or #t, but got: %S", bit);
+    return 0;                   /* dummy */
+}
+
+ScmObj Scm_Bit2Bool(ScmObj bit)
+{
+    if (SCM_EQ(bit, SCM_TRUE) || SCM_EQ(bit, SCM_MAKE_INT(1))) return SCM_TRUE;
+    if (SCM_FALSEP(bit) || SCM_EQ(bit, SCM_MAKE_INT(0))) return SCM_FALSE;
+    Scm_Error("bit value must be 0, 1, #f or #t, but got: %S", bit);
+    return SCM_UNDEFINED;       /* dummy */
+}
+
+/* init can be 0, 1, #f or #t. */
+ScmObj Scm_MakeBitvector(ScmSmallInt size, ScmObj init)
+{
+    if (size < 0) {
+        Scm_Error("bitvector size must be a positive integer, but got %d", size);
+    }
+    ScmBitvector *v = SCM_NEW(ScmBitvector);
+    SCM_SET_CLASS(v, SCM_CLASS_BITVECTOR);
+    v->size_flags = (size << 1);
+    v->bits = Scm_MakeBits(size);
+
+    int fill = Scm_Bit2Int(init);
+    Scm_BitsFill(v->bits, 0, size, fill);
+    return SCM_OBJ(v);
+}
+
+ScmObj Scm_ListToBitvector(ScmObj lis)
+{
+    ScmSmallInt len = Scm_Length(lis);
+    if (len < 0) {
+        Scm_Error("proper list required, but got: %S", lis);
+    }
+
+    ScmBitvector *v = SCM_BITVECTOR(Scm_MakeBitvector(len, SCM_FALSE));
+    ScmObj cp;
+    ScmSmallInt i = 0;
+    SCM_FOR_EACH(cp, lis) {
+        if (Scm_Bit2Int(SCM_CAR(cp))) SCM_BITS_SET(v->bits, i);
+        else                          SCM_BITS_RESET(v->bits, i);
+        i++;
+    }
+    return SCM_OBJ(v);
+}
+
+/* Parse string of 0's and 1's to a bitvector.  If PREFIX is true,
+   "#*" prefix is assumed.  Returns #f if unparsable. */
+
+ScmObj Scm_StringToBitvector(ScmString *s, int prefix)
+{
+    const ScmStringBody *b = SCM_STRING_BODY(s);
+    const char *p = SCM_STRING_BODY_START(b);
+    /* multibyte string can't be a bitvector literal. */
+    if (SCM_STRING_BODY_SIZE(b) != SCM_STRING_BODY_LENGTH(b)) return SCM_FALSE;
+    ScmSmallInt len = SCM_STRING_BODY_LENGTH(b);
+
+    if (prefix) {
+        if (len < 2) return SCM_FALSE;
+        if (strncmp(p, "#*", 2) != 0) return SCM_FALSE;
+        p += 2;
+        len -= 2;
+    }
+
+    ScmBitvector *v = SCM_BITVECTOR(Scm_MakeBitvector(len, SCM_FALSE));
+    int i = 0;
+    for (; i < len; p++, i++) {
+        if (*p == '0')      SCM_BITS_RESET(v->bits, i);
+        else if (*p == '1') SCM_BITS_SET(v->bits, i);
+        else return SCM_FALSE;
+    }
+    return SCM_OBJ(v);
+}
+
+ScmObj Scm_BitvectorToString(ScmBitvector *v, int prefix)
+{
+    ScmObj out = Scm_MakeOutputStringPort(TRUE);
+    bitvector_write_int(v, prefix, SCM_PORT(out));
+    return Scm_GetOutputString(SCM_PORT(out), 0);
+}
+
+ScmObj Scm_BitvectorCopy(ScmBitvector *v, ScmSmallInt start, ScmSmallInt end)
+{
+    ScmSmallInt size = SCM_BITVECTOR_SIZE(v);
+    SCM_CHECK_START_END(start, end, size);
+    ScmBitvector *vv = SCM_BITVECTOR(Scm_MakeBitvector(end-start, SCM_FALSE));
+    Scm_BitsCopyX(vv->bits, 0, v->bits, start, end);
+    return SCM_OBJ(vv);
+}
+
+ScmObj Scm_BitvectorCopyX(ScmBitvector *dest, ScmSmallInt dstart,
+                          ScmBitvector *src,
+                          ScmSmallInt sstart, ScmSmallInt send)
+{
+    SCM_BITVECTOR_CHECK_MUTABLE(dest);
+    ScmSmallInt ssize = SCM_BITVECTOR_SIZE(src);
+    SCM_CHECK_START_END(sstart, send, ssize);
+    ScmSmallInt dsize = SCM_BITVECTOR_SIZE(dest);
+    ScmSmallInt dend = dstart + send - sstart;
+    if (dstart > dsize || dstart < 0 || dend > dsize) {
+        Scm_Error("destination index out of range (size=%ld, start=%ld, end=%ld)",
+                  dsize, dstart, dend);
+    }
+
+    Scm_BitsCopyX(dest->bits, dstart, src->bits, sstart, send);
+    return SCM_OBJ(dest);
+}
+
 
 /*=====================================================================
  * Utility
