@@ -1,7 +1,7 @@
 ;;;
 ;;; vminsn.scm - Virtual machine instruction definition
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -82,6 +82,7 @@
 ;;;                         of having LREF0-SOMETHING or LREF21-SOMETHING
 ;;;                         separately, we'll have LREF-SOMETHING(0,0) and
 ;;;                         LREF-SOMETHING(2,1), respectively.
+;;;           :multi-value - Can result multiple values.
 
 ;;;==============================================================
 ;;; Common Cise macros
@@ -450,7 +451,8 @@
     ($include "./vmcall.c")
     (label tail_apply_entry)
     ($define APPLY_CALL)
-    ($include "./vmcall.c")))
+    ($include "./vmcall.c"))
+  :multi-value)
 
 ;; TAIL-CALL(nargs)
 ;;  Call procedure in val0.  Same as CALL except this discards the
@@ -465,7 +467,8 @@
         (set! (-> ct top) (logand (+ (-> ct top) 1)
                                   (- (-> ct size) 1)))))
     (DISCARD-ENV)
-    ($goto-insn CALL)))
+    ($goto-insn CALL))
+  :multi-value)
 
 ;; JUMP <addr>
 ;;  Jump to <addr>.
@@ -477,7 +480,8 @@
 ;;  Pop the continuation stack.
 ;;
 (define-insn RET       0 none #f
-  (begin (RETURN-OP) CHECK-INTR NEXT))
+  (begin (RETURN-OP) CHECK-INTR NEXT)
+  :multi-value)
 
 ;; DEFINE(flag) <symbol>
 ;;  Defines global binding of SYMBOL in the current module.
@@ -599,7 +603,8 @@
     (CHECK-STACK (-> vm base maxstack))
     CHECK-INTR
     (SCM_PROF_COUNT_CALL vm (SCM_OBJ (-> vm base)))
-    NEXT))
+    NEXT)
+  :multi-value)
 
 (define-insn LOCAL-ENV-TAIL-CALL 1 none #f
   (let* ([nargs::int (cast int (- SP ARGP))] [to::ScmObj*])
@@ -613,7 +618,8 @@
           (set! (* (post++ t)) (* (post++ a))))))
     (set! ARGP to)
     (set! SP (+ to nargs))
-    ($goto-insn LOCAL-ENV-CALL)))
+    ($goto-insn LOCAL-ENV-CALL))
+  :multi-value)
 
 ;; BF <else-offset>          ; branch if VAL0 is false
 ;; BT <else-offset>          ; branch if VAL0 is true
@@ -774,7 +780,8 @@
       (for [() (> nvals 1) (post-- nvals)]
            (POP-ARG (aref (-> vm vals) (- nvals 1))))
       (POP-ARG VAL0)
-      NEXT)))
+      NEXT))
+  :multi-value)
 
 ;; LSET(depth, offset)
 ;;  Local set
@@ -827,7 +834,7 @@
 ;;
 (define-insn LREF        2 none #f
   ($lref (SCM_VM_INSN_ARG0 code)   ; depth
-         (SCM_VM_INSN_ARG1 code))) ; offset 
+         (SCM_VM_INSN_ARG1 code))) ; offset
 
 ;; Shortcut for the frequent depth/offset.
 ;; From statistics, we found that the following depth/offset combinations
@@ -906,7 +913,8 @@
                   (aref (-> vm vals) (- SCM_VM_MAX_VALUES 1)))
         (break))
       (PUSH-ARG (aref (-> vm vals) i)))
-    ($goto-insn TAIL-CALL)))
+    ($goto-insn TAIL-CALL))
+  :multi-value)
 
 ;; Inlined operators
 ;;  They work the same as corresponding Scheme primitives, but they are
@@ -1015,7 +1023,8 @@
     (POP-ARG VAL0)                      ; get proc
     (TAIL-CALL-INSTRUCTION)
     (set! VAL0 (Scm_VMApply VAL0 cp))
-    NEXT))
+    NEXT)
+  :multi-value)
 
 (define-insn TAIL-APPLY  1 none #f
   ;; Inlined apply.  Assumes the call is at the tail position.
@@ -1033,7 +1042,7 @@
   ;;
   ;; where N = SCMVM_INSN_ARG(code)-2.
   ;; rest contains a list of "all the rest arguments".
-  ;; We just push VAL0 onto stack, and move ARGP 
+  ;; We just push VAL0 onto stack, and move ARGP
   ;; We "rotate" the stack and VAL0 and jump to the
   ;; tail_apply_entry.  (The unfolding of rest
   ;; argument will be done in ADJUST_ARGUMENT_FRAME later,
@@ -1044,21 +1053,26 @@
   ;;        | argN |
   ;;        |   :  |
   ;;   ARGP>| arg0 |        VAL0=proc
-  ;;        | proc |< original ARGP postiion (proc unused) 
+  ;;        | proc |< original ARGP postiion (proc unused)
   (let* ([rest VAL0]
-         [rargc::int (check_arglist_tail_for_apply vm rest)]
          [nargc::int (- (SCM_VM_INSN_ARG code) 2)]
          [proc (* (- SP nargc 1))])
-    (when (< rargc 0) ($vm-err "improper list not allowed: %S" rest))
     (set! VAL0 proc)
     (post++ ARGP)
-    ;; a micro-optimization: if VAL0 is (), we just omit it and
+    ;; a micro-optimization: if the rest arg is (), we just omit it and
     ;; pretend this is a normal TAIL-CALL.
-    (when (== rargc 0) ($goto-insn TAIL-CALL))
+    (when (SCM_NULLP rest) ($goto-insn TAIL-CALL))
+    ;; Checking if rest argument is proper is done after tail_apply_entry.
+    ;; Here we just ensure that rest is a normal pair (If rest is a lazy
+    ;; pair, it is forced by SCM_PAIRP).
+    (unless (SCM_PAIRP rest)
+      ($vm-err "Rest argument is not proper: %S" rest))
+    (set! VAL0 proc)  ;VAL0 may be broken by lazy pair evaluation
     ;; normal path
     (set! (* (post++ SP)) rest)
     (DISCARD-ENV)
-    (goto tail_apply_entry)))
+    (goto tail_apply_entry))
+  :multi-value)
 
 (define-insn IS-A        0 none #f      ; is-a?
   ($w/argp obj
@@ -1077,14 +1091,14 @@
 (define-insn STRINGP     0 none #f ($w/argr v ($result:b (SCM_STRINGP v))))
 (define-insn SYMBOLP     0 none #f ($w/argr v ($result:b (SCM_SYMBOLP v))))
 (define-insn VECTORP     0 none #f ($w/argr v ($result:b (SCM_VECTORP v))))
-(define-insn IDENTIFIERP 0 none #f ($w/argr v ($result:b (SCM_IDENTIFIERP v))))
 (define-insn NUMBERP     0 none #f ($w/argr v ($result:b (SCM_NUMBERP v))))
 (define-insn REALP       0 none #f ($w/argr v ($result:b (SCM_REALP v))))
+(define-insn IDENTIFIERP 0 none #f ($w/argr v ($result:b (or (SCM_SYMBOLP v) (SCM_IDENTIFIERP v)))))
 
 (define-insn SETTER      0 none #f ($w/argr v ($result (Scm_Setter v))))
 
-(define-insn VALUES      1 none #f (begin ($values) NEXT))
-(define-insn VALUES-RET  1 none (VALUES RET) (begin ($values) (RETURN-OP) NEXT))
+(define-insn VALUES      1 none #f (begin ($values) NEXT) :multi-value)
+(define-insn VALUES-RET  1 none (VALUES RET) (begin ($values) (RETURN-OP) NEXT) :multi-value)
 
 (define-insn VEC         1 none #f      ; vector
   (let* ([nargs::int (SCM_VM_INSN_ARG code)]
@@ -1137,6 +1151,8 @@
     (POP-ARG vec)
     ($type-check vec SCM_VECTORP "vector")
     ($type-check ind SCM_INTP "fixnum")
+    (when (SCM_VECTOR_IMMUTABLE_P vec)
+      ($vm-err "vector is immutable: %S" vec))
     (let* ([k::int (SCM_INT_VALUE ind)] [v VAL0])
       (when (or (< k 0) (>= k (SCM_VECTOR_SIZE vec)))
         ($vm-err "vector-set! index out of range: %d" k))
@@ -1156,6 +1172,8 @@
 (define-insn VEC-SETI    1 none #f
   ($w/argp vec
     ($type-check vec SCM_VECTORP "vector")
+    (when (SCM_VECTOR_IMMUTABLE_P vec)
+      ($vm-err "vector is immutable: %S" vec))
     (let* ([k::int (SCM_VM_INSN_ARG code)] [v VAL0])
       (when (or (< k 0) (>= k (SCM_VECTOR_SIZE vec)))
         ($vm-err "vector-set! index out of range: %d" k))
@@ -1235,7 +1253,7 @@
              ($result:f (* (Scm_GetDouble arg) (Scm_GetDouble VAL0)))]
             [else ($result (Scm_Mul arg VAL0))])
       (if (SCM_FLONUMP VAL0)
-        (cond [(and (== (SCM_MAKE_INT 0) arg) 
+        (cond [(and (== (SCM_MAKE_INT 0) arg)
                     (not (SCM_IS_INF (SCM_FLONUM_VALUE VAL0)))
                     (not (SCM_IS_NAN (SCM_FLONUM_VALUE VAL0))))
                ($result (SCM_MAKE_INT 0))]
@@ -1406,17 +1424,34 @@
 ;; Dynamic handlers
 ;;
 
-;; PUSH-HANDLERS
+;; PUSH-HANDLERS(n)
 ;; POP-HANDLERS
 ;;   Used for dynamic-wind and alike.
+;;   Push-handlers takes two procedures (before proc in SP top, and
+;;   after proc in VAL0), and zero or more args.  The number of args
+;;   is in parameter.
 
-(define-insn PUSH-HANDLERS 0 none #f    ; push dynamic handlers
-  (let* ((before) (after VAL0))
-    (VM-ASSERT (>= (- SP (-> vm stackBase)) 1))
+(define-insn PUSH-HANDLERS 1 none #f    ; push dynamic handlers
+  (let* ([nargs::long (SCM_VM_INSN_ARG code)]
+         [before]
+         [after VAL0]
+         [args SCM_NIL])
+    (VM-ASSERT (>= (- SP (-> vm stackBase)) (+ 1 nargs)))
+    (when (> nargs 0)
+      (SCM_FLONUM_ENSURE_MEM VAL0)
+      (set! args (Scm_Cons VAL0 SCM_NIL))
+      (while (> (pre-- nargs) 0)
+        (let* ([v])
+          (POP-ARG v)
+          (SCM_FLONUM_ENSURE_MEM v)
+          (set! args (Scm_Cons v args))))
+      (POP-ARG after))
     (POP-ARG before)
     (SCM_FLONUM_ENSURE_MEM before)
     (SCM_FLONUM_ENSURE_MEM after)
-    (set! (-> vm handlers) (Scm_Acons before after (-> vm handlers)))
+    (set! (-> vm handlers)
+          (Scm_Cons (make_handler_entry before after args)
+                    (-> vm handlers)))
     NEXT))
 
 (define-insn POP-HANDLERS 0 none #f     ; pop dynamic handlers
