@@ -1,7 +1,7 @@
 /*
  * write.c - writer
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,12 +33,12 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/class.h"
 #include "gauche/writer.h"
 #include "gauche/priv/builtin-syms.h"
 #include "gauche/priv/macroP.h" /* PVREF stuff.  Will go in future */
 #include "gauche/priv/writerP.h"
 #include "gauche/priv/portP.h"
+#include "gauche/priv/stringP.h"
 #include "gauche/char_attr.h"
 
 #include <ctype.h>
@@ -166,9 +166,9 @@ ScmWriteState *Scm_MakeWriteState(ScmWriteState *proto SCM_UNUSED)
 static void cleanup_port_write_state(ScmPort *port)
 {
     port->flags &= ~(SCM_PORT_WALKING|SCM_PORT_WRITESS);
-    if (port->writeState != NULL) {
-        ScmWriteState *s = port->writeState;
-        port->writeState = NULL;
+    ScmWriteState *s = Scm_PortWriteState(port);
+    if (s != NULL) {
+        Scm_PortWriteStateSet(port, NULL);
         /* The table for recursive/shared detection should be GC'ed after
            we drop the reference to it.  However, the table can be quite big
            after doing write-shared on a large graph, and our implementation of
@@ -283,7 +283,7 @@ int Scm_WriteLimited(ScmObj obj, ScmObj p, int mode, int width)
     }
 
     ScmObj out = Scm_MakeOutputStringPort(TRUE);
-    SCM_PORT(out)->writeState = SCM_PORT(port)->writeState;
+    Scm_PortWriteStateSet(SCM_PORT(out), Scm_PortWriteState(port));
     ScmWriteContext ctx;
     write_context_init(&ctx, mode, 0, width);
 
@@ -297,7 +297,7 @@ int Scm_WriteLimited(ScmObj obj, ScmObj p, int mode, int width)
     } else {
         write_rec(obj, SCM_PORT(out), &ctx);
     }
-    
+
     ScmString *str = SCM_STRING(Scm_GetOutputString(SCM_PORT(out), 0));
     int nc = SCM_STRING_BODY_LENGTH(SCM_STRING_BODY(str));
     if (nc > width) {
@@ -337,7 +337,7 @@ static void write_general(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
    write-object.   We can't use VMApply here since this function can be
    called deep in the recursive stack of Scm_Write, so the control
    may not return to VM immediately. */
-static void write_object(ScmObj obj, ScmPort *port, 
+static void write_object(ScmObj obj, ScmPort *port,
                          ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_ApplyRec(SCM_OBJ(&Scm_GenericWriteObject),
@@ -386,7 +386,7 @@ static size_t write_char(ScmChar ch, ScmPort *port, ScmWriteContext *ctx)
 
         Scm_PutzUnsafe("#\\", -1, port);
         if (ch <= 0x20)       cname = char_names[ch];
-        else if (ch == 0x7f)  cname = "del";
+        else if (ch == 0x7f)  cname = "delete";
         else {
             switch (Scm_CharGeneralCategory(ch)) {
             case SCM_CHAR_CATEGORY_Mn:
@@ -427,8 +427,9 @@ static size_t write_char(ScmChar ch, ScmPort *port, ScmWriteContext *ctx)
  */
 ScmObj Scm__WritePrimitive(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    const ScmWriteControls *wp = Scm_GetWriteControls(ctx, port->writeState);
-    
+    const ScmWriteControls *wp =
+        Scm_GetWriteControls(ctx, Scm_PortWriteState(port));
+
 #define CASE_ITAG_RET(obj, str)                 \
     case SCM_ITAG(obj):                         \
         Scm_PutzUnsafe(str, -1, port);          \
@@ -476,6 +477,13 @@ ScmObj Scm__WritePrimitive(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
         Scm_PutzUnsafe(buf, -1, port);
         return SCM_MAKE_INT(k);
     }
+    else if (SCM_STRING_CURSOR_SMALL_P(obj)) {
+        char buf[SPBUFSIZ];
+        int k = snprintf(buf, SPBUFSIZ, "#<string-cursor %ld>",
+                         SCM_STRING_CURSOR_SMALL_OFFSET(obj));
+        Scm_PutzUnsafe(buf, -1, port);
+        return SCM_MAKE_INT(k);
+    }
     return SCM_FALSE;
 }
 
@@ -532,8 +540,9 @@ ScmObj Scm__WritePrimitive(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 static void write_walk(ScmObj obj, ScmPort *port)
 {
     static ScmObj proc = SCM_UNDEFINED;
-    SCM_ASSERT(port->writeState);
-    ScmHashTable *ht = port->writeState->sharedTable;
+    ScmWriteState *s = Scm_PortWriteState(port);
+    SCM_ASSERT(s);
+    ScmHashTable *ht = s->sharedTable;
     SCM_ASSERT(ht != NULL);
     SCM_BIND_PROC(proc, "%write-walk-rec", Scm_GaucheInternalModule());
     Scm_ApplyRec3(proc, obj, SCM_OBJ(port), SCM_OBJ(ht));
@@ -555,7 +564,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
     char numbuf[50];  /* enough to contain long number */
     ScmObj stack = SCM_NIL;
-    ScmWriteState *st = port->writeState;
+    ScmWriteState *st = Scm_PortWriteState(port);
     ScmHashTable *ht = (st? st->sharedTable : NULL);
     const ScmWriteControls *wp = Scm_GetWriteControls(ctx, st);
     int stack_depth = 0;        /* only used when !ht */
@@ -584,12 +593,12 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
             }                                                           \
         }                                                               \
     } while (0)
-    
+
 
     for (;;) {
     write1:
         if (ctx->flags & WRITE_LIMITED) {
-            if (port->src.ostr.length >= ctx->limit) return;
+            if (PORT_OSTR(port)->length >= ctx->limit) return;
         }
 
         /* number may be heap allocated, but we don't use srfi-38 notation. */
@@ -617,7 +626,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                 goto next;
             } else if (k > 1) {
                 /* This object will be seen again. Put a reference tag. */
-                ScmWriteState *s = port->writeState;
+                ScmWriteState *s = Scm_PortWriteState(port);
                 snprintf(numbuf, 50, "#%d=", s->sharedCounter);
                 Scm_HashTableSet(ht, obj, SCM_MAKE_INT(-s->sharedCounter), 0);
                 s->sharedCounter++;
@@ -710,7 +719,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                 } else {
                     Scm_PutcUnsafe(' ', port);
                     obj = SCM_VECTOR_ELEMENT(v, i);
-                    SCM_SET_CAR(top, SCM_MAKE_INT(i+1));
+                    SCM_SET_CAR_UNCHECKED(top, SCM_MAKE_INT(i+1));
                     goto write1;
                 }
             } else {
@@ -736,8 +745,9 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                         POP();
                     } else {
                         obj = v;
-                        SCM_SET_CAR(SCM_CDR(top), SCM_MAKE_INT(count+1));
-                        SCM_SET_CDR(SCM_CDR(top), SCM_NIL);
+                        SCM_SET_CAR_UNCHECKED(SCM_CDR(top),
+                                              SCM_MAKE_INT(count+1));
+                        SCM_SET_CDR_UNCHECKED(SCM_CDR(top), SCM_NIL);
                         goto write1;
                     }
                 } else if (wp->printLength >= 0 && wp->printLength <= count) {
@@ -748,14 +758,14 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                     /* cdr part is shared */
                     Scm_PutzUnsafe(" . ", -1, port);
                     obj = v;
-                    SCM_SET_CAR(SCM_CDR(top), SCM_MAKE_INT(count+1));
-                    SCM_SET_CDR(SCM_CDR(top), SCM_NIL);
+                    SCM_SET_CAR_UNCHECKED(SCM_CDR(top), SCM_MAKE_INT(count+1));
+                    SCM_SET_CDR_UNCHECKED(SCM_CDR(top), SCM_NIL);
                     goto write1;
                 } else {
                     Scm_PutcUnsafe(' ', port);
                     obj = SCM_CAR(v);
-                    SCM_SET_CAR(SCM_CDR(top), SCM_MAKE_INT(count+1));
-                    SCM_SET_CDR(SCM_CDR(top), SCM_CDR(v));
+                    SCM_SET_CAR_UNCHECKED(SCM_CDR(top), SCM_MAKE_INT(count+1));
+                    SCM_SET_CDR_UNCHECKED(SCM_CDR(top), SCM_CDR(v));
                     goto write1;
                 }
             }
@@ -775,7 +785,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
    by the caller even if we throw an error during write. */
 static void write_ss(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    SCM_ASSERT(port->writeState == NULL);
+    SCM_ASSERT(Scm_PortWriteState(port) == NULL);
 
     /* pass 1 */
     port->flags |= SCM_PORT_WALKING;
@@ -783,7 +793,7 @@ static void write_ss(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
     ScmWriteState *s = Scm_MakeWriteState(NULL);
     s->sharedTable = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
     s->controls = ctx->controls;
-    port->writeState = s;
+    Scm_PortWriteStateSet(port, s);
 
     write_walk(obj, port);
     port->flags &= ~(SCM_PORT_WALKING|SCM_PORT_WRITESS);
@@ -1112,7 +1122,7 @@ static void vprintf_pass2(ScmPort *out, const char *fmt, ScmObj args)
 
 /* Public APIs */
 
-void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap, 
+void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap,
                  int sharedp SCM_UNUSED)
 {
     if (!SCM_OPORTP(out)) Scm_Error("output port required, but got %S", out);
