@@ -412,7 +412,7 @@
 (use data.heap)
 (test-module 'data.heap)
 
-(use srfi-1)
+(use scheme.list)
 (use srfi-27)
 (use gauche.sequence)
 (use util.match)
@@ -470,7 +470,7 @@
                                  (list x y)))
                            (iota (quotient len 2)))))
     )
-  
+
   (do-test (iota 15) default-comparator)
   (do-test (iota 253) default-comparator)
   (do-test (reverse (iota 33))
@@ -496,7 +496,7 @@
                  (remove pred lis)
                  (delete item lis))
            (let ([heap (build-binary-heap (list->vector (shuffle lis rs)))])
-             (list (if-let1 r (binary-heap-find heap pred)
+             (list (if-let1 r (binary-heap-find pred heap)
                      (pred r)
                      #f)
                    (let1 h (binary-heap-copy heap)
@@ -519,6 +519,18 @@
   (do-scan (iota 42) (^n (< (modulo n 3) 2)) 91)
   )
 
+;; check binary-heap-find old & new API
+(let ([h (build-binary-heap (vector 0 8 2 16 11 24 4 10))])
+  (test* "binary-heap-find (new)" 11
+         (binary-heap-find odd? h))
+  (test* "binary-heap-find (new)" #f
+         (binary-heap-find (cut > <> 100) h))
+  (test* "binary-heap-find (new)" 'oops
+         (binary-heap-find (cut > <> 100) h (constantly 'oops)))
+  (test* "binary-heap-find (old)" '11
+         (binary-heap-find h odd?)))
+
+
 (let ()
   (define (test-swap source actions)
     ;; actions : ((min|max item expected-result expected-min expected-max) ...)
@@ -582,11 +594,15 @@
     (test* "get" (map cdr data)
            (map (^p (dict-get m (car p))) data))
     (test* "min" '((e) . -59)
-           (priority-map-min m)
+           (priority-map-min-all m)
            compare-minmax-result)
+    (test* "min" '(e . -59)
+           (priority-map-min m))
     (test* "max" '((c g) . 9)
-           (priority-map-max m)
+           (priority-map-max-all m)
            compare-minmax-result)
+    (test* "max" (test-one-of '(g . 9) '(c . 9))
+           (priority-map-max m))
     (test* "delete" (remove (^p (eq? (car p) 'c)) data)
            (begin (dict-delete! m 'c)
                   (sort-by (dict->alist m) car))))
@@ -610,301 +626,250 @@
   )
 
 ;;;========================================================================
-;; ring-buffer
-(test-section "data.ring-buffer")
-(use data.ring-buffer)
-(test-module 'data.ring-buffer)
-(use gauche.uvector)
-(use srfi-1)
+;; range
+(test-section "data.range")
+(use data.range)
+(test-module 'data.range)
 
+;; srfi-196 is a subset of data.range, and we test it here.
 
-(define (test-ring-buffer initial-storage)
-  (define c (class-name (class-of initial-storage)))
-  (define (remove-all-from-front rb)
-    (unfold ring-buffer-empty?
-            ring-buffer-remove-front!
-            identity
-            rb))
-  (define (remove-all-from-back rb)
-    (unfold ring-buffer-empty?
-            ring-buffer-remove-back!
-            identity
-            rb))
-  (define (test-add-remove n)
-    (let1 rb (make-ring-buffer initial-storage)
-      (test* #"add front - remove front (~c, ~n))"
-             (reverse (iota n))
-             (begin
-               (dotimes (i n) (ring-buffer-add-front! rb i))
-               (remove-all-from-front rb))))
-    (let1 rb (make-ring-buffer initial-storage)
-      (test* #"add front - remove back (~c, ~n))"
-             (iota n)
-             (begin
-               (dotimes (i n) (ring-buffer-add-front! rb i))
-               (remove-all-from-back rb))))
-    (let1 rb (make-ring-buffer initial-storage)
-      (test* #"add back - remove front (~c, ~n))"
-             (iota n)
-             (begin
-               (dotimes (i n) (ring-buffer-add-back! rb i))
-               (remove-all-from-front rb))))
-    (let1 rb (make-ring-buffer initial-storage)
-      (test* #"add back - remove back (~c, ~n))"
-             (reverse (iota n))
-             (begin
-               (dotimes (i n) (ring-buffer-add-back! rb i))
-               (remove-all-from-back rb)))))
+(define-module srfi-196-tests
+  (define-syntax import (syntax-rules () [(_ . x) #f]))
+  (use scheme.list)
+  (use srfi-78)                         ; check
+  (use srfi-158)                        ; generator->list
+  (use srfi-196)
+  (include "include/srfi-196-test"))
 
-  (define (test-basic)
-    (let1 rb (make-ring-buffer initial-storage)
-      (test* #"empty (~c)" #t (ring-buffer-empty? rb))
-      (test* #"empty (~c)" 0  (ring-buffer-num-entries rb))
-      (test* #"initial (~c)" '(10 10 1 #f)
-             (begin (ring-buffer-add-front! rb 10)
-                    (list (ring-buffer-front rb)
-                          (ring-buffer-back rb)
-                          (ring-buffer-num-entries rb)
-                          (ring-buffer-empty? rb))))
-      (test* #"2 elemenst (~c)" '(10 11 2 #f)
-             (begin (ring-buffer-add-back! rb 11)
-                    (list (ring-buffer-front rb)
-                          (ring-buffer-back rb)
-                          (ring-buffer-num-entries rb)
-                          (ring-buffer-empty? rb))))))
-  (define (test-overflow)
-    (let1 rb (make-ring-buffer initial-storage :overflow-handler 'error)
-      (test* #"overflow - error (~c)" (test-error)
-             (dotimes [i (+ (size-of initial-storage) 1)]
-               (ring-buffer-add-back! rb i))))
-    (let1 rb (make-ring-buffer initial-storage :overflow-handler 'overwrite)
-      (test* #"overflow - overwrite forward (~c)"
-             (iota (size-of initial-storage)
-                   (size-of initial-storage))
-             (begin
-               (dotimes [i (* (size-of initial-storage) 2)]
-                 (ring-buffer-add-back! rb i))
-               (remove-all-from-front rb))))
-    (let1 rb (make-ring-buffer initial-storage :overflow-handler 'overwrite)
-      (test* #"overflow - overwrite backward (~c)"
-             (reverse (iota (size-of initial-storage)
-                            (size-of initial-storage)))
-             (begin
-               (dotimes [i (* (size-of initial-storage) 2)]
-                 (ring-buffer-add-front! rb i))
-               (remove-all-from-front rb)))))
+;; gauche extensions
 
-  ;; body of test-ring-buffer
-  (test-basic)
-  (test-overflow)
-  (test-add-remove 3)
-  (test-add-remove 12) ; causes realloc
+(test* "range-ref out of range" (test-error <error> #/Index out of range/)
+       (range-ref (iota-range 10) 11))
+(test* "range-ref fallback" 'oops
+       (range-ref (iota-range 10) 11 'oops))
+(test* "range-ref fallback" 'oops
+       (range-ref (iota-range 10) -1 'oops))
+(test* "range-first fallback" 'wow
+       (range-first (iota-range 0) 'wow))
+(test* "range-last fallback" 'boo
+       (range-last (iota-range 0) 'boo))
+
+(test* "range-reverse start/end" '(5 4 3)
+       (range->list (range-reverse (iota-range 7) 3 6)))
+(test* "range-reverse start/end nested" '(5 4 3)
+       (range->list (range-reverse (subrange (iota-range 10) 1 9) 2 5)))
+(test* "range-reverse start/end nested" '(6 5 4)
+       (range->list (range-reverse
+                     (subrange (subrange (iota-range 10) 1 9) 1 6)
+                     2 5)))
+
+(test* "vector-range start/end" '(c d e f g)
+       (range->list (vector-range '#(a b c d e f g) 2)))
+(test* "vector-range start/end" '(c d e)
+       (range->list (vector-range '#(a b c d e f g) 2 5)))
+(test* "uvector-range" '(0 1 2 3 4)
+       (range->list (uvector-range '#u8(0 1 2 3 4))))
+(test* "uvector-range start/end" '(1 2 3 4)
+       (range->list (uvector-range '#u8(0 1 2 3 4 5) 1 5)))
+(test* "string-range start/end" '(#\c #\d #\e #\f #\g)
+       (range->list (string-range "abcdefg" 2)))
+(test* "string-range start/end" '(#\c #\d #\e)
+       (range->list (string-range "abcdefg" 2 5)))
+
+;; sequence protocol
+(test* "range as a sequence (ref)" 3 (~ (numeric-range 1 6) 2))
+(test* "range as a sequence (iterate)" '(1 2 3 4 5)
+       (coerce-to <list> (numeric-range 1 6)))
+(test* "range as a sequence (build from list)" '(a b c d e)
+       (range->list (coerce-to <range> '(a b c d e))))
+(test* "range as a sequence (build from vector)" '(a b c d e)
+       (range->list (coerce-to <range> '#(a b c d e))))
+(test* "range as a sequence (subseq)" '(3 4 5 6 7)
+       (range->list (subseq (numeric-range 0 10) 3 8)))
+
+;; srfi-42 extension
+(define-module range-srfi-42-test
+  (use gauche.test)
+  (use data.range)
+  (use srfi-42)
+  (test* "srfi-42 :range" '((a 0) (b 1) (c 2) (d 3) (e 4))
+         (list-ec (: v (index k) (vector-range '#(a b c d e)))
+                  (list v k)))
   )
-                  
-(test-ring-buffer (make-vector 4))
-(test-ring-buffer (make-u8vector 5))
 
 ;;;========================================================================
-;; trie
-(test-section "data.trie")
-(use data.trie)
-(test-module 'data.trie)
-(use gauche.uvector)
-(use srfi-1)
-(use srfi-13)
+;; skew-list
+(test-section "data.skew-list")
+(use data.skew-list)
+(use gauche.sequence)
+(use gauche.generator)
+(test-module 'data.skew-list)
 
-(let* ((strs '("kana" "kanaono" "kanawai" "kanawai koa"
-               "kanawai mele" "kane" "Kane" "kane make" "kane makua"
-               "ku" "kua" "kua`aina" "kua`ana"
-               "liliko`i" "lilinoe" "lili`u" "lilo" "maoli" ""))
-       (lists (map string->list strs))
-       (vecs  (map list->vector lists))
-       (uvecs (map string->u8vector strs)))
+(test* "skew-list?" #f (skew-list? '()))
+(test* "skew-list?" #f (skew-list? '(a b)))
+(test* "skew-list?" #t (skew-list? (list->skew-list '(a b))))
 
-  ;; string trie tests
-  (let1 t1 (make-trie)
-    (test* "trie: constructor" '(#t 0)
-           (list (trie? t1) (trie-num-entries t1)))
-    (test* "trie: exists?" #f (trie-exists? t1 "kane"))
+(dotimes [n 30]
+  (let1 data (iota n)
+    (test* "list -> skew-list ->list" data
+           (skew-list->list (list->skew-list data)))))
 
-    (test* "trie: put!" 1
-           (begin (trie-put! t1 "lilo" 4)
-                  (trie-num-entries t1)))
-    (test* "trie: get" 4
-           (trie-get t1 "lilo"))
-    (test* "trie: get (error)" (test-error)
-           (trie-get t1 "LILO"))
-    (test* "trie: get (fallback)" 'foo
-           (trie-get t1 "LILO" 'foo))
+(test* "skew-list-cons" 'a
+       (skew-list-car (skew-list-cons 'a skew-list-null)))
+(test* "skew-list-cons/car/cdr" '((z a b c d e) a (b c d e))
+       (let1 sl (list->skew-list '(a b c d e))
+         (list (skew-list->list (skew-list-cons 'z sl))
+               (skew-list-car sl)
+               (skew-list->list (skew-list-cdr sl)))))
+(test* "skew-list-cons (error)" (test-error)
+       (skew-list-cons 'a 'b))
+(test* "skew-list-car (error)" (test-error)
+       (skew-list-car skew-list-null))
+(test* "skew-list-cdr (error)" (test-error)
+       (skew-list-cdr skew-list-null))
 
-    (test* "trie: put! more" (length strs)
-           (begin (for-each (lambda (s)
-                              (trie-put! t1 s (string-length s)))
-                            strs)
-                  (trie-num-entries t1)))
-    (test* "trie: get more" #t
-           (every (lambda (s)
-                    (= (trie-get t1 s) (string-length s)))
-                  strs))
-    (test* "trie: exists? more" #t
-           (every (cut trie-exists? t1 <>) strs))
-    (test* "trie: exists? on partial key" #f
-           (every (cut trie-exists? t1 <>) '("k" "ka" "kan")))
-    (test* "trie: partial-key?"
-           '(("k" #t) ("kana" #t) ("kanawai koa" #f) ("" #t) ("zzz" #f))
-           (map (^k (list k (trie-partial-key? t1 k)))
-                '("k" "kana" "kanawai koa" "" "zzz")))
-    (test* "trie: longest match" '(("kana" . 4)
-                                   ("kana" . 4)
-                                   ("kanawai" . 7)
-                                   ("" . 0))
-           (map (^k (trie-longest-match t1 k #f))
-                '("kana" "kanaoka" "kanawai pele" "mahalo")))
-    (test* "trie: common-prefix" '(19 12 8 4 4 3)
-           (map (^p (length (trie-common-prefix t1 p)))
-                '("" "k" "ka" "ku" "li" "lili")))
-    (test* "trie: common-prefix" '(("kua" . 3)
-                                   ("kua`aina" . 8)
-                                   ("kua`ana" . 7))
-           (trie-common-prefix t1 "kua")
-           (cut lset= equal? <> <>))
-    (test* "trie: common-prefix-keys" '("kua" "kua`aina" "kua`ana")
-           (trie-common-prefix-keys t1 "kua")
-           (cut lset= equal? <> <>))
-    (test* "trie: common-prefix-values" '(3 8 7)
-           (trie-common-prefix-values t1 "kua")
-           (cut lset= = <> <>))
-    (test* "trie: common-prefix-fold" 18
-           (trie-common-prefix-fold t1 "kua"
-                                    (lambda (k v s) (+ v s))
-                                    0))
-    (test* "trie: common-prefix-map" '("KUA" "KUA`AINA" "KUA`ANA")
-           (trie-common-prefix-map t1 "kua"
-                                   (lambda (k v) (string-upcase k)))
-           (cut lset= equal? <> <>))
-    (test* "trie: common-prefix-for-each" '("KUA" "KUA`AINA" "KUA`ANA")
-           (let1 p '()
-             (trie-common-prefix-for-each t1 "kua"
-                                          (lambda (k v)
-                                            (push! p (string-upcase k))))
-             p)
-           (cut lset= equal? <> <>))
-    (test* "trie: trie-fold" (fold (lambda (k s) (+ (string-length k) s))
-                                      0 strs)
-           (trie-fold t1 (lambda (k v s) (+ v s)) 0))
-    (test* "trie: trie-map" (fold (lambda (k s) (+ (string-length k) s))
-                                     0 strs)
-           (apply + (trie-map t1 (lambda (k v) v))))
-    (test* "trie: trie-for-each"
-           (fold (lambda (k s) (+ (string-length k) s))
-                 0 strs)
-           (let1 c 0 (trie-for-each t1 (lambda (k v) (inc! c v))) c))
-    (test* "trie: trie->list"
-           (map (^s (cons s (string-length s))) strs)
-           (trie->list t1)
-           (cut lset= equal? <> <>))
-    (test* "trie: trie-keys"
-           strs
-           (trie-keys t1)
-           (cut lset= equal? <> <>))
-    (test* "trie: trie-values"
-           (map string-length strs)
-           (trie-values t1)
-           (cut lset= equal? <> <>))
-    (test* "trie: trie-update!" 16
-           (begin (trie-update! t1 "liliko`i" (cut + <> 8))
-                  (trie-get t1 "liliko`i")))
-    (test* "trie: trie-update! (nonexistent)" (test-error)
-           (trie-update! t1 "humuhumu" (cut + <> 8)))
-    (test* "trie: trie-update! (nonexistent)" 16
-           (begin (trie-update! t1 "humuhumu" (cut + <> 8) 8)
-                  (trie-get t1 "humuhumu")))
-    (test* "trie: delete!" '(19 #f)
-           (begin (trie-delete! t1 "humuhumu")
-                  (list (trie-num-entries t1)
-                        (trie-get t1 "humuhumu" #f))))
-    (test* "trie: delete! (nonexistent)" '(19 #f)
-           (begin (trie-delete! t1 "HUMUHUMU")
-                  (list (trie-num-entries t1)
-                        (trie-get t1 "HUMUHUMU" #f))))
-    (test* "trie: delete! (everything)" 0
-           (begin (for-each (cut trie-delete! t1 <>) strs)
-                  (trie-num-entries t1)))
-    )
-  ;; trie and trie-with-keys
-  (let1 t2 (trie '() '("foo" . 0) '("foof" . 1) '("far" . 2))
-    (test* "trie: trie" '(("foo" . 0) ("foof" . 1) ("far" . 2))
-           (trie->list t2)
-           (cut lset= equal? <> <>)))
-  (let1 t3 (trie-with-keys '() "foo" "foof" "far")
-    (test* "trie: trie-with-keys"
-           '(("foo" . "foo") ("foof" . "foof") ("far" . "far"))
-           (trie->list t3)
-           (cut lset= equal? <> <>)))
+(dotimes [n 20]
+  (test* #"skew-list-length ~n" n
+         (skew-list-length (list->skew-list (make-list n)))))
 
-  ;; heterogeneous tries
-  (let1 t4 (make-trie)
-    (for-each (cut for-each
-                   (lambda (seq)
-                     (trie-put! t4 seq (class-of seq)))
-                   <>)
-              (list strs lists vecs uvecs))
-    (test* "trie(hetero): put!" (* 4 (length strs))
-           (trie-num-entries t4))
-    (test* "trie(hetero): get" <vector> (trie-get t4 '#()))
-    (test* "trie(hetero): get" <u8vector> (trie-get t4 '#u8()))
-    (test* "trie(hetero): get" <pair> (trie-get t4 '(#\k #\u)))
+(let ([series '(0 1 2 3 5 8 13 21)])
+  (dolist [x series]
+    (dolist [y series]
+      (test* #"skew-list-length<=? ~x ~y" (<= x y)
+             (skew-list-length<=? (list->skew-list (iota x)) y)))))
 
-    (test* "trie(hetero): delete!" <string>
-           (begin (trie-delete! t4 '()) (trie-get t4 "")))
-    (test* "trie(hetero): delete!" (* 3 (length strs))
-           (begin (for-each (cut trie-delete! t4 <>) lists)
-                  (trie-num-entries t4)))
-    )
+(let* ([data '(a b c d e f g h i j k l m n o p q r s t u v w x y z)]
+       [sl (list->skew-list data)])
+  (test* "skew-list-ref" data
+         (map-with-index (^[i _] (skew-list-ref sl i)) data))
+  (test* "skew-list-ref (out of range)" (test-error)
+         (skew-list-ref sl (length data)))
+  (test* "skew-list-ref fallback" 'oops
+         (skew-list-ref sl (length data) 'oops))
+  (dotimes [n (length data)]
+    (test* #"skew-list-set ~n"
+           (rlet1 seq (list-copy data)
+             (list-set! seq n 'z))
+           (skew-list->list (skew-list-set sl n 'z)))))
 
-  ;; customizing tables
-  (let1 t5 (make-trie list
-                      (cut assoc-ref <> <> #f char-ci=?)
-                      (lambda (t k v)
-                        (if v
-                          (assoc-set! t k v char-ci=?)
-                          (alist-delete! k t char-ci=?)))
-                      (lambda (t f s) (fold f s t)))
-    (test* "trie(custom): put!" (- (length strs) 1)
-           (begin
-             (for-each (^s (trie-put! t5 s (string-length s))) strs)
-             (trie-num-entries t5)))
-    (test* "trie(custom): get" 99
-           (begin
-             (trie-put! t5 "LILIKO`I" 99)
-             (trie-get t5 "liliko`i")))
-    )
+;; mapping
+(let1 sl (list->skew-list (iota 22))
+  (define (f x) (+ (* 2 x) 1))
+  (test* "skew-list-map" (map f (skew-list->list sl))
+         (skew-list->list (skew-list-map sl f))))
 
-  ;; collection api
-  (let1 t6 #f
-    (test* "trie(collection): builder" (length strs)
-           (begin
-             (set! t6 (coerce-to <trie> (map (cut cons <> #t) strs)))
-             (and (trie? t6) (size-of t6))))
-    (test* "trie(collection): iterator" strs
-           (let1 p '()
-             (call-with-iterator t6
-                                 (lambda (end next)
-                                   (until (end)
-                                     (push! p (car (next))))))
-             p)
-           (cut lset= equal? <> <>))
-    (test* "trie(collection): coerce to list" (map (cut cons <> #t) strs)
-           (coerce-to <list> t6)
-           (cut lset= equal? <> <>))
-    (test* "trie(collection): coerce to vector"
-           (map (cut cons <> #t) strs)
-           (vector->list (coerce-to <vector> t6))
-           (cut lset= equal? <> <>))
-    (test* "trie(collection): coerce to hashtable" #t
-           (let1 h (coerce-to <hash-table> t6)
-             (every (cut hash-table-get h <>) strs)))
-    )
+;; generator & lseq
+(let* ([data '(a b c d e f g h i j k l m n o p q r s t u v w x y z)]
+       [sl (list->skew-list data)])
+  (define (t-gen . args)
+    (test* #"skew-list->generator ~args"
+           (apply subseq data args)
+           (generator->list (apply skew-list->generator sl args)))
+    (test* #"skew-list->lseq ~args"
+           (apply subseq data args)
+           (apply skew-list->lseq sl args)))
+  (t-gen)
+  (t-gen 5)
+  (t-gen 9 17)
+  (t-gen 20 20)
+  (t-gen 25 26))
+
+;; take
+(let ()
+  (define (t-take orig-len take-len)
+    (let1 input (iota orig-len)
+      (test* #"skew-list-take ~|take-len| from ~|orig-len|"
+             (take input take-len)
+             (skew-list->list
+              (skew-list-take (list->skew-list input) take-len)))
+      (test* #"skew-list-drop ~|take-len| from ~|orig-len|"
+             (drop input take-len)
+             (skew-list->list
+              (skew-list-drop (list->skew-list input) take-len)))
+      (test* #"skew-list-split-at ~|take-len| from ~|orig-len|"
+             (values->list (split-at input take-len))
+             (receive (a b)
+                 (skew-list-split-at (list->skew-list input) take-len)
+               (list (skew-list->list a)
+                     (skew-list->list b))))
+      ))
+  ;; We cover all the cases
+  ;; edge case
+  (t-take 0 0)
+  (t-take 10 0)
+  ;; trivial prefix
+  (t-take 26 1)   ; 26 = [1 3 7 15]
+  (t-take 26 4)
+  (t-take 26 11)
+  (t-take 26 26)
+  (t-take 29 7)   ; 29 = [7 7 15]
+  (t-take 29 14)  ; 29 = [7 7 15]
+  (t-take 29 29)  ; 29 = [7 7 15]
+  ;; [1 n ...] case
+  (t-take 8 2)    ; 8 = [1 7]
+  (t-take 8 5)    ; 8 = [1 7]
+  ;; [n ...] (n > 3) case
+  (t-take 14 1)   ; 14 = [7 7]
+  (t-take 14 2)
+  (t-take 14 4)
+  (t-take 15 1)   ; 15 = [15]
+  (t-take 15 2)
+  (t-take 15 5)
+  (t-take 15 8)
+  ;; [3 ...] case
+  (t-take 6 1)    ; 6 = [3 3]
+  (t-take 6 2)
+  ;; general case
+  (t-take 6 3)
+  (t-take 6 4)
+  (t-take 6 5)
+  (t-take 26 2)
+  (t-take 26 3)
+  (t-take 26 5)
+  (t-take 26 10)
+  (t-take 26 15)
   )
 
+;; append
+(let ()
+  (define (t-append i j)
+    (let ([x (iota i)]
+          [y (iota j)])
+      (test* "skew-list-append" (append x y)
+             (skew-list->list (skew-list-append (list->skew-list x)
+                                                (list->skew-list y))))))
+  (t-append 0 0)
+  (t-append 0 1)
+  (t-append 1 0)
+  (t-append 1 1)
+  (t-append 1 2)
+  (t-append 1 3)
+  (t-append 1 4)
+  (t-append 1 5)
+  (t-append 2 1)
+  (t-append 2 2)
+  (t-append 2 3)
+  (t-append 2 6)
+  (t-append 3 1)
+  (t-append 3 2)
+  (t-append 3 3)
+  (t-append 3 4)
+  (t-append 3 6)
+  (t-append 3 7)
+  (t-append 3 10)
+  (t-append 5 10)
+  )
+
+;; sequence protocol
+(test* "skew-list size-of" 5
+       (size-of (list->skew-list '(a b c d e))))
+(test* "skew-list iterator" '(a b c d e f g h i j k l m n o p q)
+       (coerce-to <list> (list->skew-list '(a b c d e f g h i j k l m n o p q))))
+(test* "skew-list fold" '(e d c b a)
+       (fold cons '() (list->skew-list '(a b c d e))))
+(test* "skew-list ref" '(a b c d e)
+       (map (cute ref (list->skew-list '(a b c d e)) <>) (iota 5)))
+
+
+
 (test-end)
