@@ -3,7 +3,6 @@
 ;;
 
 (use gauche.test)
-(use gauche.parameter)
 
 (test-start "dynamic-wind and call/cc")
 
@@ -58,7 +57,7 @@
       (if (null? r)
         (begin (set! r s) (cc -1))
         (list r s)))))
-    
+
 (test "call/cc (inline)" '((1 2 3 4 5 6 7 8) (1 2 3 4 -1 6 7 8))
       callcc-test2)
 
@@ -278,7 +277,7 @@
                      (^[] (push! k 'h)))
                  (push! k 'i)))
           (reverse k))))
-            
+
 (test "dynamic-wind - error in after thunk"
       '(a b c d e f h)
       (^[] (let1 k '()
@@ -371,7 +370,7 @@
 
 (test "stack overflow (apply)" (/ (* 2000 2001) 2)
       (^[] (sum-rec-apply 2000)))
-      
+
 (test "stack overflow (apply)" (/ (* 3000 3001) 2)
       (^[] (sum-rec-apply 3000)))
 
@@ -513,7 +512,7 @@
         (^[] (a-normal '(lambda (x) (lambda (y) (x y))))))
   )
 
-;; Typed Printf via Delimited Continuations 
+;; Typed Printf via Delimited Continuations
 ;; http://www.cs.ox.ac.uk/ralf.hinze/WG2.8/27/slides/kenichi2.pdf
 (let ()
   (define (sprintf format)
@@ -727,6 +726,25 @@
               (reset (error "[E02]"))
               (display "[E03]"))))))
 
+(test* "reset/shift + guard 1"
+       "[W01][D01][D02][W01][D01][D01][E01][D02][D02]"
+       (with-output-to-string
+         (^[]
+           (define queue '())
+           (define (yield) (shift k (push! queue k)))
+           (push! queue (lambda ()
+                          (guard (e (else (display (~ e 'message))))
+                            (yield)
+                            (error "[E01]"))))
+           (while (and (pair? queue) (pop! queue))
+             => next
+             (display "[W01]")
+             (reset
+              (dynamic-wind
+               (lambda () (display "[D01]"))
+               next
+               (lambda () (display "[D02]"))))))))
+
 (test* "dynamic-wind + reset/shift 1"
        "[d01][d02][d03][d04]"
        ;"[d01][d02][d04][d01][d03][d04]"
