@@ -1,6 +1,5 @@
-;; test exception handling system 
+;; test exception handling system
 ;; this must come after primsyn, error, macro and object tests.
-;; $Id: exception.scm,v 1.12 2007-01-18 10:01:48 shirok Exp $
 
 (use gauche.test)
 (test-start "exceptions")
@@ -77,7 +76,7 @@
           (condition-ref e 'message)
           (condition-ref e 'errno)
           (condition-ref e 'port))))
-          
+
 (test* "make-compound-condition"
        `(#t #t #t "message" 12 ,(current-input-port))
        (let ((e (make-compound-condition
@@ -114,7 +113,7 @@
           (list (condition-ref i 'message) (condition-ref i 'port))
           (list (condition-ref s 'message) (condition-ref s 'errno)))
          ))
-       
+
 ;;--------------------------------------------------------------------
 (test-section "srfi-35 style condition definitions")
 
@@ -167,6 +166,36 @@
            (lambda () (error &c1 :a "ok"))))
   )
 
+;;--------------------------------------------------------------------
+(test-section "with-exception-handler")
+
+(test* "with-exception-handler" '(got oops)
+       (call/cc
+        (lambda (ret)
+          (with-exception-handler
+           (lambda (e) (ret `(got ,e)))
+           (lambda () (raise 'oops) 'huh?)))))
+
+(test* "with-exception-handler (returning)" 'huh?
+       (with-exception-handler
+        (lambda (e) `(got ,e))
+        (lambda () (raise 'oops) 'huh?)))
+
+(test* "with-exception-handler (returning mv)" '(aha! oops)
+       (values->list
+        (with-exception-handler
+         (lambda (e) (values 'aha! e))
+         (lambda () (raise 'oops)))))
+
+;; Now built-in with-exception-hanlder has r7rs semantics
+(test* "with-exception-handler (raise in handler)"  '(outer (inner oops))
+       (with-exception-handler
+        (lambda (e) `(outer ,e))
+        (lambda ()
+          (with-exception-handler
+           (lambda (e) (raise `(inner ,e)))
+           (lambda () (raise 'oops))))))
+
 ;;--------------------------------------------------------------------
 (test-section "guard")
 
@@ -175,7 +204,7 @@
                ((symbol? x) (cons 'symbol x))
                ((is-a? x <error>) 'caught-error))
          (raise 'a)))
-       
+
 (test* "guard" 'caught-error
        (guard (x
                ((symbol? x) (cons 'symbol x))
@@ -188,7 +217,7 @@
                ((<error> x) 'caught-error))
          (car 'a)))
 
-(test* "guard (uncaught error)" *test-error*
+(test* "guard (uncaught error)" (test-error)
        (guard (x
                ((symbol? x) (cons 'symbol x))
                ((is-a? x <error>) 'caught-error))
@@ -248,6 +277,9 @@
 ;;--------------------------------------------------------------------
 (test-section "unwind-protect")
 
+;; NB: unwind-protect behavior on exit can't be tested here, for we
+;; need to invoke a subprocess.  The tests are in process.scm instead.
+
 (let ()
   (define aaa '())
   (define (foo thunk)
@@ -262,13 +294,6 @@
             (foo (lambda () (push! aaa 'z)))
             (push! aaa 'e))
            aaa))
-  (test* "unwind-protect (raise)" '(boo e d b a)
-         (guard (e (else (push! aaa e) aaa))
-           (set! aaa '())
-           (unwind-protect
-            (foo (lambda () (raise 'boo)))
-            (push! aaa 'e))
-           aaa))
   (test* "unwind-protect (error)" '(boo e d b a)
          (guard (e (else (push! aaa 'boo) aaa))
            (set! aaa '())
@@ -276,7 +301,24 @@
             (foo (lambda () (error "boo")))
             (push! aaa 'e))
            aaa))
-
+;  (test* "unwind-protect (raise)" '(boo d b a)
+  (test* "unwind-protect (raise)" '(boo d a d b a)
+         (guard (e (else (push! aaa e) aaa))
+           (set! aaa '())
+           (unwind-protect
+            (foo (lambda () (raise 'boo)))
+            (push! aaa 'e))
+           aaa))
+  (test* "unwind-protect (raise & continue)" '(e d c boo a d b a)
+         (begin
+           (set! aaa '())
+           (with-exception-handler
+            (lambda (e) (push! aaa e))
+            (lambda ()
+              (unwind-protect
+                  (foo (lambda () (raise 'boo)))
+                (push! aaa 'e))))
+           aaa))
   (test* "unwind-protect (restart)" '(e d c a d z b a)
          (begin
            (set! aaa '())
@@ -289,6 +331,22 @@
                 (push! aaa 'e)))
              (when k (let ((k0 k)) (set! k #f) (k0 0))))
            aaa))
+
+  ;; We no longer make this an error.  there're legit cases that you want
+  ;; to reenter into BODY even HANDLERs are already called. We just don't
+  ;; execute HANDLERs more than once.
+  (test* "unwind-protect (reenter)"
+         (reverse '(a b z c d e a c d))
+         (begin
+           (set! aaa '())
+           (let ((k #f))
+             (unwind-protect
+                 (foo (lambda ()
+                        (let/cc k2
+                          (set! k k2) (push! aaa 'z))))
+               (push! aaa 'e))
+             (when k (let ((k0 k)) (set! k #f) (k0 0))))
+           aaa))
   )
 
 ;;--------------------------------------------------------------------
