@@ -72,7 +72,28 @@
   (test-length '(a b c d e) 5))
 
 ;;--------------------------------------------------------------------------
-(test-section "append and reverse")
+(test-section "copy, append and reverse")
+
+(test* "copy" '((a b c d e) #f)
+       (let* ([x '(a b c d e)]
+              [y (list-copy x)])
+         (list x (eq? x y))))
+(test* "copy" '((a b c d . e) #f)
+       (let* ([x '(a b c d . e)]
+              [y (list-copy x)])
+         (list x (eq? x y))))
+(test* "copy" '(x #t)
+       (let* ([x 'x]
+              [y (list-copy x)])
+         (list x (eq? x y))))
+(test* "copy" '(() #t)
+       (let* ([x '()]
+              [y (list-copy x)])
+         (list x (eq? x y))))
+(test* "copy" (test-error)              ;detect circular list
+       (let* ([x '#0=(a b c . #0#)]
+              [y (list-copy x)])
+         (list x (eq? x y))))
 
 (test* "append (inlined)" '()           (append))
 (test* "append (inlined)" 1             (append 1))
@@ -225,25 +246,25 @@
 (test* "delete" '("b" "c" "d" "e")
        (delete "A" '("a" "b" "a" "c" "d" "a" "e") string-ci=?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(a b a c a d a e a) eq?))
+       (delete! 'a (list-copy '(a b a c a d a e a)) eq?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(a b a c a d a e) eq?))
+       (delete! 'a (list-copy '(a b a c a d a e)) eq?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(b a c a d a e a) eq?))
+       (delete! 'a (list-copy '(b a c a d a e a)) eq?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(b a c a d a e) eq?))
+       (delete! 'a (list-copy '(b a c a d a e)) eq?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(b c d e) eq?))
+       (delete! 'a (list-copy '(b c d e)) eq?))
 (test* "delete!" '(b c d e . f)
-       (delete! 'a '(b c a d e . f) eq?))
+       (delete! 'a (list-copy '(b c a d e . f)) eq?))
 (test* "delete!" '(2.0 3.0 4.0 5.0)
-       (delete! '1.0 '(1.0 2.0 1.0 3.0 4.0 1.0 5.0) eqv?))
+       (delete! '1.0 (list 1.0 2.0 1.0 3.0 4.0 1.0 5.0) eqv?))
 (test* "delete!" '("b" "c" "d" "e")
-       (delete! "a" '("a" "b" "a" "c" "d" "a" "e")))
+       (delete! "a" (list "a" "b" "a" "c" "d" "a" "e")))
 (test* "delete!" '("b" "c" "d" "e")
-       (delete! "a" '("a" "b" "a" "c" "d" "a" "e") string=?))
+       (delete! "a" (list "a" "b" "a" "c" "d" "a" "e") string=?))
 (test* "delete!" '("b" "c" "d" "e")
-       (delete! "A" '("a" "b" "a" "c" "d" "a" "e") string-ci=?))
+       (delete! "A" (list "a" "b" "a" "c" "d" "a" "e") string-ci=?))
 (test* "delete-duplicates" '(a b c d e)
        (delete-duplicates '(a b c d e) eq?))
 (test* "delete-duplicates" '(a b c d e)
@@ -259,17 +280,17 @@
 (test* "delete-duplicates" '("A" "b" "c" "d" "e")
        (delete-duplicates '("A" "b" "a" "B" "c" "d" "a" "e") string-ci=?))
 (test* "delete-duplicates!" '(a b c d e)
-       (delete-duplicates! '(a b c d e) eq?))
+       (delete-duplicates! (list-copy '(a b c d e)) eq?))
 (test* "delete-duplicates!" '(a b c d e)
-       (delete-duplicates! '(a b a c b a d d a e) eq?))
+       (delete-duplicates! (list-copy '(a b a c b a d d a e)) eq?))
 (test* "delete-duplicates!" '(1.0 2.0 3.0 4.0 5.0)
-       (delete-duplicates! '(1.0 2.0 1.0 2.0 3.0 3.0 4.0 1.0 5.0) eqv?))
+       (delete-duplicates! (list 1.0 2.0 1.0 2.0 3.0 3.0 4.0 1.0 5.0) eqv?))
 (test* "delete-duplicates!" '("a" "b" "c" "d" "e")
-       (delete-duplicates! '("a" "b" "b" "a" "b" "c" "d" "a" "e")))
+       (delete-duplicates! (list "a" "b" "b" "a" "b" "c" "d" "a" "e")))
 (test* "delete-duplicates!" '("a" "b" "c" "d" "e")
-       (delete-duplicates! '("a" "b" "a" "a" "c" "d" "a" "e") string=?))
+       (delete-duplicates! (list "a" "b" "a" "a" "c" "d" "a" "e") string=?))
 (test* "delete-duplicates!" '("A" "b" "c" "d" "e")
-       (delete-duplicates! '("A" "b" "a" "B" "c" "d" "a" "e") string-ci=?))
+       (delete-duplicates! (list "A" "b" "a" "B" "c" "d" "a" "e") string-ci=?))
 
 (test* "any" #f (any even? '()))
 
@@ -324,8 +345,8 @@
 (test* "drop-right" '(1)       (drop-right '(1 2 3 . d) 2))
 (test* "take-right" 'd         (take-right '(1 2 3 . d) 0))
 (test* "drop-right" '(1 2 3)   (drop-right '(1 2 3 . d) 0))
-(test* "take!"      '(1 2)     (take! '(1 2 3 . d) 2))
-(test* "drop-right!" '(1 2)    (drop-right! '(1 2 3 . d) 1))
+(test* "take!"      '(1 2)     (take! (list-copy '(1 2 3 . d)) 2))
+(test* "drop-right!"'(1 2)     (drop-right! (list-copy '(1 2 3 . d)) 1))
 (test* "split-at" '((a b c) (d e f g h))
        (call-with-values (^[] (split-at '(a b c d e f g h) 3))
          list))
@@ -603,7 +624,6 @@
   )
 
 ;;--------------------------------------------------------------------------
-
 (test-section "monotonic-merge")
 
 ;; monotonic-merge is a core function to implement Dylan-style class
@@ -632,4 +652,81 @@
               (hv-grid horizontal-grid vertical-grid grid-layout object)
               (vh-grid vertical-grid horizontal-grid grid-layout object)))))
 
+;;--------------------------------------------------------------------------
+(test-section "immutable pairs")
+
+(test* "ipair" #t (ipair? (ipair 1 2)))
+(test* "ipair" #f (ipair? '()))
+(test* "ipair" #f (ipair? (cons 1 2)))
+(test* "ipair" '(1 . 2)
+       (let1 p (ipair 1 2)
+         (cons (car p) (cdr p))))
+
+(test* "ilist" '(#t #t #t #f)
+       (let1 p (ilist 1 2 3)
+         (list (ipair? p)
+               (ipair? (cdr p))
+               (ipair? (cddr p))
+               (ipair? (cdddr p)))))
+(test* "ilist" '() (ilist))
+
+(test* "ipair immutability" (test-error)
+       (let1 p (ipair 1 2)
+         (set-car! p 5)))
+(test* "ipair immutability" (test-error)
+       (let1 p (ipair 1 2)
+         (set-cdr! p 5)))
+
+;;--------------------------------------------------------------------------
+(test-section "extended pairs")
+
+(let ()
+  (test* "extended pair" #f (extended-pair? (cons 1 2)))
+  ;(test* "extended pair" #t (extended-pair? '(1 2)))
+  (test* "extended pair" #t (extended-pair? (extended-cons 1 2)))
+  (test* "pair-attributes" '() (pair-attributes (cons 1 2)))
+  (test* "pair-attributes" '() (pair-attributes (ipair 1 2)))
+  (test* "pair-attributes" '() (pair-attributes (extended-cons 1 2)))
+  (test* "pair-attributes" '((a . b) (c d))
+         (pair-attributes (extended-cons 1 2 '((a . b) (c d)))))
+  (test* "pair-attributes-get/set!" 'hoi
+         (let1 z (extended-cons 1 2)
+           (pair-attribute-set! z 'foo 'hoi)
+           (pair-attribute-get z 'foo)))
+  (test* "pair-attributes-get/set!" 'poi
+         (let1 z (extended-cons 1 2 '((foo . hoi)))
+           (pair-attribute-set! z 'foo 'poi)
+           (pair-attribute-get z 'foo)))
+  (test* "set-car! to extended pair" '(3 . 2)
+         (rlet1 z (extended-cons 1 2)
+           (set-car! z 3)))
+  (test* "set-cdr! to extended pair" '(1 . 4)
+         (rlet1 z (extended-cons 1 2)
+           (set-cdr! z 4)))
+  )
+
+(let* ((data1 '((a . b) (c . d)))
+       (data2 (list (cons 'A 'B) (cons 'C 'D)))
+       (p1 (extended-cons 1 2 data1))
+       (p2 (extended-cons 1 2 data2)))
+  ;; extended-pair-set! shouldn't clobber the original list
+  (test* "pair-attribute-set! noclobber 1" '((e . z) (a . b) (c . y))
+         (begin
+           (pair-attribute-set! p1 'e 'z)
+           (pair-attribute-set! p1 'c 'y)
+           (pair-attributes p1)))
+  (test* "pair-attribute-set! noclobber 2" '((E . Z) (A . Y) (C . D))
+         (begin
+           (pair-attribute-set! p2 'A 'Y)
+           (pair-attribute-set! p2 'E 'Z)
+           (pair-attributes p2)))
+  (test* "pair-attribute-set! noclobber" #t
+         (equal? data1 '((a . b) (c . d))))
+  (test* "pair-attribute-set! noclobber" #t
+         (equal? data2 '((A . B) (C . D))))
+  ;; prevent optimization
+  (set! data1 #f)
+  (set! data2 #f)
+  )
+
 (test-end)
