@@ -19,7 +19,7 @@
   `(test ,msg ',expect (lambda () (unident (%macroexpand ,form)))))
 
 ;;----------------------------------------------------------------------
-;; 
+;;
 
 (test-section "ER macro basics")
 
@@ -79,8 +79,7 @@
          (let* ([first (car clauses)]
                 [rest  (cdr clauses)]
                 [test  (car first)])
-           (cond [(and (or (symbol? test)
-                           (identifier? test))
+           (cond [(and (identifier? test)
                        (c test (r 'else)))
                   `(,(r 'begin) ,@(cdr first))]
                  [else `(,(r 'if) ,test
@@ -110,8 +109,7 @@
            (let* ([first (car clauses)]
                   [rest  (cdr clauses)]
                   [test  (car first)])
-             (cond [(and (or (symbol? test)
-                             (identifier? test))
+             (cond [(and (identifier? test)
                          (c test (r 'else)))
                     `(,(r 'begin) ,@(cdr first))]
                    [else `(,(r 'if) ,test
@@ -141,7 +139,7 @@
 ;; cf. http://saito.hatenablog.jp/entry/2014/11/18/233209
 (define (er-test-traverse proc obj)
   (let loop ((obj obj))
-    (cond [(or (symbol? obj) (identifier? obj))    (proc obj)]
+    (cond [(identifier? obj) (proc obj)]
           [(pair? obj)   (cons (loop (car obj)) (loop (cdr obj)))]
           [(vector? obj) (vector-map loop obj)]
           [else obj])))
@@ -259,6 +257,43 @@
              (foo 4)
              bar))))
 
+;; Mixing syntax-rules and er-macro requires unhygienic identifiers to be
+;; explicitly "injected".
+;; (This does not work with the current compiler)
+
+;; (define-syntax eri-test-loop
+;;   (eri-macro-transformer
+;;    (lambda (x r c i)
+;;      (let ((body (cdr x)))
+;;        `(,(r 'call-with-current-continuation)
+;;          (,(r 'lambda) (,(i 'exiit))
+;;           (,(r 'let) ,(r 'f) () ,@body (,(r 'f)))))))))
+
+;; (define-syntax eri-test-foo
+;;   (syntax-rules ()
+;;     ((_ x) (eri-test-loop (exiit x)))))
+
+;; (test* "Mixing syntax-rules and eri-macro" 'yot
+;;        (let ((exiit 42))
+;;          (eri-test-foo exiit)))
+
+;;----------------------------------------------------------------------
+;; er identifier macros
+;;  EXPERIMENTAL - The interface of er-macro-transformer may change
+
+(test-section "er identifier macros")
+
+(test "er identifier macro" '((1 2) 3)
+      (lambda ()
+        (let ((x 1) (y 2))
+          (let-syntax ((x (er-macro-transformer
+                           (lambda (f r c)
+                             (if (pair? f)
+                               (quasirename r `(list x y))
+                               (quasirename r `(+ x y))))
+                           :identifier-macro? #t)))
+            (list (x) x)))))
+
 ;;----------------------------------------------------------------------
 ;; basic tests
 
@@ -364,11 +399,11 @@
                        ((_ ((?a ?b ... ?c) ... ?d))
                         ((?a ...) ((?b ...) ...) (?c ...) ?d))))
 
-(test-macro "nest4"((a d f)  
+(test-macro "nest4"((a d f)
                     ((b) () (g h i))
                     (c e j)
                     (k l m))
-            (nest4 ((a b c) (d e) (f g h i j) (k l m)))) 
+            (nest4 ((a b c) (d e) (f g h i j) (k l m))))
 
 (define-syntax nest5 (syntax-rules () ; r7rs
                        ((_ (?a (?b ... ?c ?d) ... . ?e))
@@ -430,7 +465,7 @@
 
 ;; test that wrong usage of ellipsis is correctly identified
 (test "bad ellipsis 1" (test-error)
-      (lambda () 
+      (lambda ()
         (eval '(define-syntax badellipsis
                  (syntax-rules () [(t) (3 ...)]))
               (interaction-environment))))
@@ -530,7 +565,7 @@
                          ((_ x) `(a ,x))))
 (test "unwrap1" '(a 3) (lambda () (unwrap1 3))
       (lambda (x y) (and (eq? (car x) (car y)) (eq? (cadr x) (cadr y)))))
-(test "unwrap1" '(a 4) (lambda () (let ((a 4)) (unwrap1 a))) 
+(test "unwrap1" '(a 4) (lambda () (let ((a 4)) (unwrap1 a)))
       (lambda (x y) (and (eq? (car x) (car y)) (eq? (cadr x) (cadr y)))))
 
 ;; regression check for quasiquote hygienty handling code
@@ -635,7 +670,7 @@
 
 ;; Note: if you "unident" the expansion result of %letrec, you see a symbol
 ;; "newtemp" appears repeatedly in the let binding, seemingly expanding
-;; into invalid syntax.  Internally, however, those symbols are treated 
+;; into invalid syntax.  Internally, however, those symbols are treated
 ;; as identifiers with the correct identity, so the expanded code works
 ;; fine (as tested in the second test).
 (test-macro "%letrec"
@@ -795,7 +830,7 @@
                                ((_ ?x ?y ...) (a ?y ...))
                                ((_) 2))))
             (a 8 9 10)))))
-      
+
 (test "letrec-syntax (nest)"
       '(9 11)
       (lambda ()
@@ -820,6 +855,15 @@
                               ((e? (x . xs)) (o? xs)))))
           (e? '(a a a a)))))
 
+;; shadowing variable binding with syntactic binding
+;; it was allowed up to 0.9.10, but no longer.
+(test "shadowing" (test-error <error> #/Non-identifier-macro can't appear/)
+      (lambda ()
+        (eval '(let ((x 0))
+                 (let-syntax ((x (syntax-rules () ((_) 1))))
+                   (list x)))
+              (current-module))))
+
 ;; This is from comp.lang.scheme posting by Antti Huima
 ;; http://groups.google.com/groups?hl=ja&selm=7qpu5ncg2l.fsf%40divergence.tcs.hut.fi
 (test "let-syntax (huima)" '(1 3 5 9)
@@ -1234,8 +1278,8 @@
   (syntax-rules ()
     ((_ key (alist ...))
      (letrec-syntax ((fooj (syntax-rules (key)
-			    ((_ (key . e) res (... ...)) '(key . e))
-			    ((_ (a . d) res (... ...)) (fooj res (... ...))))))
+                            ((_ (key . e) res (... ...)) '(key . e))
+                            ((_ (a . d) res (... ...)) (fooj res (... ...))))))
        (fooj alist ...)))))
 
 (test "recursive reference in macro-defined-macro" '(c . d)
@@ -1265,6 +1309,35 @@
   (test "literal identifier comparison car car-alias" #t
         (lambda () (free-identifier=?? car car-alias))))
 
+;; macro defining macro from other module
+;; https://github.com/shirok/Gauche/issues/532
+
+(define-module macro-defining-macro-toplevel
+  (export x1)
+  (define-syntax x1
+    (syntax-rules ()
+      ((x1 y1)
+       (x2 x3 y1))))
+
+  (define-syntax x2
+    (syntax-rules ()
+      ((x2 x3 y1)
+       (begin
+         (define-syntax x3
+           (syntax-rules ()
+             ((x3 x4) x4)))
+         (define-syntax y1
+           (syntax-rules ()
+             ((y1 y2) (x3 y2)))))))))
+
+(define-module macro-defining-macro-toplevel-user
+  (use gauche.test)
+  (import macro-defining-macro-toplevel)
+  (x1 bar)
+  ;; without fix, (bar 1) fails with "unbound variable: #<identifier ... x3>"
+  (test "macro defining macro in other module" 1
+        (lambda () (eval '(bar 1) (current-module)))))
+
 ;;----------------------------------------------------------------------
 ;; identifier comparison
 
@@ -1580,9 +1653,9 @@
 ;;----------------------------------------------------------------------
 ;; compiler macros
 
-(test-section "define-inline/syntax")
+(test-section "define-hybrid-syntax")
 
-(define-inline/syntax cpm
+(define-hybrid-syntax cpm
   (lambda (a b) (+ a b))
   (er-macro-transformer
    (lambda (f r c) `(,(r '*) ,(cadr f) ,(caddr f)))))
@@ -1615,7 +1688,7 @@
       (lambda ()
         (guard [e (else (let1 xs (filter <compile-error-mixin>
                                          (slot-ref e '%conditions))
-                          (cons (slot-ref e 'message)
+                          (cons (condition-message e e)
                                 (map (lambda (x) (slot-ref x 'expr)) xs))))]
           (eval '(list (test-syntax-error x y z))
                 (interaction-environment)))))
@@ -1626,7 +1699,7 @@
       (lambda ()
         (guard [e (else (let1 xs (filter <compile-error-mixin>
                                          (slot-ref e '%conditions))
-                          (cons (slot-ref e 'message)
+                          (cons (condition-message e e)
                                 (map (lambda (x) (slot-ref x 'expr)) xs))))]
           (eval '(list (test-syntax-error x y))
                 (interaction-environment)))))
@@ -1812,6 +1885,21 @@
 (test* "hygienic let-keyword expansion" '(1 2 3 4)
        ((with-module let-keyword-hygeiene-use call-klambda) 1 2 3 4))
 
+;; Cf. http://chaton.practical-scheme.net/gauche/a/2020/11/05#entry-5fa3ba50-dc7d3
+(define-syntax let-keywords-hygiene-test-1-inner
+  (er-macro-transformer
+   (^[f r c]
+     (let-keywords (cdr f) ([a 1]
+                            [b 2])
+       (quasirename r `(+ ,a ,b))))))
+(define-syntax let-keywords-hygiene-test-1-outer
+  (syntax-rules ()
+    [(_ x) (let-keywords-hygiene-test-1-inner :b x)]))
+
+(test* "hygienic let-keyword match" 10
+       (let-keywords-hygiene-test-1-outer 9))
+
+
 ;;----------------------------------------------------------------------
 ;; srfi-147 begin
 ;; (not yest supported)
@@ -1837,5 +1925,5 @@
                                             [(_ b) (+ b x)]))])])
           (define-syntax bar (foo 3))
           (bar 5))))
-      
+
 (test-end)
