@@ -469,8 +469,8 @@
 ;; Another tricky interference between modules and implicit setter.
 ;; The :accessor slot option may create the setter generic function
 ;; with the weird name |setter of <getter-name>|.  The name isn't supposed
-;; to be visible from the user ever.  However, until 0.9.6, when the 
-;; MB.base below does not export the setter name, the derived class 
+;; to be visible from the user ever.  However, until 0.9.6, when the
+;; MB.base below does not export the setter name, the derived class
 ;; couldn't find the setter gf and created a separate setter gf, making it
 ;; incompatible with base class and derived class.
 (define-module MB.base
@@ -539,10 +539,8 @@
              (ref <y> 'redefined)))
 
 (test* "subclass redefinition <y> (links)"
-       '(#f #f #f #f #f)
+       '(#f #f #f)
        (list (eq? <y> <y>-orig)
-             (not (memq <y> (ref <x> 'direct-subclasses)))
-             (not (memq <y>-orig (ref <x>-orig 'direct-subclasses)))
              (not (memq <x> (ref <y> 'direct-supers)))
              (not (memq <x>-orig (ref <y>-orig 'direct-supers)))))
 
@@ -552,10 +550,8 @@
             (list <x>-orig <x> <y>-orig <y>)))
 
 (test* "subclass redefinition <w> (links)"
-       '(#f #f #f #f #f)
+       '(#f #f #f)
        (list (eq? <w> <w>-orig)
-             (not (memq <w> (ref <y> 'direct-subclasses)))
-             (not (memq <w>-orig (ref <y>-orig 'direct-subclasses)))
              (not (memq <y> (ref <w> 'direct-supers)))
              (not (memq <y>-orig (ref <w>-orig 'direct-supers)))))
 
@@ -733,8 +729,10 @@
 ;;----------------------------------------------------------------
 (test-section "object hash protocol")
 
-(test* "object-hash" (test-error)
-       (hash (make <cmp> :x (list 1 2))))
+;; Without definining object-hash, it returns a constant value
+(test* "object-hash" #t
+       (eqv? (hash (make <cmp> :x (list 1 2)))
+             (hash (make <cmp> :x (list 1 2)))))
 
 (define-method object-hash ((obj <cmp>))
   (+ (hash (slot-ref obj 'x)) 1))
@@ -753,7 +751,7 @@
        (equal? (hash (make <cmp> :x (cons 1 2)))
                (hash (make <cmp> :x (cons 2 1)))))
 
-(use srfi-1)
+(use scheme.list)
 
 (define xht (make-hash-table 'equal?))
 
@@ -823,6 +821,31 @@
          (hash-table-delete! xht (make <cmp> :x (vector (cons 'a 'b) 3+3i)))
          (hash-table-get xht (make <cmp> :x (vector (cons 'a 'b) 3+3i)) #f)))
 
+;; Allowing to use user-defined objects in a hashtable with
+;; a comparator with default-hash.
+;; We use eq-/eqv-hash regardless of the compoarator's hash setting
+;; If the equality predicate is eq?/eqv?.  See srfi-125.
+;; Also https://github.com/shirok/Gauche/issues/708
+(define-class <user-defined-hashed> ()
+  ((a :init-keyword :a)))
+
+(let ([eqtab (make-hash-table (make-eq-comparator))]
+      [eqvtab (make-hash-table (make-eqv-comparator))]
+      [x0 (make <user-defined-hashed> :a 1)]
+      [x1 (make <user-defined-hashed> :a 2)])
+  (hash-table-put! eqtab x0 1)
+  (hash-table-put! eqtab x1 2)
+  (hash-table-put! eqvtab x0 3)
+  (hash-table-put! eqvtab x1 4)
+
+  (test* "user-defined object in eq/eqv comparator"
+         '(1 2)
+         (map (cut hash-table-get eqtab <>) (list x0 x1)))
+  (test* "user-defined object in eq/eqv comparator"
+         '(3 4)
+         (map (cut hash-table-get eqvtab <>) (list x0 x1)))
+  )
+
 ;;----------------------------------------------------------------
 (test-section "object-apply protocol")
 
@@ -879,15 +902,16 @@
   ())
 
 (define-method initialize ((class <auto-accessor-class>) initargs)
-  (let ((slots (get-keyword :slots initargs '())))
-    (for-each (lambda (slot)
-                (unless (get-keyword :accessor (cdr slot) #f)
-                  (set-cdr! slot (list* :accessor
-                                        (string->symbol
-                                         (format #f "~a-of" (car slot)))
-                                        (cdr slot)))))
-              slots)
-    (next-method)))
+  (let ((slots (map (lambda (slot)
+                      (if (get-keyword :accessor (cdr slot) #f)
+                        slot
+                        (cons (car slot)
+                              (list* :accessor
+                                     (string->symbol
+                                      (format #f "~a-of" (car slot)))
+                                     (cdr slot)))))
+                    (get-keyword :slots initargs '()))))
+    (next-method class (list* :slots slots initargs))))
 
 (define-class <zz> ()
   (a b c)
@@ -1154,7 +1178,7 @@
 ;;----------------------------------------------------------------
 (test-section "metaclass/instance-pool")
 
-(use srfi-1)
+(use scheme.list)
 (use gauche.mop.instance-pool)
 
 (define-class <pool-meta> (<instance-pool-meta>)
@@ -1418,6 +1442,26 @@
        (map (cut slot-ref *member-ted* <>)
             '(full-name occupation sex cname)))
 
+;;----------------------------------------------------------------
+(test-section "class redefinition (part 3 - multiple inheritance)")
+
+;; http://chaton.practical-scheme.net/gauche/a/2020/11/22#entry-5fbab148-42ea5
+(define-module redefinition-3
+  (use gauche.test)
+  (define-class <foo> () ())
+  (define-class <bar> () ())
+  (define-class <foobar> (<foo> <bar>) ())
+  (define <foobar>-save <foobar>)
+
+  ;; This triggers redefinition of <foo> AND <foobar>.
+  (eval '(define-class <foo> () ()) (current-module))
+
+  ;; Make sure reference to the old <foobar> isn't
+  (test* "multiple inheriance redefinition" '(#t #f)
+         (list (boolean (memq <foobar> (~ <bar>'direct-subclasses)))
+               (boolean (memq <foobar>-save (~ <bar>'direct-subclasses)))))
+  )
+
 ;;----------------------------------------------------------------
 (test-section "method application customization")
 
@@ -1593,5 +1637,21 @@
 (test* "object-hygiene-1-c" 4
        (let1 z (make <object-hygiene-1> :c 4)
          (object-hygiene-1-c z)))
-  
+
+;; Hygienic keyword match.
+;; http://chaton.practical-scheme.net/gauche/a/2020/11/15#entry-5fb18659-edde4
+
+(define-module object-hygiene-2
+  (export <foo-meta> <foo>)
+  (define-class <foo-meta> (<class>) ())
+  (define-syntax define-foo
+    (syntax-rules ()
+      ((_ name) (define-class name () () :metaclass <foo-meta>))))
+  (define-foo <foo>)
+  )
+
+(import (object-hygiene-2 :prefix oh2:))
+(test* "hygienic keyword match" oh2:<foo-meta>
+       (class-of oh2:<foo>))
+
 (test-end)
