@@ -2,7 +2,7 @@
 
 (use gauche.test)
 (use gauche.vm.insn)
-(use srfi-1)
+(use scheme.list)
 (use gauche.uvector)
 (use util.match)
 
@@ -60,6 +60,37 @@
 (test* "inlining apply" '(((CONST-PUSH) :a) ((CONSTI 2)) ((CONS)) ((RET)))
        (proc->insn/split (^[] (let1 xs '(2) (apply cons :a xs)))))
 
+;; This should give up apply inlining, for the content of xs may be
+;; altered when passed to a global function.
+(test* "inlining apply (give up)"
+       '(((CONSTI 2)) ((LIST 1)) ((PUSH-LOCAL-ENV 1)) ((PRE-CALL 1) 8)
+         ((LREF0-PUSH)) ((GREF-CALL 1) foo)
+         ((GREF-PUSH) cons) ((CONSTI-PUSH 1)) ((LREF0)) ((TAIL-APPLY 3))
+         ((RET)))
+       (unwrap-syntax
+        (proc->insn/split
+         (^[] (let1 xs (list 2) (foo xs) (apply cons 1 xs))))))
+;; Similar to above, but if xs is constant, so we don't worry about mutation.
+(test* "inlining apply (ok)"
+       '(((PRE-CALL 1) 6) ((CONST-PUSH) (2)) ((GREF-CALL 1) foo)
+         ((CONSTI-PUSH 1)) ((CONSTI 2)) ((CONS))
+         ((RET)))
+       (unwrap-syntax
+        (proc->insn/split
+         (^[] (let1 xs '(2) (foo xs) (apply cons 1 xs))))))
+;; This is the one that exhibits the bug in the issue
+;; https://github.com/shirok/Gauche/issues/685
+(test* "inlining apply (give up)"
+       '(((CONST) a) ((LIST 1)) ((PUSH-LOCAL-ENV 1))
+         ((PRE-CALL 2) 10) ((LREF0-PUSH)) ((LREF0-PUSH))
+         ((GREF-CALL 2) set-cdr!)
+         ((GREF-PUSH) list) ((LREF0)) ((TAIL-APPLY 2)) ((RET)))
+       (unwrap-syntax
+        (proc->insn/split
+         (^[] (let ((x (list 'a)))
+                (set-cdr! x x)
+                (apply list x))))))
+
 (define-inline (apply-inline-1 . args) (apply + 3 args))
 
 (test* "inlining apply" '(((LREF2-NUMADDI 3))
@@ -71,7 +102,7 @@
 (test* "inlining apply (later stage)" '(((CONSTI-PUSH 5))
                                         ((LREF2))
                                         ((NUMMUL2))
-                                        ((PUSH)) 
+                                        ((PUSH))
                                         ((LREF1))
                                         ((NUMMUL2))
                                         ((PUSH))
@@ -94,6 +125,16 @@
        '(((LREF0-PUSH)) ((CONSTI 0)) ((UVEC-REF 1)) ((RET)))
        (proc->insn/split (late-inline-test-1 u8vector-ref)))
 
+
+;; pass3/receive
+(test* "inlining receive-apply-values pattern" '(((LREF0-CAR))
+                                                 ((PUSH-LOCAL-ENV 1))
+                                                 ((PRE-CALL 1) 8)
+                                                 ((CONST-PUSH) "foo")
+                                                 ((GREF-CALL 1) print)
+                                                 ((LREF0-RET)))
+       (unwrap-syntax (proc->insn/split (^x (begin0 (car x) (print "foo"))))))
+
 (test-section "lambda lifting")
 
 ;; bug reported by teppey
@@ -109,6 +150,12 @@
 (test* "lifting constant lambda" '()
        (filter-insn (^(xs) (map (^k (* k k)) xs)) 'CLOSURE))
 
+(test* "lifting constant lambda 2" '()
+       (filter-insn (^[] (let ([a (^[x r] (* x r))])
+                           (let ([b (^[xs] (fold a '() xs))])
+                             (f b))))
+                    'CLOSURE))
+
 ;; See if constant lambda keeps identity.
 ;; NB: This isn't a guaranteed behavior, but it holds in the
 ;; current compiler, and there's no reason to lose it.
@@ -191,6 +238,41 @@
            (test (bar)))
          (foo)))
 
+;; This test exhibits a bug on inlining a procedure within the same
+;; compilation unit.  Fixed by commit 43b479fb0.
+(test* "inlining procedure in the same compilation unit"
+       #f
+       (let ((m (make-module #f)))
+         ;; If foo is inline-expanded, gref to foo won't remain.
+         (define (reference-to-foo? opcode)
+           (and (wrapped-identifier? opcode)
+                (eq? (identifier->symbol opcode) 'foo)))
+
+         (eval '(begin
+                  (define-inline (foo x) (pair? x))
+                  (define (bar y) (foo (car y))))
+               m)
+         (find
+          (^[insn]
+            (match insn
+              [(_ (? reference-to-foo? x)) #t]
+              [_ #f]))
+          (proc->insn/split (global-variable-ref m 'bar)))))
+
+;; generic function pre-dispatch
+;; NB: We're still not sure how to expose this feature in general.
+;; For the time being, we test the inlining logic (it's in pass 3)
+;; with the hand-wired setting.  Don't copy this way to your production code.
+(define-inline static-gf (make <generic> :name 'static-gf))
+(define-method static-gf :locked ((x <string>)) 'string)
+(define-method static-gf :locked ((x <number>)) 'number)
+((with-module gauche.object generic-seal!) static-gf)
+
+(test* "generic function pre-dispatch"
+       '(((CONST-RET) string))
+       (proc->insn/split (^[] (static-gf "abc"))))
+(test* "generic function pre-dispatch"
+       '(((CONST-RET) number))
+       (proc->insn/split (^[] (static-gf 123))))
 
 (test-end)
-
