@@ -108,7 +108,7 @@
 (prim-test "apply^2" '(1 . 2) (lambda () (Apply Apply cons '((1 2)))))
 (prim-test "apply^2" '(3 . 4) (lambda () (Apply Apply cons 3 '((4)))))
 (prim-test "apply^2" '(5 . 6) (lambda () (Apply Apply (list cons 5 '(6)))))
-                                          
+
 
 (prim-test "apply" '(6 7 8) (lambda ()  (Apply Apply (list list 6 7 '(8)))))
 
@@ -129,6 +129,50 @@
                  (set-car! (cdr new) '100)
                  orig))))
 
+;; Detect circular list in the argument
+;; https://github.com/shirok/Gauche/issues/684
+;; NB: At this point, we haven't tested #0= reader notation,
+;; and to avoid optimization, these data should be in global space.
+(define *apply-circular-data-1*
+  (let ((x (list 'a)))
+    (set-cdr! x x)
+    x))
+(define *apply-circular-data-2*
+  (let ((x (list 'a 'a)))
+    (set-cdr! (cdr x) x)
+    x))
+
+(prim-test "apply, circular list 1"
+           "improper list not allowed: #0=(a . #0#)"
+           (lambda ()
+             (with-error-handler
+                 (lambda (e) (slot-ref e 'message))
+               (lambda ()
+                 (apply list *apply-circular-data-1*)))))
+(prim-test "apply, circular list 2"
+           "improper list not allowed: #0=(a . #0#)"
+           (lambda ()
+             (with-error-handler
+                 (lambda (e) (slot-ref e 'message))
+               (lambda ()
+                 (apply list 'a *apply-circular-data-1*)))))
+(prim-test "apply, circular list 3"
+           "improper list not allowed: #0=(a a . #0#)"
+           (lambda ()
+             (with-error-handler
+                 (lambda (e) (slot-ref e 'message))
+               (lambda ()
+                 (apply list 'a *apply-circular-data-2*)))))
+(prim-test "apply, circular list 4"
+           "improper list not allowed: #0=(a a . #0#)"
+           (lambda ()
+             (with-error-handler
+                 (lambda (e) (slot-ref e 'message))
+               (lambda ()
+                 ;; This is caught in different place (#<subr apply>),
+                 ;; rather than VM APPLY instruction.
+                 (Apply list 'a *apply-circular-data-2*)))))
+
 ;; This test exhibits the optimizer bug reported by Michael Campbell.
 (define bug-optimizer-local-inliner
   (lambda (flag)
@@ -370,7 +414,7 @@
 ;; Fix this after we have separate compile-error condition.
 (define (test-max-literal-args msg expr)
   (prim-test (string-append "max literal arguments for " msg)
-             'caught 
+             'caught
              (lambda ()
                (with-error-handler (lambda (e) 'caught)
                  (lambda () (eval expr (interaction-environment)))))))
@@ -530,4 +574,3 @@
 
 
 (test-end)
-
