@@ -133,8 +133,9 @@
 ;;--------------------------------------------------------------------
 (test-section "rfc.cookie")
 (use rfc.cookie)
-(use srfi-19)
 (test-module 'rfc.cookie)
+(use scheme.list)
+(use srfi-19)
 
 (test* "parse, old" '(("foo" "bar")
                       ("aaa" "bbb" :path "/a/b" :domain "a.b.com")
@@ -193,6 +194,36 @@
           ("foo" "baz" :expires ,(make-date 0 40 46 1 9 9 2001 0)))
         0))
 
+(test* "cookie date" (make-date 0 1 2 3 11 12 2023 0)
+       (parse-cookie-date "Sun, 11-Dec-2023 03:02:01 GMT"))
+(test* "cookie date" (make-date 0 3 4 5 2 1 1999 0)
+       (parse-cookie-date "Sun, 2-January-99 5:4:3 garbage"))
+
+(let ([jar (make-cookie-jar)])
+  (define (test-jar n host port path expect)
+    (test* #"cookie-jar-get ~n" expect
+           (map (^e (list (~ e'name) (~ e'value)))
+                (cookie-jar-get jar host port path))
+           (cut lset= equal? <> <>)))
+
+  (cookie-jar-put! jar "foo.example.com" "/index.html"
+                   '(("a" "ABC" :path "/" :domain "example.com")
+                     ("d" "DEF" :path "/" :domain "example.org")
+                     ("g" "GHI" :path "/" :domain "foo.example.com")))
+  (test-jar 1 "foo.example.com" 80 "/"
+            '(("a" "ABC") ("g" "GHI")))
+
+  (cookie-jar-put! jar "foo.bar.example.org" "/x/y/z"
+                   '(("x" "XYZ" :domain "bar.example.org")
+                     ("y" "YZX" :domain "foo.example.org")
+                     ("z" "ZXY" :path "/x/")
+                     ("w" "WXY" :path "/x/w")))
+  (test-jar 2 "foo.bar.example.org" #f "/x/y"
+            '(("x" "XYZ") ("z" "ZXY")))
+  (test-jar 3 "foo.bar.example.org" #f "/x/w"
+            '(("z" "ZXY") ("w" "WXY")))
+  )
+
 ;;--------------------------------------------------------------------
 (test-section "rfc.ftp")
 (use rfc.ftp)
@@ -349,7 +380,7 @@
        (receive r (uri-parse "mailto:shiro@example.com") r))
 
 (let ([base0 "http://a/b/c/d;p?q"])
-  (define (t base rel expect) 
+  (define (t base rel expect)
     (test* (format "merging ~s onto ~s" rel base)
            expect (uri-merge base rel)))
   (define t0 (pa$ t base0))
@@ -482,10 +513,9 @@
 
 (use rfc.822)
 (use rfc.mime)
-(use gauche.parameter)
 
 (test* "http-user-agent" #"gauche.http/~(gauche-version)"
-       (and (is-a? http-user-agent <parameter>)
+       (and (parameter? http-user-agent)
             (http-user-agent)))
 
 (test* "http-compose-query" "/search?q=foo%20bar&n=20"
@@ -738,7 +768,7 @@
   (define (tester msg thunk)
     (test* #"http-post (multipart/form-data) ~msg" expected
            (receive (code headers body) (thunk)
-             
+
              (and-let* ([ (equal? code "200") ]
                         [ (equal? headers '(("content-type" "text/plain"))) ]
                         [r (read-from-string body)]
@@ -774,4 +804,52 @@
 
 (sys-waitpid -1)
 
+
+;;--------------------------------------------------------------------
+(test-section "rfc.uuid")
+
+(use rfc.uuid)
+(test-module 'rfc.uuid)
+(use srfi-13)
+(use srfi-27)
+
+(let1 u "f81d4fae-7dec-11d0-a765-00a0c91e6bf6"
+  (test* "uuid-parse & write 1" u
+         (uuid->string (parse-uuid u)))
+  (test* "uuid-parse & write 2" u
+         (uuid->string (parse-uuid #"urn:uuid:~u")))
+  (test* "uuid-parse & write 3" u
+         (uuid->string (parse-uuid #"{~u}")))
+  (test* "uuid-parse & write 4" u
+         (uuid->string (parse-uuid (string-delete #\- u))))
+  )
+
+(let* ([u1 (uuid1)]
+       [u2 (uuid1)])
+  (test* "uuid-version" 1 (uuid-version u1))
+  (test* "uuid1 timestamp" #t (<? uuid-comparator u1 u2))
+  (test* "uuid1 node id" #t
+         (u8vector=? (uvector-alias <u8vector> (uuid-value u1) 10)
+                     (uvector-alias <u8vector> (uuid-value u2) 10)))
+  )
+
+(let* ([u (uuid4)])
+  (test* "uuid-version" 4 (uuid-version u)))
+
+;; test random source parameterization
+(let* ([rs (make-random-source)]
+       [uuid00 #f]
+       [uuid01 #f]
+       [uuid10 #f]
+       [uuid11 #f])
+  (random-source-state-set! rs (random-source-state-ref (uuid-random-source)))
+  (set! uuid00 (uuid4))
+  (parameterize ([uuid-random-source rs])
+    (set! uuid10 (uuid4))
+    (set! uuid11 (uuid4)))
+  (set! uuid01 (uuid4))
+  (test* "uuid-random-source" '(#t #t)
+         (list (equal? (uuid->string uuid00) (uuid->string uuid10))
+               (equal? (uuid->string uuid01) (uuid->string uuid11)))))
+
 (test-end)
