@@ -167,7 +167,9 @@
     (define (filter-copy infile outfile)
       (file-filter (^[in out]
                      (dolist [line (port->string-list in)]
-                       (display ($ regexp-replace-all* line #/@@/ ""
+                       (display ($ regexp-replace-all* line
+                                   #/@@author@@/ ""
+                                   #/@@/ ""
                                    #/\(cf-output-default\)/
                                    "(cf-define 'HAVE_STDIO_H \"1\")\n\
                                     (cf-config-headers \"config.h\")\n\
@@ -176,9 +178,14 @@
                        (newline out)))
                    :input infile
                    :output outfile))
-    (filter-copy (build-path extdir "template.configure") "test.o/configure")
-    (filter-copy (build-path extdir "template.package.scm") "test.o/package.scm")
-    (filter-copy (build-path extdir "template.Makefile.in") "test.o/Makefile.in")
+    (filter-copy (build-path extdir "package-templates" "configure")
+                 "test.o/configure")
+    (filter-copy (build-path extdir "package-templates" "configure-compat")
+                 "test.o/configure-compat")
+    (filter-copy (build-path extdir "package-templates" "package.scm")
+                 "test.o/package.scm")
+    (filter-copy (build-path extdir "package-templates" "Makefile.in")
+                 "test.o/Makefile.in")
 
     (with-output-to-file "test.o/src/Makefile.in"
       (^[]
@@ -210,7 +217,7 @@
          (and (file-exists? "test.o/VERSION")
               (file->string "test.o/VERSION")))
   (test* "srcdir etc."
-         '("srcdir = ." "top_srcdir = .." "builddir = ." "top_builddir = ..")
+         '("srcdir = ." "top_srcdir = ../" "builddir = ." "top_builddir = ../")
          (file->string-list "test.o/src/Makefile"))
 
   (test* "configure --version" "package configure 1.0"
@@ -231,7 +238,7 @@
                (file-exists? "test2.o/src/Makefile")))
 
   (test* "srcdir etc."
-         '("srcdir = ../test.o/src" "top_srcdir = ../../test.o" "builddir = ." "top_builddir = ..")
+         '("srcdir = ../test.o/src" "top_srcdir = ../../test.o" "builddir = ." "top_builddir = ../")
          (file->string-list "test2.o/src/Makefile"))
 
   (dolist [d '("test.o" "test2.o")]
@@ -269,19 +276,24 @@
     (define (filter-copy infile outfile)
       (file-filter (^[in out]
                      (dolist [line (port->string-list in)]
-                       (display ($ regexp-replace-all* line #/@@/ "") out)
+                       (display (regexp-replace-all* line
+                                                     #/@@author@@/ ""
+                                                     #/@@/ "")
+                                out)
                        (newline out)))
                    :input infile
                    :output outfile))
-    (filter-copy (build-path extdir "template.package.scm") "test.o/package.scm")
-    (filter-copy (build-path extdir "template.Makefile.in") "test.o/Makefile.in")
+    (filter-copy (build-path extdir "package-templates" "package.scm")
+                 "test.o/package.scm")
+    (filter-copy (build-path extdir "package-templates" "Makefile.in")
+                 "test.o/Makefile.in")
 
     (with-output-to-file "test.o/configure"
       (^[]
         ($ for-each write
            '((use gauche.configure)
              (cf-init-gauche-extension)
-             (cf-check-headers '("stdio.h" "stdlib.h" 
+             (cf-check-headers '("stdio.h" "stdlib.h"
                                  "no-such-header-should-exist.h"))
              (cf-check-types '("a_t" "b_t" "struct c_t" "d_t")
                              :includes '("typedef int a_t;\n"
@@ -343,30 +355,30 @@
            '("LIBS = -lm"))
          (file->string-list "test.o/src/Makefile"))
 
-  (test* "cf-check-headers and cf-check-lib to set defines"
-         `("#define HAVE_STDIO_H 1"
-           "#define HAVE_STDLIB_H 1"
-           "/* #undef HAVE_NO_SUCH_HEADER_SHOILD_EXIST_H */"
-           "#define HAVE_A_T 1"
-           "/* #undef HAVE_B_T */"
-           "#define HAVE_STRUCT_C_T 1"
-           "/* #undef HAVE_D_T */"
-           "#define HAVE_DECL_A 1"
-           "#define HAVE_DECL_B 1"
-           "#define HAVE_DECL_C 1"
-           "#define HAVE_DECL_D 1"
-           "#define HAVE_DECL_E 1"
-           "#define HAVE_DECL_F 0"
-           "/* #undef HAVE_STRUCT_FOO_A */"
-           "#define HAVE_STRUCT_FOO_B 1"
-           "#define HAVE_PRINTF 1"
-           "/* #undef HAVE_NONEXISTENT_WEIRD_FUNCTION */"
-           ,(if (#/darwin/ (gauche-architecture))
-              "/* #undef HAVE_LIBM */"
-              "#define HAVE_LIBM 1")
-           "/* #undef HAVE_LIBNO_SUCH_LIBRARY_SHOULD_EXIST */")
-         ($ filter #/HAVE_/
-            $ file->string-list "test.o/config.h"))
+  (test*/diff "cf-check-headers and cf-check-lib to set defines"
+              `("#define HAVE_STDIO_H 1"
+                "#define HAVE_STDLIB_H 1"
+                "/* #undef HAVE_NO_SUCH_HEADER_SHOILD_EXIST_H */"
+                "#define HAVE_A_T 1"
+                "/* #undef HAVE_B_T */"
+                "#define HAVE_STRUCT_C_T 1"
+                "/* #undef HAVE_D_T */"
+                "#define HAVE_DECL_A 1"
+                "#define HAVE_DECL_B 1"
+                "#define HAVE_DECL_C 1"
+                "#define HAVE_DECL_D 1"
+                "#define HAVE_DECL_E 1"
+                "#define HAVE_DECL_F 0"
+                "/* #undef HAVE_STRUCT_FOO_A */"
+                "#define HAVE_STRUCT_FOO_B 1"
+                "#define HAVE_PRINTF 1"
+                "/* #undef HAVE_NONEXISTENT_WEIRD_FUNCTION */"
+                ,(if (#/darwin/ (gauche-architecture))
+                   "/* #undef HAVE_LIBM */"
+                   "#define HAVE_LIBM 1")
+                "/* #undef HAVE_LIBNO_SUCH_LIBRARY_SHOULD_EXIST */")
+              ($ filter #/HAVE_/
+                 $ file->string-list "test.o/config.h"))
   )
 
 (wrap-with-test-directory configure-test-1 '("test.o" "test2.o") #f)
@@ -465,7 +477,7 @@
 (test-script
  (build-path (or (sys-getenv "top_srcdir") "..") "src" "gauche-package.in"))
 
-(define (package-generate-tests)
+(define (package-generate-tests scheme-only?)
   (define (file-check name)
     (test* #"checking existence of ~name" #t
            (file-exists? #"test.o/Test/~name")))
@@ -480,14 +492,26 @@
     (build-path (gauche-architecture-directory) "gauche-package"))
   (define generate-command
     (if in-place?
-      `(../gosh -ftest ./run generate Test test.module)
-      `(,gauche-package generate Test test.module)))
+      `(../gosh -ftest
+                ,(build-path *top-srcdir* "src" "gauche-package.in")
+                generate
+                ,@(cond-list [scheme-only? "--scheme-only"])
+                --template-dir ,(build-path *top-srcdir* "ext"
+                                            "package-templates")
+                Test test.tester)
+      `(,gauche-package generate
+                        ,@(cond-list [scheme-only? "--scheme-only"])
+                        Test test.tester)))
   (define compile-command
     (if in-place?
       `(../../gosh -q -I../../../src -I../../../lib
                    ../run compile
                    --verbose test test.c testlib.stub)
-      `(,gauche-package compile --verbose test test.c testlib.stub)))    
+      `(,gauche-package compile --verbose test test.c testlib.stub)))
+  (define test-command
+    (if in-place?
+      `(../../gosh -ftest -I. ./test.scm)
+      `(,(build-path (gauche-architecture-directory) "gosh") -I. ./test.scm)))
 
   (when in-place?
     (with-output-to-file "test.o/run"
@@ -498,7 +522,7 @@
              (define sep (cond-expand [gauche.os.windows ";"][else ":"]))
              (define top-srcdir ,*top-srcdir*)
              ;; This is used to copy templates from.
-             (define (gauche-library-directory) #"~|top-srcdir|/ext/x")
+             (define (gauche-library-directory) #"~|top-srcdir|/ext/templates/x")
              ;; Intercept gauche-config to override compiler flags
              (define gauche-config-orig
                (with-module gauche.config gauche-config))
@@ -511,39 +535,99 @@
              (load #"~|top-srcdir|/src/gauche-package.in")))))
 
   (test-log "Running ~a" generate-command)
-  
-  ($ run-process generate-command
-     :output :null :error :null :wait #t :directory "test.o/")
 
-  (for-each file-check '("configure" "Makefile.in"
-                         "test.c" "test.h" "test.scm" "testlib.stub"
-                         "test/module.scm"))
+  ($ run-process generate-command :wait #t :directory "test.o/")
+
+  (for-each file-check
+            (if scheme-only?
+              '("configure" "Makefile.in" "package.scm"
+                "test.scm" "test/tester.scm")
+              '("configure" "Makefile.in" "package.scm"
+                "test.c" "test.h" "test.scm" "testlib.stub"
+                "test/tester.scm")))
 
   (test-log "Running ~a" compile-command)
-  
-  (test* "gauche-package compile" #t
+
+  (unless scheme-only?
+    (test* "gauche-package compile" #t
+           (let* ([p ($ run-process
+                        compile-command
+                        :redirects '((>& 2 1) (> 1 out))
+                        :directory "test.o/Test")]
+                  [o (port->string (process-output p 'out))])
+             (process-wait p)
+             ;; if compilation fails, returns the output for better diagnostics.
+             (or (zero? (process-exit-status p)) o))))
+
+  (test* "running test" #t
          (let* ([p ($ run-process
-                      compile-command
-                      :redirects '((>& 2 1) (> 1 out)) :directory "test.o/Test")]
+                      test-command
+                      :redirects  '((>& 2 1) (> 1 out))
+                      :directory "test.o/Test")]
                 [o (port->string (process-output p 'out))])
            (process-wait p)
-           ;; if compilation fails, returns the output for better diagnostics.
            (or (zero? (process-exit-status p)) o)))
   )
 
-(wrap-with-test-directory package-generate-tests '("test.o"))
+(wrap-with-test-directory (cut package-generate-tests #f) '("test.o"))
+(wrap-with-test-directory (cut package-generate-tests #t) '("test.o"))
 
 ;;=======================================================================
 (test-section "precomp")
 
+(define (do-precomp! files extra-options)
+  (do-process! `("../../src/gosh" "-ftest"
+                 ,#"-I~|*top-srcdir*|/test/test-precomp"
+                 ,(build-path *top-srcdir* "src/precomp")
+                 "--strip-prefix"
+                 ,(fix-path (build-path *top-srcdir* "test/test-precomp"))
+                 ,@extra-options
+                 ,@(map (^[file] (fix-path (build-path *top-srcdir* "test/test-precomp" file)))
+                        files))
+               :directory "test.o"))
+
+(define (do-compile! output files)
+  (do-process!
+   `("../../src/gosh" "-ftest"
+     ,(build-path *top-srcdir* "src/gauche-package.in")
+     "compile"
+     ,#"--cppflags=-I~(fix-path (build-path *top-srcdir* \"src\")) \
+                   -I~(fix-path (build-path *top-srcdir* \"gc/include\")) \
+                   -I~(fix-path (build-path *top-builddir* \"src\")) \
+                   -I~(fix-path (build-path *top-builddir* \"gc/include\"))"
+     ,#"--ldflags=-L~(fix-path (build-path *top-srcdir* \"src\")) \
+                  -L~(fix-path (build-path *top-builddir* \"src\"))"
+     ,output
+     ,@files)
+   :directory "test.o"))
+
+;; On Windows we can't remove dll file that's being used in the active
+;; process, so we spawn a child gosh and load it.
+(define-syntax dynload-and-eval
+  (syntax-rules ()
+    [(_ libname expr)
+     (cond-expand
+      [gauche.os.windows
+       (with-output-to-file "test.o/t.scm"
+         (^[]
+           (write '(add-load-path "."))
+           (write '(load libname))
+           (write '(write expr))
+           (write '(exit 0))))
+       (let* ([p (run-process '("../../src/gosh" "-ftest" "./t.scm")
+                              :output :pipe :directory "test.o")]
+              [result (read (process-output p))])
+         (process-wait p)
+         result)]
+      [else
+       (load libname :paths '("./test.o"))
+       expr])]))
+
+
 (define (precomp-test-1)
-  (do-process `("../../src/gosh" "-ftest"
-                ,#"-I~|*top-srcdir*|/test/test-precomp"
-                ,(build-path *top-srcdir* "src/precomp") "--strip-prefix"
-                ,(fix-path (build-path *top-srcdir* "test/test-precomp"))
-                ,@(map (^[file] (fix-path (build-path *top-srcdir* "test/test-precomp" file)))
-                       '("foo.scm" "foo/bar1.scm" "foo/bar2.scm" "foo/bar3.scm")))
-              :directory "test.o")
+  (test* "running precomp 1" #t
+         (do-precomp! '("foo.scm" "foo/bar1.scm" "foo/bar2.scm" "foo/bar3.scm")
+                      '()))
   (test* "precomp generated files"
          '("test.o/foo--bar1.c"
            "test.o/foo--bar2.c"
@@ -556,7 +640,6 @@
          (sort (map fix-path (directory-fold "test.o" cons '()))))
   )
 
-
 (add-load-path "test.o")
 (use srfi-42)
 (use scheme.vector :only (vector-every))
@@ -580,14 +663,9 @@
                       [else (= x y)]))]
           [else (equal? x y)]))
 
-  (do-process `("../../src/gosh" "-ftest"
-                ,#"-I~|*top-srcdir*|/test/test-precomp"
-                ,(build-path *top-srcdir* "src/precomp") "--strip-prefix"
-                ,(fix-path (build-path *top-srcdir* "test/test-precomp"))
-                "--single-interface"
-                ,@(map (^[file] (fix-path (build-path *top-srcdir* "test/test-precomp" file)))
-                       '("foo.scm" "foo/bar1.scm" "foo/bar2.scm" "foo/bar3.scm")))
-              :directory "test.o")
+  (test* "running precomp 2" #t
+         (do-precomp! '("foo.scm" "foo/bar1.scm" "foo/bar2.scm" "foo/bar3.scm")
+                      '("--single-interface")))
   (test* "precomp generated files with --single-interface"
          '("test.o/foo--bar1.c"
            "test.o/foo--bar2.c"
@@ -596,77 +674,114 @@
            "test.o/foo.sci")
          (sort (map fix-path (directory-fold "test.o" cons '()))))
 
-  (test* "consolidated sci file"
-         '(";; generated automatically.  DO NOT EDIT"
-           "#!no-fold-case"
-           "(define-module foo.bar2 (use util.match) (export bar2))"
-           "(select-module foo.bar2)"
-           "(dynamic-load \"foo\" :init-function \"Scm_Init_foo__bar2\")"
-           "(provide \"foo/bar2\")"
-           "(define-module foo.bar3 (use foo.bar2) (export bar3))"
-           "(select-module foo.bar3)"
-           "(dynamic-load \"foo\" :init-function \"Scm_Init_foo__bar3\")"
-           "(provide \"foo/bar3\")"
-           "(define-module foo.bar1 (use foo.bar2) (export bar1))"
-           "(select-module foo.bar1)"
-           "(dynamic-load \"foo\" :init-function \"Scm_Init_foo__bar1\")"
-           "(provide \"foo/bar1\")"
-           "(define-module foo (use foo.bar1) (use foo.bar3) (export foo-master foo-literals foo-begin1 foo-begin2) (export foo-include1 foo-include2))"
-           "(select-module foo)"
-           "(dynamic-load \"foo\" :init-function \"Scm_Init_foo\")")
-         (file->string-list "test.o/foo.sci"))
+  (test*/diff "consolidated sci file"
+              '(";; generated automatically.  DO NOT EDIT"
+                "#!no-fold-case"
+                "(define-module foo.bar2 (use util.match) (export bar2))"
+                "(select-module foo.bar2)"
+                "(dynamic-load \"foo\" :init-function \"Scm_Init_foo__bar2\")"
+                "(provide \"foo/bar2\")"
+                "(define-module foo.bar3 (use foo.bar2) (export bar3))"
+                "(select-module foo.bar3)"
+                "(dynamic-load \"foo\" :init-function \"Scm_Init_foo__bar3\")"
+                "(provide \"foo/bar3\")"
+                "(define-module foo.bar1 (use foo.bar2) (export bar1))"
+                "(select-module foo.bar1)"
+                "(dynamic-load \"foo\" :init-function \"Scm_Init_foo__bar1\")"
+                "(provide \"foo/bar1\")"
+                "(define-module foo (use gauche.uvector) (use foo.bar1) (use foo.bar3) (export foo-master foo-literals foo-shared-literals foo-begin1 foo-begin2) (export foo-include1 foo-include2))"
+                "(select-module foo)"
+                "(dynamic-load \"foo\" :init-function \"Scm_Init_foo\")")
+              (file->string-list "test.o/foo.sci"))
 
-  (test* "compile and dynload" 
+  (test* "compile 2" #t
+         (do-compile! "foo"
+                      '("foo.c" "foo--bar1.c" "foo--bar2.c" "foo--bar3.c")))
+  (test* "dynload and literals 1"
          (list (include "test-precomp/literals.scm")
                'begin1
                'begin2
                'include1
                'include2)
-         (begin
-           (do-process! `("../../src/gosh" "-ftest"
-                          ,(build-path *top-srcdir* "src/gauche-package.in")
-                          "compile"
-                          ,#"--cppflags=-I~(fix-path (build-path *top-srcdir* \"src\")) \
-                                        -I~(fix-path (build-path *top-srcdir* \"gc/include\")) \
-                                        -I~(fix-path (build-path *top-builddir* \"src\")) \
-                                        -I~(fix-path (build-path *top-builddir* \"gc/include\"))"
-                          ,#"--ldflags=-L~(fix-path (build-path *top-srcdir* \"src\")) \
-                                       -L~(fix-path (build-path *top-builddir* \"src\"))"
-                          "foo"
-                          "foo.c" "foo--bar1.c" "foo--bar2.c" "foo--bar3.c")
-                        :directory "test.o")
-           (cond-expand
-            [gauche.os.windows
-             ;; On Windows, foo.dll can't be removed if process using it exists,
-             ;; so we run another gosh in subprocess.
-             (with-output-to-file "test.o/t.scm"
-               (^[]
-                 (write '(add-load-path "."))
-                 (write '(load "foo"))
-                 (write '(write
-                          `(,((global-variable-ref 'foo 'foo-literals))
-                            ,((global-variable-ref 'foo 'foo-begin1))
-                            ,((global-variable-ref 'foo 'foo-begin2))
-                            ,((global-variable-ref 'foo 'foo-include1))
-                            ,((global-variable-ref 'foo 'foo-include2)))))
-                 (write '(exit 0))))
-             (let* ([p (run-process '("../../src/gosh" "-ftest" "./t.scm")
-                                    :output :pipe :directory "test.o")]
-                    [result (read (process-output p))])
-               (process-wait p)
-               result)]
-            [else
-             (load "foo" :paths '("./test.o"))
-             (list ((global-variable-ref 'foo 'foo-literals))
-                   ((global-variable-ref 'foo 'foo-begin1))
-                   ((global-variable-ref 'foo 'foo-begin2))
-                   ((global-variable-ref 'foo 'foo-include1))
-                   ((global-variable-ref 'foo 'foo-include2)))]))
+         (dynload-and-eval
+          "foo"
+          (list ((global-variable-ref 'foo 'foo-literals))
+                ((global-variable-ref 'foo 'foo-begin1))
+                ((global-variable-ref 'foo 'foo-begin2))
+                ((global-variable-ref 'foo 'foo-include1))
+                ((global-variable-ref 'foo 'foo-include2))))
          literal=?)
+
+  (test* "literal sharing" '(#t #t #t #t #t #t)
+         (dynload-and-eval
+          "foo"
+          (let1 vs ((global-variable-ref 'foo 'foo-shared-literals))
+            (list
+             ;; See if partial lists are shared
+             (eq? (cdr (assq-ref vs 'list1))
+                  (assq-ref vs 'list2))
+             (eq? (cdr (assq-ref vs 'list2))
+                  (assq-ref vs 'list3))
+             ;; See if vectors are shared
+             (eq? (assq-ref vs 'vec1)
+                  (assq-ref vs 'vec2))
+             (eq? (assq-ref vs 'uvec1)
+                  (assq-ref vs 'uvec2))
+             ;; See if strings are shared
+             (eq? (assq-ref vs 'str1)
+                  (assq-ref vs 'str2))
+             ;; Descriptive types
+             (eq? (assq-ref vs 'dtype1)
+                  (assq-ref vs 'dtype2))
+             ))))
+  )
+
+(define (precomp-test-3)
+  (test* "running precomp 3" #t (do-precomp! '("types-test.scm") '("-e")))
+  (test* "compile 3" #t (do-compile! "types-test" '("types-test.c")))
+
+  (test* "type reconstruction" '(#t #t #f #t)
+         (dynload-and-eval
+          "types-test"
+          (list
+           (eq? (global-variable-ref 'types-test '<A>)
+                (</> (<Tuple> (<?> <int8>) <string> <integer>)
+                     (<List> <integer> 3 10)))
+           (eq? (global-variable-ref 'types-test '<A>)
+                (global-variable-ref 'types-test '<B>))
+           (eq? (global-variable-ref 'types-test '<A>)
+                (global-variable-ref 'types-test '<C>))
+           ;; NB: The following may not be eq?, since (<List> <integer>) is
+           ;; serialized as (<List> <integer> #f #f).  The memoization is
+           ;; based on the argument list of the type constructor, so it
+           ;; doesn't match with (<List> <integer>) at runtime.
+           (equal? (global-variable-ref 'types-test '<C>)
+                   (</> (<Tuple> (<?> <int8>) <string> <integer>)
+                        (<List> <integer>)))
+           )))
+
+  (test* "assertion with reconstructed type" '((3 "ok" 8483958394)
+                                               (#f "ok" -8483958394)
+                                               (1 2 3 4 5))
+         (dynload-and-eval
+          "types-test"
+          (list
+           ((global-variable-ref 'types-test 'foo) '(3 "ok" 8483958394))
+           ((global-variable-ref 'types-test 'foo) '(#f "ok" -8483958394))
+           ((global-variable-ref 'types-test 'foo) '(1 2 3 4 5)))))
+
+  (test* "assertion with reconstructed type"
+         #t
+         (dynload-and-eval
+          "types-test"
+          (guard (e ((<error> e)
+                     (boolean (#/supposed to be of type/ (~ e'message)))))
+            ((global-variable-ref 'types-test 'foo) "ng"))))
   )
 
 (wrap-with-test-directory precomp-test-1 '("test.o"))
 (wrap-with-test-directory precomp-test-2 '("test.o"))
+(wrap-with-test-directory precomp-test-3 '("test.o"))
 
 ;;=======================================================================
 (test-section "build-standalone")
@@ -688,11 +803,11 @@
 
 (define (static-test-1)
   (test* "static link test" #t
-         (run-build-static `("-o" "staticmain" 
+         (run-build-static `("-o" "staticmain"
                              "-I" ,(build-path *top-srcdir* "test/test-static"))
-                           `(,(fix-path (build-path *top-srcdir* 
+                           `(,(fix-path (build-path *top-srcdir*
                                                     "test/test-static/main.scm"))
-                             "foo/bar.scm" 
+                             "foo/bar.scm"
                              "foo/bar-impl.scm"
                              "foo/baz.scm")))
   (test* "static link executable" "ARGS: (A b CdE)"
@@ -708,7 +823,7 @@
 
 (define (test-script-test-1)
   (with-output-to-file "test.o/script"
-    (^[] 
+    (^[]
       (write `(use gauche.uvector))
       (write `(print "bar"))
       (write `(define (bar) (foo) (foo)))
