@@ -3,7 +3,7 @@
 (use gauche.test)
 
 (test-start "multibyte (sjis)")
-(use srfi-1)
+(use scheme.list)
 
 ;;-------------------------------------------------------------------
 (test-section "string builtins")
@@ -54,82 +54,117 @@
 (test* "string-join" "ӂI΂I΂"
        (string-join '("ӂ" "΂" "΂") "I" 'strict-infix))
 
-(test* "string-scan" 7
-       (string-scan "" ""))
-(test* "string-scan" ""
-       (string-scan "" "" 'before))
-(test* "string-scan" ""
-       (string-scan "" "" 'after))
-(test* "string-scan" '("" "")
-       (receive r (string-scan "" "" 'before*) r))
-(test* "string-scan" '("" "")
-       (receive r (string-scan "" "" 'after*) r))
-(test* "string-scan" '("" "")
-       (receive r (string-scan "" "" 'both) r))
-(test* "string-scan" #f
-       (string-scan "" ""))
+(let ()
+  (define (test-string-scan out s1 s2 . opt)
+    (if (pair? out)
+      (begin
+        (test* "string-scan" (car out) (apply string-scan s1 s2 opt))
+        (test* "string-scan-right" (cadr out)
+               (apply string-scan-right s1 s2 opt)))
+      (apply test-string-scan (list out out) s1 s2 opt)))
+  (define (test-string-scan2 out1 out2 s1 s2 . opt)
+    (if (pair? out1)
+      (begin
+        (test* "string-scan" (list (car out1) (car out2))
+               (receive r (apply string-scan s1 s2 opt) r))
+        (test* "string-scan-right" (list (cadr out1) (cadr out2))
+               (receive r (apply string-scan-right s1 s2 opt) r)))
+      (apply test-string-scan2 (list out1 out1) (list out2 out2) s1 s2 opt)))
+
+  (test-string-scan '(7 20)
+                    "" "")
+  (test-string-scan '("" "")
+                      "" "" 'before)
+  (test-string-scan '("" "")
+                    "" "" 'after)
+  (test-string-scan2 '("" "")
+                     '("" "")
+                     "" "" 'before*)
+  (test-string-scan2 '("" "")
+                     '("" "")
+                     "" "" 'after*)
+  (test-string-scan2 '("" "")
+                     '("" "")
+                     "" "" 'both)
+  (test-string-scan #f "" "")
+  )
 
 ;;-------------------------------------------------------------------
-(test-section "string-pointer")
-(define sp #f)
-(test* "make-string-pointer" #t
+(test-section "string-cursor")
+
+(define str "͂hoւ")
+(define ss #f)
+(define se #f)
+(test* "string-cursor-start" #t
        (begin
-         (set! sp (make-string-pointer "͂hoւ"))
-         (string-pointer? sp)))
-(test* "string-pointer-next!" #\
-       (string-pointer-next! sp))
-(test* "string-pointer-next!" #\
-       (string-pointer-next! sp))
-(test* "string-pointer-next!" #\
-       (string-pointer-next! sp))
-(test* "string-pointer-next!" #\
-       (string-pointer-next! sp))
-(test* "string-pointer-next!" #\h
-       (string-pointer-next! sp))
-(test* "string-pointer-next!" #\o
-       (string-pointer-next! sp))
-(test* "string-pointer-next!" #\
-       (string-pointer-next! sp))
-(test* "string-pointer-prev!" #\
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\o
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\h
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #t
-       (eof-object? (string-pointer-prev! sp)))
-(test* "string-pointer-index" 0
-       (string-pointer-index sp))
-(test* "string-pointer-index" 8
-       (do ((x (string-pointer-next! sp) (string-pointer-next! sp)))
-           ((eof-object? x) (string-pointer-index sp))))
-(test* "string-pointer-substring" '("͂hoւ" "")
-       (list (string-pointer-substring sp)
-             (string-pointer-substring sp :after #t)))
-(test* "string-pointer-substring" '("͂h" "oւ")
+         (set! ss (string-cursor-start str))
+         (string-cursor? ss)))
+(test* "string-cursor-start" #t
        (begin
-         (string-pointer-set! sp 5)
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
-(test* "string-pointer-substring" '("" "͂hoւ")
-       (begin
-         (string-pointer-set! sp 0)
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
+         (set! se (string-cursor-end str))
+         (string-cursor? se)))
+(test* "string-cursor-next" #\
+       (string-ref str (string-cursor-next str ss)))
+(test* "string-cursor-forward" #\
+       (string-ref str (string-cursor-forward str ss 3)))
+(test* "string-cursor-prev" #\
+       (string-ref str (string-cursor-prev str se)))
+(test* "string-cursor-back" #\
+       (string-ref str (string-cursor-back str se 5)))
+(test* "string-index->cursor" #\
+       (string-ref str (string-index->cursor str 6)))
+(test* "string-cursor->index" 0
+       (string-cursor->index str ss))
+(test* "string-cursor->index" 8
+       (string-cursor->index str se))
+(test* "substring with cursors" "͂ho"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "͂ho"
+       (substring str
+                  2
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "͂h"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  5))
+(test* "string->list" '(#\ #\ #\ #\h #\o #\)
+       (string->list str
+                     (string-cursor-next str ss)
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\ #\ #\ #\h #\o #\)
+       (string->list str
+                     1
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\ #\ #\ #\h #\o #\)
+       (string->list str
+                     (string-cursor-next str ss)
+                     7))
+(test* "string-cursor=?" #t
+       (string-cursor=? ss (string-cursor-back str se 8)))
+(test* "string-cursor=?" #t
+       (string-cursor=? 0 0))
+(test* "string-cursor<?" #t
+       (string-cursor<? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss ss))
+(test* "string-cursor>?" #t
+       (string-cursor>? se ss))
+(test* "string-cursor>=?" #t
+       (string-cursor>=? se se))
+(test* "string-cursor-diff" 8
+       (string-cursor-diff str ss se))
+(test* "string-cursor-diff" -8
+       (string-cursor-diff str se ss))
 
 ;;-------------------------------------------------------------------
 (test-section "incomplete strings")
 
 (test* "string-length" 6 (string-length #*""))
-(test* "string-complete->incomplete" #*"" 
+(test* "string-complete->incomplete" #*""
        (string-complete->incomplete ""))
 (test* "string-complete->incomplete" #*""
        (string-complete->incomplete #*""))
@@ -178,8 +213,8 @@
 (test* "string-every" #t (string-every #[-] ""))
 (test* "string-every" #f (string-every #[-] "a"))
 (test* "string-every" #t (string-every #[-] ""))
-(test* "string-every" #t (string-every (lambda (x) (char-ci=? x #\)) ""))
-(test* "string-every" #f (string-every (lambda (x) (char-ci=? x #\)) ""))
+(test* "string-every" #t (string-every (^x (char-ci=? x #\)) ""))
+(test* "string-every" #f (string-every (^x (char-ci=? x #\)) ""))
 
 (test* "string-any" #t (string-any #\ ""))
 (test* "string-any" #f (string-any #\ ""))
@@ -187,8 +222,8 @@
 (test* "string-any" #t (string-any #[-] "["))
 (test* "string-any" #f (string-any #[-] "XL["))
 (test* "string-any" #f (string-any #[-] ""))
-(test* "string-any" #t (string-any (lambda (x) (char-ci=? x #\)) "炠"))
-(test* "string-any" #f (string-any (lambda (x) (char-ci=? x #\)) "A"))
+(test* "string-any" #t (string-any (^x (char-ci=? x #\)) "炠"))
+(test* "string-any" #f (string-any (^x (char-ci=? x #\)) "A"))
 (test* "string-tabulate" "ABCDE"
        (string-tabulate (lambda (code)
                           (integer->char (+ code
@@ -213,35 +248,70 @@
 ;;-------------------------------------------------------------------
 (test-section "char set")
 
-(use srfi-14)
+(use scheme.charset)
 
-(test* "char-set=" #t
-       (char-set= (char-set #\ #\ #\ #\ #\)
-                  (string->char-set "")))
-(test* "char-set=" #t
-       (char-set= (list->char-set '(#\ #\ #\ #\))
-                  (string->char-set "񂢂")))
-(test* "char-set=" #t
-       (char-set= (->char-set "")
+(let ((data '(#[--@-]
+              #[aA])))
+  (define (t x)
+    (test* "mutable/immutable roundtrip" #t
+           (char-set= (char-set-copy x)
+                      (char-set-copy (char-set-freeze! (char-set-copy x))))))
+  (for-each t data))
+
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\)
+                 (char-set #\ #\ #\ #\ #\))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\)
+                 (char-set #\ #\ #\ #\ #\))
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\)
+                 (char-set-union
                   (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))
+                  (integer-range->char-set (char->integer #\@)
                                            (char->integer #\))))
-
-(test* "char-set" #t
-       (char-set<= (list->char-set '(#\ #\))
-                   char-set:full))
-(test* "char-set<=" #t (char-set<= #[] #[]))
-(test* "char-set<=" #t (char-set<= #[] #[-]))
-(test* "char-set<=" #f (char-set<= #[-] #[]))
-(test* "char-set<=" #t (char-set<= #[--] #[-]))
-(test* "char-set<=" #f (char-set<= #[-] #[--]))
-(test* "char-set<=" #f (char-set<= #[--] #[-]))
-(test* "char-set<=" #f (char-set<= #[--] #[-]))
-(test* "char-set<=" #t (char-set<= #[--] #[--]))
-(test* "char-set<=" #t (char-set<= #[--] #[--]))
-(test* "char-set<=" #t (char-set<= #[--] #[--]))
-(test* "char-set<=" #t (char-set<= #[-] #[--]))
-(test* "char-set<=" #t (char-set<= #[--] #[--]))
-(test* "char-set<=" #f (char-set<= #[-] #[--]))
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\P)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))
+                  (integer-range->char-set (char->integer #\@)
+                                           (char->integer #\))))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))
+                  (integer-range->char-set (char->integer #\@)
+                                           (char->integer #\))))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))
+                  (integer-range->char-set (char->integer #\@)
+                                           (char->integer #\))))
+(test-char-set-2 "char-set=" #t char-set=
+                 (char-set #\ #\ #\ #\ #\)
+                 (string->char-set ""))
+(test-char-set-2 "char-set=" #t char-set=
+                 (list->char-set '(#\ #\ #\ #\))
+                 (string->char-set "񂢂"))
+(test-char-set-2 "char-set=" #t char-set=
+                 (->char-set "")
+                 (integer-range->char-set (char->integer #\)
+                                          (char->integer #\)))
+(test-char-set-2 "char-set<=" #t char-set<=
+                 (list->char-set '(#\ #\))
+                 char-set:full)
+(test-char-set-2 "char-set<=" #t char-set<= #[] #[])
+(test-char-set-2 "char-set<=" #t char-set<= #[] #[-])
+(test-char-set-2 "char-set<=" #f char-set<= #[-] #[])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[-])
+(test-char-set-2 "char-set<=" #f char-set<= #[-] #[--])
+(test-char-set-2 "char-set<=" #f char-set<= #[--] #[-])
+(test-char-set-2 "char-set<=" #f char-set<= #[--] #[-])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[--])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[--])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[--])
+(test-char-set-2 "char-set<=" #t char-set<= #[-] #[--])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[--])
+(test-char-set-2 "char-set<=" #f char-set<= #[-] #[--])
 
 ;;-------------------------------------------------------------------
 (test-section "ports")
@@ -401,19 +471,19 @@
 
 (define (byte-list->port p bytes)
   (set! *flusher-out* '())
-  (for-each (lambda (b) (write-byte b p)) bytes)
+  (for-each (^b (write-byte b p)) bytes)
   (close-output-port p)
   *flusher-out*)
 
 (define (char-list->port p chars)
   (set! *flusher-out* '())
-  (for-each (lambda (c) (write-char c p)) chars)
+  (for-each (^c (write-char c p)) chars)
   (close-output-port p)
   *flusher-out*)
 
 (define (string-list->port p strs)
   (set! *flusher-out* '())
-  (for-each (lambda (s) (display s p)) strs)
+  (for-each (^s (display s p)) strs)
   (close-output-port p)
   *flusher-out*)
 
