@@ -23,6 +23,31 @@
 (test* "immutable" #f (string-immutable? (string #\a #\b)))
 (test* "immutable" #f (string-immutable? (string)))
 
+(let ([data '(("abc" "abc" =)
+              (""    "abc" <)
+              ("abd" "abc" >)
+              (#*"abc" #*"abc" =)
+              (#*"a"   #*"abc" <)
+              (#*"abd" #*"abc" >)
+              ("abc" #*"abc" <)
+              ("abc" #*"abb" >)
+              (#*"abc" "abc" >)
+              (#*"ab"  "abc" <))])
+  (define (t a b result)
+    (test* (string-append "compare " a " and " b)
+           (list (boolean (memq result '(=)))
+                 (boolean (memq result '(<)))
+                 (boolean (memq result '(< =)))
+                 (boolean (memq result '(>)))
+                 (boolean (memq result '(> =))))
+           (list (string=? a b)
+                 (string<? a b)
+                 (string<=? a b)
+                 (string>? a b)
+                 (string>=? a b))))
+  (dolist [x data]
+    (apply t x)))
+
 (test* "string->list" '(#\a #\b #\c #\d #\e #\f #\g)
        (string->list "abcdefg"))
 (test* "string->list" '(#\c #\d #\e #\f #\g)
@@ -33,10 +58,6 @@
        (string->list "abcdefg" 0 1)) ;srfi-13 extension
 (test* "string->list" '() (string->list ""))
 
-;; this should be switched by native encoding.
-;(test* "string w/ char >= \\x80" (integer->char #xa1)
-;       (string-ref (string (integer->char #xa1)) 0))
-
 (test* "string-copy" '("abcde" #f)
        (let* ((x "abcde") (y (string-copy x)))
          (list y (eq? x y))))
@@ -47,6 +68,28 @@
 (define x (string-copy "abcde"))
 (test* "string-set!" "abZde" (begin (string-set! x 2 #\Z) x))
 
+(test* "string w/ char >= \\x80" #\u00a1
+       (string-ref (string #\u00a1) 0))
+
+(test* "string-reader hex-escape" '(1 2 3)
+       (let1 s "\x1;\x2;\x00003;"
+         (map (^i (char->integer (string-ref s i))) '(0 1 2))))
+(test* "string-reader hex-escape legacy support" '(1 2 0 48 48 51)
+       (let1 s "\x01\x02\x00003"
+         (map (^i (char->integer (string-ref s i))) '(0 1 2 3 4 5))))
+(test* "string-reader hex-escape error cases 1 (no hexdigits)"
+       (test-error)
+       (read-from-string "\"\\x;\""))
+(test* "string-reader hex-escape error cases 2 (integer overflow)"
+       (test-error)
+       (read-from-string "\"\\x11111111111111111111111111111111;\""))
+(test* "string-reader hex-escape error cases 3 (out of unicode range)"
+       (test-error)
+       (read-from-string "\"\\x111111;\""))
+(test* "string-reader hex-escape legacy fallback" "\x11;1111"
+       (read-from-string "\"\\x111111\""))
+
+
 (test* "string-fill!" "ZZZZZZ"
        (string-fill! (string-copy "000000") #\Z))
 (test* "string-fill!" "000ZZZ"
@@ -73,33 +116,85 @@
 (test* "string-join" "foo::bar::baz"
        (string-join '("foo" "bar" "baz") "::" 'strict-infix))
 
-(test* "string-scan" 3 (string-scan "abcdefghi" "def"))
-(test* "string-scan" 3 (string-scan "abcdefghi" "def" 'index))
-(test* "string-scan" "abc" (string-scan "abcdefghi" "def" 'before))
-(test* "string-scan" "ghi" (string-scan "abcdefghi" "def" 'after))
-(test* "string-scan" '("abc" "defghi")
-       (receive r (string-scan "abcdefghi" "def" 'before*) r))
-(test* "string-scan" '("abcdef" "ghi")
-       (receive r (string-scan "abcdefghi" "def" 'after*) r))
-(test* "string-scan" '("abc" "ghi")
-       (receive r (string-scan "abcdefghi" "def" 'both) r))
+(let ()
+  (define (test-string-scan out s1 s2 . opt)
+    (if (pair? out)
+      (begin
+        (test* "string-scan" (car out) (apply string-scan s1 s2 opt))
+        (test* "string-scan-right" (cadr out)
+               (apply string-scan-right s1 s2 opt)))
+      (apply test-string-scan (list out out) s1 s2 opt)))
+  (define (test-string-scan2 out1 out2 s1 s2 . opt)
+    (if (pair? out1)
+      (begin
+        (test* "string-scan" (list (car out1) (car out2))
+               (receive r (apply string-scan s1 s2 opt) r))
+        (test* "string-scan-right" (list (cadr out1) (cadr out2))
+               (receive r (apply string-scan-right s1 s2 opt) r)))
+      (apply test-string-scan2 (list out1 out1) (list out2 out2) s1 s2 opt)))
 
-(test* "string-scan" 4 (string-scan "abcdefghi" #\e))
-(test* "string-scan" "abcd" (string-scan "abcdefghi" #\e 'before))
-(test* "string-scan" "fghi" (string-scan "abcdefghi" #\e 'after))
-(test* "string-scan" '("abcd" "efghi")
-       (receive r (string-scan "abcdefghi" #\e 'before*) r))
-(test* "string-scan" '("abcde" "fghi")
-       (receive r (string-scan "abcdefghi" #\e 'after*) r))
-(test* "string-scan" '("abcd" "fghi")
-       (receive r (string-scan "abcdefghi" #\e 'both) r))
+  (test-string-scan 3 "abcdefghi" "def")
+  (test-string-scan 3 "abcdefghi" "def" 'index)
+  (test-string-scan 6 "abcdefghi" "ghi")
+  (test-string-scan 0 "abcdefghi" "abc")
+  (test-string-scan '(0 6) "abcabcabc" "abc")
+  (test-string-scan "abc" "abcdefghi" "def" 'before)
+  (test-string-scan "ghi" "abcdefghi" "def" 'after)
+  (test-string-scan2 "abc" "defghi" "abcdefghi" "def" 'before*)
+  (test-string-scan2 "abcdef" "ghi" "abcdefghi" "def" 'after*)
+  (test-string-scan2 "abc" "ghi" "abcdefghi" "def" 'both)
 
-(test* "string-scan (boyer-moore)" 216
-       (string-scan "abracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabr"
-                    "abracadabra"))
+  (test-string-scan 4 "abcdefghi" #\e)
+  (test-string-scan 8 "abcdefghi" #\i)
+  (test-string-scan 0 "abcdefghi" #\a)
+  (test-string-scan '(0 8) "abcdefgha" #\a)
+  (test-string-scan "abcd" "abcdefghi" #\e 'before)
+  (test-string-scan "fghi" "abcdefghi" #\e 'after)
+  (test-string-scan2 "abcd" "efghi" "abcdefghi" #\e 'before*)
+  (test-string-scan2 "abcde" "fghi" "abcdefghi" #\e 'after*)
+  (test-string-scan2 "abcd" "fghi" "abcdefghi" #\e 'both)
 
-(test* "string-scan (special case)" 0
-       (string-scan "abakjrgaker" ""))
+  ;; this tests boyer-moore
+  (test-string-scan 216
+                    "abracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabr"
+                    "abracadabra")
+  (test-string-scan '(0 365)
+                    "axaxcadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababaxaxax"
+                    "axax")
+
+  ;; results differ between leftmost and rightmost
+  (test-string-scan '(1 8) "abracadabra" "br")
+  (test-string-scan '("acadabra" "a") "abracadabra" "br" 'after)
+  (test-string-scan '("a" "abracada") "abracadabra" "br" 'before)
+  (test-string-scan2 '("abr" "abracadabr") '("acadabra" "a")
+                     "abracadabra" "br" 'after*)
+  (test-string-scan2 '("a" "abracada") '("bracadabra" "bra")
+                     "abracadabra" "br" 'before*)
+  (test-string-scan2 '("a" "abracada") '("acadabra" "a")
+                     "abracadabra" "br" 'both)
+
+  ;; edge case
+  (test-string-scan '(0 11) "abakjrgaker" "")
+  (test-string-scan '("abakjrgaker" "") "abakjrgaker" "" 'after)
+  (test-string-scan '("" "abakjrgaker") "abakjrgaker" "" 'before)
+  (test-string-scan2 '("" "abakjrgaker") '("abakjrgaker" "")
+                     "abakjrgaker" "" 'after*)
+  (test-string-scan2 '("" "abakjrgaker") '("abakjrgaker" "")
+                     "abakjrgaker" "" 'before*)
+  (test-string-scan2 '("" "abakjrgaker") '("abakjrgaker" "")
+                     "abakjrgaker" "" 'both)
+
+  ;; incomplete strings
+  (test-string-scan 3 #*"abcdefghi" "def")
+  (test-string-scan 3 "abcdefghi" #*"def")
+  (test-string-scan #*"ghi" #*"abcdefghi" "def" 'after)
+  (test-string-scan #*"abc" #*"abcdefghi" "def" 'before)
+  (test-string-scan2 #*"abcdef" #*"ghi" #*"abcdefghi" "def" 'after*)
+  (test-string-scan2 #*"abc" #*"defghi" #*"abcdefghi" "def" 'before*)
+  (test-string-scan2 #*"abc" #*"ghi" #*"abcdefghi" "def" 'both)
+  (test-string-scan2 #*"abc" #*"ghi" "abcdefghi" #*"def" 'both)
+  (test-string-scan2 #*"abcd" #*"fghi" #*"abcdefghi" #\e 'both)
+  )
 
 ;;-------------------------------------------------------------------
 (test-section "string-split")
@@ -112,7 +207,7 @@
        (string-split "aa*bbb*c**" #\*))
 (test* "string-split (char)" '("aa")
        (string-split "aa" #\*))
-(test* "string-split (char)" '("")
+(test* "string-split (char)" '()
        (string-split "" #\*))
 (test* "string-split (char)" '("" "")
        (string-split "*" #\*))
@@ -126,7 +221,7 @@
        (string-split "aa**bbb**c*c" "--"))
 (test* "string-split (string)" '("aa" "bbb" "c*c" "")
        (string-split "aa**bbb**c*c**" "**"))
-(test* "string-split (string)" '("")
+(test* "string-split (string)" '()
        (string-split "" "**"))
 (test* "string-split (string)" '("" "")
        (string-split "**" "**"))
@@ -141,14 +236,14 @@
        (string-split "--aa--bbb---c-c-" #/\w+/))
 (test* "string-split (regexp)" '("--aa--bbb---c-c-")
        (string-split "--aa--bbb---c-c-" #/z+/))
-(test* "string-split (regexp)" (test-error) ;; test detection of infinite loop
+(test* "string-split (regexp)" '("" "a" "a" "" "b" "b" "b" "" "c" "" "c" "" "")
        (string-split "--aa--bbb---c-c-" #/-*/))
 
 (test* "string-split (charset)" '("aa" "bbb" "c" "d")
        (string-split "aa---bbb***c&d" #[\W]))
 (test* "string-split (charset)" '("" "---" "***" "&" "")
        (string-split "aa---bbb***c&d" #[\w]))
-(test* "string-split (charset)" '("")
+(test* "string-split (charset)" '()
        (string-split "" #[\w]))
 (test* "string-split (charset)" '("" "")
        (string-split "a" #[\w]))
@@ -156,6 +251,99 @@
 (test* "string-split (predicate)" '("" "---" "***" "&" "")
        (string-split "aa---bbb***c&d" char-alphabetic?))
 
+(test-section "string-split(with limit)")
+
+(test* "string-split (char)" '("aa*bbb*c**")
+       (string-split "aa*bbb*c**" #\* 0))
+(test* "string-split (char)" '("aa" "bbb*c**")
+       (string-split "aa*bbb*c**" #\* 1))
+(test* "string-split (char)" '("aa" "bbb" "c**")
+       (string-split "aa*bbb*c**" #\* 2))
+
+(test* "string-split (char)" '("aa")
+       (string-split "aa" #\* 1))
+(test* "string-split (char)" '()
+       (string-split "" #\* 2))
+(test* "string-split (char)" '("" "")
+       (string-split "*" #\* 3))
+(test* "string-split (char)" '("*")
+       (string-split "*" #\* 0))
+
+(test* "string-split (1-char string)" '("aa" "bbb*c")
+       (string-split "aa*bbb*c" "*" 1))
+(test* "string-split (1-char string)" '("aa" "bbb" "c")
+       (string-split "aa*bbb*c" "*" 2))
+
+(test* "string-split (string)" '("aa**bbb**c*c")
+       (string-split "aa**bbb**c*c" "**" 0))
+(test* "string-split (string)" '("aa" "bbb**c*c")
+       (string-split "aa**bbb**c*c" "**" 1))
+(test* "string-split (string)" '("aa" "bbb" "c*c")
+       (string-split "aa**bbb**c*c" "**" 2))
+(test* "string-split (string)" '("aa" "bbb" "c*c")
+       (string-split "aa**bbb**c*c" "**" 3))
+(test* "string-split (string)" '("aa**bbb**c*c")
+       (string-split "aa**bbb**c*c" "--" 2))
+(test* "string-split (string)" '("aa**bbb**c*c**")
+       (string-split "aa**bbb**c*c**" "**" 0))
+(test* "string-split (string)" '("aa" "bbb**c*c**")
+       (string-split "aa**bbb**c*c**" "**" 1))
+(test* "string-split (string)" '("aa" "bbb" "c*c**")
+       (string-split "aa**bbb**c*c**" "**" 2))
+(test* "string-split (string)" '("aa" "bbb" "c*c" "")
+       (string-split "aa**bbb**c*c**" "**" 3))
+(test* "string-split (string)" '()
+       (string-split "" "**" 0))
+(test* "string-split (string)" '()
+       (string-split "" "**" 1))
+(test* "string-split (string)" '("**")
+       (string-split "**" "**" 0))
+(test* "string-split (string)" '("" "")
+       (string-split "**" "**" 1))
+
+(test* "string-split (regexp)" '("aa--bbb--c-c")
+       (string-split "aa--bbb--c-c" #/-+/ 0))
+(test* "string-split (regexp)" '("aa" "bbb--c-c")
+       (string-split "aa--bbb--c-c" #/-+/ 1))
+(test* "string-split (regexp)" '("aa" "bbb" "c-c")
+       (string-split "aa--bbb--c-c" #/-+/ 2))
+(test* "string-split (regexp)" '("aa" "bbb" "c" "c")
+       (string-split "aa--bbb--c-c" #/-+/ 3))
+(test* "string-split (regexp)" '("aa" "bbb" "c" "c")
+       (string-split "aa--bbb--c-c" #/-+/ 4))
+(test* "string-split (regexp)" '("aa--bbb---c-c")
+       (string-split "aa--bbb---c-c" #/--/ 0))
+(test* "string-split (regexp)" '("aa" "bbb---c-c")
+       (string-split "aa--bbb---c-c" #/--/ 1))
+(test* "string-split (regexp)" '("aa" "bbb" "-c-c")
+       (string-split "aa--bbb---c-c" #/--/ 2))
+(test* "string-split (regexp)" '("aa" "bbb" "-c-c")
+       (string-split "aa--bbb---c-c" #/--/ 3))
+(test* "string-split (regexp)" '("--aa--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 0))
+(test* "string-split (regexp)" '("" "aa--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 1))
+(test* "string-split (regexp)" '("" "aa" "bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 2))
+(test* "string-split (regexp)" '("" "aa" "bbb" "c-c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 3))
+(test* "string-split (regexp)" '("" "aa" "bbb" "c" "c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 4))
+(test* "string-split (regexp)" '("--aa--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/\w+/ 0))
+(test* "string-split (regexp)" '("--" "--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/\w+/ 1))
+(test* "string-split (regexp)" '("--" "--" "---c-c-")
+       (string-split "--aa--bbb---c-c-" #/\w+/ 2))
+(test* "string-split (regexp)" '("--" "--" "---" "-c-")
+       (string-split "--aa--bbb---c-c-" #/\w+/ 3))
+(test* "string-split (regexp)" '("--aa--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/z+/ 2))
+(test* "string-split (regexp)" '("" "a" "a" "--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/-*/ 3))
+(test* "string-split (bad limit)" (test-error)
+       (string-split "--aa--bbb---c-c-" #/-+/ 'a))
+
 ;;-------------------------------------------------------------------
 (test-section "incomplete strings")
 
@@ -198,18 +386,6 @@
 (test* "string-join" #*"a:b:c"
        (string-join '("a" "b" "c") #*":"))
 
-(test* "string-scan" 3
-       (string-scan #*"abcdefghi" "def"))
-(test* "string-scan" 3
-       (string-scan "abcdefghi" #*"def"))
-(test* "string-scan" '(#*"abc" #*"ghi")
-       (receive r (string-scan #*"abcdefghi" "def" 'both) r))
-(test* "string-scan" '(#*"abc" #*"ghi")
-       (receive r (string-scan "abcdefghi" #*"def" 'both) r))
-(test* "string-scan" '(#*"abcd" #*"fghi")
-       (receive r (string-scan #*"abcdefghi" #\e 'both) r))
-
-
 ;; NB: should we allow this?
 (test* "string-set!" #*"abQde"
        (let ((s (string-copy #*"abcde")))
@@ -236,69 +412,109 @@
        (substring #*"abcde" 0 2))
 
 ;;-------------------------------------------------------------------
-(test-section "string-pointer")
+(test-section "string-cursor")
 
-(define sp #f)
-(test* "make-string-pointer" #t
+(define str "abcdefg")
+(define ss #f)
+(define se #f)
+(test* "string-cursor-start" #t
        (begin
-         (set! sp (make-string-pointer "abcdefg"))
-         (string-pointer? sp)))
-(test* "string-pointer-next!" #\a
-       (string-pointer-next! sp))
-(test* "string-pointer-next!" #\b
-       (string-pointer-next! sp))
-(test* "string-pointer-prev!" #\b
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\a
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #t
-       (eof-object? (string-pointer-prev! sp)))
-(test* "string-pointer-index" 0
-       (string-pointer-index sp))
-(test* "string-pointer-index" 7
-       (do ((x (string-pointer-next! sp) (string-pointer-next! sp)))
-           ((eof-object? x) (string-pointer-index sp))))
-(test* "string-pointer-substring" '("abcdefg" "")
-       (list (string-pointer-substring sp)
-             (string-pointer-substring sp :after #t)))
-(test* "string-pointer-substring" '("abcd" "efg")
+         (set! ss (string-cursor-start str))
+         (string-cursor? ss)))
+(test* "string-cursor-start" #t
        (begin
-         (string-pointer-set! sp 4)
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
-(test* "string-pointer-substring" '("" "abcdefg")
-       (begin
-         (string-pointer-set! sp 0)
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
-(test* "string-pointer-substring" '("" "")
-       (let ((sp (make-string-pointer "")))
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
+         (set! se (string-cursor-end str))
+         (string-cursor? se)))
+(test* "string-cursor-next" #\b
+       (string-ref str (string-cursor-next str ss)))
+(test* "string-cursor-forward" #\d
+       (string-ref str (string-cursor-forward str ss 3)))
+(test* "string-cursor-prev" #\g
+       (string-ref str (string-cursor-prev str se)))
+(test* "string-cursor-back" #\d
+       (string-ref str (string-cursor-back str se 4)))
+(test* "string-index->cursor" #\f
+       (string-ref str (string-index->cursor str 5)))
+(test* "string-cursor->index" 0
+       (string-cursor->index str ss))
+(test* "string-cursor->index" 7
+       (string-cursor->index str se))
+(test* "substring with cursors" "cde"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "cde"
+       (substring str
+                  2
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "cde"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  5))
+(test* "string->list" '(#\b #\c #\d #\e #\f)
+       (string->list str
+                     (string-cursor-next str ss)
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\b #\c #\d #\e #\f)
+       (string->list str
+                     1
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\b #\c #\d #\e #\f)
+       (string->list str
+                     (string-cursor-next str ss)
+                     6))
+(test* "string-cursor=?" #t
+       (string-cursor=? ss (string-cursor-back str se 7)))
+(test* "string-cursor=?" #t
+       (string-cursor=? 0 0))
+(test* "string-cursor<?" #t
+       (string-cursor<? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss ss))
+(test* "string-cursor>?" #t
+       (string-cursor>? se ss))
+(test* "string-cursor>=?" #t
+       (string-cursor>=? se se))
+(test* "string-cursor-diff" 7
+       (string-cursor-diff str ss se))
+(test* "string-cursor-diff" -7
+       (string-cursor-diff str se ss))
 
-(test* "make-string-pointer (bound)" #t
-       (begin
-         (set! sp (make-string-pointer "abcdefg" 1 2 5))
-         (string-pointer? sp)))
-(test* "string-pointer-next! (bound)" #\d
-       (string-pointer-next! sp))
-(test* "string-pointer-next! (bound)" #\e
-       (string-pointer-next! sp))
-(test* "string-pointer-next! (bound)" #t
-       (eof-object? (string-pointer-next! sp)))
-(test* "string-pointer-prev! (bound)" #\e
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev! (bound)" #\d
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev! (bound)" #\c
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev! (bound)" #t
-       (eof-object? (string-pointer-prev! sp)))
-(test* "string-pointer-next! (bound)" #\c
-       (string-pointer-next! sp))
-(test* "string-pointer-substring (bound)" '("c" "de")
-       (list (string-pointer-substring sp)
-             (string-pointer-substring sp :after #t)))
+;;-------------------------------------------------------------------
+(test-section "string mappers")
+
+(test* "string-map, single arg" "O'AFU"
+       (string-map char-upcase "o'afu"))
+(test* "string-map, single arg" ""
+       (string-map char-upcase ""))
+(test* "string-map, multi arg" "orpng"
+       (string-map (^[a b] (if (char>? a b) a b))
+                   "apple" "orange"))
+(test* "string-map, srfi-13 style" "CALIFRAGILISTICEXPIALIDOCIOUS"
+       (string-map char-upcase "supercalifragilisticexpialidocious" 5))
+(test* "string-map, srfi-13 style" "CALIFRAGILISTIC"
+       (string-map char-upcase "supercalifragilisticexpialidocious" 5 20))
+(test* "string-map, bad arg" (test-error)
+       (string-map (^ _ #\space) "abc" 'def))
+(test* "string-map, bad arg" (test-error)
+       (string-map (^ _ #\space) 'abc))
+(test* "string-map, bad arg" (test-error)
+       (string-map (^ _ #\space) "abc" "def" 2))
+(test* "string-map, bad arg" (test-error)
+       (string-map (^ _ #\space) "abc" 1 2 3))
+(test* "string-for-each single arg" "maui"
+       (with-output-to-string
+         (cut string-for-each write-char "maui")))
+(test* "string-for-each multi arg" "Niuihi"
+       (with-output-to-string
+         (cut string-for-each (^[a b] (write-char (if (char>? a b) a b)))
+              "Kaua'i" "Ni'ihau")))
+(test* "string-for-each srfi-13 style" "califragilistic"
+       (with-output-to-string
+         (cut string-for-each write-char
+              "supercalifragilisticexpialidocious" 5 20)))
 
 ;;-------------------------------------------------------------------
 (test-section "input string port")
@@ -371,6 +587,12 @@
        (read (open-input-string "\"0123 \\   \n   4567\"")))
 (test* "line continuation" "0123-4567"
        (read (open-input-string "\"0123\\\n \\  \n -4567\"")))
+(test* "line continuation" "abcd"
+       (read (open-input-string "\"abcd\\\n\"")))
+(test* "line continuation" "abcd2"
+       (read (open-input-string "\"abcd2\\\r\n \"")))
+(test* "line continuation" "abcd3"
+       (read (open-input-string "\"abcd3\\\r\t\"")))
 (test* "line continuation (invalid)" (test-error)
        (read (open-input-string "\"1234\\ x\"")))
 
@@ -423,7 +645,7 @@
 
 (define (string-port-tester . args)
   (let ((out (open-output-string)))
-    (for-each (lambda (s) (display s out)) args)
+    (for-each (^s (display s out)) args)
     (get-output-string out)))
 
 (define (test-string-port signature total seg)
@@ -454,7 +676,7 @@
 (test-string-ports "(mid)"
                    (* *dstr-init-size* (+ *dstr-incr-factor* 1))
                    (- *dstr-init-size* 1) *dstr-init-size* 3)
-(test-string-ports "(mid+1)" 
+(test-string-ports "(mid+1)"
                    (+ (* *dstr-init-size* (+ *dstr-incr-factor* 1)) 1)
                    (- *dstr-init-size* 1) *dstr-init-size* 3)
 (test-string-ports "(large)" 10000
@@ -464,21 +686,34 @@
                    (* *dstr-init-size* (+ *dstr-incr-factor* 1))
                    )
 
+;;-------------------------------------------------------------------
+(test-section "immutablility")
+
+(test* "string-immutable?" '(#t #f)
+       (list (string-immutable? "abc")
+             (string-immutable? (string-copy "abc"))))
+
+(test* "string-copy-immutable" '(#t #t #t #t)
+       (list (string-immutable? (string-copy-immutable "abc"))
+             (string-immutable? (string-copy-immutable "abc" 0 1))
+             (string-immutable? (string-copy-immutable (string-copy "abc")))
+             (string-immutable? (string-copy-immutable (string-copy "abc") 0 1))))
+
 ;;-------------------------------------------------------------------
 (test-section "string interpolation")
 
 (test* "string interpolation" "string interpolation"
        (let ((x "inter") (y "polation"))
-         #`"string ,|x|,|y|"))
+         #"string ~|x|~|y|"))
 (test "string interpolation" "string interpolation"
       (lambda ()
         (define (x) "inter")
         (define (y) "polation")
-        #`"string ,(x),(y)"))
+        #"string ~(x)~(y)"))
 (test "string interpolation" "string interpolation"
       (lambda ()
         (define (x a)
           (if a "inter" "polation"))
-        #`"string ,(x #t),(x #f)"))
+        #"string ~(x #t)~(x #f)"))
 
 (test-end)
