@@ -6,8 +6,7 @@
 
 (use gauche.test)
 (use gauche.config)
-(use srfi-1)
-(use srfi-11)                           ;let-values
+(use scheme.list)
 (use srfi-13)
 
 (test-start "system")
@@ -24,23 +23,24 @@
 (define (n pathname) (sys-normalize-pathname pathname))
 
 ;; some common operations via command
+;; NB: We do not use sys-* interface yet, for we're going to test them!
 (define (cmd-rmrf dir)
   (cond-expand
    (gauche.os.windows
-    (sys-system #`"rmdir /q /s ,(n dir) > NUL 2>&1")
-    (sys-system #`"del /q ,(n dir) > NUL 2>&1"))
+    (sys-system #"rmdir /q /s ~(n dir) > NUL 2>&1")
+    (sys-system #"del /q ~(n dir) > NUL 2>&1"))
    (else
-    (sys-system #`"rm -rf ,dir > /dev/null"))))
+    (sys-system #"rm -rf ~dir > /dev/null"))))
 
 (define (cmd-mkdir dir)
   (cond-expand
-   (gauche.os.windows (sys-system #`"mkdir ,(n dir)"))
-   (else (sys-system #`"mkdir ,dir"))))
+   (gauche.os.windows (sys-system #"mkdir ~(n dir)"))
+   (else (sys-system #"mkdir ~dir"))))
 
 (define (cmd-touch path)
   (cond-expand
-   (gauche.os.windows (sys-system #`"echo \"\" > ,(n path)"))
-   (else (sys-system #`"touch ,path"))))
+   (gauche.os.windows (sys-system #"echo \"\" > ~(n path)"))
+   (else (sys-system #"touch ~path"))))
 
 (define (get-command-output command)
   (cmd-rmrf "test.out")
@@ -80,7 +80,7 @@
 
 (test* "getenv"
        (string-trim-both
-	(get-command-output (cond-expand
+        (get-command-output (cond-expand
                              (gauche.os.windows "echo %PATH%")
                              (else "echo $PATH"))))
        (sys-getenv "PATH"))
@@ -88,6 +88,62 @@
 (test* "getcwd" (get-pwd-via-pwd)
        (sys-getcwd))
 
+;; putenv
+
+(cond-expand
+ [gauche.sys.setenv
+  (test* "sys-putenv" "foo"
+         (begin
+           (sys-putenv "ZZGGGBBB=foo")
+           (sys-getenv "ZZGGGBBB")))
+  (test* "sys-putenv" "foo"
+         (begin
+           (sys-putenv "ZZGGGBBB" "foo")  ;;old API
+           (sys-getenv "ZZGGGBBB")))]
+ [else])
+
+;; setenv
+
+(cond-expand
+ [gauche.sys.setenv
+  (test* "sys-setenv" "foo"
+         (begin
+           (sys-setenv "ZZGGGBBB" "foo" #t)
+           (sys-getenv "ZZGGGBBB")))
+  (test* "sys-setenv" "foo"
+         (begin
+           (sys-setenv "ZZGGGBBB" "bar" #f)
+           (sys-getenv "ZZGGGBBB")))]
+ [else])
+
+;; unsetenv
+
+(cond-expand
+ [gauche.sys.unsetenv
+  (test* "sys-unsetenv" #f
+         (begin
+           (sys-setenv "ZZGGGBBB" "foo" #t)
+           (sys-unsetenv "ZZGGGBBB")
+           (sys-getenv "ZZGGGBBB")))]
+ [else])
+
+;; environ
+(test* "sys-environ->alist" '(("A" . "B") ("A" . "") ("" . "B") ("A" . "B=C"))
+       (sys-environ->alist '("A=B" "A=" "=B" "A=B=C")))
+
+(let ([envs (sys-environ)])
+  (define (env-test var)
+    (test* #"sys-environ (~var)" #t
+           (cond [(sys-getenv var)
+                  => (^[val] (boolean (member #"~|var|=~|val|" envs)))]
+                 [else #t])))
+  (env-test "HOME")
+  (env-test "USER")
+  (env-test "LANG")
+  (env-test "PWD")
+  (env-test "TERM")
+  (env-test "SHELL"))
+
 ;;-------------------------------------------------------------------
 (test-section "pathnames")
 
@@ -138,13 +194,16 @@
        (sys-normalize-pathname "." :absolute #t))
 (test* "normalize" (n (string-append (get-pwd-via-pwd) "/"))
        (sys-normalize-pathname "" :absolute #t))
-(cond-expand
- (gauche.os.windows #t)
- (else
-  (test* "normalize"
-         (n (string-append (get-command-output "echo $HOME") "/abc"))
-         (sys-normalize-pathname "~/abc" :expand #t))))
-
+(test* "normalize"
+       (cond-expand
+        [gauche.os.windows
+         (n (string-append (or (sys-getenv "HOME") ; MSYS
+                               (sys-getenv "USERPROFILE") ; cmd.exe
+                               "")
+                           "/abc"))]
+        [else
+         (n (string-append (slot-ref (sys-getpwuid (sys-getuid)) 'dir) "/abc"))])
+       (sys-normalize-pathname "~/abc" :expand #t))
 (test* "normalize" (n "/a/b/c/d/e")
        (sys-normalize-pathname "/a/b//.///c//d/./e"
                                :canonicalize #t))
@@ -263,6 +322,84 @@
          (sys-rmdir "test.dir")
          (sys-access "test.dir" F_OK)))
 
+;; sys-realpath
+
+(define (expected-path p)
+  (let1 pp (sys-normalize-pathname p :absolute #t :canonicalize #t)
+    (if (eqv? (string-ref pp (- (string-length pp) 1)) #\/)
+      (substring pp 0 (- (string-length pp) 1))
+      pp)))
+
+(test* "sys-realpath (/)" (sys-normalize-pathname "/") (sys-realpath "/"))
+(test* "sys-realpath (.)" (expected-path ".") (sys-realpath "."))
+
+(cond-expand
+ [gauche.sys.symlink
+  (sys-unlink "test1.o")
+  (sys-unlink "test2.o")
+  (with-output-to-file "test1.o" (cut print))
+
+  (sys-symlink "test1.o" "test2.o")
+  (test* "sys-realpath (symlink)"
+         (expected-path "./test1.o")
+         (sys-realpath "./test2.o"))
+  (sys-unlink "test2.o")
+
+  (sys-symlink "./test1.o" "test2.o")
+  (test* "sys-realpath (symlink)"
+         (expected-path "./test1.o")
+         (sys-realpath "./test2.o"))
+  (sys-unlink "test2.o")
+
+  (sys-symlink "../src/test1.o" "test2.o")
+  (test* "sys-realpath (symlink)"
+         (expected-path "./test1.o")
+         (sys-realpath "./test2.o"))
+
+  (sys-unlink "test1.o")
+  (test* "sys-realpath (dangling)"
+         (test-error)
+         (sys-realpath "./test2.o"))
+
+  (sys-mkdir "test1.o" #o777)
+  (with-output-to-file "test1.o/test.o" (cut print))
+  (test* "sys-realpath (symlink to dir)"
+         (expected-path "./test1.o/test.o")
+         (sys-realpath "./test2.o/test.o"))
+
+  (sys-unlink "test1.o/test.o")
+  (sys-rmdir "test1.o")
+  (sys-unlink "test2.o")
+
+  (test* "sys-realpath (NOENT)"
+         (test-error <system-error>)
+         (sys-realpath "./test2.o/test.o"))
+  ]
+ [else])
+
+;; sys-mkstemp/sys-mkdtemp
+(let ()
+  ;; Check if multiple temporary files or directories are created.
+  (define (test-mkxtemp name creator tester remover)
+    (test* name '(#t #t)
+       (let1 temps (map creator (iota 3))
+         (begin0 (list (every tester temps)
+                       (let1 sorted (sort temps)
+                         (every (complement equal?) sorted (cdr sorted))))
+           (for-each remover temps)))))
+
+  (test-mkxtemp "sys-mkstemp"
+                (^_ (receive (p n) (sys-mkstemp "test.o")
+                      (close-output-port p)
+                      n))
+                file-is-regular?
+                sys-unlink)
+
+  (test-mkxtemp "sys-mkdtemp"
+                (^_ (sys-mkdtemp "test.dir"))
+                file-is-directory?
+                sys-rmdir))
+
 ;;-------------------------------------------------------------------
 (test-section "time")
 
@@ -373,27 +510,20 @@
                (close-input-port in) (close-output-port out)
                (list f1 f2 f3))))))
 
-;; Kludge: MinGW32 seems not to support :none, :line buffering,
-;; so we flush and close the output pipe before reading from it.
-
 (test* "pipe and read-block(none)" 2
        (receive (in out) (sys-pipe :buffering :none)
          (display "ab" out)
-         (cond-expand (gauche.os.windows (close-output-port out)) (else))
          (let1 r (string-size (read-block 1000 in))
            (close-input-port in)
-           (cond-expand ((not gauche.os.windows) (close-output-port out))
-                        (else))
+           (close-output-port out)
            r)))
 
 (test* "pipe and read-block(line)" 2
        (receive (in out) (sys-pipe :buffering :line)
          (display "a\n" out)
-         (cond-expand (gauche.os.windows (close-output-port out)) (else))
          (let1 r (string-size (read-block 1000 in))
            (close-input-port in)
-           (cond-expand ((not gauche.os.windows) (close-output-port out))
-                        (else))
+           (close-output-port out)
            r)))
 
 ;;-------------------------------------------------------------------
@@ -405,8 +535,8 @@
    (else (sys-sleep 1))))
 
 (cond-expand
- ((and (not gauche.os.windows)  ;; win32 doesn't support fork at all.
-       (not gauche.os.cygwin))	;; cygwin's fork is not reliable.
+ [(and (not gauche.os.windows)  ;; win32 doesn't support fork at all.
+       (not gauche.os.cygwin))  ;; cygwin's fork is not reliable.
   (test* "fork & wait" #t
          (let ((pid (sys-fork)))
            (if (= pid 0)
@@ -429,7 +559,7 @@
          (let ((pid (sys-fork)))
            (if (= pid 0)
              (begin (sys-pause) (sys-exit 0))
-             (begin 
+             (begin
                (sys-kill pid SIGKILL)
                (receive (rpid code) (sys-wait)
                  (and (= rpid pid)
@@ -440,8 +570,8 @@
          (let1 pid (sys-fork)
            (if (= pid 0)
              (begin (nap) (sys-exit 0))
-             (begin 
-               (sys-kill pid SIGSTOP) 
+             (begin
+               (sys-kill pid SIGSTOP)
                (receive (rpid code) (sys-waitpid pid :untraced #t)
                  (and (= rpid pid)
                       (sys-wait-stopped? code)
@@ -463,7 +593,7 @@
                (begin (close-input-port in)
                       (display (make-string 69999) out)
                       (with-error-handler
-                          (lambda (e) (sys-exit 0))
+                          (^e (sys-exit 0))
                         (lambda ()
                           (newline out)
                           (close-output-port out)
@@ -498,37 +628,68 @@
   ;; Testing fork&exec and detached process
   ;; NB: these tests assume we're running the testing gosh in the
   ;; current directory.
+  (when (file-exists? "./gosh")
+    (cmd-rmrf "test.out")
+    ;; The child process.  Gets the expected ppid in the first arg.
+    ;; If :detached is true, we expect ppid to be 1.  However, the
+    ;; immediate parent may take time to exit, so we loop to synchronize.
+    (with-output-to-file "test.out"
+      (lambda ()
+        (write '(define (main args)
+                  (let1 expected-ppid (x->integer (cadr args))
+                    (let loop ([c 0])
+                      (sys-nanosleep #e2e8)
+                      (cond [(or (eqv? (sys-getppid) expected-ppid)
+                                 (>= c 50)) ; takes about 10s
+                             (write (list (sys-getpid) (sys-getpgrp)))]
+                            [else (loop (+ c 1))])))))))
+    (let ()
+      (define (run-and-read-test msg ppid detached)
+        (test* msg detached
+               (receive (in out) (sys-pipe :buffering :none)
+                 (let1 pid (sys-fork-and-exec "./gosh"
+                                              `("./gosh" "-ftest" "./test.out"
+                                                ,(x->string ppid))
+                                              :iomap `((1 . ,out))
+                                              :detached detached)
+                   (close-port out)
+                   (let1 result (read in)
+                     (sys-waitpid pid)
+                     (eqv? (car result) (cadr result)))))))
+      (run-and-read-test "fork, exec and detached process (not detached)"
+                         (sys-getpid) #f)
+      (run-and-read-test "fork, exec and detached process (detached)"
+                         1 #t)))
   (cmd-rmrf "test.out")
-  (with-output-to-file "test.out"
-    (lambda ()
-      (write '(begin
-                (sys-nanosleep #e2e8)
-                (write (list (sys-getpid) (sys-getppid) (sys-getpgrp)))))))
-  (receive (in out) (sys-pipe :buffering :none)
-    (define (run-and-read detached)
-      (let1 pid (sys-fork-and-exec "./gosh"
-                                   `("./gosh" "-ftest" "./test.out")
-                                   :iomap `((1 . ,out))
-                                   :detached detached)
-        (begin0 (read in) (sys-waitpid pid))))
-    (test* "fork, exec and detached process (not detached)"
-           `(parent ,(sys-getpid) pgrp ,(sys-getpgrp))
-           (let1 r (run-and-read #f)
-             `(parent ,(cadr r) pgrp ,(caddr r))))
-    (test* "fork, exec and detached process (detached)"
-           `(parent 1 pgrp #t)
-           (let1 r (run-and-read #t)
-             `(parent ,(cadr r) pgrp ,(= (car r) (caddr r)))))
-    )
-  (cmd-rmrf "test.out")
-  ) ; !gauche.os.windows
- (else))
+
+  ;; Testing GC in forked process---we don't explicitly spawn a thread here,
+  ;; but some architecture (OSX 10.7.3, at least) seems to create a thread
+  ;; implicitly.  In the child process all threads are gone except one,
+  ;; and GC needs to recognize that.  The following test fails on OSX 10.7.3
+  ;; without HANDLE_FORK defined in gc.
+  ;; NB: HANDLE_FORK doesn't seem to work on OSX 10.7.4.  Disabling for now.
+  ;; (test* "GC in forked process" "done"
+  ;;        (receive (in out) (sys-pipe)
+  ;;          (let ([pid (sys-fork)])
+  ;;            (if (= pid 0)
+  ;;              (begin
+  ;;                (dotimes (i 1000)
+  ;;                  (make-vector 10000))
+  ;;                (gc)
+  ;;                (display "done\n" out)
+  ;;                (sys-exit 0))
+  ;;              (let ((line (read-line in)))
+  ;;                (sys-waitpid pid)
+  ;;                line)))))
+
+  ] ; !gauche.os.windows
+ [else])
 
 ;;-------------------------------------------------------------------
 (test-section "select")
 
 (cond-expand
- (gauche.sys.select
+ [gauche.sys.select
   (test* "fdset" '(3 #t #f #t #t #f)
          (let ((fdset (make <sys-fdset>)))
            (set! (sys-fdset-ref fdset (current-input-port)) #t)
@@ -536,7 +697,7 @@
            (sys-fdset-set! fdset 3 #t)
            (sys-fdset-set! fdset 4 #f)
            (cons (sys-fdset-max-fd fdset)
-                 (map (lambda (i) (sys-fdset-ref fdset i)) (iota 5)))))
+                 (map (^i (sys-fdset-ref fdset i)) (iota 5)))))
 
   (test* "fdset" '(-1 7 7 4 10 10 -1)
          (let ((fdset (make <sys-fdset>))
@@ -572,31 +733,37 @@
   (test* "sys-fdset-clear!" '()
          (sys-fdset->list (sys-fdset-clear! (sys-fdset 1 2 3))))
 
-  (test* "select" '(0 #f #f #f #f 1 #t #f #f #t #\x)
-         (let*-values (((in out) (sys-pipe))
-                       ((pid) (sys-fork)))
-           (if (= pid 0)
-             (begin (sys-select #f #f #f 100000)
-                    (display "x" out)
-                    (close-output-port out)
-                    (sys-exit 0))
-             (let ((rfds (make <sys-fdset>)))
-               (sys-fdset-set! rfds in #t)
-               (receive (an ar aw ae)
-                   (sys-select rfds #f #f 0)
-                 (receive (bn br bw be)
-                     (sys-select! rfds #f #f #f)
-                   (begin0
-                    (list an (eq? ar rfds) aw ae
-                          (sys-fdset-ref ar in)
-                          bn (eq? br rfds) bw be
-                          (sys-fdset-ref rfds in)
-                          (read-char in))
-                    (sys-waitpid pid)))))
-             ))
-         )
-  )
- (else)) ; cond-expand gauche.sys.select
+  ;; NB: Windows' select() can't select on non-socket fds, so we skip
+  ;; this test.
+
+  (cond-expand
+   [(not gauche.os.windows)
+    (test* "select" '(0 #f #f #f #f 1 #t #f #f #t #\x)
+           (let*-values (((in out) (sys-pipe))
+                         ((pid) (sys-fork)))
+             (if (= pid 0)
+               (begin (sys-select #f #f #f 100000)
+                      (display "x" out)
+                      (close-output-port out)
+                      (sys-exit 0))
+               (let ((rfds (make <sys-fdset>)))
+                 (sys-fdset-set! rfds in #t)
+                 (receive (an ar aw ae)
+                     (sys-select rfds #f #f 0)
+                   (receive (bn br bw be)
+                       (sys-select! rfds #f #f #f)
+                     (begin0
+                         (list an (eq? ar rfds) aw ae
+                               (sys-fdset-ref ar in)
+                               bn (eq? br rfds) bw be
+                               (sys-fdset-ref rfds in)
+                               (read-char in))
+                       (sys-waitpid pid)))))
+               ))
+           )]
+   [else]) ; cond-expand (not gauche.os.windows)
+  ]
+ [else]) ; cond-expand gauche.sys.select
 
 ;;-------------------------------------------------------------------
 (test-section "signal handling")
@@ -704,8 +871,8 @@
              (call/cc
               (lambda (k)
                 (set-signal-handler! SIGINT  k)
-                (set-signal-handler! SIGCHLD (lambda (k) (sys-wait) (set! chld #t)))
-                (set-signal-handler! SIGHUP  (lambda (k) (set! sig 'hup)))
+                (set-signal-handler! SIGCHLD (^k (sys-wait) (set! chld #t)))
+                (set-signal-handler! SIGHUP  (^k (set! sig 'hup)))
                 (sys-sigmask SIG_BLOCK mask1)
                 (let ((pid (sys-fork)))
                   (if (= pid 0)
@@ -772,11 +939,11 @@
    [(and gauche.sys.sigwait
          (not gauche.os.cygwin))
     (let ()
-      (define z (lambda (n) (raise 'foo)))
-      
+      (define z (^n (raise 'foo)))
+
       (set-signal-handler! SIGCHLD #t)
       (set-signal-handler! SIGINT z)
-      
+
       (test* "sys-sigwait" SIGHUP
              (receive (in out) (sys-pipe)
                (let1 pid (sys-fork)
@@ -803,8 +970,7 @@
     ]
    [else]) ; (not (and gauche.sys.sigwait (not gauche.os.cygwin)))
 
-  ] 
+  ]
  [else]) ; gauche.os.windows
 
 (test-end)
-
