@@ -5,11 +5,6 @@
 (use gauche.test)
 (test-start "text utilities")
 
-;;-------------------------------------------------------------------
-(test-section "console")
-(use text.console)
-(test-module 'text.console)
-
 ;;-------------------------------------------------------------------
 (test-section "csv")
 (use text.csv)
@@ -94,7 +89,7 @@
 
   (test* "make-csv-header-parser (regexps)" '#(3 2 6 5)
          (any (make-csv-header-parser header-slots2) data))
-  
+
   (test* "make-csv-record-parser (strings)"
          '(("Land of Lisp" "1958" "551,435,453" "39994")
            ("United States of Formula Translators" "1957" "4,343,225,434"
@@ -116,7 +111,7 @@
                                              '((#/year/i #/^\d+$/)
                                                #/country/i #/popu/i #/gdp/i))
                      data))
-  
+
   (test* "csv-rows->tuples (allow-gap? #f)"
          '(("Land of Lisp" "1958" "551,435,453" "39994")
            ("United States of Formula Translators" "1957" "4,343,225,434"
@@ -169,6 +164,65 @@ fuga
        (with-output-to-string
          (lambda () (diff-report diff-a diff-b))))
 
+(test* "diff-report/context"
+       (string-join '("***************"
+                      "*** 1,6 ****"
+                      "  foo"
+                      "  bar"
+                      "- bar"
+                      "  baz"
+                      "! baz"
+                      "  hoge"
+                      "--- 1,6 ----"
+                      "  foo"
+                      "  bar"
+                      "  baz"
+                      "! fuga"
+                      "  hoge"
+                      "+ fuga")
+                    "\n" 'suffix)
+       (with-output-to-string
+         (lambda () (diff-report/context diff-a diff-b))))
+
+(test* "diff-report/context (one-side empty)"
+       (string-join '("***************"
+                      "*** 0 ****"
+                      "--- 1,6 ----"
+                      "+ foo"
+                      "+ bar"
+                      "+ baz"
+                      "+ fuga"
+                      "+ hoge"
+                      "+ fuga")
+                    "\n" 'suffix)
+       (with-output-to-string
+         (lambda () (diff-report/context "" diff-b))))
+
+(test* "diff-report/unified"
+       (string-join '("@@ -1,6 +1,6 @@"
+                      " foo"
+                      " bar"
+                      "-bar"
+                      " baz"
+                      "-baz"
+                      "+fuga"
+                      " hoge"
+                      "+fuga")
+                    "\n" 'suffix)
+       (with-output-to-string
+         (lambda () (diff-report/unified diff-a diff-b))))
+
+(test* "diff-report/context (one-side empty)"
+       (string-join '("@@ -1,6 +0,0 @@"
+                      "-foo"
+                      "-bar"
+                      "-bar"
+                      "-baz"
+                      "-baz"
+                      "-hoge")
+                    "\n" 'suffix)
+       (with-output-to-string
+         (lambda () (diff-report/unified diff-a ""))))
 
 ;;-------------------------------------------------------------------
 (test-section "edn")
@@ -185,7 +239,7 @@ fuga
              (namespace/name . " namespace/name")
              (:keyword . ":keyword")
              ("string" . "\"string\"")
-             ("weird \" characters \t\n\r\\" 
+             ("weird \" characters \t\n\r\\"
               . "\"weird \\\" characters \\t\\n\\r\\\\\"")
              (#\a . "\\a")
              (#\newline . "\\newline")
@@ -238,7 +292,7 @@ fuga
            edn-equal?)))
 
 ;; custom object
-(register-edn-object-handler! 'u8vector 
+(register-edn-object-handler! 'u8vector
                               (^[tag vec] (vector->u8vector vec)))
 (define-method edn-write ((x <u8vector>))
   (display "#u8vector")
@@ -252,116 +306,17 @@ fuga
 ;; invalid
 (let1 invalids '("foo/bar/baz" "/foo" "foo/")
   (dolist [d invalids]
-    (test* (format "parse (invalid) ~s" d) 
+    (test* (format "parse (invalid) ~s" d)
            (test-error <edn-parse-error> #/invalid token/)
            (parse-edn-string d))))
 (test* (format "valid symbol name") '(#f #f #f #f #t)
-       (map edn-valid-symbol-name? 
+       (map edn-valid-symbol-name?
             '("nil" "bar/nil" "true" "false" "nile")))
 
 ;;-------------------------------------------------------------------
-(test-section "gap-buffer")
-(use text.gap-buffer)
-(use gauche.uvector)
-
-(test-module 'text.gap-buffer)
-
-;; stringify gap-buffer for test purpose
-(define (gap-buffer-visualize gbuf)
-  (with-output-to-string
-    (^[]
-      (let1 buf (~ gbuf'buffer) ;accessing internal
-        (dotimes [i (gap-buffer-gap-start gbuf)]
-          (display (integer->char (~ buf i))))
-        (dotimes [i (- (gap-buffer-gap-end gbuf) (gap-buffer-gap-start gbuf))]
-          (display #\_))
-        (dotimes [i (- (gap-buffer-capacity gbuf) (gap-buffer-gap-end gbuf))]
-          (display (integer->char (~ buf (+ i (gap-buffer-gap-end gbuf))))))))))
-
-(test* "constuct" "abcde___"
-       (gap-buffer-visualize (string->gap-buffer "abcde")))
-(test* "constuct" "___abcde"
-       (gap-buffer-visualize (string->gap-buffer "abcde" 0 'beginning)))
-(test* "constuct" "bcd_"
-       (gap-buffer-visualize (string->gap-buffer "abcde" 0 'end 1 4)))
-  
-(let1 gbuf (string->gap-buffer "abcde")
-  (test* "move" "abcd___e"
-         (gap-buffer-visualize (gap-buffer-move! gbuf -1 'current)))
-  (test* "move" "a___bcde"
-         (gap-buffer-visualize (gap-buffer-move! gbuf 1)))
-  (test* "move" "abc___de"
-         (gap-buffer-visualize (gap-buffer-move! gbuf -2 'end)))
-  (test* "insert" "abcZ__de"
-         (gap-buffer-visualize (gap-buffer-insert! gbuf #\Z)))
-  (test* "insert" "abcZxyde"
-         (gap-buffer-visualize (gap-buffer-insert! gbuf "xy")))
-  (test* "insert" "abcZxyw_______de"
-         (gap-buffer-visualize (gap-buffer-insert! gbuf #\w)))
-  (test* "insert"
-         "abcZxyw012345678901234567890123456789_________________________de"
-         (gap-buffer-visualize
-          (gap-buffer-insert! gbuf "012345678901234567890123456789")))
-  (test* "delete"
-         "abcZxyw012345678901234567890123456789__________________________e"
-         (gap-buffer-visualize
-          (gap-buffer-delete! gbuf 1)))
-  (test* "delete"
-         "abc____________________________________678901234567890123456789e"
-         (begin (gap-buffer-move! gbuf 3)
-                (gap-buffer-visualize
-                 (gap-buffer-delete! gbuf 10))))
-  (test* "delete"
-         (test-error)
-         (gap-buffer-visualize (gap-buffer-delete! gbuf 100)))
-  (test* "change"
-         "abcdef______________________________________1234567890123456789e"
-         (gap-buffer-visualize (gap-buffer-change! gbuf 5 "def")))
-  )
-
-(let1 gbuf (string->gap-buffer "abcde")
-  (test* "->string (gap at end)" "abcde"
-         (gap-buffer->string gbuf))
-  (test* "->string (gap at beginning)" "abcde"
-         (begin (gap-buffer-move! gbuf 0)
-                (gap-buffer->string gbuf)))
-  (test* "->string (gap at middle)" "abcde"
-         (begin (gap-buffer-move! gbuf 3 'current)
-                (gap-buffer->string gbuf)))
-  )
-
-(let1 gbuf (gap-buffer-copy (string->gap-buffer "abcde"))
-  (test* "undo and redo"
-         '("abXYZcde" "abXYde" "aQRSYde" "abXYde" "abXYZcde"
-           "abcde" "aQRSYde" "abcde")
-         (let* ([u0 (begin (gap-buffer-move! gbuf 2)
-                           (gap-buffer-edit! gbuf '(i #f "XYZ")))]
-                [s0 (gap-buffer->string gbuf)] ; abXYZcde
-                [u1 (begin (gap-buffer-move! gbuf -1 'current)
-                           (gap-buffer-edit! gbuf '(d #f 2)))]
-                [s1 (gap-buffer->string gbuf)] ; abXYde
-                [u2 (begin (gap-buffer-move! gbuf -3 'current)
-                           (gap-buffer-edit! gbuf '(c #f 2 "QRS")))]
-                [s2 (gap-buffer->string gbuf)] ; aQRSYde
-                [r2 (begin (gap-buffer-move! gbuf 0)
-                           (gap-buffer-edit! gbuf u2))] ; undo change!
-                [s3 (gap-buffer->string gbuf)] ; abXYde
-                [r1 (gap-buffer-edit! gbuf u1)] ; undo delete!
-                [s4 (gap-buffer->string gbuf)] ; abXYZcde
-                [r0 (begin (gap-buffer-move! gbuf -1 'end)
-                           (gap-buffer-edit! gbuf u0))] ; undo insert!
-                [s5 (gap-buffer->string gbuf)] ; abcde
-                [u0 (gap-buffer-edit! gbuf r0)] ; redo insert!
-                [u1 (gap-buffer-edit! gbuf r1)] ; redo delete!
-                [u2 (gap-buffer-edit! gbuf r2)] ; redo change!
-                [s6 (gap-buffer->string gbuf)] ; aQRSYde
-                [s7 (begin (gap-buffer-edit! gbuf u2)
-                           (gap-buffer-edit! gbuf u1)
-                           (gap-buffer-edit! gbuf u0)
-                           (gap-buffer->string gbuf))] ; abcde
-                )
-           (list s0 s1 s2 s3 s4 s5 s6 s7)))
-  )
+(test-section "external-editor")
+(use text.external-editor)
+(test-module 'text.external-editor)
 
 ;;-------------------------------------------------------------------
 (test-section "html-lite")
@@ -413,7 +368,7 @@ fuga
           ((null? (car x)) (flatten-rec (cdr x) r))
           (else (flatten-rec (cdr x) (cons (tos (car x)) r)))))
   (define (flatten x) (string-concatenate-reverse (flatten-rec x '())))
-  
+
   (test* "html, head, body"
          "<html><head><title>foo</title></head><body>foo</body></html>"
          (flatten (html:html (html:head (html:title "foo"))
@@ -538,7 +493,7 @@ fuga
 (test* "skip-while" '(eof eof)
        (test-parseutil skip-while "yxxxd"
                        (^x (and (char? x)
-				(char-alphabetic? x)))))
+                                (char-alphabetic? x)))))
 
 (test* "next-token" '("" #\d)
        (test-parseutil next-token "xxxd" #[ax ] #[d] "next token"))
@@ -553,9 +508,9 @@ fuga
 (test* "next-token" '("aeio" #\tab)
        (test-parseutil next-token "   aeio\tnjj"
                        (^x (and (char? x)
-				(char-whitespace? x)))
+                                (char-whitespace? x)))
                        (^x (or (eof-object? x)
-			       (char-whitespace? x)))
+                               (char-whitespace? x)))
                        "next token"
                        ))
 
@@ -628,7 +583,7 @@ fuga
                                            - #\. "E" (number "-3")
                                            (number "1.2") (number ".3")
                                            )
-                                           
+
        (sql-tokenize "0 -12 +12 .123 123. 123.45 -.123 -123. -123.45
                       +.123 +123. +123.45 0E0 -1E3 -1.E3 -.1E3
                       -1.2E3 1E-3 1.E-3 .1E-3 -.E-3 1.2.3"))
@@ -663,50 +618,9 @@ fuga
        (sql-tokenize "?,?,:foo, :bar , ?"))
 
 ;;-------------------------------------------------------------------
-(test-section "line-edit")
-(use text.line-edit)
-(test-module 'text.line-edit)
-
-;; Test internal functions
-(let ()
-  (define (t s pos expect)
-    (test* #"buffer-find-matching-paren: ~s ~pos" expect
-           ((with-module text.line-edit buffer-find-matching-paren)
-            ((with-module text.gap-buffer string->gap-buffer) s)
-            0 pos)))
-
-  (t "(abc)" 4 0)
-  (t "aa(abc)zz" 6 2)
-  (t "(a(abc)z)" 6 2)
-  (t "(a)(abc)z" 7 3)
-  (t "(a\\(b)z" 5 0)
-  (t "(a\\(\\))z" 6 0)
-  (t "(a\\)\\()z" 6 0)
-  (t "(a(abc)(def)z)" 11 7)
-  (t "(a(abc)(def)z)" 13 0)
-  (t "aaaabc)zz" 6 #f)
-  (t "(a{b}c)" 4 2)
-  (t "(a{b}c)" 6 0)
-  (t "(a{b(}c)" 5 #f)
-  (t "(a{b(}c)" 7 4)
-  (t "(a{b)}c)" 4 #f)
-  (t "(a{b)}c)" 5 2)
-  (t "(a\"(\"b)" 6 0)
-  (t "(a\")\"b)" 6 0)
-  (t "(a\"\\\"(\"b)" 8 0)
-  (t "(a#/bc([d\\//)]" 12 0)
-  (t "(a#/bc([d\\//)]" 13 #f)
-  (t "(a#(b)c)" 5 3)
-  (t "(a#(b)c)" 7 0)
-  (t "(a#[b[:space:]]c)" 14 3)
-  (t "(a#[b[:space:]]c)" 13 5)
-  (t "(a#[b[:space:]]c)" 16 0)
-  (t "(a#{b}c)" 5 3)
-  (t "(a#{b)c}" 5 #f)
-  (t "(a#{b)c}" 7 3)
-  (t "(a\\#(b)c)" 6 4)
-  (t "(a#\\(b)c)" 6 0)
-  )
+(test-section "pager")
+(use text.pager)
+(test-module 'text.pager)
 
 ;;-------------------------------------------------------------------
 (test-section "template")
@@ -720,15 +634,15 @@ fuga
         (make-template-environment
          :bindings `(abc 3 def 9 jkl ,format pqr 1000))))
 
-(test* "expand-template-file"
-       "<html><head><title>Test</title></head><body><p><a href=\"foo\">bar</a\n>baz</p\n></body></html>\n"
-       (expand-template-file
-        "template.txt"
-        (make-template-environment
-         :imports '(text.tree)
-         :bindings `(title "Test"
-                     body ,(html:p (html:a :href "foo" "bar") "baz")))
-        '("../test/data")))
+(test*/diff "expand-template-file"
+            "<html><head><title>Test</title></head><body><p><a href=\"foo\">bar</a\n>baz</p\n></body></html>\n"
+            (expand-template-file
+             "template.txt"
+             (make-template-environment
+              :imports '(text.tree)
+              :bindings `(title "Test"
+                                body ,(html:p (html:a :href "foo" "bar") "baz")))
+             '("../test/data")))
 
 ;;-------------------------------------------------------------------
 (test-section "tree")
