@@ -3,7 +3,7 @@
 (use gauche.test)
 
 (test-start "multibyte (utf-8)")
-(use srfi-1)
+(use scheme.list)
 
 ;;-------------------------------------------------------------------
 (test-section "string builtins")
@@ -54,65 +54,167 @@
 (test "string-join" "ふぅ！ばぁ！ばず"
       (lambda () (string-join '("ふぅ" "ばぁ" "ばず") "！" 'strict-infix)))
 
-(test "string-scan" 7
-      (lambda () (string-scan "あえいうえおあおあいうえお" "おあい")))
-(test "string-scan" "あえいうえおあ"
-      (lambda () (string-scan "あえいうえおあおあいうえお" "おあい" 'before)))
-(test "string-scan" "うえお"
-      (lambda () (string-scan "あえいうえおあおあいうえお" "おあい" 'after)))
-(test "string-scan" '("あえいうえおあ" "おあいうえお")
-      (lambda ()
-        (receive r (string-scan "あえいうえおあおあいうえお" "おあい" 'before*) r)))
-(test "string-scan" '("あえいうえおあおあい" "うえお")
-      (lambda ()
-        (receive r (string-scan "あえいうえおあおあいうえお" "おあい" 'after*) r)))
-(test "string-scan" '("あえいうえおあ" "うえお")
-      (lambda ()
-        (receive r (string-scan "あえいうえおあおあいうえお" "おあい" 'both) r)))
-(test "string-scan" #f
-      (lambda () (string-scan "あえいうえおあおあいうえお" "おい")))
+(let ()
+  (define (test-string-scan out s1 s2 . opt)
+    (if (pair? out)
+      (begin
+        (test* "string-scan" (car out) (apply string-scan s1 s2 opt))
+        (test* "string-scan-right" (cadr out)
+               (apply string-scan-right s1 s2 opt)))
+      (apply test-string-scan (list out out) s1 s2 opt)))
+  (define (test-string-scan2 out1 out2 s1 s2 . opt)
+    (if (pair? out1)
+      (begin
+        (test* "string-scan" (list (car out1) (car out2))
+               (receive r (apply string-scan s1 s2 opt) r))
+        (test* "string-scan-right" (list (cadr out1) (cadr out2))
+               (receive r (apply string-scan-right s1 s2 opt) r)))
+      (apply test-string-scan2 (list out1 out1) (list out2 out2) s1 s2 opt)))
+
+  (test-string-scan '(7 20)
+                    "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい")
+  (test-string-scan '("あえいうえおあ" "あえいうえおあおあいうえおあえいうえおあ")
+                      "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'before)
+  (test-string-scan '("うえおあえいうえおあおあいうえお" "うえお")
+                    "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'after)
+  (test-string-scan2 '("あえいうえおあ" "あえいうえおあおあいうえおあえいうえおあ")
+                     '("おあいうえおあえいうえおあおあいうえお" "おあいうえお")
+                     "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'before*)
+  (test-string-scan2 '("あえいうえおあおあい" "あえいうえおあおあいうえおあえいうえおあおあい")
+                     '("うえおあえいうえおあおあいうえお" "うえお")
+                     "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'after*)
+  (test-string-scan2 '("あえいうえおあ" "あえいうえおあおあいうえおあえいうえおあ")
+                     '("うえおあえいうえおあおあいうえお" "うえお")
+                     "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'both)
+  (test-string-scan #f "あえいうえおあおあいうえお" "おい")
+  )
 
 ;;-------------------------------------------------------------------
-(test-section "string-pointer")
-(define sp #f)
-(test "make-string-pointer" #t
-      (lambda ()
-        (set! sp (make-string-pointer "いろはにhoへと"))
-        (string-pointer? sp)))
-(test "string-pointer-next!" #\い
-      (lambda () (string-pointer-next! sp)))
-(test "string-pointer-next!" #\ろ
-      (lambda () (string-pointer-next! sp)))
-(test "string-pointer-prev!" #\ろ
-      (lambda () (string-pointer-prev! sp)))
-(test "string-pointer-prev!" #\い
-      (lambda () (string-pointer-prev! sp)))
-(test "string-pointer-prev!" #t
-      (lambda () (eof-object? (string-pointer-prev! sp))))
-(test "string-pointer-index" 0
-      (lambda () (string-pointer-index sp)))
-(test "string-pointer-index" 8
-      (lambda () (do ((x (string-pointer-next! sp) (string-pointer-next! sp)))
-                     ((eof-object? x) (string-pointer-index sp)))))
-(test "string-pointer-substring" '("いろはにhoへと" "")
-      (lambda () (list (string-pointer-substring sp)
-                       (string-pointer-substring sp :after #t))))
-(test "string-pointer-substring" '("いろはにh" "oへと")
-      (lambda ()
-        (string-pointer-set! sp 5)
-        (list (string-pointer-substring sp)
-              (string-pointer-substring sp :after #t))))
-(test "string-pointer-substring" '("" "いろはにhoへと")
-      (lambda ()
-        (string-pointer-set! sp 0)
-        (list (string-pointer-substring sp)
-              (string-pointer-substring sp :after #t))))
+(test-section "string-cursor")
+
+(define str "いろはにhoへと")
+(define ss #f)
+(define se #f)
+(test* "string-cursor-start" #t
+       (begin
+         (set! ss (string-cursor-start str))
+         (string-cursor? ss)))
+(test* "string-cursor-start" #t
+       (begin
+         (set! se (string-cursor-end str))
+         (string-cursor? se)))
+(test* "string-cursor-next" #\ろ
+       (string-ref str (string-cursor-next str ss)))
+(test* "string-cursor-forward" #\に
+       (string-ref str (string-cursor-forward str ss 3)))
+(test* "string-cursor-prev" #\と
+       (string-ref str (string-cursor-prev str se)))
+(test* "string-cursor-back" #\に
+       (string-ref str (string-cursor-back str se 5)))
+(test* "string-index->cursor" #\へ
+       (string-ref str (string-index->cursor str 6)))
+(test* "string-cursor->index" 0
+       (string-cursor->index str ss))
+(test* "string-cursor->index" 8
+       (string-cursor->index str se))
+(test* "substring with cursors" "はにho"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "はにho"
+       (substring str
+                  2
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "はにh"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  5))
+(test* "string->list" '(#\ろ #\は #\に #\h #\o #\へ)
+       (string->list str
+                     (string-cursor-next str ss)
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\ろ #\は #\に #\h #\o #\へ)
+       (string->list str
+                     1
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\ろ #\は #\に #\h #\o #\へ)
+       (string->list str
+                     (string-cursor-next str ss)
+                     7))
+(test* "string-cursor=?" #t
+       (string-cursor=? ss (string-cursor-back str se 8)))
+(test* "string-cursor=?" #t
+       (string-cursor=? 0 0))
+(test* "string-cursor<?" #t
+       (string-cursor<? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss ss))
+(test* "string-cursor>?" #t
+       (string-cursor>? se ss))
+(test* "string-cursor>=?" #t
+       (string-cursor>=? se se))
+(test* "string-cursor-diff" 8
+       (string-cursor-diff str ss se))
+(test* "string-cursor-diff" -8
+       (string-cursor-diff str se ss))
+
+;;-------------------------------------------------------------------
+(test-section "string index building")
+
+(let* ([make-cycle (^[start-char count]
+                     ($ circular-list
+                        $* map ucs->char (iota count (char->ucs start-char))))]
+       [cycle1 (make-cycle #\a 26)]
+       [cycle2 (make-cycle #\α 25)]
+       [cycle3 (make-cycle #\あ 85)])
+  (define (make-str max-size)
+    (let loop ([size 0] [r '()]
+               [srcs (circular-list cycle1 cycle2 cycle3)]
+               [counts (circular-list 1 2 3)])
+      (let ([next-char (caar srcs)]
+            [next-count (car counts)])
+        (if (> (+ size next-count) max-size)
+          (list->string (reverse r))
+          (loop (+ size next-count)
+                (cons next-char r)
+                (circular-list (cadr srcs) (caddr srcs) (cdar srcs))
+                (cdr counts))))))
+  (define (test-string-index max-size)
+    (let1 s (string-build-index! (make-str max-size))
+      ;;((with-module gauche.internal %string-index-dump) s)
+      (test* #"string indexing (size=~max-size)"
+             #f
+             (any (^p (and (not (eqv? (car p) (string-ref s (cdr p))))
+                           (list 'pos (cdr p)
+                                 'expect (car p)
+                                 'actual (string-ref s (cdr p)))))
+                  (map cons (string->list s) (liota))))))
+  (test-string-index 0)
+  (test-string-index 15)
+  (test-string-index 33)
+  (test-string-index 34)
+  (test-string-index 49)
+  (test-string-index 63)
+  (test-string-index 64)
+  (test-string-index 65)
+  (test-string-index 255)
+  (test-string-index 256)
+  (test-string-index 257)
+  (test-string-index 8191)
+  (test-string-index 8193)
+  (test-string-index 65535)
+  (test-string-index 65537)
+  (test-string-index 131072)
+  (test-string-index 131073)
+  )
 
 ;;-------------------------------------------------------------------
 (test-section "incomplete strings")
 
 (test "string-length" 9 (lambda () (string-length #*"あいう")))
-(test "string-complete->incomplete" #*"あいう" 
+(test "string-complete->incomplete" #*"あいう"
       (lambda () (string-complete->incomplete "あいう")))
 (test "string-complete->incomplete" #*"あいう"
       (lambda () (string-complete->incomplete #*"あいう")))
@@ -135,18 +237,22 @@
       (lambda () (string-incomplete->complete #*"あいう\xe3\xe3" :omit)))
 (test "string-incomplete->complete (omit)" "あいう"
       (lambda () (string-incomplete->complete #*"あいう\xe3\x80" :omit)))
-(test "string-incomplete->complete (replace)" "あいふう"
-      (lambda () (string-incomplete->complete #*"あい\x80う" #\ふ)))
+(test "string-incomplete->complete (replace)" "あい?う"
+      (lambda () (string-incomplete->complete #*"あい\x80う" :replace)))
 (test "string-incomplete->complete (replace)" "ふあいう"
-      (lambda () (string-incomplete->complete #*"\x80あいう" #\ふ)))
+      (lambda () (string-incomplete->complete #*"\x80あいう" :replace #\ふ)))
 (test "string-incomplete->complete (replace)" "ふふあいう"
-      (lambda () (string-incomplete->complete #*"\x80\xe3あいう" #\ふ)))
+      (lambda () (string-incomplete->complete #*"\x80\xe3あいう" :replace #\ふ)))
 (test "string-incomplete->complete (replace)" "あいうふ"
-      (lambda () (string-incomplete->complete #*"あいう\xe3" #\ふ)))
+      (lambda () (string-incomplete->complete #*"あいう\xe3" :replace #\ふ)))
 (test "string-incomplete->complete (replace)" "あいうふふ"
-      (lambda () (string-incomplete->complete #*"あいう\xe3\xe3" #\ふ)))
+      (lambda () (string-incomplete->complete #*"あいう\xe3\xe3" :replace #\ふ)))
 (test "string-incomplete->complete (replace)" "あいうふふ"
-      (lambda () (string-incomplete->complete #*"あいう\xe3\x80" #\ふ)))
+      (lambda () (string-incomplete->complete #*"あいう\xe3\x80" :replace #\ふ)))
+(test "string-incomplete->complete (escape)" "あ??い?91う?80え"
+      (lambda () (string-incomplete->complete #*"あ?い\x91う\x80え" :escape)))
+(test "string-incomplete->complete (escape)" "あ__い_91う_80え"
+      (lambda () (string-incomplete->complete #*"あ_い\x91う\x80え" :escape #\_)))
 
 (test "string=?" #t (lambda () (string=? #*"あいう" #*"あいう")))
 
@@ -189,8 +295,8 @@
 (test "string-every" #t (lambda () (string-every #[あ-ん] "ああいあ")))
 (test "string-every" #f (lambda () (string-every #[あ-ん] "ああaあ")))
 (test "string-every" #t (lambda () (string-every #[あ-ん] "")))
-(test "string-every" #t (lambda () (string-every (lambda (x) (char-ci=? x #\あ)) "ああああ")))
-(test "string-every" #f (lambda () (string-every (lambda (x) (char-ci=? x #\あ)) "あいあい")))
+(test "string-every" #t (lambda () (string-every (^x (char-ci=? x #\あ)) "ああああ")))
+(test "string-every" #f (lambda () (string-every (^x (char-ci=? x #\あ)) "あいあい")))
 
 (test "string-any" #t (lambda () (string-any #\あ "ああああ")))
 (test "string-any" #f (lambda () (string-any #\あ "いうえお")))
@@ -198,8 +304,8 @@
 (test "string-any" #t (lambda () (string-any #[あ-ん] "すきーむ")))
 (test "string-any" #f (lambda () (string-any #[あ-ん] "スキーム")))
 (test "string-any" #f (lambda () (string-any #[あ-ん] "")))
-(test "string-any" #t (lambda () (string-any (lambda (x) (char-ci=? x #\あ)) "らららあ")))
-(test "string-any" #f (lambda () (string-any (lambda (x) (char-ci=? x #\あ)) "ラララア")))
+(test "string-any" #t (lambda () (string-any (^x (char-ci=? x #\あ)) "らららあ")))
+(test "string-any" #f (lambda () (string-any (^x (char-ci=? x #\あ)) "ラララア")))
 (test "string-tabulate" "アィイゥウ"
       (lambda ()
         (string-tabulate (lambda (code)
@@ -225,22 +331,71 @@
 ;;-------------------------------------------------------------------
 (test-section "char set")
 
-(use srfi-14)
+(use scheme.charset)
 
-(test "char-set" #t
-      (lambda () (char-set= (char-set #\あ #\い #\う #\え #\お)
-                            (string->char-set "おうえいあ"))))
-(test "char-set" #t
-      (lambda () (char-set= (list->char-set '(#\あ #\い #\う #\ん))
-                            (string->char-set "んんいいいああう"))))
-(test "char-set" #t
-      (lambda () (char-set<= (list->char-set '(#\ほ #\げ))
-                             char-set:full)))
-(test "char-set" #t
-      (lambda ()
-        (char-set= (->char-set "ぁぃぅぇぉあいうえ")
-                   (integer-range->char-set (char->integer #\ぁ)
-                                            (char->integer #\お)))))
+(let ((data '(#[ぁ-ん他-唖ァ-ン]
+              #[aあ唖ア])))
+  (define (t x)
+    (test* "char-set mutable/immutable roundtrip" #t
+           (char-set= (char-set-copy x)
+                      (char-set-copy (char-set-freeze! (char-set-copy x))))))
+  (for-each t data))
+
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\あ)
+                 (char-set #\あ #\い #\う #\え #\お))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\ぁ)
+                 (char-set #\あ #\い #\う #\え #\お))
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\ほ)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\ぁ)
+                                           (char->integer #\ん))
+                  (integer-range->char-set (char->integer #\ァ)
+                                           (char->integer #\ン))))
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\ケ)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\ぁ)
+                                           (char->integer #\ん))
+                  (integer-range->char-set (char->integer #\ァ)
+                                           (char->integer #\ン))))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\ん)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\ぁ)
+                                           (char->integer #\ん))
+                  (integer-range->char-set (char->integer #\ァ)
+                                           (char->integer #\ン))))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\ヴ)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\ぁ)
+                                           (char->integer #\ん))
+                  (integer-range->char-set (char->integer #\ァ)
+                                           (char->integer #\ン))))
+
+(test-char-set-2 "char-set=" #t char-set=
+                 (char-set #\あ #\い #\う #\え #\お)
+                 (string->char-set "おうえいあ"))
+(test-char-set-2 "char-set=" #t char-set=
+                 (list->char-set '(#\あ #\い #\う #\ん))
+                 (string->char-set "んんいいいああう"))
+(test-char-set-2 "char-set<=" #t char-set<=
+                 (list->char-set '(#\ほ #\げ))
+                 char-set:full)
+(test-char-set-2 "char-set<=" #t char-set<= #[あい] #[あい])
+(test-char-set-2 "char-set<=" #t char-set<= #[あい] #[あ-い])
+(test-char-set-2 "char-set<=" #f char-set<= #[あ-い] #[あい])
+(test-char-set-2 "char-set<=" #t char-set<= #[あ-いう-え] #[あ-え])
+(test-char-set-2 "char-set<=" #f char-set<= #[あ-え] #[あ-いう-え])
+(test-char-set-2 "char-set<=" #f char-set<= #[あ-いう-ぉ] #[あ-え])
+(test-char-set-2 "char-set<=" #f char-set<= #[ぁ-いう-え] #[あ-え])
+(test-char-set-2 "char-set<=" #t char-set<= #[あ-いか-き] #[あ-うお-け])
+(test-char-set-2 "char-set<=" #t char-set<= #[あ-いか-き] #[あ-うお-き])
+(test-char-set-2 "char-set<=" #t char-set<= #[あ-いか-き] #[あ-うか-こ])
+(test-char-set-2 "char-set<=" #t char-set<= #[か-き] #[あ-うか-こ])
+(test-char-set-2 "char-set<=" #t char-set<= #[か-きく-け] #[あ-うか-こ])
+(test-char-set-2 "char-set<=" #f char-set<= #[う-く] #[あ-えか-こ])
+(test-char-set-2 "->char-set" #t char-set=
+                 (->char-set "ぁぃぅぇぉあいうえ")
+                 (integer-range->char-set (char->integer #\ぁ)
+                                          (char->integer #\お)))
 
 ;;-------------------------------------------------------------------
 (test-section "ports")
@@ -439,19 +594,19 @@
 
 (define (byte-list->port p bytes)
   (set! *flusher-out* '())
-  (for-each (lambda (b) (write-byte b p)) bytes)
+  (for-each (^b (write-byte b p)) bytes)
   (close-output-port p)
   *flusher-out*)
 
 (define (char-list->port p chars)
   (set! *flusher-out* '())
-  (for-each (lambda (c) (write-char c p)) chars)
+  (for-each (^c (write-char c p)) chars)
   (close-output-port p)
   *flusher-out*)
 
 (define (string-list->port p strs)
   (set! *flusher-out* '())
-  (for-each (lambda (s) (display s p)) strs)
+  (for-each (^s (display s p)) strs)
   (close-output-port p)
   *flusher-out*)
 
@@ -619,4 +774,91 @@
          (rxmatch-after m) ;; memoizes start and length
          (list (rxmatch-before m) (rxmatch-substring m))))
 
+;;-------------------------------------------------------------------
+(test* "regexp/unicode-ci (aa)" "λ"
+       (cond ((rxmatch #/λ/i "λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (aA)" "Λ"
+       (cond ((rxmatch #/λ/i "Λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (Aa)" "λ"
+       (cond ((rxmatch #/Λ/i "λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (AA)" "Λ"
+       (cond ((rxmatch #/Λ/i "Λ") => rxmatch-substring)
+             (else #f)))
+
+(test* "regexp/unicode-ci (uncase + backref, aa)" "λλ"
+       (cond ((rxmatch #/(λ)(?i:\1)/ "λλ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (uncase + backref, aA)" "λΛ"
+       (cond ((rxmatch #/(λ)(?i:\1)/ "λΛ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (uncase + backref, Aa)" "Λλ"
+       (cond ((rxmatch #/(Λ)(?i:\1)/ "Λλ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (uncase + backref, AA)" "ΛΛ"
+       (cond ((rxmatch #/(Λ)(?i:\1)/ "ΛΛ") => rxmatch-substring)
+             (else #f)))
+
+(test* "regexp/unicode-ci (charset, aa)" "λ"
+       (cond ((rxmatch #/[λ]/i "λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (charset, aA)" "Λ"
+       (cond ((rxmatch #/[λ]/i "Λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (charset, Aa)" "λ"
+       (cond ((rxmatch #/[Λ]/i "λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (charset, AA)" "Λ"
+       (cond ((rxmatch #/[Λ]/i "Λ") => rxmatch-substring)
+             (else #f)))
+
+(test* "regexp/unicode-ci" "ΒΓ"
+       (cond ((rxmatch #/βγ/i "ΑΒΓΔ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "βΓ"
+       (cond ((rxmatch #/Βγ/i "ΑβΓΔ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "βγ"
+       (cond ((rxmatch #/ΒΓ/i "ΑβγΔ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/Βγ/ "ΑβΓΔ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/ΒΓ/ "ΑΒγΔ") => rxmatch-substring)
+             (else #f)))
+
+(test* "regexp/unicode-ci" "ОНА"
+       (cond ((rxmatch #/о[а-я]а/i "ОНА") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "она"
+       (cond ((rxmatch #/О[А-Я]А/i "она") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "они"
+       (cond ((rxmatch #/[а-пР-Я][А-Пр-я]./i "они") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/о[а-я]а/ "ОНА") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/О[А-Я]А/ "она") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/[а-пР-Я][А-Пр-я]./ "они") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "она"
+       (cond ((rxmatch (string->regexp "о[А-Я]а" :case-fold #t) "она")
+              => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch (string->regexp "о[А-Я]а") "она")
+              => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "σζ\x01C5;"
+       (cond ((rxmatch (string->regexp "[[:LOWER:]]*" :case-fold #t) "σζ\x01C5;")
+              => rxmatch-substring)
+             (else #f)))
+
 (test-end)
